package com.urovo.wgmf.db.dao;

import android.content.Context;
import android.database.Cursor;
import android.database.SQLException;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;

import com.urovo.wugumofang.system.BaseAppManager;


public class DBTool extends SQLiteOpenHelper {
    private final static String DBName = "wgmoDB.db";
    private static DBTool dbTool = null;
    private final static int NEW_VERSION = 20221230;

    private DBTool(Context context) {
        super(context, DBName, null, NEW_VERSION);
        BaseAppManager.loger().info("SQLVersion:" + NEW_VERSION);
    }

    public synchronized static DBTool getInstance(Context context) {
        if (dbTool == null) {
            dbTool = new DBTool(context);
        }
        return dbTool;
    }

    @Override
    public void onCreate(SQLiteDatabase db) {
        doCreateTable(db);
    }

    @Override
    public void onDowngrade(SQLiteDatabase db, int oldVersion, int newVersion) {
        db.beginTransaction();
        try {
            //只要是回滚，就把基础资料相关表都删干净
            // ，然后重建，避免出现数据库问题（以下表的来源取自0310221的全量下载）
            db.execSQL("drop table CPBARBILL001");
            db.execSQL("drop table CPBARSHOP");
            db.execSQL("drop table CPBARBILL002");
            db.execSQL("drop table ITEM001");
            db.execSQL("drop table CPBILL002");
            db.execSQL("drop table XBZPF");
            db.execSQL("drop table STORECS_GRP002");
            //全部重建
            doCreateTable(db);
            resetTableTopos(db);//以上几个表都是基础资料下载下来的，所以整个清了也无所谓
            db.setTransactionSuccessful();
        } catch (SQLException e) {
            e.printStackTrace();
            //这里可以改为弹窗提示，并丢到Act里处理重新下载*/
        } finally {
            //结束事务
            db.endTransaction();
        }
    }

    @Override
    public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion) {
        BaseAppManager.loger().info("SQLVersion:oldVersion<==>" + oldVersion);
        //20161110版本增加上增加下单部分字段；
        if (oldVersion < 20161110) {
            db.execSQL("alter table 'DJXS001' add 'sentProPointOrNot' number NULL");
            db.execSQL("alter table 'DJXS001' add 'giftPoints' number NULL");
            db.execSQL("alter table 'DJXS001' add 'giftMoney' number NULL");
            db.execSQL("alter table 'DJXS002' add 'specialPrice' number NULL");//子表
            resetTableTopos(db);//重建时间戳表
            db.execSQL("alter table 'STORE001' add 'ZGNAME' varchar(20) NULL");//主管id
            db.execSQL("alter table 'STORE001' add 'ZGNO' varchar(20) NULL");//主管name
        } else if (oldVersion == 20161110) {
            db.execSQL("alter table 'STORE001' add 'ZGNAME' varchar(20) NULL");//主管id
            db.execSQL("alter table 'STORE001' add 'ZGNO' varchar(20) NULL");//主管name
        }

        if (oldVersion < 20170215) {
            db.execSQL("alter table 'ITEM001' add 'INGREDIENT' varchar(120) null");
            db.execSQL("alter table 'ITEM001' add 'SAVECONDITION' varchar(60) null");
            db.execSQL("alter table 'CZYDA001' add 'MOBILE' varchar(20) null");
            db.execSQL("alter table 'ITEM001' add 'BARCODE' varchar(25) null");
        }

        if (oldVersion < 20170322) {//生意管家v3.3.0版本之前
            db.execSQL("drop table DJPD002");
            db.execSQL("CREATE TABLE [CHECKDETAIL] ([BILL] varchar(30) NULL  "
                    + ", [REMARK] varchar(500) NULL  "
                    + ", [URL] varchar(500) NULL  "
                    + ", [PRO_NAME] varchar(40) NULL  "
                    + ", [PRO_CODE] varcar(20) NULL  "
                    + ", [PRO_PRICE] number NULL  "
                    + ", [PD_QTY] number NULL  "
                    + ", [PD_AMOUNT] number NULL  "
                    + ", [STOCK_QTY] number NULL  "
                    + ", [DIFF_QTY] number NULL  "
                    + ", [DIFF_AMOUNT] number NULL  "
                    + ", [PRO_ID] number NULL  "
                    + ", [LOCATION_ID] number NULL  "
                    + ", [PD_ALTER_NUM] number NULL  "
                    + ", [PD_BILLCODE] varcar(30) NULL  "
                    + ", [PD_ID] number NULL  "
                    + ", [PRO_UNIT] varcar(10) NULL  "
                    + ", [PRO_UNIT_ID] number NULL  "
                    + ", [STORE_ID] number NULL  "
                    + ", [TYPE] varchar(20) default 0  "
                    + ", [PD_USER_NO] varchar(20) NULL  "
                    + ", [PD_STATUS] varchar(10) NULL)");
            db.execSQL("alter table 'ITEM001' add 'PROAREA' varchar(60) null");
            db.execSQL("alter table 'ITEM001' add 'PROSUPPLIER' varchar(60) null");
        } else if (oldVersion < 20170324) {
            db.execSQL("alter table 'ITEM001' add 'PROAREA' varchar(60) null");
            db.execSQL("alter table 'ITEM001' add 'PROSUPPLIER' varchar(60) null");
        } else if (oldVersion < 20170412) {
            db.execSQL("alter table 'CXYKQ' add 'MOREIMAGEURL' varchar(500) null");
        }

        if (oldVersion < 20170510) {
            db.execSQL("drop table DJPS001");
            db.execSQL("drop table DJPS002");
            db.execSQL("drop table DJPSSH001");
            db.execSQL("drop table DJPSSH002");
            db.execSQL("CREATE TABLE  IF NOT EXISTS [psTable] ([BILL] varchar(20) NULL   " +
                    ", [dataStr] text NULL" +
                    ", [COMM] varchar(40) NULL  " +
                    ", [IsUp] int NULL)");
        }

        if (oldVersion < 20170818) {
            db.execSQL("alter table 'Cpbarbill002' add 'BarCodePaper' varchar(10) null");
            db.execSQL("alter table 'Cpbarbill002' add 'PrintRule' varchar(100) null");
            db.execSQL("alter table 'CZYDA001' add 'WGMFROLE' char(60) null");
        }

        if (oldVersion < 20171207) {
            db.execSQL("alter table 'ITEM001' add 'BRAND' varchar(20) null");
            db.execSQL("alter table 'CPBILL002' add 'ISDECIMAL' varchar(5) null");
        }

        if (oldVersion < 20180104) {
            db.execSQL("alter table 'ITEM001' add 'CLASSIDORDER' int null");
        }

        if (oldVersion < 20180320) {
            // 权限privilege
            db.execSQL("CREATE TABLE IF NOT EXISTS  [privilege] ([userNo] varchar(20) NULL   "
                    + ", [resKey] varchar( 10 ) NULL )");
            db.execSQL("CREATE TABLE IF NOT EXISTS  [privilegeRel] ([reskey] varchar(20) NULL   "
                    + ", [linkClass] varchar( 60 ) NULL "
                    + ", [params] varchar(10) NULL "
                    + ", [title] varchar(30) NULL)");
        }

        if (oldVersion < 20180515) {
            db.execSQL("alter table 'DJXS002' add 'brandType' varchar(5) NULL");
        }

        if (oldVersion < 20180607) {
            // 条码扫码表
            db.execSQL("CREATE TABLE IF NOT EXISTS  [barcode_scan] ("
                    + "[ItemId] varchar( 20 ) NULL   "
                    + ", [BarType] varchar( 5 ) NULL "
                    + ", [Barcode] varchar( 5 ) NULL "
                    + ", [ScanRule] varchar( 50 ) NULL "
                    + ", [barcodelens] varchar( 5 ) NULL )"
            );
        }

        if (oldVersion < 20180809) {
            //打印机响应码信息表
            db.execSQL("CREATE TABLE IF NOT EXISTS  [printer_retcode] ("
                    + " [retcode] varchar( 10 ) NULL   "
                    + ", [content] varchar( 100 ) NULL )"
            );
        }

        if (oldVersion < 20180911) {
            addKeyIfNotExist(db, "DJXS002", "isDecimal", "varchar(5) NULL");
        }

        if (oldVersion < 20181114) {
            addKeyIfNotExist(db, "CZYDA001", "BRANDLIST", "char(20) default ''");
            addKeyIfNotExist(db, "ITEM001", "BRANDID", "int(10) default 0");
        }

        if (oldVersion < 20191204) {
            addKeyIfNotExist(db, "DJXS001", "orderBillNo", "char(300) default ''");
            addKeyIfNotExist(db, "DJXS001", "ordertype", "int(10) default 0");
        }

        if (oldVersion < 20200721) {
            db.execSQL("drop table DJPD001");
            db.execSQL("CREATE TABLE [DJPD001] ([BILL] varchar(20) NULL  "
                    + ", [STOREID] varchar(10) NULL  "
                    + ", [STWHSID] varcar(20) NULL  "
                    + ", [YNGC] varchar(1) NULL  "
                    + ", [PDDESC] varchar(40) NULL  "
                    + ", [PDDATE] datetime NULL"
                    + ", [STATUS] varchar(1) NULL  "
                    + ", [OPER_LR] varchar(10) NULL  "
                    + ", [LR_NAME] varchar(20) NULL  "
                    + ", [REMARK] varchar(500) NULL  "
                    + ", [URL] varchar(500) NULL  "
                    + ", [WDATE] datetime NULL"
                    + ", [IsUp] int NULL)");
            db.execSQL("drop table CHECKDETAIL");
            db.execSQL("CREATE TABLE [CHECKDETAIL] ([BILL] varchar(30) NULL  "
                    + ", [REMARK] varchar(500) NULL  "
                    + ", [URL] varchar(500) NULL  "
                    + ", [PRO_NAME] varchar(40) NULL  "
                    + ", [PRO_CODE] varcar(20) NULL  "
                    + ", [PRO_PRICE] number NULL  "
                    + ", [PD_QTY] number NULL  "
                    + ", [PD_AMOUNT] number NULL  "
                    + ", [STOCK_QTY] number NULL  "
                    + ", [DIFF_QTY] number NULL  "
                    + ", [DIFF_AMOUNT] number NULL  "
                    + ", [PRO_ID] number NULL  "
                    + ", [LOCATION_ID] number NULL  "
                    + ", [PD_ALTER_NUM] number NULL  "
                    + ", [PD_BILLCODE] varcar(30) NULL  "
                    + ", [PD_ID] number NULL  "
                    + ", [PRO_UNIT] varcar(10) NULL  "
                    + ", [PRO_UNIT_ID] number NULL  "
                    + ", [STORE_ID] number NULL  "
                    + ", [TYPE] varchar(20) default 0  "
                    + ", [PD_USER_NO] varchar(20) NULL  "
                    + ", [PD_STATUS] varchar(10) NULL)");
        }

        if (oldVersion < 20200821) {
            db.execSQL("drop table ITEM001");
            // 产品基础资料表
            db.execSQL("CREATE TABLE IF NOT EXISTS  [ITEM001] ([ITEMID] varchar(20) NULL  "
                    + ", [ITEMNAME] varchar(60) NULL  "
                    + ", [ITEMMODE] varchar(1) NULL  "
                    + ", [CLASSID] varchar(20) NULL  "
                    + ", [ITEMTYPE] varchar(20) NULL  "
                    + ", [ITEMGX] varchar(300) NULL  "
                    + ", [XSGRPID] varchar(10) NULL  " + ", [SORTNO] number NULL  "
                    + ", [KCUOM] varchar(10) NULL  "
                    + ", [QTYINTEGER] varchar(1) NULL  "
                    + ", [DHPACK] number NULL  " + ", [UNIT] number NULL  "
                    + ", [LASTMONTH_QTY] varchar(20) NULL  "
                    + ", [STOCK_QTY] varchar(20) NULL  "
                    + ", [NORMRPRICE] number NULL"
                    + ", [INGREDIENT] char(120) NULL"
                    + ", [SAVECONDITION] char(60) NULL"
                    + ", [BARCODE] char(25) NULL"
                    + ", [PROAREA] char(60) NULL"
                    + ", [PROSUPPLIER] char(60) NULL"
                    + ", [CLASSIDORDER] int NULL"
                    + ", [BRAND] varchar(20) NULL"
                    + ", [BRANDID] int NULL "
                    + ", [BOMSPEC] varchar(10) NULL) ");
        }

        if (oldVersion < 20211019) {
            //App响应码信息表
            db.execSQL("CREATE TABLE IF NOT EXISTS [app_retcode] ("
                    + "  [retcode] varchar( 10 )  NULL "
                    + ", [content] varchar( 100 )  NULL )"
            );
        }

        if(oldVersion < 20221229){
            db.execSQL("alter table 'DJXS002' add 'groupId' number NULL");
            db.execSQL("alter table 'DJXS002' add 'proportRelation' number NULL");
        }
    }

    /**
     * 清空TABLE_TOPOS时间戳表
     */
    private void resetTableTopos(SQLiteDatabase db) {
        db.execSQL("drop table TABLE_TOPOS");
        db.execSQL("CREATE TABLE [TABLE_TOPOS] ([TABLENAME] varchar(30) PRIMARY KEY NOT NULL UNIQUE   "
                + ", [VERSIONTIME] varchar(30) NULL"
                + ", [COMM] varchar(30) NULL )");//COMM是预留的备注栏
    }

    /**
     * 给表添加新字段（如果不存在才添加，如果存在就不添加）
     *
     * @param tableName
     * @param key
     */
    private void addKeyIfNotExist(SQLiteDatabase db, String tableName, String key, String font) {
        Cursor mCount = db.rawQuery("select count(*) from sqlite_master where name='" + tableName + "' and sql like '%" + key + "%'", null);
        mCount.moveToFirst();
        int count = mCount.getInt(0);
        mCount.close();
        if (count <= 0) {//如果不存在，才添加该字段。
            db.execSQL("alter table " + tableName + " add column " + key + " " + font);
        }
    }

    private boolean doCreateTable(SQLiteDatabase db) {
        //App响应码信息表
        db.execSQL("CREATE TABLE IF NOT EXISTS [app_retcode] ("
                + "  [retcode] varchar( 10 )  NULL "
                + ", [content] varchar( 100 )  NULL )"
        );

        //打印机响应码信息表
        db.execSQL("CREATE TABLE IF NOT EXISTS  [printer_retcode] ("
                + " [retcode] varchar( 10 ) NULL   "
                + ", [content] varchar( 100 ) NULL )"
        );

        // 条码扫码表
        db.execSQL("CREATE TABLE IF NOT EXISTS  [barcode_scan] ("
                + "[ItemId] varchar( 20 ) NULL   "
                + ", [BarType] varchar( 5 ) NULL "
                + ", [Barcode] varchar( 5 ) NULL "
                + ", [ScanRule] varchar( 50 ) NULL "
                + ", [barcodelens] varchar( 5 ) NULL )"
        );

        // 权限privilege
        db.execSQL("CREATE TABLE IF NOT EXISTS  [privilege] ([userNo] varchar(10) NULL  "
                + ", [resKey] varchar( 10 ) NULL )");
        //本地维护的权限reskey映射表
        db.execSQL("CREATE TABLE IF NOT EXISTS  [privilegeRel] ([reskey] varchar(20) NULL   "
                + ", [linkClass] varchar( 60 ) NULL "
                + ", [params] varchar(10) NULL "
                + ", [title] varchar(30) NULL)");
        // 基础数据版本时间戳（TABLENAME为主键，用于刷新同一数据项的时间戳）
        db.execSQL("CREATE TABLE IF NOT EXISTS  [TABLE_TOPOS] ([TABLENAME] varchar(30) PRIMARY KEY NOT NULL UNIQUE  "
                + ", [VERSIONTIME] varchar(30) NULL"
                + ", [COMM] varchar(30) NULL )");//COMM是预留的备注栏
        // 产品分类档案表
        db.execSQL("CREATE TABLE IF NOT EXISTS  [CLASSDA]" + "([CLASSID] varchar(20) NULL , "
                + "[CLASSDP] varchar(40) NULL  , "
                + "[CLASSLV] varchar(1) NULL  , "
                + "[LOCID] varchar(10) NULL  , "
                + "[FATHERID] varchar(20) NULL  , "
                + "[COMM] varchar(40) NULL  ) ");
        // 新添加的表
        db.execSQL("CREATE TABLE IF NOT EXISTS  [Cpbarbill001] ([DirId] varchar(100) NULL  "
                + ", [DirName] varchar(100) NULL  "
                + ", [BarType] varchar(10) NULL  "
                + ", [Status] varchar(10) NULL  "
                + ", [Remark] varchar(1000) NULL  "
                + ", [Oper_Lr] varchar(50) NULL  "
                + ", [Lr_Name] varchar(50) NULL  "
                + ", [WDate] datetime NULL  "
                + ", [Oper_Sh] varchar(50) NULL  "
                + ", [Sh_Name] varchar(50) NULL  "
                + ", [BDate] datetime NULL  )");

        // 新添加的表
        db.execSQL("CREATE TABLE IF NOT EXISTS  [Cpbarbill002] ([DirId] varchar(100) NULL  "
                + ", [ItemId] varchar(100) NULL  "
                + ", [Barcode] varchar(100) NULL  "
                + " ,[BarType] varchar(30) NULL"
                + ",[BarCodePaper] varchar(10) NULL"
                + ",[PrintRule] varchar(100) NULL  )");

        // 新添加的表
        db.execSQL("CREATE TABLE IF NOT EXISTS  [Cpbarshop] ([DirId] varchar(100) NULL  "
                + ", [StoreId] varchar(100) NULL  "
                + ", [Oper_Lr] varchar(50) NULL  "
                + ", [Lr_Name] varchar(50) NULL  "
                + ", [WDate] datetime NULL  )");

        // 产品目录子表
        db.execSQL("CREATE TABLE IF NOT EXISTS  [CPBILL002] ([BILL] varchar(20) NULL  "
                + ", [ITEMID] varchar(20) NULL  "
                + ", [DHSTAT] varchar(1) NULL  "
                + ", [PDSTAT] varchar(1) NULL  "
                + ", [SNO] number NULL  "
                + ", [XSGRPID] varchar(10) NULL  "
                + ", [SORTNO] number NULL  "
                + ", [ISDECIMAL] varchar(5) NULL  )");
        // 营养顾问考勤表中间表
        db.execSQL("CREATE TABLE IF NOT EXISTS  [CXYKQ] ([BILL] varchar(20) NULL  "
                + ", [STOREID] varchar(10) NULL  " + ", [KQNO] number NULL  "
                + ", [POSID] varchar(6) NULL  "
                + ", [SKYID] varchar(10) NULL  "
                + ", [KQTYPE] varchar(1) NULL  "
                + ", [KQDATE] varchar(10) NULL  "
                + ", [KQTIME] varchar(8) NULL  "
                + ", [LONGITUDE] varchar(20) NULL  "
                + ", [LATITUDE] varchar(20) NULL  "
                + ", [valid] varchar(5) NULL  "
                + ", [ADDRESS] varchar(500) NULL  "
                + ", [IMAGEURL] varchar(200) NULL  " + ", [IsUp] int NULL" + ", [MOREIMAGEURL] varchar(500) NULL  )");
        // 人员资料表
        db.execSQL("CREATE TABLE IF NOT EXISTS  [CZYDA001] ([OPERATORID] varchar(10) NULL  "
                + ", [OPERNAME] varchar(20) NULL  "
                + ", [STAFFID] varchar(20) NULL  "
                + ", [PASSWORD] varchar(40) NULL  "
                + ", [SEX] varchar(2) NULL  " + ", [OPTYPE] varchar(2) NULL  "
                + ", [DEPTCSID] varchar(20) NULL  "
                + ", [STOREID] varchar(10) NULL  "
                + ", [STATUS] varchar(1) NULL  " + ", [ENV] varchar(10) NULL"
                + ", [MOBILE] char(20) NULL"
                + ", [WGMFROLE] char(60) NULL"
                + ", [BRANDLIST] char(20) NULL )");
        // 门店销售组织档案表----------
        db.execSQL("CREATE TABLE IF NOT EXISTS  [DEPTCSDA] ("
                + "[DEPTCSID] varchar(20) NULL  "
                + ", [DEPTCSDP] varchar(40) NULL  "
                + ", [DEPTCSLV] varchar(1) NULL  "
                + ", [LOCID] varchar(10) NULL  "
                + ", [FATHERID] varchar(20) NULL  "
                + ", [COMM] varchar(40) NULL)");

        // 新增的表
        db.execSQL("CREATE TABLE IF NOT EXISTS  [DHSH001] ([BILL] varchar(20) NULL  "
                + ", [StoreID] varchar(10) NULL  "
                + ", [Wdate] varchar(20) NULL  " + ", [ItemCount] int NULL  "
                + ", [Amount] number NULL  " + ", [State] varchar(10) NULL  "
                + ", [IsUp] int NULL  " + ", [Oper_Sh] varchar(100) NULL  "
                + ", [Sh_Wdate] varchar(20) NULL  )");
        // 新增的表
        db.execSQL("CREATE TABLE IF NOT EXISTS  [DHSH002] ([BILL] varchar(20) NULL  "
                + ", [OrderQty] number NULL  " + ", [OplQty] number NULL  "
                + ", [ExamineQty] number NULL  "
                + ", [ItemID] varchar(10) NULL  )");
        // 门店订货单主表
        db.execSQL("CREATE TABLE IF NOT EXISTS  [DJBH001] ([BILL] varchar(20) NULL  "
                + ", [STOREID] varchar(10) NULL  "
                + ", [STWHSID] varchar(20) NULL  "
                + ", [PSSTOREID] varchar(10) NULL  "
                + ", [UDATE] datetime NULL  " + ", [BHMODE] varchar(1) NULL  "
                + ", [BHTYPE] varchar(1) NULL  "
                + ", [BHSYS] varchar(1) NULL  "
                + ", [STATUS] varchar(1) NULL  "
                + ", [OPER_LR] varchar(10) NULL  "
                + ", [LR_NAME] varchar(20) NULL  "
                + ", [WDATE] datetime NULL  "
                + ", [OPER_SH] varchar(10) NULL  "
                + ", [SH_NAME] varchar(20) NULL  "
                + ", [BDATE] datetime NULL  " + ", [COMM] varchar(40) NULL  "
                + ", [YN_TX] varchar(1) NULL  " + ", [IsUp] int NULL  )");

        // 门店订货产品表
        db.execSQL("CREATE TABLE IF NOT EXISTS  [DJBH002] ([BILL] varchar(20) NULL  "
                + ", [ITEMID] varchar(20) NULL  " + ", [BHQTY] number NULL  "
                + ", [PRICE] number NULL  " + ", [TOTNET] number NULL  "
                + ", [COMM] varchar(40) NULL  " + ", [SEQ_NO] number NULL  "
                + ", [YN_TX] varchar(1) NULL)");
        // 报损单主表
        db.execSQL("CREATE TABLE IF NOT EXISTS  [DJBS001] ([BILL] varchar(20) NULL  "
                + ", [STOREID] varchar(10) NULL  "
                + ", [STWHSID] varchar(20) NULL  "
                + ", [YNGC] varchar(1) NULL  " + ", [BSYY] varchar(10) NULL  "
                + ", [OPER_LR] varchar(10) NULL  "
                + ", [LR_NAME] varchar(20) NULL  " + ", [WDATE] datetime NULL"
                + ", [STATUS] varchar(1) NULL  "
                + ", [COMM] varchar(40) NULL  " + ", [IsUp] int NULL )");
        // 报损单产品表
        db.execSQL("CREATE TABLE IF NOT EXISTS  [DJBS002] ([BILL] varchar(20) NULL  "
                + ", [ITEMID] varchar(20) NULL  " + ", [PACK] number NULL  "
                + ", [UNIT] number NULL  " + ", [SQTY] number NULL  "
                + ", [TQTY] number NULL  " + ", [PRICE] number NULL  "
                + ", [SPTMID] varchar(20) NULL  "
                + ", [COMM] varchar(40) NULL  " + ", [SEQ_NO] number NULL  "
                + ", [YN_TX] varchar(1) NULL )");
        // 冲顶加工单主表
        db.execSQL("CREATE TABLE IF NOT EXISTS  [DJCJ001] ([BILL] varchar(20) NULL  "
                + ", [STOREID] varchar(10) NULL  "
                + ", [STWHSID] varchar(20) NULL  "
                + ", [YNGC] varchar(1) NULL  "
                + ", [OPER_LR] varchar(10) NULL  "
                + ", [LR_NAME] varchar(20) NULL  " + ", [WDATE] datetime NULL"
                + ", [STATUS] varchar(1) NULL  "
                + ", [COMM] varchar(40) NULL  " + ", [IsUp] int NULL)");
        // 冲顶加工单产品表
        db.execSQL("CREATE TABLE IF NOT EXISTS  [DJCJ002] ([BILL] varchar(20) NULL  "
                + ", [ITEMID] varchar(20) NULL  " + ", [PACK] number NULL  "
                + ", [UNIT] number NULL  " + ", [SQTY] number NULL  "
                + ", [TQTY] number NULL  " + ", [PRICE] number NULL  "
                + ", [COMM] varchar(40) NULL  " + ", [SEQ_NO] number NULL )");
        // 门店调拨单主表
        db.execSQL("CREATE TABLE IF NOT EXISTS  [DJDB001] ([BILL] varchar(20) NULL  "
                + ", [STOREID] varchar(10) NULL  "
                + ", [STWHSID] varchar(20) NULL  "
                + ", [TSTOREID] varchar(10) NULL  "
                + ", [OPER_LR] varchar(10) NULL  "
                + ", [LR_NAME] varchar(20) NULL  " + ", [WDATE] datetime NULL"
                + ", [STATUS] varchar(1) NULL  "
                + ", [SHSTATUS] varchar(1) NULL  " + ", [DBNUM] number NULL  "
                + ", [COMM] varchar(40) NULL  " + ", [IsUp] int NULL )");
        // 调拨单产品表
        db.execSQL("CREATE TABLE IF NOT EXISTS  [DJDB002] ([BILL] varchar(20) NULL  "
                + ", [ITEMID] varchar(20) NULL  " + ", [PACK] number NULL  "
                + ", [UNIT] number NULL  " + ", [SQTY] number NULL  "
                + ", [TQTY] number NULL  " + ", [PRICE] number NULL  "
                + ", [COMM] varchar(40) NULL  " + ", [SEQ_NO] number NULL)");

        // 门店调拨收货单主表
        db.execSQL("CREATE TABLE IF NOT EXISTS  [DJDBSH001] ([BILL] varchar(20) NULL  "
                + ", [DB_BILL] varchar(20) NULL  "
                + ", [STOREID] varchar(10) NULL  "
                + ", [STWHSID] varchar(20) NULL  "
                + ", [FSTOREID] varchar(10) NULL  "
                + ", [FSTWHSID] varchar(20) NULL  "
                + ", [OPER_LR] varchar(10) NULL  "
                + ", [LR_NAME] varchar(20) NULL  " + ", [WDATE] datetime NULL"
                + ", [STATUS] varchar(1) NULL  "
                + ", [COMM] varchar(40) NULL  " + ", [IsUp] varchar(100) NULL)");

        // 门店调拨收货产品表
        db.execSQL("CREATE TABLE IF NOT EXISTS  [DJDBSH002] ([BILL] varchar(20) NULL  "
                + ", [ITEMID] varchar(20) NULL  " + ", [PACK] number NULL  "
                + ", [UNIT] number NULL  " + ", [SQTY] number NULL  "
                + ", [TQTY] number NULL  " + ", [PRICE] number NULL  "
                + ", [COMM] varchar(40) NULL  " + ", [SEQ_NO] number NULL)");

        // 盘点单主表
        db.execSQL("CREATE TABLE [DJPD001] ([BILL] varchar(20) NULL  "
                + ", [STOREID] varchar(10) NULL  "
                + ", [STWHSID] varcar(20) NULL  "
                + ", [YNGC] varchar(1) NULL  "
                + ", [PDDESC] varchar(40) NULL  "
                + ", [PDDATE] datetime NULL"
                + ", [STATUS] varchar(1) NULL  "
                + ", [OPER_LR] varchar(10) NULL  "
                + ", [LR_NAME] varchar(20) NULL  "
                + ", [REMARK] varchar(500) NULL  "
                + ", [URL] varchar(500) NULL  "
                + ", [WDATE] datetime NULL"
                + ", [IsUp] int NULL)");

        //新 盘点详情表
        db.execSQL("CREATE TABLE IF NOT EXISTS  [CHECKDETAIL] ([BILL] varchar(30) NULL  "
                + ", [REMARK] varchar(500) NULL  "
                + ", [URL] varchar(500) NULL  "
                + ", [PRO_NAME] varchar(40) NULL  "
                + ", [PRO_CODE] varcar(20) NULL  "
                + ", [PRO_PRICE] number NULL  "
                + ", [PD_QTY] number NULL  "
                + ", [PD_AMOUNT] number NULL  "
                + ", [STOCK_QTY] number NULL  "
                + ", [DIFF_QTY] number NULL  "
                + ", [DIFF_AMOUNT] number NULL  "
                + ", [PRO_ID] number NULL  "
                + ", [LOCATION_ID] number NULL  "
                + ", [PD_ALTER_NUM] number NULL  "
                + ", [PD_BILLCODE] varcar(30) NULL  "
                + ", [PD_ID] number NULL  "
                + ", [PRO_UNIT] varcar(10) NULL  "
                + ", [PRO_UNIT_ID] number NULL  "
                + ", [STORE_ID] number NULL  "
                + ", [TYPE] varchar(20) default 0  "
                + ", [PD_USER_NO] varchar(20) NULL  "
                + ", [PD_STATUS] varchar(10) NULL)");

        // 盘点单产品表
        db.execSQL("CREATE TABLE IF NOT EXISTS  [DJPD002] ([BILL] varchar(20) NULL  "
                + ", [ITEMID] varchar(20) NULL  , [PRICE] number NULL  "
                + ", [SP_QTY] number NULL  " + ", [SP_NET] number NULL  "
                + ", [COMM] varchar(40) NULL  " + ", [SEQ_NO] number NULL)");

        //配送单本地存储
        db.execSQL("CREATE TABLE IF NOT EXISTS  [psTable] ([BILL] varchar(20) NULL  " +
                ", [dataStr] text NULL" +
                ", [COMM] varchar(40) NULL " +
                ", [IsUp] int NULL)");

        // 试吃单主表
        db.execSQL("CREATE TABLE IF NOT EXISTS  [DJSC001] ([BILL] varchar(20) NULL  "
                + ", [STOREID] varchar(10) NULL  "
                + ", [STWHSID] varchar(20) NULL  "
                + ", [YNGC] varchar(1) NULL  "
                + ", [OPER_LR] varchar(10) NULL  "
                + ", [LR_NAME] varchar(20) NULL  " + ", [WDATE] datetime NULL"
                + ", [STATUS] varchar(1) NULL  "
                + ", [COMM] varchar(40) NULL  " + ", [IsUp] int NULL)");

        // 试吃单产品表
        db.execSQL("CREATE TABLE IF NOT EXISTS  [DJSC002] ([BILL] varchar(20) NULL  "
                + ", [ITEMID] varchar(20) NULL  " + ", [PACK] number NULL  "
                + ", [UNIT] number NULL  " + ", [SQTY] number NULL  "
                + ", [TQTY] number NULL  " + ", [PRICE] number NULL  "
                + ", [COMM] varchar(40) NULL  " + ", [SEQ_NO] number NULL)");

        // 销售单主表中间表
        db.execSQL("CREATE TABLE IF NOT EXISTS  [DJXS001] ([BILL] varchar(20) NULL  "
                + ", [STOREID] varchar(10) NULL  "
                + ", [SALEBILL] varchar(20) NULL  "
                + ", [STWHSID] varchar(20) NULL  "
                + ", [POSID] varchar(6) NULL  "
                + ", [SALEDATE] varchar(20) NULL"
                + ", [SCDATE] varchar(20) NULL"
                + ", [SALETIME] varchar(8) NULL  "
                + ", [BILLNO] number NULL  "
                + ", [MDBC] varchar(1) NULL  "
                + ", [CUSTID] varchar(20) NULL  "
                + ", [SKYID] varchar(10) NULL  "
                + ", [XSTYPE] varchar(1) NULL  "
                + ", [HYXB] varchar(1) NULL  "
                + ", [AGE] varchar(10) NULL  "
                + ", [REASON] varchar(50) NULL  "
                + ", [TLINE] number NULL  "
                + ", [TNET] number NULL  "
                + ", [SKTNET] number NULL  "
                + ", [SKROUND] number NULL  "
                + ", [ZKYHJE] number NULL  "
                + ", [KGTYPE] varchar(6) NULL  "
                + ", [YXTYPE] varchar(6) NULL  "
                + ", [DYSALEDATE] varchar(20) NULL"
                + ", [DYSALETIME] varchar(8) NULL  "
                + ", [DYBILLNO] number NULL  "
                + ", [DYPOSID] varchar(6) NULL  "
                + ", [PROMID] varchar(20) NULL  " // 活动ID
                + ", [EXCHANGECODE] varchar(20) NULL  " // 验证码
                + ", [IsUp] int NULL  "
                + ", [FAILREASON] varchar(300) NULL  "
                + ", [LR_NAME] varchar(20) NULL"
                + ", [Recommendation] varchar(20) NULL "
                + ", [sentProPointOrNot] number NULL  "
                + ", [giftPoints] number NULL  "
                + ", [ordertype] number NULL  "//订单类型  0正常下单   1补录   2微商城   3到家业务
                + ", [orderBillNo] varchar(300) NULL  " // 到家业务订单号
                + ", [giftMoney] number NULL  )"
        );

        // 销售单产品子表中间表
        db.execSQL("CREATE TABLE IF NOT EXISTS  [DJXS002] ([BILL] varchar(20) NULL  "
                + ", [NO] number NULL  "
                + ", [ITEMID] varchar(20) NULL  "
                + ", [BARCODE] varchar(20) NULL  "
                + ", [ITEMMODE] varchar(1) NULL  "
                + ", [SALEQTY] number NULL  "
                + ", [ORPRICE] number NULL  "
                + ", [ZKYHJE] number NULL  "
                + ", [ZKFTJE] number NULL  "
                + ", [NET] number NULL  "
                + ", [SXPRICE] number NULL "
                + ", [specialPrice] number NULL "
                + ", [brandType] varchar(5) NULL "
                + ", [isDecimal] varchar(5) NULL"
                + ", [groupId] number NULL"
                + ", [proportRelation] number NULL)"
        );

        // 赠送单主表
        db.execSQL("CREATE TABLE IF NOT EXISTS  [DJZS001] ([BILL] varchar(20) NULL  "
                + ", [STOREID] varchar(10) NULL  "
                + ", [STWHSID] varchar(20) NULL  "
                + ", [YNGC] varchar(1) NULL  "
                + ", [CPHONE] varchar(20) NULL  "
                + ", [OPER_LR] varchar(10) NULL  "
                + ", [LR_NAME] varchar(20) NULL  " + ", [WDATE] datetime NULL"
                + ", [STATUS] varchar(1) NULL  "
                + ", [COMM] varchar(40) NULL  " + ", [IsUp] int NULL)");

        // 赠送单产品表
        db.execSQL("CREATE TABLE IF NOT EXISTS  [DJZS002] ([BILL] varchar(20) NULL  "
                + ", [ITEMID] varchar(20) NULL  " + ", [PACK] number NULL  "
                + ", [UNIT] number NULL  " + ", [SQTY] number NULL  "
                + ", [TQTY] number NULL  " + ", [PRICE] number NULL  "
                + ", [COMM] varchar(40) NULL " + ", [SEQ_NO] number NULL)");

        // 产品基础资料表
        db.execSQL("CREATE TABLE IF NOT EXISTS  [ITEM001] ([ITEMID] varchar(20) NULL  "
                + ", [ITEMNAME] varchar(60) NULL  "
                + ", [ITEMMODE] varchar(1) NULL  "
                + ", [CLASSID] varchar(20) NULL  "
                + ", [ITEMTYPE] varchar(20) NULL  "
                + ", [ITEMGX] varchar(300) NULL  "
                + ", [XSGRPID] varchar(10) NULL  " + ", [SORTNO] number NULL  "
                + ", [KCUOM] varchar(10) NULL  "
                + ", [QTYINTEGER] varchar(1) NULL  "
                + ", [DHPACK] number NULL  " + ", [UNIT] number NULL  "
                + ", [LASTMONTH_QTY] varchar(20) NULL  "
                + ", [STOCK_QTY] varchar(20) NULL  "
                + ", [NORMRPRICE] number NULL"
                + ", [INGREDIENT] char(120) NULL"
                + ", [SAVECONDITION] char(60) NULL"
                + ", [BARCODE] char(25) NULL"
                + ", [PROAREA] char(60) NULL"
                + ", [PROSUPPLIER] char(60) NULL"
                + ", [CLASSIDORDER] int NULL"
                + ", [BRAND] varchar(20) NULL"
                + ", [BRANDID] int NULL "
                + ", [BOMSPEC] varchar(10) NULL) ");

        // 产品条码表
        db.execSQL("CREATE TABLE IF NOT EXISTS  [ITEM004] ([ITEMID] varchar(20) NULL  "
                + ", [BARTYPE] varchar(1) NULL  "
                + ", [MAINBAR] varcar(1) NULL  "
                + ", [BARCODE] varchar(20) NULL)");

        // 门店产品资料表
        db.execSQL("CREATE TABLE IF NOT EXISTS  [ITEM005] ([STOREID] varchar(10) NULL  "
                + ", [ITEMID] varchar(20) NULL  " + ", [RPRICE] number NULL  "
                + ", [GCID] varchar(10) NULL  " + ", [KCQTY] number NULL  "
                + ", [TJBILL] varchar(20) NULL  " + ", [UDATE] datetime NULL"
                + ", [NRPRICE] number NULL)");

        // 积分兑换结果表中间表
        db.execSQL("CREATE TABLE IF NOT EXISTS  [JFDH001] ([BILL] varchar(20) NULL  "
                + ", [STOREID] varchar(10) NULL  "
                + ", [STWHSID] varchar(20) NULL  "
                + ", [POSID] varchar(6) NULL  " + ", [DHDATE] datetime NULL"
                + ", [DHTIME] varchar(8) NULL  " + ", [DHNO] number NULL  "
                + ", [CUSTID] varchar(20) NULL  "
                + ", [SKYID] varchar(10) NULL  "
                + ", [ITEMID] varchar(20) NULL  "
                + ", [ITEMMODE] varchar(20) NULL  " + ", [DHQTY] number NULL  "
                + ", [PRICE] number NULL  " + ", [KCJF] number NULL  "
                + ", [IsUp] int NULL  " + ", [YXJF] number NULL  "
                + ", [lpitemid] varchar(20) NULL)");

        // 文档中没有这个表
        db.execSQL("CREATE TABLE IF NOT EXISTS  [PDAParam] ([paramId] varchar(20) NULL  "
                + ", [paramName] varchar(100) NULL  "
                + ", [paramValue] varchar(100) NULL  "
                + ", [paramDesc] varchar(100) NULL)");

        // 门店档案表
        db.execSQL("CREATE TABLE IF NOT EXISTS  [STORE001] ([STOREID] varchar(10) NULL  "
                + ", [STORENAME] varchar(60) NULL  "
                + ", [DEPTCSID] varchar(20) NULL  "
                + ", [YYMODE] varchar(10) NULL  "
                + ", [GCID] varchar(10) NULL  "
                + ", [STATUS] varchar(1) NULL  "
                + ", [LATITUDE] varchar(30) NULL  "
                + ", [LONGITUDE] varchar(30) NULL  "
                + ", [ZGNAME] varchar(20) NULL  "
                + ", [ZGNO] varchar(20) NULL  "
                + ", [CPBILL] varchar(20) NULL)");

        // 门店参数组子表
        db.execSQL("CREATE TABLE IF NOT EXISTS  [STORECS_GRP002]  "
                + " ([CSTYPE] varchar(12) NULL  "
                + ", [CS_VALUE] number NULL)");

        // 浩联数据库上的表和程序包
        db.execSQL("CREATE TABLE IF NOT EXISTS  [t_dhlpitem] ([lpitemid] varchar(20) NULL  "
                + ", [lpitemname] varchar(60) NULL  "
                + ", [lpuom] varchar(20) NULL  " + ", [dhusejf] number NULL  "
                + ", [dhdates] datetime NULL" + ", [dhdatee] datetime NULL"
                + ", [itemid] varchar(20) NULL  " + ", [ehd_kg] number NULL)");
        //		Log.e("浩联数据库上的表和程序包", "积分兑换表");

        // 新增的表
        db.execSQL("CREATE TABLE IF NOT EXISTS  [XBZPF] ([ITEMID] varchar(100) NULL  "
                + ", [XBZID] varchar(100) NULL  " + ", [WEIGHT] number NULL  "
                + ", [BZQTY] number NULL  " + ", [SJQTY] number NULL  "
                + ", [BZPRICE] number NULL  " + ", [SJPRICE] number NULL)");

        // 新增的表
        db.execSQL("CREATE TABLE IF NOT EXISTS  [xy_ChgTimeRec] ([BILL] varchar(40) NULL  "
                + ", [StaffId] varchar(10) NULL  "
                + ", [StaffName] varchar(10) NULL  "
                + ", [LoginOrgNo] varchar(10) NULL  "
                + ", [LoginOrgName] varchar(40) NULL  "
                + ", [LoginMode] varchar(10) NULL  "
                + ", [Org_Time] datetime NULL  "
                + ", [Chg_Time] datetime NULL  "
                + ", [Up_Time] datetime NULL  " + ", [IsUp] int NULL)");

        // 文档中没有这个表
        db.execSQL("CREATE TABLE IF NOT EXISTS  [ZHXX001] ([BILL] varchar(20) NULL  "
                + ", [XXNO] number NULL  " + ", [STOREID] varchar(10) NULL  "
                + ", [XXLX] varchar(10) NULL  " + ", [XXZT] varchar(30) NULL  "
                + ", [WZXX] varchar(2000) NULL  "
                + ", [ZGWZ] varchar(6) NULL  " + ", [WZPJ] varchar(1) NULL  "
                + ", [ZGLX] varchar(30) NULL  " + ", [KYSJ] varchar(20) NULL  "
                + ", [HGRQ] varchar(20) NULL  "
                + ", [HWZRQ] varchar(20) NULL  " + ", [YNTP] varchar(1) NULL  "
                + ", [WDATE] varchar(20) NULL"
                + ", [OPER_LR] varchar(10) NULL  "
                + ", [LR_NAME] varchar(20) NULL  "
                + ", [BDATE] varchar(20) NULL"
                + ", [OPER_SH] varchar(10) NULL  "
                + ", [SH_NAME] varchar(20) NULL  "
                + ", [BACKTYPE] varchar(20) NULL  "
                + ", [REPORTINFO] varchar(100) NULL  "
                + ", [ITEMID] varchar(20) NULL  " + ", [QTY] number NULL  "
                + ", [PDATE] varchar(20) NULL  "
                + ", [STATUS] varchar(1) NULL  " + ", [IsUp] int NULL"
                + ", [problemSubtype] varchar(20) NULL)"
        );

        // 两个月未回头数据
        db.execSQL("CREATE TABLE IF NOT EXISTS  [LGYWHT] ([RN] varchar(5) NULL  "
                + ", [NAME] varchar(10) NULL  "
                + ", [MOBILE] varchar(15) PRIMARY KEY NOT NULL UNIQUE"
                + ", [SEX] varchar(5) NULL  " + ", [AGE] varchar(5) NULL  "
                + ", [BUYREASON] TEXT NULL  "
                + ", [SALEDATE] VARCHAR( 10 )  NULL "
                + ", [PRODUCTNAMES] TEXT NULL"
                + ", [CALLEDTIME] VARCHAR( 10 ) NULL"
                + ", [MSGEDTIME] VARCHAR( 10 ) NULL"
                + ", [INVALID] VARCHAR( 5 ) NULL)");

        // 生日关怀数据
        db.execSQL("CREATE TABLE IF NOT EXISTS  [SRGH] ([NAME] varchar(10) NULL  "
                + ", [SEX] varchar(5) NULL  "
                + ", [MOBILE] varchar(15) PRIMARY KEY NOT NULL UNIQUE"
                + ", [BIRTHDAY] VARCHAR( 10 ) NULL "
                + ", [CALLEDTIME] VARCHAR( 10 ) NULL"
                + ", [MSGEDTIME] VARCHAR( 10 ) NULL"
                + ", [INVALID] VARCHAR( 5 ) NULL)");
        // 七天新客数据
        db.execSQL("CREATE TABLE IF NOT EXISTS  [XKQTGH] ([NAME] varchar(10) NULL  "
                + ", [MOBILE] varchar(15) PRIMARY KEY NOT NULL UNIQUE"
                + ", [SEX] varchar(5) NULL  " + ", [AGE] varchar(5) NULL  "
                + ", [BUYREASON] TEXT NULL  "
                + ", [SALEDATE] VARCHAR( 10 ) NULL "
                + ", [PRODUCTNAMES] TEXT NULL"
                + ", [CALLEDTIME] VARCHAR( 10 ) NULL"
                + ", [MSGEDTIME] VARCHAR( 10 ) NULL"
                + ", [INVALID] VARCHAR( 5 ) NULL)");

        // 通知信息
        db.execSQL("CREATE TABLE IF NOT EXISTS  [TZXX] ([MSGNO] varchar(10) PRIMARY KEY NOT NULL UNIQUE  "
                + ", [MSGTYPE] VARCHAR( 10 ) NULL"
                + ", [MSGTITLE] TEXT NULL"
                + ", [MSGCONTENT] TEXT NULL"
                + ", [REDDATE] VARCHAR( 15)  NULL"
                + ", [INVALID] VARCHAR( 5)  NULL"
                + ", [STARTTIME] VARCHAR( 15)  NULL"
                + ", [ENDTIME] VARCHAR( 15 ) NULL)");
        // 常见问题
        db.execSQL("CREATE TABLE IF NOT EXISTS  [CJWT] ([QUNO] varchar(10) PRIMARY KEY NOT NULL UNIQUE  "
                + ", [QUTYPE] VARCHAR( 10 ) NULL"
                + ", [QUTITLE] TEXT NULL"
                + ", [QUCONTENT] TEXT NULL"
                + ", [QUURL] TEXT NULL"
                + ", [REDDATE] VARCHAR( 15)  NULL"
                + ", [INVALID] VARCHAR( 5)  NULL"
                + ", [QUTIME] VARCHAR( 15)  NULL)");
        // 学习文库
        db.execSQL("CREATE TABLE IF NOT EXISTS  [XXWK] ([XXWKNO] varchar(10) PRIMARY KEY NOT NULL UNIQUE  "
                + ", [XXWKTYPE] VARCHAR( 10 ) NULL"
                + ", [XXWKTITLE] TEXT NULL"
                + ", [XXWKCONTENT] TEXT NULL"
                + ", [XXWKURL] TEXT NULL"
                + ", [REDDATE] VARCHAR( 15)  NULL"
                + ", [INVALID] VARCHAR( 5)  NULL"
                + ", [XXWKTIME] VARCHAR( 15)  NULL)");
        return true;
    }
}
