package com.urovo.wgmf.db.dao.dao;

import android.content.Context;
import android.database.Cursor;

import com.urovo.wgmf.db.dao.DBTool;
import com.urovo.wgmf.db.dao.entitys.CXYKQEntity;
import com.urovo.wgmf.db.dao.entitys.CxykqDb;
import com.urovo.wugumofang.data.DateUtil;
import com.urovo.wugumofang.system.BaseAppManager;
import com.zsy.libcore.common.android.log.SYLoger;
import com.zsy.libext.utils.DateTimeUtils;

import java.text.ParseException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * 对数据库的增，删，改，查操作
 *
 * @author Administrator
 */
public class CxykqDbOperate {

    private static DBTool dbTool;

    public CxykqDbOperate(Context context) {
        super();
        dbTool = DBTool.getInstance(context);
    }

    /**
     * 插入数据
     */
    public static Boolean insert(CxykqDb cxykq) {
        Cursor cursor = null;
        try {
            String sql = "insert into cxykq(kqNO,bill,storeID,postID,skyID,kqtype,kqdate,kqtime,isup) values(?,?,?,?,?,?,?,?,?)";
            BaseAppManager.database().execSQL(sql, new String[]{cxykq.getKqNO() + "", cxykq.getBill(), cxykq.getStoreID(), cxykq.getPosID(), cxykq.getSkyID(), cxykq.getKqtype(), cxykq.getKqdate(), cxykq.getKqtime(), cxykq.getIsup() + ""});
            return true;
        } catch (Exception e) {
            BaseAppManager.loger().info("[CxykqDbOperate.insert]" + e);
            return false;
        } finally {
            // database.close();
            if (cursor != null) {
                cursor.close();
            }
        }
    }

    /**
     * 删除某行数据
     *
     * @param kqdate 考勤打卡的日期
     */
    public static void deleteRowData(String kqdate) {
        String sql1 = "delete from cxykq where kqdate='" + kqdate + "'";
        BaseAppManager.database().beginTransaction();
        try {
            BaseAppManager.database().execSQL(sql1);
            BaseAppManager.database().setTransactionSuccessful();
        } catch (Exception e) {
            BaseAppManager.loger().info("deleteRowData(" + kqdate + ")发生异常:" + e);
        } finally {
            BaseAppManager.database().endTransaction();
        }
    }

    /**
     * 查询方法的基方法，代码的复用，万精油
     *
     * @param sql
     * @return
     */
    public static List<CxykqDb> selectBy(String sql) {
        List<CxykqDb> list = new ArrayList<>();
        Cursor cursor = null;
        String sqlStr = "select * from cxykq ";
        if (sql != null) {
            sqlStr += sql;
        }
        try {
            cursor = BaseAppManager.database().rawQuery(sqlStr, new String[]{});
            while (cursor.moveToNext()) {
                CxykqDb model = new CxykqDb();
                model.setKqNO(cursor.getInt(cursor.getColumnIndex("kqNO")));
                model.setBill(cursor.getString(cursor.getColumnIndex("bill")));
                model.setStoreID(cursor.getString(cursor.getColumnIndex("storeID")));
                model.setPosID(cursor.getString(cursor.getColumnIndex("postID")));
                model.setSkyID(cursor.getString(cursor.getColumnIndex("skyID")));
                model.setKqtype(cursor.getString(cursor.getColumnIndex("kqtype")));
                model.setKqdate(cursor.getString(cursor.getColumnIndex("kqdate")));
                model.setKqtime(cursor.getString(cursor.getColumnIndex("kqtime")));
                model.setIsup(cursor.getInt(cursor.getColumnIndex("isup")));

//                if (model.getKqNO() + "" != null && model.getKqNO() + "".length() > 0) {
                list.add(model);
//                }
            }
        } catch (Exception e) {
            BaseAppManager.loger().info("[CxykqDbOperate.selectBy]" + e);
        } finally {
            if (cursor != null) {
                cursor.close();
            }
        }
        return list;
    }

    /**
     * 根据列名查询
     *
     * @param columnName 列名
     * @param bill       单号
     * @return 。。。
     */
    public static List<String> queryColumnByBill(String columnName, String bill) {
        String column = "";
        List<String> list = new ArrayList<>();
        Cursor cursor = null;
//        SELECT DISTINCT kqdate FROM cxykq WHERE kqdate is not null and kqdate !=''
        String sqlStr = "SELECT " + columnName + " FROM cxykq WHERE " + columnName + " is not null and " + columnName + " !=''+ and bill =?";
        try {
            cursor = BaseAppManager.database().rawQuery(sqlStr, new String[]{bill});
            while (cursor.moveToNext()) {
                column = cursor.getString(0);
                list.add(column);
            }
        } catch (Exception e) {
            BaseAppManager.loger().info(e);
        } finally {
            if (cursor != null) {
                cursor.close();
            }
        }
        return list;
    }


    /**
     * 根据列名查询
     *
     * @param columnName 列名
     * @return 。。。
     */
    public static List<String> queryColumn(String columnName) {
        String column = "";
        List<String> list = new ArrayList<>();
        Cursor cursor = null;
//        SELECT DISTINCT kqdate FROM cxykq WHERE kqdate is not null and kqdate !=''
        String sqlStr = "SELECT " + columnName + " FROM cxykq WHERE " + columnName + " is not null and " + columnName + " !=''";
        try {
            cursor = BaseAppManager.database().rawQuery(sqlStr, new String[]{});
            while (cursor.moveToNext()) {
                column = cursor.getString(0);
                list.add(column);
            }
        } catch (Exception e) {
            BaseAppManager.loger().info(e);
        } finally {
            if (cursor != null) {
                cursor.close();
            }
        }
        return list;
    }

    /**
     * 处理过期的考勤记录，当天考勤记录异常单时间大于等于第二天24点就清除（删除数据库过期考勤记录，并不上传）
     *
     * @return true: 存在过期的打卡记录，并且删除成功； false：不存在过期打卡记录，原数据都在
     */
    @SuppressWarnings("all")
    public static boolean deleteInvalidDate() {
        boolean flag = false;
        List<String> kqdateList = CxykqDbOperate.queryColumn("kqdate");
        SYLoger.print("wgmf--kqdateList: " + kqdateList);
        for (String kqdate : kqdateList) {
            try {
                String[] kqdateStr = kqdate.split(" ");
                if (kqdateStr.length != 0) {
                    long invalidDate = DateTimeUtils.getTomorrowTimestamp(kqdateStr[0] + " 24:00:00");
                    long currentDate = DateTimeUtils.getTimestamp(DateUtil.getDateByFormat(new Date(), "yyyy-MM-dd HH:mm:ss"));
                    SYLoger.print("wgmf--invalidDate str: " + kqdateStr[0] + " 24:00:00");
                    SYLoger.print("wgmf--currentDate: " + currentDate);
                    SYLoger.print("wgmf--invalidDate: " + invalidDate);
                    if (currentDate >= invalidDate) {
                        CxykqDbOperate.deleteRowData(kqdate);
                        SYLoger.print("wgmf--deleteRowData: successful!");
                        flag = true;
                    } else {
                        flag = false;
                    }
                }
            } catch (ParseException e) {
                e.printStackTrace();
            }
        }
        return flag;
    }

    /**
     * @param kqdate 数据库中保留的打卡日期
     * @return true: 存在过期的打卡记录，需要上传标记； false：不存在过期打卡记录，原数据都在
     */
    public static boolean upLoadInvalidDate(String kqdate) {
        boolean flag = false;
        try {
            String[] kqdateStr = kqdate.split(" ");
//      (废弃的需求)处理过期的考勤记录，当天考勤记录异常单时间大于等于第二天24点就清除（删除数据库过期考勤记录，并不上传）
//            if (kqdateStr.length != 0) {
//                long invalidDate = DateTimeUtils.getTomorrowTimestamp(kqdateStr[0] + " 24:00:00");
//                long currentDate = DateTimeUtils.getTimestamp(DateUtil.getDateByFormat(new Date(), "yyyy-MM-dd HH:mm:ss"));
//                if (currentDate >= invalidDate) {
//                    CxykqDbOperate.deleteRowData(kqdate);
//                    SYLoger.print("wgmf--deleteRowData: successful!");
//                    flag = true;
//                } else {
//                    flag = false;
//                }
//            }
            if (kqdateStr.length != 0) {
                long invalidDate = DateTimeUtils.getTimestamp(kqdateStr[0], "yyyy-MM-dd");
                long currentDate = DateTimeUtils.getTimestamp(DateUtil.getDateByFormat(new Date(), "yyyy-MM-dd"), "yyyy-MM-dd");
                SYLoger.print(invalidDate + "  --upLoadInvalidDate: successful--  " + currentDate);
                //过期日期小于当前日期，需要上传标记
                flag = (currentDate > invalidDate);
                SYLoger.print("upLoadInvalidDate: 是否有过期打卡记录： " + flag);
            }
        } catch (ParseException e) {
            e.printStackTrace();
        }
        return flag;
    }

    /**
     * 删除所有数据
     */
    public static Boolean deleteAllGoods(String sql) {
        Boolean result = false;
        String sqlStr = "Delete from cxykq ";
        if (sql != null) {
            sqlStr += sql;
        }
        try {
            BaseAppManager.database().execSQL(sqlStr);
            result = true;
        } catch (Exception e) {
            BaseAppManager.loger().info(e);
            result = false;
        } finally {
        }
        return result;
    }

    /**
     * 获取考勤未上传的状态
     *
     * @return
     */
    public static int GetUnUploadCount() {
        int total = 0;
        Cursor cursor = null;
        String sql = "select count(bill) from cxykq where IsUp=0";
        try {
            cursor = BaseAppManager.database().rawQuery(sql, null);
            if (cursor.moveToNext()) {
                total = cursor.getInt(0);
            }
        } catch (Exception e) {
            BaseAppManager.loger().info("获取未上传考勤单据总数发生异常:" + e);
        } finally {
            if (cursor != null) {
                cursor.close();
            }
        }
        return total;
    }

    @SuppressWarnings("unused")
    public static Boolean isKqtype(String signMode) {
        boolean signFlag = false;
        Cursor cursor = null;
        String sql = "select kqtype from cxykq where IsUp=0";
        try {
            cursor = BaseAppManager.database().rawQuery(sql, null);
            while (cursor.moveToNext()) {
                String kqType = cursor.getString(cursor.getColumnIndex("KQTYPE"));
                if (signMode != null && signMode.equals(kqType)) {
                    signFlag = true;
                }
            }
        } catch (Exception e) {
            BaseAppManager.loger().info("获取考勤类型发生异常:" + e);
        } finally {
            if (cursor != null) {
                cursor.close();
            }
        }
        return signFlag;

    }


    // / <summary>
    // / 单据上传成功后调用此函数，更新终端数据库上的单据状态IsUp 从0更新为1
    // / </summary>
    // / <param name="bill"></param>
    // public static void UpdateBillState(String bill)
    // {
    // string sqlCommand =
    // string.Format(@"update cxykq set IsUp=1 where bill='{0}'", bill);
    // try
    // {
    // SQLHelper.ExecuteNonQuery(sqlCommand, null);
    // }
    // catch (Exception ex)
    // {
    // logger.Error(string.Format("更新考勤单据上传状态发生异常:{0}\r\n{1}", ex.Message,
    // ex.StackTrace));
    // }
    // }

    /**
     * 获取未上传的考勤数据
     *
     * @return
     */
    public static List<CXYKQEntity> GetUnUpload() {
        List<CXYKQEntity> list = new ArrayList<CXYKQEntity>();
        Cursor cursor = null;
        String sql = "select  bill,storeid,posid,skyid,kqtype,kqdate,kqtime,IsUp,LONGITUDE,LATITUDE,valid,ADDRESS,IMAGEURL,MOREIMAGEURL from cxykq where IsUp=0";
        try {
            cursor = BaseAppManager.database().rawQuery(sql, null);
            while (cursor.moveToNext()) {
                CXYKQEntity ent = new CXYKQEntity();
                Date date = new Date();
                String bill = cursor.getString(cursor.getColumnIndex("BILL"));
                ent.BILL = bill == null ? "" : bill;
                String storeid = cursor.getString(cursor.getColumnIndex("STOREID"));
                ent.STOREID = storeid == null ? "" : storeid;
                String skyid = cursor.getString(cursor.getColumnIndex("SKYID"));
                ent.SKYID = skyid == null ? "" : skyid;
                String KQTYPE = cursor.getString(cursor.getColumnIndex("KQTYPE"));
                ent.KQTYPE = KQTYPE == null ? "" : KQTYPE;
                String KQDATE = cursor.getString(cursor.getColumnIndex("KQDATE"));
                ent.KQDATE = KQDATE == null ? DateUtil.getDate(date) : KQDATE;
                String KQTIME = cursor.getString(cursor.getColumnIndex("KQTIME"));
                ent.KQTIME = KQTIME == null ? DateUtil.getTime2(date) : KQTIME;
                String LONGITUDE = cursor.getString(cursor.getColumnIndex("LONGITUDE"));
                ent.LONGITUDE = LONGITUDE == null ? "" : LONGITUDE;
                String LATITUDE = cursor.getString(cursor.getColumnIndex("LATITUDE"));
                ent.LATITUDE = LATITUDE == null ? "" : LATITUDE;
                String Position_Valid = cursor.getString(cursor.getColumnIndex("valid"));
                ent.Position_Valid = Position_Valid == null ? "" : Position_Valid;
                String Address = cursor.getString(cursor.getColumnIndex("ADDRESS"));
                ent.Address = Address == null ? "" : Address;
                String ImageUrl = cursor.getString(cursor.getColumnIndex("IMAGEURL"));
                ent.ImageUrl = ImageUrl == null ? "" : ImageUrl;
                String MoreImageUrl = cursor.getString(cursor.getColumnIndex("MOREIMAGEURL"));
                ent.MoreImageUrl = MoreImageUrl == null ? "" : MoreImageUrl;
                list.add(ent);
            }
        } catch (Exception e) {

            BaseAppManager.loger().info("获取未上传的考勤数据异常:" + e);
        } finally {
            if (cursor != null) {
                cursor.close();
            }
        }
        return list;

    }

    /**
     * 更新上传信息
     *
     * @param bill
     */
    public static void UpdateBillState(String bill) {

        String sqlCommand = "update cxykq set IsUp=1 where bill=?";
        try {
            BaseAppManager.database().execSQL(sqlCommand, new String[]{bill});
        } catch (Exception ex) {
            BaseAppManager.loger().info("更新考勤单据" + bill + "上传状态发生异常:" + ex);
        }

    }

    /**
     * 更新上传信息
     *
     * @param bill
     * @param resault 0未上传  1上传成功  2坏单
     */
    public static void UpdateBillState(String bill, int resault) {

        String sqlCommand = "update cxykq set IsUp=" + resault + " where bill=?";
        try {
            BaseAppManager.database().execSQL(sqlCommand, new String[]{bill});
        } catch (Exception ex) {
            BaseAppManager.loger().info("更新考勤单据" + bill + "上传状态发生异常:" + ex);
        }

    }

    /**
     * 更新上传信息
     *
     * @param bill
     * @param skyId
     * @param moreimageurl 更改图片地址
     */
    public static void UpdateImageUrl(String bill, String skyId, String moreimageurl) {
        List<String> skyIdList = queryColumnByBill("skyid", bill);
        if (skyIdList.size() > 0) {
            String sqlCommand = "update cxykq set MOREIMAGEURL=? where bill=?";
            try {
                BaseAppManager.database().execSQL(sqlCommand, new String[]{moreimageurl, bill});
            } catch (Exception ex) {
                BaseAppManager.loger().info("更新考勤单据" + bill + "上传状态发生异常:" + ex);
            }
        } else {
            String sqlCommand = "update cxykq set MOREIMAGEURL=?,SKYID=? where bill=?";
            try {
                BaseAppManager.database().execSQL(sqlCommand, new String[]{moreimageurl, skyId, bill});
            } catch (Exception ex) {
                BaseAppManager.loger().info("更新考勤单据" + bill + "上传状态发生异常:" + ex);
            }


        }
    }


    /**
     * 保存考勤单到本地数据库
     */
    public static void SaveKaoQin(CXYKQEntity bll) {
        String sql = "insert into CXYKQ(bill,storeid,posid,skyid,kqtype,kqdate,kqtime,IsUp,LATITUDE,LONGITUDE,ADDRESS,IMAGEURL,valid,MOREIMAGEURL) " + " values(?,?,?,?,?,?,?,?,?,?,?,?,?,?)";
        try {
            BaseAppManager.database().execSQL(sql, new Object[]{bll.BILL, bll.STOREID, bll.POSID, bll.SKYID, bll.KQTYPE, bll.KQDATE, bll.KQTIME, 0, bll.LATITUDE, bll.LONGITUDE, bll.Address, bll.ImageUrl, bll.Position_Valid, bll.MoreImageUrl});
        } catch (Exception e) {

            BaseAppManager.loger().info("保存考勤单据发生异常:" + e);
        }
    }

}