package com.urovo.wgmf.db.dao.dao;

import android.content.Context;
import android.database.Cursor;

import com.urovo.wgmf.db.dao.DBTool;
import com.urovo.wgmf.db.dao.bill.DalCommon;
import com.urovo.wgmf.db.dao.entitys.Djdb002Db;
import com.urovo.wgmf.db.dao.entitys.ProductEntity;
import com.urovo.wugumofang.data.Arith;
import com.urovo.wugumofang.system.BaseAppManager;

import java.util.ArrayList;
import java.util.List;

/**
 * 对数据库的增，删，改，查操作
 * 
 * @author Administrator
 * 
 */
public class Djdb002DbOperate {

	private static DBTool dbTool;

	public Djdb002DbOperate(Context context) {
		super();
		dbTool = DBTool.getInstance(context);
	}

	/**
	 * 插入数据
	 */
	public static Boolean Insert(Djdb002Db djdb002) {
		Cursor cursor = null;
		try {
			String sql = "insert into djdb002(bill,itemID,pack,unit,sqty,tqty,price,comm,seq_NO) values(?,?,?,?,?,?,?,?,?)";
			BaseAppManager.database().execSQL(sql, new String[]{ djdb002.getBill(), djdb002.getItemID(), djdb002.getPack() + "", djdb002.getUnit() + "", djdb002.getSqty() + "", djdb002.getTqty() + "", djdb002.getPrice() + "", djdb002.getComm(), djdb002.getSeq_NO() + "", });
			return true;
		}
		catch (Exception e) {
			BaseAppManager.loger().info("[Djdb002DbOperate.Insert]" + e);
			return false;
		}
		finally {
			// database.close();
			if(cursor != null)
				cursor.close();
		}
	}

	/**
	 * 查询方法的基方法，代码的复用，万精油
	 * 
	 * @param sql
	 * @return
	 */
	public static List<Djdb002Db> selectBy(String sql) {
		List<Djdb002Db> list = new ArrayList<Djdb002Db>();
		Cursor cursor = null;
		String sqlStr = "select * from djdb002";
		if(sql != null)
			sqlStr += sql;
		try {
			cursor = BaseAppManager.database().rawQuery(sqlStr, new String[]{});
			while (cursor.moveToNext()) {
				Djdb002Db model = new Djdb002Db();
				model.setBill(cursor.getString(cursor.getColumnIndex("bill")));
				model.setItemID(cursor.getString(cursor.getColumnIndex("itemID")));
				model.setPack(cursor.getInt(cursor.getColumnIndex("pack")));
				model.setUnit(cursor.getInt(cursor.getColumnIndex("unit")));
				model.setSqty(cursor.getInt(cursor.getColumnIndex("sqty")));
				model.setTqty(cursor.getInt(cursor.getColumnIndex("tqty")));
				model.setPrice(cursor.getInt(cursor.getColumnIndex("price")));
				model.setComm(cursor.getString(cursor.getColumnIndex("comm")));
				model.setSeq_NO(cursor.getInt(cursor.getColumnIndex("seq_NO")));
				if(model.getBill() != null && model.getBill().length() > 0) {
					list.add(model);
				}
			}
		}
		catch (Exception e) {
			BaseAppManager.loger().info("[Djdb002DbOperate.selectBy]" + e);
		}
		finally {
			if(cursor != null) {
				cursor.close();
			}
		}
		return list;
	}

	/**
	 * 删除所有数据
	 */
	public static Boolean deleteAllGoods(String sql) {
		Boolean result = false;
		String sqlStr = "Delete from djdb002";
		if(sql != null) {
			sqlStr += sql;
		}
		try {
			BaseAppManager.database().execSQL(sqlStr);
			result = true;
		}
		catch (Exception e) {
			BaseAppManager.loger().info("[Djdb002DbOperate.deleteAllGoods]" + e);
			result = false;
		}
		finally {
		}
		return result;
	}

	/**
	 * 保存调拨出库单明细数据到本地数据库
	 * 
	 * @param bill
	 * @param list
	 * @return
	 */
	public static boolean SaveDiaoBoChuKuDetail(String bill, List<ProductEntity> list) {
		String sql = "insert into DJDB002(BILL,ITEMID,PACK,UNIT,SQTY,TQTY,PRICE,COMM,SEQ_NO) values(?,?,?,?,?,?,?,?,?)";
		try {
			BaseAppManager.database().beginTransaction();
			for (ProductEntity p : list) {
				BaseAppManager.database().execSQL(sql, new Object[]{ bill, p.getItemId(), p.getTQTY(), 1, 0, p.getTQTY(), p.getPrice(), "", p.getSeq_No() });
			}
			BaseAppManager.database().setTransactionSuccessful();
		}
		catch (Exception e) {

			BaseAppManager.loger().info("保存调拨出货单明细表发生异常:" + e);
			return false;
		}
		finally {
			BaseAppManager.database().endTransaction();
		}
		return true;
	}

	/**
	 * 根据调拨出货单据号获取产品明细
	 * 
	 * @param bill
	 * @return
	 */
	public static List<ProductEntity> GetBillDetail(String bill) {
		List<ProductEntity> list = new ArrayList<ProductEntity>();
		Cursor cursor = null;
		String sql = "select itemid,pack,unit,sqty,tqty,price,comm,seq_no from djdb002 where bill='" + bill + "'";
		try {
			cursor = BaseAppManager.database().rawQuery(sql, null);
			while (cursor.moveToNext()) {
				ProductEntity ent = new ProductEntity();
				ent.setItemId(cursor.isNull(0) ? "" : cursor.getString(0));
				ent.setUnit(cursor.isNull(2) ? 0 : cursor.getDouble(2));
				ent.setTQTY(cursor.isNull(4) ? 0 : cursor.getDouble(4));
				ent.setPrice(cursor.isNull(5) ? 0 : cursor.getDouble(5));
				ent.setComm(cursor.isNull(6) ? "" : cursor.getString(6));
				ent.setSeq_No(cursor.isNull(7) ? Arith.add(list.size(), 1) : cursor.getDouble(7));

				if(!DalCommon.CanEditAmount(ent.getItemId())) // 对于不可修改金额产品：金额=单价*数量
				{
					ent.setAmount(Arith.mul(ent.getPrice(), ent.getTQTY()));
				}
				else {
					ent.setAmount(ent.getPrice()); // 对于可修改金额产品：金额=单价
				}
				list.add(ent);
			}
		}
		catch (Exception e) {

			BaseAppManager.loger().info("根据调拨出货单号获取产品明细发生异常:" + e);
		}
		finally {
			if(cursor != null) {
				cursor.close();
			}
		}
		return list;
	}
}
