package com.urovo.wugumofang.data;

import android.app.DatePickerDialog;
import android.content.Context;
import android.text.TextUtils;
import android.widget.DatePicker;
import android.widget.TextView;

import java.lang.ref.WeakReference;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.ParsePosition;
import java.text.SimpleDateFormat;
import java.time.LocalDateTime;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.Locale;

/**
 * 功能描述：
 *
 * @author Administrator
 * @version 1.0
 * @Date Jul 19, 2008
 * @Time 9:47:53 AM
 */
public class DateUtil {

    public static Date date = null;

    public static DateFormat dateFormat = null;

    public static Calendar calendar = null;

    /**
     * 功能描述：格式化日期
     *
     * @param dateStr String 字符型日期
     * @param format  String 格式
     * @return Date 日期
     */
    public static Date parseDate(String dateStr, String format) {
        try {
            dateFormat = new SimpleDateFormat(format, Locale.CHINA);
            String dt = dateStr.replaceAll("-", "/");
            if ((!dt.equals("")) && (dt.length() < format.length())) {
                dt += format.substring(dt.length()).replaceAll("[YyMmDdHhSs]", "0");
            }
            date = (Date) dateFormat.parse(dt);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return date;
    }

    /**
     * 功能描述：格式化日期
     *
     * @param dateStr String 字符型日期：YYYY-MM-DD 格式
     * @return Date
     */
    public static Date parseDate(String dateStr) {
        return parseDate(dateStr, "yyyy/MM/dd");
    }

//    public static Date parseDate(String dateStr,String format) {
//        return parseDate(dateStr, format);
//    }

    // /**
    // * 功能描述：格式化日期
    // *
    // * @param dateStr
    // * String 字符型日期：YYYY-MM-DD 格式
    // * @return Date
    // */
    // public static Date parsesDateTime(String dateStr){
    // return parseDate(dateStr, "yyyy-MM-dd");
    // }

    /**
     * 功能描述：格式化输出日期
     *
     * @param date   Date 日期
     * @param format String 格式
     * @return 返回字符型日期
     */
    public static String format(Date date, String format) {
        String result = "";
        try {
            if (date != null) {
                dateFormat = new SimpleDateFormat(format);
                result = dateFormat.format(date);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return result;
    }

    /**
     * 功能描述：
     *
     * @param date Date 日期
     * @return
     */
    public static String format(Date date) {
        return format(date, "yyyy/MM/dd");
    }

    /**
     * 功能描述：返回年份
     *
     * @param date Date 日期
     * @return 返回年份
     */
    public static int getYear(Date date) {
        calendar = Calendar.getInstance();
        calendar.setTime(date);
        return calendar.get(Calendar.YEAR);
    }

    /**
     * 功能描述：返回当前年份
     *
     * @return 返回年份
     */
    public static int getCurrentYear() {
        calendar = Calendar.getInstance();
        return calendar.get(Calendar.YEAR);
    }

    /**
     * 功能描述：返回月份
     *
     * @param date Date 日期
     * @return 返回月份
     */
    public static int getMonth(Date date) {
        calendar = Calendar.getInstance();
        calendar.setTime(date);
        return calendar.get(Calendar.MONTH) + 2;
    }

    /**
     * 功能描述：返回当前月份
     *
     * @return 返回月份
     */
    public static int getCurrentMonth() {
        calendar = Calendar.getInstance();
        return calendar.get(Calendar.MONTH) + 1;
    }

    /**
     * 功能描述：返回日份
     *
     * @param date Date 日期
     * @return 返回日份
     */
    public static int getDay(Date date) {
        calendar = Calendar.getInstance();
        calendar.setTime(date);
        return calendar.get(Calendar.DAY_OF_MONTH);
    }

    /**
     * 功能描述：返回小时
     *
     * @param date 日期
     * @return 返回小时
     */
    public static int getHour(Date date) {
        calendar = Calendar.getInstance();
        calendar.setTime(date);
        return calendar.get(Calendar.HOUR_OF_DAY);
    }

    /**
     * 功能描述：返回分钟
     *
     * @param date 日期
     * @return 返回分钟
     */
    public static int getMinute(Date date) {
        calendar = Calendar.getInstance();
        calendar.setTime(date);
        return calendar.get(Calendar.MINUTE);
    }

    /**
     * 返回秒钟
     *
     * @param date Date 日期
     * @return 返回秒钟
     */
    public static int getSecond(Date date) {
        calendar = Calendar.getInstance();
        calendar.setTime(date);
        return calendar.get(Calendar.SECOND);
    }

    /**
     * 功能描述：返回毫秒
     *
     * @param date 日期
     * @return 返回毫秒
     */
    public static long getMillis(Date date) {
        calendar = Calendar.getInstance();
        calendar.setTime(date);
        return calendar.getTimeInMillis();
    }

    /**
     * 功能描述：返回字符型日期
     *
     * @param date 日期
     * @return 返回字符型日期 yyyy-MM-dd 格式
     */
    public static String getDate(Date date) {
        return format(date, "yyyy-MM-dd");
    }

    /**
     * 功能描述：返回字符型日期
     *
     * @param date 日期
     * @return 返回字符型日期 yyyy-MM-dd 格式
     */
    public static String getDateMonth(Date date) {
        return format(date, "yyyy-MM");
    }

    /**
     * 功能描述：返回字符型上个月日期
     *
     * @param date 日期
     * @return 返回字符型日期 yyyy-MM 格式
     */
    public static String getDateUpperMonth(Date date){
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        calendar.set(calendar.get(Calendar.YEAR), calendar.get(Calendar.MONTH) - 1, 1);
        return format(calendar.getTime(),"yyyy-MM");
    }

    /**
     * 获取特殊规则日期
     * Rule:日期为1号，则返回当前月份上个月月份
     *      日期超过1号，则返回当前月份
     */
    public static String getSpecialRuleDate() {
        Date date =new Date();
        int day = DateUtil.getDay(date);
        if (day == 1) {
            return getDateUpperMonth(date);
        } else {
            return getDateMonth(date);
        }
    }

    /**
     * 功能描述：返回字符型时间
     *
     * @param date Date 日期
     * @return 返回字符型时间 HHmmss 格式
     */
    public static String getTime(Date date) {
        return format(date, "HHmmss");
    }

    /**
     * 功能描述：返回字符型时间
     *
     * @param date Date 日期
     * @return 返回字符型时间 HH:mm:ss 格式
     */
    public static String getTime2(Date date) {
        return format(date, "HH:mm:ss");
    }

    /**
     * 功能描述：返回字符型日期时间
     *
     * @param date Date 日期
     * @return 返回字符型日期时间 yyyy-MM-dd HH:mm:ss 格式
     */
    public static String getDateTime(Date date) {
        return format(date, "yyyy-MM-dd HH:mm:ss");
    }

    /**
     * 功能描述：返回字符型日期时间
     *
     * @param date Date 日期
     * @return 返回字符型日期时间 yyyy-MM-dd HH:mm:ss 格式
     */
    public static String getDateTime2(Date date) {
        return format(date, "MM-dd HH:mm");
    }

    /**
     * 功能描述：返回字符型日期时间
     *
     * @param date Date 日期
     * @return 返回字符型日期时间 yyyyMMddHHmmss 格式
     */
    public static String getDateTime1(Date date) {
        return format(date, "yyyyMMddHHmmss");
    }

    /**
     * 功能描述：日期相加
     *
     * @param date Date 日期
     * @param day  int 天数
     * @return 返回相加后的日期
     */
    public static Date addDate(Date date, int day) {
        calendar = Calendar.getInstance();
        long millis = getMillis(date) + ((long) day) * 24 * 3600 * 1000;
        calendar.setTimeInMillis(millis);
        return calendar.getTime();
    }

    /**
     * 功能描述：
     *
     * @param date Date 日期
     * @param day  int 天数
     * @return 返回相加后的日期
     */
    public static Date diffDate(Date date, int day) {
        calendar = Calendar.getInstance();
        long millis = getMillis(date) - ((long) day) * 24 * 3600 * 1000;
        calendar.setTimeInMillis(millis);
        return calendar.getTime();
    }


    /**
     * 功能描述：日期相减
     *
     * @param date  Date 日期
     * @param date1 Date 日期
     * @return 返回相减后的日期
     */
    public static int diffDate(Date date, Date date1) {
        return (int) ((getMillis(date) - getMillis(date1)) / (24 * 3600 * 1000));
    }

    /**
     * 功能描述：日期相减（针对String类型的日期）
     *
     */
    public static int diffDate(String format,String dateStr1, String dateStr2) {

        Date date1 = null;
        Date date2 = null;

        try {
            date1 = new SimpleDateFormat(format).parse(dateStr1);
            date2 = new SimpleDateFormat(format).parse(dateStr2);
            return (int) ((getMillis(date1) - getMillis(date2)) / (24 * 3600 * 1000));
        } catch (ParseException e) {
            e.printStackTrace();
            return 0;
        }
    }

    /**
     * 功能描述：日期相减
     *
     * @param date  Date 日期
     * @param date1 Date 日期
     * @return 返回相减后的日期
     */
    public static int diffDate(long date, long date1) {
        return (int) (date - date1) / (24 * 3600 * 1000);
    }

    /**
     * 功能描述：取得指定月份的第一天
     *
     * @param strdate String 字符型日期
     * @return String yyyy-MM-dd 格式
     */
    public static String getMonthBegin(String strdate) {
        date = parseDate(strdate);
        return format(date, "yyyy-MM") + "-01";
    }

    /**
     * 返回指定日期的月的第一天
     *
     * @param date 指定日期
     */
    public static Date getFirstDayOfMonth(Date date) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        calendar.set(calendar.get(Calendar.YEAR),
                calendar.get(Calendar.MONTH), 1);
        return calendar.getTime();
    }


    /**
     * 功能描述：取得指定月份的最后一天
     *
     * @param strdate String 字符型日期
     * @return String 日期字符串 yyyy-MM-dd格式
     */
    public static String getMonthEnd(String strdate) {
        date = parseDate(getMonthBegin(strdate));
        calendar = Calendar.getInstance();
        calendar.setTime(date);
        calendar.add(Calendar.MONTH, 1);
        calendar.add(Calendar.DAY_OF_YEAR, -1);
        return formatDate(calendar.getTime());
    }

    /**
     * 功能描述：常用的格式化日期
     *
     * @param date Date 日期
     * @return String 日期字符串 yyyy-MM-dd格式
     */
    public static String formatDate(Date date) {
        return formatDateByFormat(date, "yyyy-MM-dd");
    }

    /**
     * 功能描述：以指定的格式来格式化日期
     *
     * @param date   Date 日期
     * @param format String 格式
     * @return String 日期字符串
     */
    public static String formatDateByFormat(Date date, String format) {
        String result = "";
        if (date != null) {
            try {
                SimpleDateFormat sdf = new SimpleDateFormat(format);
                result = sdf.format(date);
            } catch (Exception ex) {
                ex.printStackTrace();
            }
        }
        return result;
    }

    public static String getFormatNowTime() {
        Date date = new Date();

        return getDateTime(date);

    }

    public static void main(String[] args) {
        Date d = new Date();
    }

    public static int[] getYearMonthDay(Date date) {
        calendar = Calendar.getInstance();
        calendar.setTime(date);
        return new int[]{calendar.get(Calendar.YEAR), calendar.get(Calendar.MONTH), calendar.get(Calendar.DAY_OF_MONTH)};

    }

    /**
     * 获得指定日期的前一天
     *
     * @param date ...
     * @return ...
     */
    public static String getDayBefore(Date date) {
        Calendar c = Calendar.getInstance();
        c.setTime(date);
        int day = c.get(Calendar.DATE);
        c.set(Calendar.DATE, day - 1);

        String dayBefore = new SimpleDateFormat("yyyy-MM-dd", Locale.CHINA).format(c.getTime());
        return dayBefore;
    }

    /**
     * 获得指定日期的后一天
     *
     * @param date ...
     * @return ...
     */
    public static String getDayAfter(Date date) {
        Calendar c = Calendar.getInstance();
        c.setTime(date);
        int day = c.get(Calendar.DATE);
        c.set(Calendar.DATE, day + 1);
        return new SimpleDateFormat("yyyy-MM-dd", Locale.CHINA).format(c.getTime());
    }

    /**
     * 获得指定日期的后数小时
     *
     * @param date ...
     * @return ...
     */
    public static Date getDayAfterHours(Date date,int hour) {
        Calendar c = Calendar.getInstance();
        c.setTime(date);
        int oldHour = c.get(Calendar.HOUR);
        c.set(Calendar.HOUR, oldHour + hour);
        return c.getTime();
    }

    /**
     * 获得指定日期的前beforeday天
     *
     * @param date      ...
     * @param beforeday ...
     * @return ...
     */
    public static String getDayBefore(Date date, int beforeday) {
        Calendar c = Calendar.getInstance();
        c.setTime(date);
        int day = c.get(Calendar.DATE);
        c.set(Calendar.DATE, day - beforeday);
        return new SimpleDateFormat("yyyy-MM-dd",Locale.CHINA).format(c.getTime());
    }

    /**
     * 获得指定日期的前beforeday天的Date
     */
    public static Date getDateBefore(Date date, int beforeday) {
        Calendar c = Calendar.getInstance();
        c.setTime(date);
        int day = c.get(Calendar.DATE);
        c.set(Calendar.DATE, day - beforeday);
        return c.getTime();
    }

    /**
     * 获得指定日期的前specifiedDay天,并指定格式日期
     *
     * @param date         ...
     * @param specifiedDay -1：昨天；0：今天；1：明天
     * @param format       ...
     * @param locale       ...
     * @return ...
     */
    public static String getSpecifiedDay(Date date, int specifiedDay, String format, Locale locale) {
        Calendar c = Calendar.getInstance();
        c.setTime(date);
        int day = c.get(Calendar.DATE);
        c.set(Calendar.DATE, day + specifiedDay);

        return new SimpleDateFormat(format, locale).format(c.getTime());
    }

    /**
     * 获得指定日期的前specifiedDay天,并指定格式日期
     *
     * @param date          ...
     * @param spectifiedDay ...
     * @param format        ...
     * @return ...
     */
    public static String getSpecifiedDay(String date, int spectifiedDay, String format) {
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat(format, Locale.CHINA);
        Date parse = simpleDateFormat.parse(date, new ParsePosition(0));
        return getSpecifiedDay(parse, spectifiedDay, format, Locale.CANADA);
    }

    /**
     * 获得指定日期的前specifiedMonth月,并指定格式日期
     *
     * @param date           ...
     * @param specifiedMonth ...
     * @param format         ...
     * @param locale         ...
     * @return ...
     */
    public static String getSpecifiedMonth(Date date, int specifiedMonth, String format, Locale locale) {
        Calendar ca = Calendar.getInstance();//得到一个Calendar的实例
        ca.setTime(date); // 设置时间为当前时间
        ca.add(Calendar.MONTH, specifiedMonth); // 月份加减

        return new SimpleDateFormat(format, locale).format(ca.getTime());
    }

    /**
     * 获得指定日期的前specifiedMonth月,并指定格式日期
     *
     * @param date           ...
     * @param specifiedMonth ...
     * @param format         ...
     * @return ...
     */
    public static String getSpecifiedMonth(String date, int specifiedMonth, String format) {
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat(format, Locale.CHINA);
        Date parse = simpleDateFormat.parse(date, new ParsePosition(0));
        return getSpecifiedMonth(parse, specifiedMonth, format, Locale.CANADA);
    }

    /**
     * 获得指定日期的前specifiedYear年,并指定格式日期
     *
     * @param date          ...
     * @param specifiedYear ...
     * @param format        ...
     * @param locale        ...
     * @return ...
     */
    public static String getSpecifiedYear(Date date, int specifiedYear, String format, Locale locale) {
        Calendar ca = Calendar.getInstance();//得到一个Calendar的实例
        ca.setTime(date); // 设置时间为当前时间
        ca.add(Calendar.YEAR, specifiedYear); // 年份加减

        return new SimpleDateFormat(format, locale).format(ca.getTime());
    }

    /**
     * 获得指定日期的前specifiedYear年,并指定格式日期
     *
     * @param date          ...
     * @param specifiedYear ...
     * @param format        ...
     * @return ...
     */
    public static String getSpecifiedYear(String date, int specifiedYear, String format) {
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat(format, Locale.CHINA);
        Date parse = simpleDateFormat.parse(date, new ParsePosition(0));
        return getSpecifiedYear(parse, specifiedYear, format, Locale.CANADA);
    }

    /**
     * 获得指定日期的前一天 yyyy-MM-dd HH:mm:ss
     *
     * @param date ...
     * @return ...
     */
    public static String getDayBefore2(Date date) {
        Calendar c = Calendar.getInstance();
        c.setTime(date);
        int day = c.get(Calendar.DATE);
        c.set(Calendar.DATE, day - 1);
        return new SimpleDateFormat("yyyy-MM-dd HH:mm:ss", Locale.CHINA).format(c.getTime());
    }

    public static String getDayAfter2(Date date) {
        Calendar c = Calendar.getInstance();
        c.setTime(date);
        int day = c.get(Calendar.DATE);
        c.set(Calendar.DATE, day + 1);
        return new SimpleDateFormat("yyyy-MM-dd HH:mm:ss", Locale.CHINA).format(c.getTime());
    }

    /**
     * 原getDateBefore8Hours()改来的，说是不用再把时间往前推8小时了。
     */
    public static String getDateByFormat(Date date, String format) {
//        Calendar c = Calendar.getInstance();
//        c.setTime(date);
//        c.add(Calendar.HOUR_OF_DAY, -8);
//        String before = new SimpleDateFormat(format).format(c.getTime());
        return new SimpleDateFormat(format).format(date);
    }

    public static String getDateBefore8Hours(Date date, String format) {
        Calendar c = Calendar.getInstance();
        c.setTime(date);
        c.add(Calendar.HOUR_OF_DAY, -8);
        String before = new SimpleDateFormat(format).format(c.getTime());
        return before;
    }

    public static String getDateAfter8Hours(Date date, String format) {
        Calendar c = Calendar.getInstance();
        c.setTime(date);
        c.add(Calendar.HOUR_OF_DAY, 8);
        String before = new SimpleDateFormat(format).format(c.getTime());
        return before;
    }

    public static String getDateByFormat(String dayformat) {
        Date parseDate = parseDate(dayformat);
        Calendar c = Calendar.getInstance();
        c.setTime(parseDate);
        c.add(Calendar.HOUR_OF_DAY, -8);
        String before = new SimpleDateFormat("yyyy-MM-dd").format(c.getTime());
        return before;
    }

    /**
     * 将一定格式的时间字符串转换成Date
     *
     * @param str 一定格式的时间
     * @return
     */
    public static Date String2Date(String str, String format) {
        if (TextUtils.isEmpty(str)) return null;
        SimpleDateFormat sdf = new SimpleDateFormat(format);
        Date date = null;
        try {
            date = sdf.parse(str);
        } catch (ParseException e) {

        }
        return date;
    }

    /**
     * 两个日期相差的天数
     *
     * @param sdate 开始的日期
     * @param edate 结束的日期
     * @return
     */
    public static int dyasBetween(Date sdate, Date edate) {
        Calendar cal = Calendar.getInstance();
        cal.setTime(sdate);
        long time1 = cal.getTimeInMillis();// 开始的时间
        cal.setTime(edate);
        long time2 = cal.getTimeInMillis(); // 结束的时间
        long between_days = (time2 - time1) / (1000 * 3600 * 24);
        return (int) between_days;
    }

    /**
     * 两个日期相差的月份数
     *
     * @param startDate
     * @param endDate
     * @return
     */
    public static int monthBetween(Date startDate, Date endDate) {
        Calendar startCalendar = new GregorianCalendar();
        startCalendar.setTime(startDate);
        Calendar endCalendar = new GregorianCalendar();
        endCalendar.setTime(endDate);

        int diffYear = endCalendar.get(Calendar.YEAR) - startCalendar.get(Calendar.YEAR);
        return diffYear * 12 + endCalendar.get(Calendar.MONTH) - startCalendar.get(Calendar.MONTH);
    }

    /**
     * 比较两个格式为“2016-05-28”或“2016-05”之类的格式相同的字符串时间，
     *
     * @param dateL ...
     * @param dateR ...
     * @return ...
     */
    public static int compareDate(String dateL, String dateR) {
        int date1 = Integer.parseInt(dateL.replaceAll("-", ""));
        int date2 = Integer.parseInt(dateR.replaceAll("-", ""));
        if (date1 > date2) {//如果始时间大于末时间，
            return 1;
        } else if (date1 < date2) {
            return -1;
        } else {
            return 0;
        }
    }

    /**
     * @param anyValue   任意一个数(ps:上一个月)
     * @param referValue 参照数(ps:本月)
     * @return 1:任意两个数满足(-∞,referValue-1]或者[referValue,+∞,ps:不符合条件的情况)
     * 0:任意两个数相同(ps:月份相同,既是本月)
     * -1：前一个数等于后一个数减一(ps:前一个月和本月,2017年12月和2018年01月,跨年月需要特殊处理)
     * -2：未知错误
     */
    public static int compareSize(String anyValue, String referValue) {
        int anyDate = Integer.parseInt(anyValue.replaceAll("-", ""));
        int referDate = Integer.parseInt(referValue.replaceAll("-", ""));
        String[] anyDateArray = anyValue.split("-");
        String[] referDateArray = referValue.split("-");
        //处理年底12月和来年1月之间判断
        if (Integer.parseInt(anyDateArray[0]) < Integer.parseInt(referDateArray[0])
                && Integer.parseInt(anyDateArray[1]) == 12
                && Integer.parseInt(referDateArray[1]) == 1) {
            return -1;
        } else if ((anyDate > referDate || anyDate < referDate - 1)) {
            return 1;
        } else if (anyDate == referDate) {
            return 0;
        } else if (anyDate == referDate - 1) {
            return -1;
        } else {
            return -2;
        }
    }


    /**
     * 2017-05-23 转化 为2017年05月23日
     *
     * @param str ...
     * @return ...
     */
    public static String convertFormat(String str) {
        String[] container = str.split("-");
        return String.format("%s年%s月%s日", container[0], container[1], container[2]);
    }

    /***
     * 功能：点击弹出选择日期并显示。针对显示日期的TextView.
     *
     * @param context  ...
     * @param textView 需要显示日期的TextView
     * @param format   显示日期的格式  eg."yyyy-MM-dd"
     */
    public static void pickDialog(Context context, final TextView textView, final String format) {
        pickDialog(context,textView,format,null,null).show();
    }

    public interface SelectCallBack{
        void onSelected();
    }

    /**
     * 带 起始时间 和 截止时间 限定的时间选择器（记得最后自己show()）
     * @param context
     * @param startLimit 起始时间限制
     * @param endLimit 截止时间限制
     * @return
     */
    public static DatePickerDialog pickDialog(Context context, final TextView textView, final String format,Date startLimit,Date endLimit) {
        return pickDialog(context, textView,format, startLimit, endLimit,null);
    }

    public static DatePickerDialog pickDialog(Context context, final TextView textView, final String format, Date startLimit, Date endLimit, final SelectCallBack selectCallBack) {
        WeakReference<Context> refContext = new WeakReference<>(context);
        final WeakReference<TextView> refText = new WeakReference<>(textView);

        Calendar c = Calendar.getInstance();
        if (refContext.get() == null) {
            return null;
        }
        DatePickerDialog dialog = new DatePickerDialog(refContext.get(),
                new DatePickerDialog.OnDateSetListener() {
                    SimpleDateFormat sdf = new SimpleDateFormat(format, Locale.CHINA);

                    @Override
                    public void onDateSet(DatePicker view, int year,
                                          int monthOfYear, int dayOfMonth) {
                        Calendar cal = Calendar.getInstance();
                        cal.set(year, monthOfYear, dayOfMonth);
                        if(refText.get() != null){
                            refText.get().setText(sdf.format(cal.getTime()));
                        }
                        if (selectCallBack != null){//选择完后的回调
                            selectCallBack.onSelected();
                        }
                    }
                }, c.get(Calendar.YEAR), c.get(Calendar.MONTH), c.get(Calendar.DAY_OF_MONTH));
        //设置时间限定（必须要减一点，否则小米手机会报错：IllegalArgumentException Fromdate does not precede toDate）
        //2019-10-27 20:54:50 据说上述的是API 21 的CalenderView的BUG，在更高版本的SDK中已经修复了。
        //https://stackoverflow.com/questions/43322770/android-illegalargumentexception-for-date-picker
        if (startLimit != null){
            dialog.getDatePicker().setMinDate(startLimit.getTime() /*- 10000*/);
        }
        if (endLimit != null){
            dialog.getDatePicker().setMaxDate(endLimit.getTime() /*- 10000*/);
        }
        return dialog;
    }


        /**
         * 调用系统日历
         *
         * @param context ...
         * @param date    ...
         */
    public static void proDatePicker(Context context, final TextView date) {
        Calendar c = Calendar.getInstance();
        DatePickerDialog datePickerDialog = new DatePickerDialog(context, new DatePickerDialog.OnDateSetListener() {
            @Override
            public void onDateSet(DatePicker view, int year, int monthOfYear, int dayOfMonth) {
                date.setText(String.format(Locale.CHINA, "%04d-%02d-%02d", year, monthOfYear + 1, dayOfMonth));
            }
        }, c.get(Calendar.YEAR), c.get(Calendar.MONTH), c.get(Calendar.DAY_OF_MONTH));
        datePickerDialog.getDatePicker().setCalendarViewShown(false);
        datePickerDialog.show();
    }

    /**
     * 获取指定偏移月份的第一天（如，前几个月的1号）
     *
     * @param offset
     * @return
     */
    public static Date getFirstDayOfMonth(int offset) {
        Calendar cal = Calendar.getInstance();
        cal.add(Calendar.MONTH, offset);
        cal.set(Calendar.DATE, 1);
        return cal.getTime();
    }


    /**
     * 获取当前月份的第一天
     */
    public static Date getFirstDayOfMonth() {
        Calendar cal = Calendar.getInstance();
        cal.set(Calendar.DATE, 1);
        return cal.getTime();
    }

    /**
     * 获取指定偏移月份的最后一天（如，前几个月的31号）
     *
     * @param offset
     * @return
     */
    public static Date getLastDayOfMonth(int offset) {
        Calendar cal = Calendar.getInstance();
        cal.add(Calendar.MONTH, offset);
        cal.set(Calendar.DATE, cal.getActualMaximum(Calendar.DAY_OF_MONTH));
        return cal.getTime();
    }


    /**
     * 根据自定义格式返回当前时间
     *
     * @param format yyMMddHHmmss等格式可自定义
     * @return
     */
    public static String getFormatCurrentDate(String format) {
        SimpleDateFormat formatter = new SimpleDateFormat(format);
        Date curDate = new Date(System.currentTimeMillis());
        return formatter.format(curDate);
    }
}
