package com.urovo.wugumofang.loader

import android.content.Context
import android.os.Handler
import android.widget.Toast
import com.urovo.wgmf.db.dao.dao.PrivilegeDbOperate
import com.urovo.wgmf.db.dao.entitys.PrivilegeRes
import com.urovo.wugumofang.constants.ApiUrls
import com.urovo.wugumofang.constants.NetParam
import com.urovo.wugumofang.net.OkHttpDominator
import com.urovo.wugumofang.system.BaseAppManager
import com.urovo.wugumofang.utils.MyToast
import com.urovo.wugumofang.view.ProgressBarDialog
import okhttp3.Request
import org.jetbrains.anko.doAsync
import org.jetbrains.anko.uiThread
import java.lang.ref.WeakReference
import java.util.*
import kotlin.concurrent.timerTask

/**
 * @author Sasuke on 2018/3/19.
 * 权限下载
 */
class PrivilegeDownloader(context: Context?, val userNo: String, val storeId: String, val afterDeal: (() -> Unit)?) {
    private var ref: WeakReference<Context?>? = null
    private var isShowProgress: Boolean = true//是否显示小圆圈
    private var timeCount = 0//是不是第一次下载，如果是第一次，失败了就要弹窗重试；如果不是，就重试一次再加载默认权限
    private var progressDialog: ProgressBarDialog? = null
    private var handler = Handler()

    init {
        this.ref = WeakReference(context)
        getContext(ref)?.let {
            //这里传的unitCount = 20其实就是20秒超时，因为需要重试一次，一次10秒，所以总共就是20秒超时。
            progressDialog = ProgressBarDialog.Builder(it, 20).apply {
                title = "正在加载权限资料"
            }.create()
        }
    }

    fun setIsShowProgress(isShow: Boolean): PrivilegeDownloader {
        this.isShowProgress = isShow
        return this
    }


    fun download() {
        timeCount++
        //跑进度条
        getContext(ref)?.let {
            if (isShowProgress && timeCount == 1) {
                progressDialog?.show()
                Timer().schedule(timerTask {
                    progressDialog?.let {
                        handler.post { progressDialog?.setProgressPlus() }
                    }
                }, 0, 1000)//每一秒跑一次进度
            }
        }
        val params = hashMapOf<String, String>("userNo" to userNo, "storeId" to storeId)
        OkHttpDominator.postAsynTimeout(NetParam.YINXIAOBU_SERVER + ApiUrls.APPMANAGE.DOWNLOAD_PRIVILEGE, params,
                object : OkHttpDominator.ResultCallback<PrivilegeRes>() {
                    override fun onError(request: Request?, e: Exception?) {
                        if (timeCount == 1) {//只有在第一次失败的时候，才弹窗；后面要是再失败，就直接进去了
                            progressDialog?.setProgressPercent(0.5)
                            download()//这里改成直接download()，默默重新下载
                        } else {
                            doAsync {
                                PrivilegeDbOperate.writeDefaultPrivilege(userNo)//加载默认权限
                                uiThread {
                                    progressDialog?.setProgressPercent(1.0)
                                    afterDeal?.let { it.invoke(); MyToast.show("权限资料下载失败，已加载默认权限") }
                                }
                            }
                        }
                    }

                    override fun onResponse(res: PrivilegeRes?) {
                        if (res == null || res.msgCode != "0") {
                            MyToast.show(BaseAppManager.appContext(), res?.msgInfo, Toast.LENGTH_SHORT)
                            progressDialog?.dismiss()
                            return
                        }
                        doAsync {
                            val writeBasicResult = PrivilegeDbOperate.dealPrivilegeDataResult(res)
                            uiThread {
                                if (!writeBasicResult) {//写入错误
                                    //如果是第一次请求，就重试；如果不是第一次了，那就直接加载默认权限
                                    if (timeCount == 1) {
                                        download()
                                        progressDialog?.setProgressPercent(0.75)
                                    } else {
                                        doAsync {
                                            PrivilegeDbOperate.writeDefaultPrivilege(userNo)//加载默认权限
                                            uiThread {
                                                progressDialog?.setProgressPercent(1.0)
                                                afterDeal?.let { it.invoke(); MyToast.show("权限资料下载失败，已加载默认权限") }
                                            }
                                        }
                                    }
                                } else {//写入成功，那就直接用暴露的接口跳转界面
                                    progressDialog?.setProgressPercent(1.0);afterDeal?.invoke()
                                }
                            }
                        }

                    }
                }, 10)//10秒超时
    }

    /**
     * 下载失败后弹窗
     * （定的逻辑是：如果失败 -》就弹窗 -》取消，就什么都不做 -》确定，就重新下载，下载成功，就跳转界面；下载失败，就调用默认权限）
     */
    //    fun showProgressBar() {
//        timeCount++
//        AlertDialog.Builder(getContext(ref)).setMessage("权限资料下载失败！")
//                .setCancelable(false)
//                .setPositiveButton("取消", null)
//                .setNegativeButton("重试", { dialog, which ->
//                    getContext(ref)?.let {
//                        download()
//                    }
//                })//重新下载
//                .create().show()
//    }

    fun getContext(ref: WeakReference<Context?>?): Context? {
        if (ref != null && ref.get() != null) {
            return ref.get()
        } else {
            return null
        }
    }
}