package com.urovo.wugumofang.net;


import com.urovo.wugumofang.response.CommonEntity;

import java.util.List;
import java.util.Map;

/**
 * @author chenji last on 2018/1/2 20:15
 */

public interface CommonController {
    /**
     * base的View接口
     */
    interface BaseView<T> {
        /**
         * 返回view层的请求成功的回调
         *
         * @param resultInfo ...
         */
        void onResponseSuccess(CommonEntity<T> resultInfo);

        /**
         * 返回view层的请求失败的回调
         *
         * @param failInfo ...
         */
        void onFailure(String failInfo);
    }

    /**
     * 排班接口
     *
     * @param <T>
     */
    interface ShiftBaseView<T> extends BaseView<T> {
        void onConfirmResponseSuccess(String resultInfo);
    }

    /**
     * 是否显示加微及入群人数提报 页面
     *
     * @param <T>
     */
    interface showAddFriOnWeChat<T> {
        void onShowAddFriOnWeChatSuccess(Boolean resultInfo);

        void onShowAddFriOnWeChatFailure(String failInfo);
    }

    /**
     * 首页菜单
     *
     * @param <T>
     */
    interface HomeMenuView<T> {
        void onResponseSuccess4Menu(CommonEntity<T> resultInfo);

        void onFailure4Menu(String failInfo);
    }

    /**
     * 首页广告位
     */
    interface HomeAdView<T> {
        /**
         * 返回view层的请求成功的回调
         *
         * @param resultInfo ...
         */
        void onResponseSuccess4Ad(CommonEntity<T> resultInfo);

        /**
         * 返回view层的请求失败的回调
         *
         * @param failInfo ...
         */
        void onFailure4Ad(String failInfo);
    }

    /**
     * 考勤记录
     */
    interface AttenHistoryView extends BaseView {
        void init();

        void showMsg(String msg);

        void notifyCalData(List<String> dateList);

    }

    interface AttenHisPre extends BasePre {

    }


    /**
     * 购买明细
     */
    interface XiaoFeiView extends BaseView {
        void init();

        void showMsg(String msg);

        void showProgressBar(String msg);

        void refreshFinished();

        void refreshShow();

        void resetNoMoreData(int pageNo);
    }

    interface XiaoFeiPre extends BasePre {

    }

    /**
     * base的Pre接口
     */
    @SuppressWarnings("unused")
    interface BasePre<T> {
        void onStart2requestData(Map<String, String> params);

        void onResponseSuccess(CommonEntity<T> resultInfo);

        void onFailure(String failInfo);
    }

    interface CommonBasePre {
        /**
         * 开始请求
         *
         * @param params 参数集合
         */
        void onStart2RequestTask(Map<String, String> params);
    }

    /**
     * 排班
     *
     * @param <T>
     */
    interface ShiftBasePre<T> extends BasePre<T> {
        void onConfirmResponseSuccess(String resultInfo);
    }

    /**
     * 首页菜单
     *
     * @param <T>
     */
    interface HomeMenuPre<T> {
        void onResponseSuccess4Menu(CommonEntity<T> resultInfo);

        void onStart2requestData4Menu(Map<String, String> params);

        void onFailure4Menu(String failInfo);
    }

    /**
     * 首页广告位
     *
     * @param <T>
     */
    interface HomeAdPre<T> {
        void onStart2requestData4Ad();

        void onResponseSuccess4Ad(CommonEntity<T> resultInfo);

        void onFailure4Ad(String failInfo);
    }

    /**
     * 带有tag的base的View接口
     */
    interface BaseTagView<T> {
        /**
         * 返回view层的请求成功的回调
         *
         * @param tag        ...
         * @param resultInfo ...
         */
        void onResponseSuccess(String tag, CommonEntity<T> resultInfo);

        /**
         * 返回view层的请求失败的回调
         *
         * @param failInfo ...
         */
        void onFailure(String failInfo);
    }

    /**
     * 带有标志的base的Pre接口
     */
    @SuppressWarnings("unused")
    interface BaseTagPre<T> {
        void onStart2requestData(String tag, Map<String, String> params);

        void onResponseSuccess(String tag, CommonEntity<T> resultInfo);

        void onFailure(String failInfo);
    }
}
