package com.urovo.wugumofang.receiver;

import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.os.Build;
import android.provider.Settings;
import android.widget.Toast;

import com.urovo.wugumofang.ui.DialogHelper;

/**
 * @author chenji
 * @date 16/12/26
 */
public class AccessibilityUtil {

    public static boolean checkAccessibility(Context context) {
        // 判断辅助功能是否开启
        if (!AccessibilityUtil.isAccessibilitySettings(context)) {
            // 引导至辅助功能设置页面
            context.startActivity(
                    new Intent(Settings.ACTION_ACCESSIBILITY_SETTINGS)
                            .addFlags(Intent.FLAG_ACTIVITY_NEW_TASK)
            );
            Toast.makeText(context, "请先开启 \"Activity 栈\" 的辅助功能", Toast.LENGTH_LONG).show();
            return false;
        }
        return true;
    }

    /**
     * 判断辅助功能是否开启,没有就跳转到该页面设置（没有开启就不能获取栈顶act等信息）
     *
     * @param context 当前的上下文
     * @param cls     开启服务的类
     */
    private static boolean mIsRemind = true;

    public static void checkTopStackPermission(final Context context, String message, final Class<?> cls) {//TrackerService.class "请先开启生意管家的辅助功能！"
        //设置只有5.0以及以上才去判断开启辅助功能
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
            if (mIsRemind && !AccessibilityUtil.isAccessibilitySettings(context)) {
                // 引导至辅助功能设置页面
//                DialogHelper.onAlertDialog(context, message, null, new DialogInterface.OnClickListener() {
//                    @Override
//                    public void onClick(DialogInterface dialog, int which) {
//                        if (!AccessibilityUtil.isAccessibilitySettings(context)) {
//                            context.startService(new Intent(context, cls));
//                            context.startActivity(new Intent(Settings.ACTION_ACCESSIBILITY_SETTINGS).addFlags(Intent.FLAG_ACTIVITY_NEW_TASK));
//                        }
//                    }
//                });

                DialogHelper.Alert(context, "提示", message, null, new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialog, int which) {
                        context.startService(new Intent(context, cls));
                        context.startActivity(new Intent(Settings.ACTION_ACCESSIBILITY_SETTINGS).addFlags(Intent.FLAG_ACTIVITY_NEW_TASK));
                    }
                }, new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialogInterface, int i) {
                        /*取消按钮*/
                        mIsRemind = false;
                    }
                });
            }
        }
    }

    /**
     * 判断辅助功能是否开启
     *
     * @param context ...
     * @return true: 已经开启；false:还没开启
     */
    public static boolean isAccessibilitySettings(Context context) {
        int accessibilityEnabled = 0;
        try {
            accessibilityEnabled = Settings.Secure.getInt(context.getContentResolver(),
                    Settings.Secure.ACCESSIBILITY_ENABLED);
        } catch (Settings.SettingNotFoundException e) {
            e.printStackTrace();
        }

        if (accessibilityEnabled == 1) {
            String services = Settings.Secure.getString(context.getContentResolver(),
                    Settings.Secure.ENABLED_ACCESSIBILITY_SERVICES);
            if (services != null) {
                return services.toLowerCase().contains(context.getPackageName().toLowerCase());
            }
        }
        return false;
    }
}
