package com.urovo.wugumofang.system;

import com.urovo.wugumofang.constants.ApiConstants;

/**
 * Created by samy on 16/4/11.
 */
public class EnvironmentHelper {
    private static EnvironmentHelper ourInstance = new EnvironmentHelper();

    public static EnvironmentHelper getInstance() {
        return ourInstance;
    }

    public int VERSION_NUM_TYPE = 0;//根据版本号来判断的 版本类型。
    final public static int ENV_DEFAULT = 0;//如果是0，说明还没有手动设置过环境，那就按照版本号来推算域名
    final public static int ENV_RELEASE = 1;//app.wgmf.com  或 wx.wgmf.com
    final public static int ENV_BETA = 2;//beta.wgmf.com
    final public static int ENV_TEST = 3;//test.wgmf.com

    private EnvironmentHelper() {
        //根据版本号位数，来判断是release还是beta，
        VERSION_NUM_TYPE = getVersionName().split("\\.").length == 3?ENV_RELEASE : ENV_BETA;
    }

    /**
     * 根据版本号倍数判断是否是beta环境，来控制显示版本回退入口。
     */
    public boolean isBeta(){
        return VERSION_NUM_TYPE == ENV_BETA;
    }

    /**
     * 版本名称
     */
    public static String getVersionName(){
        String versionName;
        try {
            versionName = BaseAppManager.appContext().getPackageManager().getPackageInfo(BaseAppManager.appContext().getPackageName(), 0).versionName;
        } catch (Exception e) {
            versionName = "0.0.0";//默认3个点，即release版；如果4个点，就是beta版
        }
        return versionName;
    }

    /**
     * 返回App接口的正式或测试环境。
     * @param envStatus:Int
     */
    public String getEnvServer(int envStatus) {
        String tempEnvServer = null;
        switch (envStatus){
            case ENV_DEFAULT://没有手动改过环境的默认情况下。
                tempEnvServer = VERSION_NUM_TYPE == ENV_BETA? ApiConstants.productionEnvServer : ApiConstants.releaseEnvServer;
                break;
            case ENV_RELEASE://手动强制切换到正式环境。
                tempEnvServer = ApiConstants.releaseEnvServer;
                break;
            case ENV_BETA://手动强制切换到beta环境。
                tempEnvServer= ApiConstants.productionEnvServer;
                break;
            case ENV_TEST://手动强制切换到test环境。
                tempEnvServer= ApiConstants.testEnvServer;
                break;
        }
        return tempEnvServer;
    }

    /**
     * 返回App新接口的正式或测试环境。
     * @param envStatus:Int
     */
    public String getEnvApiServer(int envStatus) {
        String tempEnvServer = null;
        switch (envStatus){
            case ENV_DEFAULT://没有手动改过环境的默认情况下。
                tempEnvServer = VERSION_NUM_TYPE == ENV_BETA? ApiConstants.productionEnvApiServer : ApiConstants.releaseEnvApiServer;
                break;
            case ENV_RELEASE://手动强制切换到正式环境。
                tempEnvServer = ApiConstants.releaseEnvApiServer;
                break;
            case ENV_BETA://手动强制切换到beta环境。
                tempEnvServer= ApiConstants.productionEnvApiServer;
                break;
            case ENV_TEST://手动强制切换到test环境。
                tempEnvServer= ApiConstants.testEnvApiServer;
                break;
        }
        return tempEnvServer;
    }

    /**
     * 返回微信那边的正式或测试环境。
     * @param envStatus:Int
     */
    public String getWXServer(int envStatus) {
        String tempEnvServer = null;
        switch (envStatus){
            case ENV_DEFAULT:
            case ENV_RELEASE:
                tempEnvServer= ApiConstants.releaseWXServer;
                break;
            case ENV_BETA:
                tempEnvServer= ApiConstants.releaseWXServer;//ApiConstants.productionWXServer;
                break;
            case ENV_TEST:
                tempEnvServer= ApiConstants.testWXServer;
                break;
        }
        return tempEnvServer;
    }


    public String getEnvPort(int envStatus) {
        String tempEnvPort = null;
        switch (envStatus){
            case ENV_DEFAULT:
            case ENV_RELEASE:
                tempEnvPort= ApiConstants.releaseEnvPort;
                break;
            case ENV_BETA:
                tempEnvPort= ApiConstants.productionEnvPort;
                break;
            case ENV_TEST:
                tempEnvPort= ApiConstants.testEnvPort;
                break;
        }
        return tempEnvPort;
    }
}
