package com.urovo.wugumofang.system;

import android.content.Context;
import android.util.Log;

import com.amap.api.location.AMapLocation;
import com.amap.api.location.AMapLocationClient;
import com.amap.api.location.AMapLocationClientOption;
import com.amap.api.location.AMapLocationClientOption.AMapLocationMode;
import com.amap.api.location.AMapLocationListener;

public class LocationTool {
    public String mLongitude;
    public String mLatitude;
    public String mAddress;

    public interface OnCallback {
        void onFinish(String longitude, String latitude, String address, int code, String error);
    }

    private Context mContext;
    private OnCallback mOnCallback;
    public AMapLocationClient mLocationClient;
    public AMapLocationClientOption mLocationOption = null;
    public AMapLocationListener mLocationListener = new AMapLocationListener() {
        @Override
        public void onLocationChanged(AMapLocation amapLocation) {
            if (amapLocation != null) {
                // 1 GPS定位结果
                // 2 返回上次定位结果
                // 4 缓存定位结果
                // 5 Wifi定位结果
                // 6 基站定位结果
                if (amapLocation.getErrorCode() == 0) {
                    Log.i("Amap", "amapLocation:" + amapLocation.toStr());
                    mLongitude = String.valueOf(amapLocation.getLongitude());
                    mLatitude = String.valueOf(amapLocation.getLatitude());
                    mAddress = amapLocation.getAddress();
                    // // 定位成功回调信息，设置相关消息
                    // amapLocation.getLocationType();//
                    // 获取当前定位结果来源，如网络定位结果，详见定位类型表
                    // amapLocation.getLatitude();// 获取纬度
                    // amapLocation.getLongitude();// 获取经度
                    // amapLocation.getAccuracy();// 获取精度信息
                    // SimpleDateFormat df = new
                    // SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                    // Date date = new Date(amapLocation.getShowTime());
                    // df.format(date);// 定位时间
                    // amapLocation.getAddress();//
                    // 地址，如果option中设置isNeedAddress为false，则没有此结果
                    // amapLocation.getCountry();// 国家信息
                    // amapLocation.getProvince();// 省信息
                    // amapLocation.getCity();// 城市信息
                    // amapLocation.getDistrict();// 城区信息
                    // amapLocation.getRoad();// 街道信息
                    // amapLocation.getCityCode();// 城市编码
                    // amapLocation.getAdCode();// 地区编码
                } else {
                    // 显示错误信息ErrCode是错误码，errInfo是错误信息，详见错误码表。
                    Log.e("Amap", "location Error, ErrCode:" + amapLocation.getErrorCode() + ", errInfo:" + amapLocation.getErrorInfo());
                }
                if (null != mOnCallback) {
                    mOnCallback.onFinish(mLongitude, mLatitude, mAddress, amapLocation.getErrorCode(), amapLocation.getErrorInfo());
                }
                mLocationClient.stopLocation();// 停止定位
                mLocationClient.onDestroy();// 销毁定位客户端
            } else {
                Log.e("Amap", "onLocationChanged null");
            }
        }

    };

    public LocationTool(Context cxt, OnCallback callback) {
        try {
            mContext = cxt;
            mOnCallback = callback;
            mLongitude = "";
            mLatitude = "";
            mAddress = "";
            // 高精度定位模式：会同时使用网络定位和GPS定位，优先返回最高精度的定位结果；
            // 低功耗定位模式：不会使用GPS，只会使用网络定位（Wi-Fi和基站定位）；
            // 仅用设备定位模式：不需要连接网络，只使用GPS进行定位，这种模式下不支持室内环境的定位。
            AMapLocationClient.updatePrivacyAgree(mContext,true);
            AMapLocationClient.updatePrivacyShow(mContext,true,true);
            // 初始化定位
            mLocationClient = new AMapLocationClient(mContext.getApplicationContext());
            // 初始化定位参数
            mLocationOption = new AMapLocationClientOption();
            // 设置定位模式为高精度模式，Battery_Saving为低功耗模式，Device_Sensors是仅设备模式
            mLocationOption.setLocationMode(AMapLocationMode.Hight_Accuracy);
            // 设置是否返回地址信息（默认返回地址信息）
            mLocationOption.setNeedAddress(true);
            // 设置是否只定位一次,默认为false
            mLocationOption.setOnceLocation(false);
            // 设置是否强制刷新WIFI，默认为强制刷新
            mLocationOption.setWifiActiveScan(true);
            // 设置是否允许模拟位置,默认为false，不允许模拟位置
            mLocationOption.setMockEnable(false);
            // 设置定位间隔,单位毫秒,默认为2000ms
            mLocationOption.setInterval(2000);
            //设置定位超时时间为10秒
            mLocationOption.setHttpTimeOut(10000);
            mLocationClient.setLocationListener(mLocationListener);
            // 给定位客户端对象设置定位参数
            mLocationClient.setLocationOption(mLocationOption);
            // 启动定位
            mLocationClient.startLocation();
            Log.i("Amap", "mLocationClient.startLocation()");
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
