package com.urovo.wugumofang.system;

import android.app.ActivityManager;
import android.content.Context;
import android.os.Build;

import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.List;

/**
 * @author Sasuke on 2018/07/16.
 */
public class TopAppNameGetter {
    /**
     * 这个方法获取最近运行任伤中最上面的一个应用的包名,<br>
     * 进行了api版本的判断,然后利用不同的方法获取包名,具有兼容性
     *
     * @param context 上下文对象
     * @return 返回包名, 如果出现异常或者获取失败返回""
     */
    public static String[] getRunningPackages(Context context) {
        if (Build.VERSION.SDK_INT < 21) { // 如果版本低于22
            // 获取到activity的管理的类
            ActivityManager m = (ActivityManager) context.getSystemService(Context.ACTIVITY_SERVICE);
            ArrayList<String> packageNames = new ArrayList<>();
            for(ActivityManager.RunningTaskInfo info : m.getRunningTasks(100)){
                packageNames.add(info.topActivity.getPackageName());
            }
            return (String[])packageNames.toArray();
        } else {

            final int PROCESS_STATE_TOP = 2;
            try {
                // 获取正在运行的进程应用的信息实体中的一个字段,通过反射获取出来
                Field processStateField = ActivityManager.RunningAppProcessInfo.class.getDeclaredField("processState");
                // 获取所有的正在运行的进程应用信息实体对象
                List<ActivityManager.RunningAppProcessInfo> processes = ((ActivityManager) context
                        .getSystemService(Context.ACTIVITY_SERVICE)).getRunningAppProcesses();
                // 循环所有的进程,检测某一个进程的状态是最上面,也是就最近运行的一个应用的状态的时候,就返回这个应用的包名
                for (ActivityManager.RunningAppProcessInfo process : processes) {
                    if (process.importance <= ActivityManager.RunningAppProcessInfo.IMPORTANCE_FOREGROUND
                            && process.importanceReasonCode == 0) {
                        int state = processStateField.getInt(process);
                        if (state == PROCESS_STATE_TOP) { // 如果这个实体对象的状态为最近的运行应用
                            String[] packnames = process.pkgList;
                            // 返回应用的包名
                            return packnames;
                        }
                    }
                }
            } catch (Exception e) {
            }
            return  new String[0];
        }

    }
}
