package com.urovo.wugumofang.ui

import android.content.Context
import android.graphics.Color
import android.view.MotionEvent
import android.view.View
import android.widget.TextView
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.google.android.material.tabs.TabLayout
import com.urovo.wugumofang.libutils.R


/**
 * @author Sasuke on 2019-10-15 0015.
 * TabLayout与Rv同步滚支配者
 *
 * (仍有BUG，快速点击Tab时，Rv可能一定几率不滑动)
 */
class TabRvSyncDominator(val context: Context, val tabLayout: TabLayout, val rv: RecyclerView) {
    //tab状态变化锁，为true时，相当于锁住tab，避免列表滑动过程中引起中途tab状态变更。
    //这个锁的上锁时期为：按下tab时---到----》列表停止滑动，这段时间之内。
    //点击tab即上锁，列表停滑即解锁。
    @Volatile
    var tabStateLock = false

    //把外面的数据列表的type抽出来成为一个单独的List
    var dataTypeList: List<String> = listOf()

    //Tab们
    var titleList: List<TabInfo> = listOf()

    @Volatile
    var oldPos = -1

    init {
        rv.addOnScrollListener(object : RecyclerView.OnScrollListener() {
            var lastPos = -1
            override fun onScrolled(rv: RecyclerView, dx: Int, dy: Int) {
                //滑动Rv联动Tab
                val currHeadPos = (rv.layoutManager as LinearLayoutManager).findFirstCompletelyVisibleItemPosition()
                if (currHeadPos != lastPos && !tabStateLock) {//正在滑动 && 没有上锁，才会执行更改tab状态操作
                    //根据列表当前第一个元素的typeId来找对应的Label序号，再select
//                    pre?.getTabPosByRvHead(currHeadPos)
//                    titleList.indexOfFirst { it.tabType == dataTypeList.get(currHeadPos).toString() }
//                            .takeIf { it != -1 }?.let { tabLayout.getTabAt(it)?.select() }
                }
                lastPos = currHeadPos
            }

            override fun onScrollStateChanged(recyclerView: RecyclerView, newState: Int) {
                super.onScrollStateChanged(recyclerView, newState)
                if (newState == 0) {
                    tabStateLock = false//如果停止了滑动，那就把锁解开
//                    lastPos = (rv.layoutManager as LinearLayoutManager).findFirstCompletelyVisibleItemPosition()//并记下最新位置
                }
            }
        })

        //标签栏挂监听
        tabLayout.addOnTabSelectedListener(object : TabLayout.OnTabSelectedListener {
            override fun onTabReselected(tab: TabLayout.Tab?) {

            }

            override fun onTabUnselected(tab: TabLayout.Tab?) {
                tab?.customView?.findViewById<TextView>(R.id.tabTv)?.setTextColor(Color.parseColor("#4f4f4f"))
            }

            override fun onTabSelected(tab: TabLayout.Tab?) {
                tab?.customView?.findViewById<TextView>(R.id.tabTv)?.setTextColor(Color.parseColor("#ff9800"))
            }
        })
    }

    /**
     * 从网络获取了数据后，刷新Tab时传入
     * 传入重新组建的TabInfo的List，和把数据列表的typeId都抽出来组成的typeList
     * 注意前者是标签的List，后者是【全部】数据id的List
     * */
    fun setTabInfoNotify(titleList: List<TabInfo>, dataTypeList: List<String>) {
        rv.layoutManager = LinearLayoutManagerWithSmoothScroller(context)
        this.titleList = titleList
        this.dataTypeList = dataTypeList
        tabLayout.removeAllTabs()
        //如果标签数量小于等于4个，那就填充，否则滚动
        tabLayout.tabMode = /*if (titleList.size <= 4) TabLayout.MODE_FIXED else*/ TabLayout.MODE_SCROLLABLE
        //逐个添加tab们
        titleList.forEach { tabInfo ->
                tabLayout.addTab(tabLayout.newTab().apply {
                setCustomView(R.layout.tab_rv_sync_item_layout)
                customView?.findViewById<TextView>(R.id.tabTv)?.run { text = tabInfo.tabName;tag = tabInfo.tabType }
                //点击穿透，点击tab联动Rv
                (customView?.parent as View).setOnTouchListener { view, motionEvent ->
                    //                    if (tabStateLock) return@setOnTouchListener true//没有在滚动时，才能点击
                    if (motionEvent.action == MotionEvent.ACTION_DOWN) {
                        val posForScroll = dataTypeList.indexOfFirst { view.findViewById<TextView>(R.id.tabTv)?.tag == it }
                        //如果点击的tab不一样了，那就把锁锁上
//                        if (oldPos != posForScroll) {
                        tabStateLock = true
//                            oldPos = posForScroll
                        rv.smoothScrollToPosition(posForScroll)
//                        }
//                        (dataRv.layoutManager as LinearLayoutManager).scrollToPositionWithOffset(posForScroll,0)
                    }
                    false
                }
            })
        }
    }

    data class TabInfo(var tabName: String = "", var tabType: String = "")
}