package com.urovo.wugumofang.utils

import android.content.Context
import android.graphics.drawable.BitmapDrawable
import android.view.Gravity
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.BaseAdapter
import android.widget.ListView
import android.widget.TextView
import com.google.android.material.tabs.TabLayout
import com.google.gson.Gson
import com.google.gson.JsonParser
import com.urovo.wugumofang.libutils.R
import com.urovo.wugumofang.ui.DensityUtil
import com.urovo.wugumofang.view.CustomPopupWindow
import org.jetbrains.anko.find

/**
 * @author Sasuke on 2019-7-23 0023.
 * 省市区选择器
 *
 * 配合 override fun onWindowFocusChanged(hasFocus: Boolean) 来在Activity里刷新
 *
 */
class AddressPickerDominator(val context: Context, val rootView: View, val notifyCallback: (AddressPickerDominator) -> Unit) {

    var province: SubRegions? = null//省份
    var city: SubRegions? = null//城市
    var district: SubRegions? = null//区县

    var provinceCache: SubRegions? = null//省份
    var cityCache: SubRegions? = null//城市
    var districtCache: SubRegions? = null//区县
    var currTabPos = 0

//    //上个地区级别的列表缓存（因为考虑到可能会选了第三个地区之后，又）
//    var currListCache: List<SubRegions> = listOf()

    var listHistory: MutableList<List<SubRegions>> = mutableListOf(listOf(), listOf(), listOf())

    val addressList by lazy {
        JsonParser().parse(AssetsUtil.getFromAssets(context, "normal_region.json"))
                .asJsonArray.map {
            Gson().fromJson(it, SubRegions::class.java)
        }
    }

    val pickPop by lazy {
        CustomPopupWindow.Builder(context)
                .setContentView(R.layout.address_picker_layout)
                .setFouse(true)
                .setOutSideCancel(true)
                .setAnimationStyle(R.style.add_sell_popup_anim_style)
                .setBackgroundDrawable(BitmapDrawable())
                .setwidth(ViewGroup.LayoutParams.MATCH_PARENT)
                .setheight(DensityUtil.dip2px(context, 360f))
                .builder()
    }

    /**
     * 设置预设地址（这种情况是如果一进来Activity，就自带了地址，那就需要先设好，然后一弹出选择器，TabLayout就会预先显示这些地址，而不会显示成 “省市区县” 这些）
     */
    fun setInitedAddress(provinceName: String, provinceId: Int, cityName: String, cityId: Int, districtName: String, districtId: Int): AddressPickerDominator {
        this.province = SubRegions(provinceId, provinceName, listOf())
        this.city = SubRegions(cityId, cityName, listOf())
        this.district = SubRegions(districtId, districtName, listOf())

        return this
    }

    init {
        //当前正在显示的listView
        val listView = (pickPop.getItemView(R.id.lv) as ListView)
        //预设顶栏TabLayout
        val tabLayout = (pickPop.getItemView(R.id.tabLayout) as TabLayout).apply {
            addTab(newTab())
            addTab(newTab())
            addTab(newTab())
        }
        //View遮盖使TabLayout不可点击
//        pickPop.getItemView(R.id.clickKiller).setOnTouchListener { view, motionEvent -> true }
        tabLayout.addOnTabSelectedListener(object : TabLayout.OnTabSelectedListener {
            var unselectedTab: TabLayout.Tab? = null
            override fun onTabReselected(tab: TabLayout.Tab?) {
            }

            override fun onTabUnselected(tab: TabLayout.Tab?) {//点击时先执行我
                unselectedTab = tab
            }

            override fun onTabSelected(tab: TabLayout.Tab?) {// TabUnselected（）执行后，我才执行
                if (listHistory[tab?.position ?: 0].isEmpty()) {
                    unselectedTab?.select()
                } else {
                    currTabPos = tab?.position ?: 0
                    //刷新ListView
                    listView.adapter = AddressPickerAdapter(context, listHistory[currTabPos])
                }
            }
        })
    }


    fun show() {
        pickPop.showAsLocation(rootView, Gravity.CENTER, 0, 0)

        val tabLayout = (pickPop.getItemView(R.id.tabLayout) as TabLayout)
//TabLayout设为第一个标签激活
        tabLayout.getTabAt(0)?.select()
        tabLayout.getTabAt(0)?.text = if (province?.regionName.isNullOrBlank()) "省" else province?.regionName
        tabLayout.getTabAt(1)?.text = if (city?.regionName.isNullOrBlank()) "市" else city?.regionName
        tabLayout.getTabAt(2)?.text = if (district?.regionName.isNullOrBlank()) "区县" else district?.regionName

        //预设城市列表 ListView
        (pickPop.getItemView(R.id.lv) as ListView).apply {
            adapter = AddressPickerAdapter(context, addressList)
            //ListView 点击监听
            listHistory = mutableListOf(listOf(), listOf(), listOf())
            listHistory.set(0, addressList)
            setOnItemClickListener { parent, view, pos, id ->
                val tablayout = (pickPop.getItemView(R.id.tabLayout) as TabLayout)
                val clickedInfo = listHistory[currTabPos][pos]
                //先把当前Tab的文字改掉
                tablayout.getTabAt(currTabPos)?.setText(clickedInfo.regionName)
                //先缓存起来，但不作为最终结果
                when (currTabPos) {
                    0 -> provinceCache = SubRegions(clickedInfo.id, clickedInfo.regionName, listOf())
                    1 -> cityCache = SubRegions(clickedInfo.id, clickedInfo.regionName, listOf())
                    else -> districtCache = SubRegions(clickedInfo.id, clickedInfo.regionName, listOf())
                }
                //再把Tab移动到下一个（如果没有，就执行notify回调刷新外面的UI）
                if (tablayout.getTabAt(currTabPos + 1) == null) {
                    //这里才是真正存起来的结果，留给最后提交用。
                    province = provinceCache
                    city = cityCache
                    district = districtCache
                    notifyCallback.invoke(this@AddressPickerDominator)
                    dismiss()
                } else {//如果有，就激活下一个Tab，并更新Adapter
                    val nextList = listHistory[currTabPos][pos].subRegions
//                    adapter = AddressPickerAdapter(context, nextList)
                    listHistory.set(currTabPos + 1, nextList)
                    //如果选到最后一个了，又回头选第一个，然后再回头选最后一个，
                    // 相当于中间的都没选，后面的也没刷新，数据关联就乱了，
                    //所以如果点击了前面，就需要把后面的列表全置为空listOf()，
                    // 使后面的不可跳级点击，只能一级一级往下点，保证地址数据关联正确性
                    for (index in currTabPos + 2 until listHistory.size){
                        listHistory.set(index, listOf())
                    }
                    tablayout.getTabAt(currTabPos + 1)?.select()
                }
            }
        }
    }

    fun dismiss() {
        pickPop.dismiss()
    }

    data class SubRegions(var id: Int,
                          var regionName: String,
                          var subRegions: List<SubRegions>)

    private class AddressPickerAdapter(val context: Context, val dataList: List<SubRegions>) : BaseAdapter() {
        override fun getView(pos: Int, convertView: View?, parent: ViewGroup?): View {
            var view = convertView
            if (view == null) {
                view = LayoutInflater.from(context).inflate(R.layout.addr_pick_list_item, parent, false)
            }
            view?.find<TextView>(R.id.addrTv)?.setText(dataList[pos].regionName)

            return view!!
        }

        override fun getItem(pos: Int): Any? {
            return pos
        }

        override fun getItemId(pos: Int): Long {
            return pos.toLong()
        }

        override fun getCount(): Int {
            return dataList.size
        }
    }

}