package com.urovo.wugumofang.utils

import android.app.Activity
import android.content.Context
import android.content.Intent
import android.graphics.Color
import android.graphics.drawable.GradientDrawable
import android.net.Uri
import android.os.Build
import android.os.Handler
import android.provider.Settings
import android.view.View
import android.widget.Button
import android.widget.TextView
import androidx.annotation.DrawableRes
import androidx.fragment.app.Fragment
import com.google.gson.Gson
import com.google.gson.reflect.TypeToken
import com.urovo.wgmf.db.dao.entitys.BarcodeScanDb
import com.urovo.wugumofang.system.DeviceHelper.getPackageName
import com.urovo.wugumofang.ui.ViewUtil
import java.util.*
import java.util.regex.Pattern

/**
 *
 * @author chenji
 * @date 2017/11/3
 */
object CommonTools {
    var proCodeList: List<String> = ArrayList()
    //    var scanCodeList: List<BarcodeScanDb> = ArrayList()
    //  指定一秒内不能多次点击
    val FAST_DOUBLE_CLICK_INTERVAL: Long = 1000
    private var lastTime: Long = 0
    private val mHandler = Handler()
    private var mRunCount = 0

    /**
     * 防止按钮狂点击
     */
    fun isOneClick(): Boolean {
        try {
            return System.currentTimeMillis() - lastTime > FAST_DOUBLE_CLICK_INTERVAL
        } finally {
            lastTime = System.currentTimeMillis()
        }
    }

    //============================分水岭=============================

    //防止按钮狂点击,设置isEnabled值为false,按钮状态变灰,设置true为橙色
    private fun reFreshBtnStatues(act: Activity, refreshBtn: Button, isEnabled: Boolean) {
        //===================设置圆角背景=========================
        val drawable = GradientDrawable()
        drawable.setCornerRadius(4f)
        //        drawable.setStroke(1, Color.parseColor("#cccccc"));
        drawable.setColor(if (isEnabled) Color.parseColor("#d99d15") else Color.GRAY)
        //如果页面已销毁，再更改refresh_btn状态会崩。
        if (act.isFinishing) {
            return
        }
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.JELLY_BEAN) {
            refreshBtn.background = drawable
        } else {
            refreshBtn.setBackgroundDrawable(drawable)
        }

        //===================设置圆角背景=========================
        // 按钮是否可点击
        refreshBtn.isEnabled = isEnabled

        //        refresh_btn.setBackgroundColor(isEnabled ? Color.parseColor("#d99d15") : Color.GRAY);
    }

    private val mRunnable = object : Runnable {
        override fun run() {
            // 第一次执行则关闭定时执行操作
            if (mRunCount == 1) mHandler.removeCallbacks(this)
            ViewUtil.dismissProgress()
            mRunCount++
        }
    }

    /**
     * 每次调用只执行一次，并在设定时间内做一次延时操作
     */
    fun postDelayed(delayTime: Long) {
        mHandler.postDelayed(mRunnable, delayTime)
    }

    fun postDelayed(delayTime: Long, runnable: Runnable) {
        mHandler.postDelayed(runnable, delayTime)
    }

    //==================================分水岭=================================
    enum class ShowArrowDirection {
        //箭头方向
        left,
        right,
        top,
        bottom
    }

    /**
     * TextView 设置图片

     * @param textView         需要设置图片的textview
     * *
     * @param firstDrawableId  第一幅图片
     * *
     * @param secondDrawableId 第二幅图片
     * *
     * @param arrowFlag        true:替换为第一幅图片,false:替换为第二幅图片
     */
    fun setShowArrow(
            context: Context,
            textView: TextView,
            showArrowDirection: ShowArrowDirection,
            arrowFlag: Boolean,
            @DrawableRes firstDrawableId: Int,
            @DrawableRes secondDrawableId: Int
    ) {
        val arrowDrawable = context.resources.getDrawable(if (arrowFlag) firstDrawableId else secondDrawableId)
        arrowDrawable.setBounds(0, 0, arrowDrawable.minimumWidth, arrowDrawable.minimumHeight)
        when (showArrowDirection) {
            ShowArrowDirection.right ->
                //图片显示在右边
                textView.setCompoundDrawables(null, null, arrowDrawable, null)
            ShowArrowDirection.left ->
                //图片显示在左边
                textView.setCompoundDrawables(arrowDrawable, null, null, null)
            ShowArrowDirection.top ->
                //图片显示在上边
                textView.setCompoundDrawables(null, arrowDrawable, null, null)
            ShowArrowDirection.bottom ->
                //图片显示在下边
                textView.setCompoundDrawables(null, null, null, arrowDrawable)
            else -> {
            }
        }
    }

    /**
     * 设置一张图片
     */
    fun setShowArrow(
            context: Context,
            textView: TextView,
            showArrowDirection: ShowArrowDirection,
            @DrawableRes drawableId: Int
    ) {
        setShowArrow(context, textView, showArrowDirection, true, drawableId, -1)
    }

    /**
     * 图片文字设置的方法

     * @param textView    设置显示的textview
     * *
     * @param drawableId  设置显示的图片
     * *
     * *
     */
    fun setImgAndText(
            context: Context,
            textView: TextView,
            showArrowDirection: ShowArrowDirection,
            @DrawableRes drawableId: Int,
            tvContent: String

    ) {
        val arrowDrawable = context.resources.getDrawable(drawableId)
        arrowDrawable.setBounds(0, 0, arrowDrawable.minimumWidth, arrowDrawable.minimumHeight)
        when (showArrowDirection) {
            ShowArrowDirection.right -> {
                //图片显示在右边
                textView.setCompoundDrawables(null, null, arrowDrawable, null)
                textView.text = tvContent
            }
            ShowArrowDirection.left -> {
                //图片显示在左边
                textView.setCompoundDrawables(arrowDrawable, null, null, null)
                textView.text = tvContent
            }

            ShowArrowDirection.top -> {  //图片显示在上边
                textView.setCompoundDrawables(null, arrowDrawable, null, null)
                textView.text = tvContent
            }
            ShowArrowDirection.bottom -> {//图片显示在下边
                textView.setCompoundDrawables(null, null, null, arrowDrawable)
                textView.text = tvContent
            }
            else -> {
            }
        }
    }
//==================================分水岭=================================
    /**
     * 设置显示隐藏（view 非空）
     * @param v          ...
     * @param isVisible ...
     */
    fun setVisibility(v: View, isVisible: Boolean) {
        if (isVisible) v.visibility = View.VISIBLE else View.GONE
    }
//
//    fun replace(act: Activity, containerViewId: Int, fragment: Fragment) {
//        if (act.fragmentManager != null) {
//            act.fragmentManager.beginTransaction().replace(containerViewId, fragment).commit()
//        }
//    }

    fun replace(fra: Fragment, containerViewId: Int, fragment: Fragment) {
        if (fra.fragmentManager != null) {
            fra.fragmentManager!!.beginTransaction().replace(containerViewId, fragment).commit()
        }
    }

    //==================================分水岭=================================
    internal var pattern = Pattern.compile("^[-\\+]?[.\\d]*$")

    /**
     * 判断是否为浮点数，包括double和float

     * @param str 传入的字符串
     * *
     * @return 是浮点数返回true, 否则返回false
     */
    fun isDouble(str: String): Boolean {
        return pattern.matcher(str).matches()
    }

    /**
     * 扫码集合的转换
     */
    fun fromJson4Scan(json: String): List<BarcodeScanDb> {
        return Gson().fromJson(json, object : TypeToken<List<BarcodeScanDb>>() {}.type)
    }

    /**
     * 启动应用的设置
     */
    private const val PACKAGE_URL_SCHEME = "package:"

    fun startAppSettings(context: Context) {
        val intent = Intent(Settings.ACTION_APPLICATION_DETAILS_SETTINGS)
        intent.data = Uri.parse(PACKAGE_URL_SCHEME + getPackageName(context))
        context.startActivity(intent)
    }
}