package com.urovo.wugumofang.utils

import android.app.Activity
import android.app.AlertDialog
import android.content.Context
import android.content.Intent
import android.content.res.Resources
import android.graphics.drawable.BitmapDrawable
import android.net.Uri
import android.os.Handler
import android.os.Looper
import android.util.TypedValue
import android.view.Gravity
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.TextView
import android.widget.Toast
import androidx.core.text.HtmlCompat
import com.urovo.wugumofang.constants.Files
import com.urovo.wugumofang.libutils.R
import com.urovo.wugumofang.net.OkHttpDominator
import com.urovo.wugumofang.ui.PictureUtil
import com.urovo.wugumofang.view.CustomPopupWindow
import okhttp3.Request
import org.jetbrains.anko.doAsync
import org.jetbrains.anko.find
import java.io.File
import java.util.concurrent.Semaphore

/**
 * @author Sasuke on 2020-4-24 0024.
 * 图片批量下载支配者
 */
class PhotoPackDLDominator(val context: Context?, val urls: List<String>,val isPhotos: Boolean/*, val successCallback: (() -> Unit)? = null, val failedCallback: (() -> Unit)? = null*/) {
    val smphPool = Array(urls.size, { Semaphore(0) })

    val dialog by lazy {
        AlertDialog.Builder(context)
                .setTitle("正在下载中...")
                .setMessage("0/${urls.size}")
                .setCancelable(true)
                .create()
    }

    val rootView by lazy { ((context as Activity).findViewById(android.R.id.content) as ViewGroup).getChildAt(0) }

    val goalPath = Files.tempPicture//Environment.getExternalStoragePublicDirectory(Environment.DIRECTORY_PICTURES).absolutePath

    var currIndex = 0

    fun download() {
        if (checkIfHasAllDownload()){
            showDIYToast(context, "您已保存过${if (isPhotos)"这些图片" else "该视频"}",
                    HtmlCompat.fromHtml("请前往<font color='#ffcc01'>微信朋友圈</font>选择${if (isPhotos)"图片" else "视频"}", HtmlCompat.FROM_HTML_MODE_LEGACY))
            return
        }

        runUiThread { dialog.show() }
        //reversed使倒序下载，这样朋友圈里调用时就能以原来的正序显示
//        urls.reversed().forEachIndexed { index, url ->
//        doAsync {
        //                Log.i("SasukeLog", "doAsync[${index}] is wait")
//                smphPool[index].acquire()
//                Log.i("SasukeLog", "doAsync[${index}] is go")
//                val bmp = BitmapFactory.decodeStream(URL(url).openStream())

        downloadOne(urls.reversed()[currIndex])

//                PictureUtil.saveBitmapFile(bmp, goalPath, fileName)

//                PictureUtil.saveBmpToGallery(context,bmp,fileName)
            // 最后通知图库媒体库更新
//                context.sendBroadcast(Intent(Intent.ACTION_MEDIA_SCANNER_SCAN_FILE, Uri.parse(goalPath + File.separator +fileName)))
//                PictureUtil.saveImageToGallery(BaseAppManager.appContext(), goalPath, fileName)
            //要再删一遍，因为上面 saveImageToGallery 会产生多的图片到系统 Pictures 文件夹
//                PictureUtil.deleteFileIfExists(goalPath, fileName)
//        }//doAsync
//        }
//        smphPool[0].release()//开始释放第一个信号量
    }


    fun downloadOne(url: String) {
        doAsync {
            var fileName = Files.transUrl2Filename(url)
            PictureUtil.deleteFileIfExists(context, goalPath, fileName)//删除同名文件
            OkHttpDominator.downloadAsyn(url, Files.tempPicture + fileName, object : OkHttpDominator.ResultCallback<String>() {
                override fun onError(request: Request?, e: Exception?) {
                    dialog.dismiss()
                    serverErrorPop.showAtLocation(rootView, Gravity.CENTER, 0, 0)
                }

                override fun onResponse(response: String?) {
                    //发广播更新到媒体库
                    context?.sendBroadcast(Intent(Intent.ACTION_MEDIA_SCANNER_SCAN_FILE, Uri.fromFile(File(goalPath, fileName))))
                    runUiThread { dialog.setMessage("${currIndex + 1}/${urls.size}") }
                    if (currIndex + 1 < urls.size) {
                        downloadOne(urls.reversed()[++currIndex])
                        return
//                        Log.i("SasukeLog", "new doAsync[${index + 1}] is release")
//                            smphPool[index + 1].release()
                    }
                    runUiThread {
                        showDIYToast(context, "${if (isPhotos)"所有图片均" else "视频"}已保存成功",
                                HtmlCompat.fromHtml("请前往<font color='#ffcc01'>微信朋友圈</font>选择${if (isPhotos)"图片" else "视频"}", HtmlCompat.FROM_HTML_MODE_LEGACY))
                        dialog.dismiss()
                    }
                }
            })
        }
    }

    /**下载前先检查一下是不是已经全部下载过了*/
    fun checkIfHasAllDownload() = urls.all {
        File(goalPath,Files.transUrl2Filename(it)).exists()
    }


//    Handler(Looper.getMainLooper()).post { successCallback.invoke() }
//    PictureUtil.deleteFileIfExists(Files.tempPicture, fileName)//删除同名文件
//    PictureUtil.saveBitmapFile(bmp, Files.tempPicture, fileName)


//    data class BmpItem(val fileName: String, val bmp: Bitmap)

    /**弹出自定义Toast*/
    fun showDIYToast(context: Context?, msg: String, msg2: CharSequence = "") {
        context?.let {
            Toast(context).apply {
                //设置Toast要显示的位置，水平居中并在底部，X轴偏移0个单位，Y轴偏移70个单位，
                setGravity(Gravity.CENTER, 0, TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, -100f, Resources.getSystem().displayMetrics).toInt())
                //设置显示时间
                duration = Toast.LENGTH_SHORT
                val diyToastView = LayoutInflater.from(context).inflate(R.layout.diy_toast_layout, null)
                view = diyToastView
                diyToastView.find<TextView>(com.urovo.wugumofang.libutils.R.id.toastText1).text = msg
                diyToastView.find<TextView>(com.urovo.wugumofang.libutils.R.id.toastText2).text = msg2
                diyToastView.find<TextView>(com.urovo.wugumofang.libutils.R.id.toastText2).visibility = if (msg2.isNotBlank()) View.VISIBLE else View.GONE
            }.show()
        }
    }


    fun runUiThread(runnable: ()->Unit){
        Handler(Looper.getMainLooper()).post(runnable)
    }

    //服务器错误弹窗
    val serverErrorPop: CustomPopupWindow by lazy {
        CustomPopupWindow.Builder(context)
                .setContentView(R.layout.server_error_pop)
                .setFouse(true)
                .setOutSideCancel(false)
//                .setAnimationStyle(R.style.dropmenu_anim_style)
                .setBackgroundDrawable(BitmapDrawable())
                .setwidth(ViewGroup.LayoutParams.MATCH_PARENT)
                .setheight(ViewGroup.LayoutParams.MATCH_PARENT)
                .builder().apply {
                    getItemView(R.id.dismissBtn).setOnClickListener { dismiss() }
                }
    }
}