package com.urovo.wugumofang.view;

import android.graphics.Color;
import android.graphics.drawable.ColorDrawable;
import android.os.Bundle;
import android.view.Gravity;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.Window;
import android.view.WindowManager;
import android.view.animation.AccelerateDecelerateInterpolator;
import android.view.animation.Animation;
import android.view.animation.TranslateAnimation;

import com.urovo.wugumofang.libutils.R;

import androidx.annotation.Nullable;
import androidx.fragment.app.DialogFragment;

/**
 * Created by Sasuke on 2017/2/7.
 * 通用底部弹窗
 */

public class BottomDialog extends DialogFragment {
    public static BottomDialog newInstance(int rId) {

        Bundle args = new Bundle();
        args.putInt("rId",rId);//填充弹窗的布局
        BottomDialog fragment = new BottomDialog();
        fragment.setArguments(args);
        return fragment;
    }

    private OnSelectListener mListener;
    @Override
    public void onStart() {
        super.onStart();
        Window window = getDialog().getWindow();
        WindowManager.LayoutParams params = window.getAttributes();
        params.gravity = Gravity.BOTTOM;
        params.width = WindowManager.LayoutParams.MATCH_PARENT;
        //加这一句可以使弹出时，背景不变暗
//        params.dimAmount = 0.0f;
        window.setAttributes(params);
        window.setBackgroundDrawable(new ColorDrawable(Color.TRANSPARENT));
    }

    @Nullable
    @Override
    public View onCreateView(LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        getDialog().requestWindowFeature(Window.FEATURE_NO_TITLE);
        int rId = getArguments().getInt("rId");
        ViewGroup layout = (ViewGroup)inflater.inflate(rId,container,false);
        //因为以前最开始是用来添加图片的，但如果用于其它用途，那就不挂这种监听器了。
        if (rId == R.layout.ll_bottom_dialog){
            InsideClickListener listener = new InsideClickListener();
            for(int i = 0;i < layout.getChildCount();i++){
                layout.getChildAt(i).setOnClickListener(listener);
            }
        }
        slideToUp(layout);
        return layout;
    }

    /**
     * 在外部设置监听器
     * @param listener 传入的点击监听器
     */
    public void setItemClickListener(OnSelectListener listener){
        mListener = listener;
    }

    /**
     * 组件内部的点击监听
     */
    private class InsideClickListener implements View.OnClickListener{
        @Override
        public void onClick(View v) {
            if(mListener != null){
                if(v.getId() == R.id.camera_pick){//如果是点击拍照
                    mListener.onCameraClick(v);
                }else if(v.getId() == R.id.local_pick){//如果是点击从本地相册获取
                    mListener.onLocalClick(v);
                }
            }
        }
    }

    /**对外监听器接口*/
    public interface OnSelectListener{
        void onCameraClick(View v);
        void onLocalClick(View v);
    }

    /**弹出动画*/
    public static void slideToUp(View view){
        Animation slide = new TranslateAnimation(Animation.RELATIVE_TO_SELF, 0.0f,
                Animation.RELATIVE_TO_SELF, 0.0f, Animation.RELATIVE_TO_SELF,
                1.0f, Animation.RELATIVE_TO_SELF, 0.0f);

        slide.setDuration(200);
        slide.setFillAfter(true);
        slide.setFillEnabled(true);
        slide.setInterpolator(new AccelerateDecelerateInterpolator());
        view.startAnimation(slide);

        slide.setAnimationListener(new Animation.AnimationListener() {
            @Override
            public void onAnimationStart(Animation animation) {

            }

            @Override
            public void onAnimationEnd(Animation animation) {

            }

            @Override
            public void onAnimationRepeat(Animation animation) {

            }
        });

    }
}
