@file:JvmName("LimitedKeyboard")

package com.urovo.wugumofang.view

import android.graphics.Color
import android.graphics.drawable.ColorDrawable
import android.os.Bundle
import android.view.*
import android.view.animation.AccelerateDecelerateInterpolator
import android.view.animation.Animation
import android.view.animation.TranslateAnimation
import android.widget.GridView
import android.widget.ImageView
import android.widget.SimpleAdapter
import android.widget.TextView
import androidx.fragment.app.DialogFragment
import androidx.fragment.app.FragmentManager
import androidx.viewpager.widget.PagerAdapter
import androidx.viewpager.widget.ViewPager
import com.urovo.wugumofang.libutils.R
import com.urovo.wugumofang.ui.DensityUtil
import org.jetbrains.anko.dip
import org.jetbrains.anko.find

/**
 * @author Sasuke on 2017/10/27.
 * 自定义键盘！！
 * 受限制的自定义底部键盘！！
 * 专门用于输入内容被限制的受限制的自定义底部键盘！！！
 *
 * 注：首次是在上报反馈的生产日期后两位附加码的地方用到，可以去那里参观用法。
 */

class LimitedKeyboard : DialogFragment() {
    /**
     * 限制的输入字符范围
     */
    lateinit var mInputRule: List<List<String>>
    var tv: TextView? = null
    var kbViewPager: ViewPager? = null

    fun setTextView(tv: TextView) {
        this.tv = tv
    }

    companion object {
        val GRID_COLUME_NUM: Int = 7

        @JvmStatic fun newInstance(inputRule: ArrayList<List<String>>): LimitedKeyboard {
            val args = Bundle().apply { putSerializable("rule", inputRule) }
            val keyboard = LimitedKeyboard()
            keyboard.arguments = args
//            keyboard.isCancelable = false
            return keyboard
        }
    }


    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View? {
        dialog?.requestWindowFeature(Window.FEATURE_NO_TITLE)
        val root = inflater.inflate(R.layout.limited_keyboard_ll, container, false)
        //这东西就是限制只能输入哪些字符的List
        mInputRule = arguments?.getSerializable("rule") as List<List<String>>
        kbViewPager = root?.find<ViewPager>(R.id.kbViewPager)
        kbViewPager?.adapter = KeyboardAdapter(this, mInputRule, tv)
        //挂点击监听
        root?.find<ImageView>(R.id.backspaceBtn)?.setOnClickListener { deleteLastChar(tv,kbViewPager) }
        dialog?.setOnKeyListener { dialog, keyCode, event ->
            if (keyCode == KeyEvent.KEYCODE_BACK && event.getAction() == KeyEvent.ACTION_DOWN) {
                //删除最后一个字符
                deleteLastChar(tv,kbViewPager)
//                tv?.text = deleteLastChar(tv?.text.toString())
//                if (kbViewPager?.currentItem == 0) dialog.dismiss() else kbViewPager?.let { it.setCurrentItem(it.currentItem - 1)}
                true
            } else {
                false
            }
        }
        return if (root != null) root else super.onCreateView(inflater, container, savedInstanceState)
    }


    /**
     * 删除字符串的最后一个字符
     */
    fun deleteLastChar(tv: TextView?,vp: ViewPager?): Boolean {
        tv?.let { tv.text =  tv.text.substring(0,if(tv.text.length - 1 < 0) 0 else tv.text.length - 1 ) }
        if (vp?.currentItem == 0) dialog?.dismiss() else vp?.let { it.setCurrentItem(it.currentItem - 1)}
        return true
    }

    override fun onStart() {
        super.onStart()
        val window: Window? = dialog!!.window
        window?.let {
            val params: WindowManager.LayoutParams = it.attributes
            params.gravity = Gravity.BOTTOM
            params.width = WindowManager.LayoutParams.MATCH_PARENT
            params.height = DensityUtil.dip2px(context,220f)
            //加这一句可以使弹出时，背景不变暗
            params.dimAmount = 0.0f
            it.attributes = params
            it.setBackgroundDrawable(ColorDrawable(Color.TRANSPARENT))
        }
    }


    override fun onDetach() {
        super.onDetach()
        //如果没输入完，那就要清掉文字重新输
        if (mInputRule.size != tv?.text.toString().length && tv?.text.toString() != "无") {
            tv?.text = ""
        }
    }

    fun show(tv: TextView, manager: FragmentManager, tag: String?) {
        //如果换了一个textView，那就再滚回第一个面板
        setTextView(tv)
        super.show(manager, tag)
    }


    /**弹出动画（动画在我的索尼Z2上卡得一逼，还是不要了。。） */
    fun slideToUp(view: View) {
        val slide = TranslateAnimation(Animation.RELATIVE_TO_SELF, 0.0f,
                Animation.RELATIVE_TO_SELF, 0.0f, Animation.RELATIVE_TO_SELF,
                1.0f, Animation.RELATIVE_TO_SELF, 0.0f).apply {
            duration = 200
            fillAfter = true
            isFillEnabled = true
            interpolator = AccelerateDecelerateInterpolator()
        }

        view.startAnimation(slide)
//        slide.setAnimationListener(object : Animation.AnimationListener {
//            override fun onAnimationStart(animation: Animation) {
//
//            }
//
//            override fun onAnimationEnd(animation: Animation) {
//
//            }
//
//            override fun onAnimationRepeat(animation: Animation) {
//
//            }
//        })

    }

    class KeyboardAdapter(dialog: DialogFragment, strList: List<List<String>>, tv: TextView?) : PagerAdapter() {
        val strList: List<List<String>>
        val tv: TextView?
        val dialog: DialogFragment



        init {
            this.strList = strList
            this.tv = tv
            this.dialog = dialog
        }

        override fun getCount(): Int {
            return strList.size
        }


        override fun isViewFromObject(view: View, view2: Any): Boolean {
            return view == view2
        }

        override fun destroyItem(container: ViewGroup, position: Int, `object`: Any) {
//            super.destroyItem(container, position, `object`)
            val pViewPager = container as ViewPager
            pViewPager.removeViewAt(position)
        }

        override fun instantiateItem(container: ViewGroup, viewPagerPos: Int): Any {
//            return super.instantiateItem(container, position)
            val context = (container as ViewPager).context
            val grid = GridView(context).apply {
                numColumns = if (strList.get(viewPagerPos).size <= 9) 3 else GRID_COLUME_NUM
                stretchMode = GridView.STRETCH_COLUMN_WIDTH
                horizontalSpacing = dip(0)
                verticalSpacing = dip(0)
            }
            grid.adapter = SimpleAdapter(context, makeList(viewPagerPos), R.layout.simple_tv, arrayOf("text"), intArrayOf(R.id.text))
            //点击监听
            grid.setOnItemClickListener { parent, view, gridPos, id ->
                if(strList.get(viewPagerPos).get(gridPos) == "无") {//如果点击无，那就直接把整个输入框显示为无，并收起键盘
                    tv?.text = "无"
                    dialog.dismiss()
                    container.setCurrentItem(0)
                }else{
                    tv?.text = (if (viewPagerPos == 0) "" else tv?.text.toString()) + strList.get(viewPagerPos).get(gridPos)
                    //自动滑动到下一个输入面板
                    when (viewPagerPos) {
                        strList.size - 1 -> {
                            dialog.dismiss()
                            container.setCurrentItem(0)
                        }
                        else -> container.setCurrentItem(viewPagerPos + 1)
                    }
                }
            }
            container.offscreenPageLimit = strList.size - 1
            container.addView(grid, 0)
            //从第一个面板开始输入。
            container.setCurrentItem(0)
            return grid
        }

        /**
         * 制造Adapter使用的hashMapList
         */
        fun makeList(position: Int): MutableList<HashMap<String, String>> {
            val mapList: MutableList<HashMap<String, String>> = mutableListOf()
            strList.get(position).forEach { str -> mapList.add(hashMapOf<String, String>("text" to str)) }
            return mapList
        }
    }
}