package com.urovo.wugumofang.view;

import android.animation.ValueAnimator;
import android.annotation.SuppressLint;
import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.LinearGradient;
import android.graphics.Matrix;
import android.graphics.Paint;
import android.graphics.Path;
import android.graphics.PathMeasure;
import android.graphics.RectF;
import android.graphics.Shader;
import android.util.AttributeSet;
import android.view.View;
import android.view.animation.AccelerateDecelerateInterpolator;
import android.view.animation.LinearInterpolator;

import com.urovo.wugumofang.libutils.R;

import java.text.SimpleDateFormat;
import java.util.Date;

/**
 * 仪表盘 自定义
 * Created by hcc on 16/9/2 19:28
 * 100332338@qq.com
 * 新版芝麻信用圆环View
 * 食用方法：
 *         newCreditSesameView.setSesameValues(0.78f);//即为需要显示的数值是78%，直接传百分数的浮点型就行。
 *
 *
 * 执行顺序：onMeasure() -> onSizeChanged() -> onDraw()
 */
public class NewCreditSesameView extends View {

  // 圆环的信用等级文本
  String[] sesameStr = new String[] {
      "'", "'",
      "'", "'",
      "'", "'",
      "'", "'",
      "'"
  };

  //外缘百分比文本
  String[] percentStr = new String[]{
          "0%","50%","100%"
  };

  //进度条渐变色
  private Shader mShader;

  // 默认宽高值
  private int defaultSize;

  //尺寸倍率（实际宽高值 除以 default默认宽高值）
  private float rate = 1;

  // 两圆环相距的值
  private int arcDistance;

  // view宽度
  private int width;

  // view高度
  private int height;

  // 默认Padding值
  private  static int defaultPadding = 50;

  //  圆环起始角度
  private final static float mStartAngle = 150f;

  // 圆环结束角度
  private final static float mEndAngle = 240f;

  //外层圆环画笔
  private Paint mMiddleArcPaint;

  //内层圆环画笔
  private Paint mInnerArcPaint;

  //信用等级文本画笔
  private Paint mTextPaint;

  //大刻度画笔
  private Paint mCalibrationPaint;

  //小刻度画笔
  private Paint mSmallCalibrationPaint;

  //小刻度画笔
  private Paint mCalibrationTextPaint;

  //外缘百分比画笔
  private Paint mPercentTextPaint;

  //进度圆环画笔
  private Paint mArcProgressPaint;

  //内层进度圆环画笔
  private Paint mArcInnerProgressPaint;

  //半径
  private int radius;

  //外层矩形
  private RectF mMiddleRect;

  //内层矩形
  private RectF mInnerRect;

  //进度矩形
  private RectF mMiddleProgressRect;

  //内层进度矩形
  private RectF mInnerProgressRect;

  // 最小数字
  private float mMinNum = 0;

  // 最大数字
  private float mMaxNum = 1;

  // 当前进度
  private float mCurrentAngle = 0f;

  //总进度
  private float mTotalAngle = 240f;

  //信用等级
  private String sesameLevel = "";

  //评估时间
  private String evaluationTime = "";

  //小圆点
  private Bitmap bitmap;

  //当前点的实际位置
  private float[] pos;

  //当前点的tangent值
  private float[] tan;

  //矩阵
  private Matrix matrix;

  //小圆点画笔
  private Paint mBitmapPaint;


  public NewCreditSesameView(Context context) {

    this(context, null);
  }


  public NewCreditSesameView(Context context, AttributeSet attrs) {

    this(context, attrs, 0);
  }


  public NewCreditSesameView(Context context, AttributeSet attrs, int defStyleAttr) {

    super(context, attrs, defStyleAttr);
    defaultSize = getResSize(R.dimen.ratewatch_defaultSize);
  }


  /**
   * 初始化
   */
  private void init() {
//    Log.i("SasukeLog","init()==========================");

    //外层圆环画笔
    mMiddleArcPaint = new Paint(Paint.ANTI_ALIAS_FLAG);
    mMiddleArcPaint.setStrokeWidth(getResSize(R.dimen.ratewatch_middleArcPaint_size));
    mMiddleArcPaint.setColor(Color.parseColor("#dcdcdc"));
    mMiddleArcPaint.setStyle(Paint.Style.STROKE);
//    mMiddleArcPaint.setAlpha(80);

    //内层圆环画笔
    mInnerArcPaint = new Paint(Paint.ANTI_ALIAS_FLAG);
    mInnerArcPaint.setStrokeWidth(getResSize(R.dimen.ratewatch_mInnerArcPaint_size));
    mInnerArcPaint.setColor(Color.parseColor("#dcdcdc"));
//    mInnerArcPaint.setAlpha(80);
    mInnerArcPaint.setStyle(Paint.Style.STROKE);

    //正中间字体画笔
    mTextPaint = new Paint(Paint.ANTI_ALIAS_FLAG);
//    mTextPaint.setColor(Color.WHITE);
    mTextPaint.setTextAlign(Paint.Align.CENTER);

//    //圆环大刻度画笔
//    mCalibrationPaint = new Paint(Paint.ANTI_ALIAS_FLAG);
//    mCalibrationPaint.setStrokeWidth(4);
//    mCalibrationPaint.setStyle(Paint.Style.STROKE);
//    mCalibrationPaint.setColor(Color.WHITE);
//    mCalibrationPaint.setAlpha(120);

    //圆环小刻度画笔
//    mSmallCalibrationPaint = new Paint(Paint.ANTI_ALIAS_FLAG);
//    mSmallCalibrationPaint.setStrokeWidth(1);
//    mSmallCalibrationPaint.setStyle(Paint.Style.STROKE);
//    mSmallCalibrationPaint.setColor(Color.WHITE);
//    mSmallCalibrationPaint.setAlpha(130);

    //圆环刻度文本画笔
    mCalibrationTextPaint = new Paint(Paint.ANTI_ALIAS_FLAG);
    mCalibrationTextPaint.setTextSize(getResSize(R.dimen.ratewatch_mCalibrationTextPaint));
    mCalibrationTextPaint.setColor(Color.parseColor("#ff7819"));

    //外缘百分比画笔
    mPercentTextPaint = new Paint(Paint.ANTI_ALIAS_FLAG);
    mPercentTextPaint.setTextSize(getResSize(R.dimen.ratewatch_mPercentTextPaint));
    mPercentTextPaint.setColor(Color.parseColor("#ff7819"));

    //外层进度画笔
    mArcProgressPaint = new Paint(Paint.ANTI_ALIAS_FLAG);
    mArcProgressPaint.setStrokeWidth(getResSize(R.dimen.ratewatch_mArcProgressPaint));
//    mArcProgressPaint.setColor(Color.parseColor("#ff7819"));
    mArcProgressPaint.setStyle(Paint.Style.STROKE);
    mArcProgressPaint.setStrokeCap(Paint.Cap.ROUND);

    //内层层进度画笔
    mArcInnerProgressPaint = new Paint(Paint.ANTI_ALIAS_FLAG);
    mArcInnerProgressPaint.setStrokeWidth(getResSize(R.dimen.ratewatch_mArcInnerProgressPaint));
//    mArcInnerProgressPaint.setColor(Color.parseColor("#ff7819"));
    mArcInnerProgressPaint.setStyle(Paint.Style.STROKE);
    mArcInnerProgressPaint.setStrokeCap(Paint.Cap.BUTT);


    mBitmapPaint = new Paint();
    mBitmapPaint.setStyle(Paint.Style.FILL);
    mBitmapPaint.setAntiAlias(true);

    //初始化小圆点图片
    bitmap = BitmapFactory.decodeResource(getResources(), R.drawable.ic_circle);
    pos = new float[2];
    tan = new float[2];
    matrix = new Matrix();

  }

  /**
   * 算出真实的尺寸
   */
  private int trueSize(int size){
    return (int)(size * rate);
  }

  @Override
  protected void onMeasure(int widthMeasureSpec, int heightMeasureSpec) {
    float minSize = Math.min(resolveMeasure(widthMeasureSpec, defaultSize),resolveMeasure(heightMeasureSpec, defaultSize));
    float rateTemp = minSize / defaultSize;
    if(rateTemp != 1){
      rate = rateTemp;
    }
//    Log.i("SasukeLog","倍率rate值为："+rate);
//    Log.i("SasukeLog","minSize值为："+minSize);
//    Log.i("SasukeLog","defaultSize值为："+defaultSize);
//    Log.i("SasukeLog","=======================================");
    setMeasuredDimension((int)minSize,(int)minSize);
  }


  @Override
  protected void onSizeChanged(int w, int h, int oldw, int oldh) {

    super.onSizeChanged(w, h, oldw, oldh);
//    Log.i("SasukeLog","onSizeChanged()==================");
    width = w;
    height = h;
    radius = width / 2;
    defaultPadding = getResSize(R.dimen.ratewatch_defaultPadding);

    arcDistance = getResSize(R.dimen.ratewatch_arcDistance);
//    int truePadding = trueSize(defaultPadding);

    mMiddleRect = new RectF(
            defaultPadding, defaultPadding,
        width - defaultPadding, height - defaultPadding);

    mInnerRect = new RectF(
            defaultPadding + arcDistance,
            defaultPadding + arcDistance,
        width - defaultPadding - arcDistance,
        height - defaultPadding - arcDistance);

    mMiddleProgressRect = new RectF(
            defaultPadding, defaultPadding,
        width - defaultPadding, height - defaultPadding);

    mInnerProgressRect = new RectF(
            defaultPadding + arcDistance,
            defaultPadding + arcDistance,
            width - defaultPadding - arcDistance,
            height - defaultPadding - arcDistance
    );

    //进度条渐变色
    mShader = new LinearGradient(
            defaultPadding , defaultPadding + radius,
             width - defaultPadding, defaultPadding + radius,
            Color.parseColor("#ffa66c"),
            Color.parseColor("#ca4f00"), Shader.TileMode.CLAMP);
  }


  @Override
  protected void onDraw(Canvas canvas) {

    super.onDraw(canvas);
//    Log.i("SasukeLog","onDraw()===========================");
    init();
    drawMiddleArc(canvas);
    drawInnerArc(canvas);
    drawSmallCalibration(canvas);
    drawCalibrationAndText(canvas);
    drawCenterText(canvas);
    drawRingProgress(canvas);
  }


  /**
   * 绘制内层小刻度
   */
  private void drawSmallCalibration(Canvas canvas) {
//    //旋转画布
//    canvas.save();
//    canvas.rotate(-105, radius, radius);
//    //计算刻度线的起点结束点
//    int startDst = (int) (defaultPadding + arcDistance - mInnerArcPaint.getStrokeWidth() / 2 - 1);
//    int endDst = (int) (startDst + mInnerArcPaint.getStrokeWidth());
//    for (int i = 0; i <= 35; i++) {
//      //每旋转6度绘制一个小刻度
//      canvas.drawLine(radius, startDst, radius, endDst, mSmallCalibrationPaint);
//      canvas.rotate(6, radius, radius);
//    }
//    canvas.restore();
  }


  /**
   * 绘制外层圆环进度和小圆点
   */
  private void drawRingProgress(Canvas canvas) {

    Path path = new Path();
    path.addArc(mMiddleProgressRect, mStartAngle, mCurrentAngle);
    PathMeasure pathMeasure = new PathMeasure(path, false);
    pathMeasure.getPosTan(pathMeasure.getLength() * 1, pos, tan);
    matrix.reset();
    matrix.postTranslate(pos[0] - bitmap.getWidth() / 2, pos[1] - bitmap.getHeight() / 2);
    mArcProgressPaint.setShader(mShader);
    canvas.drawPath(path, mArcProgressPaint);

    Path path2 = new Path();
    path2.addArc(mInnerProgressRect, mStartAngle, mCurrentAngle);
    mArcInnerProgressPaint.setShader(mShader);
    canvas.drawPath(path2, mArcInnerProgressPaint);

//    //起始角度不为0时候才进行绘制小圆点
//    if (mCurrentAngle == 0) {
//      return;
//    }
//    canvas.drawBitmap(bitmap, matrix, mBitmapPaint);
//    mBitmapPaint.setColor(Color.WHITE);
//    canvas.drawCircle(pos[0], pos[1], 8, mBitmapPaint);
  }


  /**
   * 绘制中间文本
   */
  private void drawCenterText(Canvas canvas) {
//    //绘制Logo
//    mTextPaint.setTextSize(30);
//    canvas.drawText("BETA", radius, radius - 130, mTextPaint);

    //绘制信用分数
    mTextPaint.setTextSize(getResSize(R.dimen.ratewatch_mTextPaint));
    mTextPaint.setStyle(Paint.Style.STROKE);
    mTextPaint.setColor(Color.parseColor("#ff7819"));

    String percentStr = String.format("%.0f",mMinNum * 100);
    canvas.drawText(percentStr, radius - getResSize(R.dimen.ratewatch_radius_reduce), radius + getResSize(R.dimen.ratewatch_radius_plus), mTextPaint);
    float percentLength = mTextPaint.measureText(percentStr);
    mTextPaint.setTextSize(getResSize(R.dimen.ratewatch_mTextPaint_half));
    canvas.drawText("%", radius  - getResSize(R.dimen.ratewatch_radius_reduce)+ percentLength * 2/3, radius + getResSize(R.dimen.ratewatch_radius_plus), mTextPaint);

//    //绘制信用级别
//    mTextPaint.setTextSize(80);
//    canvas.drawText(sesameLevel, radius, radius + 160, mTextPaint);

    //绘制评估时间
    mTextPaint.setTextSize(getResSize(R.dimen.ratewatch_mTextPaint_half_half));
    mTextPaint.setColor(Color.parseColor("#999999"));
    canvas.drawText("目标完成率", radius, radius + getResSize(R.dimen.ratewatch_goal_radius_plus), mTextPaint);
  }


  /**
   * 绘制刻度
   */
  private void drawCalibrationAndText(Canvas canvas) {
    //旋转画布进行绘制对应的刻度
    canvas.save();
    canvas.rotate(-120, radius, radius);
    //计算刻度线的起点结束点
    int startDst = (int) (defaultPadding+ arcDistance - mInnerArcPaint.getStrokeWidth() / 2 - 1);
    int endDst = (int) (startDst + mInnerArcPaint.getStrokeWidth());
    //刻度旋转的角度
    int rotateAngle = 240 / (sesameStr.length - 1);
    for (int i = 1; i < sesameStr.length + 1; i++) {
      //我们需求不需要drawLine
//      if (i % 2 != 0) {
//        canvas.drawLine(radius, startDst, radius, endDst, mCalibrationPaint);
//      }
      // 测量文本的长度
      float textLen = mCalibrationTextPaint.measureText(sesameStr[i - 1]);
      canvas.drawText(sesameStr[i - 1], radius - textLen / 2, endDst + getResSize(R.dimen.ratewatch_endDst_plus), mCalibrationTextPaint);
      canvas.rotate(rotateAngle, radius, radius);
    }
    canvas.restore();

    //下面开始画外缘百分比文字
    canvas.save();
    canvas.rotate(-120, radius, radius);
    int rotateAngle2 = 240 / (percentStr.length - 1);
    for(int i = 0; i < percentStr.length; i++){
      float textLen = mPercentTextPaint.measureText(percentStr[i]);
      canvas.drawText(percentStr[i], radius - textLen / 2, defaultPadding - arcDistance / 2, mPercentTextPaint);
      canvas.rotate(rotateAngle2, radius, radius);
    }
    canvas.restore();
  }


  /**
   * 绘制内层圆环
   */
  private void drawInnerArc(Canvas canvas) {

    canvas.drawArc(mInnerRect, mStartAngle, mEndAngle, false, mInnerArcPaint);
  }


  /**
   * 绘制外层圆环
   */
  private void drawMiddleArc(Canvas canvas) {

    canvas.drawArc(mMiddleRect, mStartAngle, mEndAngle, false, mMiddleArcPaint);
  }


  /**
   * 根据传入的值进行测量
   */
  public int resolveMeasure(int measureSpec, int defaultSize) {

    int result = 0;
    int specSize = MeasureSpec.getSize(measureSpec);
    switch (MeasureSpec.getMode(measureSpec)) {

      case MeasureSpec.UNSPECIFIED:
        result = defaultSize;
        break;

      case MeasureSpec.AT_MOST:
        //设置warp_content时设置默认值
        result = Math.min(specSize, defaultSize);
        break;
      case MeasureSpec.EXACTLY:
        //设置math_parent 和设置了固定宽高值
        result = specSize;
        break;

      default:
        result = defaultSize;
    }

    return result;
  }


  public void setRateValue(float values) {

//    if (values <= 350) {
//      mMaxNum = values;
//      mTotalAngle = 0f;
//      sesameLevel = "信用较差";
//      evaluationTime = "评估时间:" + getCurrentTime();
//    } else if (values <= 550) {
//      mMaxNum = values;
//      mTotalAngle = (values - 350) * 80 / 400f + 2;
//      sesameLevel = "信用较差";
//      evaluationTime = "评估时间:" + getCurrentTime();
//    } else if (values <= 700) {
//      mMaxNum = values;
//      if (values > 550 && values <= 600) {
//        sesameLevel = "信用中等";
//        mTotalAngle = (values - 550) * 120 / 150f + 43;
//      } else if (values > 600 && values <= 650) {
//        sesameLevel = "信用良好";
//        mTotalAngle = (values - 550) * 120 / 150f + 45;
//      } else {
//        sesameLevel = "信用优秀";
//        mTotalAngle = (values - 550) * 120 / 150f + 48;
//      }
//      evaluationTime = "评估时间:" + getCurrentTime();
//    } else if (values <= 950) {
//      mMaxNum = values;
//      mTotalAngle = (values - 700) * 40 / 250f + 170;
//      sesameLevel = "信用极好";
//      evaluationTime = "评估时间:" + getCurrentTime();
//    } else {
//      mTotalAngle = 240f;
//    }
    mMaxNum = values;//动画要抵达的数值
    mTotalAngle = 240f * (values > 1f? 1f : values);//需要扫过的角度

    startAnim();
  }


  public void startAnim() {

    ValueAnimator mAngleAnim = ValueAnimator.ofFloat(mCurrentAngle, mTotalAngle);
    mAngleAnim.setInterpolator(new AccelerateDecelerateInterpolator());
    mAngleAnim.setDuration(1000);
    mAngleAnim.addUpdateListener(new ValueAnimator.AnimatorUpdateListener() {

      @Override
      public void onAnimationUpdate(ValueAnimator valueAnimator) {

        mCurrentAngle = (float) valueAnimator.getAnimatedValue();
        postInvalidate();
      }
    });
    mAngleAnim.start();

    // mMinNum = 350;
    ValueAnimator mNumAnim = ValueAnimator.ofFloat(mMinNum, mMaxNum);
    mNumAnim.setDuration(1000);
    mNumAnim.setInterpolator(new LinearInterpolator());
    mNumAnim.addUpdateListener(new ValueAnimator.AnimatorUpdateListener() {

      @Override
      public void onAnimationUpdate(ValueAnimator valueAnimator) {

        mMinNum = (float) valueAnimator.getAnimatedValue();
        postInvalidate();
      }
    });
    mNumAnim.start();
  }


  /**
   * dp2px
   */
  public int dp2px(int values) {

    float density = getResources().getDisplayMetrics().density;
    return (int) (values * density + 0.5f);
  }

  /**
   * 从资源文件里获取dp值（还不是为了适配小屏幕！
   * @param rId
   * @return
   */
  public int getResSize(int rId){
    return getResources().getDimensionPixelSize(rId);
  }


  /**
   * 获取当前时间
   */
  public String getCurrentTime() {

    @SuppressLint("SimpleDateFormat")
    SimpleDateFormat format = new SimpleDateFormat("yyyy:MM:dd");
    return format.format(new Date());
  }
}
