package com.urovo.wugumofang.view.baseWebViewAct;

import android.annotation.SuppressLint;
import android.content.Context;
import android.os.Build;
import android.os.Environment;
import android.util.AttributeSet;
import android.webkit.CookieManager;
import android.webkit.GeolocationPermissions;
import android.webkit.WebSettings;
import android.webkit.WebView;
import android.widget.ProgressBar;
import android.widget.TextView;

import com.urovo.wugumofang.net.CookieDominator;
import com.urovo.wugumofang.system.BaseAppManager;
import com.urovo.wugumofang.view.webview.LollipopFixedWebView;
import com.zsy.libext.widgets.webview.WebViewUtil;

/**
 * 通用WebView
 * Created by Sasuke on 2016/11/21.
 */
public class BaseWebView extends LollipopFixedWebView {

    /**
     * 网页缓存目录
     */
    private static final String cacheDirPath = Environment
            .getExternalStorageDirectory() + "/LoadingWebViewDome/webCache/";
    TextView titleTextView;
    private ProgressBar mProgressBar;
    private Context mContext;

    public BaseWebView(Context context) {
        this(context, null);
    }

    public BaseWebView(Context context, AttributeSet attrs) {
        this(context, attrs, 0);
//        initContext(context);
    }

    public BaseWebView(Context context, AttributeSet attrs, int defStyle) {
        super(context, attrs, defStyle);
        initContext(context);
        this.mContext = context;
    }

    public void putTitleView(TextView titleTextView) {
        this.titleTextView = titleTextView;
    }

    @SuppressLint("SetJavaScriptEnabled")
    private void initContext(Context context) {
        requestFocus();
//        setInitialScale(39);
        removeJavascriptInterface("searchBoxJavaBridge_");
        removeJavascriptInterface("accessibility");
        removeJavascriptInterface("accessibilityTraversal");
        getSettings().setJavaScriptCanOpenWindowsAutomatically(true);//支持通过Javascript打开新窗口
        getSettings().setJavaScriptEnabled(true);//设置WebView属性，能够执行Javascript脚本
        getSettings().setUseWideViewPort(true);//将图片调整到适合webview的大小
        getSettings().setLoadWithOverviewMode(true);// 缩放至屏幕的大小
        getSettings().setDomStorageEnabled(true);//设置是否启用了DOM Storage API（一定要true，否则打不开图片）
        getSettings().setDatabaseEnabled(true);//开启database storage API功能
        getSettings().setDatabasePath(cacheDirPath); //设置数据库缓存路径
        getSettings().setAppCachePath(cacheDirPath);//设置Application Caches缓存目录
        getSettings().setAppCacheEnabled(true);//开启Application Caches功能
//        getSettings().setCacheMode(WebSettings.LOAD_NO_CACHE);//不使用缓存。(使用缓存则返回上一页时不会reload)
        getSettings().setGeolocationEnabled(true);
        String dir = BaseAppManager.appContext().getFilesDir().getPath();
        getSettings().setGeolocationDatabasePath(dir);

        getSettings().setLoadsImagesAutomatically(true);
        getSettings().setUseWideViewPort(false);
        getSettings().setLoadWithOverviewMode(false);
        getSettings().setSaveFormData(true);
        getSettings().setSavePassword(false);//webview不要保存密码，官方未来会不支持保存密码
        getSettings().setDefaultZoom(WebSettings.ZoomDensity.MEDIUM);
        String str = getSettings().getUserAgentString() + "-hkd";
        getSettings().setUserAgentString(str);
        CookieManager.getInstance().setAcceptCookie(true);
        getSettings().setSupportZoom(false);
        getSettings().setBuiltInZoomControls(false);
        getSettings().setLayoutAlgorithm(WebSettings.LayoutAlgorithm.SINGLE_COLUMN);
        getSettings().setSupportMultipleWindows(true);
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
            getSettings().setMixedContentMode(WebSettings.MIXED_CONTENT_ALWAYS_ALLOW);//解决不加载图片问题
        }
        getSettings().setBlockNetworkImage(false);//解决不加载图片问题
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.KITKAT) {//自动化测试H5内容需要
            WebView.setWebContentsDebuggingEnabled(true);
        }
        this.setLongClickable(true);
        this.setScrollbarFadingEnabled(true);
        this.setDrawingCacheEnabled(true);
        WebViewUtil.hideBuiltInZoomControls(this);
        setWebViewClient(new CookieWebViewClient());
    }

    @Override
    public boolean onCheckIsTextEditor() {
        return true;
    }

    /**
     * 加载网页url
     *
     * @param url
     */
    public void loadMessageUrl(String url,String userId) {
        CookieDominator.setCommonCookie(mContext, url, userId);//设置通用Cookie，所有用这个WebView的都要带这个。

        loadUrl(url);
    }

    /**
     * 添加进度条
     */
    public void addProgressBar() {
        mProgressBar = new ProgressBar(getContext(), null,
                android.R.attr.progressBarStyleHorizontal);
        mProgressBar.setLayoutParams(new LayoutParams(
                LayoutParams.MATCH_PARENT, 5, 0, 0));
        /*mProgressBar.setProgressDrawable(getContext().getResources()
                .getDrawable(R.drawable.bg_pb_web_loading));*/
        addView(mProgressBar);//添加进度条至LoadingWebView中

        setWebChromeClient(new WebChromeClient());//设置setWebChromeClient对象
    }

    public class WebChromeClient extends android.webkit.WebChromeClient {
        @Override
        public void onReceivedTitle(WebView view, String title) {
            super.onReceivedTitle(view, title);
            if (titleTextView != null) {
                titleTextView.setText(title);
            }
        }

        @Override
        public void onProgressChanged(WebView view, int newProgress) {
            if (newProgress == 100) {
                mProgressBar.setVisibility(GONE);
            } else {
                if (mProgressBar.getVisibility() == GONE)
                    mProgressBar.setVisibility(VISIBLE);
                mProgressBar.setProgress(newProgress);
            }
            super.onProgressChanged(view, newProgress);
        }

        /**
         * 用来兼容低版本安卓上H5的定位
         */
        @Override
        public void onGeolocationPermissionsShowPrompt(String origin, GeolocationPermissions.Callback callback) {
            callback.invoke(origin, true, false);
        }
    }

    /**
     * 回收webview
     */
    public void destroyWebView() {
        removeAllViews();
        clearCache(true);
        clearHistory();
        destroy();
    }
}
