package com.urovo.wugumofang.view.baseWebViewAct;

import android.annotation.SuppressLint;
import android.net.Uri;
import android.net.http.SslError;
import android.text.TextUtils;
import android.webkit.SslErrorHandler;
import android.webkit.WebResourceRequest;
import android.webkit.WebResourceResponse;
import android.webkit.WebView;
import android.webkit.WebViewClient;


import com.urovo.wugumofang.system.BaseAppManager;
import com.urovo.wugumofang.system.SystemHelper;
import com.urovo.wugumofang.net.CookieDominator;

import java.util.Map;

/**
 * Created by Sasuke on 2017/6/6.
 * 用来拦截所有 WebView内部请求的WebViewClient（给拦截下来的url添加cookie）
 */

public class CookieWebViewClient extends WebViewClient {
    @Override
    public boolean shouldOverrideUrlLoading(WebView view, String url) { // 重写此方法表明点击网页里面的链接不调用系统浏览器，而是在本WebView中显示
        CookieDominator.setCommonCookie(BaseAppManager.appContext(), url, SystemHelper.LoginUserId);//设置通用Cookie，所有用这个WebView的都要带这个。
        view.loadUrl(url);
        return true;
    }

    //兼容高版本
    @SuppressLint("NewApi")
    @Override
    public WebResourceResponse shouldInterceptRequest(WebView view, final WebResourceRequest request) {
        if (request != null && request.getUrl() != null) {
            String scheme = request.getUrl().getScheme().trim();
            if (scheme.equalsIgnoreCase("http") || scheme.equalsIgnoreCase("https")) {
                return super.shouldInterceptRequest(view, new WebResourceRequest() {

                    @Override
                    public boolean isRedirect() {
                        return false;
                    }

                    @Override
                    public Uri getUrl() {
                        CookieDominator.setCommonCookie(BaseAppManager.appContext(), request.getUrl().toString(), SystemHelper.LoginUserId);
                        return Uri.parse(request.getUrl().toString());
                    }

                    @SuppressLint("NewApi")
                    @Override
                    public boolean isForMainFrame() {
                        return request.isForMainFrame();
                    }

                    @SuppressLint("NewApi")
                    @Override
                    public boolean hasGesture() {
                        return request.hasGesture();
                    }

                    @SuppressLint("NewApi")
                    @Override
                    public String getMethod() {
                        return request.getMethod();
                    }

                    @SuppressLint("NewApi")
                    @Override
                    public Map<String, String> getRequestHeaders() {
                        return request.getRequestHeaders();
                    }
                });
            }
        }
        return super.shouldInterceptRequest(view, request);
    }

    //兼容低版本
    @Override
    public WebResourceResponse shouldInterceptRequest(WebView view, String url) {
        if (!TextUtils.isEmpty(url) && Uri.parse(url).getScheme() != null) {
            String scheme = Uri.parse(url).getScheme().trim();
            if (scheme.equalsIgnoreCase("http") || scheme.equalsIgnoreCase("https")) {
                CookieDominator.setCommonCookie(BaseAppManager.appContext(), url, SystemHelper.LoginUserId);
                return super.shouldInterceptRequest(view, url);
            }
        }
        return super.shouldInterceptRequest(view, url);
    }

    @Override
    public void onReceivedSslError(WebView view, SslErrorHandler handler, SslError error) {
        if(error.getPrimaryError() == android.net.http.SslError.SSL_INVALID ){// 校验过程遇到了bug（安卓7.0可能会遇到WebView打不开的情况，所以需要允许这种情况继续执行）
            handler.proceed();
        }else{
            handler.cancel();
        }
//        super.onReceivedSslError(view, handler, error);
    }
}
