package com.urovo.wugumofang.view.salelinechart;

import android.content.Context;
import android.graphics.Color;
import android.graphics.DashPathEffect;
import android.graphics.drawable.Drawable;
import androidx.core.content.ContextCompat;

import com.github.mikephil.charting.charts.LineChart;
import com.github.mikephil.charting.components.Description;
import com.github.mikephil.charting.components.XAxis;
import com.github.mikephil.charting.components.YAxis;
import com.github.mikephil.charting.data.Entry;
import com.github.mikephil.charting.data.LineData;
import com.github.mikephil.charting.data.LineDataSet;
import com.github.mikephil.charting.formatter.DefaultValueFormatter;
import com.github.mikephil.charting.formatter.IndexAxisValueFormatter;
import com.github.mikephil.charting.highlight.Highlight;
import com.github.mikephil.charting.interfaces.datasets.ILineDataSet;
import com.github.mikephil.charting.listener.OnChartValueSelectedListener;
import com.github.mikephil.charting.utils.Utils;
import com.urovo.wugumofang.libutils.R;

import java.util.ArrayList;
import java.util.List;

/**
 * @author Sasuke on 2017/12/31.
 * 为销售数据专门订制的折线图
 * <p>
 * 食用方法：
 * List<SaleLineChartCase.loadItem> dataList = new ArrayList<>();
 * chart = (LineChart) findViewById(R.id.chart1);
 * final SaleLineChartCase saleChart = new SaleLineChartCase(this,chart);
 * saleChart.loadData(dataList);
 */

public class SaleLineChartCase {
    public static class loadItem {
        public loadItem(String xText, float yValue) {
            this.xText = xText;
            this.yValue = yValue;
        }

        String xText = "";//从外部传进来的x轴数据（如果不传就自己造）
        float yValue = 0.00f;//同上
    }

    Context context;
    List<Integer> colors = new ArrayList<>();
    int defaultXColor = Color.parseColor("#999999");
    int highLithgColor = Color.parseColor("#26b8f2");
    LineChart mChart;

    private List<loadItem> loadList = new ArrayList<>();//外面传进来

    public SaleLineChartCase(Context context, LineChart chart) {
        this.context = context;
        this.mChart = chart;
        init();
    }



    /**
     * 点击显示结点数据
     * @param rLayout
     */
    public void setMarkerView(Context context, int rLayout) {
        mChart.setDrawMarkers(true);//点击显示
        CustomMarkerView marker = new CustomMarkerView(context, R.layout.line_chart_markerview_tv);
        marker.setChartView(mChart);
        mChart.setMarker(marker);
    }

    public void init() {
        mChart.setGridBackgroundColor(Color.TRANSPARENT);//不知道是什么背景，调了也没效果
        mChart.setBorderColor(Color.TRANSPARENT);//边框？？？不知道

        mChart.getAxisRight().setEnabled(false);//右轴取消
        mChart.getAxisLeft().setEnabled(false);//左轴取消
        mChart.getAxisRight().setDrawLabels(false);//右轴不画
        mChart.getAxisLeft().setDrawLabels(false);//左轴不画

        mChart.setDrawGridBackground(false);//网格背景要画？？？没效果啊
        mChart.getLegend().setEnabled(false);//传奇？？不懂
        mChart.setPinchZoom(false);//不可放大缩小
        Description description = new Description();
        description.setText("");
        mChart.setDescription(description);//不显示横轴描述
        mChart.setTouchEnabled(true);//需要可以左右滑动的，还需要可以点击
        mChart.setDoubleTapToZoomEnabled(false);//好像是双指缩放？？不需要
        mChart.getXAxis().setEnabled(true);//月份还是要画的
        mChart.getXAxis().setPosition(XAxis.XAxisPosition.BOTTOM);//废话，X轴不放在底下你还要上天？？？
        mChart.getXAxis().setDrawGridLines(false);
        mChart.getXAxis().setAxisLineColor(Color.TRANSPARENT);//需要上X轴需要透明
        mChart.getXAxis().setAvoidFirstLastClipping(true);

        //各轴的可拖动性。
        mChart.setDragXEnabled(true);//使X轴可拖动
        mChart.setDragYEnabled(false);//使Y轴不可拖动
        mChart.setScaleYEnabled(false);//使Y轴不可缩放！！
        mChart.setScaleXEnabled(false);//使X轴不可缩放！！

        //点击显示结点数据
        setMarkerView(context, R.layout.line_chart_markerview_tv);


        //选中监听（用于改变被点击的项的x轴的颜色）
        mChart.setOnChartValueSelectedListener(new OnChartValueSelectedListener() {
            @Override
            public void onValueSelected(Entry e, Highlight h) {
                //设置X轴点到的文字颜色
//                Log.i("SasukeLog","点击到了第 "+h.getX() + "个X轴的元素");//这个是获取到X轴的第几个元素
                int colorLength = colors.size();
                for (int i = 0; i < colorLength; i++) {//先全部恢复默认颜色，再设置高亮色
                    colors.set(i, defaultXColor);
                }
                colors.set((int) h.getX(), highLithgColor);
            }

            @Override
            public void onNothingSelected() {
                int colorLength = colors.size();
                for (int i = 0; i < colorLength; i++) {//先全部恢复默认颜色，再设置高亮色
                    colors.set(i, defaultXColor);
                }
            }
        });


        //可以把左右两边空出点位置，免得markerView在点击弹出后被边缘挤掉。
        mChart.getXAxis().setSpaceMax(0.5f);
        mChart.getXAxis().setSpaceMin(0.5f);
    }


    /**
     * 加载完数据之后要做的事
     */
    private void loadAfter(int xRangeNum, int dataSize) {
        mChart.setVisibleXRangeMaximum(xRangeNum - 1);//要在设置数据源后调用，否则是无效的（这里因为要一屏显示6个月，所以要设为个区间）
        //从最后一个元素开始渲染，就是一进来先看到最后一个元素。
        if (dataSize > 0) {
            mChart.moveViewToX(dataSize - 1);
        }
//        invalidate();
        mChart.animateY(1000);

        //手动显示markerView（一定要在图表渲染后再调用，否则会崩（mChart会为空，没错，就是会为空，放在初始化里也不行！））
        Highlight highlight = new Highlight(50f, 0, 0);
        mChart.highlightValue(highlight, false);
    }


    /**
     * 传入第二个数据list，大体跟下面的loadData()差不多。。。
     */
    public void loadDataMore(List<loadItem> loadList) {
        ArrayList<Entry> values = new ArrayList<Entry>();
        int dataSize = loadList.size();
        //把y值存起来
        for (int xIndex = 0; xIndex < dataSize; xIndex++) {
            //这个表格库好像只能全局显示圆点图案，不能单个显示，所以感觉getDrawable这里没啥屁用
            values.add(new Entry(xIndex, loadList.get(xIndex).yValue));
        }

        LineDataSet set2;
        int setCount = mChart.getData().getDataSetCount();
        if (mChart.getData() != null && setCount > 0) {//其实不太懂这一段是什么意思
            if (setCount == 2) {//如果已经有第二个折线了，那就替换刷新
                set2 = (LineDataSet) mChart.getData().getDataSetByIndex(1);
                set2.setValues(values);
            } else if (setCount == 1) {//如果只有一个折线，那就直接添加
                set2 = createDateSet(values, "DataSet 2");//我猜是在绘多曲线的时候要用来区分吧。
                set2.setCircleColor(Color.TRANSPARENT);
                set2.setCircleColorHole(Color.TRANSPARENT);
                set2.setColor(context.getResources().getColor(R.color.chart_second_line));
                mChart.getLineData().addDataSet(set2);
            }

            //set2 创建或设置完了，刷新chart图表
            mChart.getData().notifyDataChanged();
            mChart.notifyDataSetChanged();
        } else {
            return;
        }
    }

    /**
     * 从外部载入数据
     * （我管你后台传什么。。。进到这个chart里来就得用这个数据模型，转也得给我转换好了再传进来 ）
     */
    public void loadData(List<loadItem>... loadListArray) {
        List<loadItem> loadList = loadListArray[0];//第一个数据源

        //初始化X轴文字
        List<String> xTexts = new ArrayList<>();
        for (loadItem item : loadList) {
            xTexts.add(item.xText);
        }
        //给X轴的文字设置颜色
        colors.clear();
        int dataSize = loadList.size();
        for (int i = 0; i < dataSize - 1; i++) {
            colors.add(defaultXColor);
        }
        colors.add(highLithgColor);//最新的一个结点X轴文字高亮
        mChart.setXAxisRenderer(new ColoredLabelXAxisRenderer(//这个东西可以让X轴文字的颜色根据传入的colors列表进行实时变换
                mChart.getViewPortHandler(), mChart.getXAxis(), mChart.getTransformer(YAxis.AxisDependency.LEFT), colors));

        //以下自定义X轴
        mChart.getXAxis().setValueFormatter(new IndexAxisValueFormatter(xTexts));
        mChart.getXAxis().setLabelCount(dataSize);
        mChart.getXAxis().setDrawLabels(true);
        mChart.getXAxis().setGranularity(1f);//这个感觉就是那个IndexAxisValueFormatter的数据偏移量，放在xValues里就是序号一次偏移多少吧（eg. 1,2,3,4或1，3，5，7）

        ArrayList<Entry> values = new ArrayList<Entry>();
        //把y值存起来
        for (int i = 0; i < dataSize; i++) {
            //这个表格库好像只能全局显示圆点图案，不能单个显示，所以感觉getDrawable这里没啥屁用
            values.add(new Entry(i, loadList.get(i).yValue, context.getResources().getDrawable(R.drawable.red_circle)));
        }

        LineDataSet set1;
        if (mChart.getData() != null && mChart.getData().getDataSetCount() > 0) {//其实不太懂这一段是什么意思
            set1 = (LineDataSet) mChart.getData().getDataSetByIndex(0);
            set1.setValues(values);
            mChart.getData().notifyDataChanged();
            mChart.notifyDataSetChanged();
        } else {
            set1 = createDateSet(values, "DataSet 1");


            ArrayList<ILineDataSet> dataSets = new ArrayList<ILineDataSet>();
            dataSets.add(set1); // add the datasets

            // create a data object with the datasets
            LineData data = new LineData(dataSets);
            // set data
            mChart.setData(data);
        }
        //加载完数据之后要做的事
        loadAfter(6, dataSize);//一屏需要显示6个数据

        //以上第一根线就已经定义完了
        //继续添加更多数据源
        if (loadListArray.length > 1) {
            for (int index = 1; index < loadListArray.length; index++) {
                loadDataMore(loadListArray[index]);
            }
        }
    }

    /**
     * 创建一个基本的折线数据集
     * @return
     */
    private LineDataSet createDateSet(ArrayList<Entry> values, String label) {
// create a dataset and give it a type
        LineDataSet set = new LineDataSet(values, label);//我猜是在绘多曲线的时候要用来区分吧。
        set.setDrawIcons(false);
        // 描曲线边缘虚线的 "- - - - - -"
//            set.enableDashedLine(10f, 5f, 0f);
//            set.enableDashedHighlightLine(10f, 5f, 0f);
        //据说是点击后那个以焦点为原点的十字线条的颜色
        set.setHighLightColor(Color.TRANSPARENT);
        //结点的默认灰色小圆点
        set.setCircleColor(Color.WHITE);
        set.setCircleColorHole(Color.parseColor("#FED7A0"));
        set.setLineWidth(1f);
        set.setCircleRadius(4f);
        set.setCircleHoleRadius(3f);
        set.setDrawCircleHole(true);
        set.setValueTextSize(9f);
        set.setValueFormatter(new DefaultValueFormatter(2));//大概是用来规定保留几位小数的吧
        //不需要虚线，谢谢~！
        set.setFormLineWidth(1f);
        set.setFormLineDashEffect(new DashPathEffect(new float[]{10f, 5f}, 0f));
        set.setFormSize(15.f);
        //不去显示数据，只有在点击的时候才显示被点击的数据（放屁，我点击了还不是没反应，还是要自己自定义markerView才能显示点击数据）
        set.setDrawValues(false);

        set.setDrawFilled(true);//填充线下方的区域（本需求里是一个渐变灰色）
        if (Utils.getSDKInt() > 18) {
            // fill drawable only supported on api level 18 and above
            Drawable drawable = ContextCompat.getDrawable(context, R.drawable.chart_fade_gray);
            set.setFillDrawable(drawable);
            //api 18以上的不显示连线（新：结果新需求还是要显示一条连线，如果不需要显示线，就设为透明）
            set.setColor(context.getResources().getColor(R.color.chart_first_line));
        } else {
            //api 18及以下的，就让它显示一条线吧。。。填充颜色好像有问题
            set.setColor(context.getResources().getColor(R.color.chart_first_line));
            set.setFillColor(Color.parseColor("#E6E6E6"));
        }
        return set;
    }
}
