package com.urovo.wugumofang.view.webview;

import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.webkit.WebViewClient;

import com.urovo.wugumofang.system.SystemHelper;

import java.util.HashMap;

/**
 *
 * @author Sasuke
 * @date 2017/5/23
 * 进一步封装组件中带Cookie的WebView(主工程中调用这个)
 */

public class BaseCookiesWebFra {

    /**
     * 启动一个带单个cookie的webView
     *
     * @param url          H5页面链接
     * @param title        webView标题
     * @param isShowBottom ...
     */
    public static void launch(Context context,
                              String url,
                              String title,
                              Boolean isShowBottom) {
        BaseCookieWebFra.launch(context, url, SystemHelper.LoginUserId, title, isShowBottom);
    }

    /**
     * 启动一个带多个cookie的webView
     */
    public static void launch(Context context,
                              String url,
                              HashMap<String, String> params,
                              String title,
                              Boolean isShowBottom) {
        BaseCookieWebFra.launch(context, url, params, title, isShowBottom);
    }

    /**
     * 添加一个WebViewClient，带单个cookie的webView
     *
     * @param url              ...
     * @param title            ...
     * @param isShowBottom     ...
     * @param newWebViewClient ...
     */
    public static void launch(Context context,
                              String url,
                              String title,
                              Boolean isShowBottom,
                              WebViewClient newWebViewClient) {
        BaseCookieWebFra.launch(context, url, SystemHelper.LoginUserId, title, isShowBottom, newWebViewClient);
    }

    /**
     * 添加一个WebViewClient，带多个cookie的webView
     */
    public static void launch(Context context,
                              String url,
                              HashMap<String, String> params,
                              String title,
                              Boolean isShowBottom,
                              WebViewClient newWebViewClient) {
        BaseCookieWebFra.launch(context, url, params, title, isShowBottom, newWebViewClient);
    }

    /**
     * 当前页面跳转的指定的带单个cookie的webview页面（WebContainerAct）
     *
     * @param context     必须是当前需要跳转的页面的content
     * @param intent       数据通过intent传递
     * @param url          \(^o^)/~h5页面链接
     * @param title        webView标题
     * @param isShowBottom 展示底部与否
     */
    public static void launch(Context context, Intent intent,
                              String url,
                              String title,
                              Boolean isShowBottom) {
        Bundle bundle = BaseCookieWebFra.webContainerBundle(context,
                url, SystemHelper.LoginUserId, title, isShowBottom);
        intent.putExtras(bundle);
        intent.setClass(context, WebContainerAct.class);
        context.startActivity(intent);
    }

    /**
     * 当前页面跳转的指定的带单个cookie的webview页面（WebContainerAct）
     *
     * @param context     必须是当前需要跳转的页面的content
     * @param intent       数据通过intent传递
     * @param url          \(^o^)/~h5页面链接
     * @param title        webView标题
     * @param isShowShareBtn 展示顶栏分享按钮
     */
    public static void launchWithShare(Context context, Intent intent,
                              String url,
                              String title,
                              Boolean isShowShareBtn) {
        Bundle bundle = BaseCookieWebFra.webContainerBundleShare(context,
                url, SystemHelper.LoginUserId, title, isShowShareBtn);
        intent.putExtras(bundle);
        intent.setClass(context, WebContainerAct.class);
        context.startActivity(intent);
    }

    /**
     * 指定当前页面跳转的指定的带多个cookie的webview页面（WebContainerAct）
     */
    public static void launch(Context context, Intent intent,
                              String url,
                              HashMap<String, String> params,
                              String title,
                              Boolean isShowBottom) {
        Bundle bundle = BaseCookieWebFra.webContainerBundle(context,
                url, params, title, isShowBottom);
        intent.putExtras(bundle);
        intent.setClass(context, WebContainerAct.class);
        context.startActivity(intent);
    }
}
