package com.zsy.libcore.model.data.net.interceptor;

import com.zsy.libcore.common.other.NetUtil;
import java.io.IOException;
import okhttp3.CacheControl;
import okhttp3.Headers;
import okhttp3.Interceptor;
import okhttp3.Request;
import okhttp3.Response;

/**
 * 缓存机制

 无网络时，也能显示数据

 * 此拦截器为网络缓存器。只会改变本地缓存时间的大小，不会影响到是否能获取到本地缓存文件
 */
public class NetCacheInterceptor implements Interceptor {
  private int maxCacheSeconds;
  private Headers commonHeaders;

  public NetCacheInterceptor() {
  }

  public NetCacheInterceptor(int maxCacheSeconds, Headers commonHeaders) {
    this.maxCacheSeconds = maxCacheSeconds;
    this.commonHeaders = commonHeaders;
  }

  @Override public Response intercept(Chain chain) throws IOException {
      /*  Request request = chain.request();
        Request.Builder builder = request.newBuilder();
        if (commonHeaders != null) {
            for (String name : commonHeaders.names()) {
                builder.addHeader(name, commonHeaders.get(name));
            }
        }
        request = builder.build();
        Response originalResponse = chain.proceed(request);
        // rewrite the response headers to support cache.
        if (NetUtil.isNetworkAvailable(null)) {
            return originalResponse.newBuilder()
//                    .header("Cache-Control", "public, max-age=" + EPocketHttpService.DEFAULT_CACHE_MAX_SECONDS)
                    .build();
        } else {
            return originalResponse.newBuilder()
                    .header("Cache-Control", "public, only-if-cached, max-stale=" + Integer.MAX_VALUE)
                    .build();
        }*/
    Request request = chain.request();
    if (!NetUtil.isNetworkAvailable(null)) {
      request = request.newBuilder().cacheControl(CacheControl.FORCE_CACHE).build();
    }
    Response response = chain.proceed(request);
    if (NetUtil.isNetworkAvailable(null)) {// 有网络时, 不缓存, 最大保存时长为0
      int maxAge = 0;
      response.newBuilder()
          .header("Cache-Control", "public, max-age=" + maxAge)
          .removeHeader("Pragma")
          .build();
    } else {// 无网络时，设置超时为4周
      int maxStale = 60 * 60 * 24 * 28;
      response.newBuilder()
          .header("Cache-Control", "public, only-if-cached, max-stale=" + maxStale)
          .removeHeader("Pragma")
          .build();
    }
    return response;
  }
}