package com.zsy.libext.widgets.tips;

import android.app.Activity;
import android.content.Context;
import android.graphics.Color;
import android.view.Gravity;
import android.view.View;
import android.view.ViewGroup;
import android.view.animation.AlphaAnimation;
import android.view.animation.Animation;
import android.view.animation.AnimationSet;
import android.view.animation.TranslateAnimation;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;
import android.widget.Toast;

/**
 * 多样式工具吐司类
 */
public class ToastUtil {
    private static Toast toast = null;
    public static int LENGTH_LONG = Toast.LENGTH_LONG;
    private static int LENGTH_SHORT = Toast.LENGTH_SHORT;

    private ToastUtil() {
    }

    public static void show(Context context, String content) {
        Toast.makeText(context, content, Toast.LENGTH_SHORT).show();
    }

    public static void show(Context context, int contentId) {
        Toast.makeText(context, contentId, Toast.LENGTH_SHORT).show();
    }

    /**
     * 普通文本消息提示
     *
     * @param context
     * @param text
     * @param duration
     */
    public static void TextToast(Context context, CharSequence text, int duration) {
        //创建一个Toast提示消息
        toast = Toast.makeText(context, text, duration);
        //设置Toast提示消息在屏幕上的位置
        toast.setGravity(Gravity.CENTER, 0, 0);
        //显示消息
        toast.show();
    }

    /**
     * 带图片消息提示
     *
     * @param context
     * @param ImageResourceId
     * @param text
     * @param duration
     */
    public static void ImageToast(Context context, int ImageResourceId, CharSequence text, int duration) {
        //创建一个Toast提示消息
        toast = Toast.makeText(context, text, Toast.LENGTH_LONG);
        //设置Toast提示消息在屏幕上的位置
        toast.setGravity(Gravity.CENTER, 0, 0);
        //获取Toast提示消息里原有的View
        View toastView = toast.getView();
        //创建一个ImageView
        ImageView img = new ImageView(context);
        img.setImageResource(ImageResourceId);
        //创建一个LineLayout容器
        LinearLayout ll = new LinearLayout(context);
        //向LinearLayout中添加ImageView和Toast原有的View
        ll.addView(img);
        ll.addView(toastView);
        //将LineLayout容器设置为toast的View
        toast.setView(ll);
        //显示消息
        toast.show();
    }

    /**
     * 自定义从顶部弹出的吐司
     *
     * @param context
     * @param str
     * @param time
     */
    public static void Toast(Context context, String str, int time) {
        final TextView textControl = new TextView(context);
        textControl.setText(str);
//        textControl.setBackgroundResource(R.drawable.tipline);
        textControl.setGravity(Gravity.CENTER_HORIZONTAL);
        textControl.setPadding(0, 8, 0, 0);
        textControl.setTextSize(18);
        textControl.setTextColor(Color.parseColor("#ffffff"));
        LinearLayout.LayoutParams params = new LinearLayout.LayoutParams(
                ViewGroup.LayoutParams.FILL_PARENT, ViewGroup.LayoutParams.WRAP_CONTENT);
        ((Activity) context).addContentView(textControl, params);
        textControl.measure(0, 0);
        TranslateAnimation ta = new TranslateAnimation(0, 0,
                -textControl.getMeasuredHeight(), 0);
        ta.setDuration(500);
        AlphaAnimation aa = new AlphaAnimation(0.5f, 1);
        aa.setDuration(500);
        TranslateAnimation ta2 = new TranslateAnimation(0, 0,
                0, -textControl.getMeasuredHeight());
        ta2.setDuration(500);
        ta2.setStartOffset(1500);
        AlphaAnimation aa2 = new AlphaAnimation(1, 0.5f);
        aa2.setDuration(500);
        aa2.setStartOffset(1500);

        AnimationSet as = new AnimationSet(true);
        as.addAnimation(ta2);
        as.addAnimation(aa2);
        as.addAnimation(ta);
        as.addAnimation(aa);
        as.setFillAfter(true);
        as.setAnimationListener(new Animation.AnimationListener() {

            @Override
            public void onAnimationStart(Animation animation) {

            }

            @Override
            public void onAnimationRepeat(Animation animation) {

            }

            @Override
            public void onAnimationEnd(Animation animation) {
                textControl.setVisibility(View.VISIBLE);
            }
        });
        textControl.startAnimation(as);
    }
}