package com.urovo.wugumofang.printermodule

import com.urovo.wgmf.db.dao.dao.XbzpfDbOperate
import com.urovo.wgmf.db.dao.entitys.ProductEntity
import com.urovo.wugumofang.data.Arith
import com.urovo.wugumofang.system.SystemHelper

/**
 * @author Sasuke on 2018/8/17.
 * 根据条码长度返回条码类型
 */

fun code2Type(barcode: String): String {
    return when (barcode.length) {
        12, 13 -> PrinterTsc.TSC_CODE_13
        else -> PrinterTsc.TSC_CODE_128
    }
}

/***
 * 计算校验码
 * barcode: 传入未添加校验码的原始条码
 * checkCodeFlag：是否需要校验码
 *
 * 返回的是添加校验码之后的完整条码，不是单独校验码
 */
fun makeCheckCode(barcode: String, checkCodeFlag: Boolean) =
        when (barcode.length) {
            //在这里加条码类型①
            12, 21 -> PrinterUtil.getInstance().barcode22Check(barcode, checkCodeFlag)// 13位单品校验(可能是大润发码)
            17, 19, 23, 27 -> PrinterUtil.getInstance().barcode20Check(barcode, checkCodeFlag)
            else -> barcode
        }

/**
 * 过滤，留下五谷磨房产品，排除鲜稻研，用于打印（因为鲜稻研产品不支持打印）
 */
fun filterWgmfForPrint(bill: List<ProductEntity>): List<ProductEntity> {
    return bill.filter { !it.isDecimal }
}


/**
 * 计算分配价格的比例，这个比例在打印条码的时候会用到。
 * 值为1时，表示不需要按比例均摊价格。
 *
 * 顺带还处理了十方的价格分摊
 */
fun specialPrice2PriceDealShiFang(proList: MutableList<ProductEntity>, actualPrice: Double, totalPrice: Double): Double {
    var newPricePercantage = totalPrice.takeIf { it != 0.0 }?.run { Arith.div(actualPrice, totalPrice) }
            ?: 1.0

    //先筛出十方的子产品，并均摊特价
    var hasSpecialShiFang = dealShiFangPros(proList)

    //点“继续添加”按钮后添加的产品specialPrice的价格为0，这样原有价格会被0覆盖，所以要先判断 >0
    proList.filter { it.specialPrice > 0 }.forEach {
        it.price = it.specialPrice
        it.amount = Arith.mul(it.price, it.tqty)//这里要用特价重新计算总价，因为可能在change数据模型的时候，转换算法问题，转的时候用的是原价
        newPricePercantage = 1.0
    }

    if (hasSpecialShiFang) {//如果存在十方产品，就把外层的百分比恢复成1.0，因为filterShiFangPros()已经做了分摊，外面的产品不需要做分摊价格了。
        newPricePercantage = 1.0
    }
    return newPricePercantage
}

/**
 * 三黑粉之类的虚拟产品，在商超里是没有的，它其实是对应的N个产品的集合（一般是十方）
 * 所以需要在打印之前就把这个虚拟产品拆分成N个十方
 *
 * 传入的产品List将踢除十方大产品，并且方法返回一个十方大产品拆分出来的各个小产品
 */
fun dealShiFangPros(proList: MutableList<ProductEntity>): Boolean {
    var tempList: MutableList<ProductEntity> = mutableListOf()
    val shiFangIds = PrinterDbOperate.GetShiFangItems(SystemHelper.LoginOrganId).map { it.itemId }
    var shiFangActualPrice = 0.0
    var shiFangTotalPrice = 0.0
    var percentShifang = 1.0
    var proNum = 0.0
    var proAmout = 0.0
    proList.filter { product -> product.itemId in shiFangIds }.forEach { product ->
        val shifangDetail = PrinterDbOperate.GetShiFangDetail(product.itemId)//获取详情
        val bomspec = PrinterDbOperate.GetBomInfo(product.itemId).BOMSPEC//获取BOMSPEC
        if ("y".equals(bomspec)) {
            proAmout += product.price * product.tqty//总价
            shifangDetail.map {
                proNum += it.bzqty * product.tqty//数量
            }
        }
    }
    proList.filter { product -> product.itemId in shiFangIds }.forEach { product ->
        //如果该产品是十方
        shiFangTotalPrice += Arith.mul(product.price, product.tqty)//十方总价
        shiFangActualPrice += Arith.mul(if (product.specialPrice == 0.0) product.price else product.specialPrice, product.tqty)//十方实付
        val shifangDetail = PrinterDbOperate.GetShiFangDetail(product.itemId)//获取详情
        val bomspec = PrinterDbOperate.GetBomInfo(product.itemId).BOMSPEC//获取BOMSPEC
        if ("y".equals(bomspec)) {
            //特殊BOM产品:BOMSPEC y-是 明细和数量规则不变，总金额取该产品的金额:NORMRPRICE；
            tempList.addAll(shifangDetail.map {
                ProductEntity().apply {
                    //转换元素模型并添加到temp
                    itemId = it.xbzid
                    itemName = it.xbzname
                    // 数量为组合数量*配单包数
                    tqty = it.bzqty * product.tqty
                    // 售价为总价/组合数量
                    price = proAmout / proNum
                    amount = Arith.mul(tqty, price)
                    var entity = XbzpfDbOperate.GetProductDetail(product.itemId)
                    itemMode = entity?.itemMode
                    kcUom = entity?.kcUom
                }
            })
        } else {
            tempList.addAll(shifangDetail.map {
                ProductEntity().apply {
                    //转换元素模型并添加到temp
                    itemId = it.xbzid
                    itemName = it.xbzname
                    // 数量为组合数量*配单包数
                    tqty = it.bzqty * product.tqty
                    // 售价为组合明细的标准售价
                    price = it.bzprice
                    amount = Arith.mul(tqty, price)
                    var entity = XbzpfDbOperate.GetProductDetail(product.itemId)
                    itemMode = entity?.itemMode
                    kcUom = entity?.kcUom
                }
            })
        }

        percentShifang = if (shiFangActualPrice == 0.0 || shiFangTotalPrice == 0.0) 1.0 else Arith.div(shiFangActualPrice, shiFangTotalPrice)
        tempList.forEach {
            //开始分摊十方特价（这里的十方子产品据说不会有特价，因为如果有特价那就会让他们单独为单品配特价，不会以十方的形式）
            it.price = Arith.mul(it.price, percentShifang)
            it.amount = Arith.mul(it.amount, percentShifang)
        }
    }

    proList.removeAll { it.itemId in shiFangIds }
    //分摊完了再把它们添加到原产品表里
    proList.addAll(tempList)
    return tempList.size > 0 && percentShifang != 1.0//返回  是否有特价十方产品（同时满足有十方并且十方有特价）
}