package com.urovo.wugumofang.printermodule;

import androidx.annotation.NonNull;

import java.util.List;


/**
 * @author chenji
 * @date 2017/8/18
 * 组合码类型打印
 */
public class CombineCodePrinter implements CodePrinter {
    private PrinterCallBack mPrinterCallBack;

    @Override
    public CodePrinter setOnPrinterCallBack(@NonNull PrinterCallBack printerCallBack) {
        this.mPrinterCallBack = printerCallBack;
        return this;
    }

    @Override
    public void printer(PrinterUtil.PrinterRule rule, List<PrinterUtil.PrinterRule> printList) {
        //初始化
        PrinterUtil.getInstance().initJudgement(rule, mPrinterCallBack);
        getCombineCodeType(rule, printList, mPrinterCallBack);
    }

    /**
     * 组合码--打印条码类型（金额，额数，数量，数额）
     *
     * @param rule 规则
     */
    private void getCombineCodeType(PrinterUtil.PrinterRule rule,
                                    List<PrinterUtil.PrinterRule> printList,
                                    PrinterCallBack printerCallBack) {

        //第一步：判断数量/金额是否超过上限
        getNumLimitation(rule, printList, printerCallBack);
        getAmountLimitation(rule, printList, printerCallBack);

        //第二步：组装条码（未验证）
        String barcode = PrinterUtil.makeCodeNotCheck(rule,rule.mTotalAMT,rule.mCount);

        //第三步：校验最后一位（根据规则判断是否需要检验）
        //校验后rule.mCode.length()就被赋值等同于整个条形码的长度
        PrinterUtil.getInstance().checkLastCode(rule, barcode, printList, printerCallBack);

        //第四步：打印排版
        PrinterUtil.getInstance().printCode(rule, barcode, printList);
    }

    private void getAmountLimitation(PrinterUtil.PrinterRule rule, List<PrinterUtil.PrinterRule> printList, PrinterCallBack printerCallBack) {
        int amountCodeLength = 0;
        int accuracy = 0;
        if ("M".equals(rule.combinedCodeTypeOne)) {
            amountCodeLength = rule.combinedCodeTypeLengthOne;
            accuracy = rule.combinedCodeTypeDotOne;
        } else if ("M".equals(rule.combinedCodeTypeTwo)) {
            amountCodeLength = rule.combinedCodeTypeLengthTwo;
            accuracy = rule.combinedCodeTypeDotTwo;
        } else if ("M".equals(rule.shelfLifeTypeThree)) {
            amountCodeLength = rule.shelfLifeLengthThree;
            accuracy = rule.shelfLifeDaysThree;
        }
        PrinterUtil.getInstance().amountLimitation(rule, amountCodeLength,
                accuracy, printList, printerCallBack);
    }

    @SuppressWarnings("all")
    private void getNumLimitation(PrinterUtil.PrinterRule rule, List<PrinterUtil.PrinterRule> printList, PrinterCallBack printerCallBack) {
        // 判断数量是否超过上限
        int numCodeLength = 0;
        int accuracy = 0;

        if ("N".equals(rule.combinedCodeTypeOne)) {
            numCodeLength = rule.combinedCodeTypeLengthOne;
            accuracy = rule.combinedCodeTypeDotOne;
        } else if ("N".equals(rule.combinedCodeTypeTwo)) {
            numCodeLength = rule.combinedCodeTypeLengthTwo;
            accuracy = rule.combinedCodeTypeDotTwo;
        } else if ("N".equals(rule.shelfLifeTypeThree)) {
            numCodeLength = rule.shelfLifeLengthThree;
            accuracy = rule.shelfLifeDaysThree;
        }
        PrinterUtil.getInstance().numLimitation(rule, numCodeLength,
                accuracy, printList, printerCallBack);
    }
}
