package com.urovo.wugumofang.printermodule;

import androidx.annotation.NonNull;
import android.text.TextUtils;

import com.urovo.wugumofang.constants.AppConstants;
import com.urovo.wugumofang.data.MoneyUtils;
import com.urovo.wugumofang.data.PreferHelper;
import com.urovo.wugumofang.system.SystemHelper;
import com.zsy.libcore.common.android.log.SYLoger;

import java.util.List;
import java.util.Locale;

/**
 * Created by chenji on 2017/8/18.
 * 单品码类型打印
 */

public class ItemCodePrinter implements CodePrinter {
    private PrinterCallBack mPrinterCallBack;

    @Override
    public CodePrinter setOnPrinterCallBack(@NonNull PrinterCallBack printerCallBack) {
        this.mPrinterCallBack = printerCallBack;
        return this;
    }

    @Override
    public void printer(PrinterUtil.PrinterRule rule, List<PrinterUtil.PrinterRule> printList) {
//        第一步：判断条码是否为空（单品码barcode就是条形码）
        if (TextUtils.isEmpty(rule.mCode)) {
            printList.clear();// 无法打印的时候清空集合数据
            mPrinterCallBack.displayError(MoneyUtils.displayError(rule.mTitle, "该商品无条码"));
            String msg = StatusDbMap.statusDbMap("PRT103","打印条码标题为：" + rule.mTitle + " 的该商品无条码，该条码无法打印！").replace(StatusDbMap.PRO_NAME,rule.mTitle);
            mPrinterCallBack.showToast(msg);
            PreferHelper.getInstance().setBoolean(AppConstants.Printer.IS_END_PRINTER, true);
            SYLoger.print("question: 条码位数，这里应该终止其他产品打印的流程。");
            return;
        }
//        第二步：条码长度是否符合规则要求且排除自定义码（自定义码不限制长度）
        if (rule.mCode.length() != rule.barCodeLength && rule.barCodeLength != 0) {// barcode长度跟规则规定长度不等并且不是自定义码（rule.barCodeLength==0）
            mPrinterCallBack.displayError(MoneyUtils.displayError(rule.mTitle, "条码不是" + rule.barCodeLength + "位"));
            String msg = StatusDbMap.statusDbMap("PRT104","打印条码标题为：" + rule.mTitle + " 的条码不是" + rule.barCodeLength + "位，该条码无法打印！").replace(StatusDbMap.PRO_NAME,rule.mTitle);
            mPrinterCallBack.showToast(msg);
            PreferHelper.getInstance().setBoolean(AppConstants.Printer.IS_END_PRINTER, true);
            SYLoger.print("条码打印A: 位数： rule.mCode:" + rule.mCode + ", rule.mCode length: " + rule.mCode.length() + ", barCodeLength:" + rule.barCodeLength);
            SYLoger.print("question: 条码位数，这里应该终止其他产品打印的流程。");
            return;
        }
//        第三步：底部价格显示区域排版需要的数据
        for (int i = 0; i < rule.mCount; i++) {//底部价格显示区域
            PrinterUtil.PrinterRule cell = rule.clone();
            String price = String.format(Locale.CHINA, "%.2f", rule.mPrice);
            cell.mCount = 1;
            cell.mSale = String.format(Locale.CHINA, "￥%s (%s*%d)", price, price, 1);//因Count为1 存在BarCode13或者12+1
            PrinterUtil.getInstance().checkLastCode(cell, cell.mCode, printList, mPrinterCallBack);//是否需要校验
            printList.add(cell);//添加到打印集合，只要集合有数据走完流程就执行到打印
            SYLoger.print("条码打印：" + "LoginOrganId=" + SystemHelper.LoginOrganId + ", " + cell);
        }
    }
}
