package com.urovo.wugumofang.printermodule;

import android.app.Activity;
import android.app.AlertDialog;
import android.bluetooth.BluetoothAdapter;
import android.content.DialogInterface;
import android.content.DialogInterface.OnCancelListener;
import android.os.Bundle;
import android.text.TextUtils;
import android.widget.SimpleAdapter;
import android.widget.Toast;

import com.alibaba.android.arouter.launcher.ARouter;
import com.urovo.wgmf.db.dao.bill.DalCommon;
import com.urovo.wgmf.db.dao.entitys.Cpbarbill002Entity;
import com.urovo.wgmf.db.dao.entitys.ProductEntity;
import com.urovo.wugumofang.constants.AppConstants;
import com.urovo.wugumofang.constants.RouterParams;
import com.urovo.wugumofang.data.Arith;
import com.urovo.wugumofang.data.PrefTool;
import com.urovo.wugumofang.data.PreferHelper;
import com.urovo.wugumofang.system.BaseAppManager;
import com.urovo.wugumofang.system.Prefs;
import com.urovo.wugumofang.system.SystemHelper;
import com.zsy.libcore.common.android.log.SYLoger;

import java.lang.ref.WeakReference;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 　　　　　　　　┏┓　　　┏┓　　　+ +
 * 　　　　　　　┏┛┻━━━┛┻┓ + +
 * 　　　　　　　┃　　　　　　　┃
 * 　　　　　　　┃　　　━　　　┃ ++ + + +
 * 　　　　　 ████━████ ┃+
 * 　　　　　　　┃　　　　　　　┃+
 * 　　　　　　　┃　　　┻　　　┃
 * 　　　　　　　┃　　　　　　　┃ + +
 * 　　　　　　　┗━┓　　　┏━┛
 * 　　　　　　　　　┃　　　┃
 * 　　　　　　　　　┃　　　┃ + + + +
 * 　　　　　　　　　┃　　　┃
 * 　　　　　　　　　┃　　　┃ + 　　　　坑爹的条码打印，佛祖保佑！
 * 　　　　　　　　　┃　　　┃
 * 　　　　　　　　　┃　　　┃ +
 * 　　　　　　　　　┃　 　　┗━━━┓ + +
 * 　　　　　　　　　┃ 　　　　　　　┣┓
 * 　　　　　　　　　┃ 　　　　　　　┏┛
 * 　　　　　　　　　┗┓┓┏━┳┓┏┛ + + + +
 * 　　　　　　　　　　┃┫┫　┃┫┫
 * 　　　　　　　　　　┗┻┛　┗┻┛+ + + +
 * 类型  打印类型  总价码组成
 * 01  13位
 * 02  13位  7位编码  5位金额  1位序列号
 * 03  13位  8位条码  4位数量  1位序列号
 * -------7位条码  5位数量  1位序列号
 * 11  18位  7位条码  5位数量  5位金额  1位校验码
 * 12  18位  6位条码  3位数量  8位金额（数量*单价）  1位校验码
 * 13  18位  7位条码  5位金额  5位数量  1位校验码
 * <p>
 * 校验码规则：
 * 13位的校验码＝    10－（奇数位和+偶数位和*3）取个位数
 * 18位的校验码＝    10－（奇数位和*3+偶数位和）取个位数
 * <p>
 * 条码：
 * 条码基础资料表——对应关系找产品条码
 * <p>
 * 打印格式：格式内容：条码明细、总金额
 * 3种打印机：
 * 1、C90：esc打印，设置16进制
 * 2、tsc：sdk打印，调用包，指定com8
 * 3、xt4231
 *
 * @author ...
 */
public class PrinterTool implements PrinterUtil.PrintCallback, CodePrinter.PrinterCallBack, OnCancelListener {
    private double mTotal;
    private double totalPrices = 0;
    private List<Double> totalAMTs = new ArrayList<>();//以前没写注释，感觉是每张纸上的10个产品的总价，用于后面逐一校验是否超过最大999.99元之类的。
    /**
     * 各类条形码数组 （在PrintUtil P904 添加的，真特么难找）
     */
    private List<PrinterUtil.PrinterRule> mPrintList = new ArrayList<>();
    private AlertDialog mAlertDialog;
    private SimpleAdapter mSimpleAdapter;
    // 控制是否打印吐司的开关
    private boolean printToastKey = true;

    /**
     * 打印标签
     */
    private List<PrinterUtil.MultipleItem> mPrintLabelList = new ArrayList<>();
    /**
     * listData书签（条码纸）集合
     */
    private List<Map<String, Object>> mListDataInfo;
    /**
     * 请求打印的次数
     */
    private int mPrintCounts;
    private OnPrinterTool mOnPrinterTool;
    private String mAddress;
    /**
     * 标识是TSC打印机
     */
    private int mMachine;
    private int mBmpX;
    private int mBmpY;
    private String mBmpPath;
    private String mFileName;
    /**
     * 打印类型：0-msg 1-bmp 2-打印标签（只是打印）
     */
    private int mType;
    /**
     * 新价格比例,默认比例为1
     */
    private double newPricePercentage = 1;
    private WeakReference<Activity> mARef;

    public PrinterTool(Activity act, OnPrinterTool l) {
        mARef = new WeakReference<>(act);
        mOnPrinterTool = l;
        mListDataInfo = new ArrayList<>();
        mSimpleAdapter = new SimpleAdapter(getActivity(), mListDataInfo, R.layout.print_info,
                new String[]{"icon", "name", "info"},
                new int[]{R.id.imageViewIcon, R.id.textViewName, R.id.textViewInfo});
        mAlertDialog = new AlertDialog.Builder(getActivity()).setTitle("正在连接打印机...")
                .setMessage("")//create的时候就要设message，这样后面再修改显示时才能看得见。
                .setAdapter(mSimpleAdapter, null)
                .setCancelable(true)
                .setOnCancelListener(this)
                .create();
    }

    public Activity getActivity() {
        if (mARef != null && mARef.get() != null) {
            return mARef.get();
        } else {
            return null;
        }
    }

    @Override
    public void onSearchPrint() {
        if (!(getActivity()).isFinishing()) {
            mAlertDialog.setTitle("正在连接打印机...");
        }
        if (null != mOnPrinterTool) {
            mOnPrinterTool.onBegin();
        }
    }

    @Override
    public void onStartPrint() {
        if (!(getActivity()).isFinishing()) {
            mAlertDialog.setTitle("开始打印...");
        }
    }

    @Override
    public void onFinishPrint() {
        if (!(getActivity()).isFinishing()) {
            mAlertDialog.dismiss();
        }
        if (null != mOnPrinterTool) {
            mOnPrinterTool.onFinish();
        }
    }

    @Override
    public void onStopPrint() {
        //标签打印的editext 为空的时候不吐司;设置"isNotEmpty",因为用户不设置edittext的时候默认数量为1
        String labelDialogCancel = PreferHelper.getInstance().getString(AppConstants.Printer.LABEL_DIALOG_CANCLE, "isNotEmpty");
        if (!labelDialogCancel.isEmpty()) {
            showMsg(StatusDbMap.statusDbMap("PRT110", "打印完成!"));
        } else {
            showMsg(StatusDbMap.statusDbMap("PRT111", "打印失败,当前标签数量不能为空!"));
        }
    }

    @Override
    public void onPrintOneSuccessed(int index, String title, String sale, String code) {
        Map<String, Object> cell = new HashMap<>(3);
        cell.put("icon", R.drawable.print_ok);
        cell.put("name", title);
        cell.put("info", "打印成功");
        mListDataInfo.add(cell);
        mSimpleAdapter.notifyDataSetChanged();
    }

    @Override
    public void onBarCodeTypeInvalid(String type) {
        showMsg(StatusDbMap.statusDbMap("PRT103", "不支持该种条码类型打印：" + type).replace(StatusDbMap.PRO_NAME, ""));
        if (!(getActivity()).isFinishing()) {
            mAlertDialog.dismiss();
        }
    }

    @Override
    public void onBarCodeIsInvalid(int index, String title, String sale, String code) {
        String msg = StatusDbMap.statusDbMap("PRT108", "该条码校验出错或者无数据：" + title + ",条码值为：" + code)
                .replace(StatusDbMap.PRO_NAME, title);
        showMsg(msg);
        Map<String, Object> cell = new HashMap<>(3);
        cell.put("icon", R.drawable.print_fail);
        cell.put("name", title);
        cell.put("info", "打印失败：code " + code);
        mListDataInfo.add(cell);
        mSimpleAdapter.notifyDataSetChanged();
    }

    @Override
    public void onAddressInvalidOrCanNotFound() {
        mPrintCounts++;
        if (!(getActivity()).isFinishing()) {
            mAlertDialog.dismiss();
        }
        if (getActivity() != null) {
            if (mPrintCounts > 1) {
                showMsg(StatusDbMap.statusDbMap("PRT113", "打印机连接超时：地址有误或者不在附近，请重启打印机再返回重新打印或者配对蓝牙"));
                if (getActivity() != null) {
                    ARouter.getInstance().build(RouterParams.ORIGIN.SYSTEMSETTINGBLUETOOTH)
                            .navigation(getActivity(), 1);
                }
            } else {
                showMsg(StatusDbMap.statusDbMap("PRT114", "打印机连接超时：程序将自动第" + (mPrintCounts + 1) + "连接！"));
                if (0 == mType) {
                    interPrintMsg();
                } else if (1 == mType) {
                    interPrintBmp();
                } else if (2 == mType) {
                    interPrintLabel();
                }
            }
        }
    }

    @Override
    public void onPrinterError(String error) {
        showMsg("打印机出问题：" + (error == null ? "null" : error));
        if (!(getActivity()).isFinishing()) {
            mAlertDialog.setTitle("打印机出问题：");
            mAlertDialog.setMessage(error);
        }
    }

    @Override
    public void onPrinterErrorBarcodeToThousand(String error) {
        PrinterUtil.getInstance().stopPrint();
        showMsg(error);
    }

    @Override
    public void onCancel(DialogInterface dialog) {
        PrinterUtil.getInstance().stopPrint();
        showMsg(StatusDbMap.statusDbMap("PRT115", "您已经取消了打印!"));
    }

    private void displayError(String title, String error) {
        Map<String, Object> cell = new HashMap<>(3);
        cell.put("icon", R.drawable.print_fail);
        cell.put("name", title);
        cell.put("info", "生成失败： " + error);
        mListDataInfo.add(cell);
        mSimpleAdapter.notifyDataSetChanged();
    }

    /**
     * 打印文字
     * 注意：传进来的bill请务必深拷贝一份再传进来 ，因为这里涉及到newPricePercentage来均摊折扣价格，会改变原价
     * 为了打印之后保存订单时，原价不被改变，所以这里打印用深拷贝传进来才行。
     *
     * @param bill        待打印的实体数组
     * @param actualPrice 实付价格
     * @param totalPrice  总价格
     */
    public void printMsg(List<ProductEntity> bill, double actualPrice, double totalPrice) {
        //一进来就算均摊价格
        this.newPricePercentage = BarcodeUtilKt.specialPrice2PriceDealShiFang(bill, actualPrice, totalPrice);
        //如果有产品含有特价（包括打折）那就不需要平摊价格
        for (ProductEntity pro : bill) {
            if (pro.getSpecialPrice() != 0) {
                this.newPricePercentage = 1;
                break;
            }
        }
        SYLoger.print("条码打印1:--newPricePercentage:" + newPricePercentage);
        if (checkStoreAndBill(bill)) {
            //保证同一个对象传入，价格满减折扣才生效
            processingActivitiesPrice(bill);//均摊价格
            makeBarcode(BarcodeUtilKt.filterWgmfForPrint(bill), actualPrice);//均摊完之后再过滤得到五谷磨房产品用来打印
            //还原总价，用于下次打印
            totalPrices = 0;
            totalAMTs.clear();

            SYLoger.print("totalPrices:--printToastKey:" + printToastKey);
            SYLoger.print("mPrintList.size():" + mPrintList.size());
            SYLoger.print("!(getActivity()).isFinishing():" + !(getActivity()).isFinishing());

            if (0 == mPrintList.size() && !(getActivity()).isFinishing()) {
                mAlertDialog.dismiss();
                if (printToastKey) {
                    showMsg(StatusDbMap.statusDbMap("PRT100", "无条码打印或不支持当前条码类型!"));
                }
            } else {
                mTotal = totalPrice;
                mPrintCounts = 0;
                interPrintMsg();
            }
        }
    }

    /**
     * 条形码标签打印入口
     */
    public void printMsg() {
        mPrintCounts = 0;
        mType = 0;
        interPrintMsg();
    }

    public void printBmp(int x, int y, String path, String fileName, String titleText, String bottomText) {
        mType = 1;
        mBmpX = x;
        mBmpY = y;
        mBmpPath = path;
        mFileName = fileName;
        interPrintBmp(titleText, bottomText);
    }

    /**
     * 打印标签入口
     */
    public void printLabel(List<PrinterUtil.MultipleItem> labelPrinterTotalList) {
        mPrintLabelList.addAll(labelPrinterTotalList);
        mPrintCounts = 0;
        mType = 2;
        interPrintLabel();
    }

    /**
     * 自定义标题文字&&底部文字
     */
    private void interPrintBmp(String titleText, String bottomText) {
        if (checkBluetoothValid()) {
            PrinterUtil.getInstance()
                    .printBitmap(mAddress, mMachine, this, mBmpX, mBmpY, mBmpPath, mFileName, titleText, bottomText);
        }
    }

    /**
     * 默认的配置
     */
    private void interPrintBmp() {
        if (checkBluetoothValid()) {
            PrinterUtil.getInstance()
                    .printBitmap(mAddress, mMachine, this, mBmpX, mBmpY, mBmpPath, mFileName, SystemHelper.LoginUserName, "企业微信专属二维码");
        }
    }

    /**
     * 打印数据Item和条形码
     */
    @SuppressWarnings("all")
    private void interPrintMsg() {
        if (checkBluetoothValid() && null != mPrintList && mPrintList.size() >= 0) {
            if (!(getActivity()).isFinishing()) {
                mAlertDialog.setTitle("打印提示");
                mAlertDialog.show();
                PrinterUtil.PrinterRule cell = mPrintList.get(0);
                if (TextUtils.isEmpty(cell.mType)) {
                    mAlertDialog.dismiss();
                    showMsg(StatusDbMap.statusDbMap("PRT103", "无打印条码类型！"));
                    return;
                }

                //排版一类型
                if (cell.mBarCodePaper.equals(AppConstants.Printer.SMALL_PAPER)
                        // 单品码
                        && (cell.barCodeType.equals(AppConstants.Printer.mItemCode)
                        //组合码
                        || cell.barCodeType.equals(AppConstants.Printer.mCombineCode))) {
                    //小条码纸打印,一个标签对应一个产品
                    PrinterUtil.getInstance().printMsg(mAddress, mMachine, this, mPrintList, mTotal);
                }

                //BIG_PAPER 排版2类型
                else if (cell.mBarCodePaper.equals(AppConstants.Printer.BIG_PAPER)
                        && (cell.barCodeType.equals(AppConstants.Printer.mCombineCode))) {
                    //大条码纸打印,一个标签对应多个产品
                    PrinterUtil.getInstance().printGroupMsg(mAddress, mMachine, this, mPrintList, mTotal);
                }

                // 除了1 2 排版外的排版 3 4 类型
                // 大润发码,目前在这个基础上,新增多一种排版,对应barCodePaper =4
                else if (cell.barCodeType.equals(AppConstants.Printer.mCombineCode)) {
                    // 大条码纸打印,一个标签对应一个产品
                    PrinterUtil.getInstance().printMsg(mAddress, mMachine, this, mPrintList, mTotal);
                }
            }
        }
    }

    /**
     * 打印标签
     */
    @SuppressWarnings("all")
    private void interPrintLabel() {
        if (checkBluetoothValid() && null != mPrintLabelList && mPrintLabelList.size() >= 0) {
            //标签打印的editext 为空的时候不弹窗
            if (!(getActivity()).isFinishing()) {
                mAlertDialog.show();
            }
            PrinterUtil.getInstance().printLabel(mAddress, mMachine, this, mPrintLabelList);
        }
    }

    /**
     * 检查是否蓝牙可用，是否连接TSC打印机
     *
     * @return 是否可用
     */
    private boolean checkBluetoothValid() {
        boolean resault = false;
        if (getActivity() != null) {
            String blueDetail = PrefTool.getStringPerferences(getActivity(), Prefs.PRINT_ADDRESS, "");
            if (!TextUtils.isEmpty(blueDetail)) {
                //直接取前面的地址
                mAddress = blueDetail.substring(0, blueDetail.indexOf("#"));
                BluetoothAdapter bluetoothAdapter = BluetoothAdapter.getDefaultAdapter();
                if (bluetoothAdapter == null) {
                    showMsg(StatusDbMap.statusDbMap("PRT112", "本设备不支持蓝牙功能"));
                } else if (!bluetoothAdapter.isEnabled()) {
                    bluetoothAdapter.enable();
                    showMsg(StatusDbMap.statusDbMap("PRT116", "正在打开蓝牙，请过会重新点击打印！"));
                } else {
                    mMachine = PrinterUtil.NONE;
                    if (blueDetail.contains("SP") || blueDetail.contains("sp") || blueDetail.contains("BHS")
                            || blueDetail.contains("bhs") || blueDetail.contains("L80F")) {//增加L80F打印机机型识别
                        mMachine = PrinterUtil.TSC;
                    }
                    if (PrinterUtil.NONE == mMachine) {
                        showMsg(StatusDbMap.statusDbMap("PRT112", "该蓝牙设备不具有打印功能！"));
                    } else {
                        resault = true;
                    }
                }
            } else {
                ARouter.getInstance().build(RouterParams.ORIGIN.SYSTEMSETTINGBLUETOOTH)
                        .navigation(getActivity(), 1);
            }
        }
        return resault;
    }

    /**
     * 验证门店和待打印的数据
     *
     * @param bill 待打印的数据
     * @return 是否验证成功
     */
    private boolean checkStoreAndBill(List<ProductEntity> bill) {
        boolean resault = false;
        if (null == bill || bill.size() <= 0) {
            showMsg(StatusDbMap.statusDbMap("PRT100", "打印清单是空的！"));
        } else {
            List<Cpbarbill002Entity> listCpBarBill002 = DalCommon.GetCpbarbill002(SystemHelper.LoginOrganId);
            if (listCpBarBill002 == null || listCpBarBill002.size() <= 0) {
                showMsg(StatusDbMap.statusDbMap("PRT101", "该门店无商超条码数据,无法打印！"));
            } else {
                resault = true;
            }
        }
        return resault;
    }

    /**
     * 处理活动价格折扣和组合码总价金额上限（这里只要是大条码纸组合码的金额上限的处理）
     *
     * @param list 产品
     * @@param flag 开关
     */
    private void processingActivitiesPrice(List<ProductEntity> list) {
        for (int i = 0; i < list.size(); i++) {
            ProductEntity p = list.get(i);
            SYLoger.print("条码打印1: name--" + p.getItemName() + ", ItemId:" + p.getItemId());
            //按活动律率折算价格
            p.setAmount(Arith.mul(p.getAmount(), newPricePercentage));
            SYLoger.print("条码打印1：getAmount：" + p.getAmount() + ", newPricePercentage: " + newPricePercentage);
            //==========大条码纸组合码多张纸金额上限处理（遍历）============
            // 当遍历到产品数量为10的长度就把总价和存到集合里，并清空之前的计算
            if (i != 0 && i % 10 == 0) {
                totalAMTs.add(totalPrices);
                totalPrices = 0;
            }
            // 产品总价求和，后累计i%10,先累加i%9
            totalPrices += p.getAmount();
            SYLoger.print("条码打印1： totalAMts: " + totalAMTs);
            // 把剩余的产品综合添加到集合中，后续只需要遍历判断，即可表示每张纸总价总和是否超过金额上限
            if (i == list.size() - 1) {
                totalAMTs.add(totalPrices);
            }
            //==========大条码纸组合码多张纸金额上限处理============
        }
        //默认并重置各种开关
        PreferHelper.getInstance().setBoolean(AppConstants.Printer.IS_OPEN, true);
        // 组合码标记清空集合开关
        PreferHelper.getInstance().setBoolean(AppConstants.Printer.IS_END_PRINTER, false);
        //组合码单页总价集合
        PreferHelper.getInstance().setList(AppConstants.Printer.TOTAL_AMTS, totalAMTs);
        //默认设置吐司开关为true,每次只弹指定吐司(针对组合码大条纸来说)
        PreferHelper.getInstance().setBoolean(AppConstants.Printer.IS_SHOW_TOAST, true);
    }

    /**
     * 组装，制作条形码
     *
     * @param bill 处理后的集合
     */
    private void makeBarcode(List<ProductEntity> bill, double actualPrice) {
        printToastKey = true;
        mPrintList.clear();
        mListDataInfo.clear();
        for (ProductEntity p : bill) {
            PrinterUtil.PrinterRule c = PrinterDbOperate.GetCanPanDianProductsForNewPrint(p.getItemId(), new PrinterUtil.PrinterRule());
            PrinterUtil.PrinterRule pr = PrinterDbOperate.getPrinterRuleByCpbarbill002(p.getItemId());
            //总数量
            c.mCount = p.getTQTY();
            //总价:
            c.mTotalAMT = p.getAmount();
            //标题
            c.mTitle = p.getItemName();
            //单价
            c.mPrice = (c.mTotalAMT / c.mCount);
            //特殊BOM产品:BOMSPEC y-是 明细和数量规则不变，总金额取该产品的金额:NORMRPRICE；
            //纸张大小
            c.mBarCodePaper = pr.mBarCodePaper;
            //配置规则
            c.mPrintRule = pr.mPrintRule;
            // 条码打印解析规则
            parsingRules2CodePrinter(c);

            /*
            条码规则为空的几种可能性分析：
                1.OE可能没有配置或者配置的barcode不符合规则，故后台返回规则为空
                2.可能后台自己弄错返回为空
                3.可能Android端自己获取为空，代码写错
            */
            if (TextUtils.isEmpty(c.mBarCodePaper) && TextUtils.isEmpty(c.mPrintRule)) {
                // Cpbarbill002 表中没有配置这个款产品或者配置的产品归纳十方了
                if (TextUtils.isEmpty(pr.mCode)) {
                    String msg = StatusDbMap.statusDbMap("PRT101", "条码目录没有" + c.mTitle + "，该条码无法打印！");
                    displayError(c.mTitle, msg);
                    showMsg(msg);
                } else {
                    String msg = StatusDbMap.statusDbMap("PRT102", c.mTitle + "的条码规则为空，该条码无法打印！");
                    displayError(c.mTitle, msg);
                    showMsg(msg);
                }
                // 不让第二条吐司弹出
                printToastKey = false;
                PreferHelper.getInstance().setBoolean(AppConstants.Printer.IS_END_PRINTER, true);
                continue;
            }

            if (TextUtils.isEmpty(c.barCodeType)
                    || !(c.barCodeType.equals(AppConstants.Printer.mItemCode)
                    || c.barCodeType.equals(AppConstants.Printer.mCombineCode))) {
                // 不让第二条吐司弹出
                printToastKey = false;
                String msg = StatusDbMap.statusDbMap("PRT101", "该商品无条码类型");
                displayError(c.mTitle, msg);
                showMsg(msg);
                // 不让第二条吐司弹出
                printToastKey = false;
                PreferHelper.getInstance().setBoolean(AppConstants.Printer.IS_END_PRINTER, true);
                break;
            }

            if (c.mCount <= 0) {
                String msg = StatusDbMap.statusDbMap("PRT100", "打印条码标题为：" + c.mTitle + " 数量为0，该条码无法打印！");
                displayError(c.mTitle, msg);
                showMsg(msg);
                // 不让第二条吐司弹出
                printToastKey = false;
                PreferHelper.getInstance().setBoolean(AppConstants.Printer.IS_END_PRINTER, true);
                break;
            }

            if (!TextUtils.isEmpty(c.mCode) && c.barCodeLength != 0 && c.barCodeLength != c.mCode.length()) {//mCode为空的情况后面会有判断的
                String msg = StatusDbMap.statusDbMap("PRT104", "配置barcode长度有误").replace(StatusDbMap.PRO_NAME, c.mTitle);
                displayError(c.mTitle, msg);
                showMsg(msg);
                // 不让第二条吐司弹出
                printToastKey = false;
                PreferHelper.getInstance().setBoolean(AppConstants.Printer.IS_END_PRINTER, true);
                break;
            }

            if (null != c.mCode) {
                c.mCode = c.mCode.trim();
            }
            SYLoger.print("条码打印:*****************************************************************************************************");
            SYLoger.print("条码打印: 组合码： " + c.barCodeType);
            // 条码类型处理
            PrinterFactory.getInstance(c.barCodeType)
                    .setOnPrinterCallBack(PrinterTool.this)
                    .printer(c, mPrintList);
        }

        // 条码打印前处理
        // 把（大条纸组合码）所有产品总价判断的开关开启，以便下次打印用
        PreferHelper.getInstance().setBoolean(AppConstants.Printer.IS_OPEN, true);
        //只要有一件商品价格超过上限，就标志一下,会执行清空集合操作。
        boolean isEndPrinter = PreferHelper.getInstance().getBoolean(AppConstants.Printer.IS_END_PRINTER, false);
        SYLoger.print(
                "totalPrices: " + "isOpen--" + PreferHelper.getInstance().getBoolean(AppConstants.Printer.IS_OPEN,
                        false) + ", isEndPrinter:" + isEndPrinter);
        if (isEndPrinter && mPrintList != null && mPrintList.size() != 0) {
            // 不让第二条吐司弹出
            printToastKey = false;
            mPrintList.clear();
            //恢复标志
            PreferHelper.getInstance().setBoolean(AppConstants.Printer.IS_END_PRINTER, false);
        }
    }

    /**
     * 条码规则解析
     *
     * @param c 规则实例
     */
    private void parsingRules2CodePrinter(PrinterUtil.PrinterRule c) {
        try {
            if (c.mPrintRule != null && !c.mPrintRule.isEmpty()) {
                String[] prMember = c.mPrintRule.split("[;]");
                SYLoger.print("" + prMember.length);

                String[] splitsOne = prMember[0].split("[,]");
                // 条码类型
                c.barCodeType = splitsOne.length == 0 ? "" : splitsOne[0];
                c.barCodeLength = splitsOne.length == 0 ? 0 : Integer.parseInt(splitsOne[1]);

                String[] splitsTwo = prMember[1].split("[,]");
                c.combinedCodeTypeOne = splitsTwo.length == 0 || TextUtils.isEmpty(splitsTwo[0]) ? "" : splitsTwo[0];
                c.combinedCodeTypeLengthOne = splitsTwo.length == 0 || TextUtils.isEmpty(splitsTwo[0]) ? 0 : Integer.parseInt(splitsTwo[1]);
                c.combinedCodeTypeDotOne = splitsTwo.length == 0 || TextUtils.isEmpty(splitsTwo[0]) ? 0 : Integer.parseInt(splitsTwo[2]);

                String[] splitsThree = prMember[2].split("[,]");
                c.combinedCodeTypeTwo = splitsThree.length == 0 || TextUtils.isEmpty(splitsThree[0]) ? "" : splitsThree[0];
                c.combinedCodeTypeLengthTwo = splitsThree.length == 0 || TextUtils.isEmpty(splitsThree[0]) ? 0 : Integer.parseInt(splitsThree[1]);
                c.combinedCodeTypeDotTwo = splitsThree.length == 0 || TextUtils.isEmpty(splitsThree[0]) ? 0 : Integer.parseInt(splitsThree[2]);

                String[] splitsFour = prMember[3].split("[,]");
                c.shelfLifeTypeThree = splitsFour.length == 0 || TextUtils.isEmpty(splitsFour[0]) ? "" : splitsFour[0];
                c.shelfLifeLengthThree = splitsFour.length == 0 || TextUtils.isEmpty(splitsFour[0]) ? 0 : Integer.parseInt(splitsFour[1]);
                c.shelfLifeDaysThree = splitsFour.length == 0 || TextUtils.isEmpty(splitsFour[0]) ? 0 : Integer.parseInt(splitsFour[2]);

                c.checkCodeFlag = "T".equals(prMember[4]);

                SYLoger.print("chenji-->" + c.barCodeType + c.barCodeLength
                        + " == " + c.combinedCodeTypeOne + c.combinedCodeTypeLengthOne + c.combinedCodeTypeDotOne
                        + " == " + c.combinedCodeTypeTwo + c.combinedCodeTypeLengthTwo + c.combinedCodeTypeDotTwo
                        + " == " + c.shelfLifeTypeThree + c.shelfLifeLengthThree + c.shelfLifeDaysThree
                        + " == " + c.checkCodeFlag
                );
            }
        } catch (Exception e) {
            showMsg(StatusDbMap.statusDbMap("PRT102", "条码打印解析规则出错!"));
            BaseAppManager.loger().error("条码打印解析规则出错" + e);
        }
    }

    public void showMsg(String msg) {
        Toast.makeText(BaseAppManager.appContext(), msg, Toast.LENGTH_SHORT).show();
    }

    @Override
    public void displayError(Bundle bundle) {
        //打印错误信息标题
        displayError(bundle.getString(AppConstants.Printer.TITLE_TIPS),
                // 打印错误信息内容
                bundle.getString(AppConstants.Printer.ERROR_TIPS));
    }

    @Override
    public void showToast(String msg) {
        if (printToastKey) {
            Toast.makeText(BaseAppManager.appContext(), msg, Toast.LENGTH_SHORT).show();
        }
        // 因为调用了上面的吐司，把不需要的其他吐司关闭。
        printToastKey = false;
    }

    public interface OnPrinterTool {
        void onBegin();

        void onFinish();
    }
}