package com.urovo.activity.attendancemanagement.attendancemanager.act

import android.os.Bundle
import android.widget.ImageButton
import android.widget.TextView
import androidx.recyclerview.widget.LinearLayoutManager
import com.github.airsaid.calendarview.widget.CalendarView
import com.urovo.activity.attendancemanagement.attendancemanager.ada.AttenHisAdapter
import com.urovo.activity.attendancemanagement.attendancemanager.ent.AttenHistoryRes
import com.urovo.activity.attendancemanagement.attendancemanager.pre.AttenHistoryPre
import com.urovo.wugumofang.R
import com.urovo.wugumofang.base.BaseActivity
import com.urovo.wugumofang.data.DateUtil
import com.urovo.wugumofang.net.CommonController
import com.urovo.wugumofang.response.CommonEntity
import com.urovo.wugumofang.system.SystemHelper
import com.urovo.wugumofang.ui.ViewUtil
import kotlinx.android.synthetic.main.attendance_his_act.*
import kotlinx.android.synthetic.main.member_title.*
import org.jetbrains.anko.find
import java.text.SimpleDateFormat
import java.util.*

/**
 * @author Sasuke on 2018/1/15.
 * 考勤记录（就是那个日历的）
 */
class AttenHistoryAct : BaseActivity(), CommonController.AttenHistoryView {
    lateinit var mPre: AttenHistoryPre
    val dateFormat = "yyyy-MM-dd"
    val tvFormat = "yyyy年MM月"
    var monthOffset = 0//月份偏移
    lateinit var rvAdapter: AttenHisAdapter
//    var queryDate = Date()
    var dataList: MutableList<AttenHistoryRes.ReturnBean> = mutableListOf()

    lateinit var dateSwitchTv: TextView
    lateinit var dateLeftBtn: ImageButton
    lateinit var dateRightBtn: ImageButton
    lateinit var calView: CalendarView

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.attendance_his_act)

        text_title.text = "考勤记录"
        goback.setOnClickListener { finish() }

        mPre = AttenHistoryPre(this,dataList).apply { start() }
        mPre.queryWorkAttendance(transDate(DateUtil.getFirstDayOfMonth(monthOffset), dateFormat),
                transDate(DateUtil.getLastDayOfMonth(monthOffset), dateFormat)
                , SystemHelper.LoginUserId)
    }

    override fun init() {
        //列表+适配器初始化
        val header = layoutInflater.inflate(R.layout.attendance_list_head_view,null,false)
        rvAdapter = AttenHisAdapter(this,dataList).apply { addHeaderView(header) }
        attenHisRv.apply {
            layoutManager = LinearLayoutManager(this@AttenHistoryAct)
            adapter = rvAdapter }
        //日期标题
        dateSwitchTv = header.find<TextView>(R.id.dateSwitchTv).apply { text = transDate(Date(), tvFormat)}
        //日历初始化
        calView = header.find<CalendarView>(R.id.calView).apply {
            dateFormatPattern = dateFormat
            isClickable = false
            calendar = Calendar.getInstance()
        }

        //日期左按钮
        dateLeftBtn = header.find<ImageButton>(R.id.dateLeftBtn).apply {
            isEnabled = true
            setOnClickListener {
                val offset = --monthOffset
                refreshTvAndCalShow(offset)
                mPre.queryWorkAttendance(transDate(DateUtil.getFirstDayOfMonth(offset), dateFormat),
                        transDate(DateUtil.getLastDayOfMonth(offset), dateFormat)
                        , SystemHelper.LoginUserId)
            }
        }
        //日期右按钮
        //因为只能查当前月和上一个月，所以左按钮true，右按钮false
        dateRightBtn = header.find<ImageButton>(R.id.dateRightBtn).apply {
            isEnabled = false
            setOnClickListener {
                val offset = ++monthOffset
                refreshTvAndCalShow(offset)
                mPre.queryWorkAttendance(transDate(DateUtil.getFirstDayOfMonth(monthOffset), dateFormat),
                        transDate(DateUtil.getLastDayOfMonth(monthOffset), dateFormat)
                        , SystemHelper.LoginUserId)
            }
        }

    }


    /**
     * 刷新日期标题、按钮、日历显示（不包含数据刷新
     */
    fun refreshTvAndCalShow(offset: Int){
        dateSwitchTv.text = transDate(DateUtil.getFirstDayOfMonth(monthOffset), tvFormat)
        dateLeftBtn.isEnabled = if(offset <= -1) false else true
        dateRightBtn.isEnabled = if(offset >= 0) false else true
        calView.calendar = Calendar.getInstance().apply { add(Calendar.MONTH,offset) }//日历换一个月份
    }


    /**
     * 日期转换
     */
    fun transDate(date: Date, format: String): String {
        return SimpleDateFormat(format).format(date)
    }

    /**
     * 刷新日历显示
     */
    override fun notifyCalData(resultList: MutableList<String>?) {
        resultList?.let {calView.selectDate = resultList }//刷新日历
        rvAdapter.notifyDataSetChanged()//刷新列表
        ViewUtil.dismissProgress()//等信息处理完了再关闭progressBar
    }


    override fun onResponseSuccess(resultInfo: CommonEntity<*>?) {
    }

    override fun onFailure(failInfo: String?) {
    }
}