package com.urovo.activity.attendancemanagement.signinandout.model

import android.view.View
import androidx.databinding.ObservableArrayList
import androidx.databinding.ObservableField
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import com.urovo.wugumofang.R
import com.urovo.wugumofang.ui.PhotoPreviewDominator
import com.urovo.wugumofang.utils.MyToast
import kotlinx.coroutines.launch

/**
 * @author Sasuke on 2019-5-29 0029.
 */
class SignInMyPhotoModel(val repo: SignInMyPhotoRepo) : ViewModel() {
    /**照片url们*/
    var photoUrls: ObservableArrayList<String> = ObservableArrayList<String>().apply { addAll(listOf("", "", "")) }

    /**记录当前拍照序号*/
    var currCameraNum = 0

    var jumpCamera: MutableLiveData<Boolean> = MutableLiveData()

    /**转圈圈显示的文字*/
    var progressText: MutableLiveData<String> = MutableLiveData()

    /**Toast显示的文字*/
    var toastText: MutableLiveData<String> = MutableLiveData()

    /**控制关闭页面*/
    var isFinish: MutableLiveData<Boolean> = MutableLiveData()

    /**传过来的工号*/
    var userId: MutableLiveData<String> = MutableLiveData<String>().apply { value = "" }

    /**是否添加模式*/
    var isAddMode: ObservableField<Boolean> = ObservableField(false)


    /**拍照*/
    fun onCamera(view: View) {
        currCameraNum = when (view.id) {
            R.id.reTry0, R.id.add0 -> 0
            R.id.reTry1, R.id.add1 -> 1
            R.id.reTry2, R.id.add2 -> 2
            else -> 2
        }
//        toastText.value = currCameraNum.toString()
        //去调相机
        jumpCamera.value = true
    }

    /**联网检查这是不是合法脸*/
    fun checkFace(localPath: String) = viewModelScope.launch {
        progressText.value = "正在校验图片..."
        repo.run {
            try {
                //checkFace()可能会抛异常，那后面的photoUrls.set()就不会做了。
                photoUrls.set(
                        currCameraNum,
                        checkFace(uploadQiniu(localPath))) //如果后台校验合法，就存进列表，刷新UI被激活。
            } catch (e: Exception) {
                toastText.value = e.message
            }
        }
        progressText.value = ""
    }

    /**
     * 看看有没有传过脸部照片
     */
    fun queryFace() = viewModelScope.launch {
        userId.value?.let {
            try {
                //这里不能直接clear()再addAll()，因为观察者模式的缘故，clear()的时候，直接就清掉了，更新UI会报List越界
                val faces = repo.queryFace(it)
                for (i in photoUrls.indices) {
                    photoUrls[i] = faces[i]
                }
            } catch (e: java.lang.Exception) {
                isAddMode.set(true)
                MyToast.show(e.message)
            }
        }
    }

    /**点击确认按钮*/
    fun onSubmit(view: View) {
        viewModelScope.launch {
            if (photoUrls.isNullOrEmpty() || photoUrls.size < 3 || photoUrls.count { it.isNullOrBlank() } > 0) {
                toastText.value = "需添加3张照片"
                return@launch
            }
            var urls = ""
            photoUrls.forEach { urls += "${it}," }
            urls = urls.substring(0, urls.length - 1)//去掉最后一位的逗号。
            progressText.value = "正在上传"
            try {
                if (repo.submitPhotoUrls(userId.value!!, urls)) {
                    toastText.value = "上传成功"
                    isFinish.value = true
                } else {
                    toastText.value = "上传失败"
                }
            } catch (e: Exception) {
                toastText.value = e.message
            }
            progressText.value = ""
        }
    }

    /**点击图片放大预览
     *
     *  本来是想直接用id来区分的，结果怎么莫名其妙get到的id跟R.id.*不一样。。。算了，每个点击写个方法吧。。唉。
     * */
    fun onPhotoClick0(view: View) {
        PhotoPreviewDominator(view, photoUrls[0], null).showPreview()
    }

    fun onPhotoClick1(view: View) {
        PhotoPreviewDominator(view, photoUrls[1], null).showPreview()
    }

    fun onPhotoClick2(view: View) {
        PhotoPreviewDominator(view, photoUrls[2], null).showPreview()
    }
}