package com.urovo.activity.attendancemanagement.signinandout.model

import com.urovo.activity.attendancemanagement.signinandout.net.CheckFaceRes
import com.urovo.activity.attendancemanagement.signinandout.net.QueryFaceRes
import com.urovo.wgmf.tool.netUtil.QiNiuDominator
import com.urovo.wugumofang.constants.ApiUrls
import com.urovo.wugumofang.constants.NetParam
import com.urovo.wugumofang.net.OkHttpDominator
import com.urovo.wugumofang.response.Base3Response
import okhttp3.Request
import kotlin.coroutines.resume
import kotlin.coroutines.resumeWithException
import kotlin.coroutines.suspendCoroutine

/**
 * @author Sasuke on 2019-5-29 0029.
 */
class SignInMyPhotoRepo {


    /**上传图片（这里需要一张一张地检查）*/
    suspend fun uploadQiniu(localPath: String) = suspendCoroutine<String> { conti ->
        if (localPath.isBlank()) {
            conti.resume("")
            return@suspendCoroutine
        }
        QiNiuDominator.uploadImages(listOf(localPath),
                { qiniuUrls ->
                    conti.resume(qiniuUrls[0])
                },
                { str ->
                    //FailedCallback
                    conti.resumeWithException(Exception(str))
                }, false)
    }

    /**查询已上传过的脸*/
    suspend fun queryFace(userNo: String) = suspendCoroutine<List<String>> { conti ->
        OkHttpDominator.postAsyn(NetParam.YINXIAOBU_SERVER + ApiUrls.IDRECOGNIZE.QUERY_FACE, mapOf("userNo" to userNo),
                object: OkHttpDominator.ResultCallback<QueryFaceRes>(){
                    override fun onError(request: Request?, e: java.lang.Exception) {
                        conti.resumeWithException(Exception("网络异常"))
                    }

                    override fun onResponse(res: QueryFaceRes?) {
                        res?.let {
                            if (!it.resultInfo.isNullOrEmpty()){
                                conti?.resume(res.resultInfo.map { it.face_url })
                            }
                        }
                    }
                })
    }


    /**查脸*/
    suspend fun checkFace(url: String) = suspendCoroutine<String> { conti ->
        OkHttpDominator.postAsyn(NetParam.YINXIAOBU_SERVER + ApiUrls.IDRECOGNIZE.CHECK_FACE,
                mapOf("url" to url),
                object : OkHttpDominator.ResultCallback<CheckFaceRes>() {
                    override fun onError(request: Request?, e: java.lang.Exception) {
                        conti.resumeWithException(e)
                    }

                    override fun onResponse(res: CheckFaceRes?) {
                        res?.let {
                            if (res.resultInfo == null || !res.resultInfo ){
                                conti.resumeWithException(Exception(res.msgInfo))
                                return
                            }else{
                                conti.resume(url)
                                return
                            }
                        }
                        conti.resumeWithException(Exception("Response is null"))
                    }
                })
    }

    /**提交照片url*/
    suspend fun submitPhotoUrls(userNo: String, urls: String) = suspendCoroutine<Boolean> { conti ->
        OkHttpDominator.postAsyn(NetParam.YINXIAOBU_SERVER + ApiUrls.IDRECOGNIZE.UPLOAD_FACE,
                mapOf("userNo" to userNo,"url" to urls),
                object : OkHttpDominator.ResultCallback<Base3Response>(){
                    override fun onError(request: Request?, e: java.lang.Exception) {
                        conti.resumeWithException(e)
                    }

                    override fun onResponse(res: Base3Response?) {
                        res?.run {
                            if (msgCode == 0){
                                conti.resume(true)
                            }else{
                                conti.resumeWithException(Exception(msgInfo))
                            }
                        }
                    }
                })
    }
}