package com.urovo.activity.attendancemanagement.signinandout.view

import android.content.Intent
import android.os.Bundle
import androidx.databinding.DataBindingUtil
import androidx.lifecycle.Observer
import androidx.lifecycle.ViewModelProviders
import com.urovo.activity.attendancemanagement.signinandout.model.SignInMyPhotoFactory
import com.urovo.activity.attendancemanagement.signinandout.model.SignInMyPhotoModel
import com.urovo.activity.attendancemanagement.signinandout.model.SignInMyPhotoRepo
import com.urovo.wugumofang.R
import com.urovo.wugumofang.base.BaseActivity
import com.urovo.wugumofang.databinding.SignInMyPhotoActBinding
import com.urovo.wugumofang.ui.PhotoPickDominator
import com.urovo.wugumofang.ui.ViewUtil

/**
 * @author Sasuke on 2019-5-29 0029.
 * 签到里的 我的照片
 *
 */
class SignInMyPhotoAct : BaseActivity() {
    companion object {
        //是否是添加照片模式，默认为false
        const val INTENT_IS_ADD = "INTENT_IS_ADD"
        //员工工号
        const val INTENT_USER_ID = "INTENT_USER_ID"

    }

    val model by lazy {
        ViewModelProviders
                .of(this, SignInMyPhotoFactory(SignInMyPhotoRepo()))
                .get(SignInMyPhotoModel::class.java)
    }

    val binding by lazy {
        DataBindingUtil.setContentView<SignInMyPhotoActBinding>(this, R.layout.sign_in_my_photo_act)
    }

    /**是否是添加模式*/
    var picker: PhotoPickDominator? = null

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        bind()
        initTitle()
        setObserver()
        init()
    }

    fun bind() {
        binding.titleText = "我的照片"
        binding.model = model
        model.userId.value = intent.getStringExtra(INTENT_USER_ID)
        model.isAddMode.set( intent.getBooleanExtra(INTENT_IS_ADD, false))
//        binding.isAddMode = model.isAddMode
    }

    /**设置偷窥者*/
    fun setObserver() {
        /**照片列表数据*/
//        model.photoUrls

        /**跳转拍照*/
        model.jumpCamera.observe(this, Observer {
            if (it) picker?.selectByCamera(this)
        })

        /**转圈圈的显隐与文字*/
        model.progressText.observe(this, Observer {
            if (it.isNullOrBlank()) {
                ViewUtil.dismissProgress()
            } else {
                ViewUtil.showProgress(this, "请稍等", it)
            }
        })

        model.toastText.observe(this, Observer { showMsg(it) })
        model.isFinish.observe(this, Observer { if (it) finish() })
    }

    fun init() {
        picker = PhotoPickDominator(this)
        //如果不是添加模式，那一进来是要请求的
        if (!model.isAddMode.get()!! ){
            model.queryFace()
        }
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        val picLocal = picker?.onPhotoPathResult(this, requestCode, resultCode, data)
        picLocal.takeIf { !it.isNullOrBlank() }?.run { model.checkFace(this) }
    }
}