package com.urovo.activity.betasuggestion.view

import android.content.Intent
import android.os.Bundle
import android.view.Gravity
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.ImageView
import android.widget.LinearLayout
import com.google.gson.Gson
import com.squareup.picasso.Picasso
import com.urovo.activity.betasuggestion.pre.BetaSuggesPre
import com.urovo.activity.betasuggestion.pre.BetaSuggestionCon
import com.urovo.wgmf.tool.UpdateManager
import com.urovo.wugumofang.R
import com.urovo.wugumofang.base.BaseActivity
import com.urovo.wugumofang.system.SystemHelper
import com.urovo.wugumofang.ui.PhotoPickDominator
import com.urovo.wugumofang.ui.ViewUtil
import com.urovo.wugumofang.view.CustomPopupWindow
import kotlinx.android.synthetic.main.beta_suggestion_act.*
import kotlinx.android.synthetic.main.beta_suggestion_input_ll.*
import org.jetbrains.anko.forEachChild
import java.io.File

/**
 * Created by Sasuke on 2017/7/23.
 * beta版本意见反馈
 */
class BetaSuggestionAct : BaseActivity(), BetaSuggestionCon.view {
    val MAX_PHOTO_NUM: Int = 3;//最多传3张图片
    lateinit var pre: BetaSuggesPre
    val viewArray: Array<Int> = arrayOf(
            R.id.noProblem,
            R.id.someProblem,
            R.id.badProblem)
    lateinit var feedBack: LinearLayout
    lateinit var picker: PhotoPickDominator
    //点击图片预览的popupwindow
    lateinit var photoPopup: CustomPopupWindow

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.beta_suggestion_act)
        initTitle();
        titleTextView.text = "版本意见反馈"
        pre = BetaSuggesPre(this)
        pre.start()
    }

    /**
     * 初始化
     */
    override fun init() {
        picker = PhotoPickDominator(this)
        //图片预览popupWindow
        photoPopup = CustomPopupWindow.Builder(this)
                .setRoot(root)
                .setwidth(ViewGroup.LayoutParams.MATCH_PARENT)
                .setheight(ViewGroup.LayoutParams.MATCH_PARENT)
                .setContentView(R.layout.photo_preview_pop_ll) //设置布局文件
                .builder()
        photoPopup.getItemView(R.id.popupRoot).setOnClickListener { photoPopup.dismiss() }
        //输入框组件
        feedBack = LayoutInflater.from(this).inflate(R.layout.beta_suggestion_input_ll, root, false) as LinearLayout
        //其它单选组件
        radioGroup.addView(feedBack, 1)//默认是第一个为checked，第一个位置为0，那它就插到1那里去
        radioGroup.setOnCheckedChangeListener { group, checkedId ->
            group.removeView(feedBack)
            group.addView(feedBack, viewArray.indexOf(checkedId) + 1)

            dontWantUse.visibility = if (checkedId == R.id.badProblem) View.VISIBLE else View.GONE
            reasonInput.hint = if (checkedId == R.id.noProblem) "有好的建议也可以反馈给我们哦！" else "请填写问题描述，也可以拍照或截图，一并上传"
            photoGroup.visibility =  if (checkedId == R.id.noProblem) View.GONE else View.VISIBLE //没有问题的时候，不需要显示添加图片按钮
        }
        //提交按钮
        submitBtn.setOnClickListener { if (checkInput(radioGroup.checkedRadioButtonId)) pre.uploadPhotos() }
        //添加照片按钮
        cameraAdd.setOnClickListener {
            when (photoGroup.childCount) {
                MAX_PHOTO_NUM + 1 -> showMsg("最多添加 $MAX_PHOTO_NUM 张图片")//加1是里面还有一个相机添加按钮
                else -> picker.showDialog()
            }
        }

    }

    /**
     * 验证输入合法性
     */
    fun checkInput(checkedId: Int): Boolean {
        return when (checkedId) {
            R.id.noProblem -> true
            R.id.someProblem, R.id.badProblem -> if (reasonInput.text.toString().isEmpty()) {
                                                                            showMsg("请输入问题描述")
                                                                            false
                                                                        } else true
            else -> false
        }
    }

    override fun setPresenter(presenter: BetaSuggestionCon.pre?) {
        this.pre = presenter as BetaSuggesPre
    }

    /**
     * 返回图片地址后的处理&&挂监听
     */
    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        val photoPath = picker?.let { it.onPhotoPathResult(this, requestCode, resultCode, data) };
        if(photoPath == "") return;//如果没有选择图片，返回回来的路径是“”
        //用Path把图片显示出来
        var iv = ImageView(this)
        photoPath?.let { s -> iv.tag = s }//图片path路径存给自己
        iv.setPadding(5, 0, 5, 0)//随便间隔一下，差不多一个意思
        Picasso.with(this).load(File(photoPath)).resize(cameraAdd.measuredWidth, cameraAdd.measuredHeight).into(iv)
        //图片点击监听
        val scrWidth = getResources().getDisplayMetrics().widthPixels
        iv.setOnClickListener { v ->
            Picasso.with(this).load(File(v.getTag() as String))
                    .resize(scrWidth, (v.height / v.width) * scrWidth)
                    .placeholder(R.drawable.progress_animation_small)
                    .into(photoPopup.getItemView(R.id.photoPreviewIv) as ImageView)
            photoPopup.showAtLocation(v, Gravity.LEFT, 0, 0)
            photoPopup.tag = photoGroup.indexOfChild(v)//把图片序号存给popupWindow
        }
        //删除图片监听
        photoPopup.getItemView(R.id.popupFab).setOnClickListener {
            photoGroup.removeViewAt(photoPopup.tag as Int)
            photoPopup.dismiss()
        }
        photoGroup.addView(iv, 0)
    }

    /**
     * 把图片路径集中返回
     */
    override fun packPhotoPaths(): List<String> {
        var paths: MutableList<String> = mutableListOf()
        photoGroup.forEachChild { view -> view.getTag()?.let { paths.add(view.getTag() as String) } }
        return paths;
    }

    /**
     * 打包反馈文字
     */
    override fun packSuggestions(urls: List<String>?): HashMap<String,String> {
        val param = hashMapOf<String,String>()
        val bugLevel = when (radioGroup.getCheckedRadioButtonId()) {
            R.id.noProblem -> "0"
            R.id.someProblem -> "1"
            R.id.badProblem -> "2"
            else -> "-1"
        }
        param.put("bugLevel",bugLevel)
        param.put("switching",if (bugLevel == "2" && dontWantUse.isChecked) "-1" else "0")
        param.put("userNo", SystemHelper.LoginUserId)
        param.put("feedback",reasonInput.text.toString())
        param.put("photoUrl",Gson().toJson(urls))
        param.put("appOs","android")
//        val data: BetaSuggesData = BetaSuggesData()
//        data.bugLevel = when (radioGroup.getCheckedRadioButtonId()) {
//            R.id.noProblem -> "0"
//            R.id.someProblem -> "1"
//            R.id.badProblem -> "2"
//            else -> "-1"
//        }
//        data.switching = if (data.bugLevel == "2" && dontWantUse.isChecked) "-1" else "0"
//        data.userNo = SystemHelper.LoginUserId
//        data.feedback = reasonInput.text.toString()
//        data.photoUrl = Gson().toJson(urls)
        return param
    }

    override fun onDestroy() {
        super.onDestroy()
        picker?.deletePicCache()
    }

    override fun dismissProgressBar() {
        ViewUtil.dismissProgress()
    }

    override fun showProgressBar(msg: String) {
        ViewUtil.showProgress(this, "请稍等", msg, false);
    }

    override fun downloadApp(appUrl: String) {
        val updateManager = UpdateManager("", appUrl)
        updateManager.showDownloadDialog(this)
    }
}

