package com.urovo.activity.careertraining.presenter

import com.urovo.activity.careertraining.net.TrainingPlanRes
import com.urovo.wugumofang.constants.ApiUrls
import com.urovo.wugumofang.constants.NetParam
import com.urovo.wugumofang.net.OkHttpDominator
import com.urovo.wugumofang.ui.TabRvSyncDominator
import com.urovo.wugumofang.view.LoadWaitDominator
import okhttp3.Request

class CareerTrainingPre(val mView: CareerTrainingContract.view) : CareerTrainingContract.pre {

    init {
        mView.setPresenter(this)
    }

    var trainingListCache: MutableList<TrainingPlanRes.ResultInfo.WeekPlans> = mutableListOf()

    override fun start() {
        mView.initUI()
    }

    /***
     * 请求入职培训数据
     */
    fun queryTrainingPlan(trainingStaffId: String) {
        mView.showLoadWait(LoadWaitDominator.LOADING)
        OkHttpDominator.getAsyn(NetParam.YINXIAOBU_SERVER + ApiUrls.YXGJ.GET_QUERY_WORKER_TRAIN_INFO,
                mapOf("staffId" to trainingStaffId),
                object : OkHttpDominator.ResultCallback<TrainingPlanRes>() {
                    override fun onError(request: Request?, e: Exception?) {
                        e?.printStackTrace()
                        mView.showLoadWait(LoadWaitDominator.NET_ERROR)
                    }

                    override fun onResponse(res: TrainingPlanRes?) {
                        mView.dismissLoadWait()
                        res?.let {
                            //异常
                            if (res.msgCode != 0) {
                                mView.showLoadWaitMsg(res.msgInfo)
                                return
                            }
                            if (null != res.result) {
                                if (null != res.result.workerStaffId && null != res.result.workerEntryTime) {
                                    mView.refreshTitleUI(res.result.workerStaffId, res.result.workerEntryTime)
                                }
                                //列表空
                                if (null != res.result.userWeekList && res.result.userWeekList.isEmpty()) {
                                    mView.showLoadWait(LoadWaitDominator.NO_DATA)
                                    return
                                }
                                //先把数据处理成线性单条列表
                                res.result.userWeekList.forEachIndexed { index, weekPlan ->
                                    if (index > 0) { //非第一周数据，取个人绩效和学习内容
                                        var contentTemp = ""
                                        var dateTemp = ""
                                        weekPlan.weekItemList.takeIf { it.isNotEmpty() }?.let {
                                            it.forEachIndexed { index, plan ->
                                                contentTemp = plan.trainItemContent
                                                dateTemp = plan.trainItemName
                                            }
                                        }
                                        //自建字段的赋值
                                        weekPlan.trainItemName = dateTemp + "："
                                        weekPlan.trainItemContent = contentTemp
                                        weekPlan.contentTitle = if (null != weekPlan && null != weekPlan.workerTrainContentM && null != weekPlan.workerTrainContentM.contentTitle) weekPlan.workerTrainContentM.contentTitle + "：" else ""
                                        weekPlan.content = if (null != weekPlan && null != weekPlan.workerTrainContentM && null != weekPlan.workerTrainContentM.content) weekPlan.workerTrainContentM.content + "：" else ""
                                    } else {
                                        //第一周数据，取日期和学习内容
                                        var contentTemp = ""
                                        weekPlan.weekItemList.takeIf { it.isNotEmpty() }?.let {
                                            it.forEachIndexed { index, plan ->
                                                //把多个内容给整合到一个字符串中，精简到一个TextView里。
                                                contentTemp += plan.trainItemTime + "    " + plan.trainItemName + "\n\n" + "学习内容：" + plan.trainItemContent + "\n\n"
                                            }
                                        }
                                        //自建字段的赋值
                                        weekPlan.contentTitle = contentTemp
                                    }
                                }
                                //更新数据 刷新列表
                                trainingListCache.run {
                                    clear()
                                    addAll(res.result.userWeekList)
                                    mView.refreshRv()
                                }
                                //刷新TabLayout
                                val titleList = res.result.userWeekList.map { TabRvSyncDominator.TabInfo(it.trainWeekName, it.trainWeekName) }
                                val typeList = trainingListCache.map { it.trainWeekName }
                                mView.refreshTabLayout(
                                        titleList,
                                        typeList)
                                mView.dismissLoadWait()
                            }
                        }
                    }
                })
    }
}