package com.urovo.activity.careertraining.view;

import android.os.Bundle;
import android.text.TextUtils;
import android.widget.TextView;

import com.google.gson.Gson;
import com.urovo.activity.careertraining.model.WorkerTrainInfo;
import com.urovo.activity.careertraining.model.WorkerTrainInfoList;
import com.urovo.activity.careertraining.model.WorkerTrainInfoResultData;
import com.urovo.activity.careertraining.ui.CareerEvaluateAdapter;
import com.urovo.wugumofang.constants.ConstantParams;
import com.urovo.wugumofang.R;
import com.urovo.wugumofang.base.BaseActivity;
import com.urovo.wugumofang.constants.ApiUrls;
import com.urovo.wugumofang.constants.NetParam;
import com.urovo.wugumofang.net.OkHttpDominator;
import com.urovo.wugumofang.system.SystemHelper;
import com.urovo.wugumofang.ui.ViewUtil;
import com.wgmf.trade.widgets.ListViewForScrollView;

import java.util.HashMap;
import java.util.List;

import okhttp3.Request;

/**
 * 培训评价查看页面
 */
public class CareerEvaluateQueryAct extends BaseActivity {
    /**
     * 导师评价内容
     */
    private TextView integrativeEvaluation;
    /**
     * 主管评价内容
     */
    private TextView supervisorEvaluation;
    /**
     * 经理评价内容
     */
    private TextView officeEvaluation;
    private ListViewForScrollView careerLv;
    private List<WorkerTrainInfo> workerTrainInfoList;
    private CareerEvaluateAdapter adapter;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.career_evaluate_query_activity);
        initTitle();
        initView();
        getData();
    }

    @Override
    public void initTitle() {
        super.initTitle();
        titleTextView.setText("评价内容");
    }

    private void initView() {
        careerLv = (ListViewForScrollView) findViewById(R.id.career_listview);
        integrativeEvaluation = (TextView) findViewById(R.id.integrativeEvaluation);
        supervisorEvaluation = (TextView) findViewById(R.id.supervisorEvaluation);
        officeEvaluation = (TextView) findViewById(R.id.officeEvaluation);
        careerLv = (ListViewForScrollView) findViewById(R.id.career_listview);
        if (workerTrainInfoList != null) {
            adapter = new CareerEvaluateAdapter(workerTrainInfoList, CareerEvaluateQueryAct.this);
            careerLv.setAdapter(adapter);
        }
    }

    private void getData() {
        if (!TextUtils.isEmpty(SystemHelper.LoginUserId)) {
            HashMap<String, String> param = new HashMap<>();
            param.put("trainStaffId", SystemHelper.LoginUserId);
            ViewUtil.showProgress(CareerEvaluateQueryAct.this, "", "请稍等.....");
            OkHttpDominator.getAsyn(NetParam.YINXIAOBU_SERVER + ApiUrls.YXGJ.GET_QUERY_TRAIN_RESULT, param,
                    new OkHttpDominator.ResultCallback<String>() {
                        @Override
                        public void onError(Request request, Exception e) {
                            ViewUtil.dismissProgress();
                            showMsg("获取" + getString(R.string.request_data_fail));
                        }

                        @Override
                        public void onResponse(String json) {
                            ViewUtil.dismissProgress();
                            if (json == null || TextUtils.isEmpty(json)) {
                                showMsg("没有返回数据");
                                return;
                            }
                            WorkerTrainInfoResultData mWorkerTrainInfoResultData = new Gson().fromJson(json, WorkerTrainInfoResultData.class);
                            if (null != mWorkerTrainInfoResultData && ConstantParams.CONSTANT_PARAMS_0.equals(mWorkerTrainInfoResultData.sysCode)) {
                                setContentData(mWorkerTrainInfoResultData);
                            } else {
                                showMsg("没有返回数据");
                            }
                        }
                    });
        }
    }

    private void setContentData(WorkerTrainInfoResultData mWorkerTrainInfoResultData) {
        WorkerTrainInfoList mWorkerTrainInfoLists = mWorkerTrainInfoResultData.getResult();
        if (null != mWorkerTrainInfoLists) {
            integrativeEvaluation.setText("综合评价:" + mWorkerTrainInfoLists.getIntegrativeEvaluation());
            supervisorEvaluation.setText(mWorkerTrainInfoLists.getSupervisorEvaluation());
            officeEvaluation.setText(mWorkerTrainInfoLists.getOfficeEvaluation());
            workerTrainInfoList = mWorkerTrainInfoLists.getWorkerTrainInfoList();
            if (null != mWorkerTrainInfoLists.getWorkerTrainInfoList() && !mWorkerTrainInfoLists.getWorkerTrainInfoList().isEmpty()) {
                adapter = new CareerEvaluateAdapter(workerTrainInfoList, CareerEvaluateQueryAct.this);
                careerLv.setAdapter(adapter);
            }
        }
    }
}
