package com.urovo.activity.careertraining.view

import android.content.Context
import android.content.Intent
import android.os.Bundle
import android.view.View
import com.urovo.activity.careertraining.presenter.CareerTrainingContract
import com.urovo.activity.careertraining.presenter.CareerTrainingPre
import com.urovo.activity.careertraining.ui.CareerTrainingAdapter
import com.urovo.wugumofang.R
import com.urovo.wugumofang.base.BaseActivity
import com.urovo.wugumofang.system.SystemHelper
import com.urovo.wugumofang.ui.TabRvSyncDominator
import com.urovo.wugumofang.view.LoadWaitDominator
import kotlinx.android.synthetic.main.career_training_act.*

/**
 * 入职培训
 *
 * 这个页面根据是否传入mentorId来判断是培训还是评估
 */
class CareerTrainingAct : BaseActivity(), CareerTrainingContract.view {
    companion object {

        const val TRAINING_STAFF_ID = "TRAINING_STAFF_ID"//入职评估分支下，传入要评估的员工工号
        const val TRAINING_STAFF_NAME = "TRAINING_STAFF_NAME"//入职评估分支下，传入要评估的员工工号
        const val HAS_EVALUATED = "HAS_EVALUATED"//是否是已评估（从已评估跳过来的，则都不可编辑；否则都可以编辑）


        @JvmStatic
        fun launch(context: Context, trainingStaffId: String?, trainingStaffName: String, hasEvaluated: String) {
            context.startActivity(Intent(context, CareerTrainingAct::class.java).apply {
                putExtra(CareerTrainingAct.TRAINING_STAFF_ID, trainingStaffId)
                putExtra(CareerTrainingAct.TRAINING_STAFF_NAME, trainingStaffName)
                putExtra(CareerTrainingAct.HAS_EVALUATED, hasEvaluated)
            })
        }
    }

    var mPre: CareerTrainingPre? = null
    var loadWait: LoadWaitDominator? = null
    var tabRvDominator: TabRvSyncDominator? = null

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.career_training_act)
        initTitle()
        mPre = CareerTrainingPre(this)
        mPre?.start()
        mPre?.queryTrainingPlan(SystemHelper.LoginUserId)
    }

    override fun initUI() {
        titleTextView.text = "入职培训"
        complete.text = "查看评价"
        complete.visibility = View.VISIBLE
        complete.setOnClickListener {
            this.startActivity(Intent(this, CareerEvaluateQueryAct::class.java))
        }
        tabRvDominator = TabRvSyncDominator(this, tabLayout, dataRv)
        loadWait = LoadWaitDominator(this, careerContainer)
        staffNameTv.text = SystemHelper.LoginUserName
        dataRv.adapter = CareerTrainingAdapter(this, mPre!!.trainingListCache) {
        }
    }

    override fun refreshTitleUI(staffName: String, workDate: String) {
        staffNameTv.text = SystemHelper.LoginUserName
        workDateTv.text = "入职时间：${workDate}"
    }

    override fun refreshRv() {
        dataRv.adapter?.notifyDataSetChanged()
    }

    override fun refreshTabLayout(titleList: List<TabRvSyncDominator.TabInfo>, dataTypeList: List<String>) {
        tabRvDominator?.setTabInfoNotify(titleList, dataTypeList)
    }

    override fun showLoadWait(state: Int) {
        loadWait?.show(state)
    }

    override fun dismissLoadWait() {
        loadWait?.dismissAll()
    }

    override fun showLoadWaitMsg(msg: String) {
        loadWait?.show(msg)
    }

    override fun setPresenter(presenter: CareerTrainingContract.pre?) {
        mPre = presenter as CareerTrainingPre
    }
}