package com.urovo.activity.checkGoodsTabAct.presenter

import com.google.gson.reflect.TypeToken
import com.urovo.activity.checkGoodsTabAct.model.QueryCheckEntity
import com.urovo.wugumofang.base.mvp.AbsPresenter
import com.urovo.wugumofang.constants.ApiUrls
import com.urovo.wugumofang.constants.AppConstants
import com.urovo.wugumofang.constants.NetParam
import com.urovo.wugumofang.data.JsonDominator
import com.urovo.wugumofang.net.OkHttpDominator
import com.urovo.wugumofang.system.SystemHelper
import com.urovo.wugumofang.utils.isNotNullOrSize
import com.zsy.libcore.model.bean.RequestResult
import okhttp3.Request
import java.text.MessageFormat
import java.util.*

/**
 * @author LW on 2022/8/18.
 */
class CheckGoodsQueryPresenter(private var view: CheckGoodsQueryContract.IView?) : AbsPresenter(),
    CheckGoodsQueryContract.IPresenter {

    var hasQuery = false

    override fun requestTransferCheck(year: Int, month: Int) {
        view?.showTipDialog("正在查询盘点数据...")
        OkHttpDominator.postAsyn(NetParam.YINXIAOBU_SERVER + ApiUrls.APP.QUERY_TRANSFER_CHECK,
            mapOf(
                "storeNo" to SystemHelper.LoginOrganId,
                "month" to year.toString() + String.format(Locale.CHINA, "%02d", month)
            ),
            object : OkHttpDominator.ResultCallback<RequestResult<List<QueryCheckEntity>>>() {
                override fun onError(request: Request, exception: Exception) {
                    view?.showDataType(2)
                    hasQuery = true
                    view?.showTitleText(
                        MessageFormat.format(
                            "{0},{1}年{2}月",
                            SystemHelper.LoginOrganName,
                            year,
                            month
                        )
                    )
                    view?.dismissDialog()
                    view?.showToast(exception.message)
                }

                override fun onResponse(response: RequestResult<List<QueryCheckEntity>>) {
                    val dataMock: List<QueryCheckEntity>? = JsonDominator.loadFromSD(
                        "QueryCheckList",
                        object : TypeToken<List<QueryCheckEntity?>?>() {}.type
                    )
                    if (dataMock.isNotNullOrSize()) {
                        view?.refreshTransferCheckToView(dataMock!!)
                    } else {
                        response.let {
                            if (it.msgCode == AppConstants.Http.RESPONSE_SUCCESS) {
                                view?.refreshTransferCheckToView(it.resultInfo)
                            } else {
                                view?.showToast(it.msgInfo)
                            }
                        }
                    }
                    view?.dismissDialog()
                }
            })
    }
}