package com.urovo.activity.checkGoodsTabAct.view

import android.content.Intent
import android.graphics.Color
import android.view.ViewGroup
import androidx.recyclerview.widget.LinearLayoutManager
import com.urovo.activity.checkGoodsTabAct.presenter.CheckGoodsSubmitListContract
import com.urovo.activity.checkGoodsTabAct.presenter.CheckGoodsSubmitListPresenter
import com.urovo.activity.checkGoodsTabAct.view.adapter.CheckSubmitListAdapter
import com.urovo.wgmf.db.dao.entitys.CanCheckRes3
import com.urovo.wugumofang.R
import com.urovo.wugumofang.base.BaseActivity
import com.urovo.wugumofang.base.BasePresenter
import com.urovo.wugumofang.data.DateUtil
import com.urovo.wugumofang.system.SystemHelper
import com.urovo.wugumofang.ui.DialogHelper
import com.urovo.wugumofang.utils.notNullAndEmpty
import com.urovo.wugumofang.view.DividerItemDecoration
import kotlinx.android.synthetic.main.check_goods_submit_list_act.*
import java.text.MessageFormat
import java.util.*

/**
 * 盘点提交
 * @author LW on 2022-08-18
 */
class CheckGoodsSubmitListActivity : BaseActivity(), CheckGoodsSubmitListContract.IView {

    override fun addRootLayoutId(): Int = R.layout.check_goods_submit_list_act

    override fun rootViewGroup(): ViewGroup = fl_container_view

    private var mCheckGoodsSubmitListPresenter: CheckGoodsSubmitListPresenter? = null

    private var mCheckSubmitListAdapterL: CheckSubmitListAdapter? = null

    private var mIsFirst = true

    private var mTransferCheckMsg: String? = ""

    override fun initTitle() {
        super.initTitle()
        titleTextView.text = getString(R.string.check_list_title)
    }

    override fun initViewNew() {
        initTitle()

        val date = DateUtil.getSpecialRuleDate()
        tv_store_title.text = MessageFormat.format("{0} \n {1}", SystemHelper.LoginOrganName, date)
        mCheckSubmitListAdapterL =
            CheckSubmitListAdapter(mutableListOf())
        mCheckSubmitListAdapterL?.setOnItemClickListener { baseQuickAdapter, _, position ->
            val item = baseQuickAdapter.getItem(position) as CanCheckRes3.ResultInfo?
            val canGoPandian =
                item?.pdState == "草稿" || item?.pdState == "未盘点"//能否点击进入盘点
            if (canGoPandian && item != null) {
                if (mTransferCheckMsg.notNullAndEmpty()) {
                    DialogHelper.onAlertDialog(mContext, mTransferCheckMsg, null, null)
                    return@setOnItemClickListener
                }
                mContext.startActivity(
                    Intent(
                        mContext,
                        CheckGoodsSubmitActivity::class.java
                    ).apply {
                        putExtra(CheckGoodsSubmitActivity.INTENT_RESULTLIST, item)
                    })
            }
        }

        rv_check_goods_list.run {
            layoutManager = LinearLayoutManager(this@CheckGoodsSubmitListActivity)
            addItemDecoration(
                DividerItemDecoration(
                    context,
                    DividerItemDecoration.HORIZONTAL_LIST,
                    1,
                    Color.parseColor("#E3E7EE")
                )
            )
            adapter = mCheckSubmitListAdapterL
        }
    }

    override fun initDataNew() {
        mCheckGoodsSubmitListPresenter?.requestCurrentTransferCheck()
        mCheckGoodsSubmitListPresenter?.requestTransferCheck()
    }

    override fun addPresenter(): BasePresenter {
        mCheckGoodsSubmitListPresenter = CheckGoodsSubmitListPresenter(this)
        return mCheckGoodsSubmitListPresenter!!
    }

    override fun onResume() {
        super.onResume()
        if (!mIsFirst) {
            mCheckGoodsSubmitListPresenter?.requestCurrentTransferCheck()
        }
        mIsFirst = false
    }

    override fun currentTransferCheckToView(resultInfo: List<CanCheckRes3.ResultInfo>?) {
        resultInfo?.let {
            mCheckSubmitListAdapterL?.setNewData(it)
        }
    }

    override fun transferCheckStatus(status: Boolean, msg: String?) {
        if (!status) {
            this.mTransferCheckMsg = msg
        }
    }
}