package com.urovo.activity.checkGoodsTabAct.view.adapter

import android.os.Build
import android.text.Editable
import android.text.TextWatcher
import androidx.annotation.RequiresApi
import com.chad.library.adapter.base.BaseQuickAdapter
import com.chad.library.adapter.base.BaseViewHolder
import com.urovo.wgmf.db.dao.entitys.CanCheckRes3
import com.urovo.wugumofang.R
import com.urovo.wugumofang.data.StringUtils
import com.urovo.wugumofang.ui.EditTextDecimal
import com.urovo.wugumofang.utils.MyToast

/**
 * 存货盘点提交适配器
 * @author LW
 */
class CheckSubmitAdapter2(
    val entity: List<CanCheckRes3.ResultInfo.DetailList>,
    private val modifyNumClickListener: ModifyNumClickListener?
) : BaseQuickAdapter<CanCheckRes3.ResultInfo.DetailList, BaseViewHolder>(
    R.layout.second_checkgoodsmenu_new_item3,
    entity
) {

    @RequiresApi(Build.VERSION_CODES.O)
    override fun convert(helper: BaseViewHolder?, item: CanCheckRes3.ResultInfo.DetailList?) {
        helper?.apply {
            item?.let {
                val etGoodsUpdateCount = getView<EditTextDecimal>(R.id.et_check_goods_count)
                etGoodsUpdateCount.isLongClickable = false
                etGoodsUpdateCount.setDecimal(if (it.proIsDecimal == "否") 0 else 2)
                if (!matchConditionsNotEdit(it)) {
                    etGoodsUpdateCount.isCursorVisible = true
                    etGoodsUpdateCount.isFocusable = true
                    etGoodsUpdateCount.isFocusableInTouchMode = true
                    etGoodsUpdateCount.addTextChangedListener(object :
                        TextWatcher {
                        override fun beforeTextChanged(
                            s: CharSequence?,
                            start: Int,
                            count: Int,
                            after: Int
                        ) {
                        }

                        override fun onTextChanged(
                            s: CharSequence,
                            start: Int,
                            before: Int,
                            count: Int
                        ) {
                        }

                        override fun afterTextChanged(s: Editable?) {
                            var pdkc = 0.0
                            val detailList = entity[adapterPosition]
                            try {
                                if (s.toString().isNotEmpty()) {
                                    pdkc = s.toString().toDouble()
                                    if (pdkc >= 0.0) {
                                        //盘点数量应大于或等于零
                                        detailList.pdAlterNum = pdkc
                                        detailList.isPanDian = true
                                        setText(
                                            R.id.tv_check_goods_amount,
                                            StringUtils.formatDouble(
                                                (detailList.proPrice
                                                    ?: 0.0) * (detailList.pdAlterNum
                                                    ?: 0.0),
                                                "0.00"
                                            )
                                        )
                                        modifyNumClickListener?.onModifyNumClickListener(
                                            entity
                                        )
                                        return
                                    }
                                }
                            } catch (e: Exception) {
                                e.printStackTrace()
                            }

                            detailList.pdAlterNum = null
                            detailList.isPanDian = false
                            setText(
                                R.id.tv_check_goods_amount,
                                "0.00"
                            )
                            modifyNumClickListener?.onModifyNumClickListener(
                                entity
                            )
                        }
                    })
                } else {
                    etGoodsUpdateCount.isCursorVisible = false
                    etGoodsUpdateCount.isFocusable = false
                    etGoodsUpdateCount.isFocusableInTouchMode = false
                    etGoodsUpdateCount.isClickable=true
                }
                etGoodsUpdateCount.setOnClickListener {
                    if (matchConditionsNotEdit(item)) {
                        MyToast.show(mContext.getString(R.string.check_goods_special_product_code_tip))
                    }
                }

                setText(R.id.tv_goods_name, it.proName)
                setText(R.id.tv_goods_unit, String.format("%s(%s)", it.proUnit, it.proCode))
                setText(R.id.tv_goods_unit_price, String.format("单价:%s", it.proPrice))

                if (matchConditionsNotEdit(it)) {
                    it.isPanDian = true
                    it.pdAlterNum = 0.0
                    setText(
                        R.id.tv_check_goods_amount,
                        "0.00"
                    )
                    etGoodsUpdateCount.setText(
                        StringUtils.formatDouble(
                            it.pdAlterNum ?: 0.0,
                            getPdQtyFormatByDecimal(it)
                        )
                    )
                }else if (it.isPanDian) {
                    setText(
                        R.id.tv_check_goods_amount,
                        StringUtils.formatDouble(
                            (it.proPrice ?: 0.0) * (it.pdAlterNum ?: 0.0),
                            "0.00"
                        )
                    )

                    etGoodsUpdateCount.setText(
                        StringUtils.formatDouble(
                            it.pdAlterNum ?: 0.0,
                            getPdQtyFormatByDecimal(it)
                        )
                    )
                } else {
                    setText(
                        R.id.tv_check_goods_amount,
                        "0.00"
                    )
                    etGoodsUpdateCount.setText("")
                }
            }
        }
    }

    interface ModifyNumClickListener {
        fun onModifyNumClickListener(list: List<CanCheckRes3.ResultInfo.DetailList>?)
    }

    /**
     * 通过后台新增字段判断返回结果是否带2位小数
     */
    fun getPdQtyFormatByDecimal(ent: CanCheckRes3.ResultInfo.DetailList): String {
        return if (ent.proIsDecimal == "否") "0" else "0.00"
    }

    /**
     * 符合下列条件，不能编辑
     */
    fun matchConditionsNotEdit(item: CanCheckRes3.ResultInfo.DetailList?): Boolean {
        item?.let {
            if ("2.20211" == it.proCode || "2.20212" == it.proCode || "2.20214" == it.proCode ||
                "2.20215" == it.proCode || "2.20216" == it.proCode
            ) {
                return true
            }
        }
        return false
    }

}