/*
 * Copyright (C) 2007 The Android Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.urovo.activity.common;

import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.DialogInterface.OnClickListener;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.NumberPicker;

import com.urovo.wugumofang.R;

import java.util.Calendar;

public class DatePickerYearMonthDialog extends AlertDialog implements OnClickListener {
    private static final String YEAR = "year";
    private static final String MONTH = "month";
    private final OnDateSetListener mCallBack;

    private NumberPicker mNumberPickerYear;
    private NumberPicker mNumberPickerMonth;
    private String mMonths[] = {"1月", "2月", "3月", "4月", "5月", "6月", "7月", "8月", "9月", "10月", "11月", "12月"};

    // Calendar calendar = Calendar.getInstance();
    // calendar.setTimeInMillis(System.currentTimeMillis());
    public interface OnDateSetListener {

        void onDateSet(int year, int monthOfYear);
    }

    public DatePickerYearMonthDialog(Context context, OnDateSetListener onDateSetListener) {
        this(context, onDateSetListener, Calendar.getInstance().get(Calendar.YEAR), Calendar.getInstance().get(Calendar.MONTH));
    }

    public DatePickerYearMonthDialog(Context context, OnDateSetListener onDateSetListener, int year, int monthOfYear) {
        super(context);
        mCallBack = onDateSetListener;
        Context themeContext = getContext();
        setButton(BUTTON_NEGATIVE, "取消", this);
        setButton(BUTTON_POSITIVE, "确认", this);
        setTitle("设置日期");
        setIcon(0);

        LayoutInflater inflater = (LayoutInflater) themeContext.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
        View view = inflater.inflate(R.layout.date_picker_year_month, null);

        mNumberPickerYear = (NumberPicker) view.findViewById(R.id.year);
        mNumberPickerMonth = (NumberPicker) view.findViewById(R.id.month);
        mNumberPickerYear.setDisplayedValues(null);
        mNumberPickerYear.setMinValue(2001);
        mNumberPickerYear.setMaxValue(Calendar.getInstance().get(Calendar.YEAR));//最大可选年份截止到今年。
        mNumberPickerYear.setWrapSelectorWheel(true);

        mNumberPickerMonth.setDisplayedValues(mMonths);
        mNumberPickerMonth.setMinValue(0);
        mNumberPickerMonth.setMaxValue(11);
        mNumberPickerMonth.setWrapSelectorWheel(true);

        mNumberPickerYear.setValue(year);
        mNumberPickerMonth.setValue(monthOfYear);

        setView(view);
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

    }

    public void onClick(DialogInterface dialog, int which) {
        if (BUTTON_POSITIVE == which) {
            tryNotifyDateSet();
        }
    }

    private void tryNotifyDateSet() {
        if (mCallBack != null) {
            mCallBack.onDateSet(mNumberPickerYear.getValue(), mNumberPickerMonth.getValue());
        }
    }

    @Override
    protected void onStop() {
        // tryNotifyDateSet();
        super.onStop();
    }

    @Override
    public Bundle onSaveInstanceState() {
        Bundle state = super.onSaveInstanceState();
        state.putInt(YEAR, mNumberPickerYear.getValue());
        state.putInt(MONTH, mNumberPickerMonth.getValue());
        return state;
    }

    @Override
    public void onRestoreInstanceState(Bundle savedInstanceState) {
        super.onRestoreInstanceState(savedInstanceState);
        int year = savedInstanceState.getInt(YEAR);
        int month = savedInstanceState.getInt(MONTH);
        mNumberPickerYear.setValue(year);
        mNumberPickerMonth.setValue(month);
    }

}
