package com.urovo.activity.customercare.presenter

import com.urovo.activity.customercare.net.CustomerCareRes
import com.urovo.wugumofang.constants.ApiUrls
import com.urovo.wugumofang.constants.NetParam
import com.urovo.wugumofang.data.DateUtil
import com.urovo.wugumofang.net.OkHttpDominator
import com.urovo.wugumofang.system.SystemHelper
import okhttp3.Request
import java.text.SimpleDateFormat

/**
 *  @author XP on 2019/8/22.
 */
class CustomerCareListPre(val mView: CustomerCareListCon.view) : CustomerCareListCon.pre {
    val simpleFormat = SimpleDateFormat("yyyy年MM月")
    val uploadFormat = SimpleDateFormat("yyyy-MM")

    init {
        mView.setPresenter(this)
    }


    /**
     * 根据月份偏移量来得到显示的日期
     */
    fun getDateTextByOff(monthOff: Int): String {
        return simpleFormat.format(DateUtil.getFirstDayOfMonth(monthOff))
    }

    /**
     * 返回用于传给后台的日期数据
     */
    fun getDateByOff(monthOff: Int): String {
        return uploadFormat.format(DateUtil.getFirstDayOfMonth(monthOff))
    }

    override fun start() {
        mView.init()
    }

    /**
     * 查询数据 ，进行中 - 待开始 - 已结束
     * state 0待开始,1进行中,2已结束
     */
    fun queryData(date: String, state: Int) {
        mView.showLoading()
        OkHttpDominator.postAsyn(NetParam.YINXIAOBU_SERVER + ApiUrls.APP.GET_CARE_TASK_LIST,
                mapOf("shopId" to SystemHelper.LoginOrganId, "ym" to date, "state" to state.toString()),
                object : OkHttpDominator.ResultCallback<CustomerCareRes>() {
                    override fun onError(request: Request?, e: Exception?) {
                        mView.showNetError()
                        e?.printStackTrace()
                    }

                    override fun onResponse(res: CustomerCareRes) {
                        if (res.resultInfo.isEmpty() || res.msgCode != 0) {
                            mView.showNoData()
                            return
                        }
                        mView.run {
                            dismissLoadWait()
                            mView.notifyData(res)
                        }
                    }
                })
    }
}