package com.urovo.activity.customercare.view

import android.os.Bundle
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.alibaba.android.arouter.facade.annotation.Route
import com.scwang.smartrefresh.layout.listener.OnRefreshListener
import com.urovo.activity.customercare.net.CustomerCareRes
import com.urovo.activity.customercare.presenter.CustomerCareListCon
import com.urovo.activity.customercare.presenter.CustomerCareListPre
import com.urovo.activity.customercare.ui.CustomerCareListAdapter
import com.urovo.wugumofang.R
import com.urovo.wugumofang.base.BaseActivity
import com.urovo.wugumofang.constants.RouterParams
import com.urovo.wugumofang.utils.CommonTools.postDelayed
import com.urovo.wugumofang.view.LoadWaitDominator
import com.wgmf.trade.widgets.RecycleViewDivider
import kotlinx.android.synthetic.main.ll_customer_care_act.*
import kotlinx.android.synthetic.main.ll_customer_care_list_radio_layout.*

/**
 *  @author XP on 2019/8/22.
 * 关怀任务
 */
@Route(path = RouterParams.Member.CUSTOMER_CARE_LIST)
class CustomerCareListAct : BaseActivity(), CustomerCareListCon.view {

    var mPre: CustomerCareListPre? = null
    var monthOff = 0//月份偏移
    var clientType = 1//0待开始,1进行中,2已结束
    var loadWaitDominator: LoadWaitDominator? = null// 加载中、暂无数据、网络错误等等状态显示的全家桶。
    val dataList: MutableList<CustomerCareRes.ResultInfo> = mutableListOf()

    companion object {
        const val WAITING = 0//待开始
        const val ONGOING = 1//进行中
        const val ENDED = 2//已结束
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.ll_customer_care_act)
        initTitle()
        titleTextView.text = "关怀任务"
        CustomerCareListPre(this).apply {
            start()
        }
        initRefreshLayout()
    }

    /**
     * 刷新头监听器初始化
     */
    private fun initRefreshLayout() {
        refreshLayout.setOnRefreshListener(OnRefreshListener { refreshlayout ->
            refreshlayout.layout.postDelayed({
                refresh()
                refreshlayout.finishRefresh()
                refreshlayout.setNoMoreData(false)
                refreshLayout.autoRefresh()
            }, 2000)
        })
    }

    private fun refresh() {
        postDelayed(30000)
        mPre?.queryData(mPre?.getDateByOff(monthOff) ?: "", clientType)
    }

    override fun init() {
        //日期字
        yearCareTv.apply {
            setTextColor(resources.getColor(R.color.title_yellow_bg))
            text = mPre?.getDateTextByOff(0)
        }
        //切换日期按钮点击
        arrayOf(leftCareBtn, rightCareBtn).forEach { btn ->
            btn.setOnClickListener {
                monthOff += if (btn == leftCareBtn) -1 else 1
                yearCareTv.text = mPre?.getDateTextByOff(monthOff)
                mPre?.queryData(mPre?.getDateByOff(monthOff) ?: "", clientType)
            }
        }
        //标签点击监听
        allCareBtn.isChecked = true
        careRadioGroup.setOnCheckedChangeListener { group, checkedId ->
            clientType = when (checkedId) {
                allCareBtn.id -> ONGOING
                newClientBtn.id -> WAITING
                oldClientBtn.id -> ENDED
                else -> -1
            }
            mPre?.queryData(mPre?.getDateByOff(monthOff) ?: "", clientType)
        }
        loadWaitDominator = LoadWaitDominator(this, container)
        customerCareList.apply {
            layoutManager = LinearLayoutManager(this@CustomerCareListAct)
            adapter = CustomerCareListAdapter(this@CustomerCareListAct, dataList)
            addItemDecoration(RecycleViewDivider(this@CustomerCareListAct, RecyclerView.HORIZONTAL, 1, resources.getColor(R.color.divider_line)))
        }
    }

    override fun onResume() {
        super.onResume()
        //请求
        mPre?.queryData(mPre?.getDateByOff(monthOff) ?: "", clientType)
    }

    /**
     * 网络返回后显示数据
     */
    override fun notifyData(res: CustomerCareRes) {
        if (this.isFinishing) {
            return
        }
        dataList.apply {
            clear();addAll(res.resultInfo)
        }
        customerCareList.adapter?.notifyDataSetChanged()//刷新列表
    }

    override fun setPresenter(presenter: CustomerCareListCon.pre?) {
        mPre = presenter as CustomerCareListPre
    }

    /**
     * 以下用于显示加载状态
     */
    override fun showLoading() {
        loadWaitDominator?.let { it.show(LoadWaitDominator.LOADING) }
    }

    override fun showNetError() {
        loadWaitDominator?.let { it.show(LoadWaitDominator.NET_ERROR) }
    }

    override fun showNoData() {
        loadWaitDominator?.let { it.show(LoadWaitDominator.NO_DATA) }
    }

    override fun dismissLoadWait() {
        loadWaitDominator?.let { it.dismissAll() }
    }
}