package com.urovo.activity.customerlabel

import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.TextView
import com.urovo.activity.customerlabel.model.CustomerLabelDetailEntity
import com.urovo.activity.customerlabel.model.CustomerLabelEntity
import com.urovo.activity.customerlabel.model.CustomerLabelSubmitEntity
import com.urovo.activity.customerlabel.presenter.CustomerLabelContract
import com.urovo.activity.customerlabel.presenter.CustomerLabelPresenter
import com.urovo.wugumofang.R
import com.urovo.wugumofang.base.BaseActivity
import com.urovo.wugumofang.base.BasePresenter
import com.urovo.wugumofang.constants.AppConstants
import com.urovo.wugumofang.utils.isNotNullOrSize
import com.urovo.wugumofang.view.LoadWaitDominator
import com.urovo.wugumofang.view.flowlayout.FlowLayout
import com.urovo.wugumofang.view.flowlayout.TagAdapter
import com.urovo.wugumofang.view.flowlayout.TagView
import kotlinx.android.synthetic.main.act_customer_label.*

/**
 * 顾客标签页
 * @author LW
 */
class CustomerLabelActivity : BaseActivity(), CustomerLabelContract.IView {

    private var mCustomerLabelPresenter: CustomerLabelPresenter? = null

    private var mCustomerLabelEntity: CustomerLabelEntity? = null

    override fun addRootLayoutId(): Int = R.layout.act_customer_label

    override fun rootViewGroup(): ViewGroup = root_container_ll

    private val mMobile: String by lazy {
        intent.getStringExtra(AppConstants.ExtraKeys.PHONE_NUM)
    }

    private val mPageEditStatus: Boolean by lazy {
        intent.getBooleanExtra(AppConstants.ExtraKeys.PAGE_EDIT_STATUS, false)
    }

    private val mCustomerLabelDetailEntity: CustomerLabelDetailEntity? by lazy {
        intent.getSerializableExtra(AppConstants.ExtraKeys.CUSTOMER_LABEL_DETAIL) as CustomerLabelDetailEntity?
    }

    override fun initTitle() {
        super.initTitle()
        if (mPageEditStatus) {
            titleTextView.text = getString(R.string.customer_lable_add_title)
            complete.apply {
                text = getString(R.string.save_btn_txt)
                setBackgroundResource(R.drawable.selector_add_pro_sell)
                visibility = View.VISIBLE
                setOnClickListener {
                    //提交标签数据
                    submitLabelData()
                }
            }
        } else {
            titleTextView.text = getString(R.string.customer_label_title)
        }
    }

    override fun initViewNew() {
        initTitle()
        if (mPageEditStatus) {

            user_feature_tag_view.setOnTagClickListener { view, position, _ ->
                val labelBean =
                    mCustomerLabelEntity?.characteristicsLabelInfo?.get(position)
                labelBean?.let {
                    val tagView =(view as TagView).tagView as TextView
                    if(!it.select){
                        tagView.setBackgroundResource(R.drawable.shape_theme_btn_state)
                        tagView.setTextColor(resources.getColor(R.color.white_txt_color))
                    }else{
                        tagView.setBackgroundResource(R.drawable.shape_white_btn_state)
                        tagView.setTextColor(resources.getColor(R.color.gray_txt_color))
                    }

                    it.select = !it.select
                }
                true
            }

            buy_use_tag_view.setOnTagClickListener { view, position, _ ->
                val labelBean = mCustomerLabelEntity?.purposeLabelInfo?.get(position)
                labelBean?.let {
                    val tagView =(view as TagView).tagView as TextView
                    if(!it.select){
                        tagView.setBackgroundResource(R.drawable.shape_theme_btn_state)
                        tagView.setTextColor(resources.getColor(R.color.white_txt_color))
                    }else{
                        tagView.setBackgroundResource(R.drawable.shape_white_btn_state)
                        tagView.setTextColor(resources.getColor(R.color.gray_txt_color))
                    }
                    it.select = !it.select
                }
                true
            }

            health_function_tag_view.setOnTagClickListener { view, position, _ ->
                val labelBean = mCustomerLabelEntity?.benefitsLabelInfo?.get(position)
                labelBean?.let {
                    val tagView =(view as TagView).tagView as TextView
                    if(!it.select){
                        tagView.setBackgroundResource(R.drawable.shape_theme_btn_state)
                        tagView.setTextColor(resources.getColor(R.color.white_txt_color))
                    }else{
                        tagView.setBackgroundResource(R.drawable.shape_white_btn_state)
                        tagView.setTextColor(resources.getColor(R.color.gray_txt_color))
                    }
                    it.select = !it.select
                }
                true
            }
        }
    }

    /**
     * 提交标签数据
     */
    private fun submitLabelData() {
        val customerLabelSubmitList = mutableListOf<CustomerLabelSubmitEntity>()

        val characteristicsLabelList =
            mCustomerLabelEntity?.characteristicsLabelInfo
        if (characteristicsLabelList.isNotNullOrSize()) {
            for (bean in characteristicsLabelList!!) {
                if (bean.select) {
                    customerLabelSubmitList.add(composeSubmitData(bean))
                }
            }
        }

        val purposeLabelInfoList = mCustomerLabelEntity?.purposeLabelInfo
        if (purposeLabelInfoList.isNotNullOrSize()) {
            for (bean in purposeLabelInfoList!!) {
                if (bean.select) {
                    customerLabelSubmitList.add(composeSubmitData(bean))
                }
            }
        }

        val benefitsLabelList = mCustomerLabelEntity?.benefitsLabelInfo
        if (benefitsLabelList.isNotNullOrSize()) {
            for (bean in benefitsLabelList!!) {
                if (bean.select) {
                    customerLabelSubmitList.add(composeSubmitData(bean))
                }
            }
        }
        if (customerLabelSubmitList.isNotNullOrSize()) {
            //提交标签数据
            mCustomerLabelPresenter?.requestSaveLabelDetail(customerLabelSubmitList)
        } else {
            showToast(R.string.customer_label_entry)
        }
    }

    /**
     * 组合提交数据
     */
    private fun composeSubmitData(bean: CustomerLabelEntity.LabelBean): CustomerLabelSubmitEntity {
        return CustomerLabelSubmitEntity(
            bean.createTime,
            bean.id,
            bean.labelId,
            bean.labelName,
            mMobile,
            bean.updateTime
        )
    }

    override fun initDataNew() {
        if (mPageEditStatus) {
            mCustomerLabelPresenter?.requestLabelList(1)
        } else {
            if (mCustomerLabelDetailEntity != null) {
                labelListDetailDataToView()
                //mCustomerLabelPresenter?.requestLabelDetailList(mMobile, 1)
            } else {
                showState(LoadWaitDominator.NO_DATA)
            }
        }
    }

    override fun labelListData(data: CustomerLabelEntity?) {
        super.labelListData(data)
        if (data != null) {
            this.mCustomerLabelEntity = data
            labelListDataToView()
        }
    }

    /**
     * 标签信息详情
     */
    private fun labelListDetailDataToView() {
        val inflater = LayoutInflater.from(mContext)
        mCustomerLabelDetailEntity?.let {
            val characteristicsLabelDetailInfo = it.characteristicsLabelDetailInfo //用户特征
            if (characteristicsLabelDetailInfo != null) {
                user_feature_tag_view.adapter = object :
                    TagAdapter<CustomerLabelDetailEntity.LabelBean>(
                        characteristicsLabelDetailInfo
                    ) {
                    override fun getView(
                        parent: FlowLayout,
                        position: Int,
                        bean: CustomerLabelDetailEntity.LabelBean
                    ): View {
                        val tagView = inflater.inflate(
                            R.layout.item_customer_label_view,
                            user_feature_tag_view, false
                        ) as TextView
                        tagView.text = bean.labelName
                        tagView.setBackgroundResource(R.drawable.shape_theme_btn_state)
                        tagView.setTextColor(resources.getColor(R.color.white_txt_color))
                        return tagView
                    }
                }
            } else {
                user_feature_tv.visibility = View.GONE
                user_feature_tag_view.visibility = View.GONE
            }

            val purposeLabelDetailInfo = it.purposeLabelDetailInfo  //	购买用途
            if (purposeLabelDetailInfo != null) {
                buy_use_tag_view.adapter = object :
                    TagAdapter<CustomerLabelDetailEntity.LabelBean>(purposeLabelDetailInfo) {
                    override fun getView(
                        parent: FlowLayout,
                        position: Int,
                        bean: CustomerLabelDetailEntity.LabelBean
                    ): View {
                        val tagView = inflater.inflate(
                            R.layout.item_customer_label_view,
                            buy_use_tag_view, false
                        ) as TextView
                        tagView.text = bean.labelName
                        tagView.setBackgroundResource(R.drawable.shape_theme_btn_state)
                        tagView.setTextColor(resources.getColor(R.color.white_txt_color))
                        return tagView
                    }
                }
            } else {
                buy_use_tv.visibility = View.GONE
                buy_use_tag_view.visibility = View.GONE
            }
            val benefitsLabelDetailInfo = it.benefitsLabelDetailInfo  //养生功效
            if (benefitsLabelDetailInfo != null) {
                health_function_tag_view.adapter = object :
                    TagAdapter<CustomerLabelDetailEntity.LabelBean>(benefitsLabelDetailInfo) {
                    override fun getView(
                        parent: FlowLayout,
                        position: Int,
                        bean: CustomerLabelDetailEntity.LabelBean
                    ): View {
                        val tagView = inflater.inflate(
                            R.layout.item_customer_label_view,
                            health_function_tag_view, false
                        ) as TextView
                        tagView.text = bean.labelName
                        tagView.setBackgroundResource(R.drawable.shape_theme_btn_state)
                        tagView.setTextColor(resources.getColor(R.color.white_txt_color))
                        return tagView
                    }
                }
            } else {
                health_function_tv.visibility = View.GONE
                health_function_tag_view.visibility = View.GONE
            }
        }
    }

    /**
     * 标签信息
     */
    private fun labelListDataToView() {
        val inflater = LayoutInflater.from(mContext)
        mCustomerLabelEntity?.let {
            val characteristicsLabelInfo = it.characteristicsLabelInfo //用户特征
            mCustomerLabelDetailEntity?.characteristicsLabelDetailInfo?.forEach { detailLabelBean ->
                val detailLabelName = detailLabelBean.labelName
                characteristicsLabelInfo?.forEach { labelBean ->
                    val labelName = labelBean.labelName
                    if (labelName.equals(detailLabelName)) {
                        labelBean.select = true
                    }
                }
            }

            if (characteristicsLabelInfo != null) {
                user_feature_tag_view.adapter = object :
                    TagAdapter<CustomerLabelEntity.LabelBean>(characteristicsLabelInfo) {
                    override fun getView(
                        parent: FlowLayout,
                        position: Int,
                        bean: CustomerLabelEntity.LabelBean
                    ): View {
                        val tagView = inflater.inflate(
                            R.layout.item_customer_label_view,
                            user_feature_tag_view, false
                        ) as TextView

                        if(bean.select){
                            tagView.setBackgroundResource(R.drawable.shape_theme_btn_state)
                            tagView.setTextColor(resources.getColor(R.color.white_txt_color))
                        }else{
                            tagView.setBackgroundResource(R.drawable.shape_white_btn_state)
                            tagView.setTextColor(resources.getColor(R.color.gray_txt_color))
                        }
                        tagView.text = bean.labelName
                        return tagView
                    }
                }
            } else {
                user_feature_tv.visibility = View.GONE
                user_feature_tag_view.visibility = View.GONE
            }

            val purposeLabelInfo = it.purposeLabelInfo  //	购买用途
            mCustomerLabelDetailEntity?.purposeLabelDetailInfo?.forEach { detailLabelBean ->
                val detailLabelName = detailLabelBean.labelName
                purposeLabelInfo?.forEach { labelBean ->
                    val labelName = labelBean.labelName
                    if (labelName.equals(detailLabelName)) {
                        labelBean.select = true
                    }
                }
            }

            if (purposeLabelInfo != null) {
                buy_use_tag_view.adapter = object :
                    TagAdapter<CustomerLabelEntity.LabelBean>(purposeLabelInfo) {
                    override fun getView(
                        parent: FlowLayout,
                        position: Int,
                        bean: CustomerLabelEntity.LabelBean
                    ): View {
                        val tagView = inflater.inflate(
                            R.layout.item_customer_label_view,
                            buy_use_tag_view, false
                        ) as TextView
                        if(bean.select){
                            tagView.setBackgroundResource(R.drawable.shape_theme_btn_state)
                            tagView.setTextColor(resources.getColor(R.color.white_txt_color))
                        }else{
                            tagView.setBackgroundResource(R.drawable.shape_white_btn_state)
                            tagView.setTextColor(resources.getColor(R.color.gray_txt_color))
                        }
                        tagView.text = bean.labelName
                        return tagView
                    }
                }
            } else {
                buy_use_tv.visibility = View.GONE
                buy_use_tag_view.visibility = View.GONE
            }


            val benefitsLabelInfo = it.benefitsLabelInfo  //养生功效
            mCustomerLabelDetailEntity?.benefitsLabelDetailInfo?.forEach { detailLabelBean ->
                val detailLabelName = detailLabelBean.labelName
                benefitsLabelInfo?.forEach { labelBean ->
                    val labelName = labelBean.labelName
                    if (labelName.equals(detailLabelName)) {
                        labelBean.select = true
                    }
                }
            }
            if (benefitsLabelInfo != null) {
                health_function_tag_view.adapter = object :
                    TagAdapter<CustomerLabelEntity.LabelBean>(benefitsLabelInfo) {
                    override fun getView(
                        parent: FlowLayout,
                        position: Int,
                        bean: CustomerLabelEntity.LabelBean
                    ): View {
                        val tagView = inflater.inflate(
                            R.layout.item_customer_label_view,
                            health_function_tag_view, false
                        ) as TextView
                        if(bean.select){
                            tagView.setBackgroundResource(R.drawable.shape_theme_btn_state)
                            tagView.setTextColor(resources.getColor(R.color.white_txt_color))
                        }else{
                            tagView.setBackgroundResource(R.drawable.shape_white_btn_state)
                            tagView.setTextColor(resources.getColor(R.color.gray_txt_color))
                        }
                        tagView.text = bean.labelName
                        return tagView
                    }
                }
            } else {
                health_function_tv.visibility = View.GONE
                health_function_tag_view.visibility = View.GONE
            }
        }
    }

    override fun saveLabelDetailStatus(status: Boolean) {
        super.saveLabelDetailStatus(status)
        if (status) {
            this.finish()
        } else {
            showToast(R.string.customer_label_submit_error)
        }
    }

    override fun addPresenter(): BasePresenter {
        mCustomerLabelPresenter = CustomerLabelPresenter(this)
        return mCustomerLabelPresenter!!
    }

}