package com.urovo.activity.customerlabel.presenter

import com.urovo.activity.customerlabel.model.CustomerLabelDetailEntity
import com.urovo.activity.customerlabel.model.CustomerLabelEntity
import com.urovo.activity.customerlabel.model.CustomerLabelSubmitEntity
import com.urovo.activity.customerlabel.service.CustomerLabelRepository
import com.urovo.wugumofang.R
import com.urovo.wugumofang.base.mvp.AbsPresenter
import com.urovo.wugumofang.constants.AppConstants
import com.urovo.wugumofang.net.ApiException
import com.urovo.wugumofang.net.callback.RetrofitCallback
import com.urovo.wugumofang.utils.isEmptyParameter
import com.urovo.wugumofang.view.LoadWaitDominator
import com.zsy.libcore.model.bean.RequestResult
import retrofit2.Call
import retrofit2.Response

/**
 * 顾客标签 Presenter
 * @author LW
 */
class CustomerLabelPresenter(
    private var view: CustomerLabelContract.IView?,
    private var depository: CustomerLabelRepository = CustomerLabelRepository()
) : AbsPresenter(), CustomerLabelContract.IPresenter {

    override fun requestLabelList(page: Int) {
        view?.showIsLoading()
        depository.requestLabelList(page, 1000)
            .enqueue(object : RetrofitCallback<RequestResult<CustomerLabelEntity>>() {
                override fun onResponse(
                    call: Call<RequestResult<CustomerLabelEntity>>,
                    res: Response<RequestResult<CustomerLabelEntity>>
                ) {
                    val body = res.body()
                    body.takeIf { it != null }?.run {
                        if (msgCode == AppConstants.Http.RESPONSE_SUCCESS) {
                            if (payload != null) {
                                view?.labelListData(payload)
                            } else {
                                view?.showState(LoadWaitDominator.NO_DATA)
                            }
                        } else {
                            view?.showError(msgInfo)
                        }
                    } ?: view?.showState(LoadWaitDominator.NET_ERROR)

                    view?.dismissLoading()
                }

                override fun onFailure(exception: ApiException) {
                    view?.dismissLoading()
                    view?.showError(exception.message)
                }
            })
    }

    override fun requestLabelDetailList(mobile: String?, page: Int) {
        if (isEmptyParameter(mobile)) {
            view?.showError(R.string.common_load_error)
            return
        }

        view?.showIsLoading()
        depository.requestLabelDetailList(mobile, page, 1000)
            .enqueue(object : RetrofitCallback<RequestResult<CustomerLabelDetailEntity>>() {
                override fun onResponse(
                    call: Call<RequestResult<CustomerLabelDetailEntity>>,
                    res: Response<RequestResult<CustomerLabelDetailEntity>>
                ) {
                    val body = res.body()
                    body.takeIf { it != null }?.run {
                        if (msgCode == AppConstants.Http.RESPONSE_SUCCESS) {
                            if (payload != null) {
                                view?.labelListDetailData(payload)
                            } else {
                                view?.showState(LoadWaitDominator.NO_DATA)
                            }
                        } else {
                            view?.showError(msgInfo)
                        }
                    } ?: view?.showState(LoadWaitDominator.NET_ERROR)

                    view?.dismissLoading()
                }

                override fun onFailure(exception: ApiException) {
                    view?.dismissLoading()
                    view?.showError(exception.message)
                }
            })
    }

    override fun requestSaveLabelDetail(customerLabelSubmitList: List<CustomerLabelSubmitEntity>) {

        view?.showTipDialog(R.string.customer_label_submit_tip)
        depository.requestSaveLabelDetail(customerLabelSubmitList)
            .enqueue(object : RetrofitCallback<RequestResult<Any>>() {
                override fun onResponse(
                    call: Call<RequestResult<Any>>,
                    res: Response<RequestResult<Any>>
                ) {
                    val body = res.body()
                    body.takeIf { it != null }?.run {
                        if (msgCode == AppConstants.Http.RESPONSE_SUCCESS) {
                            view?.saveLabelDetailStatus(true)
                        } else {
                            view?.saveLabelDetailStatus(false)
                        }
                    }
                    view?.dismissDialog()
                }

                override fun onFailure(exception: ApiException) {
                    view?.dismissDialog()
                    view?.saveLabelDetailStatus(false)
                }
            })
    }
}