package com.urovo.activity.data

import android.graphics.drawable.BitmapDrawable
import android.os.Bundle
import android.view.Gravity
import android.view.MotionEvent
import android.view.View
import android.view.ViewGroup
import android.widget.*
import androidx.recyclerview.widget.LinearLayoutManager
import com.urovo.activity.addsellactivity.ui.AutoRadioGroup
import com.urovo.activity.data.queryfinalsales.net.FinalSalesRes
import com.urovo.activity.data.queryfinalsales.presenter.FinalSalesContract
import com.urovo.activity.data.queryfinalsales.presenter.FinalSalesPre
import com.urovo.activity.data.queryfinalsales.ui.FinalSalesAdapter
import com.urovo.wgmf.ext.afterTextChanged
import com.urovo.wugumofang.R
import com.urovo.wugumofang.base.BaseActivity
import com.urovo.wugumofang.data.DateUtil
import com.urovo.wugumofang.data.MoneyUtils
import com.urovo.wugumofang.system.SystemHelper
import com.urovo.wugumofang.view.CustomPopupWindow
import com.urovo.wugumofang.view.LoadWaitDominator
import com.wgmf.trade.widgets.RecycleViewDivider
import kotlinx.android.synthetic.main.act_query_final_sales.*
import kotlinx.android.synthetic.main.view_query_data_head.*

/**
 * @author Sasuke on 2018/6/6.
 * 产品销售查询
 * (懒得去改后台配的动态地址了，所以直接把这个Activity放在外面了。。。)
 */

class QueryFinalProductSales : BaseActivity(), FinalSalesContract.view {

    companion object {
        const val ALL = "全部"
        const val XDY = "食补集"
        const val WGMF = "磨粉产品"
        const val FLOW = "流量产品"
        const val HYXS = "会员商品"
    }

    var pre: FinalSalesPre? = null
    var loadWait: LoadWaitDominator? = null//加载等待
    var radioGroup: AutoRadioGroup? = null//标签过滤栏
//    var listenerLock: Boolean = false //监听锁，防止过滤标签 与 搜索栏循环监听

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.act_query_final_sales)
        pre = FinalSalesPre(this).apply { start() }
    }

    override fun init() {
        initTitle()
        titleTextView.text = "产品销售查询"
        storeTitle.text = SystemHelper.LoginOrganName
        dateFromTo.text = "${pre?.getToday()} 至 ${pre?.getToday()}"
        loadWait = LoadWaitDominator(this, loadWaitContainer)

        //初始化过滤标签容器
        radioGroup = AutoRadioGroup(filterLabelContainer)
        //初始化分类标签栏
        initRadioGroup()

//        (filterLabelContainer.getChildAt(0) as RadioButton).isChecked = true

        //用扩展方法来实现监听文字
        searchInput.afterTextChanged {
            pre?.listsetTmp?.let { refreshShow(it) }//先把所有数据全塞进去刷一下
//            (showRv.adapter as FinalSalesAdapter).filterAndRefresh(it)//再过滤并刷新，避免越搜越少（都被过滤掉了）
        }
        //初始化列表组件
        showRv.apply {
            layoutManager = LinearLayoutManager(this@QueryFinalProductSales)
            adapter = FinalSalesAdapter(this@QueryFinalProductSales, listOf())
            addItemDecoration(
                RecycleViewDivider(
                    this@QueryFinalProductSales,
                    LinearLayoutManager.HORIZONTAL,
                    1,
                    resources.getColor(R.color.divider_line)
                )
            )
        }

        //选择日期
        linearLayout1.setOnClickListener { popupDateSelect() }
    }

    /**
     * 重新初始化RedioGroup
     * 把它提出来方便后面循环调用
     */
    fun initRadioGroup() {
        val checkedPos = radioGroup?.getCheckedPos()
        radioGroup?.removeAllChild()
        pre?.filterLabels?.forEach { labelText ->
            //以下三句：1、创建radioButton 2、在radioGroup里添加radioButton并创建监听，3、创建对应该radioButton的Fragment
            val radio: RadioButton = (layoutInflater.inflate(
                R.layout.final_sales_radio_button,
                filterLabelContainer,
                false
            ) as RadioButton)
            radioGroup?.addViewWithLayoutParams(
                view = radio,
                text = labelText,
                paramWith = RadioGroup.LayoutParams.WRAP_CONTENT,
                paramheight = RadioGroup.LayoutParams.WRAP_CONTENT,
                margin = arrayOf(4, 0, 4, 0)
            ) { pos ->
                searchInput.setText("")//切换标签的时候竟然还要清空搜索栏。。。
                pre?.listsetTmp?.let { refreshShow(it) }
            }
        }

        checkedPos?.let { radioGroup?.setChildChecked(if (checkedPos == -1) 0 else checkedPos) }
    }


    /**
     * 选择日期的弹窗
     */
    val dateSelectPop: CustomPopupWindow by lazy {
        CustomPopupWindow.Builder(this)
            .setContentView(R.layout.pop_date_select)
            .setFouse(true)
            .setOutSideCancel(true)
            .setAnimationStyle(R.style.dropmenu_anim_style)
            .setBackgroundDrawable(BitmapDrawable())
            .setwidth(ViewGroup.LayoutParams.MATCH_PARENT)
            .setheight(ViewGroup.LayoutParams.WRAP_CONTENT)
            .builder()
    }

    /**
     * 弹出选择日期的弹窗
     */
    fun popupDateSelect() {
        dateSelectPop?.let {
            //初始化显示日期
            (it.getItemView(R.id.popStoreName) as TextView).apply {
                text = SystemHelper.LoginOrganName
            }
            val startDateTv = (it.getItemView(R.id.popStartDate) as TextView).apply {
                text = dateFromTo.text.toString().split(" 至 ")[0]
            }
            val endDateTv = (it.getItemView(R.id.popEndDate) as TextView).apply {
                text = dateFromTo.text.toString().split(" 至 ")[1]
            }
            //起始日期监听
            (it.getItemView(R.id.popStartDateLayout) as LinearLayout).setOnClickListener {
                DateUtil.pickDialog(this@QueryFinalProductSales, startDateTv, "yyyy-MM-dd")
            }
            //结束日期监听
            (it.getItemView(R.id.popEndDateLayout) as LinearLayout).setOnClickListener {
                DateUtil.pickDialog(this@QueryFinalProductSales, endDateTv, "yyyy-MM-dd")
            }
            //查询监听
            (it.getItemView(R.id.queryBtn) as Button).setOnClickListener {
                //一定要先改外面的显示，再query()，因为query()是根据外面显示的文字来的
                dateFromTo.text = "${startDateTv.text.toString()} 至 ${endDateTv.text.toString()}"
                query()
                dateSelectPop.dismiss()
            }
        }
        dateSelectPop?.showAsLocation(back, Gravity.BOTTOM, 0, 0)
    }

    /**
     * 按已选择的标签过滤列表
     */
    override fun query() {
        //重新搜索的时候，要把搜索条件全部复位
        searchInput.setText("")
        radioGroup?.setChildChecked(0)
        loadWait.takeIf { it != null && !it.isLoading() }?.let {
            //只有在没有请求的时候，才发起请求；如果在转圈圈说明正在请求或加载数据中，那就不让它重复请求了。
            pre?.queryData(
                dateFromTo.text.toString().split(" 至 ")[0],
                dateFromTo.text.toString().split(" 至 ")[1],
                SystemHelper.LoginOrganId
            )
        }
    }


    /**
     * 刷新显示（就算你飞快切换点击过滤标签，显示的时候，还是会根据当前被选中的标签来过滤，所以完全不用担心标签和数据对不上，完美！）
     */
    override fun refreshShow(result: FinalSalesRes.ResultInfo) {
        var isRefreshRadioGroup = false
        //根据返回添加散装标签
        if (!result.saleXDYProducts.isNullOrEmpty() && pre?.filterLabels!!.none { it == XDY }) {
            pre?.filterLabels?.add(XDY)
            isRefreshRadioGroup = true
        }
        //根据返回添加会员商品标签
        if (!result.saleHYXSProducts.isNullOrEmpty() && pre?.filterLabels!!.none { it == HYXS }) {
            pre?.filterLabels?.add(HYXS)
            isRefreshRadioGroup = true
        }
        if (isRefreshRadioGroup) {//只要有一个变了，那就重新初始化标签栏
            initRadioGroup()
        }
        val checkedText = radioGroup?.getCheckedText()
        val key = searchInput.text.toString()//搜索关键字
        val newList = when (checkedText) {
            ALL -> result.saleAllProducts
            WGMF -> result.saleWGProducts
            FLOW -> result.flowWGProducts
            XDY -> result.saleXDYProducts
            HYXS -> result.saleHYXSProducts
            else -> null
        }?.sortedByDescending { it.productNum }//按产品数量降序
            ?.filter { key in it.productName || key in it.productId }

        //结合搜索栏，一起过滤
        (showRv.adapter as FinalSalesAdapter).setNewDataList(newList)
        //计算总金额并显示在搜索框右边
        val amount = MoneyUtils.twolastDF(newList?.sumByDouble { it.amount ?: 0.00 } ?: 0.00)
        proAmountTv.text = "销售金额：${amount}"

        //显示暂无数据
        if (newList?.size == 0) {
            showNoData()
        } else {
            dismissLoadWait()
        }
    }

    override fun setPresenter(presenter: FinalSalesContract.pre?) {
        this.pre = presenter as FinalSalesPre
    }

    override fun showLoading() {
        loadWait?.show(LoadWaitDominator.LOADING)
        showRv.visibility = View.GONE
    }

    override fun showNoData() {
        loadWait?.show(LoadWaitDominator.NO_DATA)
        showRv.visibility = View.GONE//这个没办法，因为使用材质布局的原因，没办法一边显示加载提示一边把列表盖住，必须手动控制列表的显隐
    }

    override fun showNetError() {
        loadWait?.show(LoadWaitDominator.NET_ERROR)
        showRv.visibility = View.GONE
    }

    override fun dismissLoadWait() {
        loadWait?.dismissAll()
        showRv.visibility = View.VISIBLE
    }

    /**
     *  design库的23.0.0版本的空指针BUG解决方案
     *  BUG现场：[ attempt to invoke virtual method float android.view motionevent.getx on null ~coordinatorlayout ]
     */
    override fun dispatchTouchEvent(ev: MotionEvent?): Boolean {
        try {
            return super.dispatchTouchEvent(ev)
        } catch (e: Exception) {
            return false
        }
    }
}


