package com.urovo.activity.data.queryfinalsales.ui

import android.content.Context
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.TextView
import androidx.recyclerview.widget.RecyclerView
import com.urovo.activity.data.queryfinalsales.net.FinalSalesRes
import com.urovo.wugumofang.R
import com.urovo.wugumofang.data.MoneyUtils

/**
 * @author Sasuke on 2018/6/7.
 */
class FinalSalesAdapter(val context: Context, var dataList: List<FinalSalesRes.ResultInfo.FinalSaleProduct>) : RecyclerView.Adapter<FinalSalesAdapter.FinalSaleHolder>() {

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): FinalSaleHolder {
        return FinalSaleHolder(LayoutInflater.from(context).inflate(R.layout.final_sales_item, parent, false))
    }

    override fun getItemCount(): Int {
        return dataList.size

    }

    override fun onBindViewHolder(holder: FinalSaleHolder, pos: Int) {
        val item =  dataList[pos]
        holder.apply {
            proName.text = item.productName
            proId.text = item.productId
            proNum.text = item.productNum?.let { MoneyUtils.twolastDF(it) } ?: "0.00"
            proAmount.text = item.amount?.let { MoneyUtils.twolastDF(it) } ?: "0.00"
            tbRate.text = item.tbRate ?: "--"
            hbRate.text = item.hbRate ?: "--"
        }
    }

    /**
     * 用于搜索产品（根据 产品名 或 产品ID）其实就是过滤
     */
    fun filterAndRefresh(key: String){
        dataList = dataList.filter {  key in it.productName || key in it.productId}
        notifyDataSetChanged()
    }


    /**
     * 刷新数据源。
     */
    fun setNewDataList(dataList: List<FinalSalesRes.ResultInfo.FinalSaleProduct>?) {
        if (dataList != null) {
            this.dataList = dataList
        }
        notifyDataSetChanged()
    }

    class FinalSaleHolder(view: View) : RecyclerView.ViewHolder(view) {
        var proName: TextView
        var proId: TextView
        var proNum: TextView
        var proAmount: TextView
        var tbRate: TextView
        var hbRate: TextView

        init {
            proName = view.findViewById(R.id.proNameTv) as TextView
            proId = view.findViewById(R.id.proIdTv) as TextView
            proNum = view.findViewById(R.id.proNumTv) as TextView
            proAmount = view.findViewById(R.id.proAmountTv) as TextView
            tbRate = view.findViewById(R.id.tbRateTv) as TextView
            hbRate = view.findViewById(R.id.hbRateTv) as TextView
        }
    }
}

