/*
 * Copyright (C) 2015 The Android Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.urovo.activity.data.querymyweivip.view.fragment;

import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import com.urovo.activity.data.querymyweivip.model.SweepCodeDetailListEntity;
import com.urovo.wgmf.adapter.CommonAdapter;
import com.urovo.wgmf.viewholder.ViewHolder;
import com.urovo.wugumofang.R;
import com.urovo.wugumofang.data.ObjectSaveUtil;
import com.urovo.wugumofang.system.BaseAppManager;
import com.zsy.libcore.ui.ada.helper.DividerItemDecoration;

import java.util.List;

import androidx.fragment.app.Fragment;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

public class SweepCodeDetailFra extends Fragment {
    private int mIndex;
    private List<SweepCodeDetailListEntity.IsNewListEntity> conformity;//符合
    private List<SweepCodeDetailListEntity.IsNotNewListEntity> inconformity;//不符合

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        RecyclerView rv = (RecyclerView) inflater.inflate(
                R.layout.fragment_sweep_code_detail_list, container, false);
        if (savedInstanceState == null) {
            mIndex = getArguments().getInt("index");

//            ((AppCompatActivity) getActivity()).getSupportActionBar()
//                    .setTitle("from arguments:" + mIndex);
        } else {
            mIndex = savedInstanceState.getInt("index");

//            ((AppCompatActivity) getActivity()).getSupportActionBar()
//                    .setTitle("from savedInstanceState:" + mIndex);
        }
        //获取请求的数据
        SweepCodeDetailListEntity mResultInfo = (SweepCodeDetailListEntity) ObjectSaveUtil.readObject(getActivity());
        if (mResultInfo != null) {
            conformity = mResultInfo.isNewList;//符合
            inconformity = mResultInfo.isNotNewList;//不符合
        }
        setupRecyclerView(rv);
        rv.setClickable(true);// 防止点击穿透，底层的fragment响应上层点击触摸事件
        return rv;
    }

    @Override
    public void onSaveInstanceState(Bundle outState) {
        super.onSaveInstanceState(outState);
        outState.putInt("index", mIndex);
    }

    private void setupRecyclerView(RecyclerView recyclerView) {
        //  创建线性布局
        recyclerView.setLayoutManager(new LinearLayoutManager(recyclerView.getContext()));

        //  创建列表项分隔线
        recyclerView.addItemDecoration(new DividerItemDecoration(getActivity(),
                DividerItemDecoration.VERTICAL_LIST));

        switch (mIndex) {
            case 0:
                recyclerView.setAdapter(new CommonAdapter<SweepCodeDetailListEntity.IsNewListEntity>(getActivity(),
                        R.layout.sweep_code_detail_list_item, conformity) {
                    @Override
                    public void convert(ViewHolder holder, SweepCodeDetailListEntity.IsNewListEntity conformityBean) {
                        if (conformityBean != null) {
                            TextView phoneNum = holder.getView(R.id.tv_phone_num);
                            TextView scanCodeOwnership = holder.getView(R.id.tv_scan_code_ownership);
                            phoneNum.setText(conformityBean.mobile);
//                            TextView nonMatchReason = holder.getView(R.id.tv_non_Match_reason);// 因为fragment切换是被销毁的，所以布局是重新加载的，不需要设置隐藏，本身就是隐藏
//                            nonMatchReason.setVisibility(View.GONE);
                            scanCodeOwnership.setText(conformityBean.adscription);
                        }
                    }
                });
                break;

            case 1:
                recyclerView.setAdapter(new CommonAdapter<SweepCodeDetailListEntity.IsNotNewListEntity>(getActivity(),
                        R.layout.sweep_code_detail_list_item, inconformity) {
                    @Override
                    public void convert(ViewHolder holder, SweepCodeDetailListEntity.IsNotNewListEntity inconformityBean) {
                        if (inconformityBean != null) {
                            TextView phoneNum = holder.getView(R.id.tv_phone_num);
                            TextView scanCodeOwnership = holder.getView(R.id.tv_scan_code_ownership);
                            TextView nonMatchReason = holder.getView(R.id.tv_non_Match_reason);
                            nonMatchReason.setVisibility(View.VISIBLE);

//                  ****************** 防火防盗防后台不给数据******************
                            if (inconformityBean.mobile != null) {
                                phoneNum.setText(inconformityBean.mobile);
                            } else {
                                BaseAppManager.loger().error("inconformityBean.mobile is null");
                            }
                            if (inconformityBean.adscription != null) {
                                scanCodeOwnership.setText(inconformityBean.adscription);
                            } else {
                                BaseAppManager.loger().error("inconformityBean.adscription is null");
                            }
                            if (inconformityBean.reason != null) {
                                nonMatchReason.setText(inconformityBean.reason);
                            } else {
                                BaseAppManager.loger().error("inconformityBean.reason is null");
                            }
                        }
                    }
                });
                break;
        }
    }
}
