/*
 * Copyright (C) 2015 The Android Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.urovo.activity.data.queryproductsales.view;

import android.os.Bundle;
import android.text.TextUtils;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;
import android.widget.Toast;

import androidx.fragment.app.Fragment;
import androidx.recyclerview.widget.GridLayoutManager;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.chad.library.adapter.base.entity.MultiItemEntity;
import com.google.gson.Gson;
import com.urovo.activity.data.queryproductsales.adapter.ExpandableItemAdapter;
import com.urovo.activity.data.queryproductsales.model.Level0Item;
import com.urovo.activity.data.queryproductsales.model.Level1Item;
import com.urovo.activity.data.queryproductsales.model.MonthSaleProductEntity;
import com.urovo.wgmf.CommonController;
import com.urovo.wgmf.adapter.CommonAdapter;
import com.urovo.wgmf.viewholder.ViewHolder;
import com.urovo.wugumofang.R;
import com.urovo.wugumofang.constants.AppConstants;
import com.urovo.wugumofang.data.PreferHelper;
import com.urovo.wugumofang.system.SystemHelper;
import com.urovo.wugumofang.utils.MyToast;
import com.zsy.libcore.common.android.log.SYLoger;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

/**
 * @author chenji
 */
public class MonthSaleProductFra extends Fragment implements CommonController.NotifyData {
    private RecyclerView rv;
    /**
     * 门店--全部产品
     */
    private List<MonthSaleProductEntity.SaleAllProductsEntity> saleAllProducts;

    /**
     * 门店--五谷磨房产品
     */
    private List<MonthSaleProductEntity.SaleWGProductsEntity> saleWGProducts;

    /**
     * 门店--鲜稻研产品
     */
    private List<MonthSaleProductEntity.SaleXDYProductsEntity> saleXDYProducts;
    /**
     * 门店--流量产品
     */
    private List<MonthSaleProductEntity.FlowWGProductsEntity> flowWGProducts;

    /**
     * 会员商城
     */
    private List<MonthSaleProductEntity.VipSaleProductsEntity> vipSaleProducts;

    /**
     * 折叠的集合数据
     */
    private List<MultiItemEntity> mRes = new ArrayList<>();

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        rv = (RecyclerView) inflater.inflate(R.layout.fragment_sweep_code_detail_list, container, false);
        //获取请求的数据
        final String mResultInfo = PreferHelper.getInstance().getString(AppConstants.SalesPerformance.MONTH_SALE_PRODUCT_ACT);
        SYLoger.print("wgmf--mResultInfo--" + mResultInfo);
        final MonthSaleProductEntity monthSaleProductEntity = new Gson().fromJson(mResultInfo, MonthSaleProductEntity.class);
        SYLoger.print("wgmf--mResultInfo--monthSaleProductEntity--" + monthSaleProductEntity);
        if (monthSaleProductEntity != null) {
            //降序排序
            this.saleAllProducts = monthSaleProductEntity.saleAllProducts;
            this.saleWGProducts = monthSaleProductEntity.saleWGProducts;
            this.saleXDYProducts = monthSaleProductEntity.saleXDYProducts;
            this.flowWGProducts = monthSaleProductEntity.flowWGProducts;
            this.vipSaleProducts = monthSaleProductEntity.onlineVipSaleProducts;
            sortAllProList(saleAllProducts);
            sortWGProList(saleWGProducts);
            sortXDYProList(saleXDYProducts);
            sortVipProList(vipSaleProducts);
        }

        final int mIndex = PreferHelper.getInstance().getInt(AppConstants.SalesPerformance.MONTH_SALE_INDEX);
        final String mBrand = PreferHelper.getInstance().getString(AppConstants.SalesPerformance.MONTH_SALE_BRAND);
        setupRecyclerView(rv, mIndex, mBrand);
        // 防止点击穿透，底层的fragment响应上层点击触摸事件
        rv.setClickable(true);
        return rv;
    }

    /**
     * @param recyclerView ...
     * @param index        ...
     * @param brand        brand = null : 全部产品 ； brand= wgmf: 五谷磨房产品； brand= xdy: 鲜稻研产品
     */
    private void setupRecyclerView(RecyclerView recyclerView, int index, String brand) {
        switch (index) {
            case AppConstants.SalesPerformance.STORE:
                if ("磨粉产品".equals(brand)) {
                    // 五谷磨房产品数据
                    initStoreRecycleView(convertWGProListData(saleWGProducts));
                }
//                else if (SystemHelper.isXdy(brand)) {
                else if (AppConstants.BRAND.BULK_PRO.equals(brand)) {
                    // 鲜稻研产品数据
                    initStoreRecycleView(convertXDYProListData(saleXDYProducts));
                } else if ("流量产品".equals(brand)) {
                    // 流量产品数据
                    initStoreRecycleView(convertFlowProListData(flowWGProducts));
                } else if (AppConstants.BRAND.ALL_PRO.equals(brand)) {
                    initStoreRecycleView(convertALLProListData(saleAllProducts));
                } else {
                    initStoreRecycleView(null);
                    SYLoger.print("品牌名配置出错！或者 数据为空！");
                }
                break;
            case AppConstants.SalesPerformance.MENBER_MALL_STORE:
                //  创建线性布局
                if (AppConstants.BRAND.VIP_PRO.equals(brand)) {
                    recyclerView.setLayoutManager(new LinearLayoutManager(recyclerView.getContext()));
                    recyclerView.setAdapter(new CommonAdapter<MonthSaleProductEntity.VipSaleProductsEntity>(getActivity(),
                            R.layout.month_sale_product_detail_item, vipSaleProducts) {
                        @Override
                        public void convert(ViewHolder holder, MonthSaleProductEntity.VipSaleProductsEntity vipSaleProductsEntity) {
                            if (vipSaleProductsEntity != null) {
                                //产品名称
                                TextView productName = holder.getView(R.id.month_product_name);
                                //销售额
                                TextView productAmounts = holder.getView(R.id.month_product_amounts);
                                //数量
                                TextView productNum = holder.getView(R.id.month_product_number);
                                //销售额占比
                                TextView productAmountRate = holder.getView(R.id.month_product_sales_amount_rate);
                                holder.getView(R.id.month_product_id).setVisibility(View.GONE);

                                productName.setText(vipSaleProductsEntity.productName == null ?
                                        "" : vipSaleProductsEntity.productName);
                                productAmounts.setText(vipSaleProductsEntity.amount == null ?
                                        "" : vipSaleProductsEntity.amount);
                                productNum.setText(vipSaleProductsEntity.productNum == null ?
                                        "" : vipSaleProductsEntity.productNum);
                                productAmountRate.setText(vipSaleProductsEntity.amountRate == null ?
                                        "" : vipSaleProductsEntity.amountRate);
                            }
                        }
                    });
                } else {
                    initStoreRecycleView(null);
                    SYLoger.print("品牌名配置出错！");
                }
                break;
            default:

                break;
        }
    }

    /**
     * 转化成可以用于折叠的list数据
     * 列表数据转换(为了不改变通用适配器框架原本的实体类)
     *
     * @param dateList 网络请求后得到的数据
     * @return MultiItemEntity实体列表
     */
    private List<MultiItemEntity> convertALLProListData(List<MonthSaleProductEntity.SaleAllProductsEntity> dateList) {
        SYLoger.print("wgmf--产品月销售查询--convertListData: " + dateList);
        if (dateList != null) {
            //请求前清空上次的缓存数据
            if (!mRes.isEmpty()) {
                mRes.clear();
            }
            for (int i = 0; i < dateList.size(); i++) {

                Level0Item lv0 = new Level0Item(dateList.get(i).productName + ", 销售额占比：", dateList.get(i).amountRate);
                //降序排序
                if (dateList.get(i).list != null) {
                    sortAllProSubList(dateList.get(i).list);
                }
                for (int j = 0; j < dateList.get(i).list.size(); j++) {
                    final MonthSaleProductEntity.SaleAllProductsEntity.ListEntity listEntity = dateList.get(i).list.get(j);
                    Level1Item lv1 = new Level1Item(
                            listEntity.productName,
                            listEntity.productId,
                            listEntity.amount,
                            listEntity.productNum,
                            listEntity.amountRate
                    );
                    lv0.addSubItem(lv1);
                }
                mRes.add(lv0);
            }
        }
        return mRes;
    }

    private List<MultiItemEntity> convertWGProListData(List<MonthSaleProductEntity.SaleWGProductsEntity> dateList) {
        SYLoger.print("wgmf--产品月销售查询--convertListData: " + dateList);
        if (dateList != null) {
            //请求前清空上次的缓存数据
            if (!mRes.isEmpty()) {
                mRes.clear();
            }
            for (int i = 0; i < dateList.size(); i++) {
                Level0Item lv0 = new Level0Item(dateList.get(i).productName + ", 销售额占比：", dateList.get(i).amountRate);
                //降序排序
                if (dateList.get(i).list != null) {
                    sortWGProSubList(dateList.get(i).list);
                }
                for (int j = 0; j < dateList.get(i).list.size(); j++) {
                    final MonthSaleProductEntity.SaleWGProductsEntity.ListEntity listEntity = dateList.get(i).list.get(j);
                    Level1Item lv1 = new Level1Item(
                            listEntity.productName,
                            listEntity.productId,
                            listEntity.amount,
                            listEntity.productNum,
                            listEntity.amountRate
                    );
                    lv0.addSubItem(lv1);
                }
                mRes.add(lv0);
            }
        }
        return mRes;
    }

    private List<MultiItemEntity> convertFlowProListData(List<MonthSaleProductEntity.FlowWGProductsEntity> dateList) {
        SYLoger.print("wgmf--产品月销售查询--convertListData: " + dateList);
        if (dateList != null) {
            //请求前清空上次的缓存数据
            if (!mRes.isEmpty()) {
                mRes.clear();
            }
            for (int i = 0; i < dateList.size(); i++) {
                Level0Item lv0 = new Level0Item(dateList.get(i).productName + ", 销售额占比：", dateList.get(i).amountRate);
                //降序排序
                if (dateList.get(i).list != null) {
                    sortFlowProSubList(dateList.get(i).list);
                }
                for (int j = 0; j < dateList.get(i).list.size(); j++) {
                    final MonthSaleProductEntity.FlowWGProductsEntity.ListEntity listEntity = dateList.get(i).list.get(j);
                    Level1Item lv1 = new Level1Item(
                            listEntity.productName,
                            listEntity.productId,
                            listEntity.amount,
                            listEntity.productNum,
                            listEntity.amountRate
                    );
                    lv0.addSubItem(lv1);
                }
                mRes.add(lv0);
            }
        }
        return mRes;
    }

    private List<MultiItemEntity> convertXDYProListData(List<MonthSaleProductEntity.SaleXDYProductsEntity> dateList) {
        SYLoger.print("wgmf--产品月销售查询--convertListData: " + dateList);
        if (dateList != null) {
            //请求前清空上次的缓存数据
            if (!mRes.isEmpty()) {
                mRes.clear();
            }
            for (int i = 0; i < dateList.size(); i++) {

                Level0Item lv0 = new Level0Item(dateList.get(i).productName + ", 销售额占比：", dateList.get(i).amountRate);
                //降序排序
                if (dateList.get(i).list != null) {
                    sortXDYProSubList(dateList.get(i).list);
                }
                for (int j = 0; j < dateList.get(i).list.size(); j++) {
                    final MonthSaleProductEntity.SaleXDYProductsEntity.ListEntity listEntity = dateList.get(i).list.get(j);
                    Level1Item lv1 = new Level1Item(
                            listEntity.productName,
                            listEntity.productId,
                            listEntity.amount,
                            listEntity.productNum,
                            listEntity.amountRate
                    );
                    lv0.addSubItem(lv1);
                }
                mRes.add(lv0);
            }
        }
        return mRes;
    }

    private void initStoreRecycleView(List<MultiItemEntity> res) {
        ExpandableItemAdapter adapter = new ExpandableItemAdapter(res);
        final GridLayoutManager manager = new GridLayoutManager(getActivity(), 3);
        manager.setSpanSizeLookup(new GridLayoutManager.SpanSizeLookup() {
            @Override
            public int getSpanSize(int position) {
                //因为布局是填充满的,所以都是3列铺满.
                return manager.getSpanCount();
            }
        });
        rv.setHasFixedSize(true);
        rv.setAdapter(adapter);
        rv.setLayoutManager(manager);
        adapter.expandAll(0, true);
    }


    @Override
    public void notifyDataSetChanged(int index, String brand) {
        if (SystemHelper.isWgmf(brand)
//                || SystemHelper.isXdy(brand)
                || "磨粉产品".equals(brand)
                || "流量产品".equals(brand)
                || AppConstants.BRAND.BULK_PRO.equals(brand)
                || AppConstants.BRAND.ALL_PRO.equals(brand)
                || AppConstants.BRAND.VIP_PRO.equals(brand)) {
            if (rv != null) {
                setupRecyclerView(rv, index, brand);
            }
        } else {
            //配置出错，让列表制空。
            if (rv != null) {
                setupRecyclerView(rv, index, "品牌名配置出错");
            }
            MyToast.show("品牌名配置出错！");
        }
    }

    /**
     * 列表根据销售额占比降序排列
     *
     * @param list 请求后得到的列表数据
     */
    private void sortAllProList(List<MonthSaleProductEntity.SaleAllProductsEntity> list) {
        SYLoger.print("list --" + list);
        if (list != null) {
            Collections.sort(list, new Comparator<MonthSaleProductEntity.SaleAllProductsEntity>() {
                @Override
                public int compare(MonthSaleProductEntity.SaleAllProductsEntity lhs, MonthSaleProductEntity.SaleAllProductsEntity rhs) {
                    if (lhs != null
                            && rhs != null
                            && !TextUtils.isEmpty(lhs.amountRate)
                            && !TextUtils.isEmpty(rhs.amountRate)) {
                        return commonCompare(lhs.amountRate, rhs.amountRate);
                    }
                    return 0;
                }
            });
        }
    }

    private void sortWGProList(List<MonthSaleProductEntity.SaleWGProductsEntity> list) {
        if (list != null) {
            Collections.sort(list, new Comparator<MonthSaleProductEntity.SaleWGProductsEntity>() {
                @Override
                public int compare(MonthSaleProductEntity.SaleWGProductsEntity lhs, MonthSaleProductEntity.SaleWGProductsEntity rhs) {
                    if (lhs != null
                            && rhs != null
                            && !TextUtils.isEmpty(lhs.amountRate)
                            && !TextUtils.isEmpty(rhs.amountRate)) {
                        return commonCompare(lhs.amountRate, rhs.amountRate);
                    }
                    return 0;
                }
            });
        }
    }

    private void sortXDYProList(List<MonthSaleProductEntity.SaleXDYProductsEntity> list) {
        if (list != null) {
            Collections.sort(list, new Comparator<MonthSaleProductEntity.SaleXDYProductsEntity>() {
                @Override
                public int compare(MonthSaleProductEntity.SaleXDYProductsEntity lhs, MonthSaleProductEntity.SaleXDYProductsEntity rhs) {
                    if (lhs != null
                            && rhs != null
                            && !TextUtils.isEmpty(lhs.amountRate)
                            && !TextUtils.isEmpty(rhs.amountRate)) {
                        return commonCompare(lhs.amountRate, rhs.amountRate);
                    }
                    return 0;
                }
            });
        }
    }

    /**
     * 公共比较的方法的逻辑
     *
     * @param lhsAmountRate ...
     * @param rhsAmountRate ...
     * @return ...
     */
    private int commonCompare(String lhsAmountRate, String rhsAmountRate) {
        //开始比较,降序处理
        if (lhsAmountRate.contains("%") && rhsAmountRate.contains("%")) {
            String tempA = lhsAmountRate.replace("%", "");
            String tempB = rhsAmountRate.replace("%", "");
            tempA = TextUtils.isEmpty(tempA) ? "0" : tempA;//マジで信じられんわ。。。"" を投げてくるとは、危うく俺のせいにされた。
            tempB = TextUtils.isEmpty(tempB) ? "0" : tempB;
            //精确表示,double或者float类型必须做精确处理
            BigDecimal lAmountRate = new BigDecimal(tempA);
            BigDecimal rAmountRate = new BigDecimal(tempB);
            return rAmountRate.compareTo(lAmountRate);
        } else {
            MyToast.show(getActivity(), "产品月销量数据格式错误!", Toast.LENGTH_SHORT);
            return 0;
        }
    }

    /**
     * 门店
     *
     * @param list 折叠子view集合
     */
    private void sortAllProSubList(List<MonthSaleProductEntity.SaleAllProductsEntity.ListEntity> list) {
        Collections.sort(list, new Comparator<MonthSaleProductEntity.SaleAllProductsEntity.ListEntity>() {
            @Override
            public int compare(MonthSaleProductEntity.SaleAllProductsEntity.ListEntity lhs, MonthSaleProductEntity.SaleAllProductsEntity.ListEntity rhs) {
                if (lhs != null
                        && rhs != null
                        && !TextUtils.isEmpty(lhs.amountRate)
                        && !TextUtils.isEmpty(rhs.amountRate)) {
                    return commonCompare(lhs.amountRate, rhs.amountRate);
                }
                return 0;
            }
        });
    }

    private void sortWGProSubList(List<MonthSaleProductEntity.SaleWGProductsEntity.ListEntity> list) {
        Collections.sort(list, new Comparator<MonthSaleProductEntity.SaleWGProductsEntity.ListEntity>() {
            @Override
            public int compare(MonthSaleProductEntity.SaleWGProductsEntity.ListEntity lhs, MonthSaleProductEntity.SaleWGProductsEntity.ListEntity rhs) {
                if (lhs != null
                        && rhs != null
                        && !TextUtils.isEmpty(lhs.amountRate)
                        && !TextUtils.isEmpty(rhs.amountRate)) {
                    return commonCompare(lhs.amountRate, rhs.amountRate);
                }
                return 0;
            }
        });
    }

    private void sortFlowProSubList(List<MonthSaleProductEntity.FlowWGProductsEntity.ListEntity> list) {
        Collections.sort(list, new Comparator<MonthSaleProductEntity.FlowWGProductsEntity.ListEntity>() {
            @Override
            public int compare(MonthSaleProductEntity.FlowWGProductsEntity.ListEntity lhs, MonthSaleProductEntity.FlowWGProductsEntity.ListEntity rhs) {
                if (lhs != null
                        && rhs != null
                        && !TextUtils.isEmpty(lhs.amountRate)
                        && !TextUtils.isEmpty(rhs.amountRate)) {
                    return commonCompare(lhs.amountRate, rhs.amountRate);
                }
                return 0;
            }
        });
    }

    private void sortXDYProSubList(List<MonthSaleProductEntity.SaleXDYProductsEntity.ListEntity> list) {
        Collections.sort(list, new Comparator<MonthSaleProductEntity.SaleXDYProductsEntity.ListEntity>() {
            @Override
            public int compare(MonthSaleProductEntity.SaleXDYProductsEntity.ListEntity lhs, MonthSaleProductEntity.SaleXDYProductsEntity.ListEntity rhs) {
                if (lhs != null
                        && rhs != null
                        && !TextUtils.isEmpty(lhs.amountRate)
                        && !TextUtils.isEmpty(rhs.amountRate)) {
                    return commonCompare(lhs.amountRate, rhs.amountRate);
                }
                return 0;
            }
        });
    }

    /**
     * 列表根据销售额占比降序排列
     *
     * @param list 请求后得到的列表数据
     */
    private void sortVipProList(List<MonthSaleProductEntity.VipSaleProductsEntity> list) {
        if (list != null) {
            Collections.sort(list, new Comparator<MonthSaleProductEntity.VipSaleProductsEntity>() {
                @Override
                public int compare(MonthSaleProductEntity.VipSaleProductsEntity lhs, MonthSaleProductEntity.VipSaleProductsEntity rhs) {
                    if (lhs != null
                            && rhs != null
                            && !TextUtils.isEmpty(lhs.amountRate)
                            && !TextUtils.isEmpty(rhs.amountRate)) {
                        return commonCompare(lhs.amountRate, rhs.amountRate);
                    }
                    return 0;
                }
            });
        }
    }
}
