package com.urovo.activity.display.presenter

import com.urovo.activity.display.model.DisplayReportDetailEntity
import com.urovo.activity.display.model.DisplayReportEntity
import com.urovo.activity.display.model.DisplayReportSubmitEntity
import com.urovo.activity.display.model.InspectorDisplayEntity
import com.urovo.wugumofang.base.BasePresenter
import com.urovo.wugumofang.base.mvp.AbsView


interface InspectorDisplayContract {

    interface IView : AbsView {
        /**
         * 查询门店陈列任务返回数据
         */
        fun missionByStoreNoData(data: InspectorDisplayEntity) {}

        /**
         * 新增、修改数据状态
         */
        fun submitionStatus(isSuccess: Boolean) {}

    }

    interface IPresenter : BasePresenter {
        /**
         * get /terminalDisCorrective/findTerminalDisShop
         * 查询终端陈列整改门店
         */
        fun requestFindTerminalDisShop(orgId: String?)

        /**
         * post /exhibit/queryMissionByStoreNo
         * 查询门店陈列任务
         */
        fun requestMissionByStoreNo()

        /**
         * post /exhibit/addSubmitio
         * 新增提报数据
         */
        fun requestAddSubmition(
            detailId: String?, remark: String?,
            submitUrl: String?
        )

        /**
         * post /exhibit/updateSubmition
         * 修改提报数据
         */
        fun requestUpdateSubmition(
            submitId: String?,
            remark: String?,
            submitUrl: String?
        )
    }
}

interface DisplayReportContract {

    interface IView : AbsView {
        /**
         * 查询货架陈列上报任务
         */
        fun missionByStaffIdData(data: List<DisplayReportEntity>) {}

        /**
         * 查询货架陈列上报任务明细
         */
        fun missionDetailData(data: DisplayReportDetailEntity){}

        /**
         * 提交状态
         */
        fun submitionStatus(isSuccess: Boolean) {}

    }

    interface IPresenter : BasePresenter {
        /**
         * POST /exhibit/queryMissionByStaffId
         * 查询货架陈列上报任务
         */
        fun requestMissionByStaffId()

        /**
         * POST /exhibit/queryMissionDetail
         * 查询货架陈列上报任务明细
         */
        fun requestMissionDetail(
            taskId: String?,
            shopId:String?
        )

        /**
         * POST /exhibit/submitMissionDetail
         * 提交货架陈列上报任务数据
         */
        fun requestSubmitMissionDetail(
            reportId: String?,
            auditId: String?,
            taskId: String?,
            shopId: String?,
            stockOut: String?,
            stockOutInfo: String?,
            excepSpec: String?,
            exportImages: List<String>?,
            wholeImages: List<String>?,
            answers: List<DisplayReportSubmitEntity.AnswersBean?>
        )
    }
}

