package com.urovo.activity.display.presenter

import com.urovo.activity.display.model.InspectorDisplayEntity
import com.urovo.activity.display.service.InspectorDisplayDepository
import com.urovo.wgmf.more.ent.MoreOnlineBean
import com.urovo.wugumofang.R
import com.urovo.wugumofang.base.mvp.AbsPresenter
import com.urovo.wugumofang.constants.AppConstants
import com.urovo.wugumofang.net.ApiException
import com.urovo.wugumofang.net.callback.RetrofitCallback
import com.urovo.wugumofang.system.SystemHelper
import com.urovo.wugumofang.utils.isEmptyParameter
import com.urovo.wugumofang.utils.isNullOrEmpty
import com.zsy.libcore.model.bean.RequestResult
import retrofit2.Call
import retrofit2.Callback
import retrofit2.Response

/**
 * 督察陈列检查 Presenter
 * @author LW
 */
class InspectorDisplayPresenter(
    private var rootView: InspectorDisplayContract.IView?,
    private var depository: InspectorDisplayDepository = InspectorDisplayDepository()
) : AbsPresenter(), InspectorDisplayContract.IPresenter {

    override fun requestFindTerminalDisShop(orgId: String?) {
        if (orgId.isNullOrEmpty()) {
            rootView?.showError(R.string.common_load_error)
            return
        }

        depository.requestFindTerminalDisShop(orgId).enqueue(object :
            RetrofitCallback<MoreOnlineBean>() {
            override fun onResponse(call: Call<MoreOnlineBean>, error: Response<MoreOnlineBean>) {
            }

            override fun onFailure(exception: ApiException) {
            }
        })

    }

    override fun requestMissionByStoreNo() {
        val storeNo = SystemHelper.LoginOrganId
        if (storeNo.isNullOrEmpty()) {
            rootView?.showError(R.string.common_load_error)
            return
        }

        rootView?.showIsLoading()
        depository.requestMissionByStoreNo(storeNo)
            .enqueue(object : RetrofitCallback<RequestResult<InspectorDisplayEntity>>() {
                override fun onResponse(
                    call: Call<RequestResult<InspectorDisplayEntity>>,
                    res: Response<RequestResult<InspectorDisplayEntity>>
                ) {
                    val body = res.body()
                    body?.let {
                        if (it.msgCode == AppConstants.Http.RESPONSE_SUCCESS) {
                            rootView?.missionByStoreNoData(it.resultInfo)
                        } else {
                            rootView?.showError(it.msgInfo)
                        }
                    }
                    rootView?.dismissLoading()
                }

                override fun onFailure(exception: ApiException) {
                    rootView?.dismissLoading()
                    rootView?.showError(exception.message)
                }
            })
    }

    override fun requestAddSubmition(
        detailId: String?,
        remark: String?,
        submitUrl: String?
    ) {
        val userNo = SystemHelper.LoginUserId
        val userName = SystemHelper.LoginUserName

        if (isEmptyParameter(detailId, userNo, userName, remark, submitUrl)) {
            rootView?.showError(R.string.common_load_error)
            return
        }

        rootView?.showTipDialog(R.string.inspector_display_add_data_tip)
        depository.requestAddSubmition(detailId!!, userNo, userName, remark, submitUrl)
            .enqueue(object : RetrofitCallback<RequestResult<String>>() {
                override fun onResponse(
                    call: Call<RequestResult<String>>,
                    res: Response<RequestResult<String>>
                ) {
                    val body = res.body()
                    body?.let {
                        if (it.msgCode == AppConstants.Http.RESPONSE_SUCCESS) {
                            rootView?.submitionStatus(true)
                        } else {
                            rootView?.submitionStatus(false)
                            rootView?.showToast(it.msgInfo)
                        }
                    }
                    rootView?.dismissDialog()

                }

                override fun onFailure(exception: ApiException) {
                    rootView?.dismissDialog()
                    rootView?.submitionStatus(false)
                    rootView?.showToast(exception.message)
                }
            })

    }

    override fun requestUpdateSubmition(
        submitId: String?,
        remark: String?,
        submitUrl: String?
    ) {
        val userNo = SystemHelper.LoginUserId
        val userName = SystemHelper.LoginUserName

        if (isEmptyParameter(submitId, userNo, userName, remark, submitUrl)) {
            rootView?.showToast(R.string.common_load_error)
            return
        }
        rootView?.showTipDialog(R.string.inspector_display_update_data_tip)
        depository.requestUpdateSubmition(submitId, userNo, userName, remark, submitUrl)
            .enqueue(object : RetrofitCallback<RequestResult<String>>() {
                override fun onResponse(
                    call: Call<RequestResult<String>>,
                    res: Response<RequestResult<String>>
                ) {
                    val body = res.body()
                    body?.let {
                        if (it.msgCode == AppConstants.Http.RESPONSE_SUCCESS) {
                            rootView?.submitionStatus(true)
                        } else {
                            rootView?.submitionStatus(false)
                            rootView?.showToast(it.msgInfo)
                        }
                    }
                    rootView?.dismissDialog()
                }

                override fun onFailure(exception: ApiException) {
                    rootView?.dismissDialog()
                    rootView?.submitionStatus(false)
                    rootView?.showToast(exception.message)
                }
            })
    }
}