package com.urovo.activity.display.service

import com.urovo.activity.display.model.InspectorDisplayEntity
import com.urovo.wgmf.more.ent.MoreContractBean
import com.urovo.wgmf.more.ent.MoreOnlineBean
import com.urovo.wugumofang.net.RetrofitHelper
import com.zsy.libcore.model.bean.RequestResult
import retrofit2.Call

/**
 * 督察陈列检查仓库
 * @author LW
 */
class InspectorDisplayDepository {

    private val mApiService by lazy {
        RetrofitHelper.createApi(ApiService::class.java)
    }

    fun requestFindTerminalDisShop(orgId: String?): Call<MoreOnlineBean> {
        return mApiService.findTerminalDisShop(orgId)
    }

    /**
     * 查询门店陈列任务
     * @param  storeNo
     */
    fun requestMissionByStoreNo(storeNo: String?): Call<RequestResult<InspectorDisplayEntity>> {
        return mApiService.queryMissionByStoreNo(storeNo)
    }

    /**
     * 新增提报数据
     * @param  detailId 门店任务id
     * @param  userNo 提交人工号
     * @param  userName 提交人名字
     * @param  remark 备注说明
     * @param  submitUrl 图片url,多张图片用,隔开
     */
    fun requestAddSubmition(
        detailId: String?,
        userNo: String?,
        userName: String?,
        remark: String?,
        submitUrl: String?
    ): Call<RequestResult<String>> {
        val paramsMap = mutableMapOf(
            "detailId" to detailId,
            "userNo" to userNo,
            "userName" to userName,
            "remark" to remark,
            "submitUrl" to submitUrl
        )
        return mApiService.addSubmition(paramsMap)
    }

    /**
     * 修改提报数据
     * @param submitId 提交编号
     * @param userNo  提交人工号
     * @param userName 提交人名字
     * @param remark  备注说明
     * @param submitUrl 图片url,多张图片用,隔开
     */
    fun requestUpdateSubmition(
        submitId: String?,
        userNo: String?,
        userName: String?,
        remark: String?,
        submitUrl: String?
    ): Call<RequestResult<String>> {
        val paramsMap = mutableMapOf(
            "submitId" to submitId,
            "userNo" to userNo,
            "userName" to userName,
            "remark" to remark,
            "submitUrl" to submitUrl
        )
        return mApiService.updateSubmition(paramsMap)
    }
}