package com.urovo.activity.display.view

import android.content.Intent
import android.text.Editable
import android.text.TextWatcher
import android.view.View
import android.view.ViewGroup
import androidx.core.content.ContextCompat
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.urovo.activity.display.model.DisplayReportEntity
import com.urovo.activity.display.model.DisplayReportGroup
import com.urovo.activity.display.model.TasksChild
import com.urovo.activity.display.presenter.DisplayReportContract
import com.urovo.activity.display.presenter.DisplayReportPresenter
import com.urovo.activity.display.view.adapter.DisplayReportAdapter
import com.urovo.wugumofang.R
import com.urovo.wugumofang.base.BaseActivity
import com.urovo.wugumofang.base.BasePresenter
import com.zsy.libcore.ui.ada.helper.DividerItemDecoration
import kotlinx.android.synthetic.main.public_list_view.*

/**
 * 陈列上报
 * @author LW
 */
class DisplayReportActivity : BaseActivity(), DisplayReportContract.IView,
    DisplayReportAdapter.OnClickListener {

    private var mDisplayReportPresenter: DisplayReportPresenter? = null

    private var mDisplayReportAdapter: DisplayReportAdapter? = null

    private val mDisplayReportList = mutableListOf<DisplayReportGroup>()

    private val mDisplayReportSearchList = mutableListOf<DisplayReportGroup>()

    private var mKeyWord: String = ""

    override fun addRootLayoutId(): Int = R.layout.public_list_view

    override fun rootViewGroup(): ViewGroup = mPublicContainer

    override fun initTitle() {
        super.initTitle()
        titleTextView.text = getString(R.string.display_report_title)
    }

    override fun initViewNew() {
        initTitle()

        mDisplayReportAdapter = DisplayReportAdapter(arrayListOf(), this)
        mPublicListViewLv.apply {
            layoutManager = LinearLayoutManager(mContext, RecyclerView.VERTICAL, false)
            addItemDecoration(
                DividerItemDecoration(
                    mContext,
                    DividerItemDecoration.VERTICAL_LIST,
                    1,
                    ContextCompat.getColor(mContext, R.color.pop_query_devider)
                )
            )
            adapter = mDisplayReportAdapter
        }
        mDisplayReportAdapter?.expandAll()

        mPublicSearchEt.visibility = View.VISIBLE
        mPublicSearchEt.setHint(R.string.display_report_search_tip)
        mPublicSearchEt.addTextChangedListener(object : TextWatcher {
            override fun beforeTextChanged(s: CharSequence?, start: Int, count: Int, after: Int) {

            }

            override fun onTextChanged(s: CharSequence?, start: Int, before: Int, count: Int) {

            }

            override fun afterTextChanged(s: Editable) {
                mKeyWord = mPublicSearchEt.text.toString().trim()
                filterSearchDataToList()
            }
        })
    }

    /**
     * 过滤关键数据
     */
    private fun filterSearchDataToList() {
        mDisplayReportSearchList.clear()
        if (mKeyWord.isEmpty()) {
            mDisplayReportSearchList.addAll(mDisplayReportList)
        } else {
            for (p in mDisplayReportList) {
                if (p.shopName.contains(mKeyWord) || p.shopName.contains(mKeyWord)) {
                    mDisplayReportSearchList.add(p)
                }
            }
        }
        mDisplayReportAdapter?.setNewData(mDisplayReportSearchList as List<DisplayReportGroup>)
    }

    override fun onResume() {
        super.onResume()
        mDisplayReportPresenter?.requestMissionByStaffId()
    }

    override fun addPresenter(): BasePresenter {
        mDisplayReportPresenter = DisplayReportPresenter(this)
        return mDisplayReportPresenter!!
    }

    override fun missionByStaffIdData(data: List<DisplayReportEntity>) {
        super.missionByStaffIdData(data)
        mDisplayReportList.clear()
        data.forEach {
            val group = DisplayReportGroup(it.shopId, it.shopName)
            it.tasks?.forEach {
                group.addSubItem(
                    TasksChild(
                        group.shopId,
                        group.shopName,
                        it.audit,
                        it.remark,
                        it.status,
                        it.taskId,
                        it.taskName,
                        it.type
                    )
                )
            }
            mDisplayReportList.add(group)
        }
        filterSearchDataToList()
    }

    override fun onGroupClick(view: View?, pos: Int, bean: DisplayReportGroup?,expanded:Boolean) {
        if(!expanded) {
            try {
                mPublicListViewLv.smoothScrollToPosition(pos + 1)
            } catch (e: Exception) {
                e.printStackTrace()
            }
        }
    }

    override fun onChildClick(view: View?, pos: Int, bean: TasksChild?) {
        if (bean != null) {
            jumpDisplayReportEditActivity(bean)
        }
    }

    /**
     * 跳转到 DisplayReportEditActivity
     */
    private fun jumpDisplayReportEditActivity(tasksChild: TasksChild) {
        val intent = Intent(this, DisplayReportEditActivity::class.java)
        intent.putExtra("tasksChild", tasksChild)
        startActivity(intent)
    }


}