package com.urovo.activity.display.view.adapter

import android.net.Uri
import android.view.View
import com.chad.library.adapter.base.BaseQuickAdapter
import com.chad.library.adapter.base.BaseViewHolder
import com.urovo.activity.display.model.DisplayReportDetailEntity
import com.urovo.wugumofang.R
import com.urovo.wugumofang.utils.isNotNullOrSize
import com.wgmf.trade.utils.ctx
import com.xp.pro.imagepickerlib.bean.ImageItem
import java.io.File
import com.xp.pro.imagepickerlib.widgets.ImagePickerLayout
import com.xp.pro.imagepickerlib.widgets.ImagePickerLayout.ImagePicker

/**
 * 编辑陈列上报适配器
 * @author LW
 */
class DisplayReportEditAdapter(
    entity: List<DisplayReportDetailEntity.Ema>,
    private val onPictureClickListener: OnPictureClickListener?
) : BaseQuickAdapter<DisplayReportDetailEntity.Ema, BaseViewHolder>(
    R.layout.list_item_display_report_view,
    entity
), ImagePicker {

    override fun convert(helper: BaseViewHolder?, item: DisplayReportDetailEntity.Ema?) {
        helper?.apply {
            item?.let {
                setText(
                    R.id.display_report_exception_desc_tv,
                    ctx.getString(R.string.display_report_exception_desc, it.excepSpec)
                )
                setText(
                    R.id.display_report_submit_time_tv,
                    ctx.getString(R.string.display_report_submit_time, it.submitTime)
                )
                var auditStatusStr = ""
                //0：未审核，1：已审核
                if (it.auditStatus == "1") {
                    auditStatusStr = String.format(
                        ctx.getString(R.string.display_report_audit_situation),
                        ctx.getString(R.string.display_report_audited)
                    )
                } else {
                    auditStatusStr = String.format(
                        ctx.getString(R.string.display_report_audit_situation),
                        ctx.getString(R.string.display_report_not_reviewed)
                    )
                }
                setText(R.id.display_report_audit_situation_tv, auditStatusStr)

                setText(
                    R.id.display_report_review_notes_tv,
                    ctx.getString(R.string.display_report_review_notes, it.auditRemark)
                )

                val wholeImageList = arrayList(it.wholeImages ?: arrayListOf())
                getView<ImagePickerLayout>(R.id.display_whole_image_picker_container).apply {
                    setmImageselectList(wholeImageList)
                    title = "整体照片:"
                    setTipVisibility(View.GONE)
                    isOlnyViewMode = true
                    imagePicker = this@DisplayReportEditAdapter
                    maxPhotoNum = 15
                    sizePhotoNum = 3
                    imagePickerView = this
                    refreshPhotoContentView(wholeImageList)
                    tag = layoutPosition
                }

                val exportImageList = arrayList(it.exportImages ?: arrayListOf())
                getView<ImagePickerLayout>(R.id.display_export_image_picker_container).apply {
                    setmImageselectList(exportImageList)
                    title = "导出照片:"
                    setTipVisibility(View.GONE)
                    isOlnyViewMode = true
                    imagePicker = this@DisplayReportEditAdapter
                    maxPhotoNum = 15
                    sizePhotoNum = 3
                    imagePickerView = this
                    refreshPhotoContentView(exportImageList)
                    tag = layoutPosition
                }
            }
        }
    }

    private fun arrayList(imgs: List<String>): ArrayList<ImageItem> {
        val list = arrayListOf<ImageItem>()
        if (imgs.isNotNullOrSize()) {
            for (i in imgs.indices) {
                val item = ImageItem()
                item.setImgneturl(imgs[i])
                item.setImagePath(imgs[i])
                item.setImageId(i.toString() + "")
                item.setUri(Uri.fromFile(File(imgs[i])))
                item.setType(1)
                list.add(item)
            }
        }
        return list
    }

    override fun setSelectDialog(selectType: Int, view: View) {
        onPictureClickListener?.setItemSelectDialog(selectType, view)
    }

    override fun toPhotoPreview(i: Int, arrayList: ArrayList<ImageItem>) {
        onPictureClickListener?.toItemPhotoPreview(i, arrayList)
    }

    interface OnPictureClickListener {
        fun setItemSelectDialog(selectType: Int, view: View?)
        fun toItemPhotoPreview(i: Int, arrayList: ArrayList<ImageItem>?)
    }

}