package com.urovo.activity.goodsmanager

import android.content.Intent
import android.graphics.Color
import android.os.Bundle
import com.urovo.activity.goodsmanager.service.OrderGoodsRepository
import com.urovo.activity.goodsmanager.viewmodel.OrderGoodsFactory
import com.urovo.activity.goodsmanager.viewmodel.OrderGoodsViewModel
import com.urovo.wugumofang.base.BaseActivity
import androidx.lifecycle.ViewModelProviders
import com.urovo.wugumofang.R
import java.util.*
import kotlin.collections.ArrayList
import androidx.lifecycle.Observer
import androidx.recyclerview.widget.LinearLayoutManager
import com.urovo.wugumofang.ui.ViewUtil
import kotlinx.android.synthetic.main.ordergoods_list_act.*
import com.urovo.wugumofang.view.DividerItemDecoration
import com.wgmf.trade.ui.fra.index.OrderGoodsAdapter

/**
 * 店订货/查询
 */
class OrderGoodsTabAct : BaseActivity() {

    private var mOrderTitleList: List<String> = ArrayList(Arrays.asList("订货提交", "订货查询"))

    private val mOrderGoodsViewModel by lazy {
        ViewModelProviders.of(this, OrderGoodsFactory(OrderGoodsRepository(), application)).get(OrderGoodsViewModel::class.java)
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        bindViewModel()
        setContentView(R.layout.ordergoods_list_act)
        initTitle()
        initView()
        //门店订货周期变更提醒
        mOrderGoodsViewModel.queryOrderGoodsTip(this)
    }

    private fun initView() {
        titleTextView.text = "门店订货"
        val orderGoodsAdapter = OrderGoodsAdapter(mOrderTitleList)
        orderGoodsEntryRv.run {
            layoutManager = LinearLayoutManager(this@OrderGoodsTabAct)
            addItemDecoration(DividerItemDecoration(this@OrderGoodsTabAct, DividerItemDecoration.HORIZONTAL_LIST,
                    1, Color.parseColor("#E3E7EE")))
            adapter = orderGoodsAdapter
        }
        orderGoodsAdapter.setOnItemClickListener { adapter, view, position ->
            val item = adapter.getItem(position)
            item?.let {
                when (item) {
                    mOrderTitleList[0] -> {
                        startActivity(Intent(this@OrderGoodsTabAct,
                                OrderGoodsSubmitAct::class.java))
                    }
                    else -> {
                        startActivity(Intent(this@OrderGoodsTabAct,
                                OrderGoodsQueryAct::class.java))
                    }
                }
            }
        }
    }

    private fun bindViewModel() {
        mOrderGoodsViewModel.progressBarText.observe(this, Observer {
            if (it.isNullOrBlank()) {
                ViewUtil.dismissProgress()
            } else {
                ViewUtil.showProgress(this@OrderGoodsTabAct, "请稍等", it)
            }
        })
    }
}
