package com.urovo.activity.goodsmanager

import android.view.View
import android.view.ViewGroup
import androidx.core.content.ContextCompat
import com.urovo.activity.goodsmanager.adapter.transfer.TransferDetailTableAdapter
import com.urovo.activity.goodsmanager.model.StoreAllotEntity
import com.urovo.activity.goodsmanager.model.StoreAllotUpdateEntity
import com.urovo.activity.goodsmanager.presenter.TransferContract
import com.urovo.activity.goodsmanager.presenter.TransferPresenter
import com.urovo.wugumofang.R
import com.urovo.wugumofang.base.BaseActivity
import com.urovo.wugumofang.base.BasePresenter
import com.urovo.wugumofang.constants.AppConstants
import com.urovo.wugumofang.constants.ConstantParams
import com.urovo.wugumofang.data.StringUtils
import com.urovo.wugumofang.system.SystemHelper
import com.urovo.wugumofang.ui.InputMethodManagerTool
import com.urovo.wugumofang.utils.convertDefaultValue
import com.urovo.wugumofang.utils.isNotNullOrSize
import com.urovo.wugumofang.utils.notNullAndEmpty
import com.urovo.wugumofang.view.DividerItemDecoration
import com.urovo.wugumofang.view.LoadWaitDominator
import com.urovo.wugumofang.view.dialog.PublicEditDialog
import com.urovo.wugumofang.view.scrollablePanel.PanelAdapter
import com.urovo.wugumofang.view.scrollablePanel.ScrollablePanelManager
import kotlinx.android.synthetic.main.activity_transfer_detail.*

/**
 * 调拨明细
 * @author LW
 */
class TransferDetailActivity : BaseActivity(), TransferContract.IView,
    ScrollablePanelManager.PanelLineAdapter.OnContentItemClickListener {

    private var mTransferPresenter: TransferPresenter? = null

    private var mTransferDetailTableAdapter: TransferDetailTableAdapter? = null

    private var mScrollablePanelManager: ScrollablePanelManager? = null

    private var mIsEditStatus = false

    override fun rootViewGroup(): ViewGroup = root_container_ll

    private val mStoreAllotEntity: StoreAllotEntity? by lazy {
        intent.getSerializableExtra(AppConstants.ExtraKeys.STORE_ALLOT_ENTITY) as StoreAllotEntity?
    }

    private val mStatus: Int? by lazy {
        intent.getIntExtra(AppConstants.ExtraKeys.TRANSFER_QUERY_STATUS, 1) as Int?
    }

    override fun addRootLayoutId(): Int = R.layout.activity_transfer_detail

    override fun initTitle() {
        super.initTitle()
        titleTextView.text = getString(R.string.transfer_detail_title)
    }

    override fun initViewNew() {
        initTitle()
        setStatusBackgroundColor(R.color.normal_bg)
        mStoreAllotEntity?.apply {
            //isSh是否收货(0未收货，1收货，2拒收，3取回）
            if (isSh == ConstantParams.CONSTANT_STARTS_1 || isSh == ConstantParams.CONSTANT_STARTS_2 ||
                isSh == ConstantParams.CONSTANT_STARTS_3
            ) { //是否收货
                if (isSh == ConstantParams.CONSTANT_STARTS_1) {
                    transfer_query_order_status_tv.setText(R.string.transfer_query_received)
                    transfer_query_order_status_iv.setBackgroundResource(R.mipmap.icon_complete_receive)
                    transfer_query_reason_tv.visibility = View.GONE
                } else if (isSh == ConstantParams.CONSTANT_STARTS_2) {
                    transfer_query_order_status_tv.setText(R.string.transfer_query_refusal)
                    transfer_query_order_status_iv.setBackgroundResource(R.mipmap.icon_reject)
                    if (dbReason.notNullAndEmpty()) {
                        transfer_query_reason_tv.visibility = View.VISIBLE
                        transfer_query_reason_tv.text =
                            StringUtils.formatString(
                                mContext.getString(R.string.transfer_query_reason),
                                dbReason
                            )
                    } else {
                        transfer_query_reason_tv.visibility = View.GONE
                    }
                } else if (isSh == ConstantParams.CONSTANT_STARTS_3) {
                    transfer_query_order_status_tv.setText(R.string.transfer_query_retrieved)
                    transfer_query_order_status_iv.setBackgroundResource(R.mipmap.icon_reject)
                    transfer_query_reason_tv.visibility = View.GONE
                }

                transfer_query_order_status_msg_tv.visibility = View.GONE
                transfer_query_operate_ll.visibility = View.GONE

            } else {
                transfer_query_order_status_iv.setBackgroundResource(R.mipmap.icon_waiting_time)
                transfer_query_order_status_tv.setText(R.string.transfer_query_await_receipt)
                transfer_query_order_status_msg_tv.visibility = View.VISIBLE
                transfer_query_order_status_msg_tv.setText(R.string.transfer_query_submitted_wait)

                if (mStatus == TransferQueryActivity.PRO_IN && allotUserCode != SystemHelper.LoginUserId) {
                    transfer_query_operate_ll.visibility = View.VISIBLE
                } else {
                    transfer_query_operate_ll.visibility = View.GONE
                }

                if (dbReason.notNullAndEmpty()) {
                    transfer_query_reason_tv.visibility = View.VISIBLE
                    transfer_query_reason_tv.text =
                        StringUtils.formatString(
                            mContext.getString(R.string.transfer_query_reason),
                            dbReason
                        )
                } else {
                    transfer_query_reason_tv.visibility = View.GONE
                }
            }

            transfer_query_order_number_tv.text = StringUtils.formatString(
                mContext.getString(R.string.transfer_query_order_number),
                allotmentId
            )

            transfer_query_distance_tv.text = StringUtils.formatString(
                mContext.getString(R.string.transfer_query_distance), distance.toString()
            )

            transfer_query_order_date_tv.text = allotDate
            transfer_query_out_store_tv.text = allotOutStore
            transfer_query_in_store_tv.text = allotInStore

            transfer_query_prepared_by_tv.text = StringUtils.formatString(
                mContext.getString(R.string.transfer_query_prepared_by),
                allotUserName.convertDefaultValue()
            )
            transfer_query_item_total_price_tv.text =
                StringUtils.formatString(
                    mContext.getString(R.string.transfer_query_total_price_number),
                    amount.toString().convertDefaultValue()
                )

        }
        //拒收
        transfer_query_rejection_btn.setOnClickListener {
            showRejectionEdtDialog()
        }

        transfer_query_cancel_btn.setOnClickListener {
            if (mIsEditStatus) {  //保存修改
                transfer_query_confrin_btn.visibility = View.VISIBLE
                transfer_query_rejection_btn.visibility = View.VISIBLE
                transfer_query_cancel_btn.text =
                    mContext.getString(R.string.transfer_query_update_number)
                mIsEditStatus = false
                saveChangeLogic()
            } else {   //修改数量
                transfer_query_confrin_btn.visibility = View.GONE
                transfer_query_rejection_btn.visibility = View.GONE
                transfer_query_cancel_btn.text =
                    mContext.getString(R.string.transfer_query_save_change)
                mIsEditStatus = true
            }

            InputMethodManagerTool.getInstance(mContext).hideOrShowInputMethod()
            val products = mStoreAllotEntity?.products
            if (products.isNotNullOrSize()) {
                dismissAllState()
                bindTableAdapterData(products!!)
            } else {
                showState(LoadWaitDominator.NO_DATA)
            }
            //mScrollablePanelManager?.notifyDataSetChanged()
        }
        //确认收货
        transfer_query_confrin_btn.setOnClickListener {
            confirmReceiptLogic()
        }

        val products = mStoreAllotEntity?.products
        if (products.isNotNullOrSize()) {
            dismissAllState()
            bindTableAdapterData(products!!)
        } else {
            showState(LoadWaitDominator.NO_DATA)
        }
    }

    /**
     * 输入调拨原因
     */
    private fun showRejectionEdtDialog() {
        val builder = PublicEditDialog.Builder(mContext)
        builder.apply {
            setTitle(R.string.transfer_query_dialog_rejection_reason)
            setEditHint(R.string.transfer_query_dialog_input_rejection_reason)
            setPositiveButton(R.string.cancel_btn_txt) { dialog, _ -> dialog.dismiss() }
            setNegativeButton(R.string.confirm_btn_txt) { dialog, _ ->
                val reason = builder.editContent
                rejectionLogic(reason)
                dialog.dismiss()
            }
        }.create().show()
    }

    /**
     * 保存修改
     */
    private fun saveChangeLogic() {
        val dbDetails = mutableListOf<StoreAllotUpdateEntity>()
        mStoreAllotEntity?.let {
            val products = it.products
            //updateType(1取回状态修改调拨数量 2修改收货数量)
            var updateType = ConstantParams.CONSTANT_STARTS_2
            if (it.isSh == ConstantParams.CONSTANT_STARTS_3) {
                updateType = ConstantParams.CONSTANT_STARTS_1
            }

            products?.forEach { item ->
                dbDetails.add(
                    StoreAllotUpdateEntity(
                        it.dbBillCode,
                        it.allotmentId,
                        item.proCode,
                        item.editProQty ?: item.proQty,
                        SystemHelper.LoginUserId,
                        updateType,
                        item.proDate
                    )
                )
            }
        }
        mTransferPresenter?.requestUpdateStoreAllot(dbDetails)
    }

    /**
     * 拒收
     */
    private fun rejectionLogic(reason: String?) {
        mStoreAllotEntity?.let {
            mTransferPresenter?.requestConfirmStoreAllot(
                it.dbBillCode, it.allotmentId.toString(),
                SystemHelper.LoginUserId, reason, ConstantParams.CONSTANT_STARTS_2
            )
        }
    }


    /**
     * 确认收货
     */
    private fun confirmReceiptLogic() {
        mStoreAllotEntity?.let {
            mTransferPresenter?.requestConfirmStoreAllot(
                it.dbBillCode, it.allotmentId.toString(),
                SystemHelper.LoginUserId, "", ConstantParams.CONSTANT_STARTS_1
            )
        }
    }

    /**
     * 绑定表格数据
     */
    private fun bindTableAdapterData(products: List<StoreAllotEntity.ProductBean>) {
        mTransferDetailTableAdapter =
            TransferDetailTableAdapter(mContext)

        transfer_query_recycler_content_list.addItemDecoration(
            DividerItemDecoration(
                mContext,
                DividerItemDecoration.HORIZONTAL_LIST,
                1,
                ContextCompat.getColor(mContext, R.color.color_E8E8E8)
            )
        )
        mTransferDetailTableAdapter?.setTableTitleList(
            mContext.resources.getStringArray(R.array.transfer_detail_list_title)
                .asList()
                .toMutableList()
        )
        mTransferDetailTableAdapter?.setProductList(products as MutableList<StoreAllotEntity.ProductBean>)
        mTransferDetailTableAdapter?.setEditStatus(mIsEditStatus)
        mScrollablePanelManager = ScrollablePanelManager()
        mScrollablePanelManager?.setPanelView(
            mContext,
            transfer_query_recycler_content_list,
            transfer_query_recycler_header_list,
            transfer_query_first_item
        )
        mTransferDetailTableAdapter?.setScrollablePanelManager(mScrollablePanelManager)
        mScrollablePanelManager?.setPanelAdapter(
            mTransferDetailTableAdapter,
            this@TransferDetailActivity
        )
    }

    override fun initDataNew() {
    }

    override fun addPresenter(): BasePresenter {
        mTransferPresenter = TransferPresenter(this)
        return mTransferPresenter!!
    }

    override fun OnItemClick(position: Int, panelAdapter: PanelAdapter?) {

    }

    override fun updateStoreAllotStatus(status: Boolean) {
        super.updateStoreAllotStatus(status)
    }

    override fun confirmStoreAllotStatus(status: Boolean) {
        if (status) {
            this.finish()
        }
    }
}