package com.urovo.activity.goodsmanager.adapter

import android.content.Context
import android.graphics.Color
import android.graphics.Typeface
import android.graphics.drawable.Drawable
import android.os.Handler
import android.text.SpannableString
import android.text.Spanned
import android.text.TextUtils
import android.text.method.LinkMovementMethod
import android.text.style.*
import android.view.*
import android.widget.*
import androidx.appcompat.app.AppCompatActivity
import androidx.recyclerview.widget.RecyclerView
import com.squareup.picasso.Picasso
import com.urovo.wgmf.db.dao.dao.Cpbill002DbOperate
import com.urovo.wgmf.db.dao.entitys.DingHuoOplQtyEntity
import com.urovo.wugumofang.R
import com.urovo.wugumofang.constants.AppConstants
import com.urovo.wugumofang.data.Arith
import com.urovo.wugumofang.data.StringUtils
import com.urovo.wugumofang.system.BaseAppManager
import com.urovo.wugumofang.ui.DialogHelper
import com.urovo.wugumofang.view.NumberKeyboard
import com.wgmf.trade.GlobalApp
import kotlinx.android.synthetic.main.wechat_qrcode_act.*
import org.jetbrains.anko.find


/**
 * @author Sasuke on 2018/9/24.
 */
class OrderGoodsTradeAdapter(
    val context: Context, dataList: List<DingHuoOplQtyEntity.DinghuoDetail>?,
    val notifyPriceShow: () -> Unit
) : RecyclerView.Adapter<OrderGoodsTradeAdapter.OrderGoodsHolder>() {

    /**
     * 订单列表
     */
    val orderList: MutableList<DingHuoOplQtyEntity.DinghuoDetail> = mutableListOf()


    val handler = Handler()

    /**
     * 为了用外层的最大库存天数字段，专门把整个entity传进来缓存起来。
     */
    var entityCache: DingHuoOplQtyEntity.ResultInfo? = null

    /**
     * 产品类型
     */
    var productType: Int = AppConstants.PackType.FINISHED_PRODUCT_TYPE

    /**
     * 品牌类型
     */
    var brandType: Int = AppConstants.BrandType.WGMF_BRAND_TYPE

    init {
        dataList?.let {
            orderList.clear()
            orderList.addAll(dataList)
        }
    }

    val keyboard: NumberKeyboard by lazy {
        NumberKeyboard.newInstance(null)
    }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): OrderGoodsHolder {
        return OrderGoodsHolder(
            LayoutInflater.from(context)
                .inflate(R.layout.second_ordergoodsmenu_item4, parent, false)
        )
    }

    override fun getItemCount(): Int {
        return orderList.size
    }

    override fun onBindViewHolder(holder: OrderGoodsHolder, position: Int) {
    }

    override fun onBindViewHolder(holder: OrderGoodsHolder, pos: Int, payloads: List<Any?>) {
        bindContentViewHolder(holder, pos, payloads)
    }

    private fun bindContentViewHolder(holder: OrderGoodsHolder, pos: Int, payloads: List<Any?>) {
        val data = orderList[pos]
        val isDecimal = Cpbill002DbOperate.isDecimalById(data.itemid)
        holder.apply {
            payloads.takeIf { it.isNotEmpty() && it[0] == "refrshImage" }?.let {
                name.text = refreshOrderName(data)
                return
            }
            count.tag = pos
            calculatorCyl(data, data.tqty)//计算库存率
            if (isDecimal) {
                //库存在途
                tvKucun.text = data.stock_Qty
                //建议订货量
                tvQuantityOrder.text = data.opl_qty

                if (productType == AppConstants.PackType.ACCESSORIES_SUPPLIERS) {
                    oeProDhQty.text = data.oe_pro_dh_qty
                } else {
                    oeProDhQty.text = data.lastMonth_qty
                }

                submitCount.apply {
                    visibility = View.VISIBLE
                    text = String.format("已提交 : %s", data.pro_today_dhqty)
                }
            } else {
                //库存在途
                tvKucun.text = StringUtils.formatDouble(data.stock_Qty.toDouble(), "0")
                //建议订货量
                tvQuantityOrder.text = StringUtils.formatDouble(data.opl_qty.toDouble(), "0")

                if (productType == AppConstants.PackType.ACCESSORIES_SUPPLIERS) {
                    oeProDhQty.text = StringUtils.formatDouble(data.oe_pro_dh_qty.toDouble(), "0")
                } else {
                    oeProDhQty.text = StringUtils.formatDouble(data.lastMonth_qty.toDouble(), "0")
                }

                submitCount.apply {
                    visibility = View.VISIBLE
                    text = String.format(
                        "已提交 : %s",
                        StringUtils.formatDouble(data.pro_today_dhqty.toDouble(), "0")
                    )
                }
            }

            //库存天数
            if (brandType == AppConstants.BrandType.WGMF_BRAND_TYPE &&
                productType == AppConstants.PackType.ACCESSORIES_SUPPLIERS
            ) {
                tvCyl.text = "--"
            } else {
                tvCyl.text =data.cyl.toInt().toString()
            }

            name.text = refreshOrderName(data)

            name.movementMethod = LinkMovementMethod.getInstance()

            unit.text = String.format("%s(%s)", data.itemUOM, data.itemid)
            unitPrice.text = String.format("单价:%s", data.itemPrice)
            count.setTextColor(if (data.isInvalid) Color.RED else Color.BLACK)

            if (data.isStockout) {
                count.setText(R.string.factory_out_stock)
                count.setTextColor(Color.RED)
                count.setBackgroundResource(R.drawable.shape_transparent)
                reduce.visibility = View.INVISIBLE
                add.visibility = View.INVISIBLE
            } else {
                count.setTextColor(Color.BLACK)
                count.setBackgroundResource(R.drawable.shape_f5f5f5_round_3)
                reduce.visibility = View.VISIBLE
                add.visibility = View.VISIBLE
                if (isDecimal) {
                    count.text = StringUtils.formatDouble(data.tqty, "0.00")
                } else {
                    count.text = StringUtils.formatDouble(data.tqty, "0")
                }
            }

            //减号逻辑===============
            if (data.tqty == 0.0) {
                if (data.sy_opl_qty == 0.0) {//如果订货最大数量为0，那为了在点击添加时弹出提示，所以不能把按钮enable设为false
                    reduce.isEnabled = true
                    reduce.setImageResource(R.mipmap.btn_goods_mgt_order_subtract)//虽然是可点击的，但还是要设为灰色。
                } else {
                    reduce.isEnabled = false
                    reduce.setImageResource(R.mipmap.btn_goods_mgt_order_subtract)
                }
            } else {
                reduce.isEnabled = true
                reduce.setImageResource(R.mipmap.btn_goods_mgt_order_subtract)
            }
            //加号逻辑===============
            if (data.tqty >= data.sy_opl_qty) {

                if (data.sy_opl_qty == 0.0) {//如果剩余订货量为0，那为了在点击添加时弹出提示，所以不能把按钮enable设为false
                    add.isEnabled = true
                    //虽然是可点击的，但还是要设为灰色。
                    add.setImageResource(R.mipmap.btn_goods_mgt_order_add)
                } else {
                    add.isEnabled = false
                    add.setImageResource(R.mipmap.btn_goods_mgt_order_add)
                }
            } else {
                add.isEnabled = true
                add.setImageResource(R.mipmap.btn_goods_mgt_order_add)
            }


            //通过Picasso先判断图片链接可用性，如果无效链接，则不显示“查看图片”按钮
            if (!data.isShowImageTv) {//只有在标记为false的时候，才去检查链接可用性。
                Picasso.with(context).load(orderList[pos].picUrl)
                    .into(img, object : com.squareup.picasso.Callback {
                        override fun onSuccess() {
                            if (pos >= orderList.size) return//防溢出
                            orderList[pos].isShowImageTv = true
//                                if (this@OrderGoodsTradeAdapter.)
                            //只刷新这个组件
                            handler.post { notifyItemChanged(pos, "refrshImage") }
                        }

                        override fun onError() {
                            if (pos >= orderList.size) return//防溢出
                            orderList[pos].isShowImageTv = false
                            // notifyItemChanged(pos)
                        }
                    })
            }

            //加号点击监听================================
            add.setOnClickListener {
                val cell = orderList[pos]
                if (cell.sy_opl_qty == 0.0) {//剩余订货量为0时，加号是可以点击的，但只起到弹出提示的作用。
                    DialogHelper.Alert(context, "提示",  cell.getRemark(), null) { dialog, which ->
                        dialog.dismiss()
                    }
                    return@setOnClickListener
                }
                var d = 0.0
                //【if】getPack==0情况下默认加一；
                //【else】getPack>0情况下以Box_qty为标准
                cell.dhPack = if (cell.getPack() == 0.0) 1.0 else cell.pack

                d = Arith.add(cell.getTQTY(), cell.getDHPack())
                //如果再加就超过最大值了，那就拦截点击后面的操作
                if (d > cell.sy_opl_qty) {
                    val content: String =
                        "不能超过最大订货量 " + cell.sy_opl_qty + "！\n" + if (cell.accessory) "" else ("或不能超过${
                            StringUtils.formatDouble(
                                entityCache?.maxStockLimit
                                    ?: 0.0, "0.##"
                            )
                        }天库存天数！")

                    DialogHelper.Alert(context, "提示", content, null) { dialog, which ->
                        dialog.dismiss()
                    }
                    return@setOnClickListener
                }
                cell.tqty = d
                cell.amount = Arith.mul(d, cell.itemPrice)
                if (isDecimal) {
                    count.text = StringUtils.formatDouble(d, "0.00")
                } else {
                    count.text = StringUtils.formatDouble(d, "0")
                }
                calculatorCyl(cell, d)
                if (brandType == AppConstants.BrandType.WGMF_BRAND_TYPE &&
                    productType == AppConstants.PackType.ACCESSORIES_SUPPLIERS
                ) {
                    tvCyl.text = "--"
                } else {
                    tvCyl.text = data.cyl.toInt().toString()
                }
                if (d < 0) {
                    cell.isInvalid = true
                    count.setTextColor(Color.RED)
                    reduce.isEnabled = false
                } else {
                    if (0.0 != cell.getDHPack() && 0.0 != d && d % cell.getDHPack() != 0.0) {
                        cell.isInvalid = true
                        count.setTextColor(Color.RED)
                    } else {
                        cell.isInvalid = false
                        count.setTextColor(Color.BLACK)
                    }
                    reduce.isEnabled = true
                }
                notifyPriceShow.invoke()
            }

            //减号点击监听================================
            reduce.setOnClickListener {
                val cell = orderList[pos]
                if (cell.sy_opl_qty == 0.0) {//最大订货量为0时，减号是可以点击的，但只起到弹出提示的作用。
                    DialogHelper.Alert(context, "提示",  cell.getRemark(), null) { dialog, which ->
                        dialog.dismiss()
                    }
                    return@setOnClickListener
                }
                var d = 0.0
                //【if】getPack==0情况下默认加一；
                //【else】getPack>0情况下以getPack为标准
                cell.dhPack = if (cell.getPack() == 0.0) 1.0 else cell.pack
                d = Arith.sub(cell.tqty, cell.dhPack)
                cell.tqty = d
                cell.amount = Arith.mul(d, cell.itemPrice)
                if (isDecimal) {
                    count.text = StringUtils.formatDouble(d, "0.00")
                } else {
                    count.text = StringUtils.formatDouble(d, "0")
                }
                if (d <= 0) {
                    reduce.isEnabled = false
                }
                calculatorCyl(cell, d)
                if (brandType == AppConstants.BrandType.WGMF_BRAND_TYPE &&
                    productType == AppConstants.PackType.ACCESSORIES_SUPPLIERS
                ) {
                    tvCyl.text = "--"
                } else {
                    tvCyl.text = data.cyl.toInt().toString()
                }
                if (d < 0) {
                    cell.isInvalid = true
                    count.setTextColor(Color.RED)
                    add.isEnabled = true
                } else if (d > cell.sy_opl_qty) {
                    cell.isInvalid = true
                    count.setTextColor(Color.RED)
                    add.isEnabled = false
                } else {
                    if (0.0 != cell.getDHPack() && 0.0 != d && d % cell.getDHPack() != 0.0) {
                        cell.isInvalid = true
                        count.setTextColor(Color.RED)
                    } else {
                        cell.isInvalid = false
                        count.setTextColor(Color.BLACK)
                    }
                    add.isEnabled = true
                }
                notifyPriceShow.invoke()
            }

            //数量点击监听================================
            if (!data.isStockout) {
                count.setSelectAllOnFocus(true)
                count.setOnClickListener {
                    keyboard.show(
                        count,
                        (context as AppCompatActivity).supportFragmentManager,
                        null
                    )
                        .setDecimal(2)
                        .selectAll().setOnDismissListener {
                            if (orderList.size > pos) {
                                val cell = orderList[pos]
                                if (cell.sy_opl_qty == 0.0) {
                                    DialogHelper.Alert(context, "提示",  cell.getRemark(), null) { dialog, which ->
                                        dialog.dismiss()
                                    }
                                    if (isDecimal) {
                                        count.text = StringUtils.formatDouble(
                                            0.0,
                                            "0.00"
                                        )//输错了要自动还原为0。
                                    } else {
                                        count.text = StringUtils.formatDouble(0.0, "0")
                                    }

                                    return@setOnDismissListener
                                }
                                cell.dhPack = cell.getPack()
                                val input = count.text.toString()
                                var d = 0.0
                                try {
                                    d = input.toDouble()
                                } catch (e: NumberFormatException) {
                                    d = 0.0
                                }
                                cell.tqty = d
                                if (isDecimal) {
                                    count.text = StringUtils.formatDouble(d, "0.00")
                                } else {
                                    count.text = StringUtils.formatDouble(d, "0")
                                }

                                calculatorCyl(cell, d)
                                if (brandType == AppConstants.BrandType.WGMF_BRAND_TYPE &&
                                    productType == AppConstants.PackType.ACCESSORIES_SUPPLIERS
                                ) {
                                    tvCyl.text = "--"
                                } else {
                                    tvCyl.text = data.cyl.toInt().toString()
                                }

                                var temp = ""
                                if (d <= 0) {
                                    if (d < 0) {
                                        temp = "必须输入有效的数字！"
                                    }
                                    reduce.isEnabled = false
                                }
                                if (d >= cell.sy_opl_qty) {
                                    if (d > cell.sy_opl_qty) {
                                        temp = ("不能超过最大订货量 " + cell.sy_opl_qty + "！\n"
                                                + if (cell.accessory) "" else ("或不能超过${
                                            StringUtils.formatDouble(
                                                entityCache?.maxStockLimit
                                                    ?: 0.0, "0.##"
                                            )
                                        }天库存天数！"))
                                    }
                                    add.isEnabled = false
                                }

                                if (0.0 != cell.dhPack && 0.0 != d) {
                                    if (d % cell.dhPack != 0.0) {
                                        temp = "数量必须是" + cell.dhPack + "倍数"
                                        data.tqty = data.tqty.toInt() / cell.dhPack
                                            .toInt() * cell.dhPack// 非法规格输入调整
                                        d = data.tqty
                                    }
                                    cell.amount =
                                        Arith.mul(d, cell.itemPrice)//价格一定要放在最后来算，因为前面的校验会改数量

                                    if (!TextUtils.isEmpty(temp)) {
                                        DialogHelper.Alert(context, "提示", temp, null) { dialog, which ->
                                            dialog.dismiss()
                                        }
                                        cell.isInvalid = true
                                        count.setTextColor(Color.RED)
                                    } else {
                                        cell.isInvalid = false
                                        add.isEnabled = true
                                        reduce.isEnabled = true
                                        reduce.setImageResource(R.mipmap.btn_goods_mgt_order_subtract)
                                        add.setImageResource(R.mipmap.btn_goods_mgt_order_add)
                                        count.setTextColor(Color.BLACK)
                                    }
                                    notifyPriceShow.invoke()
                                }
                            }
                        }
                }
            } else {
                count.setSelectAllOnFocus(false)
                count.setOnClickListener(null)
            }
        }
    }

    class OrderGoodsHolder(view: View) : RecyclerView.ViewHolder(view) {
        var name: TextView = view.find(R.id.goods_name) as TextView
        var img: ImageView = view.find(R.id.goods_img) as ImageView
        var unit: TextView = view.find(R.id.goods_unit) as TextView
        var unitPrice: TextView = view.find(R.id.goods_unit_price) as TextView

        var tvKucun: TextView = view.find(R.id.tv_kucun) as TextView
        var tvQuantityOrder: TextView = view.find(R.id.tv_quantity_order) as TextView
        var oeProDhQty: TextView = view.find(R.id.oe_pro_dh_qty) as TextView
        var tvCyl: TextView = view.find(R.id.tv_cyl) as TextView

        var count: TextView = view.find(R.id.goods_count) as TextView
        var submitCount: TextView = view.find(R.id.goods_submit_count) as TextView
        var add: ImageButton = view.find(R.id.goods_plus) as ImageButton
        var reduce: ImageButton = view.find(R.id.goods_decrease) as ImageButton

    }

    private fun refreshOrderName(entity: DingHuoOplQtyEntity.DinghuoDetail): SpannableString {
        return if (entity.isShowImageTv) {
            SpannableString(entity.itemName + "img").apply {
                setSpan(
                    UnderlineSpan(),
                    0,
                    entity.itemName.length,
                    Spanned.SPAN_EXCLUSIVE_EXCLUSIVE
                )

                val drawable: Drawable =
                    context.getResources().getDrawable(R.mipmap.ic_picture_order)
                drawable.setBounds(0, 0, drawable.intrinsicWidth, drawable.intrinsicHeight)
                setSpan(
                    ImageSpan(drawable),
                    this.length - 3,
                    this.length,
                    Spanned.SPAN_INCLUSIVE_EXCLUSIVE
                )

                setSpan(
                    StyleSpan(Typeface.BOLD),
                    0,
                    entity.itemName.length,
                    Spanned.SPAN_INCLUSIVE_EXCLUSIVE
                )

                setSpan(object : ClickableSpan() {
                    override fun onClick(v: View) {
                        imageShowDetail(v, entity)
                    }
                }, 0, this.length, Spanned.SPAN_INCLUSIVE_EXCLUSIVE)

                val colorSpan =
                    ForegroundColorSpan(context.resources.getColor(R.color.color_FFA019))
                setSpan(colorSpan, 0, entity.itemName.length, Spanned.SPAN_INCLUSIVE_EXCLUSIVE)
            }
        } else {
            SpannableString(entity.itemName).apply {

                setSpan(
                    StyleSpan(Typeface.BOLD),
                    0,
                    entity.itemName.length,
                    Spanned.SPAN_INCLUSIVE_EXCLUSIVE
                );
                val colorSpan =
                    ForegroundColorSpan(context.resources.getColor(R.color.color_comnont_black_one))
                setSpan(colorSpan, 0, entity.itemName.length, Spanned.SPAN_INCLUSIVE_EXCLUSIVE)
            }
        }
    }

    /**
     * 显示弹窗显示商品图片
     */
    private fun imageShowDetail(v: View, dingHuoOplQtyEntity: DingHuoOplQtyEntity.DinghuoDetail) {
        val content = LayoutInflater.from(context).inflate(R.layout.item_order_trade_popup, null)
        val imageView = content.findViewById(R.id.popup_order_trade_iv) as ImageView
        val tempW = GlobalApp.SCREEN_WIDTH - 20
        Picasso.with(context).load(dingHuoOplQtyEntity.picUrl)
            .placeholder(R.drawable.progress_animation_small).resize(tempW, tempW).into(imageView)
        val pw = PopupWindow(
            content,
            WindowManager.LayoutParams.MATCH_PARENT,
            WindowManager.LayoutParams.MATCH_PARENT,
            true
        )
        pw.isFocusable = true
        pw.isOutsideTouchable = true
        pw.showAtLocation(v, Gravity.CENTER, 0, 0)
        (content.findViewById(R.id.popup_order_trade_root_ll) as LinearLayout).setOnClickListener {
            if (pw.isShowing) {
                pw.dismiss()
            }
        }
        val product_name_id_tv = content.findViewById(R.id.product_name_id_tv) as TextView
        product_name_id_tv.text =
            "${dingHuoOplQtyEntity.itemName} + ( + ${dingHuoOplQtyEntity.itemid} + )"
    }

    /**
     * 计算库存率
     */
    private fun calculatorCyl(data: DingHuoOplQtyEntity.DinghuoDetail, counts: Double): Double {
        // 库存率=（订货数量+（库存+在途））/最近30天用量*
        var cyl: Double
        if (data.lastMonth_qty.toDouble() == 0.0 || (counts + data.stock_Qty.toDouble()) == 0.0) {
            cyl = 0.0
        } else {
            try {
                cyl = (data.tqty + data.stock_Qty.toDouble()) / data.lastMonth_qty.toDouble() * 30
            } catch (e: Exception) {
                cyl = 0.0
                BaseAppManager.loger().info(e)
            }

        }
        data.cyl = cyl
        return cyl
    }

    fun refreshDataList(newList: List<DingHuoOplQtyEntity.DinghuoDetail>) {
        orderList.run {
            clear()
            addAll(newList)
        }
    }

    /**
     * 通过标签来过滤显示的列表（后来需求要分品牌了，所以加一个品牌区分）
     */
    fun notifyProByLabel(dataEntity: DingHuoOplQtyEntity.ResultInfo?, brand: Int, type: Int) {
        handler.removeCallbacksAndMessages(null)
        productType = type
        brandType = brand
        orderList.run {
            val allList = dataEntity?.list
            entityCache = dataEntity//传进来一份缓存起来，adapter里面要用到 maxStockLimit
            clear()
            if (allList.isNullOrEmpty()) {
                addAll(listOf())
            } else {
                addAll(if (type == -1) allList.filter { it.brandId == brand } else allList.filter { it.brandId == brand && it.type == type })
            }
        }
        notifyDataSetChanged()
    }
}