package com.urovo.activity.goodsmanager.service

import com.urovo.activity.goodsmanager.model.AllotmentNotificationEntity
import com.urovo.activity.goodsmanager.model.StoreAllotEntity
import com.urovo.activity.goodsmanager.model.StoreAllotUpdateEntity
import com.urovo.wugumofang.constants.ApiUrls
import com.zsy.libcore.model.bean.RequestResult
import retrofit2.Call
import retrofit2.http.*

/**
 * 顾问标签接口
 * @author LW
 */
interface ApiService {
    /**
     * post /storeAllot/getStoreAllotNew 获取门店调拨信息(新)
     */
    @POST(ApiUrls.APP.STORE_ALLOT_NEW)
    fun getStoreAllotNew(
        @Query("storeNo") storeNo: String?,  // 门店号
        @Query("beginDate") beginDate: String?,  //开始时间
        @Query("endDate") endDate: String?,   //结束时间
        @Query("status") status: Int  //调拨状态(1:调拨出库,2:调拨收货)
    ): Call<RequestResult<List<StoreAllotEntity>>>

    /**
     * post /storeAllot/notificationStoreAllot 调拨单弹窗
     */
    @POST(ApiUrls.APP.NOTIFICATION_STORE_ALLOT)
    fun notificationStoreAllot(
        @Query("checkUserNo") checkUserNo: String?,  // 确认人id
        @Query("storeNo") storeNo: String?  //门店号
    ): Call<RequestResult<AllotmentNotificationEntity>>

    /**
     * post /storeAllot/updateStoreAllot 修改调拨收货
     */
    @POST(ApiUrls.APP.UPDATE_STORE_ALLOT)
    fun updateStoreAllot(
        @Body dbDetails: List<StoreAllotUpdateEntity>
    ): Call<RequestResult<Any>>

    /**
     * post  /storeAllot/confirmStoreAllot 门店确认收货
     */
    @POST(ApiUrls.APP.CONFIRM_STORE_ALLOT)
    fun confirmStoreAllot(
        @Query("dbBillCode") dbBillCode: String?,  // dbBillCode
        @Query("dbId") dbId: String?,  //调拨单号
        @Query("userNo") userNo: String?,  //调拨单号
        @Query("reason") reason: String?,  //拒收原因
        @Query("type") type:Int  //确认类型
    ): Call<RequestResult<Any>>

    /**
     *post /storeAllot/checkStoreAllot 门店确认发货
     */
    @POST(ApiUrls.APP.CHECK_STORE_ALLOT)
    fun checkStoreAllot(
        @Query("dbBillCode") dbBillCode: String?,  // dbBillCode
        @Query("dbId") storeNo: String?  //收货人id
    ): Call<RequestResult<Any>>
}