package com.urovo.activity.goodsmanager.stocksearch;

import android.content.Intent;
import android.graphics.Color;
import android.os.Bundle;
import android.text.TextUtils;
import android.view.View;
import android.widget.Button;
import android.widget.TextView;

import com.urovo.activity.goodsmanager.presenter.StockSearchPresenter;
import com.urovo.wgmf.CommonController;
import com.urovo.wgmf.db.dao.entitys.StockSearchEntity;
import com.urovo.wgmf.net.CommonNet;
import com.urovo.wugumofang.utils.MyToast;
import com.urovo.wugumofang.R;
import com.urovo.wugumofang.base.BaseActivity;
import com.urovo.wugumofang.data.MoneyUtils;
import com.urovo.wugumofang.net.NetHelper;
import com.urovo.wugumofang.system.SystemHelper;
import com.urovo.wugumofang.ui.ViewUtil;
import com.urovo.wugumofang.utils.CommonTools;
import com.wgmf.trade.model.bean.CommonEntity;
import com.zsy.libcore.common.android.log.SYLoger;

import java.text.MessageFormat;
import java.util.HashMap;
import java.util.Map;

import butterknife.BindView;
import butterknife.ButterKnife;
import butterknife.Unbinder;

/**
 * @author samy
 * @date 16/7/6
 * 库存查询
 */
public class StockSearchActivity extends BaseActivity implements CommonController.StockSearchView<StockSearchEntity> {
    private TextView storequire;
    private TextView texttitle;
    @BindView(R.id.last_month_sales_tv)
    TextView lastMonthSalesTv;
    @BindView(R.id.this_month_sales_tv)
    TextView thisMonthSalesTv;
    @BindView(R.id.just_in_time_inventory_tv)
    TextView justInTimeInventoryTv;
    @BindView(R.id.inventory_amount_standard_tv)
    TextView mInventoryAmountStandardTv;
    @BindView(R.id.storage_rate_tv)
    TextView storageRateTv;
    @BindView(R.id.store_detail_btn)
    Button storeDetailBtn;

    private String storeid;
    private Unbinder unbinder;
    Map<String, String> mParams = new HashMap<>(1);
    private StockSearchPresenter mPresenter;
    private StockSearchEntity stockSearchEntity;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.act_stock_search);
        unbinder = ButterKnife.bind(this);
        initTitle();
        init();
    }

    @Override
    protected void onStart() {
        super.onStart();
        SYLoger.print("onStart");
    }

    @Override
    protected void onResume() {
        super.onResume();
        SYLoger.print("onResume");
    }

    private void init() {
        mPresenter = new StockSearchPresenter(this, new CommonNet());
        storequire = (TextView) findViewById(R.id.ordergoods_quire);
        storequire.setText(MessageFormat.format("查询门店:{0}", SystemHelper.LoginOrganName));
        storeid = SystemHelper.LoginOrganId;
        query();
        texttitle = (TextView) findViewById(R.id.text_title);
        assert texttitle != null;
        texttitle.setText("库存查询");

        storeDetailBtn.setOnClickListener(new View.OnClickListener() {
                                              @Override
                                              public void onClick(View v) {
                                                  if (NetHelper.IsHaveInternet(StockSearchActivity.this)) {
                                                      Intent intent = new Intent(StockSearchActivity.this, StockGoodsActivity.class);
                                                      startActivity(intent);
                                                  } else {
                                                      MyToast.show("当前数据有问题，无法查看库存明细");
                                                  }
                                              }
                                          }
        );
    }

    /**
     * 库存查询请求
     */
    private void query() {
        if (NetHelper.IsHaveInternet(this)) {
            if (TextUtils.isEmpty(storeid)) {
                showMsg("请选择一个查询门店!");
                return;
            }
            clear();
            mParams.clear();
            // 服务器挂了，请求无响应回调的时候，6秒取消弹窗
            CommonTools.INSTANCE.postDelayed(10000);
            ViewUtil.showProgress(this, "正在玩命加载...", "请稍等...", false);
            mParams.put("storeNo", storeid);
            mPresenter.onStart2requestData(mParams);
        } else {
            MyToast.show("请检查网络是否已连接");
        }
    }

//    public void storeSelector(View view) {
//        Intent intent = new Intent(StockSearchActivity.this, StoreSelectorActivity.class);
//        startActivityForResult(intent, 200);
//    }

    @Override
    protected void onDestroy() {
        unbinder.unbind();
        super.onDestroy();
    }

    @Override
    public void onResponseSuccess(CommonEntity<StockSearchEntity> resultInfo) {
        ViewUtil.dismissProgress();
        if (this.isFinishing()) return;
        stockSearchEntity = resultInfo.t;
        if (stockSearchEntity == null) {
            showMsg("无门店库存数据");
        } else {
            lastMonthSalesTv.setText(MoneyUtils.twolastDF(stockSearchEntity.getOneMonthAmount()));
            thisMonthSalesTv.setText(MoneyUtils.twolastDF(stockSearchEntity.getShouhuoAmount()));
            justInTimeInventoryTv.setText(MoneyUtils.twolastDF(stockSearchEntity.getStockAmount()));
            mInventoryAmountStandardTv.setText(MoneyUtils.twolastDF(stockSearchEntity.getStandardAmount()));
            if (stockSearchEntity.getOneMonthAmount() == 0) {
                storageRateTv.setText("--天");
            } else {
//                double tempRate = (stockSearchEntity.getStockAmount() / stockSearchEntity.getOneMonthAmount()) * 30;
                double tempRate = stockSearchEntity.getStockDays();
                storageRateTv.setText(MessageFormat.format("{0}天", MoneyUtils.nolastDF(tempRate)));
                if (tempRate > 45) {
                    storageRateTv.setTextColor(Color.RED);
                }
            }
        }
    }

    @Override
    public void onFailure(String failInfo) {
        ViewUtil.dismissProgress();
        MyToast.show(failInfo);
    }

    /**
     * 清空所有缓存数据
     */
    private void clear() {
        if (stockSearchEntity != null) {
            lastMonthSalesTv.setText("--");
            thisMonthSalesTv.setText("--");
            justInTimeInventoryTv.setText("--");
            storageRateTv.setText("--");
        }
    }
}
