package com.urovo.activity.goodsmanager.weight

import android.content.Context
import android.graphics.Canvas
import android.util.AttributeSet
import android.view.LayoutInflater
import android.view.MotionEvent
import android.view.View
import android.view.ViewGroup
import android.widget.*
import androidx.constraintlayout.widget.ConstraintLayout
import com.urovo.wgmf.entity.response.DingHuoQueryNewRes.DinghuoEntity
import com.urovo.wugumofang.R
import org.jetbrains.anko.find
import kotlin.math.roundToInt

/**
 * 订货查询固定头部二级列表
 */
class OrderGoodsQueryListView @JvmOverloads constructor(
    context: Context,
    attrs: AttributeSet? = null,
    defStyleAttr: Int = 0
) : ExpandableListView(context, attrs, defStyleAttr) {

    private var mHeaderView: ViewGroup? = null
    private var mLlItemTitle: LinearLayout? = null
    private var mIvArrow: ImageView? = null
    private var mClItemContent: ConstraintLayout? = null

    private var mHeaderViewWidth: Int = 0
    private var mHeaderViewHeight: Int = 0

    /**
     * 当前列表组角标
     */
    private var mCurrentGroupPosition: Int = 0

    /**
     * 列表展开收起状态值
     */
    private var mExpandedMap = mutableMapOf<Int, Boolean>()

    private var previousFirstVisibleItem: Int = 0
    private var previousEventTime: Long = 0
    private var speed: Double = 0.0

    init {
        setOnGroupClickListener { parent, v, groupPosition, id ->
            val groupExpanded = isGroupExpanded(groupPosition)
            setExpanded(!groupExpanded)
            mExpandedMap[groupPosition] = !groupExpanded
            false
        }


        setOnScrollListener(object : OnScrollListener {
            override fun onScrollStateChanged(view: AbsListView?, scrollState: Int) {

            }

            override fun onScroll(
                view: AbsListView?,
                firstVisibleItem: Int,
                visibleItemCount: Int,
                totalItemCount: Int
            ) {
                if (previousFirstVisibleItem != firstVisibleItem) {
                    val currTime: Long = System.currentTimeMillis()
                    val timeToScrollOneElement = currTime - previousEventTime
                    speed = (1.0 / timeToScrollOneElement) * 1000
                    previousFirstVisibleItem = firstVisibleItem
                    previousEventTime = currTime
                }
//                if (speed > 15) {
//                    mHeaderView?.layout(0, -mHeaderViewHeight, mHeaderViewWidth, 0)
//                    return
//                }
                mHeaderView?.let { headerView ->
                    val firstGroup: Int =
                        getPackedPositionGroup(getExpandableListPosition(firstVisibleItem))
                    //val firstGroupChildCount: Int =
                    //    expandableListAdapter.getChildrenCount(firstGroup)
                    val nextPosition: Int = firstVisibleItem + 1
                    val nextGroup: Int =
                        getPackedPositionGroup(getExpandableListPosition(nextPosition))

                    if (firstGroup > mCurrentGroupPosition || firstGroup < mCurrentGroupPosition) {
                        bindListHeaderViewData(firstGroup)
                        setExpanded(mExpandedMap[firstGroup] ?: false)
                        mCurrentGroupPosition = firstGroup
                    }
                    if (mExpandedMap[firstGroup] == true) {
                        getChildAt(1)?.let { childView ->
                            val top = childView.top
                            if (firstGroup + 1 == nextGroup) {
                                val previousChildViewHeight = getChildAt(0).height
                                //if (mHeaderViewHeight == previousChildViewHeight) {
                                //默认情况，是根据mHeaderViewHeight高度来计算，View滑动
                                val delta =
                                    mHeaderViewHeight - (mHeaderViewHeight * (top / previousChildViewHeight))
                                if (top < mHeaderViewHeight) {
                                    headerView.layout(
                                        0,
                                        -delta,
                                        mHeaderViewWidth,
                                        mHeaderViewHeight - delta
                                    )
                                } else {
                                    headerView.layout(0, 0, mHeaderViewWidth, mHeaderViewHeight)
                                }
                                /*
                                } else {
                                    //由于previousChildViewHeight、mHeaderViewHeight两者的高度不一致，导致top滑动数值有偏差，所以增加倍数值。
                                    //其实也可以把两者进行比值处理，也是可以实现，但是体验不是很好。
                                    val delta = previousChildViewHeight - top
                                    if (top < previousChildViewHeight) {
                                        headerView.layout(
                                            0,
                                            -(delta * 4.0).roundToInt(),
                                            mHeaderViewWidth,
                                            mHeaderViewHeight - (delta * 4.0).roundToInt()
                                        )
                                    } else {
                                        headerView.layout(0, 0, mHeaderViewWidth, mHeaderViewHeight)
                                    }
                                }
                                */
                            } else {
                                headerView.layout(0, 0, mHeaderViewWidth, mHeaderViewHeight)
                            }
                        }
                    } else {
                        headerView.layout(0, -mHeaderViewHeight, mHeaderViewWidth, 0)
                    }
                }
            }
        })
    }

    fun showListHeaderView() {
        mHeaderView =
            LayoutInflater.from(context)
                .inflate(R.layout.ordergoods_query_list_group, null) as ViewGroup?

        mHeaderView?.layoutParams = RelativeLayout.LayoutParams(
            RelativeLayout.LayoutParams.MATCH_PARENT,
            RelativeLayout.LayoutParams.WRAP_CONTENT
        )

        mHeaderView?.let {
            mIvArrow = it.find(R.id.ivArrow)
            mClItemContent = it.find(R.id.clItemContent)
            mLlItemTitle = it.find(R.id.llItemTitle)
        }

        mExpandedMap.clear()
        for (pos in 0..expandableListAdapter.groupCount) {
            mExpandedMap[pos] = false
        }

        //获取角标为0的实体类数据，并进行数据绑定
        bindListHeaderViewData(0)

        setExpanded(false)

        requestLayout()
        postInvalidate()
    }

    /**
     * 绑定Header View 数据
     * @param entity DinghuoEntity
     */
    private fun bindListHeaderViewData(groupPosition: Int) {
        val entity: DinghuoEntity? = expandableListAdapter.getGroup(groupPosition) as DinghuoEntity?
        mHeaderView?.let {
            val tvOrderId = it.find<TextView>(R.id.tvOrderId)
            val tvDate = it.find<TextView>(R.id.tvDate)
            val tvBrand = it.find<TextView>(R.id.tvBrand)
            val tvTotalCount = it.find<TextView>(R.id.tvTotalCount)
            val tvMakingPeople = it.find<TextView>(R.id.tvMakingPeople)
            val tvTotalAmount = it.find<TextView>(R.id.tvTotalAmount)

            if (entity != null) {
                tvOrderId.text = String.format(
                    context.getString(R.string.input_order_id),
                    entity.orderGoodBill
                )
                tvDate.text = String.format(
                    context.getString(R.string.input_time),
                    entity.orderDate
                )
                tvMakingPeople.text = String.format(
                    context.getString(R.string.input_making_people),
                    entity.ordererName
                )
                tvBrand.text = String.format(
                    context.getString(R.string.input_brand),
                    entity.brandName
                )
                tvTotalCount.text =
                    String.format(
                        context.getString(R.string.input_total_count),
                        entity.totalQty.toString()
                    )
                tvTotalAmount.text =
                    String.format(
                        context.getString(R.string.input_total_amount),
                        entity.totalAmount.toString()
                    )
            }
        }
    }

    /**
     * 展开，收起状态切换
     * @param isExpanded Boolean true 展开; false 收起
     */
    private fun setExpanded(isExpanded: Boolean) {
        if (isExpanded) {
            mIvArrow?.setImageResource(R.mipmap.ic_arrow_up)
            mClItemContent?.setBackgroundResource(R.drawable.shape_white_round_top_6)
        } else {
            mIvArrow?.setImageResource(R.mipmap.ic_arrow_down)
            mClItemContent?.setBackgroundResource(R.drawable.shape_white_round_6)
        }
    }

    override fun onMeasure(widthMeasureSpec: Int, heightMeasureSpec: Int) {
        super.onMeasure(widthMeasureSpec, heightMeasureSpec)
        mHeaderView?.let {
            measureChild(it, widthMeasureSpec, heightMeasureSpec)
            mHeaderViewWidth = it.measuredWidth
            mHeaderViewHeight = it.measuredHeight
        }
    }

    override fun onLayout(changed: Boolean, l: Int, t: Int, r: Int, b: Int) {
        super.onLayout(changed, l, t, r, b)
        mHeaderView?.layout(0, -mHeaderViewHeight, mHeaderViewWidth, 0)
    }

    override fun dispatchDraw(canvas: Canvas?) {
        super.dispatchDraw(canvas)
        mHeaderView?.let {
            drawChild(canvas, it, drawingTime)
        }
    }

    private var downX = 0
    private var downY = 0

    override fun dispatchTouchEvent(ev: MotionEvent?): Boolean {
        mHeaderView?.let {
            when (ev?.action) {
                MotionEvent.ACTION_DOWN -> {
                    downX = ev.x.toInt()
                    downY = ev.y.toInt()
                }
                MotionEvent.ACTION_UP -> {
                    if (downX >= it.top && downY <= it.bottom) {
                        val position = pointToPosition(downX, downY)
                        val groupPosition =
                            getPackedPositionGroup(getExpandableListPosition(position))
                        val groupExpanded = isGroupExpanded(groupPosition)
                        if (groupPosition != INVALID_POSITION) {
                            if (groupExpanded) {
                                collapseGroup(groupPosition)
                            } else {
                                expandGroup(groupPosition)
                            }
                            setExpanded(!groupExpanded)
                            mExpandedMap[groupPosition] = !groupExpanded
                        }
                        return true
                    }
                }
            }
        }
        return super.dispatchTouchEvent(ev)
    }
}