package com.urovo.activity.home

import android.content.Context
import android.content.Intent
import android.os.Bundle
import android.os.Handler
import android.view.View
import android.widget.Button
import com.bumptech.glide.Glide
import com.urovo.activity.tabMenu.home.ent.QyGroupQrCodeRes
import com.urovo.wugumofang.R
import com.urovo.wugumofang.base.BaseActivity
import com.urovo.wugumofang.constants.ApiUrls
import com.urovo.wugumofang.constants.AppConstants
import com.urovo.wugumofang.constants.NetParam
import com.urovo.wugumofang.net.OkHttpDominator
import com.urovo.wugumofang.system.SystemHelper
import com.urovo.wugumofang.utils.MyToast
import com.urovo.wugumofang.view.webview.FraContainerAct
import com.wgmf.trade.ui.fra.index.MemberHomeFra
import com.zsy.libcore.ui.fra.FragmentArgs
import kotlinx.android.synthetic.main.enterprise_wx_code_act.*
import okhttp3.Request

/**
 * @author Xiep on 2020-10-13
 * 企业微信加群
 */
class EnterWxGroupCodeAct : BaseActivity() {

    val mPhone by lazy { intent.getStringExtra("huiyuan") ?: "" }
    val storeId by lazy { intent.getStringExtra("storeId") }

    companion object {
        @JvmStatic
        fun launch(context: Context, phone: String?) {
            context.startActivity(Intent(context, EnterWxGroupCodeAct::class.java).apply {
                putExtra("huiyuan", phone)
            })
        }
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.enterprise_wx_code_act)
        initTitle()
        init()
        queryQrcode()
    }

    fun init() {
        titleTextView.text = "扫码进群"

        //网络请求失败重新加载按钮
        buttonDownload.setOnClickListener { queryQrcode() }
        //下一步跳转
        goNextBtn.setOnClickListener {
            jump2MemberHome()
        }
    }

    /**请求二维码并验证是否加过企业微信*/
    fun queryQrcode() {
        OkHttpDominator.getAsyn(NetParam.WEIXIN_SERVER + ApiUrls.WeiXinEnterprise.IS_ADD_QY_GROUP_CHAT,
                mapOf("phone" to mPhone, "shopId" to SystemHelper.LoginOrganId), object : OkHttpDominator.ResultCallback<QyGroupQrCodeRes>() {
            override fun onError(request: Request?, e: Exception?) {
                promptText.text = "扫码加入五谷磨房专柜群，享受专属福利！"
                showNetError()
                MyToast.show(e?.message)
                if (isFinishing) return
                jump2MemberHome()//如果接口异常，就当作没有输入手机号的情况，直接跳转会员主页。
            }

            override fun onResponse(res: QyGroupQrCodeRes?) {
                qrCodeImage.visibility = View.VISIBLE
                buttonDownload.visibility = View.GONE
                if (isFinishing) return
                if (res == null) {
                    jump2MemberHome()
                    return
                }
                if (res.status != 0) {
                    promptText.text = res.msg
                    qrCodeImage.setImageResource(R.mipmap.icon_prompt_1)
                    Handler().postDelayed({ jump2MemberHome() }, 500)
                    return
                }
                if (res.data?.isAddChat == "0") {//已加入门店企业微信群
                    promptText.text = "已加入五谷磨房专柜群，可享受专属福利！"
                    qrCodeImage.setImageResource(R.mipmap.icon_prompt_1)
                    //已添加的就自动跳转
                    Handler().postDelayed({ jump2MemberHome() }, 500)
                } else {
                    promptText.text = "扫码加入五谷磨房专柜群，享受专属福利！"
                    Glide.with(this@EnterWxGroupCodeAct)
                            .load(res.data?.qrCodeUrl)
                            .placeholder(R.drawable.progress_animation_small)
                            .into(qrCodeImage)
                }
            }
        })
    }

    /**网络请求失败*/
    private fun showNetError() {
        qrCodeImage.visibility = View.GONE
        buttonDownload.visibility = View.VISIBLE
    }

    /**跳转会员主页*/
    fun jump2MemberHome() {
        FraContainerAct.launch(this@EnterWxGroupCodeAct, MemberHomeFra::class.java, FragmentArgs().apply {
            //手机号一定存在，需求说如果没填手机号的话，就不需要跳转到这个企业微信关注页面了。
            add(AppConstants.ExtraKeys.PHONE_NUM, mPhone)
        }, false)
    }
}