package com.urovo.activity.home

import android.content.Context
import android.content.Intent
import android.os.Bundle
import android.os.Handler
import android.text.TextUtils
import com.bumptech.glide.Glide
import com.urovo.activity.tabMenu.home.ent.UserQrCodeRes
import com.urovo.wugumofang.R
import com.urovo.wugumofang.base.BaseActivity
import com.urovo.wugumofang.constants.ApiUrls
import com.urovo.wugumofang.constants.AppConstants
import com.urovo.wugumofang.constants.NetParam
import com.urovo.wugumofang.net.OkHttpDominator
import com.urovo.wugumofang.system.SystemHelper
import com.urovo.wugumofang.utils.MyToast
import com.urovo.wugumofang.view.webview.FraContainerAct
import com.wgmf.trade.ui.fra.index.MemberHomeFra
import com.zsy.libcore.model.bean.RequestResult
import com.zsy.libcore.ui.fra.FragmentArgs
import kotlinx.android.synthetic.main.enterprise_wx_code_act.*
import okhttp3.Request

/**
 * @author Sasuke on 2019-8-19 0019.
 * 增加企业微信群、企业微信
 */
class EnterpriseWxCodeAct : BaseActivity() {

    /**
     * 手机号
     */
    private val mPhone by lazy { intent.getStringExtra("phone") ?: "" }

    /**
     * 微信群二维码
     */
    private val mQrCode by lazy { intent.getStringExtra("qrCode") ?: "" }

    /**
     * 现在企业微信群状态
     */
    private val mShowSwitch by lazy { intent.getIntExtra("showSwitch", 0) }

    /**
     * 是不是微信群码
     */
    private var mIsShowWxGroupCode: Boolean = true

    companion object {

        /**
         * 0:企业微信二维码
         */
        private const val ENTERPRISE_WX_CODE: String = "0"

        /**
         * 1:群二维码
         */
        private const val GROUP_CODE: String = "1"

        @JvmStatic
        fun launch(context: Context, phone: String?, qrCode: String?, showSwitch: Int) {
            context.startActivity(Intent(context, EnterpriseWxCodeAct::class.java).apply {
                putExtra("phone", phone)
                putExtra("qrCode", qrCode)
                putExtra("showSwitch", showSwitch)
            })
        }
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.enterprise_wx_code_act)
        initTitle()
        init()
        queryQrcode()
    }

    fun init() {
        titleTextView.text = "企业微信加好友"
        //下一步跳转
        goNextBtn.setOnClickListener {
            jumpNextLogic()
        }
    }

    /**
     * @param type :String 0:弹出企业微信二维码，1：弹出 群二维码
     */
    private fun popupWindowsData(type: String) {
        OkHttpDominator.postAsyn(NetParam.YINXIAOBU_SERVER + ApiUrls.APPSALER.POPUP_WINDOWS_DATA_COLLECTION,
            mapOf(
                "staffId" to SystemHelper.LoginUserId,
                "shopId" to SystemHelper.LoginOrganId,
                "mobile" to mPhone,
                "type" to type
            ), object : OkHttpDominator.ResultCallback<RequestResult<Any>>() {

                override fun onResponse(res: RequestResult<Any>?) {
                }

                override fun onError(request: Request?, e: Exception?) {
                }
            })
    }

    /**
     * 请求二维码并验证是否加过企业微信
     */
    private fun queryQrcode() {
        OkHttpDominator.postAsyn(NetParam.WEIXIN_SERVER + ApiUrls.WeiXinEnterprise.GET_USER_QR_CODE,
            mapOf(
                "userId" to SystemHelper.LoginUserId,
                "mobile" to mPhone,
                "shopId" to SystemHelper.LoginOrganId
            ), object : OkHttpDominator.ResultCallback<UserQrCodeRes>() {
                override fun onError(request: Request?, e: Exception?) {
                    MyToast.show(e?.message)
                    qrCodeImage.setImageResource(R.mipmap.icon_prompt_1)
                    if (isFinishing) return
                    jumpNextLogic()//如果接口异常，就当作没有输入手机号的情况，直接跳转会员主页。
                }

                override fun onResponse(res: UserQrCodeRes?) {
                    if (isFinishing) return
                    if (res == null) {
                        jumpNextLogic()
                        return
                    }
                    if (res.status != 0) {
                        promptText.text = res.msg
                        qrCodeImage.setImageResource(R.mipmap.icon_prompt_1)
                        Handler().postDelayed({ jumpNextLogic() }, 500)
                        return
                    }
                    if (res.data?.addedConsultantsFlag == "1") {
                        promptText.text = "此顾客已经添加企业微信好友，无需重复添加！"
                        qrCodeImage.setImageResource(R.mipmap.icon_prompt_1)
                        //已添加的就自动跳转
                        Handler().postDelayed({ jumpNextLogic() }, 500)
                    } else {
                        Glide.with(this@EnterpriseWxCodeAct)
                            .load(res.data?.qrCode)
                            .placeholder(R.drawable.progress_animation_small)
                            .into(qrCodeImage)

                        mIsShowWxGroupCode=false
                        popupWindowsData(ENTERPRISE_WX_CODE)
                    }
                }
            })
    }

    /**
     * 跳转到下一步逻辑处理
     */
    private fun jumpNextLogic() {
        if (mIsShowWxGroupCode) {
            if (mShowSwitch == 1 && !TextUtils.isEmpty(mQrCode)) {
                switchWxGroupCode()
                return
            }
        }
        jump2MemberHome()
    }

    /**
     * 切换到加企业微信群
     */
    private fun switchWxGroupCode() {
        titleTextView.text = "添加企业微信群"
        promptText.text = " 扫码加群，获得更多优惠信息"
        Glide.with(this@EnterpriseWxCodeAct)
            .load(mQrCode)
            .placeholder(R.drawable.progress_animation_small)
            .into(qrCodeImage)

        mIsShowWxGroupCode = false

        popupWindowsData(GROUP_CODE)
    }

    /**
     * 跳转会员主页
     */
    private fun jump2MemberHome() {
        FraContainerAct.launch(
            this@EnterpriseWxCodeAct,
            MemberHomeFra::class.java,
            FragmentArgs().apply {
                //手机号一定存在，需求说如果没填手机号的话，就不需要跳转到这个企业微信关注页面了。
                add(AppConstants.ExtraKeys.PHONE_NUM, mPhone)
            },
            false
        )
    }
}