package com.urovo.activity.labelprinter.act;

import android.graphics.Color;
import android.os.Bundle;
import android.text.Editable;
import android.text.TextUtils;
import android.text.TextWatcher;
import android.view.View;
import android.view.ViewGroup;
import android.widget.EditText;
import android.widget.TextView;

import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.gavin.com.library.PowerfulStickyDecoration;
import com.gavin.com.library.listener.PowerGroupListener;
import com.google.gson.Gson;
import com.google.gson.reflect.TypeToken;
import com.urovo.activity.labelprinter.ada.LabelPrinterAda;
import com.urovo.activity.labelprinter.ent.LabelPrinterEnt;
import com.urovo.activity.labelprinter.pre.LabelPrinterPre;
import com.urovo.wugumofang.R;
import com.urovo.wugumofang.base.BaseActivity;
import com.urovo.wugumofang.constants.AppConstants;
import com.urovo.wugumofang.data.PreferHelper;
import com.urovo.wugumofang.net.CommonController;
import com.urovo.wugumofang.net.NetHelper;
import com.urovo.wugumofang.system.SystemHelper;
import com.urovo.wugumofang.ui.DensityUtil;
import com.urovo.wugumofang.ui.ViewUtil;
import com.urovo.wugumofang.utils.CommonTools;
import com.urovo.wugumofang.utils.MyToast;
import com.zsy.libcore.common.android.log.Log;
import com.zsy.libcore.common.android.log.SYLoger;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import butterknife.BindView;
import butterknife.ButterKnife;
import butterknife.Unbinder;

/**
 * @author chenji last on 2018/2/6 15:28
 */

public class LabelPrinterAct extends BaseActivity implements CommonController.BaseView<List<LabelPrinterEnt>> {
    private Unbinder unbinder;
    @BindView(R.id.label_printer)
    RecyclerView mLabelPrinterRv;
    @BindView(R.id.searchInput)
    EditText searchInput;//搜索输入栏
    private String keyWord = "";//搜索用的关键字（执行搜索过滤之前，先对比一下上一次的，如果是一样的，就不执行，避免多次执行textWatcher

    private List<LabelPrinterEnt> mTotalList = new ArrayList<>();//源数据表
    private List<LabelPrinterEnt> mShowTotalList = new ArrayList<>();//用来显示的数据表

    private LabelPrinterAda mLabelPrinterAda;
    private LabelPrinterPre mLabelPrinterPre;
    PowerfulStickyDecoration decoration = null;//分割栏

    final public static String PROD = "PROD";//标签打印
    final public static String DDLINE = "DDLINE";//期限标签

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.act_label_printer);
        unbinder = ButterKnife.bind(this);
        initTitle();
        mLabelPrinterPre = new LabelPrinterPre(this);
        //清空sp中保存的日期
        clear(PreferHelper.getInstance().getString(AppConstants.Printer.LABEL_PRO_CODE));
        //清空sp中保存的数量
        clear(PreferHelper.getInstance().getString(AppConstants.Printer.LABEL_PRO_CODE_NUM));
        SYLoger.print("wgmf--label-->onCreate");
        initRv();
        start2RequestTask();
    }

    private void clear(String proCodeListJson) {
        if (!TextUtils.isEmpty(proCodeListJson) && !fromJson(proCodeListJson).isEmpty()) {
            final List<String> proCodeList = fromJson(proCodeListJson);
            for (String proCode : proCodeList) {
                SYLoger.print("wgmf-->标签打印 产品唯一标识-->" + proCode);
                PreferHelper.getInstance().remove(proCode);
            }
            //清空该集合
            proCodeList.clear();
        }
    }

    /**
     * json 转集合
     *
     * @param json ...
     * @return ...
     */
    private static List<String> fromJson(String json) {
        return new Gson().fromJson(json, new TypeToken<List<String>>() {
        }.getType());
    }

    /**
     * 网络请求
     */
    private void start2RequestTask() {
        if (NetHelper.IsHaveInternet(this)) {
            CommonTools.INSTANCE.postDelayed(30000);
            ViewUtil.showProgress(this, "正在玩命加载...", "请稍等...");
            Map<String, String> params = new HashMap<>(1);
            params.put("storeNo", SystemHelper.LoginOrganId);
            mLabelPrinterPre.onStart2requestData(params);
        } else {
            showMsg("网络连接超时，请检查您的网络状态，稍后重试！");
            finish();
        }
    }

    private void initRv() {
        mLabelPrinterRv.setLayoutManager(new LinearLayoutManager(this, RecyclerView.VERTICAL, false));

        if (decoration != null){//后面搜索的时候，会重新调用这个initRv()方法，实现Rv的刷新，然而这里如果不remove的话，就会反复add从而把上面空出很多。
            mLabelPrinterRv.removeItemDecoration(decoration);
        }

        decoration = PowerfulStickyDecoration.Builder
                .init(new PowerGroupListener() {
                    @Override
                    public String getGroupName(int position) {
                        //获取组名，用于判断是否是同一组
                        if (mShowTotalList.size() > position) {
                            return mShowTotalList.get(position).getProCode();
                        }
                        return null;
                    }

                    @Override
                    public View getGroupView(int position) {
                        //获取自定定义的组View
                        if (mShowTotalList.size() > position) {
                            View view = getLayoutInflater().inflate(R.layout.label_item_title, null, false);
                            ((TextView) view.findViewById(R.id.label_pro_code_tv)).setText(mShowTotalList.get(position).getProCode());
                            ((TextView) view.findViewById(R.id.label_pro_name_tv)).setText(mShowTotalList.get(position).getProName());
                            return view;
                        } else {
                            return null;
                        }
                    }
                })
                .setGroupHeight(DensityUtil.dip2px(LabelPrinterAct.this, 35))   //设置高度
                .setGroupBackground(Color.parseColor("#f1f1f1"))        //背景色
                .build();

        mLabelPrinterRv.addItemDecoration(decoration);

        mLabelPrinterAda = new LabelPrinterAda(this,
                mShowTotalList,
                getIntent().getStringExtra(AppConstants.ExtraKeys.ACTIVITY_TYPE));
//        mLabelPrinterAda.setOnItemChildClickListener(new BaseQuickAdapter.OnItemChildClickListener() {
//            @Override
//            public void onItemChildClick(BaseQuickAdapter baseQuickAdapter, View view, int i) {
////                Toast.makeText(BaseAppManager.appContext(), "hahahonItemChildClick: " + i, Toast.LENGTH_SHORT).show();
//            }
//        });
        mLabelPrinterRv.setAdapter(mLabelPrinterAda);

        searchInput.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {

            }

            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {

            }

            @Override
            public void afterTextChanged(Editable s) {
                if (keyWord.equals(s.toString()) || mTotalList.size() == 0) return;
                keyWord = s.toString();//用来对比，避免多次执行textChangeWatcher
                initRv();//Rv好像必须重新初始化一下，否则在搜索的时候分割栏名字会与item内容产生差异。
                if (TextUtils.isEmpty(s.toString())) {
                    mShowTotalList.clear();
                    mShowTotalList.addAll(mTotalList);
                    mLabelPrinterAda.notifyDataSetChanged();
                    return;
                }else{
                    mShowTotalList.clear();
                    for (LabelPrinterEnt ent : mTotalList){
                        if (ent.getProCode().contains(s.toString()) || ent.getProName().contains(s.toString())){
                            mShowTotalList.add(ent);
                        }
                    }
                    mLabelPrinterAda.notifyDataSetChanged();
                }

            }
        });
    }

    @Override
    public void initTitle() {
        super.initTitle();
        if (getIntent().getStringExtra(AppConstants.ExtraKeys.ACTIVITY_TYPE).equals(PROD)){
            titleTextView.setText("标签打印");
        }else{
            titleTextView.setText("销售期限标签");
        }
    }

    @Override
    protected void onDestroy() {
        if (unbinder != null) {
            unbinder.unbind();
        }
        super.onDestroy();
    }

    @Override
    public void onResponseSuccess(com.urovo.wugumofang.response.CommonEntity<List<LabelPrinterEnt>> resultInfo) {
        ViewUtil.dismissProgress();
        final List<LabelPrinterEnt> entity = resultInfo.entity;
        if (entity == null || entity.isEmpty()) {
            View notDataView = getLayoutInflater().inflate(R.layout.layout_empty_date_view, (ViewGroup) mLabelPrinterRv.getParent(), false);
            mLabelPrinterAda.setEmptyView(notDataView);
        } else {
            mTotalList.clear();
            mTotalList.addAll(entity);
            mShowTotalList.clear();
            mShowTotalList.addAll(entity);
        }
        mLabelPrinterAda.notifyDataSetChanged();
    }

    @Override
    public void onFailure(String failInfo) {
        ViewUtil.dismissProgress();
        MyToast.show(failInfo);
        if ("该门店无鲜稻研产品".equals(failInfo)) {
            finish();
        }
        Log.e("wgmf-->标签打印:" + failInfo);
    }
}
