package com.urovo.activity.labelprinter.ada;

import android.content.Context;
import android.text.Editable;
import android.text.TextUtils;
import android.text.TextWatcher;
import android.view.View;
import android.widget.EditText;
import android.widget.Toast;

import androidx.annotation.Nullable;

import com.chad.library.adapter.base.BaseMultiItemQuickAdapter;
import com.chad.library.adapter.base.BaseViewHolder;
import com.google.gson.Gson;
import com.urovo.activity.labelprinter.act.LabelPrinterAct;
import com.urovo.activity.labelprinter.ent.MultipleItem;
import com.urovo.wugumofang.R;
import com.urovo.wugumofang.constants.AppConstants;
import com.urovo.wugumofang.data.DateUtil;
import com.urovo.wugumofang.data.PreferHelper;
import com.urovo.wugumofang.utils.CommonTools;
import com.urovo.wugumofang.utils.MyToast;
import com.zsy.libcore.common.android.log.SYLoger;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.Locale;

/**
 * @author chenji
 */
public class NestAda extends BaseMultiItemQuickAdapter<MultipleItem, BaseViewHolder> {
    private Context mContext;
    private String proCodeKey = "";
    private String activityType;//普通标签打印 或 期限标签打印

    public NestAda(Context context, @Nullable List<MultipleItem> data, String activityType) {
        super(data);
        this.mContext = context;
        this.activityType = activityType;
        addItemType(MultipleItem.TEXT_CONTENT, R.layout.label_item_content);
        if (activityType.equals(LabelPrinterAct.PROD)){//普通标签打印
            addItemType(MultipleItem.TEXT_FIXED, R.layout.label_item_fixed);
        }else {//期限标签
            addItemType(MultipleItem.TEXT_FIXED, R.layout.label_item_fixed2);
        }
    }

    private List<String> proCodeList = CommonTools.INSTANCE.getProCodeList();

    private String tempProCode = "";

    @Override
    protected void convert(BaseViewHolder helper, final MultipleItem data) {
        final String[] array = data.getStringArray();

        switch (helper.getItemViewType()) {
            case MultipleItem.TEXT_CONTENT:
                if (array != null && array.length != 0) {
                    final String labelProducersKey = array[0];
                    final String labelProducersValue = array[1];
                    helper.setText(R.id.label_producers_key, labelProducersKey + ":");
                    helper.setText(R.id.label_producers_value, labelProducersValue);
                }
                break;
            case MultipleItem.TEXT_FIXED:
                proCodeKey = data.getContent();
                //=======================销售期限=======================
                if (activityType.equals(LabelPrinterAct.DDLINE)){
                    final String currentDateSp2 = PreferHelper.getInstance().getString(proCodeKey+"_deadLine");
                    //期限日期要比开封日期往后推180天
                    String currentDate2 = new SimpleDateFormat("yyyy-MM-dd", Locale.CHINA).format(DateUtil.addDate(new Date(), 180));
                    //在当前页面里,当日期改变后就显示改变的日期,重新加载该页面后需要清空sp保存的日期,默认显示与否当前的日期
                    if (TextUtils.isEmpty(currentDateSp2)) {
                        helper.setText(R.id.label_producers_deadline_date, currentDate2);
                        //确保在默认状态下从sp取数据打印
                        PreferHelper.getInstance().setString(proCodeKey + "_deadLine", currentDate2);
                    } else {
                        helper.setText(R.id.label_producers_deadline_date, currentDateSp2);
                    }
                }

                //=======================生产日期or开封日期=======================
                //默认当前的日历
                final String currentDateSp = PreferHelper.getInstance().getString(proCodeKey);
                String currentDate = DateUtil.getSpecifiedDay(new Date(), 0, "yyyy-MM-dd", Locale.CHINA);
                //在当前页面里,当日期改变后就显示改变的日期,重新加载该页面后需要清空sp保存的日期,默认显示与否当前的日期
                if (TextUtils.isEmpty(currentDateSp)) {
                    helper.setText(R.id.label_producers_date, currentDate);
                    //确保在默认状态下从sp取数据打印
                    PreferHelper.getInstance().setString(proCodeKey, currentDate);
                } else {
                    helper.setText(R.id.label_producers_date, currentDateSp);
                }

                //======================打印数量===========================
                final EditText etPrintNum = helper.getView(R.id.label_print_num_value);
                String printNumSp = PreferHelper.getInstance().getString(proCodeKey + "_etPrintNum");
                if (TextUtils.isEmpty(printNumSp)) {
                    etPrintNum.setText("1");
                    PreferHelper.getInstance().setString(proCodeKey + "_etPrintNum", "1");
                    PreferHelper.getInstance().setString(AppConstants.Printer.LABEL_DIALOG_CANCLE, "isNotEmpty");
                } else {
                    etPrintNum.setText(printNumSp);
                }

                helper.setOnClickListener(R.id.label_decrease, new View.OnClickListener() {
                    @Override
                    public void onClick(View v) {
                        String printNumStr = etPrintNum.getText().toString().trim();
                        etPrintNum.setSelection(printNumStr.length());
                        int printNum = Integer.parseInt(printNumStr.isEmpty() ? "0" : printNumStr);
                        if (printNum > 1) {
                            etPrintNum.setText(String.format(Locale.CHINA, "%d", --printNum));
                        }
                    }
                });
                helper.setOnClickListener(R.id.label_plus, new View.OnClickListener() {
                    @Override
                    public void onClick(View v) {
                        String printNumStr = etPrintNum.getText().toString().trim();
                        etPrintNum.setSelection(printNumStr.length());
                        if (!printNumStr.isEmpty()) {
                            int printNum = Integer.parseInt(printNumStr.isEmpty() ? "0" : printNumStr);
                            etPrintNum.setText(String.format(Locale.CHINA, "%d", ++printNum));
                        }
                    }
                });
                etPrintNum.addTextChangedListener(new TextWatcher() {
                    @Override
                    public void beforeTextChanged(CharSequence s, int start, int count, int after) {
                        SYLoger.print("wgmf-->Editext-->beforeTextChanged");
                    }

                    @Override
                    public void onTextChanged(CharSequence s, int start, int before, int count) {
                        SYLoger.print("wgmf-->Editext-->onTextChanged");
                    }

                    @Override
                    public void afterTextChanged(Editable s) {
                        //一次输入多个,这个editext不买单,集合永远只存储一个,坑爹,proCodeList必须在外面创建
                        SYLoger.print("wgmf-->proCodeKey-->afterTextChanged etPrintNum.getText()" + etPrintNum.getText());

                        if (!tempProCode.equals(proCodeKey + "_etPrintNum")) {
                            tempProCode = proCodeKey + "_etPrintNum";
                            proCodeList.add(tempProCode);
                        }
                        SYLoger.print("wgmf-->Editext-->afterTextChanged--" + proCodeList.size());
                        String proCodeListJson = new Gson().toJson(proCodeList);
                        PreferHelper.getInstance().setString(AppConstants.Printer.LABEL_PRO_CODE_NUM, proCodeListJson);

                        //不能为""
                        final String etPrintNumStr = etPrintNum.getText().toString().trim();
                        if (!etPrintNumStr.isEmpty()) {
                            int printNum = Integer.parseInt(etPrintNumStr);
                            etPrintNum.setSelection(etPrintNum.getText().toString().trim().length());
                            //最大打印数量为20
                            if (printNum > 20) {
                                MyToast.show(mContext, "最大打印数量为20!", Toast.LENGTH_SHORT);
                                etPrintNum.setText(String.format(Locale.CHINA, "%d", 20));
                                PreferHelper.getInstance().setString(proCodeKey + "_etPrintNum", "20");
                            } else {
                                PreferHelper.getInstance().setString(proCodeKey + "_etPrintNum", printNum + "");
                            }
                            PreferHelper.getInstance().setString(AppConstants.Printer.LABEL_DIALOG_CANCLE, "isNotEmpty");
                        } else {
                            PreferHelper.getInstance().setString(proCodeKey + "_etPrintNum", "");
                            PreferHelper.getInstance().setString(AppConstants.Printer.LABEL_DIALOG_CANCLE, "");
                        }
                    }
                });

                helper.addOnClickListener(R.id.label_producers_deadline_date);
                helper.addOnClickListener(R.id.label_producers_date);
                helper.addOnClickListener(R.id.label_printer_btn);

                break;
            default:
                break;
        }
    }
}
