package com.urovo.activity.machinerepairact.model

import android.view.View
import androidx.databinding.ObservableField
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import com.urovo.activity.common.DatePickerYearMonthDialog
import com.urovo.activity.machinerepairact.net.UndealRepairRes
import com.urovo.activity.machinerepairact.repository.MachineCallRepo
import com.urovo.activity.machinetrip.net.PatrolReportRes
import com.urovo.activity.machinetrip.view.MachineOrderSelectAct
import com.urovo.wugumofang.system.SystemHelper
import com.urovo.wugumofang.utils.MyToast
import com.urovo.wugumofang.view.LoadWaitDominator
import kotlinx.coroutines.launch
import java.text.SimpleDateFormat
import java.util.*

/**
 * @author Sasuke on 2019/5/7.
 */
class EngineerBillModel(val repo: MachineCallRepo): ViewModel() {

    /**出差事由*/
    var activityType =String()

    /**未处理单*/
    var unDealResCache = MutableLiveData<UndealRepairRes>()

    /**根据当前标签来判定类型*/
    var hasDealed = ObservableField<Boolean>(false)//默认未处理false,已处理true

    /**
     * 请求所使用的年月
     */
     var queryYm = ObservableField<String>(SimpleDateFormat("yyyy-MM").format(Date()))

    /**loadwait指示*/
    var loadWaitMsg = MutableLiveData<Int>().apply { value = -1 }

    /**
     * 从缓存读表数据
     */
    fun getListCache(): List<UndealRepairRes.ResultPack.ResultInfo>{
        return unDealResCache.value?.resultInfo?.details?.filter {  if (hasDealed.get()!!) it.status != 0 else it.status == 0 } ?: listOf()
    }

    /**巡店记录*/
    var patrolReport: MutableLiveData<List<PatrolReportRes.ResultInfo>> = MutableLiveData()

    /**查询巡店记录*/
    fun queryPatrolReport(ym: String) = viewModelScope.launch {
        try {
            loadWaitMsg.value = LoadWaitDominator.LOADING
            patrolReport.value = repo.queryPatrolReport(SystemHelper.LoginUserId, ym)

            loadWaitMsg.value = if (patrolReport.value?.isEmpty() ?:true) LoadWaitDominator.NO_DATA else -1
        } catch (e: Exception) {
            MyToast.show(e.message)
            loadWaitMsg.value = LoadWaitDominator.NET_ERROR
        }
    }

    /**
     * 请求
     */
    fun queryAllBill(ym: String) {
        viewModelScope.launch {
            try {
                loadWaitMsg.value = LoadWaitDominator.LOADING
                unDealResCache.value = repo.queryUnDealRepair(SystemHelper.LoginUserId, ym)

                loadWaitMsg.value = if (getListCache().isEmpty()) LoadWaitDominator.NO_DATA else -1
            } catch (e: Exception) {
                MyToast.show(e.message)
                loadWaitMsg.value = LoadWaitDominator.NO_DATA
            }
        }
    }

    /**日期选择点击监听*/
    fun dateSelectOnClick(view: View){
        DatePickerYearMonthDialog( view.context, { year, monthOfYear ->
            dateSelected(year, monthOfYear)
        }).show()
    }

    /**
     * 已选查询日期之后的处理
     */
    fun dateSelected(year: Int, month: Int) {
        val today = SimpleDateFormat("yyyy-MM").format(Date())
        val selectDay = "${year}-${String.format("%02d", month + 1)}"
        if (today.compareTo(selectDay) < 0) {
            MyToast.show("不能超过当月")
            return
        }
        queryYm.set(selectDay)
        if (activityType == MachineOrderSelectAct.TYPE_MACHINE_BILL){//查工单
            queryAllBill(queryYm.get()!!)
        }else if (activityType == MachineOrderSelectAct.TYPE_MACHINE_PATROL) {//查巡店
            queryPatrolReport(queryYm.get()!!)
        }else{//兼容机修查我的工单
            queryAllBill(queryYm.get()!!)
        }
    }
}