package com.urovo.activity.machinerepairact.model

import android.app.AlertDialog
import android.view.View
import android.widget.TextView
import androidx.databinding.ObservableArrayList
import androidx.databinding.ObservableField
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import com.amap.api.location.AMapLocation
import com.urovo.activity.machinerepairact.repository.MachineCallRepo
import com.urovo.wgmf.tool.netUtil.QiNiuDominator
import com.urovo.wugumofang.data.DateUtil
import com.urovo.wugumofang.net.NetHelper
import com.urovo.wugumofang.system.SystemHelper
import com.urovo.wugumofang.ui.PhotoPreviewDominator
import com.urovo.wugumofang.utils.MyToast
import com.zsy.libext.utils.DateTimeUtils
import kotlinx.coroutines.launch
import java.text.ParseException
import java.text.SimpleDateFormat
import java.util.*

/**
 * @author Sasuke on 2019-05-08 0008.
 */
class EngineerSubmitModel(val repo: MachineCallRepo) : ViewModel() {
    /**当前位置*/
    var storeName = ObservableField<String>("")
    /**门店位置*/
    var storeLocation = ObservableField<String>("")

    var mroId = MutableLiveData<Int>()

    /**当前位置*/
    var currLocation = ObservableField<AMapLocation>()

    /**完成日期*/
    var finishDate = ObservableField<String>(SimpleDateFormat("yyyy-MM-dd").format(Date()))

    /**完成时间*/
    var finishTime = ObservableField<String>(SimpleDateFormat("HH:mm").format(Date()))

    /**下拉数据源*/
    val spinnerList = arrayOf("上门解决", "电话协助","其他")

    /**处理方式*/
    var handleWay = MutableLiveData<String>()

    /**处理描述（在处理方式为“其他”时需填写）*/
    var handleDesc = ""

    /**是否显示照片选取*/
    var isShowPhotoSelect = ObservableField<Boolean>()

    /**是否显示完成时间*/
    var isShowDoneTime = ObservableField<Boolean>()

    /**现场照片*/
    var photoGenba = ObservableArrayList<String>()
    var qiniuGenba = MutableLiveData<String>()
    /**线路照片*/
    var photoSenro = ObservableArrayList<String>()
    var qiniuSenro = MutableLiveData<String>()
    /**设备照片*/
    var photoSeibi = ObservableArrayList<String>()
    var qiniuSeibi = MutableLiveData<String>()
    /**店面照片*/
    var photoMise = ObservableArrayList<String>()
    var qiniuMise = MutableLiveData<String>()

    /**上传成功并退出*/
    var successFinish = MutableLiveData<Boolean>()

    /**当前点击的添加按钮*/
    var currAdd = MutableLiveData<Int>().apply { value = -1 }


    var showProgressText = MutableLiveData<String>()

    /**
     * 完成日期点击监听
     */
    fun onClickFinishDate(view: View) {
        DateUtil.pickDialog(view.context, view as TextView, "yyyy-MM-dd", null, Date(),{
            checkTime()
        }).show()
    }

    /**
     * 完成时间点击监听
     */
    fun onClickFinishTime(view: View) {
        DateTimeUtils.pickDialog(view.context, view as TextView, "HH:mm",
                null, null, null, DateTimeUtils.SelectedCallBack {
            //选完时间后的回调，刷新截止时间
            checkTime()
        })!!.show()
    }

//============================================
    /**点击预览大图*/
    fun onClickGenba(view: View) {
        PhotoPreviewDominator(view, photoGenba[0]) {
            photoGenba.removeAt(0)
        }.showPreview()
    }

    /**点击预览大图*/
    fun onClickSenro(view: View) {
        PhotoPreviewDominator(view, photoSenro[0]) {
            photoSenro.removeAt(0)
        }.showPreview()
    }

    /**点击预览大图*/
    fun onClickSeibi(view: View) {
        PhotoPreviewDominator(view, photoSeibi[0]) {
            photoSeibi.removeAt(0)
        }.showPreview()
    }

    /**点击预览大图*/
    fun onClickMise(view: View) {
        PhotoPreviewDominator(view, photoMise[0]) {
            photoMise.removeAt(0)
        }.showPreview()
    }
//============================================
    /**
     * 校验时间合法性
     */
    fun checkTime() {
        try {
            //防止先选择更晚的日期，再选择更早的时刻，再返回来选择今天的日期，那就相当于把时刻的限制给绕过了（常规时间校验也放在这里）
            if ((finishDate.get() + " " + finishTime.get()).compareTo(SimpleDateFormat("yyyy-MM-dd HH:mm").format(Date())) > 0) {
                MyToast.show("不能晚于当前时间：" + SimpleDateFormat("yyyy-MM-dd HH:mm").format(Date()))
                finishDate.set(SimpleDateFormat("yyyy-MM-dd").format(Date()))
                finishTime.set(SimpleDateFormat("HH:mm").format(Date()))
            }
        } catch (e: ParseException) {
            e.printStackTrace()
        }
    }


    /**
     * 点击提交按钮
     */
    fun onSubmitClick(view: View) {
        viewModelScope.launch {
            try {
                if (!NetHelper.IsHaveInternet(view.context)){
                    MyToast.show("网络连接超时，请检查您的网络状态，稍后重试！")
                    return@launch
                }
                //其他处理情况==================
                if (handleWay.value == "其他"){
                    if (handleDesc.isBlank()){
                        MyToast.show("请填写工单的处理方式")
                        return@launch
                    }
                    upload(mroId.value!!, handleWay.value!!, handleDesc,SimpleDateFormat("yyyy-MM-dd HH:mm").format(Date()), "", false)
                    return@launch
                }
                //电话协助情况==================
                if (handleWay.value == "电话协助") {
                    upload(mroId.value!!, handleWay.value!!, "",finishDate.get() + " " + finishTime.get(), "", false)
                    return@launch
                }
                //现场解决情况=====================
                if (currLocation.get() == null) {
                    MyToast.show("请先定位再试")
                    return@launch
                }
                if (photoGenba.isEmpty()){
                    MyToast.show("请上传处理后的现场照片")
                    return@launch
                }
                showProgressText.value = "正在校验地理位置..."
                val checkRes = repo?.checkLocation(SystemHelper.LoginUserId, SystemHelper.LoginOrganId, currLocation.get()!!)
                val locationCorrect = when (checkRes.msgCode) {
                    "0" -> "y"
                    else -> "n"
                }
                showProgressText.value = ""
                if (checkRes.msgCode != "0") {
                    AlertDialog.Builder(view.context).setTitle("提示")
                            .setMessage("当前位置在门店范围外，将会标记为位置异常，确定要提交吗？")
                            .setPositiveButton("确定", { dialog, which ->
                                upload(mroId.value!!, handleWay.value!!, "",finishDate.get() + " " + finishTime.get(), locationCorrect, true)
                            })
                            .setNegativeButton("取消", null).show()
                } else {//msgCode == 0，直接上传
                    upload(mroId.value!!, handleWay.value!!, "",finishDate.get() + " " + finishTime.get(), locationCorrect, true)
                }
            } catch (e: Exception) {
                MyToast.show(e.message)
            }
        }
    }


    /**
     * 真正上传
     */
    fun upload(mroId: Int, handleWay: String,handleDesc: String, finishTime: String, locationCorrect: String, withPhotos: Boolean) = viewModelScope.launch {
        showProgressText.value = "正在上传图片..."
        //传图片
        if (withPhotos){
            try {
                for (i in 0..photoTransList.size - 1) {
                    val result = repo.uploadQiniu777(photoTransList[i].localPath, photoTransList[i].qiniuCache)
                    if (result != QiNiuDominator.SUCCESS) {
                        showProgressText.value = ""
                        MyToast.show("图片上传失败")
                        return@launch
                    }
                }
            } catch (e: Exception) {
                showProgressText.value = ""
                MyToast.show(e.message)

                return@launch
            }
        }
        //传数据
        showProgressText.value = "正在上传数据..."
        try {
            val photoParam = MachineCallRepo.PhotoParam(qiniuGenba.value, qiniuSenro.value, qiniuSeibi.value, qiniuMise.value)
            val res = repo?.uploadDealRepair(mroId, handleWay, handleDesc,finishTime, locationCorrect, if (withPhotos) photoParam else null)
            showProgressText.value = ""

            if (res.msgCode == 0){
                successFinish.value = true
                MyToast.show(res.msgInfo)
            }else{
                MyToast.show(res.msgInfo)
            }
        }catch (e: Exception){
            showProgressText.value = ""
            MyToast.show(e.message)
        }

    }

    data class PhotoUnit(var localPath: ObservableArrayList<String>, var qiniuCache: MutableLiveData<String>)

    /**用于递归上传的图片集*/
//    val photoTransList = listOf<PhotoUnit>(
//            PhotoUnit(photoGenba, qiniuGenba),
//            PhotoUnit(photoSenro, qiniuSenro),
//            PhotoUnit(photoSeibi, qiniuSeibi),
//            PhotoUnit(photoMise, qiniuMise))
    val photoTransList: List<PhotoUnit>
        get() {
            val list = mutableListOf<PhotoUnit>()
            if (photoGenba.isNotEmpty()) list.add(PhotoUnit(photoGenba, qiniuGenba))
            if (photoSenro.isNotEmpty()) list.add(PhotoUnit(photoSenro, qiniuSenro))
            if (photoSeibi.isNotEmpty()) list.add(PhotoUnit(photoSeibi, qiniuSeibi))
            if (photoMise.isNotEmpty()) list.add(PhotoUnit(photoMise, qiniuMise))
            return list
        }

}