package com.urovo.activity.machinerepairact.model

import android.content.Intent
import android.view.View
import androidx.databinding.ObservableField
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import com.urovo.activity.attendancemanagement.SignInAndOutActivity
import com.urovo.activity.goodsmanager.OrderGoodsTabAct
import com.urovo.activity.machinerepairact.net.EngineerMyStoresRes
import com.urovo.activity.machinerepairact.net.LightProblemRes
import com.urovo.activity.machinerepairact.net.UndealRepairRes
import com.urovo.activity.machinerepairact.repository.MachineCallRepo
import com.urovo.activity.machinerepairact.view.EngineerBillsAct
import com.urovo.activity.machinerepairact.view.EngineerMyStoresAct
import com.urovo.activity.machinerepairact.view.LightProblemStoresAct
import com.urovo.activity.machinerepairact.view.PatrolActList
import com.urovo.activity.machinetrip.view.MachineTripAct
import com.urovo.activity.systemsetting.SttingMenu
import com.urovo.wugumofang.system.SystemHelper
import com.urovo.wugumofang.system.ToActivity
import com.urovo.wugumofang.view.LoadWaitDominator
import kotlinx.coroutines.launch
import java.text.SimpleDateFormat
import java.util.*

/**
 * @author Sasuke on 2019-05-06 0006.
 */
class MachineEngineerModel(val repo: MachineCallRepo) : ViewModel() {
    var picUrl = ObservableField<String>()

    var unDealNum = ObservableField<Int>(0)

    /**我的门店数*/
    var storeCount = ObservableField<Int>(0)

    /**门店灯亮故障数*/
    var lightCount = ObservableField<Int>(0)

    /**我的门店中的有销量的门店数*/
    var storeHasSaleCount = ObservableField<Int>(0)


    var unDealResCache = MutableLiveData<UndealRepairRes>()

    /**我的门店缓存*/
    var myStoresCache = MutableLiveData<List<EngineerMyStoresRes.ResultInfo>>()

    /**门店亮灯故障缓存*/
    var lightProblemCache = MutableLiveData<LightProblemRes.ResultInfo>()

    /**用来控制下拉刷新*/
    var isRefreshing = MutableLiveData<Boolean>().apply { value = false }

    /**loadWaitDominator显示文字*/
    var loadWaitMsg = MutableLiveData<Int>().apply { value = -1 }

    /**
     * 我的工单点击
     */
    fun onClickMyBill(view: View) {
//        unDealNum.get()?.takeIf { it > 0 }?.let {
        view.context.run {
            startActivity(Intent(this, EngineerBillsAct::class.java).apply {
                //传啥传传传。。。
//                    putExtra(EngineerBillsAct.INTENT_RES, unDealResCache.value)
            })
        }
//        }
    }

    /**
     * 我的门店点击
     */
    fun onClickMyStores(view: View) {
//        unDealNum.get()?.takeIf { it > 0 }?.let {
        EngineerMyStoresAct.launch(view.context, storeCount.get() ?: 0, storeHasSaleCount.get()
                ?: 0)
//        }
    }

    /**
     * 门店灯亮故障点击
     */
    fun onClickLightProblem(view: View) {
        LightProblemStoresAct.launch(view.context)
    }

    /**
     * 巡店检查
     */
    fun onClickPatrol(view: View) {
        //巡店检查计划表
        PatrolActList.launch(view.context)
    }

    /**
     * 出差上报
     */
    fun onClickTripRecord(view: View) {
        MachineTripAct.launch(view.context)
    }

    /**
     * 获取未处理单
     */
    fun queryUnDealRepair() {
        viewModelScope.launch {
            try {
                isRefreshing.value = true//开始转圈圈
                unDealResCache.value = repo.queryUnDealRepair(SystemHelper.LoginUserId, SimpleDateFormat("yyyy-MM").format(Date()))
                unDealNum.set(unDealResCache.value?.resultInfo?.details?.count { it.status == 0 })
            } catch (e: Exception) {
//                MyToast.show(e.message)
            }
            isRefreshing.value = false//停止转圈圈
        }
    }


    /**
     * 获取我的门店
     */
    fun queryMyStores() = viewModelScope.launch {
        try {
            isRefreshing.value = true//开始转圈圈
            myStoresCache.value = repo.queryMyStores(SystemHelper.LoginUserId)
            storeCount.set(myStoresCache.value?.size)
            storeHasSaleCount.set(myStoresCache.value?.count { it.isSaledLastMonth == 1 })
        } catch (e: Exception) {
//            MyToast.show(e.message)
        }
        isRefreshing.value = false//停止转圈圈
    }

    /**
     * 获取门店亮灯故障
     */
    fun queryLightProblem() = viewModelScope.launch {
        loadWaitMsg.value = LoadWaitDominator.LOADING
        try {
            isRefreshing.value = true//开始转圈圈
            lightProblemCache.value = repo.queryLightProblem(SystemHelper.LoginUserId, SimpleDateFormat("yyyy-MM").format(Date()))
            lightCount.set(lightProblemCache.value?.notlightNum)
            //界面提示
            if (lightProblemCache.value?.detailList?.isEmpty() ?: true) {
                loadWaitMsg.value = LoadWaitDominator.NO_DATA
            } else {
                loadWaitMsg.value = LoadWaitDominator.DISMISS
            }
        } catch (e: Exception) {
            loadWaitMsg.value = LoadWaitDominator.NET_ERROR
//            MyToast.show(e.message)
        }
        isRefreshing.value = false//停止转圈圈
    }

    /**
     * 门店订货点击
     */
    fun onClickOrderGoods(view: View) {
        ToActivity.toActivity(view.context, OrderGoodsTabAct::class.java)
    }

    /**
     * 系统设置点击
     */
    fun onClickSystemSetting(view: View) {
        view.context.run { startActivity(Intent(this, SttingMenu::class.java)) }
    }


    /**
     * 上传头像（已传到七牛，这里是把七牛返回的地址上传到我们的服务器）
     */
    fun uploadUserPic(staffId: String, url: String) {
        viewModelScope.launch {
            val res = repo.uploadUserPic(staffId, url)
            if (res.msgCode == 0) {
                getUserPic(staffId)
            }
        }
    }

    /**
     * 下载头像
     */
    fun getUserPic(staffid: String) {
        viewModelScope.launch {
            try {
                val url = repo.getUserPic(staffid)
                picUrl.set(url)
            } catch (e: Exception) {
            }
        }
    }


    /**
     * 点击打卡
     */
    fun onClickSignIn(view: View) {
        ToActivity.toActivity(view.context, SignInAndOutActivity::class.java,
                SystemHelper.LoginOrganId,
                SystemHelper.LoginUserName,
                SystemHelper.LoginUserId)
    }
}