package com.urovo.activity.machinerepairact.model

import android.view.View
import androidx.databinding.ObservableField
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import com.amap.api.location.AMapLocation
import com.urovo.activity.machinerepairact.net.PatrolItemRes
import com.urovo.activity.machinerepairact.net.PatrolSubmitEntity
import com.urovo.activity.machinerepairact.repository.MachineCallRepo
import com.urovo.wugumofang.system.SystemHelper
import com.urovo.wugumofang.utils.MyToast
import com.zsy.libcore.common.android.log.Log
import kotlinx.coroutines.launch

/**
 * @author Sasuke on 2019-9-23 0023.
 */
class PatrolModel(val repo: MachineCallRepo) : ViewModel() {
    val MAX_PHOTO_NUM = 3

    /**当前位置*/
    var currLocation = ObservableField<AMapLocation>()

    /**项目数据*/
    var itemsData = MutableLiveData<List<PatrolItemRes.ResultInfo>>()

    var showProgressText = MutableLiveData<String>()

    var isFinish = MutableLiveData<Boolean>()
    /**
     * 照片路径缓存
     */
    var photoPool = mutableListOf<String>()

    /**弹窗文字*/
    var alertDialogState = MutableLiveData<String>()

    /**提交*/
    fun onClickSubmit(view: View) {
        viewModelScope.launch {
            if (itemsData.value.isNullOrEmpty()) {
                MyToast.show("数据为空，无法提交")
                return@launch
            }
            //只要有一个不属于{0 ,1}的，就说明没有选完，那就不允许提交。
            if (itemsData.value?.count { !(it.itemResult in 0..1) } ?: 0 > 0) {
                MyToast.show("所有检查项目全部确认完成后才可提交！")
                return@launch
            }
            showProgressText.value = "正在校验地理位置..."
            val checkRes = repo.checkLocation(SystemHelper.LoginUserId, SystemHelper.LoginOrganId, currLocation.get()!!)
            val locationCorrect = when (checkRes.msgCode) {
                "0" -> "y"
                else -> "n"
            }
            showProgressText.value = ""
            alertDialogState.value = locationCorrect
        }
    }

    /**真正提交 */
    fun realSubmit(locationCorrect: String) = viewModelScope.launch {
        //校验通过，开始上传
        try {
            var picUrl = ""
            if (!photoPool.isEmpty()) {
                showProgressText.value = "正在上传图片..."
                picUrl = repo.submitPhotos2Qiniu(photoPool)
            }
//            Log.i("SasukeLog", "图片上传完成")

            showProgressText.value = "正在提交巡检数据..."
            val result = repo.submitPatrolItems(PatrolSubmitEntity(
                    locationCorrect,
                    picUrl,
                    SystemHelper.LoginOrganId,
                    SystemHelper.LoginUserId,
                    itemsData.value!!))//不怕，前面有防空校验
            if (result.msgCode == 0) {
                MyToast.show("提交成功")
                isFinish.value = true
                showProgressText.value = ""
            }
        } catch (e: Exception) {
            e.printStackTrace()
            MyToast.show(e.message)
            showProgressText.value = ""
        }
    }

    /***
     * 查询项目
     */
    fun queryItems() = viewModelScope.launch {
        try {
            val items = repo.queryPatrolItems()
            itemsData.value = items
        } catch (e: Exception) {
            MyToast.show(e.message)
        }
    }

}