package com.urovo.activity.machinerepairact.repository

import androidx.databinding.ObservableArrayList
import androidx.lifecycle.MutableLiveData
import com.amap.api.location.AMapLocation
import com.google.gson.Gson
import com.urovo.activity.machinerepairact.net.*
import com.urovo.activity.machinetrip.net.PatrolReportRes
import com.urovo.wgmf.tool.netUtil.QiNiuDominator
import com.urovo.wugumofang.constants.ApiUrls
import com.urovo.wugumofang.constants.NetParam
import com.urovo.wugumofang.data.JsonDominator
import com.urovo.wugumofang.net.OkHttpDominator
import com.urovo.wugumofang.response.Base3Response
import com.urovo.wugumofang.response.BaseOldResponse
import com.urovo.wugumofang.response.BaseSimpleResponse
import com.urovo.wugumofang.system.SystemHelper
import com.wgmf.trade.model.bean.UserHeadResBean
import okhttp3.Request
import java.text.SimpleDateFormat
import java.util.*
import kotlin.coroutines.resume
import kotlin.coroutines.resumeWithException
import kotlin.coroutines.suspendCoroutine

/**
 * @author Sasuke on 2019/4/27.
 */
class MachineCallRepo {

    /**
     * 营养顾问 - 获取机修人员信息
     */
    suspend fun getRepairerInfo(storeNo: String) = suspendCoroutine<MachinRepairerRes?> { conti ->
        OkHttpDominator.postAsyn(NetParam.YINXIAOBU_SERVER + ApiUrls.APPSALER.QUERY_MR_OPERATOR, mapOf("storeNo" to storeNo), object : OkHttpDominator.ResultCallback<MachinRepairerRes>() {
            override fun onError(request: Request?, e: Exception?) {
                e?.let { conti.resumeWithException(e.fillInStackTrace()) }
            }

            override fun onResponse(res: MachinRepairerRes?) {
                res?.let {
                    conti.resume(res)
                }
            }
        })
    }

    /**
     * 查询机修单申请人的电话号码
     */
    suspend fun queryStaffPhoneMr(userNo: String) = suspendCoroutine<String> { conti ->
        OkHttpDominator.postAsyn(NetParam.YINXIAOBU_SERVER + ApiUrls.APPSALER.QUERY_STAFF_PHONE_MR,
                mapOf("userNo" to userNo), object : OkHttpDominator.ResultCallback<BaseSimpleResponse>() {
            override fun onError(request: Request?, e: java.lang.Exception?) {

            }

            override fun onResponse(res: BaseSimpleResponse?) {
                res?.let { if (res.msgCode == 0) conti.resume(res.resultInfo) }
            }
        })
    }


    /**
     * 营养顾问 - 提交机修工单
     */
    suspend fun submitMRepairBill(params: Map<String, String?>) = suspendCoroutine<Base3Response?> { conti ->
        OkHttpDominator.postAsyn(NetParam.YINXIAOBU_SERVER + ApiUrls.APPSALER.CREAT_MACHINE_REPAIR,
                params
                , object : OkHttpDominator.ResultCallback<Base3Response>() {
            override fun onError(request: Request?, e: java.lang.Exception) {
                conti.resumeWithException(e)
            }

            override fun onResponse(res: Base3Response?) {
                conti.resume(res)
            }
        })
    }

    /**
     * 把图片传到七牛
     */
    suspend fun submitPhotos2Qiniu(photoPaths: MutableList<String>) = suspendCoroutine<String> { conti ->
        if (photoPaths.size == 0) {
            conti.resume("")
            return@suspendCoroutine
        }
        QiNiuDominator.uploadImages(photoPaths,
                { qiniuUrl ->
                    //CompleteCallback
                    //照片七牛url组装
                    var packUrls = ""
                    qiniuUrl.forEach { packUrls += "${it}," }
                    if (packUrls.isNotEmpty()) packUrls = packUrls.substring(0, packUrls.length - 1)
                    conti.resume(packUrls)
                },
                { str ->
                    //FailedCallback
                    conti.resumeWithException(Exception(str))
                }, false)
    }

    /**
     * 营养顾问 - 查询机修记录
     */
    suspend fun queryHistory(userNo: String, ym: String) = suspendCoroutine<List<MachinRepairHistoryRes.ResultInfo?>> { conti ->
        OkHttpDominator.postAsyn(NetParam.YINXIAOBU_SERVER + ApiUrls.APPSALER.QUERY_MR_HISTORY,
                mapOf("userNo" to userNo, "ym" to ym.replace("年", "-").replace("月", "")),
                object : OkHttpDominator.ResultCallback<MachinRepairHistoryRes>() {
                    override fun onError(request: Request?, e: java.lang.Exception) {
                        conti.resumeWithException(e)
                    }

                    override fun onResponse(res: MachinRepairHistoryRes?) {
                        if (res?.msgCode != 0) {
                            conti.resumeWithException(Exception(res?.msgInfo))
                            return
                        }
                        if (res?.resultInfo.isNullOrEmpty()) {
                            conti.resumeWithException(Exception("暂无数据"))
                            return
                        }

                        conti?.resume(res?.resultInfo!!)
                    }
                })
    }

    /**
     * 营养顾问 - 查询工单详情
     */
    suspend fun queryMachineDetail(mroId: Int) = suspendCoroutine<MachineDetailRes> { conti ->
        OkHttpDominator.postAsyn(NetParam.YINXIAOBU_SERVER + ApiUrls.APPSALER.QUERY_MR_DETAIL, mapOf("mroId" to mroId.toString()),
                object : OkHttpDominator.ResultCallback<MachineDetailRes>() {
                    override fun onError(request: Request?, e: java.lang.Exception) {
                        conti.resumeWithException(e)
                    }

                    override fun onResponse(response: MachineDetailRes?) {
                        var res = response
                        JsonDominator.loadFromSD(MachineDetailRes::class.java)?.let { res = it }
                        if (res == null || res?.resultInfo == null) {
                            conti.resumeWithException(Exception("无数据"))
                            return
                        }
                        conti.resume(res!!)
                    }
                })
    }


    /**
     * 营养顾问 - 提交机修评价
     */
    suspend fun submitRepairDetailRating(mroId: Int?, rating: Int?, content: String) = suspendCoroutine<Base3Response> { conti ->
        OkHttpDominator.postAsyn(NetParam.YINXIAOBU_SERVER + ApiUrls.APPSALER.SUBMIT_MR_RATING,
                mapOf("mroId" to mroId?.toString(), "rating" to rating?.toString(), "content" to content),
                object : OkHttpDominator.ResultCallback<Base3Response>() {
                    override fun onError(request: Request?, e: java.lang.Exception) {
                        conti.resumeWithException(e)
                    }

                    override fun onResponse(res: Base3Response?) {
                        res?.let { conti.resume(res) }
                    }
                })
    }

    /**
     * 请求头像
     */
    suspend fun getUserPic(staffid: String) = suspendCoroutine<String?> { conti ->
        OkHttpDominator.postAsyn(NetParam.YINXIAOBU_SERVER + ApiUrls.APPSALER.USER_GETHEADPIC,
                mapOf("staffid" to staffid),
                object : OkHttpDominator.ResultCallback<UserHeadResBean>() {
                    override fun onError(request: Request?, e: java.lang.Exception) {
                        conti.resumeWithException(e)
                    }

                    override fun onResponse(res: UserHeadResBean?) {
                        res?.let { conti.resume(res.result?.headpicurl) }
                    }
                })
    }

    /**
     * 上传用户头像
     */
    suspend fun uploadUserPic(staffId: String, url: String) = suspendCoroutine<Base3Response> { conti ->
        OkHttpDominator.postAsyn(NetParam.YINXIAOBU_SERVER + ApiUrls.APPSALER.USER_UPLOAD,
                mapOf("staffId" to staffId, "url" to url), object : OkHttpDominator.ResultCallback<Base3Response>() {
            override fun onError(request: Request?, e: java.lang.Exception) {
                conti.resumeWithException(e)
            }

            override fun onResponse(res: Base3Response) {
                conti.resume(res)
            }
        })
    }

    /**
     * 机修人员 - 查询未处理工单
     * 注：这里的ym随便填，传任何月份后台都会把所有的未处理单都返回回来的。
     */
    suspend fun queryUnDealRepair(userNo: String, ym: String) = suspendCoroutine<UndealRepairRes> { conti ->
        OkHttpDominator.postAsyn(NetParam.YINXIAOBU_SERVER + ApiUrls.APPSALER.QUERY_UNDEAL_REPAIR,
                mapOf("userNo" to userNo, "ym" to ym),
                object : OkHttpDominator.ResultCallback<UndealRepairRes>() {
                    override fun onError(request: Request?, e: java.lang.Exception) {
                        conti.resumeWithException(e)
                    }

                    override fun onResponse(res: UndealRepairRes?) {
                        var resMock = res
                        if (resMock?.msgCode != 0) {
                            if (!resMock?.msgInfo.isNullOrEmpty()) {
                                conti.resumeWithException(Exception(resMock?.msgInfo))
                                return
                            }
                            conti.resumeWithException(Exception("暂无数据"))
                            return
                        }
                        JsonDominator.loadFromSD(UndealRepairRes::class.java)?.let {
                            resMock = it
                        }
                        resMock?.let {
                            conti.resume(it)
                        }
                    }
                })
    }

    /**查询巡店记录*/
    suspend fun queryPatrolReport(userNo: String, yearMonth: String) = suspendCoroutine<List<PatrolReportRes.ResultInfo>> { conti ->
        OkHttpDominator.postAsyn(NetParam.YINXIAOBU_SERVER + ApiUrls.APPSALER.QUERY_PATROL_REPORT,
                mapOf("userNo" to userNo, "yearMonth" to yearMonth),
                object : OkHttpDominator.ResultCallback<PatrolReportRes>() {
                    override fun onError(request: Request?, e: java.lang.Exception) {
                        conti.resumeWithException(e)
                    }

                    override fun onResponse(res: PatrolReportRes?) {
                        res?.let {
                            if (res.msgCode != 0) {
                                conti.resumeWithException(Exception(res.msgInfo))
                                return
                            }
                            conti.resume(res.resultInfo)
                        }
                    }
                })
    }


    /**
     * 机修人员 - 查询我的门店
     */
    suspend fun queryMyStores(userNo: String) = suspendCoroutine<List<EngineerMyStoresRes.ResultInfo>> { conti ->
        OkHttpDominator.postAsyn(NetParam.YINXIAOBU_SERVER + ApiUrls.APPSALER.QUERY_ENGINEER_STORES_INFO,
                mapOf("userNo" to userNo),
                object : OkHttpDominator.ResultCallback<EngineerMyStoresRes>() {
                    override fun onError(request: Request?, e: java.lang.Exception) {
                        conti.resumeWithException(e)
                    }

                    override fun onResponse(res: EngineerMyStoresRes?) {
                        var resMock = res
                        resMock?.let {
                            conti.resume(it.resultInfo)
                        }
                    }
                })
    }

    /**
     * 机修人员 - 查询门店亮灯故障
     */
    suspend fun queryLightProblem(userNo: String, yearMonth: String) = suspendCoroutine<LightProblemRes.ResultInfo> { conti ->
        OkHttpDominator.postAsyn(NetParam.YINXIAOBU_SERVER + ApiUrls.APPSALER.QUERY_ENGINEER_LIGHT_PROBLEM,
                mapOf("userNo" to userNo, "yearMonth" to yearMonth),
                object : OkHttpDominator.ResultCallback<LightProblemRes>() {
                    override fun onError(request: Request?, e: java.lang.Exception) {
                        conti.resumeWithException(e)
                    }

                    override fun onResponse(res: LightProblemRes?) {
                        var resMock = res
                        resMock?.let {
                            conti.resume(it.resultInfo)
                        }
                    }
                })
    }

    /**
     * 机修人员 - 查询工单详情
     */
    suspend fun queryRepairDetail(mroId: Int) = suspendCoroutine<EngineerDetailRes> { conti ->
        OkHttpDominator.postAsyn(NetParam.YINXIAOBU_SERVER + ApiUrls.APPSALER.QUERY_REPAIR_DETAIL,
                mapOf("mroId" to mroId.toString()),
                object : OkHttpDominator.ResultCallback<EngineerDetailRes>() {
                    override fun onError(request: Request?, e: java.lang.Exception) {
                        conti.resumeWithException(e)
                    }

                    override fun onResponse(res: EngineerDetailRes?) {
                        res?.let {
                            conti.resume(res)
                        }
                    }
                })
    }

    /**
     * 验证地理位置
     */
    suspend fun checkLocation(userId: String, storeId: String, location: AMapLocation) = suspendCoroutine<BaseOldResponse> { conti ->
        OkHttpDominator.postAsyn(NetParam.YINXIAOBU_SERVER + ApiUrls.APPSALER.SIGN_IN_OR_OUT_LOCATION_CHECK,
                mapOf("userId" to userId,
                        "storeId" to storeId,
                        "requestType" to "1",
                        "lng" to location?.longitude.toString(),
                        "lat" to location?.latitude.toString(),
                        "userType" to "2",
                        "address" to location?.address,
                        "type" to "1",
                        "machine" to "1",
                        "signTime" to ""), object : OkHttpDominator.ResultCallback<BaseOldResponse>() {
            override fun onError(request: Request?, e: java.lang.Exception) {
                conti.resumeWithException(e)
            }

            override fun onResponse(res: BaseOldResponse?) {
                res?.let {
                    conti.resume(res)
                }
            }
        })
    }

    /**
     * 机修人员 - 提交工单处理
     */
    data class PhotoParam(var picAfter: String? = "", var picMachineInside: String? = "", var picOtherDevices: String? = "", var picImageShop: String? = "")

    suspend fun uploadDealRepair(mroId: Int,
                                 handleWay: String,
                                 handleDesc: String,
                                 finishTime: String,
                                 locationCorrect: String?,
                                 photos: PhotoParam?) = suspendCoroutine<Base3Response> { conti ->
        var params = mapOf("mroId" to mroId.toString(),
                "handleWay" to handleWay,
                "handleDesc" to handleDesc,
                "finishTime" to finishTime + ":00",
                "locationCorrect" to locationCorrect)
        if (photos != null) {
            params = params.plus(mapOf("picAfter" to (photos?.picAfter
                    ?: ""), "picMachineInside" to (photos?.picMachineInside
                    ?: ""), "picOtherDevices" to (photos?.picOtherDevices
                    ?: ""), "picImageShop" to (photos?.picImageShop ?: "")))
        }
        OkHttpDominator.postAsyn(NetParam.YINXIAOBU_SERVER + ApiUrls.APPSALER.SUBMIT_REPAIR_DEAL,
                params, object : OkHttpDominator.ResultCallback<Base3Response>() {
            override fun onError(request: Request?, e: java.lang.Exception) {
                conti.resumeWithException(e)
            }

            override fun onResponse(res: Base3Response?) {
                res?.let {
                    conti.resume(res)
                }
            }
        })
    }

    /**
     * 机修人员 - 获取巡店检查项目
     */
    suspend fun queryPatrolItems() = suspendCoroutine<List<PatrolItemRes.ResultInfo>> { conti ->
        OkHttpDominator.postAsyn(NetParam.YINXIAOBU_SERVER + ApiUrls.APPSALER.QUERY_PATROL_ITEMS,
                null, object : OkHttpDominator.ResultCallback<PatrolItemRes>() {
            override fun onError(request: Request?, e: java.lang.Exception) {
                e.printStackTrace()
                conti.resumeWithException(e)
            }

            override fun onResponse(res: PatrolItemRes) {
                res?.let {
                    if (res.msgCode != 0) {
                        conti.resumeWithException(Exception(res.msgInfo))
                        return
                    }
                    if (res.resultInfo.isNullOrEmpty()) {
                        conti.resumeWithException(Exception("返回为空"))
                        return
                    }
                    conti.resume(res.resultInfo)
                }
            }
        })
    }

    /**
     * 机修人员 - 获取巡店检查计划
     */
    suspend fun queryPatrolListItems() = suspendCoroutine<List<PatrolItemListRes.ResultInfo>> { conti ->
        var params = mapOf("ym" to SimpleDateFormat("yyyy-MM").format(Date()),
                "operatorNo" to SystemHelper.LoginUserId)
        OkHttpDominator.postAsyn(NetParam.YINXIAOBU_SERVER + ApiUrls.APPSALER.QQUERY_PATROL_SCHEDULE,
                params, object : OkHttpDominator.ResultCallback<PatrolItemListRes>() {
            override fun onError(request: Request?, e: java.lang.Exception) {
                e.printStackTrace()
                conti.resumeWithException(e)
            }

            override fun onResponse(res: PatrolItemListRes) {
                res?.let {
                    if (res.msgCode != 0) {
                        conti.resumeWithException(Exception(res.msgInfo))
                        return
                    }
                    if (res.resultInfo.isNullOrEmpty()) {
                        conti.resumeWithException(Exception("返回为空"))
                        return
                    }
                    conti.resume(res.resultInfo)
                }
            }
        })
    }

    /**
     * 机修人员 - 提交巡店检查
     */
    suspend fun submitPatrolItems(submitEntity: PatrolSubmitEntity) = suspendCoroutine<Base3Response> { conti ->
        OkHttpDominator.postAsynByType(NetParam.YINXIAOBU_SERVER + ApiUrls.APPSALER.SUBMIT_PATROL_ITEMS,
                OkHttpDominator.JSON, mapOf("patrol" to Gson().toJson(submitEntity)), object : OkHttpDominator.ResultCallback<Base3Response>() {
            override fun onError(request: Request?, e: java.lang.Exception) {
                e.printStackTrace()
                conti.resumeWithException(e)
            }

            override fun onResponse(res: Base3Response) {
                res?.let {
                    if (res.msgCode != 0) {
                        conti.resumeWithException(Exception(res.msgInfo))
                        return
                    }
                    conti.resume(res)
                }
            }
        })
    }


    /**七牛递归上传图片*/
    suspend fun uploadQiniu777(localPath: ObservableArrayList<String>, qiniuCache: MutableLiveData<String>) = suspendCoroutine<String> { conti ->
        QiNiuDominator.uploadImages(listOf(localPath[0]),
                { qiniuUrl -> qiniuCache?.value = qiniuUrl[0]; conti.resume(QiNiuDominator.SUCCESS) },
                { failInfo ->
                    conti.resumeWithException(Exception(failInfo))
                }, false)
    }
}