package com.urovo.activity.machinerepairact.view

import android.content.Context
import android.content.Intent
import android.graphics.Color
import android.os.Bundle
import androidx.databinding.DataBindingUtil
import androidx.lifecycle.Observer
import androidx.lifecycle.ViewModelProviders
import androidx.recyclerview.widget.LinearLayoutManager
import com.urovo.activity.machinerepairact.model.EngineerMyStoreFactory
import com.urovo.activity.machinerepairact.model.EngineerMyStoreModel
import com.urovo.activity.machinerepairact.repository.MachineCallRepo
import com.urovo.activity.machinerepairact.ui.EngineerMyStoresAdapter
import com.urovo.wgmf.ext.afterTextChanged
import com.urovo.wugumofang.R
import com.urovo.wugumofang.base.BaseActivity
import com.urovo.wugumofang.databinding.EngineerMyStoresActBinding
import com.urovo.wugumofang.view.DividerItemDecoration
import com.urovo.wugumofang.view.LoadWaitDominator

/**
 * @author Sasuke on 2019-7-31 0031.
 */
class EngineerMyStoresAct : BaseActivity() {
    companion object {
        @JvmStatic
        fun launch(context: Context, storeCount: Int, storeHasSaleCount: Int) {
            context.startActivity(Intent(context, EngineerMyStoresAct::class.java).apply {
                putExtra("storeCount", storeCount)
                putExtra("storeHasSaleCount", storeHasSaleCount)
            })
        }

        @JvmStatic
        //传一个门店名称进来，用来过滤结果。用于只需要显示某个门店详情的情况。
        fun launch(context: Context, storeNameFilter: String) {
            context.startActivity(Intent(context, EngineerMyStoresAct::class.java).apply {
                putExtra("storeNameFilter", storeNameFilter)
            })
        }
    }

    val binding by lazy {
        DataBindingUtil.setContentView<EngineerMyStoresActBinding>(
                this, R.layout.engineer_my_stores_act)
    }
    val actModel by lazy { ViewModelProviders.of(this, EngineerMyStoreFactory(MachineCallRepo())).get(EngineerMyStoreModel::class.java) }

    var loadWait: LoadWaitDominator? = null

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        bind()
        initTitle()
        init()
        actModel.queryMyStores(intent.getStringExtra("storeNameFilter"))
    }

    fun bind() {
        binding.run {
            titleText = "我的门店"
            model = actModel
        }

        actModel.storeCount.set( intent.getIntExtra("storeCount",0))
        actModel.storeHasSaleCount.set( intent.getIntExtra("storeHasSaleCount",0))
        actModel.loadWaitMsg.observe(this, Observer {
            when (it) {
                -1 -> loadWait?.dismissAll()
                LoadWaitDominator.LOADING -> loadWait?.show(LoadWaitDominator.LOADING)
                LoadWaitDominator.NET_ERROR -> loadWait?.show(LoadWaitDominator.NET_ERROR)
                LoadWaitDominator.NO_DATA -> loadWait?.show(LoadWaitDominator.NO_DATA)
            }
        })

        //搜索监听
        binding.searchInput.afterTextChanged {
            refreshRv()
            if (actModel.showListCache.isEmpty()){
                actModel.loadWaitMsg.value = LoadWaitDominator.NO_DATA
            }else{
                actModel.loadWaitMsg.value = -1
            }
        }

        //数据被更新
        actModel.myStoresCache.observe(this, Observer {
            refreshRv()
        })
    }

    fun init() {
        loadWait = LoadWaitDominator(this, binding.container)
        binding.rV.layoutManager = LinearLayoutManager(this)
        binding.rV.addItemDecoration(DividerItemDecoration(this, DividerItemDecoration.HORIZONTAL_LIST, 1, Color.parseColor("#e8e8e8")))

        refreshRv()
    }

    fun refreshRv(){
        binding.rV.adapter = EngineerMyStoresAdapter(this@EngineerMyStoresAct,actModel.showListCache,actModel.searchWord)
    }
}