package com.urovo.activity.machinerepairact.view

import android.content.DialogInterface
import android.content.Intent
import android.graphics.Color
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.widget.RadioButton
import android.widget.RadioGroup
import androidx.appcompat.app.AlertDialog
import androidx.databinding.DataBindingUtil
import androidx.lifecycle.Observer
import androidx.lifecycle.ViewModelProviders
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.viewpager.widget.ViewPager
import com.urovo.activity.common.DatePickerYearMonthDialog
import com.urovo.activity.machinerepairact.model.MachineCallFactory
import com.urovo.activity.machinerepairact.model.MachineCallModel
import com.urovo.activity.machinerepairact.net.MachinRepairHistoryRes
import com.urovo.activity.machinerepairact.repository.MachineCallRepo
import com.urovo.activity.machinerepairact.ui.HistoryAdapter
import com.urovo.wgmf.adapter.MyPagerAdapter
import com.urovo.wugumofang.R
import com.urovo.wugumofang.base.BaseActivityTap
import com.urovo.wugumofang.databinding.MachineRepairActBinding
import com.urovo.wugumofang.databinding.MachineRepairCallBinding
import com.urovo.wugumofang.databinding.MachineRepairHistoryBinding
import com.urovo.wugumofang.system.SystemHelper
import com.urovo.wugumofang.ui.DensityUtil
import com.urovo.wugumofang.ui.PhotoPickDominator
import com.urovo.wugumofang.ui.ViewUtil
import com.urovo.wugumofang.view.DividerItemDecoration
import com.wgmf.trade.ui.ada.ObservableListChangeListener
import kotlinx.android.synthetic.main.machine_repair_act.*
import kotlinx.android.synthetic.main.machine_repair_call.*
import org.jetbrains.anko.find
import java.text.SimpleDateFormat
import java.util.*

/**
 * @author Sasuke on 2019-04-23 0023.
 * 呼叫机修
 */
class MachineRepairAct : BaseActivityTap() {
    val callModel by lazy { ViewModelProviders.of(this, MachineCallFactory(MachineCallRepo(), application)).get(MachineCallModel::class.java) }
    var picker: PhotoPickDominator? = null

    val mainBinding by lazy {
        DataBindingUtil.setContentView<MachineRepairActBinding>(this, R.layout.machine_repair_act)
    }

    //呼叫
    val callBinding by lazy {
        //这个CallMachineRepairBinding 是根据布局文件的名字来的
        DataBindingUtil.inflate<MachineRepairCallBinding>(LayoutInflater.from(this), R.layout.machine_repair_call, null, false)
    }
    //记录
    val historyBinding by lazy {
        DataBindingUtil.inflate<MachineRepairHistoryBinding>(LayoutInflater.from(this), R.layout.machine_repair_history, null, false)
    }

    var contentPager: ViewPager? = null//真是奇了怪了，为什么Kotlin不能直接用id找到它呢？非要我findView
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        bind()
        initTitle()
        init()
        setListeners()
        callModel.getRepairerInfo(SystemHelper.LoginOrganId)
        callModel.getStaffPhone(SystemHelper.LoginUserId)
    }

    fun init() {
        contentPager = findViewById(R.id.contentPager)
        picker = PhotoPickDominator(this, callModel.photoPool)
        contentPager?.adapter = MyPagerAdapter(this, arrayListOf(callBinding?.root, historyBinding?.root))
        //viewpager与标题同步变化
        repairTapTitle.find<RadioGroup>(R.id.tap_radio).setOnCheckedChangeListener { group, checkedId ->
            when (checkedId) {
                R.id.tap_button_left -> contentPager?.setCurrentItem(0, true)
                R.id.tap_button_right -> {
                    contentPager?.setCurrentItem(1, true)
//                    callModel.historyDate.run { if (value == null) value = SimpleDateFormat("yyyy年MM月").format(Date()) }
                    if (!historyBinding.unDeald.isChecked && !historyBinding.dealed.isChecked) {//如果两个标签都没有check
                        historyBinding.unDeald.isChecked = true//那就手动check一个，激发请求
                    }
                }
            }
        }
        contentPager?.addOnPageChangeListener(object : ViewPager.OnPageChangeListener {
            override fun onPageScrollStateChanged(state: Int) {}
            override fun onPageScrolled(position: Int, positionOffset: Float, positionOffsetPixels: Int) {}
            override fun onPageSelected(position: Int) {
                when (position) {
                    0 -> repairTapTitle.find<RadioButton>(R.id.tap_button_left).isChecked = true
                    1 -> repairTapTitle.find<RadioButton>(R.id.tap_button_right).isChecked = true
                }
            }
        })
        val rvAdapter = HistoryAdapter(this, callModel.historyShow)
        historyBinding.historyRv.run {
            layoutManager = LinearLayoutManager(this@MachineRepairAct)
            addItemDecoration(DividerItemDecoration(context, DividerItemDecoration.VERTICAL_LIST, 1, Color.parseColor("#E3E7EE")))
            adapter = rvAdapter
        }
        callModel.historyShow.addOnListChangedCallback(ObservableListChangeListener<MachinRepairHistoryRes.ResultInfo>(rvAdapter))

        var isFromJpush = intent.getBooleanExtra("FROM_JPUSH", false)
        if (isFromJpush) {
            mainBinding.repairTapTitle.tapButtonRight.isChecked = true
            historyBinding.dealed.isChecked = true
        }
    }

    fun bind() {
        //主binding
        mainBinding.model = callModel
        //机修记录binding
        historyBinding.model = callModel
        //呼叫binding
        callBinding?.run {
            callBinding?.model = callModel
            callModel.repairerRes?.observe(this@MachineRepairAct, Observer {
                jixiuName.text = if (it?.resultInfo == null) it?.msgInfo.apply { jixiuName.setTextColor(Color.RED) } else it?.resultInfo?.userName
                jixiuPhone.visibility = if (it?.resultInfo == null) View.INVISIBLE else View.VISIBLE
                jixiuPhone.text = it?.resultInfo?.phone
                if (it?.msgCode != 0) {
                    AlertDialog.Builder(this@MachineRepairAct)
                            .setMessage("无法获取到门店机修信息！点击“确定”按钮返回到更多页面")
                            .setPositiveButton("确定", DialogInterface.OnClickListener { dialogInterface, i ->
                                finish()
                            })
                }
            })//callModel.repairerRes?.observe
        }

        //申请人联系电话

        //没办法，如果想用ViewModel来使Activity关闭，好像只能这样了。用ViewModel的变化来驱动页面的变化
        callModel.isFinish.observe(this, Observer {
            if (it) finish()
        })
        //同上,
        callModel.showProgressText.observe(this, Observer {
            if (it.isNotEmpty()) {
                ViewUtil.showProgress(this, "请稍等", it)
            } else {
                ViewUtil.dismissProgress()
            }
        })

    }

    fun setListeners() {
        //添加照片
        callBinding.cameraAdd.setOnClickListener {
            when (photoGroup.childCount) {
                callModel.MAX_PHOTO_NUM + 2 -> showMsg("最多添加 ${callModel.MAX_PHOTO_NUM} 张图片")//加2是里面还有一个相机添加按钮和文字说明
                else -> picker?.showDialog()
            }
        }
        //提交
        callBinding.submitBtn.setOnClickListener {
            if (callModel.checkData()) {//如果数据合法
                callModel.submitMRepairBill()
            }
        }
        //选择日期
        historyBinding.historyDateLayout.setOnClickListener {
            DatePickerYearMonthDialog(this@MachineRepairAct, { year, monthOfYear ->
                callModel.historyDateSelected(year, monthOfYear)
            }).show()
        }

        //观察日期变化
        historyBinding.dateTitleTv.text = "${SystemHelper.LoginOrganName}\n${SystemHelper.LoginUserName}  ${SimpleDateFormat("yyyy年MM月").format(Date())}"
        callModel.historyDate.observe(this, Observer {
            if (repairTapTitle.find<RadioButton>(R.id.tap_button_right).isChecked) {//如果是机修记录页面
                historyBinding.dateTitleTv.text = "${SystemHelper.LoginOrganName}\n${SystemHelper.LoginUserName}  $it"
                callModel.getRepairHistory(true)
            }
        })

        //监听已处理 未处理按钮状态
        callModel.historyLabel.observe(this, Observer {
            callModel.getRepairHistory(callModel.historyCache.isEmpty())
        })


    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        when (requestCode) {
            MachineRepairDetailAct.REQUEST_CODE -> if (resultCode == MachineRepairDetailAct.HAS_CHANGED) {
                historyBinding.dealed.isChecked = true
                callModel.getRepairHistory(true)
            }
            else -> {
                val photoPath = picker?.let { it.onPhotoPathResult(this, requestCode, resultCode, data) };
                picker?.addPhoto2Group(photoPath,
                        DensityUtil.dip2px(this, 40f),
                        DensityUtil.dip2px(this, 70f),
                        photoGroup)
            }
        }
    }

    override fun onDestroy() {
        super.onDestroy()
        picker?.deletePicCache()
    }

}