package com.urovo.activity.machinerepairact.view

import android.os.Bundle
import androidx.databinding.DataBindingUtil
import androidx.lifecycle.Observer
import androidx.lifecycle.ViewModelProviders
import androidx.recyclerview.widget.GridLayoutManager
import com.urovo.activity.machinerepairact.model.MachineDetailFactory
import com.urovo.activity.machinerepairact.model.MachineDetailModel
import com.urovo.activity.machinerepairact.net.MachinRepairHistoryRes
import com.urovo.activity.machinerepairact.repository.MachineCallRepo
import com.urovo.activity.machinerepairact.ui.MachinePhotoGridAdapter
import com.urovo.wugumofang.R
import com.urovo.wugumofang.base.BaseActivity
import com.urovo.wugumofang.databinding.MachineRepairDetailActBinding
import com.urovo.wugumofang.ui.ViewUtil

/**
 * @author Sasuke on 2019/5/2.
 */
class MachineRepairDetailAct : BaseActivity() {
    companion object {
        //resultCode
        const val HAS_CHANGED = -1080 //改变了数据，要求返回后刷新。
        const val NOT_CHANGED = -1081//没改变数据，返回不需要刷新。
        const val REQUEST_CODE = 1080//没改变数据，返回不需要刷新。

        const val INTENT_RES = "INTENT_RES"
    }

    val model by lazy { ViewModelProviders.of(this, MachineDetailFactory(MachineCallRepo())).get(MachineDetailModel::class.java) }

    val mainBinding by lazy {
        DataBindingUtil.setContentView<MachineRepairDetailActBinding>(this, R.layout.machine_repair_detail_act)
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        bind()
        initTitle()
        model.queryDetail()
    }


    fun bind() {
        model.intentRes.value = intent.getSerializableExtra(INTENT_RES) as MachinRepairHistoryRes.ResultInfo
//        mainBinding.titleText = model.getTitleText()
        mainBinding.pageStatus = model.intentRes.value?.status
        mainBinding.model = model
        //这个是可观察的，所以它有数据后会更新UI
        model.detailResultInfo.observe(this, Observer { result ->
            model.ratingTemp.set(result.rating)
            if (result.rating == 0) result.ratingContent = ""//在未评价状态，后台竟然没有把这个置空。
            model.suggestionContent = result.ratingContent ?:" "
            //绑定到布局
            mainBinding.data = result
            mainBinding.ratingTemp = model.ratingTemp
            result.picBeforeOperation?.let { model.beforePicListShow.run { clear();addAll(it) } }//刷新图片数据
            result.picAfterOperation?.let { model.afterPicListShow.run { clear();addAll(it) } }//刷新图片数据
            mainBinding.suggestionInput.setBackgroundResource( if (result.rating == 0) R.drawable.edit_border else android.R.color.transparent)

            //刷新
            mainBinding.myPhotoRv.adapter?.notifyDataSetChanged()
            mainBinding.repairPhotoRv.adapter?.notifyDataSetChanged()
            mainBinding.model = model//重新绑定
        })

        //图片
        arrayOf(mainBinding.myPhotoRv,mainBinding.repairPhotoRv).forEach {
            it.layoutManager = object :GridLayoutManager(this@MachineRepairDetailAct, 3){
                override fun canScrollVertically(): Boolean {//不允许上下滑动
                    return false
                }
            }
        }
        mainBinding.myPhotoRv.adapter = MachinePhotoGridAdapter( model.beforePicListShow)
        mainBinding.repairPhotoRv.adapter = MachinePhotoGridAdapter( model.afterPicListShow)

        //同上,
        model.showProgressText.observe(this, Observer {
            if (it.isNotEmpty()) {
                ViewUtil.showProgress(this, "请稍等", it)
            } else {
                ViewUtil.dismissProgress()
            }
        })

        //关闭页面观察
        model.finishBy.observe(this,Observer{
            if (it == HAS_CHANGED){
                setResult(HAS_CHANGED)
                finish()
            }
        })
    }

}