package com.urovo.activity.machinerepairact.view

import android.app.AlertDialog
import android.content.Context
import android.content.DialogInterface
import android.content.Intent
import android.os.Build
import android.os.Bundle
import android.os.Handler
import android.provider.Settings
import android.view.View
import androidx.databinding.DataBindingUtil
import androidx.lifecycle.Observer
import androidx.lifecycle.ViewModelProviders
import androidx.recyclerview.widget.LinearLayoutManager
import com.amap.api.maps2d.LocationSource
import com.urovo.activity.attendancemanagement.signinandout.map.MapDominator
import com.urovo.activity.machinerepairact.model.PatrolFactory
import com.urovo.activity.machinerepairact.model.PatrolModel
import com.urovo.activity.machinerepairact.repository.MachineCallRepo
import com.urovo.activity.machinerepairact.ui.PatrolItemsAdapter
import com.urovo.wugumofang.R
import com.urovo.wugumofang.base.BaseActivity
import com.urovo.wugumofang.databinding.PatrolActBinding
import com.urovo.wugumofang.system.SystemHelper
import com.urovo.wugumofang.ui.DensityUtil
import com.urovo.wugumofang.ui.PhotoPickDominator
import com.urovo.wugumofang.ui.ViewUtil

/**
 * @author Sasuke on 2019-9-23 0023.
 * 巡店检查
 */
class PatrolAct : BaseActivity(), LocationSource {
    companion object {
        @JvmStatic
        fun launch(context: Context) {
            context.startActivity(Intent(context, PatrolAct::class.java))
        }
    }

    val binding by lazy { DataBindingUtil.setContentView<PatrolActBinding>(this, R.layout.patrol_act) }

    val patrolModel by lazy { ViewModelProviders.of(this, PatrolFactory(MachineCallRepo())).get(PatrolModel::class.java) }

    var picker: PhotoPickDominator? = null
    var mapDominator: MapDominator? = null
    var isShowLocateToast = false
//    var loadWait: LoadWaitDominator? = null

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        bind()
        initTitle()
        init()
        Handler().postDelayed({
            //加快进入界面的速度，提升体验。
            initMap(savedInstanceState)
            patrolModel.queryItems()
        }, 300)

    }

    fun bind() {
        binding.titleText = "巡店检查"
        binding.model = patrolModel
        binding.storeName = SystemHelper.LoginOrganName
        binding.storeNo = SystemHelper.LoginOrganId
        binding.titleLayout.confirm.apply {
            visibility = View.VISIBLE
            //重新定位按钮样式
            setText("重新定位")
            val drawable = if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
                getResources().getDrawable(R.drawable.ic_location_on_white_24dp, null)
            } else {
                getResources().getDrawable(R.drawable.ic_location_on_white_24dp)
            }
            drawable.setBounds(0, 0, DensityUtil.dip2px(this@PatrolAct, 15f), DensityUtil.dip2px(this@PatrolAct, 22f))
            setCompoundDrawables(drawable, null, null, null)
            //点击监听
            setOnClickListener {
                patrolModel.currLocation.set(null)
                mapDominator?.startLocate()
            }
        }

        //项目数据观察
        patrolModel.itemsData.observe(this, Observer {
            binding.rV.adapter = PatrolItemsAdapter(this@PatrolAct, it)
        })

        patrolModel.showProgressText.observe(this, Observer {
            if (it.isBlank()) {
                ViewUtil.dismissProgress()
            } else {
                ViewUtil.showProgress(this, "提示", it, false)
            }
        })

        patrolModel.isFinish.observe(this, Observer {
            if (it) {
                finish()
            }
        })
        //提交前的提醒
        patrolModel.alertDialogState.observe(this, Observer { locationCorrect ->
            AlertDialog.Builder(this).setTitle("${if (locationCorrect == "n") "异常" else ""}提醒")
                    .setMessage("${if(locationCorrect == "n") "当前位置在门店范围外,将会标记为位置异常,确定要提交吗?" else "确定是否对该门店巡店检查项目进行提交？"}")
                    .setPositiveButton("确定", DialogInterface.OnClickListener { dialogInterface, i -> patrolModel.realSubmit(locationCorrect) })
                    .setNegativeButton("取消", null)
                    .show()
        })
    }

    fun init() {
        picker = PhotoPickDominator(this, patrolModel.photoPool)
//        loadWait = LoadWaitDominator(this,binding.dataContainer)
        binding.rV.apply {
            layoutManager = object : LinearLayoutManager(this@PatrolAct) {
                override fun canScrollVertically(): Boolean {
                    return false
                }
            }
        }
        //添加图片
        binding.cameraAdd.setOnClickListener {
            when (binding.photoGroup.childCount) {
                patrolModel.MAX_PHOTO_NUM + 1 -> showMsg("最多添加 ${patrolModel.MAX_PHOTO_NUM} 张图片")//加1是里面还有一个相机添加按钮
                else -> picker?.showDialog()
            }
        }
    }

    /**
     * 初始化地图
     */
    fun initMap(instanceState: Bundle?) {
        mapDominator = MapDominator.MapBuilder(instanceState, this, binding.mapView)
                .build()//这个只是用来实例化mapDominator，并没有初始化地图
        //一定要跟上面分开，要先build()，再执行 start()
        //这个才是真正的初始化地图参数，里面的 setLocationSource()方法会导致跳转到 activate()方法，激活地图
        mapDominator?.start()/*?.setLocateButton(binding.titleLayout.confirm)*/
    }

    override fun deactivate() {
        mapDominator?.deactivate()
    }

    override fun activate(listener: LocationSource.OnLocationChangedListener?) {
        mapDominator?.activateMap(listener, successCallback = { location ->
            //只有在第0次的时候才弹定位信息Toast
            if (isShowLocateToast) showMsg("成功获取地理信息， 位置在: \n ${location.address}")
            patrolModel.currLocation.set(location)
            isShowLocateToast = true
//            mPre?.isOfflineMode = false//是否离线模式跟定位成功失败与否无关
        }, failedCallback = { location ->
            if (isShowLocateToast) {
                AlertDialog.Builder(this@PatrolAct)
                        .setMessage("获取地理信息失败，请检查设置定位权限")
                        .setNegativeButton("算了", null)
                        .setPositiveButton("去开启权限", { dialogInterface, i -> startActivity(Intent(Settings.ACTION_LOCATION_SOURCE_SETTINGS)) })//跳转系统定位设置
                        .show()
            }
            isShowLocateToast = true
        })


    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        val photoPath = picker?.let { it.onPhotoPathResult(this, requestCode, resultCode, data) };
        picker?.addPhoto2Group(photoPath,
                DensityUtil.dip2px(this, 60f),
                DensityUtil.dip2px(this, 60f),
                binding.photoGroup)
    }

}