package com.urovo.activity.machinetrip.repo

import com.urovo.activity.machinetrip.net.TripBudgetRes
import com.urovo.activity.machinetrip.net.TripRecordDetailRes
import com.urovo.activity.machinetrip.net.TripRecordRes
import com.urovo.wugumofang.constants.ApiUrls
import com.urovo.wugumofang.constants.NetParam
import com.urovo.wugumofang.net.OkHttpDominator
import com.urovo.wugumofang.response.BaseSimpleResponse
import okhttp3.Request
import kotlin.coroutines.resume
import kotlin.coroutines.resumeWithException
import kotlin.coroutines.suspendCoroutine

/**
 * @author Sasuke on 2019-10-23 0023.
 */
class MachineTripRepo {


    /***
     * 判断是否可以插入当天记录
     */
    suspend fun checkCanInsertTravelDay(userNo: String,travelDay: String) = suspendCoroutine<Boolean> { conti->
        OkHttpDominator.postAsyn(NetParam.YINXIAOBU_SERVER + ApiUrls.APPSALER.CHECK_CAN_INSERT_TRIP_RECORD,
                mapOf("userNo" to userNo ,"travelDay" to travelDay),object : OkHttpDominator.ResultCallback<BaseSimpleResponse>(){
            override fun onError(request: Request?, e: java.lang.Exception) {
                conti.resumeWithException(e)
            }

            override fun onResponse(res: BaseSimpleResponse?){
                res?.let {
                    if (res.msgCode != 0){
                        conti.resumeWithException(Exception("日期校验失败"))
                        return
                    }
                    if (res.msgCode == 0 && res.resultInfo == "false" ){
                        conti.resumeWithException(Exception("该日期已有提交记录"))
                        return
                    }
                    conti.resume(true)
                }
            }
        })
    }

    /***
     * 查询差旅预算
     */
    suspend fun queryTripBudget(userNo: String,ym: String) = suspendCoroutine<TripBudgetRes.ResultInfo> { conti->
        OkHttpDominator.postAsyn(NetParam.YINXIAOBU_SERVER + ApiUrls.APPSALER.QUERY_TRIP_BUDGET,
                mapOf("userNo" to userNo,"ym" to ym),object : OkHttpDominator.ResultCallback<TripBudgetRes>(){
            override fun onError(request: Request?, e: Exception) {
                conti.resumeWithException(e)
            }

            override fun onResponse(res: TripBudgetRes?) {
                res?.let {
                    if (res.msgCode != 0){
                        conti.resumeWithException(Exception(res.msgInfo))
                        return
                    }
                    conti.resume(res.resultInfo)
                }
            }
        })
    }

    /**查询出差记录*/
    suspend fun queryTripRecord(userNo: String,ym: String) = suspendCoroutine<List<TripRecordRes.ResultInfo>?> { conti ->
        OkHttpDominator.postAsyn(NetParam.YINXIAOBU_SERVER + ApiUrls.APPSALER.QUERY_TRIP_RECORD,
                mapOf("userNo" to userNo,"ym" to ym),object : OkHttpDominator.ResultCallback<TripRecordRes>(){
            override fun onError(request: Request?, e: java.lang.Exception) {
                conti.resumeWithException(e)
            }

            override fun onResponse(res: TripRecordRes?) {
                res?.let {
                    if (res.msgCode != 0){
                        conti.resumeWithException(Exception(res.msgInfo))
                        return
                    }
                    conti.resume(res.resultInfo)
                }
            }
        })
    }

    /**根据tid查询出差明细*/
    suspend fun queryTripRecordDetail(tid: Int) = suspendCoroutine<TripRecordDetailRes> { conti ->
        OkHttpDominator.postAsyn(NetParam.YINXIAOBU_SERVER + ApiUrls.APPSALER.QUERY_TRIP_RECORD_DETAIL,
                mapOf("tid" to tid.toString()),object : OkHttpDominator.ResultCallback<TripRecordDetailRes>(){
            override fun onError(request: Request?, e: java.lang.Exception) {
                conti.resumeWithException(e)
            }

            override fun onResponse(res: TripRecordDetailRes?) {
                res?.let {
                    if (res.msgCode != 0){
                        conti.resumeWithException(Exception(res.msgInfo))
                        return
                    }
                    if (res.resultInfo == null ){
                        conti.resumeWithException(Exception("暂无数据"))
                    }
                    res.resultInfo?.details?.forEach {
                         it.initUiProperties()//UI显示绑定的是reasonTypeUI这个字段。要同步上去
                    }
                    conti.resume(res)
                }
            }
        })
    }

    /***
     * 新建机修出差记录
     */
    suspend fun submitTripRecord(list: String) = suspendCoroutine<String> {conti ->
        OkHttpDominator.postAsynByType(NetParam.YINXIAOBU_SERVER + ApiUrls.APPSALER.CREATE_NEW_TRIP_RECORD,OkHttpDominator.JSON,
                mapOf("list" to list),object:OkHttpDominator.ResultCallback<BaseSimpleResponse>(){
            override fun onError(request: Request?, e: java.lang.Exception) {
                conti.resumeWithException(e)
            }

            override fun onResponse(res: BaseSimpleResponse?) {
                res?.let{
                    if (res.msgCode != 0 ){
                        conti.resumeWithException(Exception(res.msgInfo ?:""))
                    }
                    conti.resume(res.msgInfo)
                }
            }
        })
    }

    /***
     * 修改机修出差记录
     */
    suspend fun changeTripRecord(trip: String) = suspendCoroutine<String> {conti ->
        OkHttpDominator.postAsynByType(NetParam.YINXIAOBU_SERVER + ApiUrls.APPSALER.UPDATE_TRIP_RECORD,OkHttpDominator.JSON,
                mapOf("list" to trip),object:OkHttpDominator.ResultCallback<BaseSimpleResponse>(){
            override fun onError(request: Request?, e: java.lang.Exception) {
                conti.resumeWithException(e)
            }

            override fun onResponse(res: BaseSimpleResponse?) {
                res?.let {
                    if (res.msgCode != 0 ){
                        conti.resumeWithException(Exception(res.msgInfo ?:""))
                    }
                    conti.resume(res.msgInfo)
                }
            }
        })
    }
}