package com.urovo.activity.machinetrip.view

import android.app.Activity
import android.content.Intent
import android.graphics.Color
import android.os.Bundle
import androidx.appcompat.app.AppCompatActivity
import androidx.databinding.DataBindingUtil
import androidx.lifecycle.Observer
import androidx.lifecycle.ViewModelProviders
import androidx.recyclerview.widget.LinearLayoutManager
import com.urovo.activity.machinerepairact.model.EngineerBillFactory
import com.urovo.activity.machinerepairact.model.EngineerBillModel
import com.urovo.activity.machinerepairact.repository.MachineCallRepo
import com.urovo.activity.machinetrip.ui.WorkNoSelectBillAdapter
import com.urovo.activity.machinetrip.ui.WorkNoSelectPatrolAdapter
import com.urovo.wugumofang.R
import com.urovo.wugumofang.base.BaseActivity
import com.urovo.wugumofang.constants.AppConstants
import com.urovo.wugumofang.databinding.MachineOrderSelectActBinding
import com.urovo.wugumofang.utils.MyToast
import com.urovo.wugumofang.view.DividerItemDecoration
import com.urovo.wugumofang.view.LoadWaitDominator
import com.wgmf.trade.GlobalApp.context
import kotlinx.android.synthetic.main.machine_order_select_act.*

/**
 * @author Sasuke on 2019-10-28 0028.
 * 选择机修单 or 巡店记录的界面
 *
 * 虽说跟EngineerBillsAct.kt功能差不多，但我就是不想到时候一堆分支挤在一起，
 * 到时候万一那边的界面要改，又要加分支，头大。。。
 * 宁可Ctrl+CV拷贝一份代码在这边另写一份界面，也不想重用那边。
 */
class MachineOrderSelectAct : BaseActivity() {
    var loadWait: LoadWaitDominator? = null
    val machineOrderModel by lazy { ViewModelProviders.of(this, EngineerBillFactory(MachineCallRepo())).get(EngineerBillModel::class.java) }
    val binding by lazy { DataBindingUtil.setContentView<MachineOrderSelectActBinding>(this, R.layout.machine_order_select_act) }

    val activityType by lazy { intent.getStringExtra(AppConstants.ExtraKeys.ACTIVITY_TYPE) }

    companion object {
        const val TYPE_MACHINE_BILL = "TYPE_MACHINE_BILL"
        const val TYPE_MACHINE_PATROL = "TYPE_MACHINE_PATROL"
        const val SELECTED_ORDER = "SELECTED_ORDER"

        @JvmStatic
        fun launchForResult(activity: AppCompatActivity, activityType: String?) {
            if (activityType.isNullOrBlank()) {
                MyToast.show("ACTIVITY_TYPE参数为空，无法跳转")
                return
            }
            activity.startActivityForResult(Intent(activity, MachineOrderSelectAct::class.java).apply {
                putExtra(AppConstants.ExtraKeys.ACTIVITY_TYPE, activityType)
            }, 1)
        }
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        bind()
        initTitle()
        init()
    }

    fun bind() {
        binding.model = machineOrderModel.apply { hasDealed.set(true) }//已处理工单模式
        binding.titleText = if (activityType == TYPE_MACHINE_BILL) "选择工单" else "巡店记录选择"
        binding.ym = machineOrderModel.queryYm
        binding.isShowDateTitle = activityType == TYPE_MACHINE_BILL//只有选择工单模式，才显示日期下的文字

        machineOrderModel.activityType = activityType?:""//将出差事由类型传入model
        //已处理工单
        machineOrderModel.unDealResCache.observe(this, Observer {
            binding.billRv.adapter = WorkNoSelectBillAdapter(this, machineOrderModel.getListCache())
        })
        //巡店记录
        machineOrderModel.patrolReport.observe(this, Observer {
            binding.billRv.adapter = WorkNoSelectPatrolAdapter(this, machineOrderModel.patrolReport.value)
        })
        //loadWaitDominator
        machineOrderModel.loadWaitMsg.observe(this, Observer {
            when (it) {
                -1 -> loadWait?.dismissAll()
                LoadWaitDominator.LOADING -> loadWait?.show(LoadWaitDominator.LOADING)
                LoadWaitDominator.NET_ERROR -> loadWait?.show(LoadWaitDominator.NET_ERROR)
                LoadWaitDominator.NO_DATA -> loadWait?.show(LoadWaitDominator.NO_DATA)
            }
        })
    }

    fun init() {
        loadWait = LoadWaitDominator(this, loadWaitContainer)
        binding.billRv.layoutManager = LinearLayoutManager(this@MachineOrderSelectAct)
        binding.billRv.addItemDecoration(DividerItemDecoration(context, DividerItemDecoration.HORIZONTAL_LIST, 1, Color.parseColor("#BFBFBF")))

        if (activityType == TYPE_MACHINE_BILL) {//查工单
            machineOrderModel.queryAllBill(machineOrderModel.queryYm.get()!!)
        } else if (activityType == TYPE_MACHINE_PATROL) {//查巡店
            machineOrderModel.queryPatrolReport(machineOrderModel.queryYm.get()!!)
        }
    }

    override fun finish() {
        setResultData()
        super.finish()
    }

    fun setResultData() {
        setResult(Activity.RESULT_OK, Intent().apply {
            putExtra(SELECTED_ORDER, when (activityType) {
                TYPE_MACHINE_BILL -> machineOrderModel.getListCache().find {
                    it.isChecked?.get() ?: false
                }?.mroNo ?: ""
                else -> machineOrderModel.patrolReport.value?.find {
                    it.isChecked?.get() ?: false
                }?.id.toString()
            })
        })
    }
}