package com.urovo.activity.machinetrip.view

import android.app.Activity
import android.content.Context
import android.content.Intent
import android.os.Bundle
import android.view.View
import android.widget.TextView
import androidx.appcompat.app.AlertDialog
import androidx.databinding.DataBindingUtil
import androidx.lifecycle.ViewModelProviders
import androidx.recyclerview.widget.LinearLayoutManager
import com.urovo.activity.machinetrip.model.MachineTripFactory
import com.urovo.activity.machinetrip.model.MachineTripModel
import com.urovo.activity.machinetrip.repo.MachineTripRepo
import com.urovo.activity.machinetrip.ui.TripInputAdapter
import com.urovo.wugumofang.R
import com.urovo.wugumofang.base.BaseActivity
import com.urovo.wugumofang.constants.AppConstants
import com.urovo.wugumofang.databinding.MachineTripActBinding
import com.urovo.wugumofang.system.SystemHelper
import com.urovo.wugumofang.ui.SetFocusUtil
import com.urovo.wugumofang.ui.ViewUtil
import com.urovo.wugumofang.view.LoadWaitDominator
import kotlinx.android.synthetic.main.machine_trip_act.*
import java.text.SimpleDateFormat
import java.util.*

/**
 * @author Sasuke on 2019-10-23 0023.
 * 出差上报
 */
class MachineTripAct : BaseActivity() {
    companion object {
        const val TYPE_SUBMIT = "TYPE_SUBMIT"//常规提交模式
        const val TYPE_RECORD = "TYPE_RECORD"//查询&&修改模式

        @JvmStatic
        fun launch(context: Context) {
            context.startActivity(Intent(context, MachineTripAct::class.java))
        }

        @JvmStatic
        fun launchWithId(context: Context, id: Int?) {
            context.startActivity(Intent(context, MachineTripAct::class.java).apply {
                putExtra("tid", id)
                putExtra(AppConstants.ExtraKeys.ACTIVITY_TYPE, TYPE_RECORD)
            })
        }
    }

    var clickTextViewCache: TextView? = null

    val tripModel by lazy { ViewModelProviders.of(this, MachineTripFactory(application, MachineTripRepo())).get(MachineTripModel::class.java) }
    val binding by lazy { DataBindingUtil.setContentView<MachineTripActBinding>(this, R.layout.machine_trip_act) }
    val activityType by lazy {
        intent.getStringExtra(AppConstants.ExtraKeys.ACTIVITY_TYPE) ?: TYPE_SUBMIT
    }//Activity分支
    //tid在查记录时需要传
    val tid by lazy { intent.getIntExtra("tid", -1) }

    var loadWait: LoadWaitDominator? = null

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        bind()
        initTitle()
        init()
    }

    fun bind() {
        tripModel.activityType = activityType
        binding.titleText = if (activityType == TYPE_SUBMIT) "出差上报" else "出差上报记录"
        binding.model = tripModel.apply {
            editMode.set(activityType == TYPE_SUBMIT)//可修改态

        }
        //先直接拦截触摸事件，用显隐控制可编辑态
        binding.coverView.setOnTouchListener { view, motionEvent -> true }
        binding.titleLayout.confirm.apply {
            visibility = View.VISIBLE
            //重新定位按钮样式
            setText(if (activityType == TYPE_SUBMIT) "记录" else "修改")
            //点击监听
            setOnClickListener {
                if (activityType == TYPE_SUBMIT) {//默认提交模式
                    startActivity(Intent(this@MachineTripAct, MachineTripRecordAct::class.java))
                } else {//查记录模式
                    if (tripModel.editMode.get()!!) {//原先修改态
                        tripModel.editMode.set(false)
                        tripModel.resumeSubmitData()//恢复原始数据并刷新列表
                        setText("修改")
                        SetFocusUtil.focus(binding.titleLayout.textTitle)//避免BUG：当修改状态下，选中输入框，再关闭修改态，光标还在输入框里。
                    } else {//原先查阅态
                        if (tripModel.submitDatas.isEmpty()) return@setOnClickListener//数据为空，则不允许进入修改状态
                        tripModel.editMode.set(true)
                        setText("取消")
                    }
                }
            }
        }

        //弹窗观察者
        tripModel.alertDialogText.observe(this, androidx.lifecycle.Observer {
            AlertDialog.Builder(this).setMessage(it)
                    .setPositiveButton("我知道了", null)
                    .show()
        })

        //loadWaitDominator
        tripModel.loadWaitMsg.observe(this, androidx.lifecycle.Observer {
            when (it) {
                -1, LoadWaitDominator.DISMISS -> loadWait?.dismissAll()
                LoadWaitDominator.LOADING -> loadWait?.show(LoadWaitDominator.LOADING)
                LoadWaitDominator.NET_ERROR -> loadWait?.show(LoadWaitDominator.NET_ERROR)
                LoadWaitDominator.NO_DATA -> loadWait?.show(LoadWaitDominator.NO_DATA)
            }
        })

        //记录查询修改模式下，手动刷新列表
        tripModel.isNotifyRv.observe(this, androidx.lifecycle.Observer {
            binding.inputRv.adapter?.notifyDataSetChanged()
        })

        //转圈圈
        tripModel.progressText.observe(this, androidx.lifecycle.Observer {
            if (it.isBlank()) {
                ViewUtil.dismissProgress()
            } else {
                ViewUtil.showProgress(this, "请稍等", it)
            }
        })
        //关闭页面
        tripModel.isFinish.observe(this, androidx.lifecycle.Observer {
            if (true) finish()
        })
    }

    fun init() {
        loadWait = LoadWaitDominator(this, rvContainer)
        tripModel.isAddBtnVisible = activityType == TYPE_SUBMIT//修改记录模式下，不显示加减号按钮
        tripModel.queryTripBudget(SystemHelper.LoginUserId, SimpleDateFormat("yyyy-MM").format(Date()))
        binding.inputRv.run {
            layoutManager = LinearLayoutManager(this@MachineTripAct)
            adapter = TripInputAdapter(this@MachineTripAct, tripModel, tripModel.submitDatas) {
                smoothScrollToPosition(it)
            }
        }

        //如果是查记录模式，就请求
        if (activityType == TYPE_RECORD) {
            tripModel.queryTripRecordDetail(tid)
        }
    }


    /**“选择工单”组件专门回传result*/
    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        if (requestCode == 1) {
            when (resultCode) {
                Activity.RESULT_OK -> {
                    data?.getStringExtra(MachineOrderSelectAct.SELECTED_ORDER)?.let { selectOrder ->
                        if (selectOrder != "null" &&selectOrder.isNotBlank()) {//返回之后要修改嵌套的列表里的元素，层级有点深，只有这么写了。
                            tripModel.submitDatas.forEachIndexed { index, entity ->
                                entity.details.forEach {
                                    if (it.isWorkOrderNoSelecting) {
//                                        it.workOrderNo = selectOrder//列表赋值
                                        it.workOrderNoUI.set(selectOrder)
                                        clickTextViewCache?.text = selectOrder//改UI显示
                                        clickTextViewCache = null
                                        it.isWorkOrderNoSelecting = false//改为非选择态
                                        return
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
    }
}