package com.urovo.activity.machinetrip.view

import android.content.Intent
import android.graphics.Color
import android.os.Bundle
import androidx.appcompat.app.AppCompatActivity
import androidx.databinding.DataBindingUtil
import androidx.lifecycle.Observer
import androidx.lifecycle.ViewModelProviders
import androidx.recyclerview.widget.LinearLayoutManager
import com.urovo.activity.machinetrip.model.MachineTripFactory
import com.urovo.activity.machinetrip.model.MachineTripModel
import com.urovo.activity.machinetrip.repo.MachineTripRepo
import com.urovo.activity.machinetrip.ui.MachineRecordAdapter
import com.urovo.wugumofang.R
import com.urovo.wugumofang.base.BaseActivity
import com.urovo.wugumofang.constants.AppConstants
import com.urovo.wugumofang.databinding.MachineTripRecordActBinding
import com.urovo.wugumofang.system.SystemHelper
import com.urovo.wugumofang.view.DividerItemDecoration
import com.urovo.wugumofang.view.LoadWaitDominator
import com.wgmf.trade.GlobalApp
import kotlinx.android.synthetic.main.machine_order_select_act.*

/**
 * @author Sasuke on 2019/10/29.
 */
class MachineTripRecordAct : BaseActivity() {
    var loadWait: LoadWaitDominator? = null
    val recordModel by lazy { ViewModelProviders.of(this, MachineTripFactory(application, MachineTripRepo())).get(MachineTripModel::class.java) }
    val binding by lazy { DataBindingUtil.setContentView<MachineTripRecordActBinding>(this, R.layout.machine_trip_record_act) }

    val activityType by lazy { intent.getStringExtra(AppConstants.ExtraKeys.ACTIVITY_TYPE) }

    //是否是第一次resume()
    var isFirst = true

    companion object {

        @JvmStatic
        fun launch(activity: AppCompatActivity) {
            activity.startActivity(Intent(activity, MachineTripRecordAct::class.java))
        }
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        bind()
        initTitle()
        init()
    }

    fun bind() {
        binding.model = recordModel
        binding.titleText = "出差上报记录"
        binding.ym = recordModel.queryYm

        //已处理工单
        recordModel.tripRecordsCache.observe(this, Observer {
            binding.billRv.adapter = MachineRecordAdapter(this,it)
        })

        //loadWaitDominator
        recordModel.loadWaitMsg.observe(this, Observer {
            when (it) {
                -1,LoadWaitDominator.DISMISS -> loadWait?.dismissAll()
                LoadWaitDominator.LOADING -> loadWait?.show(LoadWaitDominator.LOADING)
                LoadWaitDominator.NET_ERROR -> loadWait?.show(LoadWaitDominator.NET_ERROR)
                LoadWaitDominator.NO_DATA -> loadWait?.show(LoadWaitDominator.NO_DATA)
            }
        })
    }

    fun init() {
        loadWait = LoadWaitDominator(this, loadWaitContainer)
        binding.billRv.layoutManager = LinearLayoutManager(this@MachineTripRecordAct)
        binding.billRv.addItemDecoration(DividerItemDecoration(GlobalApp.context, DividerItemDecoration.HORIZONTAL_LIST, 1, Color.parseColor("#BFBFBF")))

        recordModel.queryTripRecord(SystemHelper.LoginUserId,recordModel.queryYm.get()!!)
    }

    override fun onResume() {
        super.onResume()
        if (isFirst){
            isFirst = false
            return
        }
        recordModel.queryTripRecord(SystemHelper.LoginUserId,recordModel.queryYm.get()!!)

    }
}