package com.urovo.activity.membergroup;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.List;

import android.app.ProgressDialog;
import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.os.Handler.Callback;
import android.os.Message;
import android.view.LayoutInflater;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.LinearLayout;
import android.widget.ListView;

import com.urovo.wugumofang.base.BaseActivity;
import com.urovo.wugumofang.data.PrefTool;
import com.urovo.wugumofang.ui.ViewUtil;
import com.urovo.wugumofang.R;

public abstract class MemberBaseActivity<T> extends BaseActivity implements MemberBaseAdapter.OnCallOrMsgListiener {
	public static final int CALL = 10;
	public static final int MSG = 11;
	public static final String STORE = "store";
	public static final String PAGE = "page";

	private ListView mListView;
	private LinearLayout mLinearLayoutMore;
	private ProgressDialog mProgressDialog;

	protected String mMobile;
	protected String mSql;
	protected MemberBaseAdapter mAdapter;
	protected List<T> mList = new ArrayList<T>();
	protected RecommendHuaShuPop mRecommendHuaShuPop;
	protected static final int START = 0;
	protected static final int PROGRESS = 1;
	protected static final int DONE = 2;
	protected Handler mHandler = new Handler(new Callback() {
		@Override
		public boolean handleMessage(Message msg) {

			if(msg.what == START) {
				ViewUtil.dismissProgress();
				mProgressDialog = new ProgressDialog(MemberBaseActivity.this);
				mProgressDialog.setCancelable(false);
				mProgressDialog.setMax(msg.arg1);
				mProgressDialog.setProgressStyle(ProgressDialog.STYLE_HORIZONTAL);
				mProgressDialog.setTitle("正在整理数据(每天一次)");
				mProgressDialog.setMessage("请等待...");
				mProgressDialog.show();
				mList.clear();
			}
			else
				if(msg.what == PROGRESS) {
					if(null != mProgressDialog && mProgressDialog.isShowing()) {
						mProgressDialog.setProgress(msg.arg1);
					}
				}
				else
					if(msg.what == DONE) {
						if(null != mProgressDialog)
							mProgressDialog.dismiss();
						getDateFromDB();
						setListCounts(mList.size());
					}
			return true;
		}
	});

	protected abstract String getHuaShuUrl();

	protected abstract String getTitleText();

	protected abstract int getContentViewResId();

	protected abstract MemberBaseAdapter newMemberBaseAdapter();

	protected abstract String getPerferences();

	protected abstract void getDateFromDB();

	protected abstract void getDateFromNet();

	protected abstract void requestConnectTask();

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(getContentViewResId());
		initTitle();
		initHeader();
		initContent();
	}

	@Override
	protected void onDestroy() {

		mRecommendHuaShuPop.destory();
		super.onDestroy();
	}

	private void setListCounts(int counts) {
		if(counts > 0) {
			titleTextView.setText(getTitleText() + "(" + counts + ")");
		}
		else {
			titleTextView.setText(getTitleText());
		}
	}

	@Override
	public void initTitle() {

		super.initTitle();
		titleTextView.setText(getTitleText());
	}

	private void initHeader() {
		mRecommendHuaShuPop = new RecommendHuaShuPop(this, titile_bg, getHuaShuUrl());
		mLinearLayoutMore = (LinearLayout) findViewById(R.id.linearLayoutMore);
		mLinearLayoutMore.setOnClickListener(new OnClickListener() {

			@Override
			public void onClick(View v) {

				mRecommendHuaShuPop.show();
			}
		});
	}

	private void initContent() {
		mListView = (ListView) findViewById(R.id.listView1);
		mAdapter = newMemberBaseAdapter();
		mListView.setAdapter(mAdapter);
		showEmpty();
		if(alreadySavedData()) {
			getDateFromDB();
			setListCounts(mList.size());
		}
		else {
			getDateFromNet();
		}
	}

	protected void doSavedData() {
		SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
		String date = sdf.format(new java.util.Date());
		PrefTool.setStringsave(this, getPerferences(), date);
	}

	private boolean alreadySavedData() {
		SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
		String date = sdf.format(new java.util.Date());
		String exist = PrefTool.getStringPerferences(this, getPerferences(), "");
		return exist.equals(date);
	}

	protected void showEmpty() {
		View v = LayoutInflater.from(this).inflate(R.layout.listview_empty, null);
		v.setLayoutParams(new ViewGroup.LayoutParams(ViewGroup.LayoutParams.FILL_PARENT, ViewGroup.LayoutParams.FILL_PARENT));
		v.setVisibility(View.GONE);
		if(null != mListView.getTag()) {
			View old = (View) mListView.getTag();
			((ViewGroup) mListView.getParent()).removeView(old);
		}
		((ViewGroup) mListView.getParent()).addView(v);
		mListView.setTag(v);
		mListView.setEmptyView(v);
	}

	private void showNetError() {
		View v = LayoutInflater.from(this).inflate(R.layout.listview_empty_error, null);
		v.setLayoutParams(new ViewGroup.LayoutParams(ViewGroup.LayoutParams.FILL_PARENT, ViewGroup.LayoutParams.FILL_PARENT));
		v.setVisibility(View.GONE);
		Button btn = (Button) v.findViewById(R.id.buttonDownload);
		btn.setOnClickListener(new OnClickListener() {

			@Override
			public void onClick(View v) {

				requestConnectTask();
			}
		});
		if(null != mListView.getTag()) {
			View old = (View) mListView.getTag();
			((ViewGroup) mListView.getParent()).removeView(old);
		}
		((ViewGroup) mListView.getParent()).addView(v);
		mListView.setTag(v);
		mListView.setEmptyView(v);
	}

	@Override
	public void netError() {

		showNetError();
	}

	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {

		super.onActivityResult(requestCode, resultCode, data);
		switch (requestCode) {
		case CALL: {
			break;
		}
		case MSG: {
			break;
		}
		default:
			break;
		}
	}

	@Override
	public void onCall(int postion, String mobile) {

		mMobile = mobile;
	}

	@Override
	public void onMsg(int postion, String mobile) {

	}
}
