package com.urovo.activity.memorderact.model

import android.app.Application
import androidx.databinding.ObservableArrayList
import androidx.databinding.ObservableList
import androidx.lifecycle.AndroidViewModel
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.viewModelScope
import com.urovo.activity.memorderact.net.MemOrderListRes
import com.urovo.activity.memorderact.repository.MemOrderListRepo
import com.urovo.wugumofang.utils.MyToast
import kotlinx.coroutines.launch

/**
 * @author Sasuke on 2019-7-19 0019.
 */
class MemOrderListModel(val repo: MemOrderListRepo, val app: Application) : AndroidViewModel(app) {
    var dataList: ObservableList<MemOrderListRes.ResultInfo> = ObservableArrayList()
    var refresh: MutableLiveData<Boolean> = MutableLiveData()

    var loadWaitText: MutableLiveData<String> = MutableLiveData()

    //要删除的订单的id
    var deleteOrderId: MutableLiveData<Int> = MutableLiveData()

    //转圈圈弹窗
    var progressBarText: MutableLiveData<String> = MutableLiveData()


    /**
     * 获取会员商品订单详情
     */
    fun queryMemOrders(userNo: String) = viewModelScope.launch {
        try {
            dataList.run {
                clear()
                addAll(repo.queryMemOrderDetails(userNo))
                refresh.value = true
            }
            loadWaitText.value = if (dataList.isEmpty()) "暂无数据" else ""
        } catch (e: Exception) {
            loadWaitText.value = e.message
        }
    }

    /**点击了删除按钮*/
    fun onDelete(orderId: Int) {
        deleteOrderId.value = orderId
    }

    /**执行删除订单请求*/
    fun deleteOrder(orderId: Int) = viewModelScope.launch {
        progressBarText.value = "正在撤销订单..."
        try {
            var res = repo.cancelMemOrder(orderId)
            MyToast.show(res.msgInfo)
            if (res.msgCode == 0){
                dataList.find { it.id == orderId }?.let { dataList.remove(it) }//成功了就手动删掉该订单
                refresh.value = true//并刷新
            }
        } catch (e: Exception) {
            MyToast.show(e.message)
        }
        progressBarText.value = ""
    }

}