package com.urovo.activity.memorderact.model

import android.app.Application
import androidx.lifecycle.AndroidViewModel
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.viewModelScope
import com.urovo.activity.memorderact.net.MemOrderListRes
import com.urovo.activity.memorderact.net.ReceiversInfosRes
import com.urovo.activity.memorderact.repository.MemOrderSubmitRepo
import com.urovo.activity.receiveraddress.net.AddrSubmitBean
import com.urovo.wugumofang.utils.MyToast
import kotlinx.coroutines.launch

/**
 * @author Sasuke on 2019-07-21.
 */
class MemOrderSubmitModel(val repo: MemOrderSubmitRepo, val app: Application) : AndroidViewModel(app) {
    //通过Intent传过来的订单详情信息。
    var orderEntity: MutableLiveData<MemOrderListRes.ResultInfo> = MutableLiveData()

    //收货地址信息
    var receiversInfos: MutableLiveData<List<ReceiversInfosRes.ResultInfo>> = MutableLiveData()

    //给工厂留言
    var addtionalText: String = ""

    //正在使用的地址
    var addrToUse: ReceiversInfosRes.ResultInfo? = null



    /**
     * 照片路径缓存
     */
    var photoPool = mutableListOf<String>()

    /**转圈圈文字*/
    var progressBarText = MutableLiveData<String>()

    /**弹窗文字（情况：订单本身超时，点击跳转首页）*/
    var alertDialogToHomeText = MutableLiveData<String>()

    /**弹窗文字（情况： 网络异常，点击继续提交）*/
    var alertDialogReTryText = MutableLiveData<String>()

    /**关闭当前Activity*/
    var isGotoSuccessAct = MutableLiveData<Boolean>()

    /**
     * 获取收货人信息
     */
    fun queryReceiversInfo(mobile: String) = viewModelScope.launch {
        try {
            receiversInfos.value = repo.queryReceiversInfo(mobile)
        } catch (e: Exception) {
            MyToast.show(e.message)
        }
    }

    /**
     * 得到需要马上使用的地址
     *
     * 先通过查看后面新增地址时传过来的bean，因为后台不传id，所以我们需要一个一个一个一个匹配信息
     * 然后把对应的地址拉出来，
     * 然后把它设为default，其它元素都设成非default
     *
     * 如果找到了新增地址，那就使用
     * 如果没找到，就把返回的地址List里isDefault为true的拿出来用。
     */
    fun getToUseAddrSetDefault(newAddrBean: AddrSubmitBean?, addrList: List<ReceiversInfosRes.ResultInfo>?): ReceiversInfosRes.ResultInfo? {
        if (newAddrBean == null) return null

        return addrList?.find {
            newAddrBean.receiverName == it.receiverName &&
                    newAddrBean.receiverPhone == it.receiverPhone &&
                    newAddrBean.additional == it.additional &&
                    newAddrBean.provinceId == it.provinceId &&
                    newAddrBean.cityId == it.cityId &&
                    newAddrBean.districtId == it.districtId
        }?.apply { addrList.forEach { it.isDefault = false }; isDefault = true }
    }

    /**
     * 提交订单
     */
    fun submitOrder() = viewModelScope.launch {
        progressBarText.value = "正在上传图片"
        try {
            val photoUrls = repo.submitPhotos2Qiniu(photoPool)
            progressBarText.value = "正在上传订单"
            val res = repo.submitMemOrder(orderEntity.value?.id ?:0,addrToUse?.id ?:0,photoUrls,addtionalText)
            progressBarText.value = ""
            if (res.msgCode == 0){
                isGotoSuccessAct.value = true
            }else{
                alertDialogToHomeText.value = res.msgInfo//一般是订单本身超时，库存被释放的情况。
            }
        }catch (e: Exception){
            alertDialogReTryText.value = e.message
        }
    }
}