package com.urovo.activity.memorderact.view

import android.app.AlertDialog
import android.content.Intent
import android.os.Bundle
import android.widget.TextView
import androidx.databinding.DataBindingUtil
import androidx.lifecycle.Observer
import androidx.lifecycle.ViewModelProviders
import com.urovo.activity.memorderact.model.MemOrderSubmitFactory
import com.urovo.activity.memorderact.model.MemOrderSubmitModel
import com.urovo.activity.memorderact.net.MemOrderListRes
import com.urovo.activity.memorderact.repository.MemOrderSubmitRepo
import com.urovo.activity.memorderact.ui.MemOrderDetailAdapter
import com.urovo.activity.receiveraddress.net.AddrSubmitBean
import com.urovo.activity.receiveraddress.view.ReceiverAddrEditAct
import com.urovo.activity.receiveraddress.view.ReceiverAddrManageAct
import com.urovo.activity.tabMenu.view.TabMenuActivity
import com.urovo.wugumofang.R
import com.urovo.wugumofang.base.BaseActivity
import com.urovo.wugumofang.data.StringUtils
import com.urovo.wugumofang.databinding.MemOrderSubmitActBinding
import com.urovo.wugumofang.ui.DensityUtil
import com.urovo.wugumofang.ui.PhotoPickDominator
import com.urovo.wugumofang.ui.ViewUtil
import kotlinx.android.synthetic.main.mem_order_submit_act.*

/**
 * @author Sasuke on 2019-07-20.
 */
class MemOrderSubmitAct : BaseActivity() {
    companion object {
        const val INTENT_ORDER = "INTENT_ORDER"

        const val NEW_INTENT_ADDR = "NEW_INTENT_ADDR"

        const val REQUEST_ADDR_CODE = 13

        //最多传照片数
        const val ADD_PHOTO_MAX_COUNT = 3

    }

    val binding by lazy { DataBindingUtil.setContentView<MemOrderSubmitActBinding>(this, R.layout.mem_order_submit_act) }

    val model by lazy { ViewModelProviders.of(this, MemOrderSubmitFactory(MemOrderSubmitRepo(), application)).get(MemOrderSubmitModel::class.java) }

    var photoPicker: PhotoPickDominator? = null

    //从 ReceiverAddrEditAct 编辑地址界面回传回来的新地址。
    var newAddrBean: AddrSubmitBean? = null

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        bind()
        initTitle()
        init()
        model.queryReceiversInfo(/*"15102116446"*/model.orderEntity.value!!.customerTel)

    }

    override fun onNewIntent(intent: Intent?) {
        super.onNewIntent(intent)
        setIntent(intent)
        //提交的新的地址bean并马上拿来用。
        newAddrBean = intent?.getSerializableExtra(NEW_INTENT_ADDR) as AddrSubmitBean?
        //如果新地址不为空（因为后台不给id，所以只有我们自己再重新请求一下全部收货地址，然后一个一个一个一个信息地对比，把匹配的id给找出来）
        newAddrBean?.let {
            model.queryReceiversInfo(model.orderEntity.value!!.customerTel)
        }
    }

    fun bind() {
        binding.titleText = "完善订单信息"
        model.orderEntity.value = intent.getSerializableExtra(INTENT_ORDER) as MemOrderListRes.ResultInfo
        //获取收货人信息
        model.receiversInfos.observe(this, Observer {
            //再说一次，因为后台不给地址id，所以只有我们自己一个一个一个一个地匹配然后再把对应的地址信息和id拿出来。
            model.addrToUse = model.getToUseAddrSetDefault(newAddrBean, it)
                    ?: it.find { it.isDefault }
            newAddrBean = null//用完了要置空，否则会妨碍到后面选择地址时调用这里。

            binding.isCheckMode = false//这个是用来控制地址栏的显示模式，因为布局是共用的。。。如果是true，那就是带checkBox的选择地址布局。
            binding.receiverName = model.addrToUse?.receiverName
            binding.receiverPhone = model.addrToUse?.receiverPhone
            binding.receiverAddress = model.addrToUse?.getLongAddr()
        })

        //转圈圈提示
        model.progressBarText.observe(this, Observer {
            if (it.isEmpty()) {
                ViewUtil.dismissProgress()
            } else {
                ViewUtil.showProgress(this, "请稍等", it)
            }
        })

        model.alertDialogToHomeText.observe(this, Observer {
            AlertDialog.Builder(this).setTitle("订单超时")
                    .setMessage(it)
                    .setCancelable(false)
                    .setPositiveButton("确定", { dialogInterface, i -> gotoHomeTab(TabMenuActivity::class.java) })
                    .show()
        })

        model.alertDialogReTryText.observe(this, Observer {
            AlertDialog.Builder(this).setTitle("网络异常")
                    .setMessage(it)
                    .setCancelable(true)
                    .setPositiveButton("确定", { dialogInterface, i -> model.submitOrder() })
        })

        //保存成功后，关闭自己，并跳转保存成功界面
        model.isGotoSuccessAct.observe(this, Observer {
            if (it) {
                startActivity(Intent(this, MemOrderSuccessAct::class.java))
                finish()
            }
        })
    }


    fun init() {
        photoPicker = PhotoPickDominator(this, model.photoPool)
        model.orderEntity.value?.let { binding.proLv.adapter = MemOrderDetailAdapter(this, it.details) }
        binding.model = model
        binding.amoutLayout.findViewById<TextView>(R.id.name).text = ""
        binding.amoutLayout.findViewById<TextView>(R.id.count).text = "合计"
        binding.amoutLayout.findViewById<TextView>(R.id.amount).text = StringUtils.formatDouble(model.orderEntity.value?.amount
                ?: 0.0, "0.##") + "元"

        binding.addAddressLayout.setOnClickListener {
            if (model.receiversInfos.value == null) {//跳转新增收货地址
                ReceiverAddrEditAct.launch(this@MemOrderSubmitAct, model.orderEntity.value?.customerTel)
            } else {//跳转选择地址
                ReceiverAddrManageAct.launchForResult(this@MemOrderSubmitAct,
                        model.orderEntity.value?.customerTel,
                        ArrayList(model.receiversInfos.value), REQUEST_ADDR_CODE)
            }
        }

        //添加图片
        addPhoto.setOnClickListener {
            if (model.photoPool.size >= ADD_PHOTO_MAX_COUNT) {
                showMsg("最多支持三张")
                return@setOnClickListener
            }
            photoPicker?.showDialog()
        }

        //提交订单
        memOrderSubmitBtn.setOnClickListener {
            if (binding.receiverAddress.isNullOrBlank()) {
                showMsg("请添加地址")
                return@setOnClickListener
            }
            if (model.photoPool.size <= 0) {
                showMsg("请添加小票照片")
                return@setOnClickListener
            }
            model.submitOrder()
        }

    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        when (requestCode) {
            //得到更换后的收货地址
            REQUEST_ADDR_CODE -> {
                data?.let {
                    val newAddrId = data?.getIntExtra(ReceiverAddrManageAct.ADDR_RESULT_DATA, -1) /*as ArrayList<ReceiversInfosRes.ResultInfo>*/
//                    model.receiversInfos.value = newAddrList
                    model.receiversInfos.value?.apply {
                        find { it.isDefault }?.isDefault = false
                        find { it.id == newAddrId }?.isDefault = true
                        model.receiversInfos.value = this
                    }
                }
            }
            else -> {
                val photoPath = photoPicker?.let { it.onPhotoPathResult(this, requestCode, resultCode, data) };
                photoPicker?.addPhoto2Group(photoPath,
                        DensityUtil.dip2px(this, 50f),
                        DensityUtil.dip2px(this, 70f),
                        photoContainer)
            }
        }
    }

    override fun onDestroy() {
        //清除本地图片缓存
        photoPicker?.deletePicCache()
        super.onDestroy()
    }
}