package com.urovo.activity.messagecenter.presenter;

import com.google.gson.Gson;
import com.google.gson.JsonObject;
import com.google.gson.JsonParser;
import com.urovo.activity.messagecenter.model.MsgDetailData;
import com.urovo.activity.messagecenter.net.ArticleModel;
import com.urovo.activity.messagecenter.net.ELearningRes;
import com.urovo.activity.messagecenter.net.SysMsgDataRes;
import com.urovo.activity.messagecenter.net.SysMsgModel;
import com.urovo.wugumofang.base.BaseActivity;
import com.urovo.wugumofang.constants.ApiUrls;
import com.urovo.wugumofang.constants.ConstantParams;
import com.urovo.wugumofang.constants.NetParam;
import com.urovo.wugumofang.net.OkHttpDominator;
import com.urovo.wugumofang.system.SystemHelper;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import okhttp3.Request;

/**
 * Created by Sasuke on 2016/10/8.
 */

public class MsgDetailPresenter implements MsgCenterContract.detailPresenter {
    MsgCenterContract.view mView;
    MsgDetailData data;
    MsgCenterContract.view mActivity;

    public MsgDetailPresenter(MsgCenterContract.view mActivity, MsgCenterContract.view mView) {
        if (mView != null) {//在外面单独使用这个Presenter时可以传空mView
            this.mView = mView;
            this.mView.setPresenter(this);
            this.data = new MsgDetailData();
        }
        this.mActivity = mActivity;
    }


    @Override
    public MsgDetailData getData() {
        if (data == null) {
            data = new MsgDetailData();
        }
        return data;
    }

    @Override
    public void refreshAll() {
        getData().setPageNum(0);
        if ("ELearning".equals(getData().getLocalType())) {
            getDetailOfELearning();
        } else {
            getDetailByPagenum(0);
        }
    }

    /**
     * Elearning的数据请求
     */
    public void getDetailOfELearning() {
        HashMap<String, String> param = new HashMap<>();
        param.put("userNo", SystemHelper.LoginUserId);
        param.put("type", getData().getActType() + "");

        OkHttpDominator.postAsyn(NetParam.YINXIAOBU_SERVER + ApiUrls.MSG_SERVICE.GET_ELEARNING_DETAIL, param,
                new OkHttpDominator.ResultCallback<ELearningRes>() {
                    @Override
                    public void onError(Request request, Exception e) {

                    }

                    @Override
                    public void onResponse(ELearningRes res) {
                        List<SysMsgModel> tempList = new ArrayList<>();
                        tempList.addAll(res.getResultInfo());
                        getData().addNewList(tempList);

                        int msgAmount = 0;//未读数（我真TM不想再为一个未读数多请求一个接口了！！CPU资源好像不要钱似的！与详情同步也是问题）
                        for (SysMsgModel item : tempList) {
                            if (item.getUsed() == 0) {
                                msgAmount++;
                            }
                        }
                        getData().setMsgAmount(msgAmount);
                        notifyAdapter();
                    }
                });

    }

    /**
     * 非ELearning的数据请求（带分页）
     */
    public void getDetailByPagenum(int pageno) {
        HashMap<String, String> param = new HashMap<>();
        param.put("store_code", SystemHelper.LoginOrganId);
        param.put("opno", SystemHelper.LoginUserId);
        param.put("pageno", pageno + "");
        String path = "";
        switch (getData().getActType()) {
            //订货提醒
            case ConstantParams.CONSTANT_STARTS_4:
                path = ApiUrls.MSG_SERVICE.GET_ORDER_SHOP_MSG;
                //tag = "notArticleMsg";
                break;
            //盘点提醒
            case ConstantParams.CONSTANT_STARTS_5:
                path = ApiUrls.MSG_SERVICE.GET_STOCK_TAKING_SHOP_MSG;
                //tag = "notArticleMsg";
                break;
            //考勤提醒
            case ConstantParams.CONSTANT_STARTS_6:
                path = ApiUrls.MSG_SERVICE.GET_WORK_MSG;
                // tag = "notArticleMsg";
                break;
            //养生文库
            case ConstantParams.CONSTANT_STARTS_7:
                path = ApiUrls.MSG_SERVICE.GET_DOC_MSG;
                //tag = "articleMsg";
                break;
            //学习中心
            case ConstantParams.CONSTANT_STARTS_8:
                path = ApiUrls.MSG_SERVICE.GET_DOC_MSG;
                // tag = "articleMsg";
                break;
            //会员商城提成
            case ConstantParams.CONSTANT_STARTS_10:
                path = ApiUrls.MSG_SERVICE.ONLINE_ORDER_MSG;
                break;
            //千店万群销售动态
            case ConstantParams.CONSTANT_STARTS_11:
                path = ApiUrls.MSG_SERVICE.GET_TENTHOUSAND_GROUP_MSG;
                break;
            default:
        }
        //Class clazz = tag.equals("articleMsg")? ArticleNewRes.class : SysMsgDataRes.class;
//        new BaseMsgServiceTask("articleMsg", SysMsgDataRes.class,(BaseActivity)context,"正在获取详情...",
//                false,path,param).execute("");
        OkHttpDominator.postAsyn(NetParam.YINXIAOBU_SERVER + path, param,
                new OkHttpDominator.ResultCallback<SysMsgDataRes>() {
                    @Override
                    public void onError(Request request, Exception e) {

                    }

                    @Override
                    public void onResponse(SysMsgDataRes res) {
                        if (res == null || res.msgInfo == null) {
                            return;
                        }
                        getData().setMsgAmount(Integer.parseInt(res.msgInfo));
                        transData(res.getResultInfo());//里面会自己判断是不是文章
                        notifyAdapter();
                        if (res.getResultInfo() != null) {
                            getData().pageNumPlus();//每次刷新成功就要把下次页码自增，便于今后的上拉加载
                        }
                    }
                });
    }

    public void transData(String[] array) {
        if (array != null) {
            List<SysMsgModel> tempList = new ArrayList<>();
            if (getData().getActType() == 7 || getData().getActType() == 8) {//如果是文章更新
                for (String json : array) {
                    JsonObject o = new JsonParser().parse(json).getAsJsonObject();
                    ArticleModel model = new Gson().fromJson(o, ArticleModel.class);
                    model.decodeContent();//别忘了
                    tempList.add(model);
                }
            } else {//如果是别的
                for (String json : array) {
                    JsonObject o = new JsonParser().parse(json).getAsJsonObject();
                    SysMsgModel model = new Gson().fromJson(o, SysMsgModel.class);
                    tempList.add(model);
                }
            }
            getData().addNewList(tempList);
            if (tempList.size() == 0) {//前面new了，所以不可能为null
                ((BaseActivity) mActivity).showMsg("没有更多了");
            }
        }
    }


    /**
     * 上传文章已读状态（在这个详情界面，需要上传已读的只有文章了）
     */
    public void uploadMsgCount(String type, String id) {
        Map<String, String> param = new HashMap<>();
        param.put("store_code", SystemHelper.LoginOrganId);
        param.put("opno", SystemHelper.LoginUserId);
        param.put("type", type);
        param.put("id", id);
//        new BaseMsgServiceTask("sysMsg_upload_count", BaseOldResponse.class,(BaseActivity)mActivity,
//                "正在上传文章已读数...",false,"/shopmessage/updateShopMsg",param).execute("");
        OkHttpDominator.postAsyn(NetParam.YINXIAOBU_SERVER + ApiUrls.MSG_SERVICE.SYS_MSG_COUNT_UPDATE,
                param, null);
    }

    @Override
    public void loadMore() {
        getDetailByPagenum(getData().getPageNum());//获取完成后pageNum在BaseActivity里自增1
    }

    @Override
    public void notifyAdapter() {
        //加一个按时间排序吧（按时间排序，所以大的在上面）
        //文章Model也添加了long型时间格式化为timestamp的方法
        //因为有上拉加载，所以没必要让前端排序了
        /*Collections.sort(getData().getDataList(), new Comparator<SysMsgModel>() {
            @Override
            public int compare(SysMsgModel lhs, SysMsgModel rhs) {
                //日期相同的情况下比较已读未读，未读的放在前面。
                if(rhs.getTimestamp().compareTo(lhs.getTimestamp()) == 0){
                    if(rhs.getUsed() == 0){
                        return 1;
                    }
                    if(lhs.getUsed() == 0){
                        return -1;
                    }
                }
                return rhs.getTimestamp().compareTo(lhs.getTimestamp());
            }
        });*/
        //}
        mView.notifyAdapter();
    }

    @Override
    public void start() {

    }

}
