package com.urovo.activity.newGuideAct.view;

import android.os.Bundle;
import android.view.Window;
import android.view.WindowManager;

import com.alibaba.android.arouter.facade.annotation.Route;
import com.urovo.activity.newGuideAct.presenter.NewGuideContract;
import com.urovo.activity.newGuideAct.presenter.NewGuidePresenter;
import com.urovo.wugumofang.R;
import com.urovo.wugumofang.base.BaseActivity;
import com.urovo.wugumofang.constants.AppConstants;
import com.urovo.wugumofang.constants.RouterParams;
import com.urovo.wugumofang.ui.FragDominator;

import java.util.HashMap;

import butterknife.ButterKnife;
import butterknife.Unbinder;


/**
 * Created by Sasuke on 2017/2/16.
 * 引导界面（新功能介绍）
 */
@Route(path = RouterParams.ORIGIN.NEWGUIDEACTIVITY)
public class NewGuideAct extends BaseActivity implements NewGuideContract.view {
    NewGuidePresenter presenter;
    final public static String INTENT_SIGNALS = "INTENT_SIGNALS";//用Intent传一些标记过来，来决定显示哪些欢迎界面。
    final public static String KEY_WELCOME = "KEY_WELCOME";//是否显示新功能欢迎界面
    final public static String KEY_AD = "KEY_AD";//是否显示广告（宣言）
    final public static String INTENT_DECLARATION_TEXT = "INTENT_DECLARATION_TEXT";//传职业宣言文字列表过来
    HashMap<String, Boolean> signals;

    AdFra adFra;//广告页
    WelcomeFra welcomFra;//欢迎页
    FragDominator fragDominator;
    Unbinder unbinder;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        //实现全屏+无标题栏
        requestWindowFeature(Window.FEATURE_NO_TITLE);
        getWindow().setFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN, WindowManager.LayoutParams.FLAG_FULLSCREEN);
        super.onCreate(savedInstanceState);
        setContentView(R.layout.act_new_guide);
        unbinder = ButterKnife.bind(this);

        presenter = new NewGuidePresenter(this);
        presenter.start();
//        presenter.saveNowTime();//写入当前日期到本地（登陆的时候要判断一下，当天只用显示一次欢迎界面）
    }


    @Override
    public void initUI() {
        presenter.getData().fromActivity = (Class) getIntent().getSerializableExtra(AppConstants.PreKeys.NEW_GUIDE_FROM);
        signals = (HashMap<String, Boolean>) getIntent().getSerializableExtra(INTENT_SIGNALS);
        if (signals == null) {
            showMsg("没有HashMap标记");
            return;
        }

        fragDominator = new FragDominator(this, R.id.guide_container);
        //显示顺序：新功能介绍＞全屏广告页＞卫生日弹窗；（卫生日弹窗是主页的小弹窗，不是全屏的，就懒得做在一起了。）
        //如果要显示新功能介绍
        if (signals.get(KEY_WELCOME) != null && signals.get(KEY_WELCOME)) {
            welcomFra = new WelcomeFra();
            welcomFra.setPresenter(presenter);
            fragDominator.add(welcomFra);
        }

        //如果要显示广告
        //现在职业宣言砍掉了。
//        if (signals.get(KEY_AD) != null && signals.get(KEY_AD)) {
//            adFra = new AdFra();
//            adFra.setPresenter(presenter);
//            presenter.getData().declarationTexts = getIntent().getStringArrayListExtra(INTENT_DECLARATION_TEXT);
//            fragDominator.add(adFra);
//        }
        presenter.showNextPage();//按顺序显示
        isPartTimeStaff();//是否兼职人员
        isPartTimeSalesStaff();//是否兼职销售人员
    }

    @Override
    public boolean showNextPage() {
        if (fragDominator != null) {
            return fragDominator.showFraNext();
        }
        return false;//最后一个，后面没有了
    }

    @Override
    public boolean isPartTimeStaff() {
        return getIntent().getBooleanExtra(AppConstants.ExtraKeys.PART_TIME_STAFF, false);
    }

    @Override
    public boolean isPartTimeSalesStaff() {
        return getIntent().getBooleanExtra(AppConstants.ExtraKeys.PART_TIME_SALES_STAFF, false);
    }

    @Override
    public void setPresenter(NewGuideContract.presenter presenter) {
        this.presenter = (NewGuidePresenter) presenter;
    }

    @Override
    public void onBackPressed() {
//        super.onBackPressed();
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
//        ButterKnife.unbind(this);
        unbinder.unbind();
    }
}
