package com.urovo.activity.personalcenter.presenter

import com.urovo.activity.personalcenter.model.MergeMemberEntity
import com.urovo.activity.personalcenter.model.MergeMemberRequest
import com.urovo.activity.personalcenter.presenter.contract.MergeMemberContract
import com.urovo.activity.personalcenter.response.MergeMemberDepository
import com.urovo.wugumofang.R
import com.urovo.wugumofang.base.mvp.AbsPresenter
import com.urovo.wugumofang.constants.AppConstants
import com.urovo.wugumofang.constants.ConstantParams
import com.urovo.wugumofang.net.ApiException
import com.urovo.wugumofang.net.callback.RetrofitCallback
import com.urovo.wugumofang.system.SystemHelper
import com.urovo.wugumofang.utils.isEmptyParameter
import com.urovo.wugumofang.view.LoadWaitDominator
import com.zsy.libcore.model.bean.RequestResult
import retrofit2.Call
import retrofit2.Response

/**
 *  合并会员Presenter
 * @author LW
 */
class MergeMemberPresenter(
    private val view: MergeMemberContract.IView?,
    private var depository: MergeMemberDepository = MergeMemberDepository()
) : AbsPresenter(), MergeMemberContract.IPresenter {

    override fun requestMemberInfoOldMobile(oldMobile: String?, stopId: String?) {
        if (isEmptyParameter(oldMobile, stopId)) {
            view?.showToast(R.string.common_load_error)
            return
        }

        view?.showTipDialog(R.string.merge_member_search_old_mobile_info)
        depository.requestMemberInfoOldMobile(oldMobile!!, stopId!!)
            .enqueue(object : RetrofitCallback<RequestResult<MergeMemberEntity>>() {
                override fun onResponse(
                    call: Call<RequestResult<MergeMemberEntity>>,
                    res: Response<RequestResult<MergeMemberEntity>>
                ) {
                    val body = res.body()
                    body.takeIf { it != null }?.run {
                        if (msgCode == AppConstants.Http.RESPONSE_SUCCESS) {
                            if (payload != null) {
                                view?.mergeMemberInfo(ConstantParams.CONSTANT_STARTS_1, payload)
                            } else {
                                view?.mergeMemberInfo(ConstantParams.CONSTANT_STARTS_2, null)
                            }
                        } else {
                            view?.showToast(msgInfo)
                        }
                    } ?: view?.showToast(R.string.common_http_error)

                    view?.dismissDialog()
                }

                override fun onFailure(exception: ApiException) {
                    view?.dismissDialog()
                    view?.showToast(exception.message)
                }
            })
    }

    override fun requestMemberInfoNewMobile(newMobile: String?) {
        if (isEmptyParameter(newMobile)) {
            view?.showToast(R.string.common_load_error)
            return
        }

        view?.showTipDialog(R.string.merge_member_search_new_mobile_info)
        depository.requestMemberInfoNewMobile(newMobile!!)
            .enqueue(object : RetrofitCallback<RequestResult<MergeMemberEntity>>() {
                override fun onResponse(
                    call: Call<RequestResult<MergeMemberEntity>>,
                    res: Response<RequestResult<MergeMemberEntity>>
                ) {
                    val body = res.body()
                    body.takeIf { it != null }?.run {
                        if (msgCode == AppConstants.Http.RESPONSE_SUCCESS) {
                            if (payload != null) {
                                view?.mergeMemberInfo(ConstantParams.CONSTANT_STARTS_2, payload)
                            } else {
                                view?.mergeMemberInfo(ConstantParams.CONSTANT_STARTS_2, null)
                            }
                        } else {
                            view?.showToast(msgInfo)
                        }
                    } ?: view?.showToast(R.string.common_http_error)

                    view?.dismissDialog()
                }

                override fun onFailure(exception: ApiException) {
                    view?.dismissDialog()
                    view?.showToast(exception.message)
                }
            })
    }

    override fun requestMergeMemberInfoExecute(
        oldMobile: String?,
        oldCusId: String?,
        newMobile: String?,
        newCusId: String?,
        staffId: String?
    ) {
        if (isEmptyParameter(oldMobile, oldCusId, newMobile, newCusId, staffId)) {
            view?.showToast(R.string.common_load_error)
            return
        }

        view?.showTipDialog(R.string.merge_member)
        depository.requestMergeMemberInfoExecute(
            MergeMemberRequest(
                oldMobile,
                oldCusId,
                newMobile,
                newCusId,
                staffId
            )
        ).enqueue(object : RetrofitCallback<RequestResult<Any>>() {
            override fun onResponse(
                call: Call<RequestResult<Any>>,
                res: Response<RequestResult<Any>>
            ) {
                val body = res.body()
                body.takeIf { it != null }?.run {
                    if (msgCode == AppConstants.Http.RESPONSE_SUCCESS) {
                        view?.mergeMemberInfoExecuteStatus(true)
                    } else {
                        view?.showToast(msgInfo)
                    }
                } ?: view?.showToast(R.string.common_http_error)
                view?.dismissDialog()
            }

            override fun onFailure(exception: ApiException) {
                view?.dismissDialog()
                view?.showToast(exception.message)
            }
        })
    }
}