package com.urovo.activity.personalcenter.ui.voucher.adapter

import android.view.View
import android.widget.LinearLayout
import android.widget.TextView
import com.chad.library.adapter.base.BaseMultiItemQuickAdapter
import com.chad.library.adapter.base.BaseViewHolder
import com.urovo.activity.personalcenter.model.ConverRecordsBean
import com.urovo.activity.personalcenter.model.VoucherCardQueryBean
import com.urovo.activity.personalcenter.model.VoucherMultipleItem
import com.urovo.wugumofang.R
import com.urovo.wugumofang.utils.convertDefaultValue

/**
 * 权益卡查询适配器
 * @author LW
 */
class VoucherCardQueryAdapter(
    dataList: MutableList<VoucherMultipleItem<*>>,
    private val listener: OnItemChildClickListener?
) : BaseMultiItemQuickAdapter<VoucherMultipleItem<*>, BaseViewHolder>(dataList) {

    init {
        addItemType(
            VoucherMultipleItem.PURCHASE_CARD_RECORD,
            R.layout.item_voucher_card_query
        );
        addItemType(VoucherMultipleItem.CONVER_RECORD, R.layout.item_voucher_card_query);
        addItemType(VoucherMultipleItem.MEGER_RECORD, R.layout.item_voucher_card_query);
    }

    override fun convert(holder: BaseViewHolder, item: VoucherMultipleItem<*>) {
        holder.getView<TextView>(R.id.voucher_card_query_item_title_tv).apply {
            text = item.content
        }
        val voucherCardQueryBean: VoucherCardQueryBean =
            item.t as VoucherCardQueryBean

        val itemContainerLl =
            holder.getView<LinearLayout>(R.id.voucher_card_query_item_container_ll)
        if (itemContainerLl.childCount > 0) {
            itemContainerLl.removeAllViews()
        }
        when (holder.itemViewType) {
            VoucherMultipleItem.PURCHASE_CARD_RECORD -> {
                val purchaseCardRecordsList = voucherCardQueryBean.purchaseCardRecords
                purchaseCardRecordsList?.forEach {
                    val itemView =
                        View.inflate(mContext, R.layout.item_voucher_card_default_record, null)

                    itemView.findViewById<TextView>(R.id.default_record_item_one_tv).apply {
                        text =
                            mContext.getString(
                                R.string.voucher_purchase_time,
                                it.purchaseDate.convertDefaultValue()
                            )
                    }
                    itemView.findViewById<TextView>(R.id.default_record_item_two_tv).apply {
                        text = mContext.getString(
                            R.string.voucher_expire_time,
                            it.expireDate.convertDefaultValue()
                        )
                    }

                    itemView.findViewById<TextView>(R.id.default_record_item_three_tv).apply {
                        text = mContext.getString(
                            R.string.voucher_order_code,
                            it.orderId.convertDefaultValue()
                        )
                    }

                    itemView.findViewById<TextView>(R.id.default_record_item_four_tv).apply {
                        text = mContext.getString(
                            R.string.voucher_card_advisor,
                            it.salerName.convertDefaultValue()
                        )
                        visibility = View.VISIBLE
                    }

                    itemView.findViewById<TextView>(R.id.default_record_item_five_tv).apply {
                        text = mContext.getString(
                            R.string.voucher_order_status,
                            it.status.convertDefaultValue()
                        )
                        visibility = View.VISIBLE
                    }
                    itemContainerLl.addView(itemView)
                }
            }

            VoucherMultipleItem.CONVER_RECORD -> {
                val converRecordsList = voucherCardQueryBean.converRecords
                converRecordsList?.forEachIndexed { index, it ->
                    val itemView =
                        View.inflate(mContext, R.layout.item_voucher_card_exchange_record, null)

                    itemView.findViewById<TextView>(R.id.exchange_record_title_tv).apply {
                        text = mContext.getString(R.string.voucher_coupon, index + 1)
                    }

                    when (it.status) {  //0：未核销，1：已核销，2：未发券,3:已失效
                        0 -> {  //未核销
                            itemView.findViewById<TextView>(R.id.exchange_record_item_coupon_time_tv)
                                .apply {
                                    visibility = View.VISIBLE
                                    text = mContext.getString(
                                        R.string.voucher_coupon_time,
                                        it.createDate.convertDefaultValue()
                                    )
                                }

                            itemView.findViewById<TextView>(R.id.exchange_record_item_write_off_time_tv).visibility =
                                View.GONE
                            itemView.findViewById<TextView>(R.id.exchange_record_item_write_off_stores_tv).visibility =
                                View.GONE
                            itemView.findViewById<TextView>(R.id.exchange_record_item_write_off_consultant_tv).visibility =
                                View.GONE

                            itemView.findViewById<TextView>(R.id.exchange_record_item_expiration_date_tv)
                                .apply {
                                    visibility = View.VISIBLE
                                    text = mContext.getString(
                                        R.string.voucher_postpone_date,
                                        it.validtime.convertDefaultValue()
                                    )
                                }

                            itemView.findViewById<TextView>(R.id.exchange_record_item_error_tv)
                                .apply {
                                    visibility = View.VISIBLE
                                    text = mContext.getString(R.string.voucher_not_conver)
                                }
                            itemView.findViewById<LinearLayout>(R.id.exchange_record_item_bottom_ll).visibility =
                                View.VISIBLE
                            itemView.findViewById<TextView>(R.id.exchange_record_item_query_btn)
                                .apply {
                                    visibility = View.VISIBLE
                                    setOnClickListener { v ->
                                        listener?.onQueryClick(index, it)
                                    }
                                }

                            itemView.findViewById<TextView>(R.id.exchange_record_item_send_conver_code_tv)
                                .apply {
                                    visibility = View.VISIBLE
                                    setOnClickListener { v ->
                                        listener?.onSendConverCodeClick(this, index, it)
                                    }
                                }
                        }
                        1 -> {  //已核销
                            itemView.findViewById<TextView>(R.id.exchange_record_item_coupon_time_tv)
                                .apply {
                                    visibility = View.VISIBLE
                                    text = mContext.getString(
                                        R.string.voucher_coupon_time,
                                        it.createDate.convertDefaultValue()
                                    )
                                }

                            itemView.findViewById<TextView>(R.id.exchange_record_item_write_off_time_tv)
                                .apply {
                                    visibility = View.VISIBLE
                                    text = mContext.getString(
                                        R.string.voucher_write_off_time,
                                        it.useDate.convertDefaultValue()
                                    )

                                }
                            itemView.findViewById<TextView>(R.id.exchange_record_item_write_off_stores_tv)
                                .apply {
                                    visibility = View.VISIBLE
                                    text = mContext.getString(
                                        R.string.voucher_write_off_stores,
                                        it.useShopName.convertDefaultValue()
                                    )

                                }
                            itemView.findViewById<TextView>(R.id.exchange_record_item_write_off_consultant_tv)
                                .apply {
                                    visibility = View.VISIBLE
                                    text = mContext.getString(
                                        R.string.voucher_write_off_consultant,
                                        it.salerName.convertDefaultValue()
                                    )

                                }
                            itemView.findViewById<TextView>(R.id.exchange_record_item_expiration_date_tv).visibility =
                                View.GONE
                            itemView.findViewById<TextView>(R.id.exchange_record_item_error_tv).visibility =
                                View.GONE
                            itemView.findViewById<LinearLayout>(R.id.exchange_record_item_bottom_ll).visibility =
                                View.GONE
                        }
                        2 -> {   //未发券
                            itemView.findViewById<TextView>(R.id.exchange_record_item_coupon_time_tv).visibility =
                                View.GONE
                            itemView.findViewById<TextView>(R.id.exchange_record_item_write_off_time_tv).visibility =
                                View.GONE
                            itemView.findViewById<TextView>(R.id.exchange_record_item_write_off_stores_tv).visibility =
                                View.GONE
                            itemView.findViewById<TextView>(R.id.exchange_record_item_write_off_consultant_tv).visibility =
                                View.GONE

                            itemView.findViewById<TextView>(R.id.exchange_record_item_expiration_date_tv).visibility =
                                View.GONE
                            itemView.findViewById<TextView>(R.id.exchange_record_item_error_tv)
                                .apply {
                                    visibility = View.VISIBLE
                                    text = it.willSendDate
                                }
                            itemView.findViewById<LinearLayout>(R.id.exchange_record_item_bottom_ll).visibility =
                                View.GONE

                        }
                        3 -> {  //已失效
                            itemView.findViewById<TextView>(R.id.exchange_record_item_coupon_time_tv)
                                .apply {
                                    visibility = View.VISIBLE
                                    text = mContext.getString(
                                        R.string.voucher_coupon_time,
                                        it.createDate.convertDefaultValue()
                                    )
                                }

                            itemView.findViewById<TextView>(R.id.exchange_record_item_write_off_time_tv).visibility =
                                View.GONE
                            itemView.findViewById<TextView>(R.id.exchange_record_item_write_off_stores_tv).visibility =
                                View.GONE
                            itemView.findViewById<TextView>(R.id.exchange_record_item_write_off_consultant_tv).visibility =
                                View.GONE
                            itemView.findViewById<TextView>(R.id.exchange_record_item_expiration_date_tv)
                                .apply {
                                    visibility = View.VISIBLE
                                    text = mContext.getString(
                                        R.string.voucher_postpone_date,
                                        it.validtime.convertDefaultValue()
                                    )
                                }
                            itemView.findViewById<TextView>(R.id.exchange_record_item_error_tv)
                                .apply {
                                    visibility = View.VISIBLE
                                    text = mContext.getString(R.string.voucher_expired)
                                }

                            itemView.findViewById<LinearLayout>(R.id.exchange_record_item_bottom_ll)
                                .apply {
                                    if (it.isPostPone) {
                                        visibility = View.VISIBLE
                                    } else {
                                        visibility = View.GONE
                                    }
                                }
                            itemView.findViewById<TextView>(R.id.exchange_record_item_postpone_btn)
                                .apply {
                                    visibility = View.VISIBLE
                                    setOnClickListener { v ->
                                        listener?.onPostponeClick(index, it)
                                    }
                                }
                        }
                        else -> {
                        }
                    }
                    itemContainerLl.addView(itemView)
                }
            }
            VoucherMultipleItem.MEGER_RECORD -> {
                val megerRecordsList = voucherCardQueryBean.megerRecords
                megerRecordsList?.forEach {
                    val itemView =
                        View.inflate(mContext, R.layout.item_voucher_card_default_record, null)
                    itemView.findViewById<TextView>(R.id.default_record_item_one_tv)
                        .apply {
                            text = mContext.getString(R.string.voucher_combine_time, it.megerDate)
                        }
                    itemView.findViewById<TextView>(R.id.default_record_item_two_tv).apply {
                        text = mContext.getString(R.string.voucher_old_number, it.oldMobile)
                    }

                    itemView.findViewById<TextView>(R.id.default_record_item_three_tv).apply {
                        text = mContext.getString(R.string.voucher_new_number, it.newMobile)
                    }

                    itemView.findViewById<TextView>(R.id.default_record_item_four_tv).apply {
                        visibility = View.GONE
                    }

                    itemView.findViewById<TextView>(R.id.default_record_item_five_tv).apply {
                        visibility = View.GONE
                    }
                    itemContainerLl.addView(itemView)
                }
            }
        }
    }

    interface OnItemChildClickListener {
        /**
         * 延期
         */
        fun onPostponeClick(position: Int, bean: ConverRecordsBean)

        /**
         * 查询
         */
        fun onQueryClick(position: Int, bena: ConverRecordsBean)

        /**
         * 发送兑换码
         */
        fun onSendConverCodeClick(view: TextView, position: Int, bean: ConverRecordsBean)

    }
}