package com.urovo.activity.qualityBackAct.view

import android.content.Intent
import android.view.View
import android.view.ViewGroup
import android.widget.CheckBox
import android.widget.RadioButton
import android.widget.TextView
import androidx.core.content.ContextCompat
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.urovo.activity.qualityBackAct.model.DispatchProductEntity
import com.urovo.activity.qualityBackAct.ui.DeliveryBillChooseAdapter
import com.urovo.activity.qualityBackAct.ui.DeliveryBillProductChooseAdapter
import com.urovo.wugumofang.R
import com.urovo.wugumofang.base.BaseActivity
import com.urovo.wugumofang.constants.AppConstants
import com.urovo.wugumofang.constants.ConstantParams
import com.urovo.wugumofang.utils.isNotNullOrSize
import com.urovo.wugumofang.view.LoadWaitDominator
import com.wgmf.trade.GlobalApp
import com.zsy.libcore.ui.ada.helper.DividerItemDecoration
import kotlinx.android.synthetic.main.public_list_view.*
import org.jetbrains.anko.find

/**
 * 选择配送单号
 * @author LW
 */
class DispatchProductChooseActivity : BaseActivity() {

    override fun addRootLayoutId(): Int = R.layout.public_list_view

    override fun rootViewGroup(): ViewGroup = mPublicContainer

    private var mDeliveryBillChooseAdapter: DeliveryBillChooseAdapter? = null

    private var mDeliveryBillProductChooseAdapter: DeliveryBillProductChooseAdapter? = null

    private var mDispatchProductList =
        mutableListOf<DispatchProductEntity.DispatchProductBean>()

    private var mDispatchDetailList =
        mutableListOf<DispatchProductEntity.DispatchProductBean.DispatchDetailBean>()

    private val mDispatchProductEntity: DispatchProductEntity? by lazy {
        intent.getSerializableExtra(AppConstants.ExtraKeys.DISPATCH_PRODUCT_DATA) as DispatchProductEntity?
    }

    private val mDispatchProductPos: Int by lazy {
        intent.getIntExtra(AppConstants.ExtraKeys.DISPATCH_PRODUCT_POS, -1)
    }

    private val mDispathProductType: String by lazy {
        intent.getStringExtra(AppConstants.ExtraKeys.DISPATCH_PRODUCT_TYPE)
    }

    private var mCurrentPos: Int = -1

    companion object {
        const val DELIVERY_BILL: String = "DELIVERY_BILL"

        const val DELIVERY_BILL_PRODUCT: String = "DELIVERY_BILL_PRODUCT"
    }

    override fun initTitle() {
        super.initTitle()
        if (mDispathProductType == DELIVERY_BILL) {
            titleTextView.text = getString(R.string.problem_escalation_chose_delivery_bill)
        } else if (mDispathProductType == DELIVERY_BILL_PRODUCT) {
            titleTextView.text = getString(R.string.choose_product)
        }

        if (mDispathProductType == DELIVERY_BILL_PRODUCT) {
            complete.apply {
                visibility = View.VISIBLE
                text = getString(R.string.confirm_btn_txt)
                setBackgroundResource(R.drawable.shape_select_index_btn)
                setOnClickListener {
                    choseItemJumpLogic()
                }
            }
        }
    }


    override fun initViewNew() {
        initTitle()
        mCurrentPos = mDispatchProductPos

        showState(LoadWaitDominator.DISMISS)
        if (mDispathProductType == DELIVERY_BILL) {
            mDispatchProductList = mDispatchProductEntity?.resultInfo?.toMutableList()
                ?: mutableListOf()
            if (mDispatchProductList.isNotNullOrSize()) {
                mDeliveryBillChooseAdapter =
                    DeliveryBillChooseAdapter(mDispatchProductPos, mDispatchProductList).apply {
                        setOnItemClickListener { _, _, position ->
                            if (mDispatchProductPos >= 0) {
                                mDispatchProductEntity?.resultInfo?.get(mDispatchProductPos)
                                    ?.dispatchDetailList?.forEach {
                                        it.chooseStatus = false
                                    }
                            }

                            mCurrentPos = position
                            choseItemJumpLogic()
                        }
                    }
            } else {
                showState(LoadWaitDominator.NO_DATA)
            }
        } else if (mDispathProductType == DELIVERY_BILL_PRODUCT) {
            var dispatchProductBean: DispatchProductEntity.DispatchProductBean? = null

            mDispatchDetailList = if (mDispatchProductPos >= 0) {
                dispatchProductBean = mDispatchProductEntity?.resultInfo?.get(mDispatchProductPos)
                dispatchProductBean?.dispatchDetailList?.toMutableList()
                    ?: mutableListOf()
            } else {
                mutableListOf()
            }
            if (mDispatchDetailList.isNotNullOrSize()) {
                val deliveryBillChooseView =
                    layoutInflater.inflate(
                        R.layout.item_delivery_bill_choose,
                        mPublicListViewLv.getParent() as ViewGroup,
                        false
                    ).apply {
                        find<RadioButton>(R.id.select_item_radio).visibility = View.GONE

                        find<TextView>(R.id.delivery_bill).text =
                            GlobalApp.context.getString(
                                R.string.problem_escalation_delivery_bill,
                                dispatchProductBean?.waveNo
                            )

                        find<TextView>(R.id.delivery_date).text =
                            GlobalApp.context.getString(
                                R.string.problem_escalation_delivery_date,
                                dispatchProductBean?.dispatchDate
                            )

                        find<TextView>(R.id.delivery_total_amount).text =
                            GlobalApp.context.getString(
                                R.string.problem_escalation_total_amount,
                                dispatchProductBean?.allProSumAmount
                            )

                        find<TextView>(R.id.delivery_total_number).text =
                            GlobalApp.context.getString(
                                R.string.problem_escalation_total_number,
                                dispatchProductBean?.allProSumQty
                            )
                    }
                mDeliveryBillProductChooseAdapter =
                    DeliveryBillProductChooseAdapter(mDispatchDetailList).apply {
                        setOnItemClickListener { baseQuickAdapter, view, position ->
                            val dispatchDetailBean =
                                baseQuickAdapter.getItem(position) as DispatchProductEntity.DispatchProductBean.DispatchDetailBean
                            dispatchDetailBean.chooseStatus = !dispatchDetailBean.chooseStatus
                            baseQuickAdapter.notifyDataSetChanged()
                        }
                    }
                mDeliveryBillProductChooseAdapter?.addHeaderView(deliveryBillChooseView)
            } else {
                showError(R.string.choose_product_emtry_tip)
            }
        }

        mPublicListViewLv.apply {
            layoutManager = LinearLayoutManager(mContext, RecyclerView.VERTICAL, false)
            addItemDecoration(
                DividerItemDecoration(
                    mContext,
                    DividerItemDecoration.VERTICAL_LIST,
                    1,
                    ContextCompat.getColor(mContext, R.color.pop_query_devider)
                )
            )
            if (mDispathProductType == DELIVERY_BILL) {
                adapter = mDeliveryBillChooseAdapter
            } else if (mDispathProductType == DELIVERY_BILL_PRODUCT) {
                adapter = mDeliveryBillProductChooseAdapter
            }
        }

    }


    /**
     * 选择列表逻辑处理
     */
    private fun choseItemJumpLogic() {
        setResult(ConstantParams.CONSTANT_STARTS_1, Intent().apply {
            putExtra(
                AppConstants.ExtraKeys.DISPATCH_PRODUCT_DATA,
                mDispatchProductEntity
            )
            putExtra(
                AppConstants.ExtraKeys.DISPATCH_PRODUCT_POS,
                mCurrentPos
            )
            putExtra(
                AppConstants.ExtraKeys.DISPATCH_PRODUCT_TYPE,
                mDispathProductType
            )
        })
        this.finish()
    }

}