package com.urovo.activity.qualityBackAct.view

import android.graphics.drawable.BitmapDrawable
import android.os.Bundle
import android.view.Gravity
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.*
import androidx.fragment.app.Fragment
import com.github.yoojia.inputs.AndroidNextInputs
import com.github.yoojia.inputs.StaticScheme
import com.urovo.activity.qualityBackAct.model.MenuItem
import com.urovo.activity.qualityBackAct.net.BaseEntity
import com.urovo.activity.qualityBackAct.net.SuggestionEntity
import com.urovo.activity.qualityBackAct.presenter.QBackContract
import com.urovo.activity.qualityBackAct.presenter.QBackInputPresenter
import com.urovo.activity.qualityBackAct.ui.QBackExpandableAdapter
import com.urovo.wugumofang.R
import com.urovo.wugumofang.system.SystemHelper
import com.urovo.wugumofang.ui.ViewUtil
import com.urovo.wugumofang.utils.MyToast
import com.urovo.wugumofang.view.CustomPopupWindow
import kotlinx.android.synthetic.main.ll_layout_qback_suggestion.*

/**
 * 意见与建议
 */
class SuggestionFra : Fragment(), QBackContract.view {

    private var mPresenter: QBackInputPresenter? = null

    private val mInputs = AndroidNextInputs() //校验框架

    private var mThemeType: Int = -1 //主题类型一级

    private var mFeedBackType: Int = -1 //主题类型二级

    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {
        return inflater.inflate(
            R.layout.ll_layout_qback_suggestion,
            null, false
        ) as LinearLayout
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        initUI()
    }

    /**
     * 初始化主题类型PopupWindow选择框
     */
    private val topicTypePopupWindow: CustomPopupWindow by lazy {
        CustomPopupWindow.Builder(context)
            .setContentView(R.layout.popup_white_expandable_list_ll)
            .setFouse(true)
            .setOutSideCancel(true)
            .setAnimationStyle(R.style.dropmenu_anim_style)
            .setBackgroundDrawable(BitmapDrawable())
            .setwidth(ViewGroup.LayoutParams.WRAP_CONTENT)
            .setheight(ViewGroup.LayoutParams.WRAP_CONTENT)
            .builder()
    }

    /**
     * 显示主题类型PopupWindow选择框
     */
    private fun showSuggestionTypePopupWindow(array: Array<MenuItem>) {
        val pop = topicTypePopupWindow
        pop.apply {
            val exList = getItemView(R.id.popupList) as ExpandableListView
            val exAdapter = QBackExpandableAdapter(context, array)
            exList.setGroupIndicator(null)
            exList.dividerHeight = 1
            exList.setAdapter(exAdapter)
            exList.setOnGroupClickListener { _, _, groupPos, l ->
                val menuItem = array[groupPos]
                if (menuItem.childrenArray == null) { //如果该大类下没有小分支，就直接跳转；否则展开子列表。
                    suggestion_type.text = menuItem.qtype.showName
                    mThemeType = menuItem.qtype.subType
                    mFeedBackType = -1
                    pop.dismiss()
                    mInputs.test()
                }
                false
            }
            exList.setOnChildClickListener { _, _, groupPos, childPos, l ->
                suggestion_type.text = array[groupPos].childrenArray[childPos].showName
                mThemeType = array[groupPos].qtype.subType
                mFeedBackType = array[groupPos].childrenArray[childPos].subType
                pop.dismiss()
                mInputs.test()
                true
            }
        }.showAsLocation(suggestion_type, Gravity.BOTTOM, 0, 0)
    }

    override fun initUI() {
        suggestion_person_show.setText(SystemHelper.LoginUserName)
        telephone.setText(SystemHelper.LoginUserMobile)

        val topicTypeOptionsArrays = mPresenter?.suggestionData()?.getTopicTypeOptions()
        suggestion_type.setOnClickListener {
            topicTypeOptionsArrays?.let {
                showSuggestionTypePopupWindow(it)
            }
        }

        mInputs.add(suggestion_person_show, StaticScheme.Required())
        mInputs.add(telephone, StaticScheme.Required(), StaticScheme.ChineseMobile())
        mInputs.add(suggestion_type, StaticScheme.Required())
    }

    override fun showMsg(msg: String?) {
        MyToast.show(msg)
    }

    override fun packData(): BaseEntity? {
        return if (mInputs.test()) {
            // 校验通过，开始“读取数据”并“组建请求报文的Json”
            val json = SuggestionEntity()
            json.setSubmiter(suggestion_person_show.text.toString())
            json.setMobile(telephone.text.toString())
            json.setDescription(report_edit.text.toString())
            json.setShopName(SystemHelper.LoginOrganName)
            json.setTitle(mPresenter?.data?.parentType?.showName)
            json.setThemeType(mThemeType)
            if (mFeedBackType != -1)
                json.setFeedBackType(mFeedBackType)
            json
        } else {
            MyToast.show(activity, "填写有误，请检查", Toast.LENGTH_SHORT)
            null
        }
    }

    override fun showProgressBar(title: String?, msg: String?, cancelable: Boolean) {
    }

    override fun dismissProgressBar() {
        ViewUtil.dismissProgress()
    }

    override fun setPresenter(presenter: QBackContract.presenter?) {
        mPresenter = presenter as QBackInputPresenter
    }
}