package com.urovo.activity.ra90listact.view

import android.content.Intent
import android.os.Bundle
import com.alibaba.android.arouter.facade.annotation.Route
import com.urovo.activity.ra90listact.presenter.Ra90ListCon
import com.urovo.activity.ra90listact.presenter.Ra90ListPre
import com.urovo.wugumofang.R
import com.urovo.wugumofang.base.BaseActivity
import com.urovo.wugumofang.constants.RouterParams
import com.urovo.wugumofang.system.SystemHelper
import com.urovo.wugumofang.view.LoadWaitDominator
import kotlinx.android.synthetic.main.ll_ra90_act.*
import kotlinx.android.synthetic.main.ll_ra_list_radio_layout.*
import kotlinx.android.synthetic.main.ra90_data_item.*

/**
 * @author SASUKE on 2018/3/17.
 * 顾客90天未回头名单
 */
@Route(path = RouterParams.ORIGIN.RA_90_NO_RETURN_LIST)
class Ra90ListAct : BaseActivity(), Ra90ListCon.view {
    var mPre: Ra90ListPre? = null
    var monthOff = 0//月份偏移
    var clientType = 0//全部 - 新客 - 老客
    var loadWaitDominator: LoadWaitDominator? = null// 加载中、暂无数据、网络错误等等状态显示的全家桶。

    companion object {
        const val ALL_RA = 0//全部
        const val NEW_RA = 1//新客
        const val OLD_RA = 2//老客
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.ll_ra90_act)
        initTitle()
        titleTextView.text = "顾客90天未回头名单"
        Ra90ListPre(this).apply { start() }
    }

    override fun init() {
        //日期字
        yearTv.apply {
            setTextColor(resources.getColor(R.color.title_yellow_bg))
            text = mPre?.getDateTextByOff(0)
        }
        storeName.text = SystemHelper.LoginOrganName
        //切换日期按钮点击
        arrayOf(leftBtn, rightBtn).forEach {
            btn ->
            btn.setOnClickListener {
                monthOff += if (btn == leftBtn) -1 else 1
                yearTv.text = mPre?.getDateTextByOff(monthOff)
                refreshBtnState()
                mPre?.queryData(clientType, mPre?.getDateByOff(monthOff) ?: "")
            }
        }
        //标签点击监听
        allBtn.isChecked = true
        radioGroup.setOnCheckedChangeListener { group, checkedId ->
            clientType = when (checkedId) {
                allBtn.id -> ALL_RA
                newClientBtn.id -> NEW_RA
                oldClientBtn.id -> OLD_RA
                else -> -1
            }
            showData(clientType)
        }
        //查看那啥详情的跳转按钮
        jumpDetailBtn.setOnClickListener {
            startActivity(Intent(this,Ra90ListDetailAct::class.java).apply {
                putExtra(Ra90ListDetailAct.INTENT_CLIENT_TYPE,clientType)
//                putExtra(Ra90ListDetailAct.INTENT_NO_RETURN_NUM,notCallCountTv.text.toString()?.toInt())
//                putExtra(Ra90ListDetailAct.INTENT_RETURNED_NUM,callCountTv.text.toString()?.toInt())
                putExtra(Ra90ListDetailAct.INTENT_MONTH,mPre?.getDateByOff(monthOff) ?: "")//月份传过去用来网络请求的
            })
        }

        //右初始不可点击
        rightBtn.isEnabled = false
        loadWaitDominator = LoadWaitDominator(this, container)
    }

    override fun onResume() {
        super.onResume()
        //请求
        mPre?.queryData(clientType, mPre?.getDateByOff(monthOff) ?: "")
    }

    /**
     * 网络返回后显示数据（先留着，等接口好了再补上
     */
    override fun showData(clientType: Int) {
        if(this.isFinishing){
            return
        }
        val data = when (clientType) {
            ALL_RA -> mPre?.resSave?.resultInfo?.allRa
            NEW_RA -> mPre?.resSave?.resultInfo?.newRa
            OLD_RA -> mPre?.resSave?.resultInfo?.oldRa
            else -> null
        }
        reBuyCountTv.text = data?.reBuyCount.toString()
        notReBuyCountTv.text = data?.notReBuyCount.toString()
        reBuyRateTv.text = data?.reBuyRate
        callCountTv.text = data?.callCount.toString()
        notCallCountTv.text = data?.notCallCount.toString()
    }

    /**
     * 刷新按钮显示状态（只允许查两个月）
     */
    fun refreshBtnState() {
        rightBtn.isEnabled = (monthOff < 0)
//        leftBtn.isEnabled = (monthOff > -1)
    }


    override fun setPresenter(presenter: Ra90ListCon.pre?) {
        mPre = presenter as Ra90ListPre
    }

    /**
     * 以下用于显示加载状态
     */
    override fun showLoading() {
        loadWaitDominator?.let { it.show(LoadWaitDominator.LOADING) }
    }

    override fun showNetError() {
        loadWaitDominator?.let { it.show(LoadWaitDominator.NET_ERROR) }
    }

    override fun showNoData() {
        loadWaitDominator?.let { it.show(LoadWaitDominator.NO_DATA) }
    }

    override fun dismissLoadWait() {
        loadWaitDominator?.let { it.dismissAll() }
    }
}