package com.urovo.activity.receiveraddress.view

import android.content.Context
import android.content.Intent
import android.os.Bundle
import android.view.inputmethod.InputMethodManager
import com.urovo.activity.memorderact.net.ReceiversInfosRes
import com.urovo.activity.memorderact.view.MemOrderSubmitAct
import com.urovo.activity.receiveraddress.net.AddrSubmitBean
import com.urovo.activity.receiveraddress.presenter.AddrEditContract
import com.urovo.activity.receiveraddress.presenter.AddrEditPresenter
import com.urovo.wugumofang.R
import com.urovo.wugumofang.base.BaseActivity
import com.urovo.wugumofang.ui.ViewUtil
import com.urovo.wugumofang.ui.vibrate
import com.urovo.wugumofang.utils.AddressPickerDominator
import kotlinx.android.synthetic.main.receiver_addr_edit_act.*

/**
 * @author Sasuke on 2019-7-23 0023.
 */
class ReceiverAddrEditAct : BaseActivity(), AddrEditContract.addrEditView {
    companion object {
        @JvmStatic
        fun launch(context: Context, customerTel: String?) {
            context.startActivity(Intent(context, ReceiverAddrEditAct::class.java).apply {
                putExtra("customerTel", customerTel)
            })
        }

        @JvmStatic
        fun launch(context: Context, customerTel: String?,addrItem: ReceiversInfosRes.ResultInfo?) {
            context.startActivity(Intent(context, ReceiverAddrEditAct::class.java).apply {
                putExtra("customerTel", customerTel)
                putExtra("INTENT_ADDRS", addrItem)
            })
        }
    }

    var mPre: AddrEditPresenter? = null
    var addrPicker: AddressPickerDominator? = null

    //编辑模式下，而不是新增模式下，传进来的收货地址信息
    var addrInfo: ReceiversInfosRes.ResultInfo? = null

    //顾客手机号
    val customerTel by lazy {
        intent.getStringExtra("customerTel")
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.receiver_addr_edit_act)
        initTitle()
        AddrEditPresenter(this).start()
        setListeners()
    }

    override fun init() {
        titleTextView.text = if (addrInfo == null) "新增收货地址" else "编辑收货地址"
        addrPicker = AddressPickerDominator(this@ReceiverAddrEditAct, root) { dominator ->
            receiverDistrict.text = dominator.province?.regionName + dominator.city?.regionName + dominator.district?.regionName
        }
        addrInfo =  intent.getSerializableExtra("INTENT_ADDRS") as ReceiversInfosRes.ResultInfo?
        addrInfo?.let {
            //先缓存一份
            addrPicker?.setInitedAddress(it.province.regionName, it.provinceId,
                    it.city.regionName, it.cityId,
                    it.district.regionName, it.districtId)
            //然后把它显示出来
            receiverName.setText(it?.receiverName)
            receiverPhone.setText(it?.receiverPhone)
            receiverDistrict.text = it.province?.regionName + it.city?.regionName + it.district?.regionName
            receiverAdditional.setText(it?.additional)
        }
    }


    fun setListeners() {
        //选择省市区
        receiverDistrict.setOnClickListener {
            hideKeyboard()
            addrPicker?.show()
        }

        //保存并使用
        saveAndUseBtn.setOnClickListener {
            arrayOf(receiverName, receiverPhone, receiverDistrict, receiverAdditional).forEach {
                if (it.text.toString().trim().isBlank()) {
                    vibrate(it)
                    return@setOnClickListener
                }
            }
            if (receiverPhone.text.toString().trim().length != 11) {
                showMsg("请输入正确的手机号码")
                return@setOnClickListener
            }
            //校验通过，开始提交
            ViewUtil.showProgress(this, "请稍等", "正在保存...")
            mPre?.submitNewReceiverAddress(
                    customerTel?:"",
                    receiverName.text.toString().trim(),
                    receiverPhone.text.toString().trim(),
                    addrPicker!!,
                    receiverAdditional.text.toString().trim(),
                    addrInfo)
        }
    }

    /**
     * 提交成功
     */
    override fun submitSussces(msg: String, addrBean: AddrSubmitBean?) {
        ViewUtil.dismissProgress()
        showMsg(msg)
        startActivity(Intent(this, MemOrderSubmitAct::class.java).apply {
            addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP)
            putExtra(MemOrderSubmitAct.NEW_INTENT_ADDR, addrBean)
        })
    }

    /**
     * 提交失败
     */
    override fun submitFailed(msg: String) {
        ViewUtil.dismissProgress()
        showMsg(msg)
    }

    override fun setPresenter(presenter: AddrEditContract.addrEditPre?) {
        mPre = presenter as AddrEditPresenter
    }

    /***
     * 隐藏键盘
     */
    fun hideKeyboard() {
        val imm = getSystemService(Context.INPUT_METHOD_SERVICE) as InputMethodManager
        imm.showSoftInput(receiverDistrict, InputMethodManager.SHOW_FORCED)
        imm.hideSoftInputFromWindow(receiverDistrict.getWindowToken(), 0) //强制隐藏键盘
    }
}