package com.urovo.activity.report;


import android.content.Context;
import android.content.Intent;
import android.net.Uri;
import android.os.Bundle;
import android.text.TextUtils;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.ListView;
import android.widget.TextView;

import com.google.gson.Gson;
import com.google.gson.reflect.TypeToken;
import com.urovo.wgmf.db.dao.dao.Store001DbOprate;
import com.urovo.wgmf.db.dao.entitys.LogisticsTelPhoneNumberEntity;
import com.urovo.wugumofang.R;
import com.urovo.wugumofang.base.BaseActivity;
import com.urovo.wugumofang.constants.NetParam;
import com.urovo.wugumofang.net.OkHttpDominator;
import com.urovo.wugumofang.system.SystemHelper;
import com.urovo.wugumofang.ui.ViewUtil;
import com.zsy.libext.widgets.dialog.effectsdialog.EffectsAlertDialog;

import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;

import okhttp3.Request;

/**
 * 上报反馈；物流人员信息；
 */
public class TelLogisticsActivity extends BaseActivity {
    private String TAG = TelLogisticsActivity.class.getSimpleName();
    private ListView mListView;
    private MyAdapter mAdapter;
    private ArrayList<LogisticsTelPhoneNumberEntity> childrenlistItem = new ArrayList<LogisticsTelPhoneNumberEntity>();

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_store_list);
        initTitle();
        initView();
        getData();
    }

    private void getData() {
        String id = Store001DbOprate.GetDeptcsidByStoreId(SystemHelper.LoginOrganId);
        if (!TextUtils.isEmpty(id)) {
            HashMap<String, String> param = new HashMap<>();
            param.put("code",id);
            ViewUtil.showProgress(TelLogisticsActivity.this, "", "请稍等.....");
            OkHttpDominator.getAsyn(NetParam.URL_STREAM_DEALPERSON, param,
                    new OkHttpDominator.ResultCallback<String>() {
                        @Override
                        public void onError(Request request, Exception e) {
                            ViewUtil.dismissProgress();
                            showMsg("获取物流人员"+ getString(R.string.request_data_fail));
                        }

                        @Override
                        public void onResponse(String json) {
                            ViewUtil.dismissProgress();
                            if(json == null || TextUtils.isEmpty(json)){
                                showMsg("没有返回数据");
                                return;
                            }
                            Type listType = new TypeToken<LinkedList<LogisticsTelPhoneNumberEntity>>() {}.getType();
                            Gson gson = new Gson();
                            try{
                                LinkedList<LogisticsTelPhoneNumberEntity> resources = gson.fromJson(json, listType);
                                for (Iterator iterator = resources.iterator(); iterator.hasNext(); ) {
                                    LogisticsTelPhoneNumberEntity entity= (LogisticsTelPhoneNumberEntity) iterator.next();
                                    childrenlistItem.add(entity);
                                }
                            }catch(Exception e){
                                e.printStackTrace();
                                showMsg("数据解析错误");
                                return;
                            }

                            mAdapter.notifyDataSetChanged();
                        }
                    });
//            RequestParams params = new RequestParams();
//            params.addQueryStringParameter("code", id);
            // 发送管理门店数据请求
//            mHttpUtils.send(HttpRequest.HttpMethod.GET, NetParam.URL_STREAM_DEALPERSON,
//                    params, mStreamDealPersonRequestCallBack);
        }
    }

    @Override
    public void initTitle() {
        super.initTitle();
        titleTextView.setText("拨打物流专员电话");
    }

    private void initView() {

        mListView = (ListView) findViewById(R.id.shop_lv);
        mAdapter = new MyAdapter(this, childrenlistItem);
        mListView.setAdapter(mAdapter);

    }


    class MyAdapter extends BaseAdapter implements View.OnClickListener{
        private Context context;
        private ArrayList<LogisticsTelPhoneNumberEntity> childrenlistItem;

        public MyAdapter(Context context,
                         ArrayList<LogisticsTelPhoneNumberEntity> childrenlistItem) {
            super();
            this.context = context;
            this.childrenlistItem = childrenlistItem;
        }

        @Override
        public int getCount() {
            return childrenlistItem.size();
        }

        @Override
        public Object getItem(int position) {
            return position;
        }

        @Override
        public long getItemId(int position) {
            return position;
        }

        @Override
        public View getView(int position, View convertView, ViewGroup parent) {
            ViewHolder holder;

            if (convertView == null) {
                holder = new ViewHolder();
                convertView = TelLogisticsActivity.this.getLayoutInflater()
                        .inflate(R.layout.logistics_telphone_list_item, null);
                holder.principal_position_txt = (TextView) convertView.findViewById(R.id.principal_position_txt);
                holder.principal_name_txt = (TextView) convertView
                        .findViewById(R.id.principal_name_txt);
                holder.principal_phone_txt = (TextView) convertView
                        .findViewById(R.id.principal_phone_txt);
                holder.principal_mobile_txt = (TextView) convertView
                        .findViewById(R.id.principal_mobile_txt);
                holder.country_principal1_position_txt = (TextView) convertView
                        .findViewById(R.id.country_principal1_position_txt);
                holder.country_principal1_Name_txt = (TextView) convertView
                        .findViewById(R.id.country_principal1_Name_txt);
                holder.country_principal1_phone_txt = (TextView) convertView
                        .findViewById(R.id.country_principal1_phone_txt);
                holder.country_principal1_mobile_txt = (TextView) convertView
                        .findViewById(R.id.country_principal1_mobile_txt);
                holder.country_principal2_position_txt = (TextView) convertView
                        .findViewById(R.id.country_principal2_position_txt);
                holder.country_principal2_name_txt = (TextView) convertView
                        .findViewById(R.id.country_principal2_name_txt);
                holder.country_principal2_mobile_txt = (TextView) convertView
                        .findViewById(R.id.country_principal2_mobile_txt);
//                holder.principal_phone_txt.setOnClickListener(this);
//                holder.principal_mobile_txt.setOnClickListener(this);
//                holder.country_principal1_phone_txt.setOnClickListener(this);
//                holder.country_principal1_mobile_txt.setOnClickListener(this);
//                holder.country_principal2_mobile_txt.setOnClickListener(this);
                convertView.setTag(holder);
            } else {
                holder = (ViewHolder) convertView.getTag();
            }
            setItemData(position, holder);
            return convertView;
        }

        private void setItemData(int position, ViewHolder holder) {
            LogisticsTelPhoneNumberEntity tempLog = childrenlistItem.get(position);
            holder.principal_position_txt.setText(tempLog.getPrincipalPosition());
            holder.principal_name_txt.setText(tempLog.getPrincipalName());


            holder.principal_phone_txt.setText(tempLog.getPrincipalPhone());
            holder.principal_mobile_txt.setText(tempLog.getPrincipalMobile());

            holder.country_principal1_position_txt.setText(tempLog.getCountryPrincipal1Position());
            holder.country_principal1_Name_txt.setText(tempLog.getCountryPrincipal1Name());

            holder.country_principal1_phone_txt.setText(tempLog.getCountryPrincipal1Phone());
            holder.country_principal1_mobile_txt.setText(tempLog.getCountryPrincipal1Mobile());

            holder.country_principal2_position_txt.setText(tempLog.getCountryPrincipal2Position());
            holder.country_principal2_name_txt.setText(tempLog.getCountryPrincipal2Name());

            holder.country_principal2_mobile_txt.setText(tempLog.getCountryPrincipal2Mobile());
        }

        @Override
        public void onClick(View v) {
            final String phoneNum = ((TextView)v).getText().toString().trim();
            if (!TextUtils.isEmpty(phoneNum)) {
                final EffectsAlertDialog telDialog = EffectsAlertDialog.getInstance(TelLogisticsActivity.this);
                telDialog.withTitle(phoneNum).withMessage("").setRightButtonClick(new View.OnClickListener() {
                    @Override public void onClick(View v) {
                        Intent intentCall = new Intent(Intent.ACTION_CALL, Uri.parse("tel:" + phoneNum));
//                        Intent intentCall = new Intent(Intent.ACTION_DIAL, Uri.parse("tel:" + phoneNum));
//                         Intent intentCall = new Intent("android.intent.action.CALL", Uri.parse("tel:" + phoneNum));
                        intentCall.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
                        TelLogisticsActivity.this.startActivity(intentCall);
                        telDialog.dismiss();
                    }
                }).show();
            } else {
                showMsg("暂无电话");
            }
        }

        public class ViewHolder {
            TextView principal_position_txt;
            TextView principal_name_txt;
            TextView principal_phone_txt;
            TextView principal_mobile_txt;
            TextView country_principal1_position_txt;
            TextView country_principal1_Name_txt;
            TextView country_principal1_phone_txt;
            TextView country_principal1_mobile_txt;
            TextView country_principal2_position_txt;
            TextView country_principal2_name_txt;
            TextView country_principal2_mobile_txt;
        }
    }

//    private RequestCallBack<String> mStreamDealPersonRequestCallBack = new RequestCallBack<String>() {
//
//        private LogisticsTelPhoneNumberEntity LogisticsTelPhoneNumberEntity;
//
//        @Override
//        public void onStart() {
//            super.onStart();
//            ViewUtil.showProgress(TelLogisticsActivity.this, "", "请稍等.....");
//            Log.d(TAG, "请求的路径：" + this.getRequestUrl());
//        }
//
//        @Override
//        public void onFailure(HttpException arg0, String arg1) {
//            ViewUtil.dismissProgress();
//            showMsg("获取物流人员"
//                    + getString(R.string.request_data_fail));
//            /*ToastUtil.show(TelLogisticsActivity.this, "获取物流人员"
//                    + getString(R.string.request_data_fail));*/
//            GlobalApp.loger.info(TAG + "onFailure" + ":" + arg1);
//        }
//
//        @Override
//        public void onSuccess(ResponseInfo<String> arg0) {
//            ViewUtil.dismissProgress();
//            String json = arg0.result;
//
//            GlobalApp.loger.info(TAG + "onSuccess" + ":" + json);
//
//            Type listType = new TypeToken<LinkedList<LogisticsTelPhoneNumberEntity>>() {
//            }.getType();
//
//            Gson gson = new Gson();
//            LinkedList<LogisticsTelPhoneNumberEntity> resources = gson
//                    .fromJson(json, listType);
//            for (Iterator iterator = resources.iterator(); iterator.hasNext(); ) {
//                LogisticsTelPhoneNumberEntity = (LogisticsTelPhoneNumberEntity) iterator
//                        .next();
//                childrenlistItem.add(LogisticsTelPhoneNumberEntity);
//            }
//            mAdapter.notifyDataSetChanged();
//        }
//    };

}
