package com.urovo.activity.resetpassword.view;

import android.content.Intent;
import android.graphics.Color;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.view.View;
import android.widget.EditText;
import android.widget.TextView;

import com.alibaba.android.arouter.facade.annotation.Route;
import com.urovo.activity.resetpassword.presenter.ResetPassContract;
import com.urovo.activity.resetpassword.presenter.VerifyAccountPresenter;
import com.urovo.activity.resetpassword.web.CheckCodeResponse;
import com.urovo.wugumofang.R;
import com.urovo.wugumofang.base.BaseActivity;
import com.urovo.wugumofang.constants.AppConstants;
import com.urovo.wugumofang.constants.RouterParams;
import com.urovo.wugumofang.data.PreferHelper;
import com.urovo.wugumofang.response.BaseResponse;
import com.urovo.wugumofang.system.BaseAppManager;

import butterknife.BindView;
import butterknife.ButterKnife;
import butterknife.OnClick;
import butterknife.Unbinder;

/**
 *找回密码 之 验证账户
 * Created by Sasuke on 2016/6/14.
 */
@Route(path = RouterParams.ORIGIN.VERIFYACCOUNTACT)
public class VerifyAccountAct extends BaseActivity implements ResetPassContract.View,View.OnClickListener{
    VerifyAccountPresenter mPresenter;
    @BindView(R.id.user_id_input) EditText userIdInput;
    @BindView(R.id.telephone_input) EditText phoneInput;
    @BindView(R.id.verify_code_input) EditText codeInput;
    @BindView(R.id.get_verify_code) TextView getCodeBtn;
    final int SHOW_COUNT_DOWN = 0;
    Unbinder unbinder;
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.act_verify_account);
        unbinder = ButterKnife.bind(this);
        initTitle();
        super.titleTextView.setText("验证账户");

        mPresenter = new VerifyAccountPresenter(this);
        mPresenter.start();

    }

    @Override
    public void setPresenter(Object presenter) {
        this.mPresenter = (VerifyAccountPresenter) presenter;
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        mPresenter.timerCancel();
        uiHandler.removeCallbacksAndMessages(this);

//        ButterKnife.unbind(this);
        unbinder.unbind();
    }

    /**
     * 预填用户ID（懒癌福音）
     * @param userId 从上个界面（登陆界面）传过来的ID
     */
    public void setUserIdInput(String userId) {
        userIdInput.setText(userId);
    }

    public String getIntentPass() {
        return getIntent().getStringExtra("userId");
    }

    @OnClick({R.id.get_verify_code,R.id.btn_next_step})
    public void onClick(View v) {
        int id = v.getId();
        switch (id) {
            case R.id.get_verify_code://请求验证码（包含校验
                changeCountBtnState(false);
                mPresenter.requestCode(userIdInput.getText().toString().trim(), phoneInput.getText().toString().trim());//请求验证码并存入data
                break;
            case R.id.btn_next_step:
                mPresenter.checkCode(codeInput.getText().toString().trim(),
                                                      userIdInput.getText().toString().trim(),
                                                      phoneInput.getText().toString().trim());//校验验证码
                break;
        }
    }

    /**
     * 验证码倒计时
     * @param countDown
     */
    public void showCountDown(int countDown) {//在UI线程刷组件
        Message msg = new Message();
        msg.what = SHOW_COUNT_DOWN;
        Bundle bundle = new Bundle();
        bundle.putInt("countDown",countDown);
        msg.setData(bundle);
        uiHandler.sendMessage(msg);
    }

    /**
     * 获取验证码 按钮是否可点击
     * @param clickable
     */
    public void changeCountBtnState(boolean clickable) {
        getCodeBtn.setClickable(clickable);
        if(clickable){
            getCodeBtn.setText("获取验证码");
            getCodeBtn.setTextColor(Color.parseColor("#28b6f6"));//蓝
            getCodeBtn.setBackgroundResource(R.drawable.btn_blue_border_selector);
        }else{
            getCodeBtn.setTextColor(Color.parseColor("#aaa5a2"));//灰
            getCodeBtn.setBackgroundColor(Color.WHITE);
        }
    }

    @Override
    public void displayResult(String tag, BaseResponse response) {
        super.displayResult(tag, response);
        if(response == null){
            changeCountBtnState(true);
            showMsg("网络连接超时，请检查您的网络状态，稍后重试！");
            if(mPresenter.getTimer() != null){
                mPresenter.timerCancel();
                mPresenter.set10minStart(-1);
            }
            return ;
        }

        if(tag.equals("requestCode")){//获取验证码
            CheckCodeResponse res = (CheckCodeResponse)response;
            changeCountBtnState(true);
            if(res.getReturnBean() == null){
                mPresenter.timerCancel();
                showMsg(res.msgContent);
                mPresenter.set10minStart(-1);
                return;
            }
            BaseAppManager.loger().info("验证码为："+res.getReturnBean());
            mPresenter.startCountDown();//只有收到返回时，才开始计时
            mPresenter.set10minStart(System.currentTimeMillis());//把验证码存入data，期限"10分钟"的起始时间也存入prefs
        }if(tag.equals("checkCode")){//校验验证码
            CheckCodeResponse res = (CheckCodeResponse)response;
            if(!res.msgCode.equals("0")){
                showMsg(res.msgContent);//验证码错误或已失效
                return;
            }
            //CheckCodePrefHelper.getInstance().setStartTime(-1);
            //PreferHelper.getInstance().setLong(AppConstants.PreKeys.CHECK_CODE_START,-1);
            showMsg("验证成功，请修改密码");
            Intent intent = new Intent(this,ResetPasswordAct.class);
            intent.putExtra("passData",mPresenter.getData());
            startActivity(intent);
        }
    }


    /**
     * 主要用来刷新按钮UI
     */
    Handler uiHandler = new Handler(){
        @Override
        public void handleMessage(Message msg) {
            switch(msg.what){
                case SHOW_COUNT_DOWN://刷新倒计时显示的数字
                    int countDown = msg.getData().getInt("countDown");
                    if(countDown == 0){
                        changeCountBtnState(true);
                        //倒计时完毕然而并没有收到验证码
                        if(PreferHelper.getInstance().getLong(AppConstants.PreKeys.CHECK_CODE_START ) == -1){
                            showMsg("网络超时，请重试");
                        }
                        break;
                    }
                    getCodeBtn.setText(countDown+"秒后重新获取");
                    break;
            }
        }
    };

}
