package com.urovo.activity.salesdetailnet.view

import android.graphics.drawable.BitmapDrawable
import android.os.Bundle
import android.view.Gravity
import android.view.ViewGroup
import android.widget.Button
import android.widget.LinearLayout
import android.widget.TextView
import androidx.core.content.ContextCompat
import androidx.databinding.DataBindingUtil
import androidx.lifecycle.ViewModelProviders
import com.google.android.material.tabs.TabLayout
import com.urovo.activity.salesdetailnet.model.SalesDetailFactory
import com.urovo.activity.salesdetailnet.model.SalesDetailModel
import com.urovo.activity.salesdetailnet.repo.SalesDetailRepo
import com.urovo.wgmf.adapter.SaleDetailAdapter3
import com.urovo.wgmf.adapter.SaleDetailAdapter4
import com.urovo.wgmf.db.dao.entitys.ProductsreturnList
import com.urovo.wgmf.db.dao.entitys.SaleProductList
import com.urovo.wugumofang.R
import com.urovo.wugumofang.base.BaseActivity
import com.urovo.wugumofang.data.DateUtil
import com.urovo.wugumofang.databinding.QuerySalesDetailNetActivityBinding
import com.urovo.wugumofang.system.SystemHelper
import com.urovo.wugumofang.view.CustomPopupWindow
import com.urovo.wugumofang.view.LoadWaitDominator
import kotlinx.android.synthetic.main.query_sales_detail_net_activity.*
import org.jetbrains.anko.dip
import java.util.*

/**
 * @author Sasuke on 2019-9-17 0017.
 * 销售明细查询
 */
class QuerySalesDetailNet : BaseActivity(), SaleDetailAdapter3.OnItemViewListener {

    private var parentslist: List<ProductsreturnList>? = ArrayList()
    private var childrenlist: List<SaleProductList>? = ArrayList()
    private var groupPosition = 0

    override fun onViewListener(groupPosition: Int, consId: String) {
        this.groupPosition = groupPosition
        if (1 == saleModel.queryType.get()!!) {
            saleModel.queryDetail(consId)
        }
    }

    val binding by lazy {
        DataBindingUtil.setContentView<QuerySalesDetailNetActivityBinding>(
            this,
            R.layout.query_sales_detail_net_activity
        )
    }

    val saleModel by lazy {
        ViewModelProviders.of(this, SalesDetailFactory(SalesDetailRepo()))
            .get(SalesDetailModel::class.java)
    }

    var loadWait: LoadWaitDominator? = null
    val selectDatePop by lazy {
        CustomPopupWindow.Builder(this)
            .setContentView(R.layout.select_date_pop_layout)
            .setFouse(true)
            .setOutSideCancel(true)
            .setAnimationStyle(R.style.dropmenu_anim_style)
            .setBackgroundDrawable(BitmapDrawable())
            .setwidth(ViewGroup.LayoutParams.MATCH_PARENT)
            .setheight(ViewGroup.LayoutParams.WRAP_CONTENT)
            .builder()
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        bind()
        initTitle()
        init()
        start()
    }

    fun init() {
        //展开某个分组时，并关闭其他分组
        binding.exListView.setOnGroupExpandListener {
            for (groupPosition in parentslist!!.indices) {
                if (groupPosition != it) {
                    binding.exListView.collapseGroup(groupPosition)
                }
            }
        }
    }

    fun bind() {
        binding.model = saleModel
        binding.titleText = "销售明细查询"
        binding.storeName = SystemHelper.LoginOrganName
        binding.titleLayout.setOnClickListener {
            showSelectPop()
        }

        saleModel.loadWaitType.observe(this, androidx.lifecycle.Observer {
            loadWait?.show(it)
        })

        //刷新列表数据
        saleModel.saleDataList.observe(this, androidx.lifecycle.Observer {
            if (it.isNullOrEmpty()) {
                loadWait?.show(LoadWaitDominator.NO_DATA)
                return@Observer
            }
            parentslist = it
            groupPosition = 0 //父级数据刷新，将选中位置清零
            if (SalesDetailModel.TYPE_STORE_SALE == saleModel.queryType.get()!!) {
                binding.exListView.setAdapter(
                    SaleDetailAdapter3(
                        saleModel.queryType.get()!!,
                        this,
                        parentslist,
                        childrenlist,
                        this,
                        true,
                        false
                    )
                )
                binding.exListView.expandGroup(0)
                binding.exListView.setGroupIndicator(null)
                //获取到父级数据，请求第一个字数据展开
                saleModel.queryDetail(parentslist!!.get(0).consId)
            } else {
                binding.exListView.setAdapter(
                    SaleDetailAdapter4(
                        saleModel.queryType.get()!!,
                        parentslist,
                        this,
                        true,
                        false
                    )
                )
                binding.exListView.expandGroup(0)
                binding.exListView.setGroupIndicator(null)
            }
        })
        //刷新列表详情数据
        saleModel.saleDataDetailList.observe(this, androidx.lifecycle.Observer {
            childrenlist = it
            if (SalesDetailModel.TYPE_STORE_SALE == saleModel.queryType.get()!!) {
                binding.exListView.setAdapter(
                    SaleDetailAdapter3(
                        saleModel.queryType.get()!!,
                        this,
                        parentslist,
                        childrenlist,
                        this,
                        true,
                        false
                    )
                )
                binding.exListView.expandGroup(groupPosition)
                binding.exListView.setSelectedGroup(groupPosition)
//                binding.exListView.collapseGroup(groupPosition)
                binding.exListView.setGroupIndicator(null)
            } else {
                binding.exListView.setAdapter(
                    SaleDetailAdapter4(
                        saleModel.queryType.get()!!,
                        parentslist,
                        this,
                        true,
                        false
                    )
                )
                binding.exListView.expandGroup(groupPosition)
//                binding.exListView.collapseGroup(groupPosition)
                binding.exListView.setGroupIndicator(null)
            }
        })
    }

    fun start() {
        loadWait = LoadWaitDominator(this, binding.dataContainer)
        //初始化Tab
        val tabLayout = typeTabLayout.apply {
            addTab(newTab())
            addTab(newTab())
            getTabAt(0)?.select()
            getTabAt(0)?.text = "门店销售"
            getTabAt(1)?.text = "个人会员商品"
        }
        //给TabLayout添加分割线
        (tabLayout.getChildAt(0) as LinearLayout).run {
            showDividers = LinearLayout.SHOW_DIVIDER_MIDDLE
            dividerDrawable = ContextCompat.getDrawable(
                this@QuerySalesDetailNet,
                R.drawable.layout_divider_vertical2
            )
            dividerPadding = dip(12)
        }
        //挂tabLayout监听
        tabLayout.addOnTabSelectedListener(object : TabLayout.OnTabSelectedListener {
            override fun onTabReselected(tab: TabLayout.Tab?) {
            }

            override fun onTabUnselected(tab: TabLayout.Tab?) {//点击时先执行我

            }

            override fun onTabSelected(tab: TabLayout.Tab?) {// TabUnselected（）执行后，我才执行
                saleModel.queryType.set(if (tab?.position == 0) SalesDetailModel.TYPE_STORE_SALE else SalesDetailModel.TYPE_MEMBER_SALE)
                saleModel.query(
                    SystemHelper.LoginUserId,
                    SystemHelper.LoginOrganId,
                    saleModel.startDate.get()!!,
                    saleModel.endDate.get()!!
                )
            }
        })
        //开始查询
        saleModel.query(
            SystemHelper.LoginUserId,
            SystemHelper.LoginOrganId,
            saleModel.startDate.get()!!,
            saleModel.endDate.get()!!
        )
    }

    //弹出日期选择弹窗
    fun showSelectPop() {
        selectDatePop?.showAsLocation(titleTextView, Gravity.BOTTOM, 0, 0)
        (selectDatePop?.getItemView(R.id.popStoreNameTv) as TextView).text =
            SystemHelper.LoginOrganName
        //起始日期
        (selectDatePop?.getItemView(R.id.popStartDateTv) as TextView).apply {
            text = saleModel.popStartDate.value
            setOnClickListener {
                DateUtil.pickDialog(this@QuerySalesDetailNet, this, "yyyy-MM-dd", null, null) {
                    saleModel.popStartDate.value = this.text.toString()
                }.show()
            }
        }
        //终止日期
        (selectDatePop?.getItemView(R.id.popEndDateTv) as TextView).apply {
            text = saleModel.popEndDate.value
            setOnClickListener {
                DateUtil.pickDialog(this@QuerySalesDetailNet, this, "yyyy-MM-dd",
                    null, DateUtil.getDayAfterHours(Date(), -24), {
                        saleModel.popEndDate.value = this.text.toString()
                    }).show()
            }
        }
        //查询按钮
        (selectDatePop?.getItemView(R.id.popQueryBtn) as Button).setOnClickListener {
            if (saleModel.popStartDate.value!!.compareTo(saleModel.popEndDate.value!!) > 0) {
                showMsg("开始日期不能大于结束日期！")
                return@setOnClickListener
            }
            if (DateUtil.diffDate(
                    "yyyy-MM-dd",
                    saleModel.popEndDate.value!!,
                    saleModel.popStartDate.value!!
                ) > 31
            ) {
                showMsg("日期相隔不能超过31天！")
                return@setOnClickListener
            }
            saleModel.run {
                startDate.set(popStartDate.value)
                endDate.set(popEndDate.value)
                saleModel.query(
                    SystemHelper.LoginUserId,
                    SystemHelper.LoginOrganId,
                    saleModel.startDate.get()!!,
                    saleModel.endDate.get()!!
                )
            }
            selectDatePop?.dismiss()
        }
    }
}