package com.urovo.activity.salesmanager;

import android.content.Intent;
import android.os.Bundle;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.ImageView;
import android.widget.TextView;

import com.urovo.wgmf.AsyncTask.QueryDayTaskStatusTask;
import com.urovo.wugumofang.base.BaseActivity;
import com.urovo.wugumofang.system.SystemHelper;
import com.urovo.wugumofang.response.BaseOldResponse;
import com.urovo.wugumofang.response.BaseResponse;
import com.urovo.wgmf.db.dao.entitys.CalendarBaseModelResponse;
import com.urovo.wgmf.db.dao.entitys.DayTaskStatusEntity;
import com.urovo.wgmf.db.dao.entitys.DayTaskStatusEntity.DayTaskStatusReturnList;
import com.urovo.wgmf.view.DayTaskCalendar;
import com.urovo.wgmf.view.DayTaskCalendar.OnDaySelectListener;
import com.urovo.wgmf.view.DayTaskCalendar.calendarGridViewAdapter;
import com.urovo.wugumofang.R;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @ClassName: DayTaskActivity
 * @Description: 日目标管理
 * @author XP
 * @date 2015-11-16 下午4:02:51
 */
public class DayTaskActivity extends BaseActivity implements OnClickListener {
	private TextView titleMonth;
	private ImageView titleLeft;
	private ImageView titleRight;
	private String choiceMonth;
	private SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM");// 日期格式化
	private DayTaskCalendar dayTaskCalendar;
	private calendarGridViewAdapter gridViewAdapter;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.store_manager_day_task);
		init();
		initTitle();
		titleTextView.setText("日目标管理");
		FindStatusTask();
	}

	/**
	 * 
	 * @Title: FindStatusTask
	 * @Description: 查询日任务状态Task
	 * @return void 返回类型
	 * @throws
	 */
	private void FindStatusTask() {
		Map<String, String> params = new HashMap<String, String>();
		params.put("code1", SystemHelper.LoginOrganId);// 机构编码
		params.put("yearMonth", choiceMonth);// 查询月份
		new QueryDayTaskStatusTask(this, "正在查询...", false, params).execute("");
	}

	private void init() {
		choiceMonth = DayTaskCalendar.getChoiceMonth();
		dayTaskCalendar = findViewById(R.id.day_task_calendar);
		dayTaskCalendar.setOnDaySelectListener(new OnDaySelectListener() {

			@Override
			public void onDaySelectListener(String date) {
				Intent intent = new Intent(DayTaskActivity.this, DayTaskSetupActivity.class);
				intent.putExtra("Data", date);
				startActivity(intent);
			}
		});

		titleMonth = findViewById(R.id.title_month);
		titleMonth.setText(choiceMonth);
		titleLeft = findViewById(R.id.month_row_left);
		titleRight = findViewById(R.id.month_row_right);
		titleLeft.setVisibility(View.INVISIBLE);
		titleRight.setVisibility(View.VISIBLE);
		titleLeft.setOnClickListener(this);
		titleRight.setOnClickListener(this);
	}

	@Override
	protected void onRestart() {
		super.onRestart();
		FindStatusTask();
	}

	@Override
	public void displayResult(String tag, BaseResponse response) {
		super.displayResult(tag, response);
		if(!((BaseOldResponse)response).msgCode.equals("0")) {
			return;
		}
		else
			if(null != response) {
				DayTaskStatusEntity dayTaskStatusEntity = (DayTaskStatusEntity) response;
				List<DayTaskStatusReturnList> entitys = dayTaskStatusEntity.resultList;
				List<CalendarBaseModelResponse> mStatusList = new ArrayList<CalendarBaseModelResponse>();
				mStatusList.clear();
				if(null != entitys) {
					if(null != entitys && entitys.size() > 0) {
						for (int i = 0; i < entitys.size(); i++) {
							CalendarBaseModelResponse cell = new CalendarBaseModelResponse();
							cell.dayShow = entitys.get(i).dayShow;// 日期
							cell.count = entitys.get(i).count;// 该日期已设置了几家
							cell.total = entitys.get(i).total;// 该工号的管理者（主管）下辖的门店总数
							mStatusList.add(cell);
						}
					}
					dayTaskCalendar.setDaysHasThingList(mStatusList);// 填充事务
				}
			}
	}

	@Override
	public void onClick(View v) {
		Calendar ca = Calendar.getInstance();// 得到一个Calendar的实例
		ca.setTime(new Date()); // 设置时间为当前时间
		Date resultDate = ca.getTime(); // 结果
		choiceMonth = dateFormat.format(resultDate);
		switch (v.getId()) {
		case R.id.month_row_left:
			titleMonth.setText(choiceMonth);
			titleLeft.setVisibility(View.INVISIBLE);
			titleRight.setVisibility(View.VISIBLE);
			SetChangedCanlanderData(ca, choiceMonth);
			break;
		case R.id.month_row_right:
			ca.add(Calendar.MONTH, 1);// 月份加1
			resultDate = ca.getTime(); // 结果
			choiceMonth = dateFormat.format(resultDate);
			titleMonth.setText(choiceMonth);
			titleLeft.setVisibility(View.VISIBLE);
			titleRight.setVisibility(View.INVISIBLE);
			SetChangedCanlanderData(ca, choiceMonth);
			break;
		default:
			break;
		}
	}

	/**
	 * 
	 * @Title: SetChangedCanlanderData
	 * @Description: 月份改变时刷新日历状态数据
	 * @param @param ca
	 * @param @param num 参数说明
	 * @return void 返回类型
	 * @throws
	 */
	private void SetChangedCanlanderData(Calendar ca, String month) {
		try {
			ca.setTime(dateFormat.parse(month));
		}
		catch (ParseException e) {

		}
		ca.set(Calendar.DATE, 1);
		DayTaskCalendar.setChoiceMonth(month);
		int tempSum = DayTaskCalendar.countNeedHowMuchEmpety(ca);// 获取当前月第一天为星期几
		int dayNumInMonth = DayTaskCalendar.getDayNumInMonth(ca);// 获取当前月有多少天
		DayTaskCalendar.setGvListData(tempSum, dayNumInMonth);
		dayTaskCalendar.gridViewAdapter.notifyDataSetChanged();
		FindStatusTask();
	}
}
