package com.urovo.activity.salesmanager;

import android.content.Context;
import android.os.Bundle;
import android.text.TextUtils;
import android.view.LayoutInflater;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.View.OnFocusChangeListener;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.TextView;

import com.google.gson.Gson;
import com.urovo.wgmf.AsyncTask.BaseAppServerTask;
import com.urovo.wgmf.AsyncTask.QueryDayTaskListTask;
import com.urovo.wugumofang.base.BaseActivity;
import com.urovo.wugumofang.system.SystemHelper;
import com.urovo.wgmf.adapter.MyBaseAdapter;
import com.urovo.wugumofang.response.BaseOldResponse;
import com.urovo.wugumofang.response.BaseResponse;
import com.urovo.wgmf.db.dao.entitys.DayTaskListEntity;
import com.urovo.wgmf.db.dao.entitys.DayTaskListEntity.DayTaskListReturnList;
import com.urovo.wugumofang.data.DataMonthUtils;
import com.urovo.wugumofang.R;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @ClassName: DayTaskSetupActivity
 * @Description: 日目标设置
 * @author XP
 * @date 2015-11-17 下午3:21:40
 */
public class DayTaskSetupActivity extends BaseActivity implements
		OnClickListener {

	private static String checkdata;
	private TextView daytaskdate;
	private ListView listView;
	private MyAdapter adapter;
	private LinearLayout mLinearLayoutEdit, mLinearLayoutEditTwo;
	private Button btnEdit, btnEditCancel, btnEditConfirm;
	private List<BaseModel> mBaseModel = new ArrayList<BaseModel>();
	private List<BaseModel> mListUser = new ArrayList<BaseModel>();

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.day_task_setup);
		init();
		initTitle();
		initLisener();
		titleTextView.setText("日任务管理");
	}

	protected BaseAdapter newBaseAdapter() {
		return new MyAdapter(mBaseModel, this);
	}

	private void init() {
		// 从Intent对象中extras中获取Data
		Bundle extras = getIntent().getExtras();
		checkdata = (String) extras.get("Data");
		daytaskdate = (TextView) findViewById(R.id.day_task_date);
		daytaskdate.setText(checkdata);
		// 添加固定在底部的按钮
		mLinearLayoutEdit = (LinearLayout) findViewById(R.id.ll_edit);
		mLinearLayoutEditTwo = (LinearLayout) findViewById(R.id.ll_edit_twobtn);
		btnEdit = (Button) findViewById(R.id.btn_edit);
		btnEditCancel = (Button) findViewById(R.id.btn_edit_cancel);
		btnEditConfirm = (Button) findViewById(R.id.btn_edit_confirm);
		listView = (ListView) findViewById(R.id.listview);
		adapter = new MyAdapter(mBaseModel, this);
		listView.setAdapter(adapter);
		query();
	}

	private void query() {
		Map<String, String> params = new HashMap<String, String>();
		params.put("day", checkdata);// 目标天日期
		params.put("code1", SystemHelper.LoginOrganId);// 机构编码
		new QueryDayTaskListTask(this, "正在查询...", true, params).execute("");
	}

	private void initLisener() {
		btnEdit.setOnClickListener(this);// 编辑按钮
		btnEditCancel.setOnClickListener(this);// 二次编辑取消按钮
		btnEditConfirm.setOnClickListener(this);// 二次编辑完成按钮
	}

	@Override
	public void displayResult(String tag, BaseResponse response) {
		super.displayResult(tag, response);
		if ("dayTasksubmit".equals(tag) && null != response) {
			if ("0".equals(response.sysCode)) {
				finish();
				showMsg("上传日任务成功！");
				//Toast.makeText(this, "上传日任务成功！", 1000).show();
			}
		} else if (!"0".equals(((BaseOldResponse)response).msgCode)) {
			return;
		} else if (null != response) {
			DayTaskListEntity dayTaskListEntity = (DayTaskListEntity) response;
			List<DayTaskListReturnList> entitys = dayTaskListEntity.resultList;
			mBaseModel.clear();
			if (null != entitys && entitys.size() > 0) {
				Date date = new Date();
				SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd");
				try {
					Date date1 = formatter.parse(checkdata);// 选中的日期
					Date date2 = formatter.parse(formatter.format(date));// 今天
					if ((DataMonthUtils.getCompareDate(date1, date2) < 0)) {
						// 只能编辑（设置/修改）今天以后的日目标，今天以前的（包括今天），只可查看，没有编辑按钮
						mLinearLayoutEdit.setVisibility(View.VISIBLE);
						mLinearLayoutEditTwo.setVisibility(View.GONE);
					}
				} catch (ParseException e) {

				}

				for (int j = 0; j < entitys.size(); j++) {
					BaseModel cell = new BaseModel();
					cell.shop_id = isNull(entitys.get(j).shop_id);
					cell.shop_name = isNull(entitys.get(j).shop_name);
					cell.day_sale_task = isNull(entitys.get(j).day_sale_task);
					cell.day_deal_task = isNull(entitys.get(j).day_deal_task);
					cell.day_taste_task = isNull(entitys.get(j).day_taste_task);
					cell.day_we_chat_task = isNull(entitys.get(j).day_we_chat_task);
					setStatus(cell);
					mBaseModel.add(cell);
				}
			}
		}
		SortStatus();
		listviewChange();
		adapter.notifyDataSetChanged();
	}

	/**
	 * 
	 * @Title: setStatus
	 * @Description: 添加状态字段
	 * @param @param cell 参数说明
	 * @return void 返回类型
	 * @throws
	 */
	private void setStatus(BaseModel cell) {
		if ((cell.day_sale_task.equals("")) && (cell.day_deal_task.equals(""))
				&& (cell.day_taste_task.equals(""))
				&& (cell.day_we_chat_task.equals(""))) {
			cell.mStatus = 1;
		} else if ((cell.day_sale_task.equals(""))
				|| cell.day_deal_task.equals("")
				|| cell.day_taste_task.equals("")
				|| cell.day_we_chat_task.equals("")) {
			cell.mStatus = 2;// 部分设置
		} else {
			cell.mStatus = 3;// 已设置
		}
	}

	/**
	 * 
	 * @Title: SortStatus
	 * @Description: 根据日目标设置状态排序
	 * @param @param lists 参数说明
	 * @return void 返回类型
	 * @throws
	 */
	public void SortStatus() {
		Collections.sort(mBaseModel, new Comparator<BaseModel>() {

			@Override
			public int compare(BaseModel lhs, BaseModel rhs) {
				if (lhs.mStatus > rhs.mStatus) {
					return 1;
				} else if (lhs.mStatus == rhs.mStatus) {
					return 0;
				} else {
					return -1;
				}
			}
		});
	}

	private String isNull(String str) {
		return ("null".equals(str) || null == str) ? "" : str;
	}

	private class BaseModel {
		boolean isInvalid;
		int mStatus;// 日目标设置状态
		String shop_id;// 门店的编号
		String shop_name;// 门店的名字
		String day;// 设置的那一天的日期
		String emp_no;// 工号
		String day_sale_task;// 日销售任务
		String day_deal_task;// 日客单数任务
		String day_taste_task;// 日试吃任务
		String day_we_chat_task;// 日微会员任务
	}

	private class MyAdapter extends MyBaseAdapter<BaseModel, ListView> {
		public MyAdapter(List<BaseModel> lists, Context context) {
			super(lists, context);
		}

		@Override
		public View getView(final int position, View convertView,
				ViewGroup parent) {
			final ViewHolder viewHolder;
			if (convertView == null) {
				convertView = LayoutInflater.from(context).inflate(
						R.layout.day_task_setup_item, null);
				viewHolder = new ViewHolder();
				viewHolder.store_name = (TextView) convertView
						.findViewById(R.id.store_name);
				viewHolder.et_day_sale_target = (EditText) convertView
						.findViewById(R.id.et_day_sale_target);
				viewHolder.et_day_guest_target = (EditText) convertView
						.findViewById(R.id.et_day_guest_target);
				viewHolder.et_day_free_target = (EditText) convertView
						.findViewById(R.id.et_day_free_target);
				viewHolder.et_day_wvip_target = (EditText) convertView
						.findViewById(R.id.et_day_wvip_target);
				viewHolder.status = (TextView) convertView
						.findViewById(R.id.task_status);
				convertView.setTag(viewHolder);
			} else {
				viewHolder = (ViewHolder) convertView.getTag();
			}
			final BaseModel cell = mBaseModel.get(position);

			SetText(viewHolder, cell);// 给控件设置内容

			viewHolder.et_day_sale_target.setTag(position);
			viewHolder.et_day_guest_target.setTag(position);
			viewHolder.et_day_free_target.setTag(position);
			viewHolder.et_day_wvip_target.setTag(position);

			setViewHolderStatus(viewHolder.et_day_sale_target);
			setViewHolderStatus(viewHolder.et_day_guest_target);
			setViewHolderStatus(viewHolder.et_day_free_target);
			setViewHolderStatus(viewHolder.et_day_wvip_target);

			SetOnClick(viewHolder);
			SetOnFocusChange(position, viewHolder);
			SortStatus();
			return convertView;
		}

		private void setViewHolderStatus(final EditText editText) {
			if (backFlag) {
				editText.setEnabled(true);
				editText.setSelectAllOnFocus(true);
				editText.setBackgroundResource(R.drawable.btn_manager_style);
				editText.setTextColor(getResources().getColor(R.color.black));
			} else {
				editText.setEnabled(false);
				editText.setSelectAllOnFocus(false);
				editText.setBackgroundResource(R.color.white);
				editText.setTextColor(getResources().getColor(R.color.black));
			}
		}

		/**
		 * 
		 * @Title: SetOnFocusChange
		 * @Description: 设置edittext焦点改变事件
		 * @param @param position
		 * @param @param viewHolder 参数说明
		 * @return void 返回类型
		 * @throws
		 */
		private void SetOnFocusChange(final int position,
				final ViewHolder viewHolder) {
			viewHolder.et_day_sale_target
					.setOnFocusChangeListener(new OnFocusChangeListener() {

						@Override
						public void onFocusChange(View v, boolean hasFocus) {

							BaseModel cell = lists.get(position);
							cell.day_sale_task = viewHolder.et_day_sale_target
									.getText().toString();
						}
					});
			viewHolder.et_day_guest_target
					.setOnFocusChangeListener(new OnFocusChangeListener() {

						@Override
						public void onFocusChange(View v, boolean hasFocus) {
							BaseModel cell = lists.get(position);
							cell.day_deal_task = viewHolder.et_day_guest_target
									.getText().toString();
						}
					});
			viewHolder.et_day_free_target
					.setOnFocusChangeListener(new OnFocusChangeListener() {

						@Override
						public void onFocusChange(View v, boolean hasFocus) {
							BaseModel cell = lists.get(position);
							cell.day_taste_task = viewHolder.et_day_free_target
									.getText().toString();
						}
					});
			viewHolder.et_day_wvip_target
					.setOnFocusChangeListener(new OnFocusChangeListener() {

						@Override
						public void onFocusChange(View v, boolean hasFocus) {
							BaseModel cell = lists.get(position);
							cell.day_we_chat_task = viewHolder.et_day_wvip_target
									.getText().toString();
						}
					});
		}

		/**
		 * 
		 * @Title: SetOnClick
		 * @Description: 设置edit点击事件
		 * @param @param viewHolder 参数说明
		 * @return void 返回类型
		 * @throws
		 */
		private void SetOnClick(final ViewHolder viewHolder) {
			viewHolder.et_day_sale_target
					.setOnClickListener(new OnClickListener() {
						@Override
						public void onClick(View view) {
							viewHolder.et_day_sale_target
									.setText(viewHolder.et_day_sale_target
											.getText().toString());
							viewHolder.et_day_sale_target.selectAll();
						}
					});
			viewHolder.et_day_guest_target
					.setOnClickListener(new OnClickListener() {
						@Override
						public void onClick(View view) {
							viewHolder.et_day_guest_target
									.setText(viewHolder.et_day_guest_target
											.getText().toString());
							viewHolder.et_day_guest_target.selectAll();
						}
					});
			viewHolder.et_day_free_target
					.setOnClickListener(new OnClickListener() {
						@Override
						public void onClick(View view) {
							viewHolder.et_day_free_target
									.setText(viewHolder.et_day_free_target
											.getText().toString());
							viewHolder.et_day_free_target.selectAll();
						}
					});
			viewHolder.et_day_wvip_target
					.setOnClickListener(new OnClickListener() {
						@Override
						public void onClick(View view) {
							viewHolder.et_day_wvip_target
									.setText(viewHolder.et_day_wvip_target
											.getText().toString());
							viewHolder.et_day_wvip_target.selectAll();
						}
					});
		}

		class ViewHolder {
			TextView store_name;
			EditText et_day_sale_target;
			EditText et_day_guest_target;
			EditText et_day_free_target;
			EditText et_day_wvip_target;
			TextView status;
		}

		/**
		 * 
		 * @Title: SetText
		 * @Description: 文字设置
		 * @param @param viewHolder
		 * @param @param cell 参数说明
		 * @return void 返回类型
		 * @throws
		 */
		private void SetText(final ViewHolder viewHolder, final BaseModel cell) {
			viewHolder.store_name.setText(cell.shop_name);
			viewHolder.et_day_sale_target.setText(cell.day_sale_task);
			viewHolder.et_day_guest_target.setText(cell.day_deal_task);
			viewHolder.et_day_free_target.setText(cell.day_taste_task);
			viewHolder.et_day_wvip_target.setText(cell.day_we_chat_task);
			if (cell.mStatus == 1) {
				cell.isInvalid = true;
				viewHolder.status.setTextColor(getResources().getColor(
						R.color.manager_red));
			} else if (cell.mStatus == 2) {
				cell.isInvalid = true;
				viewHolder.status.setTextColor(getResources().getColor(
						R.color.manager_yellow));
			} else if (cell.mStatus == 3) {
				cell.isInvalid = false;
				viewHolder.status.setTextColor(getResources().getColor(
						R.color.manager_green));
			}
			listviewChange();
			String mState = switchStatus(cell);
			viewHolder.status.setText(mState);
		}

		private String switchStatus(final BaseModel cell) {
			String mState = "（未设置）";
			switch (cell.mStatus) {
			case 1:
				mState = "（未设置）";
				break;
			case 2:
				mState = "（部分设置）";
				break;
			case 3:
				mState = "（已设置）";
				break;

			default:
				break;
			}
			return mState;
		}
	}

	/**
	 * 
	 * @Title: listviewChange
	 * @Description: 提交按钮变化
	 * @return void 返回类型
	 * @throws
	 */
	public void listviewChange() {
		int invalidCount = 0;
		for (BaseModel p : mBaseModel) {
			if (p.isInvalid)
				invalidCount++;
		}
		// if (invalidCount > 0) {
		// btnEditConfirm.setEnabled(false);
		// btnEditConfirm.setText("请将信息填写完整再上传！");
		// } else {
		// if (!btnEditConfirm.isEnabled()) {
		// btnEditConfirm.setEnabled(true);
		// btnEditConfirm.setText("完成");
		// }
		// }
	}

	@Override
	public void onClick(View v) {
		switch (v.getId()) {
		case R.id.btn_edit:
			if (!backFlag) {
				mLinearLayoutEdit.setVisibility(View.GONE);
				mLinearLayoutEditTwo.setVisibility(View.VISIBLE);
			} else {
				mLinearLayoutEdit.setVisibility(View.VISIBLE);
				mLinearLayoutEditTwo.setVisibility(View.GONE);
			}
			backFlag = !backFlag;
			break;
		case R.id.btn_edit_cancel:
			backFlag = false;
			mLinearLayoutEdit.setVisibility(View.VISIBLE);
			mLinearLayoutEditTwo.setVisibility(View.GONE);
			break;
		case R.id.btn_edit_confirm:
			backFlag = false;
			mLinearLayoutEdit.setVisibility(View.VISIBLE);
			mLinearLayoutEditTwo.setVisibility(View.GONE);
			commitDayTask();
			break;
		default:
			break;
		}
		listviewChange();
		adapter.notifyDataSetChanged();
	}

	/**
	 * 
	 * @Title: commitDayTask
	 * @Description: 提交上传日目标
	 * @return void 返回类型
	 * @throws
	 */
	private void commitDayTask() {
		mListUser.clear();
		for (BaseModel cell : mBaseModel) {

			BaseModel t = new BaseModel();
			t.shop_id = TextUtils.isEmpty(cell.shop_id) ? "" : cell.shop_id;// 门店ID
			t.day = checkdata;// 设置的那一天的日期
			t.emp_no = SystemHelper.LoginUserId;// 工号
			t.day_sale_task = TextUtils.isEmpty(cell.day_sale_task) ? ""
					: cell.day_sale_task;// 日销售任务
			t.day_deal_task = TextUtils.isEmpty(cell.day_deal_task) ? ""
					: cell.day_deal_task;// 日客单数任务
			t.day_taste_task = TextUtils.isEmpty(cell.day_taste_task) ? ""
					: cell.day_taste_task;// 日试吃任务
			t.day_we_chat_task = TextUtils.isEmpty(cell.day_we_chat_task) ? ""
					: cell.day_we_chat_task; // 日微会员任务
			mListUser.add(t);
		}
		Map<String, String> params = new HashMap<String, String>();
		params.put("json", new Gson().toJson(mListUser));
		new BaseAppServerTask("dayTasksubmit", BaseOldResponse.class,
				DayTaskSetupActivity.this, "正在上传日任务数据...", true,
				"/dayTask/save", params).execute("");
	}
}
