package com.urovo.activity.slidemenu.presenter

import com.urovo.activity.slidemenu.presenter.contract.UploadShoppingReceiptContract
import com.urovo.activity.slidemenu.response.UploadShoppingReceiptResponse
import com.urovo.wugumofang.R
import com.urovo.wugumofang.base.mvp.AbsPresenter
import com.urovo.wugumofang.constants.AppConstants
import com.urovo.wugumofang.net.ApiException
import com.urovo.wugumofang.net.callback.RetrofitCallback
import com.urovo.wugumofang.system.SystemHelper
import com.urovo.wugumofang.utils.isEmptyParameter
import com.urovo.wugumofang.utils.isNullOrEmpty
import com.zsy.libcore.model.bean.RequestResult
import retrofit2.Call
import retrofit2.Response

/**
 *  上传购物小票Presenter
 * @author LW
 */
class UploadShoppingReceiptPresenter(
    private var view: UploadShoppingReceiptContract.IView?,
    private var depository: UploadShoppingReceiptResponse = UploadShoppingReceiptResponse(view)
) : AbsPresenter(), UploadShoppingReceiptContract.IPresenter {

    /**
     * 购物小票图片
     */
    var mCertificateUrl: String = ""

    override fun requestSubmitPhotosToQinIu(photos: String?) {
        if (photos.isNullOrEmpty()) {
            view?.showToast(R.string.buy_benefit_card_picture_entry)
            return
        }
        depository.requestSubmitPhotosToQinIu(mutableListOf(photos!!))
    }

    override fun requestUploadCertificate(mobile: String?) {

        val staffId = SystemHelper.LoginUserId
        val shopId = SystemHelper.LoginOrganId

        if (isEmptyParameter(shopId, mobile, staffId)) {
            view?.showToast(R.string.common_load_error)
            return
        }

        view?.showTipDialog(R.string.loading)

        depository.requestUploadCertificate(shopId, mobile!!, staffId, mCertificateUrl)
            .enqueue(object : RetrofitCallback<RequestResult<Any>>() {

                override fun onResponse(
                    call: Call<RequestResult<Any>>,
                    res: Response<RequestResult<Any>>
                ) {
                    val body = res.body()
                    body?.let {
                        if (it.msgCode == AppConstants.Http.RESPONSE_SUCCESS) {
                            view?.submitionStatus(true)
                        } else {
                            view?.submitionStatus(false)
                            view?.showToast(it.msgInfo)
                        }
                    }
                    view?.dismissDialog()
                }

                override fun onFailure(exception: ApiException) {
                    view?.submitionStatus(false)
                    view?.showToast(exception.message)
                    view?.dismissDialog()
                }
            })

    }
}