package com.urovo.activity.slidemenu.ui

import android.content.Intent
import android.view.View
import com.urovo.activity.slidemenu.presenter.UploadShoppingReceiptPresenter
import com.urovo.activity.slidemenu.presenter.contract.UploadShoppingReceiptContract
import com.urovo.wugumofang.R
import com.urovo.wugumofang.base.BaseActivity
import com.urovo.wugumofang.base.BasePresenter
import com.urovo.wugumofang.constants.AppConstants
import com.urovo.wugumofang.ui.PhotoPickDominator
import com.urovo.wugumofang.utils.isNullOrEmpty
import com.urovo.wugumofang.utils.notNullAndEmpty
import com.wgmf.trade.utils.ImageLoader
import kotlinx.android.synthetic.main.act_upload_shopping_receipt.*
import java.io.File

/**
 * 上传购物小票
 * @author LW
 */
class UploadShoppingReceiptActivity : BaseActivity(), UploadShoppingReceiptContract.IView {

    private var mUploadShoppingReceiptPresenter: UploadShoppingReceiptPresenter? = null

    private var mPicker: PhotoPickDominator? = null

    private val mPhone: String by lazy {
        intent.getStringExtra(AppConstants.ExtraKeys.PHONE_NUM)
    }

    override fun addRootLayoutId(): Int = R.layout.act_upload_shopping_receipt

    private var mSelectPhotos: String = ""

    override fun initTitle() {
        super.initTitle()
        titleTextView.text = getString(R.string.upload_shopping_receipt_title)

        complete.apply {
            visibility = View.VISIBLE
            text = getString(R.string.upload_btn_txt)
            setBackgroundResource(R.drawable.shape_select_index_btn)
            setOnClickListener {
                //上传权益卡小票
                uploadCertificate()
            }
        }
    }

    override fun initViewNew() {
        initTitle()

        mPicker = PhotoPickDominator(this)

        mUploadPictureIv.setOnClickListener {
            if (mSelectPhotos.notNullAndEmpty()) {
                mPicker?.previewPhotoUrl(
                    mSelectPhotos,
                    mUploadPictureIv
                ) { mSelectPhotos = "" }
            }
        }

        mAddPictureBtn.setOnClickListener {
            mPicker?.showDialog()
        }
    }

    override fun addPresenter(): BasePresenter {
        mUploadShoppingReceiptPresenter = UploadShoppingReceiptPresenter(this)
        return mUploadShoppingReceiptPresenter!!
    }

    private fun uploadCertificate() {
        mUploadShoppingReceiptPresenter?.apply {
            if (mCertificateUrl.isNullOrEmpty()) {
                requestSubmitPhotosToQinIu(mSelectPhotos)
            } else {
                requestUploadCertificate(mPhone)
            }
        }

    }

    override fun submitPhotosLoading() {
        mUploadProgress.visibility = View.VISIBLE
    }

    override fun submitPhotosSuccess(photos: String) {
        runOnUiThread {
            mUploadProgress.visibility = View.GONE
        }
        mUploadShoppingReceiptPresenter?.apply {
            mCertificateUrl = photos
            requestUploadCertificate(mPhone)
        }

    }

    override fun submitPhotosFailed(error: String) {
        runOnUiThread {
            mUploadProgress.visibility = View.GONE
            mUploadShoppingReceiptPresenter?.mCertificateUrl = ""
        }
    }

    override fun submitionStatus(isSuccess: Boolean) {
        mUploadProgress.visibility = View.GONE
        if (isSuccess) {
            jumpPrintEntitlementCardBarCodeActivity()
        }
    }

    private fun jumpPrintEntitlementCardBarCodeActivity() {
        VoucherCardQrCodeActivity.launch(mContext, mPhone)
        finish()
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        val photoPath: String =
            mPicker?.onPhotoPathResult(this, requestCode, resultCode, data) ?: ""

        if (photoPath.isEmpty()) return

        ImageLoader.loadFileImg(
            mContext,
            File(photoPath),
            mUploadPictureIv,
            object : ImageLoader.OnImageLoadingListener<File> {
                override fun onException(e: Exception?) {
                    mSelectPhotos = ""
                }

                override fun onResourceReady(t: File?) {
                    mSelectPhotos = photoPath
                }
            })
    }

    override fun onDestroy() {
        super.onDestroy()
        mPicker?.deletePicCache()
    }
}