package com.urovo.activity.superdietitian.view

import android.content.Intent
import android.os.Bundle
import android.view.View
import android.widget.ImageView
import com.alibaba.android.arouter.facade.annotation.Route
import com.urovo.activity.superdietitian.net.DietitianQueryRes
import com.urovo.activity.superdietitian.presenter.SuperDietitianPre
import com.urovo.wgmf.base.BaseContract
import com.urovo.wgmf.tool.netUtil.QiNiuDominator
import com.urovo.wugumofang.R
import com.urovo.wugumofang.base.BaseActivity
import com.urovo.wugumofang.constants.RouterParams
import com.urovo.wugumofang.system.SystemHelper
import com.urovo.wugumofang.ui.PhotoPickDominator
import com.urovo.wugumofang.ui.ViewUtil
import com.wgmf.trade.utils.ImageLoader
import kotlinx.android.synthetic.main.ll_super_dietitian_act.*
import org.jetbrains.anko.dip

/**
 * @author Sasuke on 2018/6/25..
 * 超级营养师
 */
@Route(path = RouterParams.ORIGIN.SUPER_DIETITIAN_ACT)
class SuperDietitianAct : BaseActivity(), BaseContract.view {
    var mPre: SuperDietitianPre? = null
    var isEditable: Boolean = false
    var picker: PhotoPickDominator? = null
    var ivTemp: ImageView? = null//缓存当前点击的imageView
    var imageLoader: ImageLoader? = null;

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.ll_super_dietitian_act)
        initTitle()
        SuperDietitianPre(this).run { start() }
    }

    override fun init() {
        titleTextView.text = "我是健康管理师"
        imageLoader = ImageLoader(this)
        picker = PhotoPickDominator(this)
    }

    override fun query() {
        mPre?.queryCertifyInfo(SystemHelper.LoginUserId)
    }

    /**
     * 根据返回刷新显示
     */
    fun refreshShow(resultInfo: DietitianQueryRes.ResultInfo?) {
        //初始显示
        dietitianUserId.text = resultInfo?.staffId ?: SystemHelper.LoginUserId
        dietitianUserName.text = resultInfo?.name
        dietitianDaqu.text = resultInfo?.areaOffice
        dietitianTheoryScore.apply { setText(resultInfo?.theoryScore); setTag("理论知识成绩") }
        dietitianOperScore.apply { setText(resultInfo?.operScore); setTag("操作技能成绩") }
        dietitianCertifyNum.apply { setText(resultInfo?.certNo); setTag("证书编号") }
        arrayOf(certPic, peoplePic).forEach {
            imageLoader?.loadRoundImgUrl(when (it) {
                certPic -> resultInfo?.certPicUrl.apply { it.tag = this }//把URL存到图片imageView的tag里，后面读取的时候就从tag读。
                peoplePic -> resultInfo?.peoplePicUrl.apply { it.tag = this }
                else -> null
            }, it as ImageView, dip(10), it.measuredWidth, it.measuredHeight)
        }

        //如果证书编号为空，那就设为可编辑状态
        isEditable = resultInfo?.certNo.isNullOrEmpty()
        if (isEditable) dietitianTheoryScore.requestFocus()

        //图片点击监听
        arrayOf(certPic, peoplePic).forEach {
            it.setOnClickListener {
                if (isEditable) {//只有在编辑态的时候才允许打开预览图
                    ivTemp = it as ImageView
                    if (it.tag == null || it.tag?.toString().isNullOrEmpty()) {//如果tag为空，说明没有图片
                        picker?.showDialog()//那就请用户传图片
                    } else {
                        picker?.previewPhotoUrl(it.tag?.toString(), it) { it.tag = null } //否则就弹全屏预览图片
                    }
                }
            }
        }
        //提交监听
        commitBtn.setOnClickListener {
            if (isEditable == false) {//如果不可编辑，再点击，就说明想编辑
                isEditable = true
                switchEditState(true)
            } else {//如果分数为空，说明是要提交
                commit()
            }
        }
        //如果没有分数，则说明并没有上传过，那就可编辑
        switchEditState(isEditable)
    }


    /**
     * 切换是否可编辑状态
     */
    fun switchEditState(editable: Boolean) {
        //信息输入框
        arrayOf(dietitianTheoryScore, dietitianOperScore, dietitianCertifyNum).forEach {
            it.apply {
                setBackgroundResource(if (editable) R.drawable.edit_unselected else 0)
                isFocusable = editable
                isFocusableInTouchMode = editable
            }
        }
        //显示在照片区域上的小相机图标、还有“请上传”的字样
        arrayOf(certPicCamera, cerPicTv, peoplePicCamera, peoplePicTv).forEach {
            it.visibility = if (editable) View.VISIBLE else View.GONE
        }
        //照片区
        arrayOf(certPicLayout, peoplePicLayout).forEach {
            it.apply { isClickable = editable }
        }
        //提交按钮
        commitBtn.text = if (editable) "提交" else "信息有误，重新提交"
    }

    /**
     * 提交数据
     */
    fun commit() {
        arrayOf(dietitianTheoryScore, dietitianOperScore, dietitianCertifyNum).forEach {
            if (it.text.toString().isNullOrEmpty()) {
                showMsg("请输入 ${it.tag}")
                return
            }
        }
        if (certPic.tag?.toString().isNullOrEmpty()) {
            showMsg("请添加证书图片")
            return
        }
        if (peoplePic.tag?.toString().isNullOrEmpty()) {
            showMsg("请添加头像")
            return
        }
        //开始上传
        ViewUtil.showProgress(this, "请稍等", "正在上传...")
        mPre?.uploadCertifyInfo(dietitianUserId.text.toString(),
                dietitianDaqu.text.toString(),
                dietitianTheoryScore.text.toString(),
                dietitianOperScore.text.toString(),
                certPic.tag?.toString(),
                dietitianCertifyNum.text.toString(),
                peoplePic.tag?.toString())
    }


    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        val photoPath = picker?.onPhotoPathResult(this, requestCode, resultCode, data)
        if (photoPath.isNullOrEmpty()) return
        //显示转圈圈
        val currentProgress = when (ivTemp) {
            certPic -> certProgress
            peoplePic -> peopleProgress
            else -> null
        }
        currentProgress?.let { it.visibility = View.VISIBLE }
        val uploadingIV = ivTemp//如果上传的时候再点击另一个上传，那可能ivTemp会变，所以要转存一下。

        //传完了回调，把七牛url存到ImageView的tag里
        QiNiuDominator.uploadImages(mutableListOf(photoPath)) { qiniuUrl ->
            uploadingIV?.apply {
                tag = qiniuUrl[0]//存tag
                imageLoader?.loadRoundImgUrl(tag?.toString(), this, dip(10), measuredWidth, measuredHeight)//显示图片
            }
            currentProgress?.visibility = View.GONE
        }
    }

    override fun onDestroy() {
        picker?.let { it.deletePicCache() }
        super.onDestroy()
    }

    override fun showLoading() {
    }

    override fun showNoData() {
    }

    override fun showNetError() {
    }

    override fun dismissLoadWait() {
    }

    override fun setPresenter(presenter: BaseContract.pre?) {
        this.mPre = presenter as SuperDietitianPre?
    }
}