package com.urovo.activity.systemsetting;

import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.Build;
import android.os.Bundle;
import android.provider.Settings;
import android.text.TextUtils;
import android.view.KeyEvent;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.Button;
import android.widget.ImageButton;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.TextView;

import com.alibaba.android.arouter.facade.annotation.Route;
import com.urovo.wgmf.adapter.MyBaseAdapter;
import com.urovo.wugumofang.R;
import com.urovo.wugumofang.base.BaseActivity;
import com.urovo.wugumofang.constants.RouterParams;
import com.urovo.wugumofang.data.PrefTool;
import com.urovo.wugumofang.system.BaseAppManager;
import com.urovo.wugumofang.system.Prefs;
import com.urovo.wugumofang.ui.ViewUtil;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;
@Route(path = RouterParams.ORIGIN.SYSTEMSETTINGBLUETOOTH)
public class SystemSettingBluetooth extends BaseActivity {

    private TextView text_title;
    private ImageButton goback;
    private Button btn_switch;
    private Button btn_search_devices;
    private ListView lv_blue_devices;
    private BluetoothAdapter bluetoothAdapter;
    private List<BluetoothDevice> devices;
    private DevicesAdapter devicesAdapter;
    private LinearLayout ll_saved;
    private TextView tv_saved_printer;

    private boolean flag = false;

    @Override
    protected void onDestroy() {

        unregisterReceiver(receiver);
        super.onDestroy();
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.setting_bluetooth);

        text_title = findViewById(R.id.text_title);
        goback = findViewById(R.id.goback);
        btn_switch = findViewById(R.id.btn_switch);
        btn_search_devices = findViewById(R.id.btn_search_devices);
        lv_blue_devices = findViewById(R.id.lv_blue_devices);
        ll_saved = findViewById(R.id.ll_saved);
        tv_saved_printer = findViewById(R.id.tv_saved_printer);

        bluetoothAdapter = BluetoothAdapter.getDefaultAdapter();
        devices = new ArrayList<>();

        text_title.setText("蓝牙设置");
        goback.setOnClickListener(new OnClickListener() {

            @Override
            public void onClick(View v) {
                finish();
            }
        });

        btn_search_devices.setText("搜索蓝牙设备");
        if(bluetoothAdapter.isEnabled()) {
            btn_switch.setText("关闭蓝牙");
            btn_search_devices.setEnabled(true);
        }
        else {
            btn_switch.setText("打开蓝牙");
            btn_search_devices.setEnabled(false);
        }
        if(bluetoothAdapter == null) {
            showMsg("本设备不支持蓝牙功能");
            btn_switch.setEnabled(false);
            btn_search_devices.setEnabled(false);
        }
        else {
            String saved = PrefTool.getStringPerferences(SystemSettingBluetooth.this, Prefs.PRINT_ADDRESS, "");
            if(TextUtils.isEmpty(saved)) {
                ll_saved.setVisibility(View.GONE);
            }
            else {
                ll_saved.setVisibility(View.VISIBLE);
                String name = "";
                if(saved.length() > saved.indexOf("#") + 1) {
                    name = saved.substring(saved.indexOf("#") + 1) + "(" + saved.substring(0, saved.indexOf("#")) + ")";
                }
                else {
                    name = saved.substring(0, saved.indexOf("#"));
                }
                tv_saved_printer.setText(name);
            }

            btn_switch.setOnClickListener(new OnClickListener() {

                @Override
                public void onClick(View v) {
                    // 打开蓝牙
                    if(!bluetoothAdapter.isEnabled()) {
                        // Intent intent = new
                        // Intent(BluetoothAdapter.ACTION_REQUEST_ENABLE);
                        // // 设置蓝牙可见性，最多300秒
                        // intent.putExtra(BluetoothAdapter.EXTRA_DISCOVERABLE_DURATION,
                        // 300);
                        // startActivity(intent);
                        bluetoothAdapter.enable();
                        // while(!bluetoothAdapter.isEnabled()){
                        //
                        // }
                        // btn_search_devices.setEnabled(true);
                    }
                    else {
                        bluetoothAdapter.disable();
                        btn_search_devices.setEnabled(false);
                    }

                }
            });

            initIntentFilter();

            btn_search_devices.setOnClickListener(new OnClickListener() {

                @Override
                public void onClick(View v) {
                    if(isLocationEnabled(mContext)) {
                        if (bluetoothAdapter.isDiscovering())
                            bluetoothAdapter.cancelDiscovery();
                        bluetoothAdapter.startDiscovery();
                        devices.clear();
                        addDevicesToListView();
                        flag = true;
                    }else{
                        startActivity(new Intent(Settings.ACTION_LOCATION_SOURCE_SETTINGS));
                    }
                }
            });

            // 为每个item绑定监听，用于设备间的配对
            lv_blue_devices.setOnItemClickListener(new OnItemClickListener() {

                @Override
                public void onItemClick(AdapterView<?> arg0, View arg1, int arg2, long arg3) {
                    try {
                        BluetoothDevice device = devices.get(arg2);
                        Method createBondMethod = BluetoothDevice.class.getMethod("createBond");
                        createBondMethod.invoke(device);
                        String address = device.getAddress();
                        String name = device.getName();
                        PrefTool.setStringsave(SystemSettingBluetooth.this, Prefs.PRINT_ADDRESS, address + "#" + name);
                        Intent intent = new Intent();
                        setResult(102, intent);
                        finish();
                    }
                    catch (Exception e) {
                        showMsg("配对失败");
                        BaseAppManager.loger().info(e);
                    }

                }
            });
        }
    }

    /**
     *  判断定位服务
     */
    public static boolean isLocationEnabled(Context context) {
        int locationMode = 0;
        String locationProviders;
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.KITKAT) {
            try {
                locationMode = Settings.Secure.getInt(context.getContentResolver(), Settings.Secure.LOCATION_MODE);
            } catch (Settings.SettingNotFoundException e) {
                e.printStackTrace();
                return false;
            }
            return locationMode != Settings.Secure.LOCATION_MODE_OFF;
        } else {
            locationProviders = Settings.Secure.getString(context.getContentResolver(), Settings.Secure.LOCATION_PROVIDERS_ALLOWED);
            return !TextUtils.isEmpty(locationProviders);
        }
    }

    private void addDevicesToListView() {
        if(devicesAdapter == null) {
            devicesAdapter = new DevicesAdapter(SystemSettingBluetooth.this, devices);
            lv_blue_devices.setAdapter(devicesAdapter);
        }
        else {
            devicesAdapter.notifyDataSetChanged();
        }
    }

    private void initIntentFilter() {
        // 设置广播信息过滤
        IntentFilter intentFilter = new IntentFilter();
        intentFilter.addAction(BluetoothDevice.ACTION_FOUND);
        intentFilter.addAction(BluetoothAdapter.ACTION_DISCOVERY_STARTED);
        intentFilter.addAction(BluetoothAdapter.ACTION_DISCOVERY_FINISHED);
        intentFilter.addAction(BluetoothAdapter.ACTION_STATE_CHANGED);
        // 注册广播接收器，接收并处理搜索结果
        registerReceiver(receiver, intentFilter);

    }

    /**
     * 蓝牙广播接收器
     */
    private BroadcastReceiver receiver = new BroadcastReceiver() {

        @Override
        public void onReceive(Context context, Intent intent) {
            String action = intent.getAction();
            if(BluetoothDevice.ACTION_FOUND.equals(action)) {
                BluetoothDevice device = intent.getParcelableExtra(BluetoothDevice.EXTRA_DEVICE);
                if(device.getBondState() == BluetoothDevice.BOND_BONDED) {
                    devices.add(device);
                }
                else {
                    devices.add(device);
                }
                addDevicesToListView();
            }
            else
            if(BluetoothAdapter.ACTION_DISCOVERY_STARTED.equals(action)) {
                ViewUtil.showProgress(SystemSettingBluetooth.this, "系统提示", "搜索蓝牙设备中...");
            }
            else
            if(BluetoothAdapter.ACTION_DISCOVERY_FINISHED.equals(action)) {
                ViewUtil.dismissProgress();
                addDevicesToListView();
                // bluetoothAdapter.cancelDiscovery();
            }
            if(BluetoothAdapter.ACTION_STATE_CHANGED.equals(action)) {
                if(bluetoothAdapter.getState() == BluetoothAdapter.STATE_ON) {
                    btn_switch.setText("关闭蓝牙");
                    btn_search_devices.setEnabled(true);
                }
                else
                if(bluetoothAdapter.getState() == BluetoothAdapter.STATE_OFF) {
                    btn_switch.setText("打开蓝牙");
                    btn_search_devices.setEnabled(false);
                }
            }

        }

    };

    private class DevicesAdapter extends MyBaseAdapter<BluetoothDevice, ListView> {

        public DevicesAdapter(Context context, List<BluetoothDevice> device) {
            super(device, context);
        }

        @Override
        public View getView(final int position, View convertView, ViewGroup parent) {
            View view;
            if(convertView == null) {
                view = View.inflate(context, R.layout.listview_item_bluetooth, null);
            }
            else {
                view = convertView;
            }
            TextView tv_device = (TextView) view.findViewById(R.id.tv_device);
            TextView tv_address = (TextView) view.findViewById(R.id.tv_address);
            String name = lists.get(position).getName();
            String address = lists.get(position).getAddress();
            if(TextUtils.isEmpty(name)) {
                name = lists.get(position).getAddress();
            }
            else {
                if(TextUtils.isEmpty(name.trim())) {
                    name = lists.get(position).getAddress();
                }
            }
            tv_device.setText(name);
            tv_address.setText(address);
            return view;
        }
    }

    @Override
    public boolean onKeyDown(int keyCode, KeyEvent event) {
        if(keyCode == KeyEvent.KEYCODE_BACK) {
            if(flag) {
                ViewUtil.dismissProgress();
                bluetoothAdapter.cancelDiscovery();
                flag = false;
            }
            else {
                return super.onKeyDown(keyCode, event);
            }
        }
        return super.onKeyDown(keyCode, event);
    }
}
