package com.urovo.activity.tabMenu.home.act;


import android.animation.AnimatorSet;
import android.animation.ObjectAnimator;
import android.app.Activity;
import android.content.Intent;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.os.Bundle;
import android.text.TextUtils;
import android.view.View;
import android.view.ViewGroup;
import android.widget.LinearLayout;
import android.widget.TextView;

import com.chad.library.adapter.base.BaseQuickAdapter;
import com.chad.library.adapter.base.listener.OnItemDragListener;
import com.chad.library.adapter.base.listener.OnItemSwipeListener;
import com.google.gson.Gson;
import com.urovo.activity.tabMenu.home.ada.MultipleItemQuickAdapter;
import com.urovo.activity.tabMenu.home.callback.MultipleItemDragAndSwipeCallback;
import com.urovo.activity.tabMenu.home.ent.ChannelBean;
import com.urovo.activity.tabMenu.home.ent.MultipleItem;
import com.urovo.activity.tabMenu.home.holder.BaseMultiItemHolder;
import com.urovo.activity.tabMenu.home.menu.ShortcutMenuPre;
import com.urovo.activity.tabMenu.home.menu.ShortcutMenuPreImpl;
import com.urovo.activity.tabMenu.home.pre.DragMenuPre;
import com.urovo.activity.tabMenu.home.utils.RxDisposeManager;
import com.urovo.activity.tabMenu.home.view.ShortcutMenuView;
import com.urovo.wugumofang.R;
import com.urovo.wugumofang.base.BaseActivity;
import com.urovo.wugumofang.constants.AppConstants;
import com.urovo.wugumofang.net.CommonController;
import com.urovo.wugumofang.response.CommonEntity;
import com.urovo.wugumofang.system.SystemHelper;
import com.urovo.wugumofang.ui.DensityUtil;
import com.urovo.wugumofang.utils.MyToast;
import com.zsy.libcore.common.android.log.SYLoger;

import org.greenrobot.eventbus.EventBus;
import org.greenrobot.eventbus.Subscribe;
import org.greenrobot.eventbus.ThreadMode;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import androidx.recyclerview.widget.GridLayoutManager;
import androidx.recyclerview.widget.ItemTouchHelper;
import androidx.recyclerview.widget.RecyclerView;
import butterknife.BindView;
import butterknife.ButterKnife;
import butterknife.OnClick;
import butterknife.Unbinder;

/**
 * 快捷菜单编辑页面
 *
 * @author chenji
 * @date 2017/12/16
 */
public class ShortcutMenuAct extends BaseActivity implements ShortcutMenuView, CommonController.BaseView<String> {
    @BindView(R.id.item_horizontal_lines)
    View mHorizontalLines;
    @BindView(R.id.rv_list)
    RecyclerView mMineRv;
    @BindView(R.id.news_channel_more_rv)
    RecyclerView mMoreRv;
    private MultipleItemQuickAdapter mMineAdapter;
    private MultipleItemQuickAdapter mMoreAdapter;
    private MultipleItemDragAndSwipeCallback mItemDragAndSwipeCallback;
    private ShortcutMenuPre mPresenter;
    private Unbinder unbinder;
    boolean flag;
    /**
     * 编辑/完成按钮切换的flag
     */
    boolean isEnableEditFlag;
    DragMenuPre dragMenuPre;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_item_touch_use);
        unbinder = ButterKnife.bind(this);
        initTitle();
        if (!EventBus.getDefault().isRegistered(this)) {
            EventBus.getDefault().register(this);
        }
        //初始化
        mPresenter = new ShortcutMenuPreImpl(this);
        // 加载编辑页的快捷菜单,多布局,每个item占用1/4,spanSize为1
        mPresenter.lodeChannelsRequest(1);
        dragMenuPre = new DragMenuPre(this);

        //填充白色线条
        setHorizontalLine();
    }

    /**
     * 设置白色线条
     */
    private void setHorizontalLine() {
        mHorizontalLines.setBackgroundColor(Color.WHITE);
        //高度设置为18dp
        LinearLayout.LayoutParams params = new LinearLayout.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, DensityUtil.dip2px(this, 18));
        mHorizontalLines.setLayoutParams(params);
    }

    @Override
    public void initTitle() {
        super.initTitle();
        titleTextView.setText("快捷菜单");
        submit.setVisibility(View.VISIBLE);
        submit.setText("编辑");
        submit.setTextColor(getResources().getColor(R.color.title_yellow_bg));
    }

    /**
     * 我的菜单(默认)
     */
    @Override
    public void currentFunctionList(List<MultipleItem> multipleItemMine) {
        mMineRv.setLayoutManager(new GridLayoutManager(this, 4));
        //recyclerView 的宽高不会因为增删内容而改变
        mMineRv.setHasFixedSize(true);
        setBtnAttribute(multipleItemMine, false, isEnableEditFlag, 0, 1);
        mMineAdapter = new MultipleItemQuickAdapter(multipleItemMine);
        mMineRv.setAdapter(mMineAdapter);
        setEnableClick();
    }

    /**
     * 当前功能列表拖拽和滑动的配置
     */
    private void configDragAndSwipe() {
        OnItemDragListener listener = new OnItemDragListener() {
            @Override
            public void onItemDragStart(RecyclerView.ViewHolder viewHolder, int pos) {
                BaseMultiItemHolder holder = ((BaseMultiItemHolder) viewHolder);
                ObjectAnimator scaleX = holder.setPropertyAnima(R.id.menu_icon, "ScaleX", 1f, 1.1f);
                ObjectAnimator scaleY = holder.setPropertyAnima(R.id.menu_icon, "ScaleY", 1f, 1.1f);
                ObjectAnimator alpha = holder.setPropertyAnima(R.id.menu_icon, "Alpha", 0f, 0.5f, 1f);
                AnimatorSet animSet = new AnimatorSet();
                animSet.play(scaleX).with(scaleY).with(alpha);
                animSet.setDuration(300L);
                animSet.start();
            }

            @Override
            public void onItemDragMoving(RecyclerView.ViewHolder source, int from, RecyclerView.ViewHolder target, int to) {
            }

            @Override
            public void onItemDragEnd(RecyclerView.ViewHolder viewHolder, int pos) {
                BaseMultiItemHolder holder = ((BaseMultiItemHolder) viewHolder);
                ObjectAnimator scaleX = holder.setPropertyAnima(R.id.menu_icon, "ScaleX", 1.1f, 1f);
                ObjectAnimator scaleY = holder.setPropertyAnima(R.id.menu_icon, "ScaleY", 1.1f, 1f);
                AnimatorSet animSet = new AnimatorSet();
                animSet.play(scaleX).with(scaleY);
                animSet.setDuration(300L);
                animSet.start();
            }
        };

        final Paint paint = new Paint();
        paint.setAntiAlias(true);
        paint.setTextSize(20);
        paint.setColor(Color.BLACK);
        OnItemSwipeListener onItemSwipeListener = new OnItemSwipeListener() {
            @Override
            public void onItemSwipeStart(RecyclerView.ViewHolder viewHolder, int pos) {
//                BaseMultiItemHolder holder = ((BaseMultiItemHolder) viewHolder);
//                holder.setTextColor(R.id.tv, Color.WHITE);
            }

            @Override
            public void clearView(RecyclerView.ViewHolder viewHolder, int pos) {
//                BaseMultiItemHolder holder = ((BaseMultiItemHolder) viewHolder);
//                holder.setTextColor(R.id.tv, Color.BLACK);
            }

            @Override
            public void onItemSwiped(RecyclerView.ViewHolder viewHolder, int pos) {
            }

            @Override
            public void onItemSwipeMoving(Canvas canvas, RecyclerView.ViewHolder viewHolder, float dX, float dY, boolean isCurrentlyActive) {
//                canvas.drawColor(ContextCompat.getColor(ShortcutMenuAct.this, R.color.color_light_blue));
//                canvas.drawText("Just some text", 0, 40, paint);
            }
        };

        mItemDragAndSwipeCallback = new MultipleItemDragAndSwipeCallback(ShortcutMenuAct.this, mMineAdapter);
        ItemTouchHelper mItemTouchHelper = new ItemTouchHelper(mItemDragAndSwipeCallback);
        mItemTouchHelper.attachToRecyclerView(mMineRv);

        mItemDragAndSwipeCallback.setDragMoveFlags(ItemTouchHelper.LEFT | ItemTouchHelper.RIGHT | ItemTouchHelper.UP | ItemTouchHelper.DOWN);
//        mItemDragAndSwipeCallback.setSwipeMoveFlags(ItemTouchHelper.START | ItemTouchHelper.END);
        mMineAdapter.enableSwipeItem();
        mMineAdapter.setOnItemSwipeListener(onItemSwipeListener);
        mMineAdapter.enableDragItem(mItemTouchHelper);
        mMineAdapter.setOnItemDragListener(listener);
//        mMineRv.addItemDecoration(new GridItemDecoration(this ,R.drawable.list_divider));

//        mMineRv.addOnItemTouchListener(new OnItemClickListener() {
//            @Override
//            public void onSimpleItemClick(final BaseQuickAdapter adapter, final View view, final int position) {
//                ToastUtils.showShortToast("点击了" + position);
//            }
//        });
    }

    /**
     * 当前功能列表可以点击设置
     */
    private void setEnableClick() {
        mMineAdapter.setOnItemClickListener(new BaseQuickAdapter.OnItemClickListener() {
            @Override
            public void onItemClick(BaseQuickAdapter adapter, View view, int position) {
                if (isEnableEditFlag) {
                    if (position == 0 || position == 1) {
                        return;
                    }
                    MultipleItem mimeItem = mMineAdapter.getData().get(position);
                    mMoreAdapter.getData().add(mimeItem);
                    mMoreAdapter.notifyDataSetChanged();
                    mMineAdapter.getData().remove(position);
                    mMineAdapter.notifyDataSetChanged();
//                    //进行添加或删除操作后，要更新的列表 进行存储
//                    mPresenter.onItemAddOrRemove(mMineAdapter.getData(), mMoreAdapter.getData());

                    //当前操作我的菜单,更新菜单
                    getUpdateMenuPositionList();
                }

                refreshBtn(isEnableEditFlag);
            }
        });
    }

    /**
     * 控制菜单item 的显隐
     *
     * @param isEnableEditFlag 编辑/完成按钮的flag
     */
    List<MultipleItem> mimeList;
    List<MultipleItem> moreList;

    private void setBtnAttribute(List<MultipleItem> multipleItemList, boolean isAdd, boolean isShow) {
        if (multipleItemList != null && !multipleItemList.isEmpty()) {
            for (MultipleItem multipleItem : multipleItemList) {
                multipleItem.setAdd(isAdd);
                multipleItem.setShow(isShow);
            }
        }
    }

    /**
     * @param multipleItemList
     * @param isAdd
     * @param isShow
     * @param limitPosition
     */
    private void setBtnAttribute(List<MultipleItem> multipleItemList, boolean isAdd, boolean isShow, int... limitPosition) {
        if (multipleItemList != null && !multipleItemList.isEmpty()) {
            for (MultipleItem multipleItem : multipleItemList) {
                if (multipleItemList.indexOf(multipleItem) == limitPosition[0]
                        || multipleItemList.indexOf(multipleItem) == limitPosition[1]) {
                    multipleItem.setShow(false);
                } else {
                    multipleItem.setAdd(isAdd);
                    multipleItem.setShow(isShow);
                }
            }
        }
    }

//    private void setBtnAttribute(List<MultipleItem> multipleItemList, boolean isEnableEditFlag, String mode) {
//        if ("more".equals(mode)) {
//            setBtnAttribute(multipleItemList, true, isEnableEditFlag);
//        } else {
//            setBtnAttribute(multipleItemList, false, isEnableEditFlag, 0, 1);
////            for (MultipleItem mimeItem : multipleItemList) {
////                if (multipleItemList.indexOf(mimeItem) == 0 || multipleItemList.indexOf(mimeItem) == 1) {
////                    mimeItem.setShow(false);
////                } else {
////                    mimeItem.setShow(true);
////                    mimeItem.setAdd(false);
////                }
////            }
//        }
//    }

    private void refreshBtn(boolean isEnableEditFlag) {
        mimeList = mMineAdapter.getData();
        moreList = mMoreAdapter.getData();

        if (isEnableEditFlag) {
            setBtnAttribute(mimeList, false, true, 0, 1);
            setBtnAttribute(moreList, true, true);
//            for (MultipleItem mimeItem : mimeList) {
//                mimeItem.setAdd(false);
//                mimeItem.setShow(true);
//            }
//            for (MultipleItem moreItem : moreList) {
//                moreItem.setAdd(true);
//                moreItem.setShow(true);
//            }
        }
//        else {
//            setBtnAttribute(mimeList, false, false);
//            setBtnAttribute(moreList, true, false);
////            for (MultipleItem mimeItem : mimeList) {
////                mimeItem.setAdd(false);
////                mimeItem.setShow(false);
////            }
////            for (MultipleItem moreItem : moreList) {
////                moreItem.setAdd(true);
////                moreItem.setShow(false);
////            }
//        }
        //进行添加或删除操作后，要更新的列表 进行存储
        mPresenter.onItemAddOrRemove(mimeList, moreList);

        mMineAdapter.notifyDataSetChanged();
        mMoreAdapter.notifyDataSetChanged();

    }

    /**
     * 更新后菜单的集合
     * 把更新后的position集合字符串提交到后台
     */
    private void getUpdateMenuPositionList() {
        mUpdateMenuPositionList.clear();
        mimeTypeIdList.clear();
        moreTypeIdList.clear();

        for (MultipleItem mimeItem : mMineAdapter.getData()) {
            mimeTypeIdList.add(mimeItem.getTypeId());
        }
        for (MultipleItem moreItem : mMoreAdapter.getData()) {
            moreTypeIdList.add(moreItem.getTypeId());
        }

        mUpdateMenuPositionList.add(mimeTypeIdList);
        mUpdateMenuPositionList.add(moreTypeIdList);
    }

    private List<List<Integer>> mUpdateMenuPositionList = new ArrayList<>();
    private List<Integer> mimeTypeIdList = new ArrayList<>();
    private List<Integer> moreTypeIdList = new ArrayList<>();

    /**
     * 更多功能列表
     */
    @Override
    public void moreFunctionList(List<MultipleItem> multipleItemMore) {
        mMoreRv.setLayoutManager(new GridLayoutManager(this, 4));
        //recyclerView 的宽高不会因为增删内容而改变
        mMoreRv.setHasFixedSize(true);
        setBtnAttribute(multipleItemMore, true, isEnableEditFlag);
        mMoreAdapter = new MultipleItemQuickAdapter(multipleItemMore);
        mMoreRv.setAdapter(mMoreAdapter);
        mMoreAdapter.setOnItemClickListener(new BaseQuickAdapter.OnItemClickListener() {
            @Override
            public void onItemClick(BaseQuickAdapter adapter, View view, int position) {
                if (isEnableEditFlag) {
                    // 需要根据上层view的多少来确定
                    if (mMineAdapter.getData().size() != 7) {
                        final MultipleItem moreItem = mMoreAdapter.getData().get(position);
                        mMoreAdapter.getData().remove(position);
                        mMoreAdapter.notifyDataSetChanged();
                        mMineAdapter.getData().add(moreItem);
                        mMineAdapter.notifyDataSetChanged();

                        //当前操作更多功能,更新菜单
                        getUpdateMenuPositionList();

                    } else {
                        MyToast.show("最多只能添加7个");
                    }
                    //进行添加或删除操作后，要更新的列表 进行存储
//                    mPresenter.onItemAddOrRemove(mMineAdapter.getData(), mMoreAdapter.getData());
                }
                refreshBtn(isEnableEditFlag);
            }
        });
    }

//    private void saveListData(List<List<HomeMenuOnlineBean.ResultInfoBean>> menuEntity) {
//        final List<HomeMenuOnlineBean.ResultInfoBean> onlineMineList = menuEntity.get(0);
//        final List<HomeMenuOnlineBean.ResultInfoBean> onlineMoreList = menuEntity.get(1);
//        SYLoger.print("mine-1->" + onlineMineList);
//        SYLoger.print("more-1->" + onlineMoreList);
//        final String onlineMine = new Gson().toJson(onlineMineList);
//        final String onlineMore = new Gson().toJson(onlineMoreList);
//        PreferHelper.getInstance().setString(AppConstants.Home.ONLINE_MINE, onlineMine);
//        PreferHelper.getInstance().setString(AppConstants.Home.ONLINE_MORE, onlineMore);
//    }

    /**
     * 接收MultipleItemDragAndSwipeCallback 拖拽item后的最新列表数据
     *
     * @param event ...
     */
    @SuppressWarnings("unused")
    @Subscribe(threadMode = ThreadMode.MAIN)
    public void onChannelEvent(ChannelBean event) {
        List<MultipleItem> list = event.getList();
        //发送到首页更新
        mPresenter.onItemSwap(list);
    }

    // 带考究
    @Override
    protected void onDestroy() {
        super.onDestroy();
        RxDisposeManager.get().cancel("newsMinechannel");
        RxDisposeManager.get().cancel("newsMorechannel");
        EventBus.getDefault().unregister(this);
        if (unbinder != null) {
            unbinder.unbind();
        }
    }

    /**
     * 入口
     *
     * @param activity ...
     */
    public static void start(Activity activity) {
        Intent intent = new Intent(activity, ShortcutMenuAct.class);
        activity.startActivity(intent);
        activity.overridePendingTransition(R.anim.fade_in, R.anim.fade_out);
    }

    @OnClick(R.id.confirm)
    public void onClick(TextView mConfirmBtn) {
//        if (CommonTools.INSTANCE.isOneClick()) {
            //可以拖拽
            if (toggle()) {
                ObjectAnimator scaleX = setPropertyAnima(mConfirmBtn, "ScaleX", 1f, 1.1f);
                ObjectAnimator scaleY = setPropertyAnima(mConfirmBtn, "ScaleY", 1f, 1.1f);
                ObjectAnimator alpha = setPropertyAnima(mConfirmBtn, "Alpha", 0f, 0.5f, 1f);
                AnimatorSet animSet = new AnimatorSet();
                animSet.play(scaleX).with(scaleY).with(alpha);
                animSet.setDuration(300L);
                animSet.start();

                mConfirmBtn.setText("完成");
                isEnableEditFlag = true;
                configDragAndSwipe();
                mItemDragAndSwipeCallback.setLongPressEnabled(true);
                mMineAdapter.notifyDataSetChanged();
            }
            //不可以拖拽
            else {
                ObjectAnimator scaleX = setPropertyAnima(mConfirmBtn, "ScaleX", 1.1f, 1f);
                ObjectAnimator scaleY = setPropertyAnima(mConfirmBtn, "ScaleY", 1.1f, 1f);
                AnimatorSet animSet = new AnimatorSet();
                animSet.play(scaleX).with(scaleY);
                animSet.setDuration(300L);
                animSet.start();

                mConfirmBtn.setText("编辑");
                isEnableEditFlag = false;
                mItemDragAndSwipeCallback.setLongPressEnabled(false);

                String updatePositionJson = new Gson().toJson(mUpdateMenuPositionList);

                //正则截取字符串第一个[]里面的内容
                updatePositionJson = matcher(updatePositionJson);
                SYLoger.print("wancheng -->" + updatePositionJson);

                if (!"[]".equals(updatePositionJson) && !"".equals(updatePositionJson)) {
                    // 用户更改快捷菜单后的position集合上传到后台
                    start2requestTask(updatePositionJson);
                }
            }
//        }
        mPresenter.lodeChannelsRequest(1);
    }

    /**
     * 更新菜单(网络请求)
     *
     * @param updatePositionJson 组装的数组json串,更新菜单列表的位置后的typeid集合
     */
    private void start2requestTask(String updatePositionJson) {
        Map<String, String> params = new HashMap<>(3);
        params.put("userNo", SystemHelper.LoginUserId);
        params.put("type", AppConstants.TabActState.HOME_STATE);
        params.put("data", updatePositionJson);
        dragMenuPre.onStart2requestData(params);
    }

    /**
     * 正则截取字符串第一个[]里面的内容
     */
    private String matcher(String updatePositionJson) {
        String updatePositionJson1 = "[]";
        if (!TextUtils.isEmpty(updatePositionJson)) {
            String regex = "\\[(.*)]";
            Pattern p = Pattern.compile(regex);
            Matcher m = p.matcher(updatePositionJson);
            while (m.find()) {
                updatePositionJson1 = m.group(1);
            }
        }
        return updatePositionJson1;
    }

    /**
     * 切换开关
     *
     * @return ...
     */
    public boolean toggle() {
        return flag = !flag;
    }

    /**
     * 属性动画
     *
     * @param view         ...
     * @param propertyName ...
     * @param value        ...
     * @return ...
     */
    public ObjectAnimator setPropertyAnima(View view, String propertyName, float... value) {
        return ObjectAnimator.ofFloat(view, propertyName, value);
    }

    @Override
    public void onResponseSuccess(CommonEntity<String> resultInfo) {
        final String entity = resultInfo.entity;
        SYLoger.print(entity);
        MyToast.show(entity);
    }

    @Override
    public void onFailure(String failInfo) {
        MyToast.show(failInfo);
    }
}
