package com.urovo.activity.tabMenu.home.callback;


import android.content.Context;
import android.graphics.Canvas;
import android.view.View;

import com.chad.library.adapter.base.BaseQuickAdapter;
import com.urovo.activity.tabMenu.home.ada.BaseMultiItemDraggableAdapter;
import com.urovo.activity.tabMenu.home.ent.ChannelBean;

import org.greenrobot.eventbus.EventBus;

import androidx.recyclerview.widget.GridLayoutManager;
import androidx.recyclerview.widget.ItemTouchHelper;
import androidx.recyclerview.widget.RecyclerView;

/**
 * @author luoxw
 * @date 2016/6/20
 */
public class MultipleItemDragAndSwipeCallback extends ItemTouchHelper.Callback {

//    private static final String TAG = ItemDragAndSwipeCallback.class.getSimpleName();

    private BaseMultiItemDraggableAdapter mAdapter;

    private float mMoveThreshold = 0.1f;
    private float mSwipeThreshold = 0.7f;
    private int mDragMoveFlags = ItemTouchHelper.UP | ItemTouchHelper.DOWN | ItemTouchHelper.LEFT | ItemTouchHelper.RIGHT;
    private int mSwipeMoveFlags = ItemTouchHelper.END;
    private boolean mIsLongPressEnabled;
    private Context mContext;

    public MultipleItemDragAndSwipeCallback(Context context, BaseMultiItemDraggableAdapter adapter) {
        mAdapter = adapter;
        mContext = context;
    }

    /**
     * 长按是否可拖拽
     *
     * @param longPressEnabled true:可拖拽
     */
    public void setLongPressEnabled(boolean longPressEnabled) {
        mIsLongPressEnabled = !longPressEnabled;
    }

    @Override
    public boolean isLongPressDragEnabled() {
        return false;
    }

    @Override
    public boolean isItemViewSwipeEnabled() {
        return mAdapter.isItemSwipeEnable();
    }

    @Override
    public void onSelectedChanged(RecyclerView.ViewHolder viewHolder, int actionState) {
// 1.设置深色背景
//        if (actionState != ItemTouchHelper.ACTION_STATE_IDLE) {
//            //给被拖曳的 item 设置一个深颜色背景
//            viewHolder.itemView.setBackgroundColor(Color.parseColor("#F6F6F6"));
//        }

        if (actionState == ItemTouchHelper.ACTION_STATE_DRAG
                && !isViewCreateByAdapter(viewHolder)) {
            mAdapter.onItemDragStart(viewHolder);
            viewHolder.itemView.setTag(com.chad.library.R.id.BaseQuickAdapter_dragging_support, true);
//            viewHolder.itemView.setBackgroundColor(Color.WHITE);//这样写有阴影效果,颜色随便设置
        } else if (actionState == ItemTouchHelper.ACTION_STATE_SWIPE
                && !isViewCreateByAdapter(viewHolder)) {
            mAdapter.onItemSwipeStart(viewHolder);
            viewHolder.itemView.setTag(com.chad.library.R.id.BaseQuickAdapter_swiping_support, true);
        }
        super.onSelectedChanged(viewHolder, actionState);
    }

    @Override
    public void clearView(RecyclerView recyclerView, RecyclerView.ViewHolder viewHolder) {
        super.clearView(recyclerView, viewHolder);
        if (isViewCreateByAdapter(viewHolder)) {
            return;
        }
//2.给已经完成拖曳的 item 恢复开始的背景。这里设置的颜色尽量和item 在 xml 中设置的颜色保持一致才生效,子层不能设置颜色
//        viewHolder.itemView.setBackgroundColor(Color.WHITE);
        if (viewHolder.itemView.getTag(com.chad.library.R.id.BaseQuickAdapter_dragging_support) != null
                && (Boolean) viewHolder.itemView.getTag(com.chad.library.R.id.BaseQuickAdapter_dragging_support)) {
            mAdapter.onItemDragEnd(viewHolder);
            viewHolder.itemView.setTag(com.chad.library.R.id.BaseQuickAdapter_dragging_support, false);
        }

        if (viewHolder.itemView.getTag(com.chad.library.R.id.BaseQuickAdapter_swiping_support) != null
                && (Boolean) viewHolder.itemView.getTag(com.chad.library.R.id.BaseQuickAdapter_swiping_support)) {
            mAdapter.onItemSwipeClear(viewHolder);
            viewHolder.itemView.setTag(com.chad.library.R.id.BaseQuickAdapter_swiping_support, false);
        }
    }

    @Override
    public int getMovementFlags(RecyclerView recyclerView, RecyclerView.ViewHolder viewHolder) {
        //满足条件不能拖拽也不能移动
        if (isViewCreateByAdapter(viewHolder)
                || mIsLongPressEnabled
                || viewHolder.getLayoutPosition() == 0
                || viewHolder.getLayoutPosition() == 1
                ) {
            return makeMovementFlags(0, 0);
        } else if (recyclerView.getLayoutManager() instanceof GridLayoutManager) {
            int swipeFlags = 0;
            return makeMovementFlags(mDragMoveFlags, swipeFlags);
        } else {
//            int dragFlags = ItemTouchHelper.UP | ItemTouchHelper.DOWN;
//            int swipeFlags = ItemTouchHelper.START | ItemTouchHelper.END;
            return makeMovementFlags(mDragMoveFlags, mSwipeMoveFlags);
        }
//        return makeMovementFlags(mDragMoveFlags, mSwipeMoveFlags);
    }

    @Override
    public boolean onMove(RecyclerView recyclerView, RecyclerView.ViewHolder source, RecyclerView.ViewHolder target) {
//        //起始位置,得到拖动ViewHolder的position
//        int fromPosition = source.getAdapterPosition();
        //终点位置,得到目标ViewHolder的position
        int toPosition = target.getAdapterPosition();

        //满足条件返回false表示不能被拖动到目标位置
        return !(toPosition == 0 || toPosition == 1 || mIsLongPressEnabled) && source.getItemViewType() == target.getItemViewType();
    }

    @SuppressWarnings("unchecked")
    @Override
    public void onMoved(RecyclerView recyclerView, RecyclerView.ViewHolder source, int fromPos, RecyclerView.ViewHolder target, int toPos, int x, int y) {
        super.onMoved(recyclerView, source, fromPos, target, toPos, x, y);
        mAdapter.onItemDragMoving(source, target);
        //拖拽item的时候把改变的数据列表发送到编辑菜单页面,最终是为了首页菜单更新
        EventBus.getDefault().post(new ChannelBean((mAdapter.getData())));
    }

    @Override
    public void onSwiped(RecyclerView.ViewHolder viewHolder, int direction) {
        if (!isViewCreateByAdapter(viewHolder)) {
            mAdapter.onItemSwiped(viewHolder);
        }
    }

//    @Override
//    public float getMoveThreshold(RecyclerView.ViewHolder viewHolder) {
//        return mMoveThreshold;
//    }

    @Override
    public float getSwipeThreshold(RecyclerView.ViewHolder viewHolder) {
        return mSwipeThreshold;
    }

    /**
     * Set the fraction that the user should move the View to be considered as swiped.
     * The fraction is calculated with respect to RecyclerView's bounds.
     * <p>
     * Default value is .5f, which means, to swipe a View, user must move the View at least
     * half of RecyclerView's width or height, depending on the swipe direction.
     *
     * @param swipeThreshold A float value that denotes the fraction of the View size. Default value
     *                       is .8f .
     */
    public void setSwipeThreshold(float swipeThreshold) {
        mSwipeThreshold = swipeThreshold;
    }


    /**
     * Set the fraction that the user should move the View to be considered as it is
     * dragged. After a view is moved this amount, ItemTouchHelper starts checking for Views
     * below it for a possible drop.
     *
     * @param moveThreshold A float value that denotes the fraction of the View size. Default value is
     *                      .1f .
     */
//    public void setMoveThreshold(float moveThreshold) {
//        mMoveThreshold = moveThreshold;
//    }

    /**
     * <p>Set the drag movement direction.</p>
     * <p>The value should be ItemTouchHelper.UP, ItemTouchHelper.DOWN, ItemTouchHelper.LEFT, ItemTouchHelper.RIGHT or their combination.</p>
     * You can combine them like ItemTouchHelper.UP | ItemTouchHelper.DOWN, it means that the item could only move up and down when dragged.
     *
     * @param dragMoveFlags the drag movement direction. Default value is ItemTouchHelper.UP | ItemTouchHelper.DOWN | ItemTouchHelper.LEFT | ItemTouchHelper.RIGHT.
     */
    public void setDragMoveFlags(int dragMoveFlags) {
        mDragMoveFlags = dragMoveFlags;
    }

    /**
     * <p>Set the swipe movement direction.</p>
     * <p>The value should be ItemTouchHelper.START, ItemTouchHelper.END or their combination.</p>
     * You can combine them like ItemTouchHelper.START | ItemTouchHelper.END, it means that the item could swipe to both left or right.
     *
     * @param swipeMoveFlags the swipe movement direction. Default value is ItemTouchHelper.END.
     */
    public void setSwipeMoveFlags(int swipeMoveFlags) {
        mSwipeMoveFlags = swipeMoveFlags;
    }

    @Override
    public void onChildDrawOver(Canvas c, RecyclerView recyclerView, RecyclerView.ViewHolder viewHolder,
                                float dX, float dY, int actionState, boolean isCurrentlyActive) {
        super.onChildDrawOver(c, recyclerView, viewHolder, dX, dY, actionState, isCurrentlyActive);

        if (actionState == ItemTouchHelper.ACTION_STATE_SWIPE
                && !isViewCreateByAdapter(viewHolder)) {
            View itemView = viewHolder.itemView;

            c.save();
            if (dX > 0) {
                c.clipRect(itemView.getLeft(), itemView.getTop(),
                        itemView.getLeft() + dX, itemView.getBottom());
                c.translate(itemView.getLeft(), itemView.getTop());
            } else {
                c.clipRect(itemView.getRight() + dX, itemView.getTop(),
                        itemView.getRight(), itemView.getBottom());
                c.translate(itemView.getRight() + dX, itemView.getTop());
            }

            mAdapter.onItemSwiping(c, viewHolder, dX, dY, isCurrentlyActive);
            c.restore();

        }
    }

    private boolean isViewCreateByAdapter(RecyclerView.ViewHolder viewHolder) {
        int type = viewHolder.getItemViewType();
        return type == BaseQuickAdapter.HEADER_VIEW || type == BaseQuickAdapter.LOADING_VIEW
                || type == BaseQuickAdapter.FOOTER_VIEW || type == BaseQuickAdapter.EMPTY_VIEW;
    }
}
