package com.urovo.activity.tabMenu.home.view;


import android.animation.Animator;
import android.animation.ValueAnimator;
import android.annotation.SuppressLint;
import android.content.Context;
import android.content.res.TypedArray;
import android.util.AttributeSet;
import android.util.Log;
import android.view.GestureDetector;
import android.view.MotionEvent;
import android.view.View;
import android.view.animation.DecelerateInterpolator;
import android.widget.RelativeLayout;

import com.urovo.activity.tabMenu.home.view.listener.OnItemClickListener;
import com.urovo.activity.tabMenu.home.view.listener.OnItemSelectedListener;
import com.urovo.activity.tabMenu.home.view.listener.OnLoopViewTouchListener;
import com.urovo.wugumofang.R;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;
import java.util.List;
import java.util.ListIterator;

/***
 * 水平旋转轮播控件
 * @author ...
 */
public class LoopRotarySwitchView extends RelativeLayout {
    /**
     * 默认半径大，再在布局中设置r即可保证一样大
     */
    private final static int LOOP_R = 600;
    /**
     * 竖直
     */
    @SuppressWarnings("unused")
    private final static int VERTICAL = 0;
    /**
     * 水平
     */
    private final static int HORIZONTAL = 1;
    /**
     * 方向
     */
    @SuppressWarnings("all")
    private int mOrientation = HORIZONTAL;
    /**
     * 上下文
     */
    @SuppressWarnings("all")
    private Context mContext;
    /**
     * 回位动画
     */
    private ValueAnimator restAnimator = null;
    /**
     * 半径动画
     */
    private ValueAnimator rAnimation = null;

    private ValueAnimator zAnimation = null;

    private ValueAnimator xAnimation = null;
    /**
     * x轴旋转和轴旋转，y轴无效果
     */
    private int loopRotationX = 0, loopRotationZ = 0;
    /**
     * 手势类
     */
    private GestureDetector mGestureDetector = null;
    /**
     * 当前选择项
     */
    private int selectItem = 0;
    /**
     * 个数
     */
    private int size = 0;
    /**
     * 半径
     */
    private float r = LOOP_R;
    /**
     * 倍数
     */
    private float multiple = 2f;
    /**
     * camera和观察的旋转物体距离， 距离越长,最大物体和最小物体比例越不明显
     */
    private float distance = multiple * r;
    /**
     * 旋转的角度
     */
    private float angle = 0;
    /**
     * 最后的角度，用来记录上一次取消touch之后的角度
     */
    private float lastAngle = 0;
    /**
     * 自动旋转
     */
    private boolean autoRotation = true;
    /**
     * 正在触摸
     */
    private boolean touching = false;
    /**
     * 默认自动滚动是从右往左
     */
    private AutoScrollDirection autoRotatinDirection = AutoScrollDirection.left;
    /**
     * 子view引用列表
     */
    private List<View> views = new ArrayList<>();
    /**
     * 选择事件接口
     */
    private OnItemSelectedListener onItemSelectedListener = null;
    /**
     * 选择事件接口
     */
    private OnLoopViewTouchListener onLoopViewTouchListener = null;
    /**
     * 被点击的回调
     */
    private OnItemClickListener onItemClickListener = null;
    /**
     * 是否可以点击回调
     */
    private boolean isCanClickListener = true;
    /**
     * 移动的x是否符合回调点击事件
     */
    private float x;
    /**
     * 滑动倒最低30
     */
    @SuppressWarnings("all")
    private float limitX = 30;

    private boolean isCanSwitchItem = true;

    public enum AutoScrollDirection {
        //自动滚动方向
        left, right
    }

    /**
     * 构造方法
     *
     * @param context ...
     */
    public LoopRotarySwitchView(Context context) {
        this(context, null);
    }

    /**
     * 构造方法
     *
     * @param context ...
     * @param attrs   ...
     */
    public LoopRotarySwitchView(Context context, AttributeSet attrs) {
        this(context, attrs, 0);
    }

    /**
     * 构造方法
     *
     * @param context      ...
     * @param attrs        ...
     * @param defStyleAttr ...
     */
    public LoopRotarySwitchView(Context context, AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
        this.mContext = context;
        TypedArray typedArray = context.obtainStyledAttributes(attrs, R.styleable.LoopRotarySwitchView);
        mOrientation = typedArray.getInt(R.styleable.LoopRotarySwitchView_orientation, HORIZONTAL);
        autoRotation = typedArray.getBoolean(R.styleable.LoopRotarySwitchView_autoRotation, false);
        r = typedArray.getDimension(R.styleable.LoopRotarySwitchView_r, LOOP_R);
        int direction = typedArray.getInt(R.styleable.LoopRotarySwitchView_direction, 0);
        typedArray.recycle();
        mGestureDetector = new GestureDetector(context, getGeomeryController());
        //如果是水平 z值为0  如果是竖直z值为90
        if (mOrientation == HORIZONTAL) {
            loopRotationZ = 0;
        } else {
            loopRotationZ = 90;
        }
        //设置自定滚动的方向
        if (direction == 0) {
            autoRotatinDirection = AutoScrollDirection.left;
        } else {
            autoRotatinDirection = AutoScrollDirection.right;
        }
        loopHandler.setLoop(autoRotation);
    }

    /**
     * handler处理
     */
    BaseLoopRotarySwitchViewHandler loopHandler = new BaseLoopRotarySwitchViewHandler(10000) {
        @Override
        public void doScroll() {
            try {
                //判断自动滑动从那边开始
                if (size != 0) {
                    int perAngle = 0;
                    switch (autoRotatinDirection) {
                        case left:
                            perAngle = 360 / size;
                            break;
                        case right:
                            perAngle = -360 / size;
                            break;
                        default:
                            break;
                    }
                    // 360 表示角度
                    if (angle == 360) {
                        angle = 0f;
                    }
                    AnimRotationTo(angle + perAngle, null);
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    };

    /**
     * 排序
     * 對子View 排序，然后根据变化选中是否重绘,这样是为了实现view 在显示的时候来控制当前要显示的是哪三个view，可以改变排序看下效果
     *
     * @param list ...
     */
    @SuppressWarnings("all")
    private <T> void sortList(List<View> list) {

        @SuppressWarnings("rawtypes")
        Comparator comparator = new SortComparator();
        T[] array = list.toArray((T[]) new Object[list.size()]);

        Arrays.sort(array, comparator);
        int i = 0;
        ListIterator<T> it = (ListIterator<T>) list.listIterator();
        while (it.hasNext()) {
            it.next();
            it.set(array[i++]);
        }
        for (int j = 0; j < list.size(); j++) {
            list.get(j).bringToFront();
        }
    }

    /**
     * 筛选器
     */
    private class SortComparator implements Comparator<View> {
        @Override
        public int compare(View lhs, View rhs) {
            int result = 0;
            try {
                result = (int) (1000 * lhs.getScaleX() - 1000 * rhs.getScaleX());
            } catch (Exception e) {// ...
            }
            return result;
        }
    }

    /**
     * 手势
     *
     * @return ...
     */
    private GestureDetector.SimpleOnGestureListener getGeomeryController() {
        return new GestureDetector.SimpleOnGestureListener() {
            @Override
            public boolean onScroll(MotionEvent e1, MotionEvent e2, float distanceX, float distanceY) {
                angle += Math.cos(Math.toRadians(loopRotationZ)) * (distanceX / 4)
                        + Math.sin(Math.toRadians(loopRotationZ)) * (distanceY / 4);
                initView();
                return true;
            }
        };
    }

    public void initView() {
        for (int i = 0; i < views.size(); i++) {
            double radians = angle + 180 - i * 360 / size;
            float x0 = (float) Math.sin(Math.toRadians(radians)) * r;
            float y0 = (float) Math.cos(Math.toRadians(radians)) * r;
            //计算子view之间的比例，可以看到distance越大的话 比例越小，也就是大小就相差越小
            float scale0 = (distance - y0) / (distance + r);
            //对view进行缩放
            views.get(i).setScaleX(Math.max(scale0, 0.5f));
            //对view进行缩放
            views.get(i).setScaleY(Math.max(scale0, 0.5f));

            views.get(i).setAlpha(Math.max(scale0, 0.5f));

            float rotationXY = (float) Math.sin(Math.toRadians(loopRotationX * Math.cos(Math.toRadians(radians)))) * r;
            float rotationZY = -(float) Math.sin(Math.toRadians(-loopRotationZ)) * x0;
            float rotationZX = (((float) Math.cos(Math.toRadians(-loopRotationZ)) * x0) - x0);
            views.get(i).setTranslationX(x0 + rotationZX);
            views.get(i).setTranslationY(rotationXY + rotationZY);
        }
        List<View> arrayViewList = new ArrayList<>();
        arrayViewList.clear();
//        for (int i = 0; i < views.size(); i++) {
//            arrayViewList.add(views.get(i));
//        }
        arrayViewList.addAll(views);
        sortList(arrayViewList);
        postInvalidate();
    }

    @Override
    protected void onSizeChanged(int w, int h, int oldw, int oldh) {
        super.onSizeChanged(w, h, oldw, oldh);
        initView();
        if (autoRotation) {
            loopHandler.sendEmptyMessageDelayed(BaseLoopRotarySwitchViewHandler.MSG_ID, loopHandler.loopTime);
        }
    }

    @Override
    protected void onLayout(boolean changed, int l, int t, int r, int b) {
        super.onLayout(changed, l, t, r, b);
        if (changed) {
            checkChildView();
            if (onItemSelectedListener != null) {
                isCanClickListener = true;
                onItemSelectedListener.selected(selectItem, views.get(selectItem));
            }
            rAnimation();
        }
    }

    public void rAnimation() {
        rAnimation(1f, r);
    }

    @SuppressWarnings("unused")
    public void rAnimation(boolean fromZeroToLoopR) {
        if (fromZeroToLoopR) {
            rAnimation(1f, LOOP_R);
        } else {
            rAnimation(LOOP_R, 1f);
        }
    }

    public void rAnimation(float from, float to) {
        rAnimation = ValueAnimator.ofFloat(from, to);
        rAnimation.addUpdateListener(new ValueAnimator.AnimatorUpdateListener() {
            @Override
            public void onAnimationUpdate(ValueAnimator valueAnimator) {
                r = (Float) valueAnimator.getAnimatedValue();
                initView();
            }
        });
        rAnimation.setInterpolator(new DecelerateInterpolator());
        rAnimation.setDuration(2000);
        rAnimation.start();
    }


    /**
     * 初始化view
     */
    public void checkChildView() {
        //先清空views里边可能存在的view防止重复
        for (int i = 0; i < views.size(); i++) {
            views.remove(i);
        }
        //获取子View的个数
        final int count = getChildCount();
        size = count;

        for (int i = 0; i < count; i++) {
            //获取指定的子view
            View view = getChildAt(i);
            final int position = i;
            views.add(view);
            view.setOnClickListener(new OnClickListener() {
                @Override
                public void onClick(View v) {
                    //对子view添加点击事件
                    if (position != selectItem) {
                        if (isCanSwitchItem) {
                            setSelectItem(position);
                        }
                    } else {
                        if (isCanClickListener && onItemClickListener != null) {
                            onItemClickListener.onItemClick(position, views.get(position));
                        }
                    }
                }
            });

        }

    }

    /**
     * 复位
     */
    private void restPosition() {
        if (size == 0) {
            return;
        }
        float finale;
        //一份的角度
        float part = 360 / size;
        if (angle < 0) {
            part = -part;
        }
        //最小角度
        float minValue = (int) (angle / part) * part;
        //最大角度
        float maxValue = (int) (angle / part) * part + part;
        //分为是否小于0的情况
        if (angle >= 0) {
            if (angle - lastAngle > 0) {
                finale = maxValue;
            } else {
                finale = minValue;
            }
        } else {
            if (angle - lastAngle < 0) {
                finale = maxValue;
            } else {
                finale = minValue;
            }
        }
        AnimRotationTo(finale, null);
    }


    /**
     * 动画
     *
     * @param finale   ...
     * @param complete ...
     */
    @SuppressWarnings("all")
    private void AnimRotationTo(float finale, final Runnable complete) {
        //如果相同说明不需要旋转
        if (angle == finale) {
            return;
        }
        restAnimator = ValueAnimator.ofFloat(angle, finale);
        //设置旋转减速插值器
        restAnimator.setInterpolator(new DecelerateInterpolator());
        restAnimator.setDuration(300);

        restAnimator.addUpdateListener(new ValueAnimator.AnimatorUpdateListener() {
            @Override
            public void onAnimationUpdate(ValueAnimator animation) {
                if (!touching) {
                    angle = (Float) animation.getAnimatedValue();
                    initView();
                }
            }
        });
        restAnimator.addListener(new Animator.AnimatorListener() {
            @Override
            public void onAnimationStart(Animator animation) {
                isCanSwitchItem = false;
            }

            @Override
            public void onAnimationEnd(Animator animation) {
                isCanSwitchItem = true;
                if (!touching) {
                    selectItem = calculateItem();
                    if (selectItem < 0) {
                        selectItem = size + selectItem;
                    }
                    if (onItemSelectedListener != null) {
                        onItemSelectedListener.selected(selectItem, views.get(selectItem));
                    }
                }
            }

            @Override
            public void onAnimationCancel(Animator animation) {

            }

            @Override
            public void onAnimationRepeat(Animator animation) {

            }
        });

        if (complete != null) {
            restAnimator.addListener(new Animator.AnimatorListener() {
                @Override
                public void onAnimationStart(Animator animation) {

                }

                @Override
                public void onAnimationEnd(Animator animation) {
                    complete.run();
                }

                @Override
                public void onAnimationCancel(Animator animation) {

                }

                @Override
                public void onAnimationRepeat(Animator animation) {

                }
            });
        }
        restAnimator.start();
    }

    /**
     * 通过角度计算是第几个item
     *
     * @return ...
     */
    private int calculateItem() {
        return (int) (angle / (360 / size)) % size;
    }

    /**
     * 触摸操作
     *
     * @param event ...
     * @return ...
     */
    @SuppressWarnings("all")
    private boolean onTouch(MotionEvent event) {
        if (event.getAction() == MotionEvent.ACTION_DOWN) {
            lastAngle = angle;
            touching = true;
        }
        boolean sc = mGestureDetector.onTouchEvent(event);
        if (sc) {
            this.getParent().requestDisallowInterceptTouchEvent(true);//通知父控件勿拦截本控件
        }
        if (event.getAction() == MotionEvent.ACTION_UP || event.getAction() == MotionEvent.ACTION_CANCEL) {
            touching = false;
            restPosition();
            return true;
        }
        return true;
    }


    /**
     * 触摸方法
     *
     * @param event ...
     * @return ...
     */
    @SuppressLint("ClickableViewAccessibility")
    @Override
    public boolean onTouchEvent(MotionEvent event) {
        if (onLoopViewTouchListener != null) {
            onLoopViewTouchListener.onTouch(event);
        }
        isCanClickListener(event);
        return true;
    }


    /**
     * 触摸停止计时器，抬起设置可下啦刷新
     */
    @Override
    public boolean dispatchTouchEvent(MotionEvent ev) {
        onTouch(ev);
        if (onLoopViewTouchListener != null) {
            onLoopViewTouchListener.onTouch(ev);
        }
        isCanClickListener(ev);
        return super.dispatchTouchEvent(ev);
    }

    /**
     * 是否可以点击回调
     *
     * @param event ...
     */
    public void isCanClickListener(MotionEvent event) {
        switch (event.getAction()) {
            case MotionEvent.ACTION_DOWN:
                x = event.getX();
                if (autoRotation) {
                    loopHandler.removeMessages(BaseLoopRotarySwitchViewHandler.MSG_ID);
                }
                break;
            case MotionEvent.ACTION_MOVE:
                break;
            case MotionEvent.ACTION_UP:
            case MotionEvent.ACTION_CANCEL:
                if (autoRotation) {
                    loopHandler.sendEmptyMessageDelayed(BaseLoopRotarySwitchViewHandler.MSG_ID, loopHandler.loopTime);
                }
                isCanClickListener = !(event.getX() - x > limitX || x - event.getX() > limitX);
                break;
            default:
                break;
        }
    }

    /**
     * 获取所有的view
     *
     * @return ...
     */
    @SuppressWarnings("unused")
    public List<View> getViews() {
        return views;
    }

    /**
     * 获取角度
     *
     * @return ...
     */
    @SuppressWarnings("unused")
    public float getAngle() {
        return angle;
    }


    /**
     * 设置角度
     *
     * @param angle ...
     */
    @SuppressWarnings("unused")
    public void setAngle(float angle) {
        this.angle = angle;
    }

    /**
     * 获取距离
     *
     * @return ...
     */
    @SuppressWarnings("unused")
    public float getDistance() {
        return distance;
    }

    /**
     * 设置距离
     *
     * @param distance ...
     */
    @SuppressWarnings("unused")
    public void setDistance(float distance) {
        this.distance = distance;
    }

    /**
     * 获取半径
     *
     * @return ...
     */
    public float getR() {
        return r;
    }

    /**
     * 获取选择是第几个item
     *
     * @return ...
     */
    @SuppressWarnings("unused")
    public int getSelectItem() {
        return selectItem;
    }

    int bottom = 0;

    /**
     * 设置指定位置
     *
     * @param pos ...
     */
    public void setSelectItem(int pos) {
        if (pos < 0 || pos >= views.size()) {
            return;
        }
        bottom = calculateItem();
        if (bottom == pos) {
            Log.e("setToPos", "bottom");
        }
        int difPos = bottom - pos;
        if (difPos < 0 - Math.floor(views.size() / 2)) {
            difPos += views.size();
        }
        if (difPos > Math.floor(views.size() / 2)) {
            difPos -= views.size();
        }
        if (views.size() % 2 == 0 && difPos == views.size() / 2) {
            difPos = views.size() / 2;
        }
        float angle1 = angle - difPos * 360 / views.size();
        AnimRotationTo(angle1, null);
    }


    /**
     * 设置半径
     *
     * @param r ...
     */
    public LoopRotarySwitchView setR(float r) {
        this.r = r;
        distance = multiple * r;
        return this;
    }

    /**
     * 选中回调接口实现
     *
     * @param onItemSelectedListener ...
     */
    public void setOnItemSelectedListener(OnItemSelectedListener onItemSelectedListener) {
        this.onItemSelectedListener = onItemSelectedListener;
    }

    /**
     * 点击事件回调
     *
     * @param onItemClickListener ...
     */
    public void setOnItemClickListener(OnItemClickListener onItemClickListener) {
        this.onItemClickListener = onItemClickListener;
    }

    /**
     * 触摸时间回调
     *
     * @param onLoopViewTouchListener ...
     */
    public void setOnLoopViewTouchListener(OnLoopViewTouchListener onLoopViewTouchListener) {
        this.onLoopViewTouchListener = onLoopViewTouchListener;
    }

    /**
     * 设置是否自动切换
     *
     * @param autoRotation ...
     */
    @SuppressWarnings("all")
    public LoopRotarySwitchView setAutoRotation(boolean autoRotation) {
        this.autoRotation = autoRotation;
        loopHandler.setLoop(autoRotation);
        return this;
    }

    /**
     * 获取自动切换时间
     *
     * @return ...
     */
    @SuppressWarnings("unused")
    public long getAutoRotationTime() {
        return loopHandler.loopTime;
    }

    /**
     * 设置自动切换时间间隔
     *
     * @param autoRotationTime ...
     */
    @SuppressWarnings("all")
    public LoopRotarySwitchView setAutoRotationTime(long autoRotationTime) {
        loopHandler.setLoopTime(autoRotationTime);
        return this;
    }

    /**
     * 是否自动切换
     *
     * @return ...
     */
    @SuppressWarnings("unused")
    public boolean isAutoRotation() {
        return autoRotation;
    }

    /**
     * 设置倍数
     *
     * @param mMultiple ...
     * @return ...
     */
    public LoopRotarySwitchView setMultiple(float mMultiple) {
        this.multiple = mMultiple;
        distance = multiple * r;
        return this;
    }

    public LoopRotarySwitchView setAutoScrollDirection(AutoScrollDirection mAutoScrollDirection) {
        this.autoRotatinDirection = mAutoScrollDirection;
        return this;
    }

    @SuppressWarnings("unused")
    public void createXAnimation(int from, int to, boolean start) {
        if (xAnimation != null) {
            if (xAnimation.isRunning()) {
                xAnimation.cancel();
            }
        }
        xAnimation = ValueAnimator.ofInt(from, to);
        xAnimation.addUpdateListener(new ValueAnimator.AnimatorUpdateListener() {
            @Override
            public void onAnimationUpdate(ValueAnimator animation) {
                loopRotationX = (Integer) animation.getAnimatedValue();
                initView();
            }
        });
        xAnimation.setInterpolator(new DecelerateInterpolator());
        xAnimation.setDuration(2000);
        if (start) {
            xAnimation.start();
        }
    }

    @SuppressWarnings("all")
    public ValueAnimator createZAnimation(int from, int to, boolean start) {
        if (zAnimation != null) {
            if (zAnimation.isRunning()) {
                zAnimation.cancel();
            }
        }
        zAnimation = ValueAnimator.ofInt(from, to);
        zAnimation.addUpdateListener(new ValueAnimator.AnimatorUpdateListener() {
            @Override
            public void onAnimationUpdate(ValueAnimator animation) {
                loopRotationZ = (Integer) animation.getAnimatedValue();
                initView();
            }
        });
        zAnimation.setInterpolator(new DecelerateInterpolator());
        zAnimation.setDuration(2000);
        if (start) {
            zAnimation.start();
        }
        return zAnimation;
    }

    /**
     * 设置方向
     *
     * @param mOrientation ...
     * @return ...
     */
    @SuppressWarnings("unused")
    public LoopRotarySwitchView setOrientation(int mOrientation) {
        setHorizontal(mOrientation == HORIZONTAL, false);
        return this;
    }

    @SuppressWarnings("all")
    public LoopRotarySwitchView setHorizontal(boolean horizontal, boolean anim) {
        if (anim) {
            if (horizontal) {
                createZAnimation(getLoopRotationZ(), 0, true);
            } else {
                createZAnimation(getLoopRotationZ(), 90, true);
            }
        } else {
            if (horizontal) {
                setLoopRotationZ(0);
            } else {
                setLoopRotationZ(90);
            }
            initView();
        }
        return this;
    }

    @SuppressWarnings("unused")
    public LoopRotarySwitchView setLoopRotationX(int loopRotationX) {
        this.loopRotationX = loopRotationX;
        return this;
    }

    @SuppressWarnings("all")
    public LoopRotarySwitchView setLoopRotationZ(int loopRotationZ) {
        this.loopRotationZ = loopRotationZ;
        return this;
    }

    @SuppressWarnings("unused")
    public int getLoopRotationX() {
        return loopRotationX;
    }

    @SuppressWarnings("unused")
    public int getLoopRotationZ() {
        return loopRotationZ;
    }

    @SuppressWarnings("unused")
    public ValueAnimator getRestAnimator() {
        return restAnimator;
    }

    @SuppressWarnings("unused")
    public ValueAnimator getrAnimation() {
        return rAnimation;
    }

    @SuppressWarnings("unused")
    public void setzAnimation(ValueAnimator zAnimation) {
        this.zAnimation = zAnimation;
    }

    @SuppressWarnings("unused")
    public ValueAnimator getzAnimation() {
        return zAnimation;
    }

    @SuppressWarnings("unused")
    public void setxAnimation(ValueAnimator xAnimation) {
        this.xAnimation = xAnimation;
    }

    @SuppressWarnings("unused")
    public ValueAnimator getxAnimation() {
        return xAnimation;
    }
}
