package com.urovo.activity.tabMenu.util.JPush

import android.content.Context
import android.content.Intent
import android.os.Bundle
import com.alibaba.android.arouter.launcher.ARouter
import com.google.gson.Gson
import com.google.gson.JsonParser
import com.urovo.activity.learning.LearningCenterDetailActivity
import com.urovo.activity.login.view.LoginActivity
import com.urovo.activity.machinerepairact.view.EngineerBillsAct
import com.urovo.activity.machinerepairact.view.MachineRepairAct
import com.urovo.activity.messagecenter.view.MsgDetailAct
import com.urovo.activity.tabMenu.util.ArticleExtraEntity
import com.urovo.activity.tabMenu.view.TabMenuActivity
import com.urovo.activity.user.PersonalAct
import com.urovo.wugumofang.constants.AppConstants
import com.urovo.wugumofang.constants.ConstantParams
import com.urovo.wugumofang.constants.RouterParams
import com.urovo.wugumofang.system.BaseAppManager
import com.urovo.wugumofang.system.SystemHelper
import com.urovo.wugumofang.system.ToActivity

/**
 * @author Sasuke on 2019-10-18 0018.
 * 利用ACTIVITY_TYPE来做跳转页面分支时的跳转处理封装
 */
class ActivityTypeDominator {
    companion object {
//        @JvmStatic
//        fun isParamJump(url: String) = "@" in url

        /***
         * 跟极光推送点击跳转的地方共用
         * 如果不是从极光推送过来的，那bundle就为空，以此来判断是否是极光推送过来的。
         *
         */
        @JvmStatic
        fun jumpNativeByJson(
            context: Context,
            json: String,
            notifyTitle: String? = "",
            bundle: Bundle? = null
        ) {
            val type = JsonParser().parse(json).getAsJsonObject().get("type")?.asInt ?: 1//默认1
            when (type) {
                //个人信息确认（如果在后台找到了该应用，那就直接把APP拉到前台，并正常弹出推送的Activity
//                5, 11 -> startActivity(context,if (moveAppToFront()) PersonalAct::class.java else LoginActivity::class.java)
                5, 11 -> startPersonInfoConfirmIntent(context, json)
                //学习中心、养生文库
                2, 3 -> {
                    if (SystemHelper.LoginUserId.isNullOrEmpty()) {
                        startActivity(context, LoginActivity::class.java, bundle)
                    } else {
                        val articleIntent = makeLearningCenterIntent(context, json/*notifyContent*/)
                        context.startActivity(articleIntent)
                    }
                }
                10 -> {//线上订单、会员商品
                    if (SystemHelper.LoginUserId.isNullOrEmpty()) {
                        startActivity(context, LoginActivity::class.java, bundle)
                    } else {
                        context.startActivity(
                            Intent(context, MsgDetailAct::class.java).apply {
                                putExtra("type", 10)
                                putExtra("title", notifyTitle)
                                addFlags(Intent.FLAG_ACTIVITY_NEW_TASK or Intent.FLAG_ACTIVITY_CLEAR_TOP)
                            }
                        )
                    }
                }
                14 -> {//【机修人员模式】未处理机修单
                    if (SystemHelper.LoginUserId.isNullOrEmpty()) {
                        startActivity(context, LoginActivity::class.java, bundle)
                    } else {
                        context.startActivity(
                            Intent(context, EngineerBillsAct::class.java).apply {
                                putExtra("FROM_JPUSH", bundle != null)
                                addFlags(Intent.FLAG_ACTIVITY_NEW_TASK or Intent.FLAG_ACTIVITY_CLEAR_TOP)
                            }
                        )
                    }
                }
                15 -> {//【机修营养顾问模式】已处理机修单
                    if (SystemHelper.LoginUserId.isNullOrEmpty()) {
                        startActivity(context, LoginActivity::class.java, bundle)
                    } else {
                        context.startActivity(
                            Intent(context, MachineRepairAct::class.java).apply {
                                putExtra("FROM_JPUSH", bundle != null)
                                addFlags(Intent.FLAG_ACTIVITY_NEW_TASK or Intent.FLAG_ACTIVITY_CLEAR_TOP)
                            }
                        )
                    }
                }
                16 -> { //消息 系统消息
                    if (SystemHelper.LoginUserId.isNullOrEmpty()) {
                        startActivity(context, LoginActivity::class.java, bundle)
                    } else {
                        context.startActivity(
                            Intent(context, TabMenuActivity::class.java).apply {
                                putExtra(
                                    AppConstants.ExtraKeys.JPUSH_TAB_INDEX,
                                    ConstantParams.CONSTANT_STARTS_3
                                )
                                putExtra( AppConstants.ExtraKeys.JPUSH_CENTER_DEFAULT_LABEL, ConstantParams.CONSTANT_STARTS_1)
                                addFlags(Intent.FLAG_ACTIVITY_NEW_TASK or Intent.FLAG_ACTIVITY_CLEAR_TOP)
                            }
                        )
                    }
                }
            }
        }

        private fun startActivity(context: Context, cls: Class<*>, bundle: Bundle?) {
            val intent = Intent(context, cls).apply {
                addFlags(Intent.FLAG_ACTIVITY_NEW_TASK or Intent.FLAG_ACTIVITY_CLEAR_TOP)
                bundle?.let { putExtras(it) }
            }
            context.startActivity(intent)
        }


        /**
         * 构造传给学习中心的Intent
         * 传的json需要：articleTitle，type，articleUrl，pid
         *
         * @param json ...
         */
        private fun makeLearningCenterIntent(context: Context, json: String): Intent {
            val title =
                JsonParser().parse(json).getAsJsonObject().get("articleTitle")?.asString ?: ""
            val articleExtra = Gson().fromJson(json, ArticleExtraEntity::class.java)
//            title = if (title == null) "" else title
            val intent = Intent()
            //打开自定义的Activity
            intent.apply {
                setClass(context, LearningCenterDetailActivity::class.java)
                putExtra(AppConstants.ExtraKeys.IS_FAVORITED_ARTICLE, false)//因为是新推送的文章所以不会被收藏
                putExtra(
                    AppConstants.ExtraKeys.ARTICLE_READ_TIMES_COUNT,
                    -1
                )//LeaningCenterDetailActivity里默认为0，所以这里传个-1去可以激起它去联网获取
                putExtra(AppConstants.ExtraKeys.ACTIVITY_TYPE, articleExtra.type)//2学习中心 3养生文库
                putExtra("detailTag", "content")//说明是内容列表，而不是收藏列表
            }
//            if (title.contains("】")) {//把前面的【学习中心】or【养生文库】字样给去掉再传到显示界面
//                val titleFront = title.split("】")[0] + "】"
//                title = title.replace(titleFront, "")
//            } else if (title.contains("]")) {
//                val titleFront = title.split("]")[0] + "]"
//                title = title.replace(titleFront, "")
//            }

            intent.putExtra("title", title)
//        intent.putExtra("articleUrl", String.format(NetParam.LEARNINGCENTER_JPUSH_URL,
//                articleExtra.getpid(), articleExtra.getType().toString()))
            intent.putExtra(
                "articleUrl",
                JsonParser().parse(json).getAsJsonObject().get("articleUrl")?.asString ?: ""
            )
            intent.putExtra(
                "articleId",
                JsonParser().parse(json).getAsJsonObject().get("pid")?.asString ?: ""
            )
            intent.flags = Intent.FLAG_ACTIVITY_NEW_TASK or Intent.FLAG_ACTIVITY_CLEAR_TOP
            return intent
        }


        /**
         * 个人信息确认跳转
         */
        private fun startPersonInfoConfirmIntent(context: Context, json: String) {
            BaseAppManager.loger().debug("makePersonInfoConfirmIntent json: " + json)
            val personalPushEntity = Gson().fromJson(json, ArticleExtraEntity::class.java)
            BaseAppManager.loger().info("[JPushReceiver] extraStr： " + personalPushEntity.getType())
            //        Intent userIntent = new Intent(context, UserActivity.class);
            val userIntent = Intent(context, PersonalAct::class.java)
//        val type = personalPushEntity.getType()
            //        userIntent.putExtra(AppConstants.ExtraKeys.PERSONAL_INFO_CONFIRM_TYPE_TAG, type);//传递参数，根据参数判断跳转的界面
            userIntent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK or Intent.FLAG_ACTIVITY_CLEAR_TOP)
            //如果在后台找到了该应用，那就直接把APP拉到前台，并正常弹出推送的Activity
            if (!SystemHelper.LoginUserId.isNullOrEmpty()) {
                //            个人信息确认
//            if (type == 4) {
                context.startActivity(userIntent)
//            }
            } else {//如果当前位于栈顶的页面的是登录界面，则弹出推送的界面为LoginActivity
                val loginIntent = Intent(context, LoginActivity::class.java)
                loginIntent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK or Intent.FLAG_ACTIVITY_CLEAR_TOP)
                context.startActivity(loginIntent)
            }
        }


        /**
         * 老式直接用@拼接式跳转，适用于不需要传参的情况。
         */
        @JvmStatic
        fun jumpNative(context: Context, link: String, error: (() -> Unit)? = null) {
            var realLink = link.trim()
            var param = ""
            //如果链接里带了参数，要分割出来
            if ("@" in link.trim() && link.trim().indexOf("@") != link.trim().length - 1) {
                realLink = link.trim().split("@")[0]
                param = link.trim().split("@")[1]
            }
            if (realLink.startsWith("com")) {
                try {
                    ToActivity.toActivity(
                        context, Class.forName(realLink),
                        SystemHelper.LoginOrganId,
                        SystemHelper.LoginUserName,
                        SystemHelper.LoginUserId,
                        param
                    )
                    //                        ToActivity.toActivity(context, Class.forName(clickLink.trim()));
                } catch (e: ClassNotFoundException) {
                    error?.invoke()
                }
            } else {
                ARouter.getInstance().build(link.trim())
                    .withString(ToActivity.LOGIN_ORGAN_ID, SystemHelper.LoginOrganId)
                    .withString(ToActivity.LOGIN_USER_NAME, SystemHelper.LoginUserName)
                    .withString(ToActivity.LOGIN_USER_ID, SystemHelper.LoginUserId)
                    .withString(AppConstants.ExtraKeys.ACTIVITY_TYPE, param)
                    .navigation()
            }
        }

    }
}