package com.urovo.activity.tabMenu.util.JPush

import android.content.Context
import android.os.Handler
import cn.jpush.android.api.JPushInterface
import cn.jpush.android.api.TagAliasCallback
import com.google.gson.Gson
import com.urovo.wgmf.AsyncTask.JPushYinXiaoBindCallback
import com.urovo.wgmf.AsyncTask.JPushYinXiaoBindTask
import com.urovo.wugumofang.constants.NetParam
import com.urovo.wugumofang.data.PrefTool
import com.urovo.wugumofang.manager.ThreadManager
import com.urovo.wugumofang.net.OkHttpDominator
import com.urovo.wugumofang.response.Base3Response
import com.urovo.wugumofang.system.BaseAppManager
import com.urovo.wugumofang.system.SystemHelper
import okhttp3.Request
import java.util.*

/**
 * @author Sasuke on 2019-05-10 0010.
 */
class JPushBindDominator(val context: Context,val userId: String) {
    /**
     * 为极光推送的设置别名和标签
     */
    fun setJPushAliasAndTag(isAdmin: Boolean) {
        //如果已绑定成功 并且存在本地的账号竟然不跟当前登陆账号相同！，就再重新绑定
        if (PrefTool.getStringPerferences(context, "JPushTagAccount", "") != userId) {
            //给极光后台绑
            val tags = HashSet<String>()
            //是不是营养顾问模式（目前只给营养顾问发推送）
            if (isAdmin) {
                //督导标签
                tags.add("admin")
            } else {
                //营养顾问标签
                tags.add("staff")
            }
            JPushInterface.setAliasAndTags(context, userId, tags, mAliasCallback)
        }

        if (PrefTool.getStringPerferences(context, "JPushYinXiaoBind", "") != userId) {
            //给公司后台绑
            val p = HashMap<String, String>()
            //是不是营养顾问模式（目前只给营养顾问发推送）
            if (isAdmin) {
                p["tag"] = "admin"
            } else {
                p["tag"] = "staff"
            }
            p["staffId"] = userId
            p["alias"] = userId
            p["platform"] = "android"
            p["type"] = "2"
            //别名绑完极光服务器再绑到公司服务器（绑定失败后每60秒回调一次JPushYinXiaoBindCallback）
            //            new Thread(new JPushYinXiaoBindTask(
            //                    this, NetParam.JPUSH_YINXIAO_BIND_PATH, p, new JPushYinXiaoBindCallback(this))).start();

            //          http://blog.csdn.net/u013851082/article/details/70211943

            //            ThreadFactory threadFactory = Executors.defaultThreadFactory();
            //            ExecutorService singleThreadPool = new ThreadPoolExecutor(1, 1,
            //                    0L, TimeUnit.MILLISECONDS,
            //                    new LinkedBlockingQueue<Runnable>(1024), threadFactory, new ThreadPoolExecutor.AbortPolicy());
            //            singleThreadPool.execute(new JPushYinXiaoBindTask(this, NetParam.JPUSH_YINXIAO_BIND_PATH, p, new JPushYinXiaoBindCallback(this)));
            //            singleThreadPool.shutdown();//执行完后计数为0时，就是main 线程的事情了

            ThreadManager.getThreadPool().execute(JPushYinXiaoBindTask(context, NetParam.JPUSH_YINXIAO_BIND_PATH, p, JPushYinXiaoBindCallback(context)))
        }
    }

    private val mAliasCallback = TagAliasCallback { responseCode, alias, tags ->
        when (responseCode) {
            //标签or别名 成功
            0 -> {
                BaseAppManager.loger().info("JPUSH:Set tag and alias success")
                //PrefTool.setBooleanSave(TabMenuActivity.this,"JPushTagSuccess",true);//存一个Prefs下次就不用绑了
                PrefTool.setStringsave(context, "JPushTagAccount", userId)
            }
            //绑定标签or别名失败
            6002 -> {
                BaseAppManager.loger().info("JPUSH:Failed to set alias and tags due to timeout. Try again after 60s.")
                //PrefTool.setBooleanSave(TabMenuActivity.this,"JPushTagSuccess",false);
                PrefTool.setStringsave(context, "JPushTagAccount", "")
                Handler().postDelayed({ setJPushAliasAndTag(SystemHelper.IsAdmin) }, 60000)//绑定标签or别名失败，60s后重新搞起
            }
            //极光于2020/03/10对【别名设置】的上限进行限制，最多允许绑定10个设备。超过10个设备时会报如下错
            // https://community.jiguang.cn/notice/197
            6027,6017,5 ->{
                BaseAppManager.loger().info("JPUSH: 别名设置超过上限，error = $responseCode")
                getRegistrationIds(userId)
//                Handler().postDelayed({ setJPushAliasAndTag(SystemHelper.IsAdmin) }, 60000)//绑定标签or别名失败，60s后重新搞起
            }
            else -> BaseAppManager.loger().info("JPUSH:Failed with errorCode = $responseCode")
        }
    }

    /**获取指定工号下绑定的极光id*/
    private fun getRegistrationIds(userId: String) {
        OkHttpDominator.getAsynAuth(NetParam.JPUSH_REG_IDS + userId,
                NetParam.JPUSH_APPKEY, NetParam.JPUSH_MASTER_SECRET,
                null,
                object : OkHttpDominator.ResultCallback<JPushRegIdsRes>() {
                    override fun onError(request: Request?, e: Exception?) {
                        BaseAppManager.loger().info("JPUSH: 查询绑定关系失败，现在重试绑定")
                        Handler().postDelayed({ setJPushAliasAndTag(SystemHelper.IsAdmin) }, 60000)//绑定标签or别名失败，60s后重新搞起
                    }

                    override fun onResponse(res: JPushRegIdsRes?) {
                        res?.registration_ids?.takeIf { it.size >= 10 }?.let {
                            //这个查出来的绑定list是从头插新记录进去的，所以尾部记录较旧，所以把尾部一半发回去给极光，让后台清除掉绑定
                            removeRegistrationIds(it.subList(5, 10))
//                            removeRegistrationIds(it)
                            BaseAppManager.loger().info("JPUSH: 找到现有绑定关系 - ${Gson().toJson(res)}")
                        }
                    }
                })
    }

    /**批量解除指定id们的绑定关系 */
    fun removeRegistrationIds(ids: List<String>){
        OkHttpDominator.postAsynAuth(NetParam.JPUSH_REG_IDS + userId,
                NetParam.JPUSH_APPKEY, NetParam.JPUSH_MASTER_SECRET,
                mapOf("极光解除id绑定关系JSON" to Gson().toJson(Registration(Registration.IdElement(ids)))),
                object : OkHttpDominator.ResultCallback<Base3Response>() {
                    override fun onError(request: Request?, e: Exception?) {
                        Handler().postDelayed({ setJPushAliasAndTag(SystemHelper.IsAdmin) }, 60000)//绑定标签or别名失败，60s后重新搞起
                    }

                    override fun onResponse(res: Base3Response?) {
                        BaseAppManager.loger().info("JPUSH: 已解除多余绑定关系，开始重新绑定本机}")
                        setJPushAliasAndTag(SystemHelper.IsAdmin)
                    }
                })
    }

    data class Registration(val registration_ids: IdElement){
        data class IdElement(val remove: List<String>)
    }

}