@file:JvmName("JPushCase")

package com.urovo.activity.tabMenu.util.JPush

import android.app.*
import android.content.Context
import android.content.Intent
import android.net.Uri
import android.os.Build
import android.os.Bundle
import android.os.SystemClock
import androidx.core.app.NotificationCompat
import cn.jpush.android.api.BasicPushNotificationBuilder
import cn.jpush.android.api.JPushInterface
import com.google.gson.JsonParser
import com.urovo.activity.login.view.LoginActivity
import com.urovo.activity.messagecenter.view.MsgDetailAct
import com.urovo.activity.resetpassword.view.ResetPasswordAct
import com.urovo.activity.resetpassword.view.VerifyAccountAct
import com.urovo.activity.tabMenu.view.GlobalDialogBridgeAct
import com.urovo.wugumofang.R
import com.urovo.wugumofang.constants.AppConstants
import com.urovo.wugumofang.system.SystemHelper

/**
 * @author Sasuke on 2018/2/27.
 * 极光推送相关处理
 */
class JPushCase(val context: Context, var bundle: Bundle) {
    val notifyTitle: String? by lazy {
        bundle.getString(JPushInterface.EXTRA_NOTIFICATION_TITLE)
    }
    val notifyContent: String? by lazy {
        bundle.getString(JPushInterface.EXTRA_ALERT)
    }
    val extras: String? by lazy {
        bundle.getString(JPushInterface.EXTRA_EXTRA)
    }
    val notifyId: Int by lazy {
        bundle.getInt(JPushInterface.EXTRA_NOTIFICATION_ID, System.currentTimeMillis().toInt())
    }

    companion object {
        /**
         * 安卓8.0+的系统上，需要创建通知渠道
         */
        @JvmStatic
        fun createNotificationChannel(context: Context) {
            //通知渠道只支持8.0或以上的
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
                val notiMgr = context.getSystemService(Context.NOTIFICATION_SERVICE) as NotificationManager

                val channel = NotificationChannel(
                        AppConstants.NOTI_CHANNEL_ID.RINGTONE_MONEY,
                        "会员商城提醒",
                        NotificationManager.IMPORTANCE_HIGH).apply {
                    setSound(Uri.parse("android.resource://" + context.packageName + "/" + R.raw.diaoluo_da), Notification.AUDIO_ATTRIBUTES_DEFAULT)
                    enableVibration(true)
                    vibrationPattern = longArrayOf(100, 200, 300)
                }

                notiMgr.createNotificationChannel(NotificationChannel(
                        AppConstants.NOTI_CHANNEL_ID.HIGH,
                        "重要通知",
                        NotificationManager.IMPORTANCE_HIGH))

                notiMgr.createNotificationChannel(NotificationChannel(
                        AppConstants.NOTI_CHANNEL_ID.DEFAULT,
                        "默认通知",
                        NotificationManager.IMPORTANCE_DEFAULT))

                notiMgr.createNotificationChannel(channel)
            }
        }

    }


    /**
     * 一收到通知就执行（需要点击跳转的不要在这里面做，请去onClicked()里面执行。
     * 因为如果在这里写了pendingIntent跳转，那就会把极光自己的Pending给覆盖了，
     * 然后在极光的JPushReceiver 里的“用户点击了通知”的广播接收里就接收不到点击广播了）
     */
    fun onReceived() {
        //json串(内含判断的标志type)
        if (notifyContent.isNullOrEmpty() || notifyTitle.isNullOrEmpty()) {
            return
        }

        if (!extras.isNullOrEmpty()) {
            val type = JsonParser().parse(extras).getAsJsonObject().get("type")?.asInt
            when (type) {
                //会员商城购买成功生意提醒（在onClicked()里也要写一次，因为在vivo手机里，必须在这里用pendingIntent，才能在通知栏里面点击跳转
                //                                          而onclicked()里的是用来在弹出的通知栏进行跳转用的。。不太懂有什么区别，反正就是如果想要两边都能点击跳转，那就要两边都要写。）
                10 -> {
                    //通知提醒后跳转的页面
                    val intent: Intent? =
                            if (!SystemHelper.LoginUserId.isNullOrEmpty()) Intent(context, MsgDetailAct::class.java).apply {
                                putExtra("type", 10)
                                putExtra("title", notifyTitle)
                            }
                            else Intent(context, LoginActivity::class.java)
                    val notificationBuilder = buildNewNotification(intent, AppConstants.NOTI_CHANNEL_ID.RINGTONE_MONEY)
//                    notificationBuilder.setSound(Uri.parse("android.resource://" + context.getPackageName() + "/" + R.raw.diaoluo_da))
                    (context.getSystemService(Context.NOTIFICATION_SERVICE) as NotificationManager).notify(notifyId, notificationBuilder.build())
                }
                9 -> {// 打卡提醒全局弹窗
                    val intent = Intent(context, GlobalDialogBridgeAct::class.java).apply {
                        putExtra(GlobalDialogBridgeAct.INTENT_EXTRA, GlobalDialogBridgeAct.RemindExtra(type.toString() + "", notifyTitle!!, notifyContent!!))
                        addFlags(Intent.FLAG_ACTIVITY_NEW_TASK or Intent.FLAG_ACTIVITY_CLEAR_TOP)
                    }
                    context.startActivity(intent)
                    (context.getSystemService(Context.NOTIFICATION_SERVICE) as NotificationManager).notify(notifyId,
                            buildNewNotification(intent, AppConstants.NOTI_CHANNEL_ID.DEFAULT).build())
                }
                else -> {
                }
            }
        }

    }

    /**
     * 用来设置自定义铃声
     */
    fun buildNewNotification(intent: Intent?, channelId: String): NotificationCompat.Builder {
        //前提是首先设置极光默认没有铃声
        JPushInterface.setDefaultPushNotificationBuilder(BasicPushNotificationBuilder(context).apply {
            notificationDefaults = Notification.DEFAULT_LIGHTS or Notification.DEFAULT_VIBRATE
        })
        //添加自定义声音
        return NotificationCompat.Builder(context, channelId).apply {
            setAutoCancel(true)
            //通知内容
            setContentText(notifyContent)
            //通知标题
            setContentTitle(notifyTitle)
            //默认图标
            setSmallIcon(R.drawable.jpush_notification_icon)
            //设置铃声
            setSound(Uri.parse("android.resource://" + context.packageName + "/" + R.raw.diaoluo_da))
            //点击跳转
            intent?.let { setContentIntent(PendingIntent.getActivity(context, SystemClock.currentThreadTimeMillis().toInt(), intent, PendingIntent.FLAG_CANCEL_CURRENT)) }
        }
    }


    /**
     * 点击通知
     */
    fun onClicked() {
        if (!extras.isNullOrEmpty()) {
            ActivityTypeDominator.jumpNativeByJson(context, extras!!, notifyTitle, bundle)
        }
    }


    /**
     * 将APP置于前台（如果当APP不在前台时点击了通知，就执行这个）
     */
    private fun moveAppToFront(): Boolean {
        //获取ActivityManager
        val mAm = context.getSystemService(Context.ACTIVITY_SERVICE) as ActivityManager
        //获得当前运行的task
        val taskList = mAm.getRunningTasks(100)
        for (rti in taskList) {
            //找到当前应用的task，并启动task的栈顶activity，达到程序切换到前台
            //栈顶若是登陆、找回密码，都判定为“未登陆”
            if (rti.topActivity?.packageName == context.packageName
                    && rti.topActivity?.className != LoginActivity::class.java.name
                    && rti.topActivity?.className != ResetPasswordAct::class.java.name
                    && rti.topActivity?.className != VerifyAccountAct::class.java.name) {
                mAm.moveTaskToFront(rti.id, 0)
                return true
            }
        }
        //若没有找到运行的task，用户结束了task或被系统释放
        return false
    }


//    private fun startActivityPending(context: Context, cls: Class<*>, bundle: Bundle, notification: NotificationCompat.Builder) {
//        val loginIntent = Intent(context, cls).apply {
//            addFlags(Intent.FLAG_ACTIVITY_NEW_TASK or Intent.FLAG_ACTIVITY_CLEAR_TOP)
//            putExtras(bundle)
//        }
//        notification.setContentIntent(PendingIntent.getActivity(context, notifyId, loginIntent, PendingIntent.FLAG_UPDATE_CURRENT or PendingIntent.FLAG_ONE_SHOT))
//    }
}