package com.urovo.activity.user;

import android.graphics.Color;
import android.graphics.drawable.Drawable;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.RelativeLayout;
import android.widget.TextView;
import android.widget.Toast;

import com.google.gson.Gson;
import com.urovo.activity.user.adapter.LeaveApplicationFormAdapter;
import com.urovo.activity.user.bean.LeaveApplicationBean;
import com.urovo.wugumofang.R;
import com.urovo.wugumofang.response.BaseResponse;

import java.util.ArrayList;
import java.util.List;

import androidx.recyclerview.widget.RecyclerView;
import butterknife.BindView;
import butterknife.ButterKnife;
import butterknife.OnClick;
import butterknife.Unbinder;


/**
 * Created by chenji on 2017/8/8.
 * 离职申请
 */

public class LeaveApplicationFormFragment extends BackHandledFragment implements UserContract.view, LeaveApplicationFormAdapter.OnChangeBackGroundListener {
    @BindView(R.id.recyclerview)
    RecyclerView recyclerView;
    @BindView(R.id.user_name)
    RelativeLayout mUserName;
    @BindView(R.id.entry_data)
    RelativeLayout mEntryData;
    @BindView(R.id.term_date)
    RelativeLayout mTermDate;
    @BindView(R.id.commit_check)
    Button mCommitCheckBtn;
    private LeaveApplicationBean leaveApplicationBean;
    private LeaveApplicationFormAdapter adapter;
    private List<LeaveApplicationBean.LeavingReasonBean> datas = new ArrayList<>();
    private String[] leaveReasons = {
            "工作压力",
            "工作环境",
            "工作氛围",
            "薪资问题",
            "个人发展",
            "与上级相处问题",
            "卖场原因",
            "个人身体原因",
            "家庭原因",
            "其他"
    };
    Unbinder unbinder;
    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        View view = inflater.inflate(R.layout.fragment_leave_application_form, container, false);
        unbinder = ButterKnife.bind(this, view);
        selfCenterListPresenter = new UserPresenter(this);
        selfCenterListPresenter.start();
        view.setClickable(true);// 防止点击穿透，底层的fragment响应上层点击触摸事件
        return view;
    }

    @OnClick(R.id.commit_check)
    public void onCommitCheck() {
        if (!adapter.getSeletorData().isEmpty()) {
            if (adapter.isCommitFlag()) {//为其他有内容
                leaveApplicationBean.setName("王小二");//
                leaveApplicationBean.setEntryData("2015-12-28");
                leaveApplicationBean.setTermDate("2017-08-14");
                leaveApplicationBean.setLeavingReason(adapter.getSeletorData());
                String commitInfo = new Gson().toJson(leaveApplicationBean);//  提交到主管的离职信息数据
                Toast.makeText(getActivity(), "提交成功！", Toast.LENGTH_SHORT).show();
//                Log.d(TAG, "onCommitCheck: " + commitInfo);
                adapter.clearAll();
            } else {
                Toast.makeText(getActivity(), "“其他”选项不能为空", Toast.LENGTH_SHORT).show();
            }
        } else {
            Toast.makeText(getActivity(), "你还没有勾选离职原因！", Toast.LENGTH_SHORT).show();
        }
    }

    @Override
    public void init() {
        leaveApplicationBean = new LeaveApplicationBean();
        TextView mUsersTitle = (TextView) mUserName.findViewById(R.id.title);
        TextView mUsersName = (TextView) mUserName.findViewById(R.id.name);
        TextView mEntryDataTitle = (TextView) mEntryData.findViewById(R.id.title);
        TextView mEntryDataName = (TextView) mEntryData.findViewById(R.id.name);
        TextView mTermDateTitle = (TextView) mTermDate.findViewById(R.id.title);
        TextView mTermDateName = (TextView) mTermDate.findViewById(R.id.name);

        mUsersTitle.setText("姓名");
        mUsersName.setText("王小二");//
        mEntryDataTitle.setText("入职日期");
        mEntryDataName.setText("2015-12-28");
        mTermDateTitle.setText("选择拟离职日期");
        mTermDateName.setText("2017-08-14");

//      箭头
        Drawable drawable = getResources().getDrawable(R.drawable.down_arrows);
        drawable.setBounds(0, 0, drawable.getMinimumWidth(), drawable.getMinimumHeight());
        mTermDateName.setCompoundDrawables(null, null, drawable, null);

        mTermDateName.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                Toast.makeText(getActivity(), "选择拟离职日期", Toast.LENGTH_SHORT).show();
            }
        });

//      添加离职列表
        for (String leaveReason : leaveReasons) {
            datas.add(new LeaveApplicationBean.LeavingReasonBean(leaveReason, ""));
        }
        adapter = new LeaveApplicationFormAdapter(getActivity(), datas);
        adapter.setOnChangeBackGroundListener(LeaveApplicationFormFragment.this);
        recyclerView.setLayoutManager(new CustomLinearLayoutManager(getActivity()));
        recyclerView.setAdapter(adapter);
        recyclerView.setHasFixedSize(true);
    }

    @Override
    public void refresh(String tag, BaseResponse response) {

    }

    @Override
    protected boolean onBackPressed() {
        return false;
    }

    @Override
    public void onDestroyView() {
        super.onDestroyView();
//        ButterKnife.unbind(this);
        unbinder.unbind();
    }

    /**
     * 提交按钮颜色变化回调
     *
     * @param flag true:勾选CheckBox；false：未勾选CheckBox
     */
    @Override
    public void onChageBackGround(boolean flag) {
        mCommitCheckBtn.setBackgroundColor(Color.parseColor(flag ? "#F4B505" : "#ADADAD"));
    }
}
