package com.urovo.activity.user;

import android.content.Context;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.TextView;

import androidx.annotation.StringRes;

import com.urovo.wgmf.db.dao.entitys.PersonalEntity;
import com.urovo.wugumofang.R;
import com.urovo.wugumofang.base.BaseActivity;
import com.urovo.wugumofang.constants.NetParam;
import com.urovo.wugumofang.net.NetHelper;
import com.urovo.wugumofang.net.OkHttpDominator;
import com.urovo.wugumofang.response.Base3Response;
import com.urovo.wugumofang.response.BaseResponse;
import com.urovo.wugumofang.system.SystemHelper;
import com.urovo.wugumofang.ui.ViewUtil;

import java.util.HashMap;
import java.util.List;

import butterknife.BindView;
import butterknife.ButterKnife;
import butterknife.Unbinder;
import okhttp3.Request;

/**
 * 作者：xiep on 2017/3/13 09:10
 * 邮箱：2975837870@qq.com
 *
 * @author ...
 */

public class PersonalAct extends BaseActivity implements UserContract.view, View.OnClickListener {
    @BindView(R.id.lv_user_content)
    ListView mLvUserContent;
    @BindView(R.id.cb_user_confirm)
    CheckBox mCheckBox;
    @BindView(R.id.btn_user_commit)
    Button mCommitBtn;
    /**
     * 用户点击按钮后显示的信息
     */
    @BindView(R.id.tv_user_confirm)
    TextView mConFirmInfo;
    /**
     * 隐藏显示list列表下的其他控件
     */
    @BindView(R.id.ll_user_container)
    LinearLayout mUserContainer;
    /**
     * 后台返回用户提交信息的日期
     */
    private String mDataFormat;
    Unbinder unbinder;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_user_info_confirm);
        unbinder = ButterKnife.bind(this);
        initTitle();
        UserPresenter selfCenterListPresenter = new UserPresenter(this);
        selfCenterListPresenter.start();
    }

    @Override
    public void init() {
        //网络连接检查
        if (NetHelper.IsHaveInternet(this)) {
            getPersonalInfoList();
        } else {
            showMsg("网络连接超时，请检查您的网络状态，稍后重试！");
        }
        mCheckBox.setOnClickListener(this);
        mCommitBtn.setOnClickListener(this);
    }

    /**
     * 用户确认信息列表任务
     */
    private void getPersonalInfoList() {
        ViewUtil.showProgress(this, "正在加载...", "请稍后...", false);
        HashMap<String, String> param = new HashMap<>(1);
        param.put("userNo", SystemHelper.LoginUserId);
        OkHttpDominator.postAsyn(NetParam.PERSONALINFO, param, new OkHttpDominator.ResultCallback<PersonalEntity>() {
            @Override
            public void onError(Request request, Exception e) {
                ViewUtil.dismissProgress();
                showMsg(R.string.ser_access_err);
            }

            @Override
            public void onResponse(PersonalEntity response) {
                if (null != response) {
                    //请求成功。
                    ViewUtil.dismissProgress();
                    dealPersonalInfo(response);
                }
            }
        });
    }

    /**
     * Toast
     *
     * @param resId Toast的内容id
     */
    private void showMsg(@StringRes int resId) {
        showMsg(getString(resId));
    }

    /**
     * 用户提交确认信息任务
     */
    private void getPersonalInfoCommit() {
        ViewUtil.showProgress(this, "正在加载...", "请稍后...", false);
        HashMap<String, String> param = new HashMap<>(1);
        param.put("userNo", SystemHelper.LoginUserId);
        OkHttpDominator.postAsyn(NetParam.PERSONALINFOCONFIRM, param, new OkHttpDominator.ResultCallback<Base3Response>() {
            @Override
            public void onError(Request request, Exception e) {
                ViewUtil.dismissProgress();
                showMsg(R.string.ser_access_err);
            }

            @Override
            public void onResponse(Base3Response response) {
                if (null != response) {
                    //请求成功。
                    ViewUtil.dismissProgress();
                    dealPersonalInfoConfirm(response);
                }
            }
        });
    }

    private void dealPersonalInfo(PersonalEntity response) {
        if (null != response && null != response.msgInfo) {
            PersonalEntity.ResultInfoEntity resultInfo = response.getResultInfo();
            if (response.msgCode == 0
                    && null != resultInfo
//                    && null != resultInfo.getConfirmStatus()
                    ) {
                mDataFormat = getDataFormat((String) resultInfo.getConfirmDate());
                //“-1”：用户没有提交确认信息；“0”：已经提交确认信息
                String mNoSubmit = "-1";
                if (mNoSubmit.equals(resultInfo.getConfirmStatus())) {
                    mUserContainer.setVisibility(View.VISIBLE);
                } else {
                    mUserContainer.setVisibility(View.GONE);
                    mConFirmInfo.setText(mDataFormat);
                    mConFirmInfo.setVisibility(View.VISIBLE);
                }
                mLvUserContent.setAdapter(new PersonalAdapter<>(resultInfo.getPersonalInfoList(), PersonalAct.this));

            } else {
                showMsg(response.msgInfo);
                finish();
//                    Toast.makeText(getActivity(), "OE后台暂无该工号的信息返回", Toast.LENGTH_LONG).show();
//                    getActivity().onBackPressed();
            }
        }
    }
//}

    private void dealPersonalInfoConfirm(Base3Response response) {
        if (null != response && response.msgCode == 0) {
            mUserContainer.setVisibility(View.GONE);
            mConFirmInfo.setText(mDataFormat);
            mConFirmInfo.setVisibility(View.VISIBLE);
            mCheckBox.setClickable(false);
        } else {
            showMsg("员工资料更改确认失败！请稍后再试！");
        }
    }

    private String getDataFormat(String format) {
        return String.format("以上信息你已于%s确认过，无需再次确认！", format);
    }

    @Override
    public void refresh(String tag, BaseResponse response) {
//        super.refresh(tag, response);
    }

    @Override
    public void onDestroy() {
        unbinder.unbind();
        super.onDestroy();
    }

    @Override
    public void initTitle() {
        super.initTitle();
        titleTextView.setText("个人信息确认");
    }

    @Override
    public void onClick(View v) {
        switch (v.getId()) {
            //CheckBox
            case R.id.cb_user_confirm:
                commitStatus(mCheckBox.isChecked());
                break;
            //CommitBtn
            case R.id.btn_user_commit:
                if (mCheckBox.isChecked()) {
                    getPersonalInfoCommit();
                }
                break;
            default:
                break;
        }
    }

    /**
     * 确认的状态
     *
     * @param flag ...
     */
    private void commitStatus(boolean flag) {
        mCommitBtn.setBackgroundResource(flag ? R.drawable.orange_user_background : R.drawable.gray_userl_background);
    }

    @Override
    public void setPresenter(UserContract.presenter presenter) {

    }

    private class ViewHolder {
        TextView title;
        TextView content;
    }

    private class PersonalAdapter<T> extends BaseAdapter {
        private List<T> mLists;
        private Context mContext;

        private PersonalAdapter(List<T> lists, Context context) {
            super();
            mLists = lists;
            mContext = context;
        }

        @Override
        public int getCount() {
            return mLists == null ? 0 : mLists.size();
        }

        @Override
        public Object getItem(int position) {
            return mLists.get(position);
        }

        @Override
        public long getItemId(int position) {
            return position;
        }

        @Override
        public View getView(int position, View convertView, ViewGroup parent) {
            final ViewHolder mViewHolder;
            if (convertView == null) {
                convertView = LayoutInflater.from(mContext).inflate(R.layout.listview_person_info_item, parent, false);
                mViewHolder = new ViewHolder();
                mViewHolder.title = (TextView) convertView.findViewById(R.id.textViewTitle);
                mViewHolder.content = (TextView) convertView.findViewById(R.id.textViewContent);
                convertView.setTag(mViewHolder);
            } else {
                mViewHolder = (ViewHolder) convertView.getTag();
            }
            PersonalEntity.ResultInfoEntity.PersonalInfoListEntity entity =
                    (PersonalEntity.ResultInfoEntity.PersonalInfoListEntity) mLists.get(position);
            mViewHolder.title.setText(entity.getKey());
            mViewHolder.content.setText(entity.getValue());
            return convertView;
        }
    }
}
