package com.urovo.activity.user;

import android.content.Context;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.TextView;

import androidx.annotation.StringRes;

import com.urovo.wgmf.db.dao.entitys.PersonalEntity;
import com.urovo.wugumofang.R;
import com.urovo.wugumofang.constants.NetParam;
import com.urovo.wugumofang.net.NetHelper;
import com.urovo.wugumofang.net.OkHttpDominator;
import com.urovo.wugumofang.response.Base3Response;
import com.urovo.wugumofang.response.BaseResponse;
import com.urovo.wugumofang.system.SystemHelper;
import com.urovo.wugumofang.ui.ViewUtil;

import java.util.HashMap;
import java.util.List;

import butterknife.BindView;
import butterknife.ButterKnife;
import butterknife.Unbinder;
import okhttp3.Request;

/**
 * 作者：xiep on 2017/3/13 09:10
 * 邮箱：2975837870@qq.com
 * @author ...
 */

public class PersonalFragment extends BackHandledFragment implements UserContract.view, View.OnClickListener {
    @BindView(R.id.lv_user_content)
    ListView mLvUserContent;
    @BindView(R.id.cb_user_confirm)
    CheckBox mCheckBox;
    @BindView(R.id.btn_user_commit)
    Button mCommitBtn;
    @BindView(R.id.tv_user_confirm)
    TextView mConFirmInfo;//用户点击按钮后显示的信息
    @BindView(R.id.ll_user_container)
    LinearLayout mUserContainer;//隐藏显示list列表下的其他控件
    private String mDataFormat;//后台返回用户提交信息的日期
    Unbinder unbinder;
    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        View view = inflater.inflate(R.layout.activity_user_info_confirm, container, false);
        unbinder = ButterKnife.bind(this, view);
        selfCenterListPresenter = new UserPresenter(this);
        selfCenterListPresenter.start();
        view.setClickable(true);// 防止点击穿透，底层的fragment响应上层点击触摸事件
        return view;
    }

    @Override
    public void init() {
//        super.init();
        if (NetHelper.IsHaveInternet(getActivity())) {//网络连接检查
            getPersonalInfoList();
        } else {
            showMsg("网络连接超时，请检查您的网络状态，稍后重试！");
        }
        mCheckBox.setOnClickListener(this);
        mCommitBtn.setOnClickListener(this);
    }

    /**
     * 用户确认信息列表任务
     */
    private void getPersonalInfoList() {
        ViewUtil.showProgress(getActivity(), "正在加载...", "请稍后...", false);
        HashMap<String, String> param = new HashMap<>();
        param.put("userNo", SystemHelper.LoginUserId);//post 请求字段
        OkHttpDominator.postAsyn(NetParam.PERSONALINFO, param, new OkHttpDominator.ResultCallback<PersonalEntity>() {
            @Override
            public void onError(Request request, Exception e) {
                ViewUtil.dismissProgress();
                showMsg(R.string.ser_access_err);
            }

            @Override
            public void onResponse(PersonalEntity response) {
                if (null != response) {
                    ViewUtil.dismissProgress();//请求成功。
                    dealPersonalInfo(response);
                }
            }
        });
    }

    /**
     * Toast
     *
     * @param resId Toast的内容id
     */
    private void showMsg(@StringRes int resId) {
        if (getActivity() != null) {
            showMsg(getActivity().getString(resId));
        }
    }

    /**
     * 用户提交确认信息任务
     */
    private void getPersonalInfoCommit() {
        ViewUtil.showProgress(getActivity(), "正在加载...", "请稍后...", false);
        HashMap<String, String> param = new HashMap<>();
        param.put("userNo", SystemHelper.LoginUserId);//post 请求字段
        OkHttpDominator.postAsyn(NetParam.PERSONALINFOCONFIRM, param, new OkHttpDominator.ResultCallback<Base3Response>() {
            @Override
            public void onError(Request request, Exception e) {
                ViewUtil.dismissProgress();
                showMsg(R.string.ser_access_err);
            }

            @Override
            public void onResponse(Base3Response response) {
                if (null != response) {
                    ViewUtil.dismissProgress();//请求成功。
                    dealPersonalInfoConfirm(response);
                }
            }
        });
    }

    private void dealPersonalInfo(PersonalEntity response) {
        if (null != response && null != response.msgInfo) {
            PersonalEntity.ResultInfoEntity resultInfo = response.getResultInfo();
            if (response.msgCode == 0
                    && null != resultInfo
                    && null != resultInfo.getConfirmDate()
                    && null != resultInfo.getConfirmStatus()
                    && null != mLvUserContent
                    && null != mUserContainer
                    && null != mConFirmInfo) {
                mDataFormat = getDataFormat((String) resultInfo.getConfirmDate());
                if (resultInfo.getConfirmStatus().equals("-1")) {//“-1”：用户没有提交确认信息；“0”：已经提交确认信息
                    mUserContainer.setVisibility(View.VISIBLE);
                } else {
                    mUserContainer.setVisibility(View.GONE);
                    mConFirmInfo.setText(mDataFormat);
                    mConFirmInfo.setVisibility(View.VISIBLE);
                }
                mLvUserContent.setAdapter(new PersonalAdapter<>(resultInfo.getPersonalInfoList(), getActivity()));

            } else {
                if (getActivity() != null) {
                    showMsg(response.msgInfo);
//                    Toast.makeText(getActivity(), "OE后台暂无该工号的信息返回", Toast.LENGTH_LONG).show();
                    getActivity().onBackPressed();
                }
            }
        }
    }

    private void dealPersonalInfoConfirm(Base3Response response) {
        if (null != response && response.msgCode == 0) {
            mUserContainer.setVisibility(View.GONE);
            mConFirmInfo.setText(mDataFormat);
            mConFirmInfo.setVisibility(View.VISIBLE);
            mCheckBox.setClickable(false);
        } else {
            showMsg("员工资料更改确认失败！请稍后再试！");
        }
    }


    private String getDataFormat(String format) {
        return String.format("以上信息你已于%s确认过，无需再次确认！", format);
    }

    @Override
    public void refresh(String tag, BaseResponse response) {
//        super.refresh(tag, response);
    }

    @Override
    public void onDestroyView() {
//        ButterKnife.unbind(this);
        unbinder.unbind();
        super.onDestroyView();
    }

    @Override
    protected boolean onBackPressed() {
        return false;
    }

    @Override
    public void onClick(View v) {
        switch (v.getId()) {
            case R.id.cb_user_confirm://CheckBox
                commitStatus(mCheckBox.isChecked());
                break;
            case R.id.btn_user_commit://CommitBtn
                if (mCheckBox.isChecked()) {
                    getPersonalInfoCommit();
                }
                break;
            default:
                break;
        }
    }

    private void commitStatus(boolean flag) {
        mCommitBtn.setBackgroundResource(flag ?  R.drawable.orange_user_background :  R.drawable.gray_userl_background);
    }

    private class ViewHolder {
        TextView title;
        TextView content;
    }

    private class PersonalAdapter<T> extends BaseAdapter {
        private List<T> mLists;
        private Context mContext;

        private PersonalAdapter(List<T> lists, Context context) {
            super();
            mLists = lists;
            mContext = context;
        }

        @Override
        public int getCount() {
            return mLists == null ? 0 : mLists.size();
        }

        @Override
        public Object getItem(int position) {
            return mLists.get(position);
        }

        @Override
        public long getItemId(int position) {
            return position;
        }

        @Override
        public View getView(int position, View convertView, ViewGroup parent) {
            final ViewHolder mViewHolder;
            if (convertView == null) {
                convertView = LayoutInflater.from(mContext).inflate(R.layout.listview_person_info_item, parent, false);
                mViewHolder = new ViewHolder();
                mViewHolder.title = (TextView) convertView.findViewById(R.id.textViewTitle);
                mViewHolder.content = (TextView) convertView.findViewById(R.id.textViewContent);
                convertView.setTag(mViewHolder);
            } else {
                mViewHolder = (ViewHolder) convertView.getTag();
            }
            PersonalEntity.ResultInfoEntity.PersonalInfoListEntity entity =
                    (PersonalEntity.ResultInfoEntity.PersonalInfoListEntity) mLists.get(position);
            mViewHolder.title.setText(entity.getKey());
            mViewHolder.content.setText(entity.getValue());
            return convertView;
        }
    }
}
