package com.urovo.activity.user;

import android.text.TextUtils;

import com.urovo.activity.user.bean.ShopDataRes;
import com.urovo.wgmf.db.dao.dao.PrivilegeDbOperate;
import com.urovo.wgmf.entity.response.NianJiaResponse;
import com.urovo.wugumofang.constants.ApiUrls;
import com.urovo.wugumofang.constants.AppConstants;
import com.urovo.wugumofang.constants.NetParam;
import com.urovo.wugumofang.net.OkHttpDominator;
import com.urovo.wugumofang.response.BaseResponse;
import com.urovo.wugumofang.system.SystemHelper;
import com.urovo.wugumofang.ui.ViewUtil;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import okhttp3.Request;

/**
 * 作者：xiep on 2017/3/2 14:48
 * 邮箱：2975837870@qq.com
 */

public class UserPresenter implements UserContract.presenter {

    private UserContract.view selfCenterListView;
    public ShopDataRes shopDataRes;

    public class FunItem {
        FunItem(String funName, boolean isShow) {
            this.funName = funName;
            this.isShow = isShow;
        }

        /**
         * 显示名字
         */
        public String funName;
        /**
         * 是否显示（配合后台动态显示）
         */
        public boolean isShow;
    }

    /**
     * 薪资查询中列表item的权限
     */
    private String[] functionPrivilege = {
            AppConstants.More.NATIVE_OFFICIAL_SALARY_QUERY,
            AppConstants.More.URL_REAL_TIME_SALARY_QUERY,
            AppConstants.More.URL_SALARY_CALCULATORS,
            AppConstants.More.NATIVE_FLOW_SALES_PERSON_COMPLETION,
            AppConstants.More.NATIVE_YEAR_AWARD,
            AppConstants.More.URL_STORE_PROFIT,
            AppConstants.More.URL_UNION_OPERATION
    };

    /**
     * 存储有权限的薪资查询中列表item
     */
    private List<FunItem> mFunItems = new ArrayList<>();

    /**
     * 薪资查询中列表item
     */
    public FunItem[] functionNames = {
            new FunItem("正式薪资查询", true),
            new FunItem("本月薪资预估", true),
            new FunItem("薪资计算器", true),
            new FunItem("流动顾问销售完成情况", true),
            new FunItem("年终奖查询", true),
            new FunItem("门店利润查询", false),//默认不显示，后台返回判断显示才显示
            new FunItem("联营门店利润查询", false)//默认不显示，后台返回判断显示才显示


//            "年假查询",
//            "个人信息确认",
//            "离职申请",
//            "离职申请查询",
//            "常见问题解答"
    };

    @Override
    public void start() {
        selfCenterListView.init();
    }

    UserPresenter(UserContract.view mView) {
        this.selfCenterListView = mView;
        mView.setPresenter(this);
    }

    public List<FunItem> getFunctions() {
        //过滤掉没有权限的
        mFunItems.clear();
        for (int i = 0; i < functionNames.length; i++) {
            if (PrivilegeDbOperate.checkUserPrivilege(functionPrivilege[i])) {
                mFunItems.add(functionNames[i]);
            }
        }
        return mFunItems;
    }

    void setRefresh(String tag, BaseResponse baseResponse) {
        selfCenterListView.refresh(tag, baseResponse);
    }

    /**
     * 查询年假
     */
    void queryVactionDetail() {
        Map<String, String> params = new HashMap<>(2);
        params.put("empno", SystemHelper.LoginUserId);
        params.put("year", Calendar.getInstance().get(Calendar.YEAR) + "");
        OkHttpDominator.postAsyn(NetParam.YINXIAOBU_SERVER + ApiUrls.APPSALER.QUERY_VACTIONDETAIL, params,
                new OkHttpDominator.ResultCallback<NianJiaResponse>() {
                    @Override
                    public void onError(Request request, Exception e) {
                        ViewUtil.dismissProgress();
                    }

                    @Override
                    public void onResponse(NianJiaResponse res) {
                        ViewUtil.dismissProgress();
                        if (res == null) {
                            return;
                        }
                        selfCenterListView.refresh("nianjia", res);
                    }
                });
    }


    /**
     * 后台检查是否显示门店利润
     */
    void checkIsShowShopData() {
        Map<String, String> params = new HashMap<>(1);
        params.put("shopId", SystemHelper.LoginOrganId);
        OkHttpDominator.postAsyn(NetParam.YINXIAOBU_SERVER + ApiUrls.APP.IS_SHOW_SHOP_DATA,
                params,
                new OkHttpDominator.ResultCallback<ShopDataRes>() {
                    @Override
                    public void onError(Request request, Exception e) {

                    }

                    @Override
                    public void onResponse(ShopDataRes res) {
                        if (res != null && res.getResultInfo() != null ) {//使它显示！
                            shopDataRes = res;//缓存一下，后面点击跳转的时候要用
                            if (!TextUtils.isEmpty(res.getResultInfo().getUnionOperationUrl())){
                                res.getResultInfo().setUnionOperationUrl(res.getResultInfo().getUnionOperationUrl().replace("https://", "http://"));
                            }
                            for (FunItem item : functionNames) {
                                if ("门店利润查询".equals(item.funName)) {
                                    item.isShow = res.getResultInfo().getShowShopProfit();
                                    selfCenterListView.refresh("functions", res);
                                }else if ("联营门店利润查询".equals(item.funName)) {
                                    item.isShow = res.getResultInfo().getShowUnionOperationData();
                                    selfCenterListView.refresh("functions", res);
                                }
                            }
                        }
                    }
                });
    }

    @Override
    public void onDestroy() {
        selfCenterListView = null;
    }
}
