package com.urovo.wgmf.more.pre;

import android.app.Activity;
import android.content.DialogInterface;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.CheckBox;
import android.widget.TextView;

import androidx.appcompat.app.AlertDialog;

import com.urovo.activity.tabMenu.view.TabMenuActivity;
import com.urovo.wgmf.more.ent.MoreContractBean;
import com.urovo.wgmf.more.ent.MoreOnlineBean;
import com.urovo.wgmf.more.net.MoreNet;
import com.urovo.wugumofang.R;
import com.urovo.wugumofang.base.AbstractFailureConsumer;
import com.urovo.wugumofang.base.AbstractSuccessfulConsumer;
import com.urovo.wugumofang.base.mvp.AbsPresenter;
import com.urovo.wugumofang.net.CommonController;
import com.urovo.wugumofang.response.Base3Response;
import com.urovo.wugumofang.response.CommonEntity;
import com.urovo.wugumofang.system.SystemHelper;
import com.urovo.wugumofang.utils.Transformer;
import com.zsy.libcore.common.android.log.SYLoger;
import com.zsy.libcore.model.bean.RequestResult;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import io.reactivex.disposables.Disposable;
import io.reactivex.functions.Function;
import retrofit2.Call;
import retrofit2.Callback;
import retrofit2.Response;

/**
 * @author chenji
 * @date 18/1/12
 */
public class MoreIndexPre extends AbsPresenter implements CommonController.BasePre<List<List<MoreOnlineBean.ResultInfoBean>>> {
    private CommonController.BaseView<List<List<MoreOnlineBean.ResultInfoBean>>> mMoreView;
    private MoreNet mMoreNet;
    private TabMenuActivity tabMenuActivity;//专门为TabMenuActivity做一个

    public MoreIndexPre(CommonController.BaseView<List<List<MoreOnlineBean.ResultInfoBean>>> moreView) {
        this.mMoreView = moreView;
        this.mMoreNet = new MoreNet();
    }

    public MoreIndexPre(TabMenuActivity tabMenuActivity) {
        this.tabMenuActivity = tabMenuActivity;
        this.mMoreNet = new MoreNet();
    }

    /**
     * 每次onResume时所执行的请求
     */
    public void queryContractDialog(final Activity activity) {
        //以前的更多页更多弹窗，现在不要了，改为合同条款弹窗。
        mMoreNet.queryContractDialog(SystemHelper.LoginOrganId, SystemHelper.LoginUserId).enqueue(new Callback<RequestResult<MoreContractBean>>() {
            @Override
            public void onResponse(Call<RequestResult<MoreContractBean>> call, Response<RequestResult<MoreContractBean>> res) {
                if (res.body() != null && res.body().msgCode == 0 && res.body().resultInfo.getShowFlag() && !activity.isFinishing()) {
                    String msg = "";
                    for (String content : res.body().resultInfo.getContent()) {
                        msg += content + "\n";
                    }
                    View view = LayoutInflater.from(activity).inflate(R.layout.contract_dialog, null);
                    ((TextView) view.findViewById(R.id.contractMessage)).setText(msg);
                    AlertDialog dialog = new AlertDialog.Builder(activity)
                            .setTitle(res.body().resultInfo.getTitle())
                            .setView(view)
                            .setCancelable(false)
                            .setPositiveButton("确定", new DialogInterface.OnClickListener() {
                                @Override
                                public void onClick(DialogInterface dialog, int which) {
                                    if (((CheckBox) view.findViewById(R.id.contractCheckBox)).isChecked()) {
                                        submitContractRead();//上传已读状态
                                    }
                                }
                            }).create();
                    dialog.show();
                }
            }

            @Override
            public void onFailure(Call<RequestResult<MoreContractBean>> call, Throwable throwable) {

            }
        });
    }

    /**
     * 上传已读合同状态
     */
    private void submitContractRead() {
        mMoreNet.submitContractRead(SystemHelper.LoginUserId).enqueue(new Callback<Base3Response>() {
            @Override
            public void onResponse(Call<Base3Response> call, Response<Base3Response> response) {
            }

            @Override
            public void onFailure(Call<Base3Response> call, Throwable throwable) {
            }
        });
    }

    @Override
    public void onStart2requestData(Map<String, String> params) {
        Disposable disposable = mMoreNet.getMoreList(params)
                .map(new Function<MoreOnlineBean, List<List<MoreOnlineBean.ResultInfoBean>>>() {
                    @Override
                    public List<List<MoreOnlineBean.ResultInfoBean>> apply(MoreOnlineBean moreOnlineBean) throws Exception {
                        if (moreOnlineBean != null) {
                            if (moreOnlineBean.msgCode == 0) {
                                return moreOnlineBean.getResultInfo();
                            } else {
                                //接口异常提示语
                                mMoreView.onFailure(moreOnlineBean.msgInfo);
                            }
                        }
                        return new ArrayList<>();
                    }
                })
                .compose(Transformer.<List<List<MoreOnlineBean.ResultInfoBean>>>switchSchedulers())
                // 第一种输出方式(自己封装,待优化)
                .subscribe(new AbstractSuccessfulConsumer<List<List<MoreOnlineBean.ResultInfoBean>>>() {
                    @Override
                    protected void onSuccess(List<List<MoreOnlineBean.ResultInfoBean>> lists) throws Exception {
                        if (!lists.isEmpty()) {
                            mMoreView.onResponseSuccess(new CommonEntity<>(lists));
                        }
                        SYLoger.print("wgmf--accept: " + lists);
                    }
                }, new AbstractFailureConsumer<Throwable>() {
                    @Override
                    protected void onFailure(String errorMsg) throws Exception {
                        mMoreView.onFailure(errorMsg);
                    }
                });
        addDisposable(disposable);
    }

    @Override
    public void onResponseSuccess(CommonEntity<List<List<MoreOnlineBean.ResultInfoBean>>> resultInfo) {
        mMoreView.onResponseSuccess(resultInfo);
    }

    @Override
    public void onFailure(String failInfo) {
        mMoreView.onFailure(failInfo);
    }

}