package com.urovo.wgmf.view

import android.app.Dialog
import android.content.Context
import android.content.DialogInterface
import android.view.Gravity
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.Button
import android.widget.LinearLayout
import android.widget.TextView
import com.urovo.wugumofang.R
import com.urovo.wugumofang.utils.notNullAndEmpty

/**
 * @author LW
 */
class PublicTipDialog(context: Context, theme: Int) : Dialog(context, theme) {

    init {
        //这句代码设置这个对话框是否能被用户触摸取消掉
        setCanceledOnTouchOutside(false)
        //这句代码设置这个对话框是否能被用户按[返回键]而取消掉
        setCancelable(true)
    }

    companion object {

        class Builder(private val context: Context) {
            private var title: String? = ""
            private var message: String? = ""
            private var layoutId: Int = -1
            private var layoutParams: ViewGroup.LayoutParams? = null
            private var positiveButtonText: String? = null
            private var negativeButtonText: String? = null
            private var positiveButtonClickListener: DialogInterface.OnClickListener? = null
            private var negativeButtonClickListener: DialogInterface.OnClickListener? = null

            fun setTitle(title: Int): Builder {
                this.title = context.getText(title) as String
                return this
            }

            fun setTitle(title: String?): Builder {
                this.title = title
                return this
            }

            fun setMessage(message: String?): Builder {
                this.message = message
                return this
            }

            fun setMessage(message: Int): Builder {
                this.message = context.getText(message) as String
                return this
            }

            fun setContentLayoutId(layoutId: Int): Builder {
                this.layoutId = layoutId
                return this
            }

            fun setContentLayoutParams(layoutParams: ViewGroup.LayoutParams): Builder {
                this.layoutParams = layoutParams
                return this
            }

            fun setPositiveButton(
                positiveButtonTextId: Int = -1,
                listener: DialogInterface.OnClickListener?
            ): Builder {
                if (positiveButtonTextId > 0)
                    this.positiveButtonText = context
                        .getText(positiveButtonTextId) as String
                positiveButtonClickListener = listener
                return this
            }

            fun setPositiveButton(
                positiveButtonText: String?,
                listener: DialogInterface.OnClickListener?
            ): Builder {
                this.positiveButtonText = positiveButtonText
                positiveButtonClickListener = listener
                return this
            }

            fun setNegativeButton(
                negativeButtonTextId: Int = -1,
                listener: DialogInterface.OnClickListener?
            ): Builder {
                if (negativeButtonTextId > 0)
                    this.negativeButtonText = context
                        .getText(negativeButtonTextId) as String
                negativeButtonClickListener = listener
                return this
            }

            fun setNegativeButton(
                negativeButtonText: String?,
                listener: DialogInterface.OnClickListener?
            ): Builder {
                this.negativeButtonText = negativeButtonText
                negativeButtonClickListener = listener
                return this
            }

            fun create(): PublicTipDialog {
                val dialog = PublicTipDialog(context, R.style.textview_dialog)
                val resource = if (layoutId > 0) {
                    layoutId
                } else {
                    R.layout.view_public_tip_dialog
                }

                val layout: View =
                    LayoutInflater.from(context)
                        .inflate(resource, null)

                dialog.addContentView(
                    layout,
                    ViewGroup.LayoutParams(
                        ViewGroup.LayoutParams.MATCH_PARENT,
                        ViewGroup.LayoutParams.WRAP_CONTENT
                    )
                )
                dialog.window?.apply {
                    setGravity(Gravity.CENTER)
                }

                if (layoutParams != null) {
                    layout.findViewById<LinearLayout>(R.id.public_tip_dialog_root).layoutParams =
                        layoutParams
                }

                layout.findViewById<TextView>(R.id.public_tip_dialog_title_tv).text = title

                layout.findViewById<TextView>(R.id.public_tip_dialog_content_tv).text = message

                layout.findViewById<Button>(R.id.public_tip_cancel_btn)?.apply {
                    if (positiveButtonText.notNullAndEmpty()) {
                        text = positiveButtonText
                    }
                    if (positiveButtonClickListener != null) {
                        setOnClickListener {
                            positiveButtonClickListener?.onClick(
                                dialog,
                                BUTTON_POSITIVE
                            )
                        }
                    } else {
                        visibility = View.GONE
                    }
                }

                layout.findViewById<Button>(R.id.public_tip_confirm_btn)?.apply{
                    if (negativeButtonText.notNullAndEmpty()) {
                        text = negativeButtonText
                    }

                    if (negativeButtonClickListener != null) {
                        setOnClickListener {
                            negativeButtonClickListener?.onClick(
                                dialog,
                                BUTTON_NEGATIVE
                            )
                        }
                    } else {
                        visibility = View.GONE
                    }
                }

                layout.findViewById<Button>(R.id.public_tip_close_btn)?.apply{
                    setOnClickListener {
                        dialog.dismiss()
                    }
                }
                return dialog
            }
        }
    }

}