package com.urovo.wgmf.zxing;

import android.content.Intent;
import android.content.pm.PackageManager;
import android.content.res.AssetFileDescriptor;
import android.graphics.Bitmap;
import android.graphics.Color;
import android.media.AudioManager;
import android.media.MediaPlayer;
import android.media.MediaPlayer.OnCompletionListener;
import android.os.Build;
import android.os.Bundle;
import android.os.Handler;
import android.os.Vibrator;
import android.view.Gravity;
import android.view.SurfaceHolder;
import android.view.SurfaceHolder.Callback;
import android.view.SurfaceView;
import android.view.View;
import android.view.WindowManager;
import android.widget.ArrayAdapter;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;

import com.google.zxing.BarcodeFormat;
import com.google.zxing.Result;
import com.urovo.activity.tabMenu.home.act.ScanResultAct;
import com.urovo.activity.tabMenu.home.ent.ScanCodeListUtil;
import com.urovo.wugumofang.R;
import com.urovo.wugumofang.base.BaseActivity;
import com.urovo.wugumofang.data.PreferHelper;
import com.urovo.wugumofang.utils.MyToast;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Vector;

import butterknife.BindView;
import butterknife.ButterKnife;
import butterknife.Unbinder;

/**
 * @author ...
 */
public class CaptureActivity extends BaseActivity implements Callback, View.OnClickListener {
    private CaptureActivityHandler handler;
    @BindView(R.id.viewfinder_view)
    ViewfinderView viewfinderView;
    @BindView(R.id.open_flash)
    TextView flashView;
    private boolean hasSurface;
    private Vector<BarcodeFormat> decodeFormats;
    private String characterSet;
    // private TextView txtResult;
    private InactivityTimer inactivityTimer;
    private MediaPlayer mediaPlayer;
    private boolean playBeep;
    private static final float BEEP_VOLUME = 0.80f;
    private boolean vibrate;
    @BindView(R.id.txtResult)
    ListView listView;
    @BindView(R.id.preview_view)
    SurfaceView surfaceView;
    private List<String> list;
    private ArrayAdapter<String> adapter;
    private Unbinder unbinder;
    private CameraManager cameraManager;

    /**
     * Called when the activity is first created.
     */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        if (PreferHelper.getInstance().getBoolean(Preferences.FLAG_TRANSLUCENT_STATUS, false)) {
            getWindow().setBackgroundDrawable(null);
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.KITKAT) {
                getWindow().addFlags(WindowManager.LayoutParams.FLAG_TRANSLUCENT_STATUS);
            }
        }
        super.onCreate(savedInstanceState);
        setContentView(R.layout.zxing_main);
        unbinder = ButterKnife.bind(this);

        //防止特殊情况，HomeAct页面没有清除集合
        if (PreferHelper.getInstance().getBoolean("isClear", false)
                && !ScanCodeListUtil.getInstance().getList().isEmpty()) {
            ScanCodeListUtil.getInstance().clearAll();
            PreferHelper.getInstance().remove("isClear");
//            ToastUtil.show(this, "防止特殊情况，HomeAct页面没有清除集合" + PreferHelper.getInstance().getBoolean("isClear", false));
        }

        // 初始化 CameraManager
        CameraManager.init(getApplication());
        cameraManager = CameraManager.get();
//        viewfinderView = (ViewfinderView) findViewById(R.id.viewfinder_view);
//        listView = (ListView) findViewById(R.id.txtResult);
//        flashView = (TextView) findViewById(R.id.open_flash);
        //闪光灯点击事件
        flashView.setOnClickListener(this);
        hasSurface = false;
        inactivityTimer = new InactivityTimer(this);

        list = new ArrayList<>();
        adapter = new ArrayAdapter<>(CaptureActivity.this, android.R.layout.simple_list_item_1, list);
        listView.setAdapter(adapter);

        initTitle();

        setScanInfo(PreferHelper.getInstance().getString(Preferences.SCAN_TOP), PreferHelper.getInstance().getString(Preferences.SCAN_BOTTOM));
    }

    /**
     * 设置扫描框上下文信息
     *
     * @param scanTop    ...
     * @param scanBottom ...
     */
    public void setScanInfo(String scanTop, String scanBottom) {
        viewfinderView.scanTop = scanTop;
        viewfinderView.scanBottom = scanBottom;
    }

    @Override
    public void initTitle() {
        super.initTitle();
        complete.setVisibility(View.VISIBLE);
        complete.setText("完成");
        titleTextView.setText("扫描条码");
        complete.setOnClickListener(this);
        titile_bg.setBackgroundColor(Color.parseColor("#AA000000"));
        titile_bg.setVisibility(PreferHelper.getInstance().getBoolean(Preferences.FLAG_TRANSLUCENT_STATUS, false) ? View.GONE : View.VISIBLE);
    }

    @Override
    protected void onResume() {
        super.onResume();
//        SurfaceView surfaceView = (SurfaceView) findViewById(R.id.preview_view);
        SurfaceHolder surfaceHolder = surfaceView.getHolder();
        if (hasSurface) {
            initCamera(surfaceHolder);
        } else {
            surfaceHolder.addCallback(this);
            surfaceHolder.setType(SurfaceHolder.SURFACE_TYPE_PUSH_BUFFERS);
        }
        decodeFormats = null;
        characterSet = null;

        playBeep = true;
        AudioManager audioService = (AudioManager) getSystemService(AUDIO_SERVICE);
        if (audioService != null && audioService.getRingerMode() != AudioManager.RINGER_MODE_NORMAL) {
            playBeep = false;
        }
        initBeepSound();
        vibrate = true;
    }

    @Override
    protected void onPause() {
        super.onPause();
        if (handler != null) {
            handler.quitSynchronously();
            handler = null;
        }
        CameraManager.get().closeDriver();
    }

    @Override
    protected void onDestroy() {
        PreferHelper.getInstance().remove(Preferences.KEY_DECODE_1D);
        PreferHelper.getInstance().remove(Preferences.KEY_DECODE_QR);
        PreferHelper.getInstance().remove(Preferences.KEY_DECODE_DATA_MATRIX);
        PreferHelper.getInstance().remove(Preferences.FLAG_TRANSLUCENT_STATUS);
        PreferHelper.getInstance().remove(Preferences.SCAN_TOP);
        PreferHelper.getInstance().remove(Preferences.SCAN_BOTTOM);
        if (inactivityTimer != null) {
            inactivityTimer.shutdown();
        }
        if (unbinder != null) {
            unbinder.unbind();
        }
        super.onDestroy();
    }

    private void initCamera(SurfaceHolder surfaceHolder) {
        try {
            CameraManager.get().openDriver(surfaceHolder);
        } catch (IOException ioe) {
            return;
        } catch (RuntimeException e) {
            return;
        }
        if (handler == null) {
            handler = new CaptureActivityHandler(this, decodeFormats, characterSet);
        }
    }

    @Override
    public void surfaceChanged(SurfaceHolder holder, int format, int width, int height) {

    }

    @Override
    public void surfaceCreated(SurfaceHolder holder) {
        if (!hasSurface) {
            hasSurface = true;
            initCamera(holder);
        }

    }

    @Override
    public void surfaceDestroyed(SurfaceHolder holder) {
        hasSurface = false;

    }

    public ViewfinderView getViewfinderView() {
        return viewfinderView;
    }

    public Handler getHandler() {
        return handler;
    }

    public void drawViewfinder() {
        viewfinderView.drawViewfinder();

    }

    public void handleDecode(Result obj, Bitmap barcode) {
        inactivityTimer.onActivity();
        // viewfinderView.drawResultBitmap(barcode);
        playBeepSoundAndVibrate();
        list.add(obj.getText());
        Collections.reverse(list);
        adapter.notifyDataSetInvalidated();
        // onPause();
        // onResume();
        // txtResult.setText(obj.getBarcodeFormat().toString() + ":"
        // + obj.getText());

        Intent intent = new Intent();
        intent.putExtra("code", obj.getText());
        setResult(1, intent);
        //默认是关闭页面的
//        if (PreferHelper.getInstance().getBoolean(Preferences.IS_CLOSE_PAGE, true)) {
        finish();
//        }
    }

    private void initBeepSound() {
        if (playBeep && mediaPlayer == null) {
            // The volume on STREAM_SYSTEM is not adjustable, and users found it
            // too loud,
            // so we now play on the music stream.
            setVolumeControlStream(AudioManager.STREAM_MUSIC);
            mediaPlayer = new MediaPlayer();
            mediaPlayer.setAudioStreamType(AudioManager.STREAM_MUSIC);
            mediaPlayer.setOnCompletionListener(beepListener);

            AssetFileDescriptor file = getResources().openRawResourceFd(R.raw.beep);
            try {
                mediaPlayer.setDataSource(file.getFileDescriptor(), file.getStartOffset(), file.getLength());
                file.close();
                mediaPlayer.setVolume(BEEP_VOLUME, BEEP_VOLUME);
                mediaPlayer.prepare();
            } catch (IOException e) {
                mediaPlayer = null;
            }
        }
    }

    private static final long VIBRATE_DURATION = 200L;

    private void playBeepSoundAndVibrate() {
        if (playBeep && mediaPlayer != null) {
            mediaPlayer.start();
        }
        if (vibrate) {
            Vibrator vibrator = (Vibrator) getSystemService(VIBRATOR_SERVICE);
            vibrator.vibrate(VIBRATE_DURATION);
        }
    }

    /**
     * When the beep has finished playing, rewind to queue up another one.
     */
    private final OnCompletionListener beepListener = new OnCompletionListener() {
        @Override
        public void onCompletion(MediaPlayer mediaPlayer) {
            mediaPlayer.seekTo(0);
        }
    };
    boolean switchFlag;

    @Override
    public void onClick(View v) {
        switch (v.getId()) {
            //闪光灯
            case R.id.open_flash:
                try {
                    if (!getPackageManager().hasSystemFeature(PackageManager.FEATURE_CAMERA_FLASH)) {
                        Toast.makeText(CaptureActivity.this, "你的手机没有闪光灯!", Toast.LENGTH_LONG).show();
                    } else {
                        switchFlag = !switchFlag;
                        if (switchFlag) {
                            cameraManager.openFlash();
                            flashView.setText("关闭闪光灯");
                        } else {
                            cameraManager.closeFlash();
                            flashView.setText("开启闪光灯");
                        }
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                    //权限弹窗
                    permissionDialog();
                }
                break;

            //扫码完成按钮
            case R.id.complete:
                if (!ScanCodeListUtil.getInstance().isEmpty()) {
                    flashView.setText("开启闪光灯");
                    switchFlag = !switchFlag;
                    startActivity(new Intent(CaptureActivity.this, ScanResultAct.class));
                } else {
                    MyToast.showToast(CaptureActivity.this, "请扫码后查看扫描结果", Gravity.CENTER);
                }
                break;
            default:
                break;
        }
    }


//    /**
//     * 启动应用的设置
//     */
//    private static final String PACKAGE_URL_SCHEME = "package:";
//
//    private void startAppSettings() {
//        Intent intent = new Intent(Settings.ACTION_APPLICATION_DETAILS_SETTINGS);
//        intent.setData(Uri.parse(PACKAGE_URL_SCHEME + getPackageName()));
//        startActivity(intent);
//    }
}