package com.wgmf.trade.model.bean

import com.urovo.wgmf.db.dao.dao.Item001DbOperate
import com.urovo.wgmf.db.dao.entitys.ProductEntity
import com.urovo.wugumofang.constants.AppConstants
import com.urovo.wugumofang.data.MoneyUtils
import com.wgmf.trade.utils.ArithUtil
import java.util.*

/**
 * @author Sasuke on 2018/9/10.
 * 以前的逻辑看着是真的烦。。。。
 * 所以专门写一个工具类，
 * 来处理活动相关的乱七八糟的计算呀、集合操作呀之类的。
 * （なにより、kotlinも加勢するため、処理も楽々。）
 */
object MemActionDominator {
    /**
     * 检查是否该产品列表里只有鲜稻研产品
     * （用于在打印的时候，如果只有鲜稻研产品，那就把打印按钮禁用掉，因为鲜稻研不支持打印）
     */
    @JvmStatic
    fun hasWgmfExist(productInfoBeansAll: ArrayList<ProductInfoBean>): Boolean {
        return productInfoBeansAll.any { !it.isDecimal }
    }

    /**
     * 同上（两个数据类型完全不同。
     */
    @JvmStatic
    fun hasWgmfExist(productInfoBeansAll: List<ProductEntity>): Boolean {
        return productInfoBeansAll.any { !it.isDecimal }
    }

    /***
     * 如果产品组合的外层特价字段不为0，则把它分配到该产品组合下的每个产品的特价字段。
     */
    @JvmStatic
    fun dealProGroupSpecialPrice(memberActionList: List<MemberAction>?) {
        memberActionList?.forEach { action ->
            action.rule?.proGroupInfo?.forEach { proInfo ->
                if (proInfo.specialPrice != 0.0) {
                    proInfo.product.forEach { pro -> pro.specialPrice = proInfo.specialPrice }
                }
            }
        }
    }

    /**
     * 过滤，留下五谷磨房产品，排除鲜稻研，用于打印（因为鲜稻研产品不支持打印）
     */
    @JvmStatic
    fun filterWgmfForPrint(bill: List<ProductEntity>): List<ProductEntity> {
        return bill.filter { !it.isDecimal }
    }

    /**
     * 用于在点击活动进入时校验品牌
     * （满减活动里，如果既有五谷磨房又有散装，那在减价之后，分别打印条码时，不好分配这个减的价配在哪里，所以不允许这种情况
     *  所以这个是用来填补这个漏洞的。
     *
     *  brandCheck: 品牌校验，1：只允许五谷产品，2：只允许散装，3：不限制产品
     */
    @JvmStatic
    fun checkActionBrand(action: MemberAction): Boolean {
        return when (action.brandCheck) {
            1 -> action.rule?.productInfo?.all { !it.isDecimal }
                    ?: true //SystemHelper.hasWgmf() && !Item001DbOperate.hasDecimalPro()
            2 -> action.rule?.productInfo?.all { it.isDecimal }
                    ?: true//!SystemHelper.hasWgmf() && Item001DbOperate.hasDecimalPro()//据说一般不会有这种情况
            else -> true
        }
    }

    /**
     * 把支持到家业务的活动排序到后面
     */
    @JvmStatic
    fun sort2HomeOrderToBottom(memberActions: List<MemberAction>): List<MemberAction> {
        return memberActions.sortedBy {
            when (true) {
                it.brandType == AppConstants.BRAND.XDY -> 1
                it.arriveHome == 1 && it.stick == 0 -> 2 //是到家并且不是置顶就排最后
                else -> 0
            }
        }
    }


    /**
     * 用于在点击活动进入时校验规则是否合法
     * （场景：在活动的产品或赠品里，同时包含isDecimal为Y或N时，即同时包含定型包装和散装时，
     *  则不允许进入这个活动，因为这样会引起后面校验方式的混乱。）
     *
     *  以下的逻辑是：看是不是所有的组合产品和组合赠品里的各个产品的isDecimal都与”第一个元素的isDecimal“相同，如果都相同，则合法。
     */
    @JvmStatic
    fun checkActionGroupSameDecimal(action: MemberAction): Boolean {
        //product为空时，则为true
        //proGroupInfo为空，则为true，因为根本不需要验证规则是否合法
        //校验前，都要先filter一下item001看存不存在该产品，不存在就算了，不用强行比较它的isDecimal
        return (action.rule?.proGroupInfo?.all { proGroup ->
            proGroup.product?.filter { Item001DbOperate.GetProductByBarcode(it.code) != null }?.let { filterList -> filterList.all { it.isDecimal == filterList[0].isDecimal } }
                    ?: true
        } ?: true
                && action.giftRule?.giftProGroupInfo?.all { proGroup ->
            proGroup.product?.filter { Item001DbOperate.GetProductByBarcode(it.code) != null }?.let { filterList -> filterList.all { it.isDecimal == filterList[0].isDecimal } }
                    ?: true
        } ?: true)
    }

    /**
     *  散装组合活动情况下，在点击活动进入时，要校验一下是否后台有配置money
     *  如果没配置，要弹窗提示
     *  返回：如果合法，则返回“”；如果不合法，则返回错误信息。
     */
    @JvmStatic
    fun checkDecimalHaveRule(action: MemberAction): String {
        var isNoMoney = false
        var isNoNum = false
        action.rule.proGroupInfo?.forEach {
            if (it.product?.all { it.isDecimal } ?: false) {//有时候product后台会配成空
                if (it.money?.toDouble() ?: 0.0 == 0.0) isNoMoney = true
                if (!("+" in it.num) && !("," in it.num) && it.num?.toDouble() ?: 0.0 == 0.0) isNoNum = true
            }
        }
        return if (!isNoMoney) ""
        else "产品组合没有 ${if (isNoMoney) "金额" else ""} ，联系活动配置人员！"
    }

    /**
     * 校验 产品，产品组合，赠品，赠品组合 是否有产品，合法才能跳转MemberSaleFra
     */
    @JvmStatic
    fun checkHasFoundAllPro(memberAction: MemberAction, notFoundProSet: HashSet<ProductInfoBean>, notFoundGiftSet: HashSet<ProductInfoBean>): String {
        //校验规则的时候，在数据库里没有找到相应的活动商品，就存在HashSet里，到时候弹Toast提示。

        //校验产品和产品组合
        memberAction.rule?.run {
            productInfo?.filter { Item001DbOperate.GetProductByBarcode(it.code) == null }?.forEach { notFoundProSet.add(it) }
            proGroupInfo?.forEach {
                it.product?.filter { Item001DbOperate.GetProductByBarcode(it.code) == null }?.forEach { notFoundProSet.add(it) }
            }
        }
        //校验赠品和赠品组合
        var giftCount = 0
        memberAction.giftRule?.run {
            giftProductInfo?.filter { Item001DbOperate.GetProductByBarcode(it.code) == null }?.forEach { notFoundProSet.add(it) }
            giftProGroupInfo?.forEach {
                it.product?.filter { Item001DbOperate.GetProductByBarcode(it.code).apply { giftCount++ } == null }?.forEach { notFoundGiftSet.add(it) }
            }
        }
        var notFoundProNames = ""
        //产品，产品组合，赠品，只要有一个产品在DB里没找到，就判false
        notFoundProSet.takeIf { it.size > 0 }?.forEach { notFoundProNames += "${it.name}(${it.code}) \n" }
        //赠品组合必须全部都找不到才判false，哪怕只找到一个，都允许true
        notFoundGiftSet.takeIf { it.size > 0 && it.size == giftCount }?.forEach { notFoundProNames += "${it.name}(${it.code}) \n" }
        return notFoundProNames
    }

    /**
     *  产品，产品组合， 是否有产品，合法才能跳转MemberSaleFra
     */
    @JvmStatic
    fun checkHasFoundPro(memberAction: MemberAction, notFoundProSet: HashSet<ProductInfoBean>): String {
        //校验规则的时候，在数据库里没有找到相应的活动商品，就存在HashSet里，到时候弹Toast提示。

        //校验产品和产品组合
        memberAction.rule?.run {
            productInfo?.filter { Item001DbOperate.GetProductByBarcode(it.code) == null }?.forEach { notFoundProSet.add(it) }
            proGroupInfo?.forEach {
                it.product?.filter { Item001DbOperate.GetProductByBarcode(it.code) == null }?.forEach { notFoundProSet.add(it) }
            }
        }
        var notFoundProNames = ""
        //产品，产品组合，赠品，只要有一个产品在DB里没找到，就判false
        notFoundProSet.takeIf { it.size > 0 }?.forEach { notFoundProNames += "${it.name}(${it.code}) \n" }
        return notFoundProNames
    }

    /**
     * 校验 赠品，赠品组合 是否有产品，合法才能跳转MemberSaleFra
     */
    @JvmStatic
    fun checkHasFoundGift(memberAction: MemberAction, notFoundGiftSet: HashSet<ProductInfoBean>): String {
        //校验规则的时候，在数据库里没有找到相应的活动商品，就存在HashSet里，到时候弹Toast提示。

        //校验赠品和赠品组合
        var giftCount = 0
        memberAction.giftRule?.run {
            giftProductInfo?.filter { Item001DbOperate.GetProductByBarcode(it.code).apply { giftCount++ } == null }?.forEach { notFoundGiftSet.add(it) }
            giftProGroupInfo?.forEach {
                it.product?.filter { Item001DbOperate.GetProductByBarcode(it.code).apply { giftCount++ } == null }?.forEach { notFoundGiftSet.add(it) }
            }
        }
        var notFoundProNames = ""
        //产品，产品组合，赠品，只要有一个产品在DB里没找到，就判false
        //赠品组合必须全部都找不到才判false，哪怕只找到一个，都允许true
        notFoundGiftSet.takeIf { it.size > 0 && it.size == giftCount }?.forEach { notFoundProNames += "${it.name}(${it.code}) \n" }
        return notFoundProNames
    }

    /**
     *  商品列表(商品,商品组合,继续添加)价格计算,不参与活动计算
     * 注：严格按照这样的格式返回：
     * arrayOf(总价，实付，特价)
     * arrayOf(totalPrice，actualPrice，specialActualPrice)
     */
    @JvmStatic
    fun calculatePrice(proList: List<ProductEntity>):Array<Double>{
        /**总价格*/
        //不区分品牌，按照是否有小数位划分：Y：true ，保留小数位    N:false ，不保留小数位，整数。
        val totalPrice = proList.sumByDouble { if (it.isDecimal) it.amount else ArithUtil.mul(it.tqty, it.price) }
        /**总实际价格*/
        val actualPrice = proList.sumByDouble { if (it.isDecimal) it.amount else ArithUtil.mul(it.tqty, it.price) }
        val specialActualPrice = 0.0

        return arrayOf(formatPrice(totalPrice), formatPrice(actualPrice), specialActualPrice)
    }


    /**
     *  商品列表(商品,商品组合,继续添加)价格计算
     * 注：严格按照这样的格式返回：
     * arrayOf(总价，实付，特价)
     * arrayOf(totalPrice，actualPrice，specialActualPrice)
     */
    @JvmStatic
    fun calculatePrice(proList: List<ProductInfoBean>, memberAction: MemberAction): Array<Double> {
        /**总价格*/
        //不区分品牌，按照是否有小数位划分：Y：true ，保留小数位    N:false ，不保留小数位，整数。
        val totalPrice = proList.sumByDouble { if (it.isDecimal) it.amount else ArithUtil.mul(it.proNum, it.priceOriginal) }
        /**总实际价格*/
        var actualPrice = proList.sumByDouble { if (it.isDecimal) it.amount else ArithUtil.mul(it.proNum, it.price) }
        var specialActualPrice = 0.0

        //活动（常规、全场、专享）正常情况
        if (memberAction.checkRuleOrNot == 0) {
            if (actualPrice != totalPrice) specialActualPrice = actualPrice
            memberAction.giftRule?.run {
                //赠送金额
                takeIf { giftMoney > 0 && actualPrice >= formatPrice(memberAction.rule.money.toDouble()) }?.let {
                    actualPrice = ArithUtil.sub(actualPrice, giftMoney)
                }
                //单品 打折计算
                takeIf { giftDiscount > 0 && actualPrice >= formatPrice(memberAction.rule.money.toDouble()) }?.let {
                    //                    actualPrice = ArithUtil.mul(actualPrice, giftDiscount * 0.1)
                    //从活动传来的产品信息里，抽出需要打折的单品的产品id
                    val discountProIds: MutableList<String> = mutableListOf()//从活动传来的 需要单独打折的产品或产品组合
                    memberAction.rule.proGroupInfo?.forEach { discountProIds.addAll(it.product.map { it.code }) }
                    memberAction.rule.productInfo?.map { it.code }?.let { discountProIds.addAll(it) }
                    //根据产品id来判断订单中哪些产品需要打折，然后覆盖之前计算的实付金额
                    actualPrice = proList.sumByDouble {
                        //计算单品打折
                        //如果遍历的产品存在于“打折产品id List”中，就打折；
                        // 或者 "打折产品id List“为空，表示全部产品都打折
                        val discount = if (it.code in discountProIds || discountProIds.isEmpty()) giftDiscount * 0.1 else 1.0
                        it.discountedPrice = ArithUtil.mul(it.price, discount)//缓存打折后的价格，后面传给打印之前需要用它覆盖深拷贝后的列表的特价，用于计算单品总价来打印
                        ArithUtil.mul(if (it.isDecimal) it.amount else ArithUtil.mul(it.proNum, it.price), discount)
                    }
                }
            }
        } else {//立享优惠情况；
            memberAction.giftRule?.takeIf { it.giftMoney > 0 }?.let { actualPrice = ArithUtil.sub(totalPrice, it.giftMoney) }
        }
        return arrayOf(formatPrice(totalPrice), formatPrice(actualPrice), specialActualPrice)
    }


    /**
     * 格式化价格
     */
    @JvmStatic
    fun formatPrice(tempPrice: Double): Double {
        return MoneyUtils.twolastDF(tempPrice).toDouble()
    }

    /**
     * 创造一个打折产品明细的TextView显示内容
     */
    @JvmStatic
    fun makeDiscountTextString(memberAction: MemberAction): String {
        //先把需要打折的产品名字都列出来
        val discoundProNames = mutableListOf<String>().apply {
            memberAction.rule.productInfo?.map { it.name }?.let { addAll(it) }
            memberAction.rule.proGroupInfo?.forEach { it.product?.map { it.name }?.let { addAll(it) } }
        }
        return if (discoundProNames.isEmpty()) "全场  ${formatPrice(memberAction.giftRule.giftDiscount)}折"
        else StringBuffer().apply { discoundProNames.forEach { append("${it}  ${formatPrice(memberAction.giftRule.giftDiscount)}折\n") } }.trim().toString()
    }
}