package com.wgmf.trade.ui.fra.index;

import android.app.Activity;
import android.content.Intent;
import android.os.Bundle;
import android.text.TextUtils;
import android.view.View;
import android.widget.ExpandableListView;

import com.urovo.wugumofang.R;
import com.urovo.wugumofang.constants.AppConstants;
import com.urovo.wugumofang.constants.ConstantParams;
import com.urovo.wugumofang.data.Arith;
import com.urovo.wugumofang.ui.DialogHelper;
import com.wgmf.trade.model.bean.ProGroupInfoBean;
import com.wgmf.trade.model.bean.ProductInfoBean;
import com.wgmf.trade.ui.ada.ProSelectedAda;
import com.wgmf.trade.utils.ArithUtil;
import com.wgmf.trade.utils.VerifyUtil;
import com.zsy.libext.ui.fra.BaseFra;

import java.util.ArrayList;

import androidx.annotation.NonNull;

import org.jetbrains.annotations.NotNull;

import butterknife.BindView;
import butterknife.ButterKnife;
import butterknife.OnClick;
import butterknife.Unbinder;

/**
 * 产品选择(商品或者赠品)
 * Created by samy on 16/9/22.
 */
public class ProductSelectionFra extends BaseFra {

    @BindView(R.id.product_selection_lv)
    ExpandableListView productSelectionLv;

    protected ProSelectedAda mProSelectedAda;
    private ArrayList<ProGroupInfoBean> mProGroupInfoBeans;
    private boolean mIsGift;
    private int mActivityType, mActivityId;
    private String mProductTitle, mPhomeNum;

    private String mFlag = ""; // 标记特价活动是否存在满足条件的产品
    private String mIsDecimal = ""; // 是否散装标记

    @Override
    protected void getBundleExtras(Bundle extras) {
        super.getBundleExtras(extras);
        mIsGift = extras.getBoolean(AppConstants.ExtraKeys.IS_GIFT);
        mActivityType = extras.getInt(AppConstants.ExtraKeys.ACTIVITY_TYPE);
        mActivityId = extras.getInt(AppConstants.ExtraKeys.ACTIVITY_ID);
        mProductTitle = extras.getString(AppConstants.ExtraKeys.PRODUCT_TITLE);
        mPhomeNum = extras.getString(AppConstants.ExtraKeys.PHONE_NUM);
        mProGroupInfoBeans = (ArrayList<ProGroupInfoBean>) extras.getSerializable(AppConstants.ExtraKeys.PRODUCT_INFO_BEAN);
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        unbinder.unbind();
    }

    Unbinder unbinder;

    @Override
    protected void initWidgetsAndEvents(View view) {
        unbinder = ButterKnife.bind(this, view);
        baseAct.commonToolbar.setTitle(mIsGift ? "赠品选择" : "产品选择");

        mProSelectedAda = new ProSelectedAda(getActivity(), mProGroupInfoBeans, mIsGift, mActivityType, mProductTitle);
        productSelectionLv.setAdapter(mProSelectedAda);

        productSelectionLv.setGroupIndicator(null);
        //遍历所有group,将所有项设置成默认展开
        int groupCount = productSelectionLv.getCount();
        for (int i = 0; i < groupCount; i++) {
            productSelectionLv.expandGroup(i);
        }
        productSelectionLv.setOnGroupClickListener((parent, v, groupPosition, id) -> true);
    }

    @Override
    protected View getLoadingTargetView() {
        return null;
    }

    @Override
    protected int getContentViewLayoutID() {
        return R.layout.fra_product_selection;
    }

    @OnClick(R.id.commit_btn)
    public void commit_btn() {
        double mainMultiple = -1;  //梯度折扣中主比例

        for (ProGroupInfoBean proGroupInfoBean : mProGroupInfoBeans) {
            String num = proGroupInfoBean.num;
            int proportRelation = proGroupInfoBean.proportRelation;
            double proNum = 0;
            double proMoney = 0.0;
            for (ProductInfoBean productInfoBean : proGroupInfoBean.product) {
                proNum = Arith.add(proNum, productInfoBean.getProNum());
                if (productInfoBean.isDecimal()) {//如果有小数位，那就应该直接加总价，而不是另外计算
                    mIsDecimal += "T";
                    proMoney = ArithUtil.add(proMoney, productInfoBean.getAmount());
                } else {
                    mIsDecimal += "F";
                    proMoney = ArithUtil.add(proMoney, productInfoBean.getAmountActual());//跟ProSelectedAda里的总价显示保持一致的算法
                }
            }
            //散装校验金额，非散装校验数量
            if (mIsDecimal.contains("T")) {
                //如果金额规则和数量规则都为空，这是非法的，说明后台配错了！报个提示就行。
                if (TextUtils.isEmpty(proGroupInfoBean.money)) {
                    showTipDialog("产品组合没有金额，联系活动配置人员!");
                    return;
                }
                if (!VerifyUtil.compareMoney(proGroupInfoBean.money, proMoney)) {
                    showTipDialog("金额不满足活动要求，请输入正确的值！");
                    return;
                }
            } else {
                if (TextUtils.isEmpty(num)) {
                    showTipDialog("产品组合没有数量，联系活动配置人员!");
                    return;
                }
                if (ConstantParams.CONSTANT_STARTS_9 == mActivityType) {// 如果特价活动，只要有一个满足数量规则即可
                    if (VerifyUtil.compareLogicNum(num, proNum)) {
                        mFlag += "T";
                    } else {
                        mFlag += "F";
                    }
                } else if (ConstantParams.CONSTANT_STARTS_14 == mActivityType && !mIsGift) { //梯度折扣，必须满足倍数关系
                    double multiple;
                    if (!TextUtils.isEmpty(num)) {
                        multiple = ArithUtil.div(proNum, Double.parseDouble(num));
                        proGroupInfoBean.multipleRelation = multiple; //倍数关系
                        if (proNum > 0) {
                            if (proportRelation == ConstantParams.CONSTANT_STARTS_1) {
                                mainMultiple = multiple;
                                if (proNum % Double.parseDouble(num) >=0) {
                                    mFlag += "T";
                                } else {
                                    mFlag += "F";
                                }
                            }else{
                                if (proNum % Double.parseDouble(num) == 0) {
                                    mFlag += "T";
                                } else {
                                    mFlag += "F";
                                }
                            }
                        } else {
                            mFlag += "F";
                        }
                    }else{
                        mFlag += "T";
                    }

                } else {
                    if (!VerifyUtil.compareNum(num, proNum)) {
                        showTipDialog("数量不满足活动要求，请输入正确的值！");
                        return;
                    }
                }
            }
        }

        if (mainMultiple>=0 && ConstantParams.CONSTANT_STARTS_14 == mActivityType && !mIsGift) {
            for (ProGroupInfoBean proGroupInfoBean : mProGroupInfoBeans) {
                if (proGroupInfoBean.proportRelation != ConstantParams.CONSTANT_STARTS_1 &&
                        proGroupInfoBean.multipleRelation > mainMultiple) {
                    mFlag += "F";
                }
            }
        }

        if (!mIsDecimal.contains("T")) {
            if (ConstantParams.CONSTANT_STARTS_9 == mActivityType) {// 如果特价活动，只要有一个满足数量规则即可
                if (!mFlag.contains("T")) {// 非散装才校验数量
                    mFlag = "";
                    showTipDialog("数量不满足活动要求，请输入正确的值！");
                    return;
                }
            } else if (ConstantParams.CONSTANT_STARTS_14 == mActivityType) {
                if (mFlag.contains("F")) {
                    mFlag = "";
                    showTipDialog(mProductTitle);
                    return;
                }
            }
        }
        //梯度折扣特殊处理
        if (ConstantParams.CONSTANT_STARTS_14 == mActivityType) {
            for (ProGroupInfoBean proGroupInfoBean : mProGroupInfoBeans) {
                int groupId = proGroupInfoBean.groupId;
                int proportRelation = proGroupInfoBean.proportRelation;
                for (ProductInfoBean productInfoBean : proGroupInfoBean.product) {
                    productInfoBean.groupId = groupId;
                    productInfoBean.proportRelation = proportRelation;
                }
            }
        }
        jumpPageResult();
    }

    /**
     * 显示消息提示框
     *
     * @param message 数据
     */
    private void showTipDialog(String message) {
        if (!TextUtils.isEmpty(message)) {
            DialogHelper.onAlertDialog(mContext, message, null, (dialog, which) -> {

            });
        }
    }

    /**
     * 返回页面数据
     */
    private void jumpPageResult() {
        Intent intent = new Intent();
        intent.putExtra(AppConstants.ExtraKeys.IS_GIFT, mIsGift);
        intent.putExtra(AppConstants.ExtraKeys.PRODUCT_INFO_BEAN, mProGroupInfoBeans);
        baseAct.setResult(Activity.RESULT_OK, intent);
        baseAct.finish();
    }

    @Override
    public void setPresenter(@NonNull @NotNull Object presenter) {

    }
}
