package com.wgmf.trade.utils;

import java.lang.reflect.Method;

/**
 * Created by chenji on 2017/4/26.
 * <p>
 * 反射工具
 */

public class Reflect {
    private static Reflect mInstance;
    public Class cls = null;
    public Method method = null;
    public Object o = null;

    private Reflect() {
    }

    //单例
    private static Reflect getInstance() {
        if (mInstance == null) {
            synchronized (Reflect.class) {
                if (mInstance == null) {
                    mInstance = new Reflect();
                }
            }
        }
        return mInstance;
    }


     /*-------------------------------对外提供的方法 start--------------------*/


    /**
     * @param className 需要被反射的对象名
     * @return 返回一个class对象
     */
    public static Class<?> getClass(String className) {
        return Reflect.getInstance()._getClass(className);
    }

    /**
     * @param cls 反射对象的class
     * @return 返回一个反射的对象
     */
    public static Object newInstance(Class<?> cls) {
        return Reflect.getInstance()._newInstance(cls);
    }

    /**
     * 根据对象，获取某个方法
     *
     * @param cls            Class对象
     * @param methodName     方法名
     * @param parameterTypes 该方法需传的参数类型，如果不需传参，则不传
     */
    public static Method getMethod(Class<?> cls, String methodName, Class<?>... parameterTypes) {
        return Reflect.getInstance()._getMethod(cls, methodName, parameterTypes);
    }

    /**
     * 根据方法对象，启用该方法。
     *
     * @param method   需要被调用的方法对象
     * @param receiver 需要被反射的对象的实例
     * @param args     被调用的方法需要传递的参数
     */
    public static void invoke(Method method, Object receiver, Object... args) {
        Reflect.getInstance()._invoke(method, receiver, args);
    }

  /* -------------------------------对外提供的方法 end--------------------*/

    private void _invoke(Method method,
                         Object receiver,
                         Object... args) {
        try {
            method.invoke(receiver, args);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private Method _getMethod(Class<?> cls,
                              String methodName,
                              Class<?>... parameterTypes) {
        try {
            Method method = cls.getDeclaredMethod(methodName, parameterTypes);
            method.setAccessible(true);
            return method;
        } catch (NoSuchMethodException e) {
            e.printStackTrace();
            return null;
        }
    }

    private Class<?> _getClass(String className) {
        try {
            return Class.forName(className);
        } catch (ClassNotFoundException e) {
            e.printStackTrace();
            return null;
        }
    }

    private Object _newInstance(Class<?> cls) {
        try {
            if (cls != null) {
                return cls.newInstance();
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }
}

