package com.wgmf.trade.wx.utils;

import android.content.ContentResolver;
import android.content.Context;
import android.content.res.Resources;
import android.net.Uri;
import java.util.List;

/**
 * Created by samy on 16/8/1.
 * 处理各种图片的加载样式问题；
 * 本地图片：在native代码进行字符串过滤；判断这个file://开头进行处理；
 * src="file://android_asset/image/icon_launcher.png";
 */
public class ResourceUtils {

  public static int getResourceUri(Uri uri) {
    //if (BuildConfig.DEBUG) {
    //  SYLoger.debug("Load --Resource--->>" + uri);
    //}

    String host = uri.getEncodedAuthority();
    List<String> pathSegments = uri.getPathSegments();

    String typeName = pathSegments.get(0);
    String resName = pathSegments.get(1);
    int resId = -1;
    try {
      Class<?> clazz = Class.forName(host + ".R$" + typeName);
      resId = clazz.getField(resName).getInt(null);
    } catch (Exception e) {
      e.printStackTrace();
    }
    return resId;
  }

  public static Uri getResourceUri(Context packageContext, int res) {
    try {
      Resources resources = packageContext.getResources();
      return getResourceUri(resources, packageContext.getPackageName(), res);
    } catch (Resources.NotFoundException e) {
      return null;
    }
  }

  public static Uri getResourceUri(Resources resources, String appPkg, int res) {
    String resPkg = resources.getResourceName(res);
    String type = resources.getResourceTypeName(res);
    String name = resources.getResourceEntryName(res);
    return makeResourceUri(appPkg, resPkg, type, name);
  }

  /**
   * 解析url后拿到各个参数，反射去R文件拿对应的Field的int值。拿到就是Resid了；
   * @param appPkg
   * @param resPkg
   * @param type
   * @param name
   * @return
   */
  public static Uri makeResourceUri(String appPkg, String resPkg, String type, String name) {
    Uri.Builder uriBuilder = new Uri.Builder();
    uriBuilder.scheme(ContentResolver.SCHEME_ANDROID_RESOURCE);
    uriBuilder.encodedAuthority(appPkg);
    uriBuilder.appendEncodedPath(type);
    if (!appPkg.equals(resPkg)) {
      uriBuilder.appendEncodedPath(resPkg + ":" + name);
    } else {
      uriBuilder.appendEncodedPath(name);
    }
    return uriBuilder.build();
  }
}
